# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.file.controllers import ImportFileController, ImportFileException
from basesoft.web.core.importers import Utf8CsvImporter


class MockFileController(ImportFileController, Utf8CsvImporter):
    """
    Class that simulates a LineImporter extending ImportFileController
    """
    MAX_RECORDS_FOR_INSTANT_IMPORT = 5
    IMPORT_FILE_TYPE = 'basesoft.web.test'
    callback = None

    @staticmethod
    def get_importer(tmp_file):
        """
        static method to return the right importer for the given file
        """
        return MockFileController

    def set_notification_callback(self):
        self.callback = {
            'js_module': 'test123',
            'parameters': []
        }

    def get_notification_callback(self):
        return self.callback

    def get_failure_notification_callback(self):
        return self.callback

    @staticmethod
    def get_notification_subject():
        return 'New regulation period'

    @staticmethod
    def get_notification_text():
        return 'Use the perform action button to view the changes compared to the previous period'

    def _import_row(self, row, controller, dryrun, **kwargs):
        values = []

        for i in xrange(len(self._file_headers)):

            if not row[i].startswith('row'):
                raise ImportFileException('Column should be a string')

            values.append(row[i])
        self.set_notification_callback()
        return values

    def read_row(self, file_path):
        return self._read_row(file_path)
