
from sqlalchemy.orm import aliased
from sqlalchemy import func
from basesoft.web.contrib.i18n.models import Country

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryModel, CustomQueryField, SQLCustomQuery,\
    SQLCustomQueryTable, SQLCustomQueryField, SQLCustomQueryUnion
from basesoft.web.contrib.license.models import License, Group
from basesoft.web.contrib.mailing.models import Mail, Mailing
from basesoft.web.contrib.file.models import File, FileCategory
from basesoft.web.contrib.contact.models import Contact, Person, Address
from basesoft.web.contrib.i18n.custom_query import DomainCountryCustomQueryModel,\
    DomainCountryCustomQueryField
from basesoft.web.contrib.branding.models import Theme

m2 = aliased(Mail)
license = aliased(License)
group = aliased(Group)

class TestMailCustomQuery(CustomQuery):
    def tables(self):
        return (CustomQueryModel(m2), CustomQueryModel(Mailing))
    def fields(self):
        return (CustomQueryField(m2.id, 'ID', sortable=False), CustomQueryField(m2.name, 'Name'))
    hidden_fields = (m2.id, )

class TestFileCustomQuery(CustomQuery):
    def tables(self):
        return (CustomQueryModel(File), CustomQueryModel(FileCategory, onclause=File.category_id==FileCategory.id))

    def fields(self):
        return (CustomQueryField(File.id.label('tuut'), 'File ID'), CustomQueryField(FileCategory.id, 'Category Name'))

class TestGroupConcatQuery(CustomQuery):
    def tables(self):
        return (
            CustomQueryModel(Theme),
        )

    def fields(self):
        return (
            CustomQueryField(func.group_concat(', ', Theme.id), 'Theme ID', key='theme_id', use_when_separated=Theme.id),
        )

class TestContactCustomQuery(CustomQuery):

    permissions = ('view_contact', )

    def tables(self):
        return (
            CustomQueryModel(Contact),
            CustomQueryModel(Person, onclause=Contact.person_id==Person.id),
            CustomQueryModel(Address),
            DomainCountryCustomQueryModel(),
        )

    def fields(self):
        return (
            CustomQueryField(Contact.id, 'Contact ID', is_filter=True),
            CustomQueryField(Person.first_name, 'First name'),
            CustomQueryField(Person.last_name, 'Last name', is_filter=True),
            CustomQueryField(Person.department, 'Department', is_filter=True, filter_options=Person.department, is_group_by=True),
            CustomQueryField(Person.position, 'Position', is_filter=True, filter_options=Person.position),
            CustomQueryField(Person.gender, 'Gender', key='gender', is_filter=True, is_group_by=True),
            CustomQueryField(Person.created, 'Created', is_filter=True),
            DomainCountryCustomQueryField(None, 'Country', key='country_name', in_select=False, is_filter=True, is_group_by=True),
        )

class TestSQLCustomQuery(SQLCustomQuery):

    def tables(self):
        return (
            SQLCustomQueryTable('tblContact', 'c'),
            SQLCustomQueryTable('tblPerson', 'p', onclause=[('c', 'person_id', 'p', 'id', )]),
        )

    def fields(self):
        return (
            SQLCustomQueryField('c', 'id', 'ContactID', key='contact_id'),
            SQLCustomQueryField('p', 'first_name', 'FirstName', key='first_name', is_filter=True),
            SQLCustomQueryField('p', 'gender', 'Gender', key='gender'),
        )

    group_by = (('c', 'id'), )


class TestMockUnionQueryOne(SQLCustomQuery):
    def tables(self):
        return (
            SQLCustomQueryTable('tblContact', 'c'),
        )

    def fields(self):
        return (
            SQLCustomQueryField('c', 'id', 'UnionID', key='union_id', is_filter=True),
        )


class TestMockUnionQueryTwo(SQLCustomQuery):
    def tables(self):
        return (
            SQLCustomQueryTable('tblPerson', 'p'),
        )

    def fields(self):
        return (
            SQLCustomQueryField('p', 'id', 'UnionID', key='union_id', is_filter=True),
        )


class TestSQLUnionCustomQuery(SQLCustomQueryUnion):

    def custom_queries(self):
        return (
            TestMockUnionQueryOne,
            TestMockUnionQueryTwo,
        )


class TestFalseFieldCustomQuery(CustomQuery):

    def tables(self):
        return (
            CustomQueryModel(Country),
        )

    def fields(self):
        return (
            CustomQueryField(Country.id, 'Country ID', is_filter=True),
            CustomQueryField(Country.name, 'First name'),
            CustomQueryField(False, 'ISO A2', 'iso_a2', use_when_separated=Country.iso_3166_a2),
            CustomQueryField(Country.iso_3166_a3, 'ISO A3', 'iso_a3'),
        )

custom_queries = (
    TestMailCustomQuery,
    TestFileCustomQuery,
    TestContactCustomQuery,
    TestSQLCustomQuery,
    TestGroupConcatQuery,
    TestSQLUnionCustomQuery,
    TestFalseFieldCustomQuery
)
