# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.mailing.controllers import MailBaseController, AbstractMailStateController
from basesoft.web.contrib.mailing.schedule import MailingTask
from basesoft.web.test.models import MockMail, MockMailing

class MockMailStateController(AbstractMailStateController):
    """
    Takes care of the MockMail state
    """
    _instance_class = MockMail
    _compose_plugin_module = 'test_mock_mail_plugin'

    # permissions needed for using this kind of mail
    CREATE_ACCESS_PERMISSIONS   = ['create_mock_mail']
    VIEW_ACCESS_PERMISSIONS     = ['view_mock_mail']
    UPDATE_ACCESS_PERMISSIONS   = ['update_mock_mail']
    DELETE_ACCESS_PERMISSIONS   = ['delete_mock_mail']
    SCHEDULE_ACCESS_PERMISSIONS = ['schedule_mock_mail']

class MailController(MailBaseController):
    """
    Class that's on top of the MailingBaseController. This is used for
    handling MockMail objects for testing.
    """
    # mapping between object type and the controller
    _state_controllers = {
        'MockMail': MockMailStateController,
    }

class MockMailingTask(MailingTask):
    """
    Mock mailing task extends task and allows altering and sending of MockMailings
    """
    mailing_model = MockMailing

    def __init__(self, *args, **kwargs):
        MailingTask.__init__(self, *args, **kwargs)

        # make a list that keeps track of all sent messages (for asserting purposes)
        self.sent_messages = []

    def _send_mail(self, email, msg, result, mail_from=None, return_path=None):
        """
        Override _contact_addr_to cuz we dont wanna send to anyone that is called 'Kees' (see test)
        """
        if email == 'noreply@iqportalmail.com':
            email = ''

        # else call return parent
        return MailingTask._send_mail(self, email, msg, result, mail_from, return_path)

    def _build_message(self, contact, mailing, mail_from=None, license=None):
        """
        The mock-mailing-task overrides _build_message from MailingTask so we can keep track of which messages have been sent.
        """
        msg, result = MailingTask._build_message(self, contact, mailing, mail_from, license)
        self.sent_messages.append(msg)
        return (msg, result)

    def get_plain_text(self, html):
        """
        This is only to test the private method _plain_text(html) of the mailing task
        """
        return self._plain_text(html)

    def __repr__(self):
        return 'MockMailingTask'


