/**
 * # Copyright (c) 2009 BaseSoft B.V.
 * # See LICENSE for details.
 */

BASESOFT.test.TestCases.push(new YAHOO.tool.TestCase({

	name: "Test L10n Settings",
	assertClean: true,

	setUp: function() {
		// object that contains the L10n settings for the locales that we're testing
		// (only what's different in comparison to the default locale)
		this.L10n_settings = {
			en: {
			    DATE_FIELD_DELIMITER: '/',
			    MDY_MONTH_POSITION: 1,
			    MDY_DAY_POSITION: 2,
			    MDY_YEAR_POSITION: 3,
			    START_WEEKDAY: 0
			},
			nl: {
			    DATE_FIELD_DELIMITER: '-',
			    MDY_MONTH_POSITION: 2,
			    MDY_DAY_POSITION: 1,
			    MDY_YEAR_POSITION: 3,
			    START_WEEKDAY: 1
			}
		};
	},

	/**
	 * method for setting the specific locale settings in the environment
	 * @scope private
	 * @param {String} locale: the locale code that we want the settings from ('en' or 'nl')
	 */
	_setL10nEnvironment: function(locale) {
		var obj = this.L10n_settings[locale];
		for (var i in obj)
			BASESOFT.env.L10n[i] = obj[i];
	},

	/**
	 * test that the BASESOFT.util.date_* methods work properly
	 */
	test_date_to_string: function() {
		try {
			var d = new Date();
			d.setFullYear(1982, 11, 7); // set the date to the 7th of December 1982. day of the week was Tuesday
			d.setHours(14); // set the time to 14:05
			d.setMinutes(5);

			// switch the locales to 'en'
			this._setL10nEnvironment('en');

			// test the date->string conversion methods
			YAHOO.util.Assert.areEqual(BASESOFT.util.date_short(d), '12/07/1982');
			YAHOO.util.Assert.areEqual(BASESOFT.util.date_time_short(d), '12/07/1982 14:05');
			YAHOO.util.Assert.areEqual(BASESOFT.util.date_time_short(d, true),
				BASESOFT.env.L10n.WEEKDAYS_SHORT[d.getDay()] + ' 12/07/1982 14:05');
			YAHOO.util.Assert.areEqual(BASESOFT.util.time_short(d), '14:05');

			// test the date also without leading zeroes
			BASESOFT.env.L10n.SHOW_DATE_LEADING_ZERO = false;
			YAHOO.util.Assert.areEqual(BASESOFT.util.date_short(d), '12/7/1982');
			BASESOFT.env.L10n.SHOW_DATE_LEADING_ZERO = true;

			// switch the locales to 'nl'
			this._setL10nEnvironment('nl');

			// test the date->string conversion methods
			YAHOO.util.Assert.areEqual(BASESOFT.util.date_short(d), '07-12-1982');
			YAHOO.util.Assert.areEqual(BASESOFT.util.date_time_short(d), '07-12-1982 14:05');
			YAHOO.util.Assert.areEqual(BASESOFT.util.date_time_short(d, true),
				BASESOFT.env.L10n.WEEKDAYS_SHORT[d.getDay()] + ' 07-12-1982 14:05');
			YAHOO.util.Assert.areEqual(BASESOFT.util.time_short(d), '14:05');

			// test the date also without leading zeroes
			BASESOFT.env.L10n.SHOW_DATE_LEADING_ZERO = false;
			YAHOO.util.Assert.areEqual(BASESOFT.util.date_short(d), '7-12-1982');
			BASESOFT.env.L10n.SHOW_DATE_LEADING_ZERO = true;

		} catch(e){
			console.error(e);
			throw e;
		}
	}

}));
YAHOO.register("test_L10n_settings", null, {version: "1.0.0", build: "1"});
