/*
 * Copyright (c) 2008 BaseSoft Solutions B.V.
 * See LICENSE for details.
 */

(function() {
	"use strict";

BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "Contact Manager Test",

	setUp : function () {
		// create the container
		var form = document.createElement('form');
		this.div = document.createElement('div');
		this.div.id = 'id_contactmanager';
		form.appendChild(this.div);
		YAHOO.util.Dom.get('sandbox').innerHTML = "";
		YAHOO.util.Dom.get('sandbox').appendChild(form);

		// get a broker for calling the method for resetting the db
		var test_rpc = "/basesoft.web.test/selenium/contact_manager/test_data/rpc/";
		this._rpc = new BASESOFT.util.JsonBroker(test_rpc, 10000);
		this._garbage.push(this._rpc);

		//test contacts
		this.contacts = [{first_name: 'Johnny',last_name: 'Smith', email: 'johnny@mail-test.basesoft.com', contact_type: 'Contact'},
                         {first_name: 'Frank',last_name: 'Herbert', email: 'frank@mail-test.basesoft.com', contact_type: 'Contact'},
                         {first_name: 'Will',last_name: 'Franklin', email: 'will@mail-test.basesoft.com', contact_type: 'Contact'},
                         {first_name: 'Jerry',last_name: 'Seinfeld', email: 'jerry@mail-test.basesoft.com', contact_type: 'Contact'},
                         {first_name: 'George',last_name: 'Costanza', email: 'george@mail-test.basesoft.com', contact_type: 'Contact'},
                         {first_name: 'Elaine',last_name: 'Benes', email: 'elaine@mail-test.basesoft.com', contact_type: 'Contact'},
                         {first_name: 'Cosmo',last_name: 'Kramer', email: 'cosmo@mail-test.basesoft.com', contact_type: 'Contact'}];

        // test contact lists
        this.contact_lists = [  {name: "Seinfeld show"},
							   {name: "nobodies"},
							   {name: "common"}];

		//url for the ContactManagerPanel rpc requests
		this.sRpcUrl = {sRpcUrl: "/basesoft.web.contrib.contact/manager/rpc/"};
		this.loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.test/selenium/yui/account/', {executeIfLoggedOut : true});

	},

	/**
	 * test save contacts
	 */
	testSaveContacts : function () {
		var perms = ['create_contact', 'view_contact'];
		// login first
		this.loginrpc.call('create_account_and_login', perms, function(result){
			this.resume(function(){ // continue the test after login
				BASESOFT.env.account.id = result.id;
				try {
					var cpm = new BASESOFT.contact.ContactsPoolManager();
					this._garbage.push(cpm);

					cpm.cqc.saveContacts(this.contacts);

					cpm.cqc.subscribe("onContactsSaved", function(args){
						this.resume(function(){
							YAHOO.util.Assert.areEqual(args.length, 7);

							/*cpm.cq.subscribe('dataLoaded', function() {
								console.warn("LOADED");
								this.resume(function(){
									this._checkDataset(cpm.cq, [[
										 {contact: 'Cosmo  Kramer', person_email: 'cosmo@mail-test.basesoft.com', contact_type: 'Contact'},
										 {contact: 'Elaine  Benes', person_email: 'elaine@mail-test.basesoft.com', contact_type: 'Contact'},
										 {contact: 'Frank  Herbert', person_email: 'frank@mail-test.basesoft.com', contact_type: 'Contact'},
										 {contact: 'George  Costanza', person_email: 'george@mail-test.basesoft.com', contact_type: 'Contact'},
										 {contact: 'Jerry  Seinfeld', person_email: 'jerry@mail-test.basesoft.com', contact_type: 'Contact'},
										 {contact: 'Johnny  Smith', person_email: 'johnny@mail-test.basesoft.com', contact_type: 'Contact'},
										 {contact: 'Will  Franklin', person_email: 'will@mail-test.basesoft.com', contact_type: 'Contact'}
										 ]]);
								});
							}, null, this);
							cpm.cq.update();
							this.wait();*/
						});
					}, null, this);
					this.wait();
				} catch (e) {
					if (!e.delay)
						debugger;
					throw e;
				}
			});
		}, null, this);
		this.wait(); // wait for login
	},

	/*
	 * private; remove contacts
	 */
	_contactsMarkAsDeleted: function(args, cpm) {
		//args is the list with ids of the created contacts
		//ids to delete
		var ids = [ args[0], //Johnny
				    args[2], //Will
				    args[4] //George
				  ];
		var broker = new BASESOFT.util.JsonBroker('/basesoft.web.core/objects/rpc/', 60000);
		broker.call("mark_as_deleted", ["Contact", ids], function(result){
			this.resume(function(){
				cpm.cq.update();

				this._checkDataset(cpm.cq,
								   [[{contact: 'Cosmo  Kramer', person_email: 'cosmo@mail-test.basesoft.com'},
								     {contact: 'Elaine  Benes', person_email: 'elaine@mail-test.basesoft.com'},
								     {contact: 'Frank  Herbert', person_email: 'frank@mail-test.basesoft.com'},
								     {contact: 'Jerry  Seinfeld', person_email: 'jerry@mail-test.basesoft.com'}
								     ]
								   ]);
			});
		}, null, this);
		this.wait();
	},

	/**
	 * test remove contacts
	 */
	testContactsMarkAsDeleted: function () {
		var perms = ['create_contact', 'delete_contact', 'view_contact'];
		// login first
		this.loginrpc.call('create_account_and_login', perms, function(result){
			this.resume(function(){ // continue the test after login
				BASESOFT.env.account.id = result.id;

				var cpm = new BASESOFT.contact.ContactsPoolManager();
				this._garbage.push(cpm);

				cpm.cqc.saveContacts(this.contacts);

				cpm.cqc.subscribe("onContactsSaved", function(args){
					this.resume(function(){
						// remove some contacts
						this._contactsMarkAsDeleted(args, cpm);
					});
				},null, this);

				this.wait();
			});
		}, null, this);
		this.wait(); // wait for login
	},

	/**
	 * test save contact lists
	 */
	testSaveContactLists : function () {
		var perms = ['create_contact', 'create_contact_list', 'view_contact', 'view_contact_list'];
		// login first
		this.loginrpc.call('create_account_and_login', perms, function(result){
			this.resume(function(){ // continue the test after login
				BASESOFT.env.account.id = result.id;

				var clm = new BASESOFT.contact.ContactListsManager();
				this._garbage.push(clm);

				clm.cqc.saveContactLists(this.contact_lists);

				clm.cqc.subscribe("onContactListsSaved", function(e) {
					this.resume(function(){

						this._checkDataset(clm.cq,
										   [[  {name: "common", no_of_contacts: 0},
											   {name: "nobodies", no_of_contacts: 0},
											   {name: "Seinfeld show", no_of_contacts: 0}]
										   ]);
					});
				}, null, this);
				this.wait();
			});
		}, null, this);
		this.wait(); // wait for login
	},

	/*
	 * private; links contacts to a contact list
	 */
	_linkContacts: function(c_args, cl_args, clm, cpm) {

		//link contacts to contact lists
		var cids = [c_args[3],c_args[4],c_args[5],c_args[6]];

		//get the object that can do the linking
		var cl_combo = new BASESOFT.contact.ContactListsComboPanel(null, {contacts: cids});

		cl_combo.subscribe("onCLComboboxLoaded", function(e) {
			this.resume( function() {
				var cl = {};
				cl.id = cl_args[0];
				cl.name = this.contact_lists[0].name;
				cl_combo.linkContacts(cl);

				cl_combo.subscribe("onContactsLinked", function(e) {
					cl_combo.destroy();

					this.resume( function() {

						clm.cq.update();
						this._checkDataset(clm.cq,
										   [[  {name: "common", no_of_contacts: 0},
											   {name: "nobodies", no_of_contacts: 0},
											   {name: "Seinfeld show", no_of_contacts: 4}]
										   ]);
					});
				}, null, this);
				this.wait();
			});
		}, null, this);
		this.wait();
	},

	/**
	 * test link contacts to contact list
	 */
	testLinkContacts : function () {
		var perms = ['create_contact', 'create_contact_list', 'view_contact', 'view_contact_list', 'link_to_contact_list'];
		// login first
		this.loginrpc.call('create_account_and_login', perms, function(result){
			this.resume(function(){ // continue the test after login
				BASESOFT.env.account.id = result.id;

				var clm = new BASESOFT.contact.ContactListsManager();
				this._garbage.push(clm);

				clm.cqc.saveContactLists(this.contact_lists);

				clm.cqc.subscribe("onContactListsSaved", function(cl_args) {

					this.resume(function(){

						var cpm = new BASESOFT.contact.ContactsPoolManager();
						this._garbage.push(cpm);

						cpm.cqc.saveContacts(this.contacts);

						cpm.cqc.subscribe("onContactsSaved", function(c_args) {
							this.resume(function() {
								//link contacts to contact lists
								this._linkContacts(c_args, cl_args, clm, cpm);
							});
						}, null, this);
						this.wait();
					});
				}, null, this);
				this.wait();
			});
		}, null, this);
		this.wait(); // wait for login
	},

	/*
	 * private; links contacts to a contact list and then duplicates that contact list
	 */
	_linkAndDuplicate: function(c_args, cl_args, clm, cpm) {

		//link contacts to contact lists
		var cids = [c_args[3],c_args[4],c_args[5],c_args[6]];

		//get the object that can do the linking
		var cl_combo = new BASESOFT.contact.ContactListsComboPanel(null, {contacts: cids});

		cl_combo.subscribe("onCLComboboxLoaded", function(e) {
			this.resume( function() {
				var cl = {};
				cl.id = cl_args[0];
				cl.name = this.contact_lists[0].name;
				cl_combo.linkContacts(cl);
				cl_combo.subscribe("onContactsLinked", function(e) {
					cl_combo.destroy();
					this.resume( function() {

						//duplicate the contact list where contacts have been linked
						clm.cqc.duplicateContactLists([cl_args[0]]);
						clm.cqc.subscribe("onContactListsDuplicated", function(e) {
							this.resume(function(){
								try {

									this._checkDataset(clm.cq,
													   [[  {name: "Seinfeld show", no_of_contacts: 4},
														   {name: "common", no_of_contacts: 0},
														   {name: "nobodies", no_of_contacts: 0},
														   {name: "Seinfeld show", no_of_contacts: 4}]
													   ]);
								} catch (e) {
									e.delay || console.error(e);
									throw e;
								}
							});
						}, null, this);
						console.warn('waiting for onContactListsDuplicated');
						this.wait();
					});
				}, null, this);
				console.warn('waiting for onContactsLinked');
				this.wait();
			});
		}, null, this);
		console.warn('waiting for onCLComboboxLoaded');
		this.wait();
	},

	/**
	 * test duplicate contact lists
	 */
	testDuplicateContactLists : function () {
		var perms = ['create_contact_list', 'view_contact_list', 'create_contact', 'view_contact', 'link_to_contact_list'];
		// login first
		this.loginrpc.call('create_account_and_login', perms, function(result){
			this.resume(function(){ // continue the test after login
				BASESOFT.env.account.id = result.id;

				var clm = new BASESOFT.contact.ContactListsManager();
				this._garbage.push(clm);

				clm.cqc.saveContactLists(this.contact_lists);

				clm.cqc.subscribe("onContactListsSaved", function(cl_args) {

					this.resume(function(){

						var cpm = new BASESOFT.contact.ContactsPoolManager();
						this._garbage.push(cpm);

						cpm.cqc.saveContacts(this.contacts);

						cpm.cqc.subscribe("onContactsSaved", function(c_args) {

							this.resume(function() {
								// link contacts and duplicate a contact list
								this._linkAndDuplicate(c_args, cl_args, clm, cpm);
							});
						}, null, this);

						console.warn('waiting onContactsSaved');
						this.wait();
					});
				}, null, this);

				console.warn('waiting onContactListsSaved');
				this.wait();
			});
		}, null, this);
		this.wait();
	},

	/*
	 * private; links contacts to contact lists and then removes a contact lict
	 */
	_linkAndRemove: function(c_args, cl_args, contacts_delete_option) {
		//link contacts to contact lists
		var cids = [c_args[3],c_args[4],c_args[5],c_args[6]];
		//get the object that can do the linking
		var cl_combo = new BASESOFT.contact.ContactListsComboPanel(null, {contacts: cids});

		cl_combo.subscribe("onCLComboboxLoaded", function(e) {
			this.resume( function() {
				var cl_id = cl_args[0];
				var cl = {};
				cl.id = cl_id;
				cl.name = this.contact_lists[0].name;
				cl_combo.linkContacts(cl);
				cl_combo.subscribe("onContactsLinked", function(e) {
					cl_combo.destroy();
					this.resume(function() {
						//link some more contacts
						var cids1 = [c_args[0],c_args[2],c_args[5]];
						var cl_combo1 = new BASESOFT.contact.ContactListsComboPanel(null, {contacts: cids1});

						cl_combo1.subscribe("onCLComboboxLoaded", function(e) {
							this.resume( function() {
								var cl_id1 = cl_args[2];
								var cl1 = {};
								cl1.id = cl_args[2];
								cl1.name = this.contact_lists[2].name;
								cl_combo1.linkContacts(cl1);
								cl_combo1.subscribe("onContactsLinked", function(e) {
									cl_combo1.destroy();
									this.resume( function(){
										//remove the contact list where contacts have been linked
										this.cm.contact_lists.removeContactLists([cl_id], contacts_delete_option);
										this.cm.contact_lists.subscribe("onContactListsRemoved", function(e) {
											this.resume(function(){
												var expected = [];
												if(contacts_delete_option == 1)
													//check contacts were not delete
													expected =[[ {first_name: 'Johnny',last_name: 'Smith', email: 'johnny@mail-test.basesoft.com'},
										                         {first_name: 'Frank',last_name: 'Herbert', email: 'frank@mail-test.basesoft.com'},
										                         {first_name: 'Will',last_name: 'Franklin', email: 'will@mail-test.basesoft.com'},
										                         {first_name: 'Jerry',last_name: 'Seinfeld', email: 'jerry@mail-test.basesoft.com'},
										                         {first_name: 'George',last_name: 'Costanza', email: 'george@mail-test.basesoft.com'},
										                         {first_name: 'Elaine',last_name: 'Benes', email: 'elaine@mail-test.basesoft.com'},
										                         {first_name: 'Cosmo',last_name: 'Kramer', email: 'cosmo@mail-test.basesoft.com'}]
															   ];
												if(contacts_delete_option == 2)
													//check contacts that were not delete
													expected = [[{first_name: 'Johnny',last_name: 'Smith', email: 'johnny@mail-test.basesoft.com'},
										                         {first_name: 'Frank',last_name: 'Herbert', email: 'frank@mail-test.basesoft.com'},
										                         {first_name: 'Will',last_name: 'Franklin', email: 'will@mail-test.basesoft.com'},
										                         {first_name: 'Elaine',last_name: 'Benes', email: 'elaine@mail-test.basesoft.com'}]
															   ];
												if(contacts_delete_option == 3)
													//check contacts that were not delete
													expected = [[{first_name: 'Johnny',last_name: 'Smith', email: 'johnny@mail-test.basesoft.com'},
										                         {first_name: 'Frank',last_name: 'Herbert', email: 'frank@mail-test.basesoft.com'},
										                         {first_name: 'Will',last_name: 'Franklin', email: 'will@mail-test.basesoft.com'}]
															   ];
												this._checkDataset(this.cm.contacts.cq, expected);
											});
										}, null, this);
										this.wait();
									});
								},null, this);
								this.wait();
							});
						},null, this);
						this.wait();
					});
				}, null, this);
				this.wait();
			});
		}, null, this);
		this.wait();
	},

	/**
	 * private; to be called by the Remove Contact Lists test with different delete contacts option
	 */
	_removeContactLists : function (contacts_delete_option) {
		var perms = ['create_contact_list', 'delete_contact_list', 'view_contact_list'];
		// login first
		this.loginrpc.call('create_account_and_login', perms, function(result){
			this.resume(function(){ // continue the test after login
				BASESOFT.env.account.id = result.id;

				//add some contact lists
				this.cm.contact_lists.saveContactLists(this.contact_lists);

				this.cm.contact_lists.subscribe("onContactListsSaved", function(cl_args) {
					this.resume(function(){
						//add some Contacts
						this.cm.contacts.saveContacts(this.contacts);
						this.cm.contacts.subscribe("onContactsSaved", function(c_args) {
							this.resume(function() {
								// links some contacts and then remove contact list
								this._linkAndRemove(c_args, cl_args, contacts_delete_option);
							});
						}, null, this);
						this.wait();
					});
				}, null, this);
				this.wait();
			});
		}, null, this);
		this.wait(); // wait for login
	},

	/*
	 * test remove Contact Lists with CONTACTS_DO_NOT_DELETE option
	 * TODO: fix after we have this option back in contact manager
	 */
	 /*testRemoveContactListsNoContacts: function()
	 {
		this._removeContactLists(1);
	 },*/
	 /*
	 * test remove Contact Lists with CONTACTS_DELETE_ONLY_EXCLUSIVE option
	 * TODO: fix after we have this option back in contact manager
	 */
	 /*testRemoveContactListsExclusiveContacts: function()
	 {
		this._removeContactLists(2);
	 },*/
	 /*
	 * test remove Contact Lists with CONTACTS_DELETE_ALL option
	 * TODO: fix after we have this option back in contact manager
	 */
	 /*testRemoveContactListsAllContacts: function()
	 {
		this._removeContactLists(3);
	 },*/

	 /*
	  * test the contacts import
	  */
	 testImportContacts: function() {

		var perms = ['create_contact', 'create_contact_list', 'view_contact', 'view_contact_list', 'import_contacts',
		         'view_file', 'create_file'];

		// login first
		this.loginrpc.call('create_account_and_login', perms, function(result){
			this.resume(function(){ // continue the test after login
				BASESOFT.env.account.id = result.id;

				 this._rpc.call("create_file",[], function(result) {
						this.resume(function() {

							// don't open the panel for linking the contacts to a contact list after import
							// that is already tested
							this.ci = new BASESOFT.contact.ImportContactsPanel(null, {link_contacts: false});
							this._garbage.push(this.ci);

							// set the field mappings
							this.ci.field_mappings = [
								{file_header: "first name", column: "first_name", target: 1},
								{file_header: "middle name", column: "do_not_import", target: 0},
								{file_header: "last name", column: "last_name", target: 1},
								{file_header: "email address", column: "email", target: 1},
								{file_header: "phone number", column: "phone", target: 1},
								{file_header: "country", column: "country_name", target: 1}
							];
							// created file's hash
							this.ci.file_hash = {file_hashes: [result]};

							// fields separator
							this.ci.separator = ",";
							// do not import first row of the file
							this.ci.import_row1 = 0;

							// test the file validation and contats import
							this._testFileValidation(this.ci);

						});
				 }, null, this);
				 this.wait(10000);
			});
		}, null, this);
		this.wait(); // wait for login
	},

	_testFileValidation: function(ci)
	{
		// ci = ImportContactsPanel instance
		ci.validate_file();
		ci.subscribe("onValidationCheck", function(args) {
			//args are the validation result
			this.resume(function() {

				console.warn('resuming onValidationCheck', args);

				YAHOO.util.Assert.areEqual(args.length, 2);

				YAHOO.util.Assert.areEqual(args[0].row_index, 0);
				YAHOO.util.Assert.isTrue(typeof args[0].errors.country_name != "undefined");
				YAHOO.util.Assert.isTrue(typeof args[0].errors.email == "undefined");

				YAHOO.util.Assert.areEqual(args[1].row_index, 2);
				YAHOO.util.Assert.isTrue(typeof args[1].errors.country_name == "undefined");
				YAHOO.util.Assert.isTrue(typeof args[1].errors.email != "undefined");

				// set the updated and excluded rows so we can do the import
				ci.updated_rows = [{row_index: 1,
									 row_values: {first_name: "John",
												middle_name: "",
												last_name: "Smith",
												email: "john@mail-test.basesoft.com",
												phone: "0040740123456",
												country_name: "romania"}
									 }];
				ci.excluded_rows = [2];

				//import
				this._testContactsImport(ci);

			});
		}, null, this);
		this.wait();
	},

	_testContactsImport: function(ci)
	{
		// ci = ImportContactsPanel instance
		ci.import_contacts();
		ci.subscribe("onContactsImported", function(args) {
			// args contain the ids of the imported contacts
			// we have two imported contacts
			this.resume(function(){
				console.warn('resuming onContactsImported');

				YAHOO.util.Assert.areEqual(args.length, 2);
			});
		}, null, this);
		this.wait();
	},

	_checkDataset: function(cq, expected)
	{
		cq.subscribe("dataLoaded", function(e) {
			this.resume(function() {

				console.warn('resuming dataLoaded', cq);

				try {
					//no of tables
					YAHOO.util.Assert.areEqual(cq.getDataTables().length, expected.length);

					//for each table in dataset
					for (var k = 0; k < expected.length; k++) {
						var records = cq.getDataTables()[k].getRecordSet().getRecords();
						console.warn('records', records);


						//no of rows in table
						YAHOO.util.Assert.areEqual(records.length, expected[k].length);

						//rows in each table
						for (var i = 0; i < expected[k].length; i++) {
							//data in each row
							for (var key in expected[k][i]) {
								YAHOO.util.Assert.areEqual(records[i].getData(key), expected[k][i][key]);
							}
						}
					}

				} catch (e) {
					console.error(e);
					throw e;
				}
			});
		}, this, true);

		console.warn('waiting for dataLoaded');
		this.wait();
	}

}));
YAHOO.register("test_contact_manager", null, {version: "1.0.0", build: "1"});

})();
