/**
 * # Copyright (c) 2009 BaseSoft B.V.
 * # See LICENSE for details.
 */
BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name : "Custom Query Test",
	assertClean : true,

	setUp : function() {
		this.sandbox = YAHOO.util.Dom.get('sandbox');
		this.testRpc = new BASESOFT.util.JsonBroker('/basesoft.web.test/selenium/custom_query/rpc/', {
			executeIfLoggedOut : true
		});
		this.loginRpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {
			executeIfLoggedOut : true
		});
	},

	test_basic : function() {

		var start = function() {

			this.loginRpc.call('login', ['test_account', 'test_password'], function(result) {
				this.resume(function() {

					BASESOFT.env.account.id = result.id;

					try {
						console.debug('logged in.. calling fillData..');
						fillData.call(this);
					} catch (e) {
						e.delay || console.error(e);
						throw e;
					}
				});
			}, null, this);

			this.wait(this.timeout);
		};

		var fillData = function() {

			this.testRpc.call('fill_data', [], function(ids) {

				this.resume(function() {

					console.info('creating cq');

					var configs = {
						name : 'basesoft.web.test.TestContactCustomQuery',
						show_column_select : true,
						show_row_selector : true,
						column_properties : {
							contact_id : {
								formatter : function(elCell, oRecord, oColumn, oData) {

									var a = document.createElement("a");
									a.appendChild(document.createTextNode(oData));
									elCell.appendChild(a);

									YAHOO.util.Event.addListener(a, "click", function(e, first_name) {
										alert('clicky ' + first_name);
									}, oRecord.getData("person_first_name"), this);
								}
							}
						}
					};

					var cq = new BASESOFT.widget.CustomQuery(configs);
					cq.appendTo(this.sandbox);
					this._garbage.push(cq);

					// when the cq is added it starts updating directly. this results in fireing two events
					// and we have to wait with continueing the test before both are fired.. so lets wait...

					var dataLoaded1 = false;
					var filtersLoaded1 = false;

					var dataLoaded1Cb = function() {
						cq.unsubscribe('dataLoaded', dataLoaded1Cb);
						dataLoaded1 = true;
						if (dataLoaded1 && filtersLoaded1)
							continueTest1.call(this, cq, ids);

					};

					var filtersLoaded1Cb = function() {
						cq.unsubscribe('filtersLoaded', filtersLoaded1Cb);
						filtersLoaded1 = true;
						if (dataLoaded1 && filtersLoaded1)
							continueTest1.call(this, cq, ids);
					};

					cq.on('dataLoaded', dataLoaded1Cb, null, this);
					cq.filterFieldset.on('filtersLoaded', filtersLoaded1Cb, null, this);

					this.wait();

				});

			}, null, this, 120000);

			this.wait(120000);

		};


		var continueTest1 = function(cq, ids) {

			this.resume(function() {

				try {
					console.info('received filters loaded');

					cq.filterFieldset.setGroupBy('gender');
					cq.filterFieldset.addFilter('contact_id', BASESOFT.widget.filter.FILTER_NOT_EQUAL, ids[3]['contact_id']);
					cq.filterFieldset.addFilter('contact_id', null, ids[4]['contact_id']);
					cq.filterFieldset.addFilter('contact_id', null, ids[6]['contact_id']);
					cq.filterFieldset.addFilter('person_created', BASESOFT.widget.filter.FILTER_BETWEEN, new Date(2010, 0, 1), new Date(2500, 0, 1));
					// test fails in about 500 years ;)
					var filters = cq.filterFieldset.getFilterValues();

					YAHOO.util.Assert.isTrue(BASESOFT.util.isEqual(filters, {
						contact_id : {
							logicalOperator : BASESOFT.widget.filter.LOGICAL_OPERATOR_OR,
							operator : BASESOFT.widget.filter.FILTER_NOT_EQUAL,
							values : [ids[3]['contact_id'], ids[4]['contact_id'], ids[6]['contact_id']]
						},

						person_created : {
							logicalOperator : BASESOFT.widget.filter.LOGICAL_OPERATOR_OR,
							operator : BASESOFT.widget.filter.FILTER_BETWEEN,
							values : [[new Date(2010, 0, 1), new Date(2500, 0, 1)]]
						}
					}));

					cq.on('dataLoaded', function() {
						this.resume(function() {
							try {

								var tables = cq.getDataTables();

								// we expect two datasets (male, female) with 25 rows (first page)
								YAHOO.util.Assert.areEqual(tables.length, 2);
								YAHOO.util.Assert.areEqual(25, tables[0].getRecordSet().getLength());
								YAHOO.util.Assert.areEqual(25, tables[1].getRecordSet().getLength());

								// the paginator should be set to 99/98 rows
								YAHOO.util.Assert.areEqual(98, tables[0].get('paginator').getTotalRecords());
								YAHOO.util.Assert.areEqual(99, tables[1].get('paginator').getTotalRecords());

							} catch (e) {
								e.delay || console.error(e);
								throw e;
							}
						});
					}, null, this);

					cq.update();
					this.wait();

				} catch (e) {
					e.delay || console.error(e);
					throw e;
				}
			}, null, this);

		};


		start.call(this);

	},

	test_with_toolbar : function () {
		var start = function() {

			this.loginRpc.call('login', ['test_account', 'test_password'], function(result) {
				BASESOFT.env.account.id = result.id;
				this.resume(function() {
					try {
						console.debug('logged in.. calling fillData..');
						fillData.call(this);
					} catch (e) {
						e.delay || console.error(e);
						throw e;
					}
				});
			}, null, this);

			this.wait(this.timeout);
		};
		var fillData = function() {

			this.testRpc.call('fill_data', [], function(ids) {

				this.resume(function() {

					try {
						console.info('creating cq');

						var configs = {
							name : 'basesoft.web.test.TestContactCustomQuery',
							show_column_select : true,
							show_row_selector : true,
							export_excel : true,
							use_toolbar : true
						};

						var cq = new BASESOFT.widget.CustomQuery(configs);
						cq.appendTo(this.sandbox);
						this._garbage.push(cq);


					} catch (e) {
						e.delay || console.error(e);
						throw e;
					}
				});
			}, null, this, 60000);
			this.wait(60000);

		};

		start.call(this);


	},


	test_period_select : function() {
		var start = function() {

			this.loginRpc.call('login', ['test_account', 'test_password'], function() {
				this.resume(function() {
					try {
						console.debug('logged in.. calling fillData..');
						fillData.call(this);
					} catch (e) {
						e.delay || console.error(e);
						throw e;
					}
				});
			}, null, this);

			this.wait(this.timeout);
		};
		var fillData = function() {

			this.testRpc.call('fill_data', [], function(ids) {

				this.resume(function() {

					try {
						console.info('creating cq');

						var configs = {
							name : 'basesoft.web.test.TestContactCustomQuery',
							show_column_select : true,
							show_row_selector : true,
							column_properties : {
								contact_id : {
									formatter : function(elCell, oRecord, oColumn, oData) {

										var a = document.createElement("a");
										a.appendChild(document.createTextNode(oData));
										elCell.appendChild(a);

										YAHOO.util.Event.addListener(a, "click", function(e, first_name) {
											alert('clicky ' + first_name);
										}, oRecord.getData("person_first_name"), this);
									}
								}
							},
							filterConfig : {
								filterDisplayTypeMap : {
									person_created : BASESOFT.widget.filter.LAYOUT_PERIOD_SELECT
								}
							}
						};

						var cq = new BASESOFT.widget.CustomQuery(configs);
						cq.appendTo(this.sandbox);
						this._garbage.push(cq);
						/*
						 cq.filterFieldset.on('filtersLoaded', function(){
						 this.resume(function(){
						 try {

						 // TODO: add asserts.
						 }
						 catch (e) {
						 e.delay || console.error(e);
						 throw e;
						 }
						 }, null, this);
						 }, null, this);
						 this.wait();
						 */

					} catch (e) {
						e.delay || console.error(e);
						throw e;
					}
				});
			}, null, this, 60000);
			this.wait(60000);

		};

		start.call(this);
	}
}));
