/**
 * Copyright (c) 2008 BaseSoft B.V.
 * See LICENSE for details.
 */
if(typeof BASESOFT.test == 'undefined') {
	BASESOFT.test = {};
}

(function() {

	BASESOFT.test.TestWidget = function(manager, attr) {
		attr = attr || {};
		attr.loaderName = 'test_dashboard';
		attr.objectName = 'TestWidget';
		attr.help = 'This is a help text test for dashboard';

		BASESOFT.test.TestWidget.superclass.constructor.call(this, manager, attr);
	};

	YAHOO.extend(BASESOFT.test.TestWidget, BASESOFT.dashboard.Widget, {
		setBody : function() {

			BASESOFT.dashboard.Widget.superclass.setBody.call(this, "");

			BASESOFT.load('test_dashboard_es6', function() {
				this.body.innerHTML = '<div class="test-widget-js-gettext">' + gettext('test Hello World!') + '</div>';
				this.body.innerHTML += '<div class="test-widget-es6-gettext">' + BASESOFT.test.get_dashboard_es6_text() + '</div>';

			}, this);





		}
	});

	BASESOFT.dashboard.Manager.prototype.moduleMapping.TestWidget = BASESOFT.test.TestWidget;


	// TestCases
	if(typeof BASESOFT.test.TextCases != 'undefined') {
		BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

			name : "Dashboard Test",
			assertClean : true,

			setUp : function() {
				this.div = document.createElement('div');
				this.div.id = 'bs-dashboard-container';
				YAHOO.util.Dom.get('sandbox').appendChild(this.div);

				this._account_rpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {
					executeIfLoggedOut : true
				});

			},
			// must be called after login
			_initManager : function() {
				this.manager = new BASESOFT.dashboard.Manager();

				this.manager.init({
					'composition_el' : this.div
				});

				this.manager.widgets = [{
					name : 'test_dashboard'
				}];

				// add the this.manager to the testcase garbage so it gets destroyed at tearDown
				this._garbage.push(this.manager);
			},
			tearDown : function() {

				// kill all growls if any
				BASESOFT.env.growler.killAll();

				// call parent by static call (no superclass possible!)
				BASESOFT.tool.TestCase.prototype.tearDown.call(this);

			},
			test_open_new_dashboard : function() {

				this._account_rpc.call('login', ['test_account', 'test_password'], function(result) {
					this.resume(function() {
						BASESOFT.env.account.id = result.id;

						this._initManager();

						this.manager.openNew();

						YAHOO.util.Assert.areEqual(1, this.manager.canvas.children['tabs'].length);

					});
				}, null, this);

				console.warn('waiting');

				this.wait();

			},
			test_save_dashboard : function() {

				try {

					this._account_rpc.call('login', ['test_account', 'test_password'], function(result) {
						this.resume(function() {

							try {

								BASESOFT.env.account.id = result.id;

								this._initManager();

								this.manager.openNew();

								this.manager.canvas.set('name', 'my dashboard');

								var tab = new BASESOFT.dashboard.Tab(this.manager);
								this.manager.canvas.add('tabs', tab);

								var block = tab.children['blocks'][0];
								var column1 = block.children['columns'][0];

								var column2 = new BASESOFT.dashboard.Column(this.manager);
								block.add('columns', column2);

								var w1 = new TestWidget(this.manager);
								column1.add('widgets', w1);

								var w2 = new TestWidget(this.manager);
								column2.add('widgets', w2);

								var event = this.manager.save();

								event.subscribe(function() {
									this.resume(function() {
										// noop
									});
								}, null, this);
							} catch (e) {
								console.error(e);
							}
							this.wait(5 * 60 * 1000);

						});
					}, null, this);

					this.wait(5 * 60 * 1000);

				} catch (e) {
					e.delay || console.error(e);
					throw e;
				}

			},
			test_load_dashboard : function() {
				this._account_rpc.call('login', ['test_account', 'test_password'], function(result) {
					this.resume(function() {

						BASESOFT.env.account.id = result.id;

						var state = {
							dashboard : {
								objectName : 'Dashboard',
								children : {
									tabs : [{
										objectName : 'DashboardTab',
										label : 'My Tab',
										children : {
											blocks : [{
												objectName : 'DashboardBlock',
												children : {
													columns : [{
														objectName : 'DashboardColumn',
														children : {
															widgets : [{
																objectName : 'TestWidget',
																title : 'My Test Widget'
															}]
														}
													}]
												}
											}]
										}
									}, {
										objectName : 'DashboardTab',
										label : 'My Second Tab',
										children : {}
									}]
								}
							}
						};

						try {
							this._initManager();

							this.manager.subscribe('widgetsLoaded', function() {

								this.resume(function() {

									this.manager.loadState(state);

									// wait a bit for loading.. that gives the calendar widget time to load
									var _this = this;
									setTimeout(function() {
										_this.resume(function() {
											YAHOO.util.Assert.areEqual(2, _this.manager.canvas.children['tabs'].length);
											console.info('test finished');
										});
									}, 5000);

									this.wait(5 * 60 * 1000);

								});
							}, null, this);

							this.wait();

						} catch(e) {
							e.delay || console.error(e);
							throw e;
						}

					});
				}, null, this);

				this.wait(5 * 60 * 1000);

			},
			test_remove_tab : function() {

				try {
					this._initManager();
					this.manager.openNew();

					var tab1 = new BASESOFT.dashboard.Tab(this.manager);
					this.manager.canvas.add('tabs', tab1);

					var tab2 = new BASESOFT.dashboard.Tab(this.manager);
					this.manager.canvas.add('tabs', tab2);

					var tab3 = new BASESOFT.dashboard.Tab(this.manager);
					this.manager.canvas.add('tabs', tab3);

					tab2.remove();
				} catch (e) {
					console.error(e);
					throw e;
				}

			}
			// TODO: make a test for activate/decativate widgets - #3175

		}));
	}
})();

YAHOO.register("test_dashboard", BASESOFT.test.TestWidget, {
	version : "1.0.0",
	build : "1"
});
