/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 */

(function() {

var UA = YAHOO.util.UserAction;
var Dom = YAHOO.util.Dom;

BASESOFT.test.TestCases.push(new YAHOO.tool.TestCase({

	name: "DataPilotSelector TestCase",

	setUp : function () {
		var sandbox = YAHOO.util.Dom.get('sandbox');
		sandbox.innerHTML = ''; // clear

		this.tree_div = document.createElement('div');
		this.tree_div.style.cssFloat = 'left';
		this.tree_div.style.width = '20%';
		sandbox.appendChild(this.tree_div);

		this.dps_div = document.createElement('div');
		this.dps_div.style.cssFloat = 'left';
		this.dps_div.style.width = '50%';
		sandbox.appendChild(this.dps_div);

		var div = document.createElement('div');
		div.style.clear = 'both';
		sandbox.appendChild(div);

		// create the CubeBrowser and DataPilotSelector
		try {
			this.cb = new BASESOFT.widget.mdx.CubeBrowser(this.tree_div, '/basesoft.web.test/selenium/cube_browser/');
			this.dps = new BASESOFT.widget.DataPilotSelector(this.dps_div, this.cb);
		} catch (e) {
			console.error(e);
			throw e;
		}

		// create a textfield and a button for mdx validation

		var hr = document.createElement('hr');
		sandbox.appendChild(hr);

		var text = document.createElement('textarea');
		text.style.width = '1024px';
		text.style.height = '100px';
		text.id = 'text_mdx';
		sandbox.appendChild(text);

		var input = document.createElement('input');
		input.type = 'button';
		input.value = 'mdx!';
		input.id = 'generate_mdx';
		var _this = this;
		input.onclick = function() {
			text.value = _this.dps.getMdx();
		};
		sandbox.appendChild(input);
	},

	_dragDropNode: function(nNode, target) {

		//console.debug('_dragDropNode', nNode, target);

		targets = {
			'data_fields':   { pageX: Dom.getX('bs-dsp-data_fields-0'), pageY: Dom.getY('bs-dsp-data_fields-0') },
			'row_fields':	{ pageX: Dom.getX('bs-dsp-row_fields-0'), pageY: Dom.getY('bs-dsp-row_fields-0') },
			'column_fields': { pageX: Dom.getX('bs-dsp-column_fields-0'), pageY: Dom.getY('bs-dsp-column_fields-0') }
		};

		// this is the actual  div on which we start dragging
		var div = Dom.get(nNode.labelElId);
		var x = Dom.getX(div) + 2;
		var y = Dom.getY(div) + 2;

		// Get the DD from the first child and click it
		// Then move it to the data_fields td and drop it
		YAHOO.log('_dragDropNode: Mousedown on ' + nNode.ddfield.getEl() + 'x:'+x+' y:'+y, 'info', 'test_datapilot');
		nNode.ddfield.handleMouseDown({
			pageX: x, pageY: y, // coords of the 'Quantity td'
			target: nNode.ddfield.getEl()
		});
		YAHOO.log('MouseMove to '+targets[target].pageX+','+targets[target].pageY, 'info', 'test_datapilot');
		YAHOO.util.DDM.handleMouseMove(targets[target]);

		YAHOO.log('MouseUp on '+targets[target].pageX+','+targets[target].pageY, 'info', 'test_datapilot');
		YAHOO.util.DDM.handleMouseUp(targets[target]);
	},

	/**
	 * Sets the datapilot_selector up with a simple MDX statement and asserts its mdx result
	 */
	testDataPilotSelector : function () {

		// scope problems below so lets reference 'this'
		var _this = this;

		// prepare a function for asserting.. this is called when the test finishes
		var assert_mdx = function() {

			// as in IE drag'n drop testing isnt working we cant assert mdx.. so we just return here.. too bad!
			if (YAHOO.env.ua.ie > 0)
				return;

			// now press the mdx button to generate mdx!
			UA.click('generate_mdx');

			// assert mdx
			var expected_mdx = "SELECT {[CRSO].[Participant 2].[User 1],[CRSO].[Participant 2].[User 2]} ON COLUMNS, \n"+
			"{[Kwak].[All Members]} ON ROWS \n" +
			"FROM [TestCube] \n" +
			"WHERE ([Measures].[Quantity],[Measures].[Amount]) \n";
			YAHOO.util.Assert.areEqual(Dom.get('text_mdx').value, expected_mdx, "MDX assert failed");
		};

		// select the TestCube.. this is async so we have to wait
		this.cb.selectCube('TestCube').subscribe(function(type, args) {

			// Once all nodes are loaded we open the Measure node (this is the onclick of it)
			// this will fire the 'expandComplete' on the tree (see below)
			YAHOO.log('Opening Measures', 'info', 'test_datapilot');
			UA.click('ygtvt5');

		}, this);

		// subscribe whenever a node is expanded..
		this.cb._tree.subscribe("expandComplete", function(nNode) {
			try {

				YAHOO.log('DataPilotSelector TestCase: Received event expandComplete: '+nNode, 'info', 'test_datapilot');

				// when the measures node is expanded we move Quantity to data fields
				if (nNode.label == 'Measures') {

					YAHOO.log('Dragging Quanitiy and Amount', 'info', 'test_datapilot');
					_this._dragDropNode(nNode.children[0], 'data_fields');
					_this._dragDropNode(nNode.children[1], 'data_fields');

					// open the Dimension nodes.. for some reason directly opening this node doesnt work.
					YAHOO.log('Opening Dimension', 'info', 'test_datapilot');
					setTimeout(function() { UA.click('ygtvt6'); }, 1);
				}

				if (nNode.label == 'Dimensions') {
					YAHOO.log('Dragging Kwak', 'info', 'test_datapilot');
					_this._dragDropNode(nNode.children[1], 'row_fields');

					YAHOO.log('Opening CRSO', 'info', 'test_datapilot');
					setTimeout(function() { UA.click('ygtvt10'); }, 1);
				}

				if (nNode.label == 'CRSO') {
					YAHOO.log('Opening Participant 2', 'info', 'test_datapilot');
					setTimeout(function() { UA.click('ygtvt13'); }, 1);
				}

				if (nNode.label == 'Participant 2') {
					YAHOO.log('Dragging User1 and User2 to column_fields', 'info', 'test_datapilot');
					_this._dragDropNode(nNode.children[0], 'column_fields');
					_this._dragDropNode(nNode.children[1], 'column_fields');

					// End of the datapilot setup.. Tell the testrunner to assert mdx
					YAHOO.log('Resuming test', 'info', 'test_datapilot');
					_this.resume(assert_mdx);
				}

			} catch (e) {
				console.error(e);
				throw e;
			}

		}, this);

		// this tells the yui-testrunner that the test insnt finished.. the resume above function will do the assertion
		this.wait();
	}

}));

}());


YAHOO.register("test_datapilot_selector", null, {version: "1.0.0", build: "1"});


