/*
 * Copyright (c) 2008 BaseSoft B.V.
 * See LICENSE for details.
 */

(function() {
	"use strict";

	var TestPanel = function() {

		TestPanel.superclass.constructor.call(this, {name: 'test-design-panel', height: '700px', width: '800px'});


	};
	YAHOO.extend(TestPanel, BASESOFT.widget.Panel, {

		setHeader : function() {
			TestPanel.superclass.setHeader.call(this, 'initital title');
		},

		setFooter : function() {
			var i;
			var div_buttons = document.createElement("div");
			div_buttons.id = YAHOO.util.Dom.generateId();

			var saveb = new YAHOO.widget.Button({
				disabled : true,
				container : div_buttons,
				label : gettext("Save (disabled)")
			});

			var savecb = new YAHOO.widget.Button({
				container : div_buttons,
				label : gettext("Save and close")
			});
			savecb.on("click", function(e) {
				this.destroy();
			}, null, this);

			var cancelb = new YAHOO.widget.Button({
				container : div_buttons,
				label : gettext("Cancel")
			});
			cancelb.on("click", function(e) {
				this.destroy();
			}, null, this);

			TestPanel.superclass.setFooter.call(this, div_buttons);
		}

	});

	BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({
		name : "Form Manager Test",

		setUp : function() {
			this.sandbox = $('#sandbox')[0];
		},

		test_design1 : function() {

			var make_stuff = function(parentEl) {

				// create a tab view with 2 tabs
				var tabView = new BASESOFT.widget.TabView();
				YAHOO.util.Dom.addClass(tabView.get('element'), 'bs-tabview');

				var tab1 = new BASESOFT.widget.Tab({
					label : "Tab 1",
					active : true
				});
				tabView.addTab(tab1);

				var tab2 = new BASESOFT.widget.Tab({
					label : "Tab 2",
					active : true
				});
				tabView.addTab(tab2);

				var tab3 = new BASESOFT.widget.Tab({
					label : "Tab 3",
					active : true
				});
				tabView.addTab(tab3);

				// add a form to first tab
				var myform = new BASESOFT.util.form.Form();

				myform.appendTo(tab1.get('contentEl'));

				// input txt
				myform.addInput(BASESOFT.util.form.TextInput, {
					label : 'TextInput'
				}, {
					name : 'input_text'
				});

				var disabledButtonInput = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {
					name : 'btn_disabled',
					disabled : true,
					label : 'ButtonInput disabled'
				});

				// button (opens a new panel)
				var buttonInput = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {
					name : 'btn_open_panel',
					label : 'Open panel'
				});
				buttonInput.on('click', function() {
					var panel = new TestPanel();
					make_stuff(panel.body);

					window.panel = panel;

				}, null, this);

				// disable form
				buttonInput = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {
					name : 'btn_disable_form',
					label : 'Disable form'
				});
				buttonInput.on('click', function() {
					myform.set('disabled', true);
					BASESOFT.util.setTimeout(function() {
						myform.set('disabled', false);
					}, 5000, null, this);
				}, null, this);

				// file upload
				var input1 = myform.addInput(BASESOFT.util.form.FileUpload, {
					label : 'Upload file'
				}, {
					name : 'my_uploader',
					maxFiles : 1,
					allowMultipleFiles : false
				});

				// pdf button
				var btnPdf = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {
					name : 'btn_pdf',
					type : "button",
					label : "\ue854"
				});

				// print button
				var btnPrint = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {
					name : 'btn_print',
					type : "button",
					label : "\ue859"
				});

				// combobox list
				var ds = new YAHOO.util.LocalDataSource([ 'value1', 'value2', 'value3' ]);
				var combobox = myform.addInput(BASESOFT.util.form.Combobox, {
					'label' : 'my combobox'
				}, {
					'name' : 'combobox',
					'datasource' : ds
				});

				// Growls
				var buttonInputWarning = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {
					name : 'btn_show_growl_warning',
					label : 'Growl :: Warning'
				});
				buttonInputWarning.on('click', function() {
					BASESOFT.env.growler.growl(new BASESOFT.widget.Growl({
						type : 'warning',
						title : gettext('Warning'),
						message : gettext('warning message lorum ipsum etc pleurum'),
						lifetime : 500000
					}));
				}, null, this);

				var buttonInputInformation = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {
					name : 'btn_show_growl_information',
					label : 'Growl :: Information'
				});
				buttonInputInformation.on('click', function() {
					BASESOFT.env.growler.growl(new BASESOFT.widget.Growl({
						type : 'information',
						title : gettext('Information'),
						message : gettext('Informational message to make you tell something'),
						lifetime : 500000
					}));
				}, null, this);

				var buttonInputCritical = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {
					name : 'btn_show_growl_critical',
					label : 'Growl :: Critical'
				});
				buttonInputCritical.on('click', function() {
					BASESOFT.env.growler.growl(new BASESOFT.widget.Growl({
						type : 'critical',
						title : gettext('Critical Error'),
						message : gettext('critical message lorum ipsum etc pleurum'),
						lifetime : 500000
					}));
				}, null, this);

				// checkbox (switches to viewmode)
				var chk1 = myform.addInput(BASESOFT.util.form.CheckboxInput, {
					label : 'Checkbox (toggle view mode)'
				}, {
					name : 'checkbox1',
					value : false
				});
				chk1.on('valueChange', function(args) {
					myform.set('asView', args.newValue);
					myform.updateLayout();
				});

				// select
				var input = myform.addInput(BASESOFT.util.form.Select, {
					label : 'select-label'
				}, {
					name : 'input_select',
					value : 'option-value-4'
				});
				for (var i = 0; i < 10; i++)
					input.addOption('option-name-' + i, 'option-value-' + i);

				// fancy select
				var fs = new BASESOFT.widget.FancySelect(document.createElement("div"), {
					name : 'fancy_select',
					orderSelectedItems : true,
					defaultoptions : [ {
						label : 'One',
						value : '1'
					}, {
						label : 'Two',
						value : '2'
					}, {
						label : 'Three',
						value : '3'
					} ],
					selectedoptions : [ {
						label : 'Four',
						value : '4'
					} ]
				});
				fs.addCustomButton({
					label : gettext("custom buttom example")
				}, function() {
				}, null);

				var fs_container = new BASESOFT.util.form.InputContainer({
					label : gettext('Fancy select'),
					labelDirection : BASESOFT.util.form.LABEL_TOP
				});
				fs_container.add(fs);
				myform.add(fs_container);

				var dt = myform.addInput(BASESOFT.util.form.DateTime, {
					label : 'select date'
				}, {
					name : 'my_datetime'
				});

				var wikiInput = myform.addInput(BASESOFT.util.form.WikiInput, {
					label : 'wiki_input'
				}, {
					name : 'wiki_input'
				});

				var inputs = [ {
					label : 'Value A',
					input : new BASESOFT.util.form.TextInput({
						name : 'value_a'
					})
				}, {
					label : 'Value B',
					input : new BASESOFT.util.form.TextInput({
						name : 'value_b'
					})
				} ];

				var rowcontrolcontainer = new BASESOFT.util.form.RowControlContainer({
					inputs : inputs
				}, {
					name : 'myrowcontrol'
				});
				myform.add(rowcontrolcontainer);

				// render stuff

				myform.set('layout', new BASESOFT.util.form.Fieldset({
					legend : 'My Fieldset',
					children : [
							new BASESOFT.util.form.ColumnView({
								children : [

										new BASESOFT.util.form.Column({
											children : [ 'input_text', 'btn_disabled', 'btn_pdf', 'btn_print',
													'btn_open_panel', 'btn_disable_form', 'my_datetime',
													'btn_show_growl_information', 'btn_show_growl_warning',
													'btn_show_growl_critical' ]
										}),

										new BASESOFT.util.form.Column({
											children : [ 'input_select', 'checkbox1', 'combobox' ]
										}) ]
							}), new BASESOFT.util.form.Layout({
								children : [ 'fancy_select', 'wiki_input', 'myrowcontrol', 'my_uploader' ]
							})

					]
				}));

				myform.updateLayout();
				tabView.appendTo(parentEl);

				// goto first tab
				tabView.set('activeIndex', 0);

			};

			BASESOFT.load({
				require : [ 'jquery', 'dragdrop', 'iq_fancyselect', 'iq_form_manager', 'iq_tabview', 'iq_wiki_input',
						'autocomplete' ],
				onSuccess : function() {

					this.resume(function() {
						try {
							make_stuff.call(this, this.sandbox);
						} catch (e) {
							console.error(e);
							throw e;
						}
					});

				},
				scope : this
			});
			this.wait();
		}

	}));

	YAHOO.register("test_form_manager", null, {
		version : "1.0.0",
		build : "1"
	});

})();
