/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 */

BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "Test Dialogs",
	assertClean: true,

	setUp: function(){
		this.div = document.createElement('div');
		this.div.id = 'bs-dialog-container';
		YAHOO.util.Dom.get('sandbox').appendChild(this.div);
	},

	testSimpleDialogs: function(){
		try {
			var dialogs_to_test = {
				'Information': BASESOFT.widget.dialog.Information,
				'Warning': BASESOFT.widget.dialog.Warning,
				'Critical': BASESOFT.widget.dialog.Critical
			};

			for (var key in dialogs_to_test) {
				var dialog = new dialogs_to_test[key]({
					title: key + ' dialog title',
					text: 'Some ' + key + ' text'
				});
				this._garbage.push(dialog);

				YAHOO.util.Assert.areEqual(dialog.get('title'), key + ' dialog title');
				dialog.set('title', 'new ' + key + ' dialog title');
				YAHOO.util.Assert.areEqual(dialog.get('title'), 'new ' + key + ' dialog title');
				dialog.buttons['ok'] || YAHOO.util.Assert.fail('we should have an "ok" button');

			}
		}
		catch (e) {
			console.error(e);
			throw e;
		}
	},

	testQuestionDialog: function(){
		try {
			var dialog = new BASESOFT.widget.dialog.Question({
				title: 'Question dialog title',
				text: 'Some question text ?'
			});
			this._garbage.push(dialog);

			YAHOO.util.Assert.areEqual(dialog.get('title'), 'Question dialog title');
			dialog.set('title', 'new question dialog title');
			YAHOO.util.Assert.areEqual(dialog.get('title'), 'new question dialog title');
			dialog.buttons['yes'] || YAHOO.util.Assert.fail('we should have a "yes" button');
			dialog.buttons['no'] || YAHOO.util.Assert.fail('we should have a "no" button');

		}
		catch (e) {
			console.error(e);
			throw e;
		}
	},

	testFeedbackDialog: function(){
		try {
			var dialog = new BASESOFT.widget.dialog.Feedback({
				title: 'Feedback dialog title',
				text: 'Please provide some feedback',
				feedback: 'my value'
			});
			this._garbage.push(dialog);

			// add the listener to feedbackFinished event
			// this test doesn't check what happens when you click the button,
			// but it's good for developing
			dialog.feedbackFinished.subscribe(function() {
				console.debug('feedback', dialog.get('feedback'));
			});
			YAHOO.util.Assert.areEqual(dialog.get('title'), 'Feedback dialog title');
			YAHOO.util.Assert.areEqual(dialog.get('feedback'), 'my value');
			dialog.set('title', 'new feedback dialog title');
			dialog.set('feedback', 'new feedback value');
			YAHOO.util.Assert.areEqual(dialog.get('title'), 'new feedback dialog title');
			YAHOO.util.Assert.areEqual(dialog.get('feedback'), 'new feedback value');
			dialog.buttons['ok'] || YAHOO.util.Assert.fail('we should have an "ok" button');
			dialog.buttons['cancel'] || YAHOO.util.Assert.fail('we should have a "cancel" button');

		}
		catch (e) {
			console.error(e);
			throw e;
		}
	},

	testLoginDialog: function(){
		try {
			var new_dialog = new BASESOFT.util.registration.LoginDialog({showregistration: false});
			this._garbage.push(new_dialog);
			YAHOO.util.Assert.isUndefined(new_dialog.buttons['register'], 'we shouldnt have a "register" button');

			var dialog = new BASESOFT.util.registration.LoginDialog({
				title: 'Please login'
			});
			this._garbage.push(dialog);

			YAHOO.util.Assert.areEqual(dialog.get('title'), 'Please login');
			dialog.set('title', 'new login title');
			YAHOO.util.Assert.areEqual(dialog.get('title'), 'new login title');
			dialog.buttons['login'] || YAHOO.util.Assert.fail('we should have a "login" button');
		}
		catch (e) {
			console.error(e);
			throw e;
		}
	}

}));
YAHOO.register("test_dialogs", null, {version: "1.0.0", build: "1"});
