/*
 * Copyright (c) 2009 BaseSoft B.V.
 * See LICENSE for details.
 */
BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "Dynamic Table Test",
	setUp: function() {
		this.sandbox = YAHOO.util.Dom.get('sandbox');
		this.sandbox.innerHTML = '';
	},

	__assertNestedList: function(result, expected) {
		var i, j;
		for(i in expected) {
			YAHOO.util.Assert.areEqual(expected[i].length, result[i].length);
			for(j in expected[i]) {
				YAHOO.util.Assert.areEqual(expected[i][j].length, result[i][j].length);
				YAHOO.util.Assert.areEqual(expected[i][j], result[i][j]);
			}
		}
		for(i in result) {
			YAHOO.util.Assert.areEqual(expected[i].length, result[i].length);
			for(j in result[i]) {
				console.info('result', result[i][j], expected[i][j]);
				YAHOO.util.Assert.areEqual(expected[i][j].length, result[i][j].length);
				YAHOO.util.Assert.areEqual(expected[i][j], result[i][j]);
			}
		}
	},

	test_simple_dynamic_table: function() {
		try{
		var myTable = new BASESOFT.widget.DynamicTable({firstCellEmpty: true, initial_data:
			[['', 'header1', 'header2', 'header3'],
			 ['row1', '1.1', '1.2', '1.3'],
			 ['row2', '2.1', '2.2', '2.3']]
		});
		this._garbage.push(myTable);
		myTable.appendTo(this.sandbox);
		} catch(e) {
			console.error(e);
			throw e;
		}
		this.__assertNestedList(myTable.getData(), [['', 'header1', 'header2', 'header3'],
			 ['row1', '1.1', '1.2', '1.3'],
			 ['row2', '2.1', '2.2', '2.3']]);
	},

	test_dynamic_table_add_row: function() {
		var myTable = new BASESOFT.widget.DynamicTable({firstCellEmpty: true, initial_data:
			[['', 'header1', 'header2'],
			 ['row1', '1.1', '1.2'],
			 ['row2', '2.1', '2.2']]
		});
		myTable.appendTo(this.sandbox);
		myTable.addRow(['row3', '3.1', '3.2']);
		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header2'],
				 ['row1', '1.1', '1.2'],
				 ['row2', '2.1', '2.2'],
				 ['row3', '3.1', '3.2']]);
		// Add row somewhere in the middle of the table.
		myTable.addRow(['row0,5', '0,5.1', '0,5.2'],1);
		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header2'],
				 ['row0,5', '0,5.1', '0,5.2'],
				 ['row1', '1.1', '1.2'],
				 ['row2', '2.1', '2.2'],
				 ['row3', '3.1', '3.2']]);
		this._garbage.push(myTable);
	},

	test_dynamic_table_remove_row: function() {
		var myTable = new BASESOFT.widget.DynamicTable({firstCellEmpty: true, initial_data:
			[['', 'header1', 'header2', 'header3'],
			 ['row1', '1.1', '1.2', '1.3'],
			 ['row2', '2.1', '2.2', '2.3'],
			 ['row3', '3.1', '3.2', '3.3']]
		});
		this._garbage.push(myTable);
		myTable.appendTo(this.sandbox);
		// Remove the last row.
		myTable.removeRow();
		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header2', 'header3'],
				 ['row1', '1.1', '1.2', '1.3'],
				 ['row2', '2.1', '2.2', '2.3']]);
		// Remove row somewhere in the middle of the table.
		myTable.removeRow(1);
		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header2', 'header3'],
				 ['row2', '2.1', '2.2', '2.3']]);
	},

	test_dynamic_table_add_column: function() {
		var myTable = new BASESOFT.widget.DynamicTable({firstCellEmpty: true, initial_data:
			[['', 'header1', 'header2'],
			 ['row1', '1.1', '1.2'],
			 ['row2', '2.1', '2.2']]
		});
		this._garbage.push(myTable);
		myTable.appendTo(this.sandbox);
		// Add column at the end of the table.
		myTable.addColumn(['header3', '1.3', '2.3']);
		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header2', 'header3'],
				 ['row1', '1.1', '1.2', '1.3'],
				 ['row2', '2.1', '2.2', '2.3']]);
		// Add column somewhere in the middle of the table
		myTable.addColumn(['header1,5', '1.1,5', '2.1,5'], 2);
		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header1,5', 'header2', 'header3'],
				 ['row1', '1.1', '1.1,5', '1.2', '1.3'],
				 ['row2', '2.1', '2.1,5','2.2', '2.3']]);
	},

	test_dynamic_table_remove_column: function() {
		var myTable = new BASESOFT.widget.DynamicTable({firstCellEmpty: true, initial_data:
			[['', 'header1', 'header2', 'header3'],
			 ['row1', '1.1', '1.2', '1.3'],
			 ['row2', '2.1', '2.2', '2.3'],
			 ['row3', '3.1', '3.2', '3.3']]
		});
		this._garbage.push(myTable);
		myTable.appendTo(this.sandbox);
		// Remove the last column.
		myTable.removeColumn();
		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header2'],
				 ['row1', '1.1', '1.2'],
				 ['row2', '2.1', '2.2'],
				 ['row3', '3.1', '3.2']]);
		// Remove column somewhere in the middle of the table.
		myTable.removeColumn(1);
		this.__assertNestedList(myTable.getData(),
				[['', 'header2'],
				 ['row1', '1.2'],
				 ['row2', '2.2'],
				 ['row3', '3.2']]);
	},

	test_row_limits: function() {
		var myTable = new BASESOFT.widget.DynamicTable({
			firstCellEmpty: true,
			initial_data: [['', 'header1', 'header2', 'header3'],
			               ['row1', '1.1', '1.2', '1.3'],
			               ['row2', '2.1', '2.2', '2.3']],
			maxRows: 3
		});
		this._garbage.push(myTable);
		myTable.appendTo(this.sandbox);
		myTable.addRow(['row3', '3.1', '3.2', '3.3']);

		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header2', 'header3'],
	               ['row1', '1.1', '1.2', '1.3'],
	               ['row2', '2.1', '2.2', '2.3']]);

		myTable.addRow(['row1.5', '1.5.1', '1.5.2', '1.5.3'],2);

		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header2', 'header3'],
	               ['row1', '1.1', '1.2', '1.3'],
	               ['row2', '2.1', '2.2', '2.3']]);

	},

	test_col_limits: function() {
		var myTable = new BASESOFT.widget.DynamicTable({
			firstCellEmpty: true,
			initial_data: [['', 'header1', 'header2'],
			               ['row1', '1.1', '1.2'],
			               ['row2', '2.1', '2.2']],
			maxColumns: 3
		});
		this._garbage.push(myTable);
		myTable.appendTo(this.sandbox);
		myTable.addColumn(['header3', '1.3', '2.3']);

		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header2'],
	               ['row1', '1.1', '1.2'],
	               ['row2', '2.1', '2.2']]);

		myTable.addColumn(['header1.5', '1.1.5', '2.1.5'], 2);

		this.__assertNestedList(myTable.getData(),
				[['', 'header1', 'header2'],
	               ['row1', '1.1', '1.2'],
	               ['row2', '2.1', '2.2']]);

	}
}));

YAHOO.register("test_dynamictable", null, {version: "1.0.0", build: "1"});