/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 */

(function() {
	"use strict";


	var lorum_1 = 'lorum1 Lorem ipsum dolor sit amet, consectetur adipiscing elit.';
	var lorum_2 = 'lorum2 In suscipit tincidunt cursus. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Sed dapibus molestie mi, quis pellentesque risus porttitor vel. Vivamus bibendum dolor at ligula feugiat sed vestibulum risus rutrum. Quisque convallis diam vitae quam rhoncus at commodo enim pulvinar. Duis condimentum eros nec sapien auctor quis tempus enim volutpat. Sed tortor felis, tincidunt id ullamcorper id, auctor eget velit. Nam accumsan risus in lectus sagittis vitae consequat metus adipiscing. Nam sapien libero, eleifend sit amet ornare sed, auctor a massa. Praesent et molestie ante. Nam et nulla nec massa pharetra vehicula in id eros. Pellentesque malesuada convallis magna, non condimentum mauris consectetur in. Nulla nec lobortis dolor. Sed nec nibh augue, faucibus volutpat arcu. Nam eu nunc vitae est commodo laoreet sit amet vel odio. Aliquam gravida dolor quis sapien pulvinar id sagittis metus ornare. Suspendisse iaculis purus ut urna posuere ut dictum urna consequat. Sed vehicula venenatis risus, sed ornare lorem auctor sit amet.';
	var lorum_3 = 'lorum3 Cras in orci ut mi vulputate vulputate. Etiam in tortor ante, quis bibendum diam. Sed tellus mi, vestibulum accumsan scelerisque vitae, congue at arcu. Suspendisse odio orci, consequat blandit adipiscing vitae, condimentum ac orci. Donec suscipit mauris id lorem suscipit nec sollicitudin mi ultrices. Integer posuere interdum congue. Morbi sed fermentum ipsum. Pellentesque eleifend elit lobortis eros dignissim ac cursus enim euismod. Suspendisse ultrices venenatis felis. Ut vel est eu dolor laoreet pretium at at nisi. Proin sed magna ut turpis pharetra lobortis nec quis metus. Proin et sem neque, vel mattis elit. Quisque eu sapien ante, id mattis risus. Nam commodo interdum sapien dapibus ornare. Etiam porta massa pellentesque magna laoreet et vehicula enim fermentum. Nunc tellus arcu, molestie ac sollicitudin eu, lacinia a justo. Maecenas rhoncus erat ut risus vehicula ullamcorper. Aenean id enim dui.';

	/**
	 * TestPanel
	 */
	var TestPanel = function(){
		TestPanel.superclass.constructor.apply(this, arguments);
	};
	YAHOO.extend(TestPanel, BASESOFT.widget.Panel, {

		setBody: function(){
			TestPanel.superclass.setBody.call(this);

			var div1 = document.createElement('div');
			div1.appendChild(document.createTextNode("clicky to edit with rte"));

			var div2 = document.createElement('div');
			div2.appendChild(document.createTextNode("clicky to edit with textarea"));

			var div3 = document.createElement('div');
			div3.appendChild(document.createTextNode("clicky to edit with input"));

			this.body.appendChild(div1);
			this.body.appendChild(div2);
			this.body.appendChild(div3);

			var emec = BASESOFT.env.EditorManager.makeRteEditable(div1, {
				replacementFields: {
					'date.weekday': 'Today\'s weekday',
					'Contact': {
						'contact.first_name': 'First Name',
						'contact.last_name': 'Last Name',
						'contact.telephone': 'Telephone'
					},
					'Company': {
						'company.name': 'Name',
						'address': 'Address',
						'Country': {
							'contact.country.name': 'Name',
							'contact.country.currency': 'Currency'
						}
					}
				}
			});

			BASESOFT.env.EditorManager.makeTextareaEditable(div2);
			BASESOFT.env.EditorManager.makeInlineEditable(div3);

		}
	});


BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "EditorManager Test",

	setUp : function () {

		this.sandbox = YAHOO.util.Dom.get('sandbox');
		this.sandbox.innerHTML = '';

		// force a new editor manager if not exists (might be destroyed in previous testrun)
		if (!BASESOFT.env.EditorManager)
			BASESOFT.env.EditorManager = new BASESOFT.util.editor_manager.EditorManager();

	},

	ztearDown: function() {

		// call parent by static call (no superclass possible!)
		BASESOFT.tool.TestCase.prototype.tearDown.call(this);

		// destroy the editor manager and nullify it here which forces a new editor manager in case
		BASESOFT.env.EditorManager.destroy();
		BASESOFT.env.EditorManager = null;

	},

	/**
	 * tests the opening of a RTE editor by setting a text in the RTE and verifyig that the text goes back into the element
	 */
	testRTE : function () {

		try {

			var div1 = document.createElement('div');
			div1.innerHTML = 'Click to edit text';
			this.sandbox.appendChild(div1);
			var emec1 = BASESOFT.env.EditorManager.makeRteEditable(div1);
			this._garbage.push(emec1);

			var spacer = document.createElement('div');
			spacer.style.height = '100px';
			this.sandbox.appendChild(spacer);

			var div2 = document.createElement('div');
			div2.innerHTML = 'Click to edit text';
			this.sandbox.appendChild(div2);
			var emec2 = BASESOFT.env.EditorManager.makeRteEditable(div2);
			this._garbage.push(emec2);

			var func1 = function(e){
				this.resume(function(){
					try {
						// unsubscribe so this wont be called again when we click again
						emec1.unsubscribe('editorReady', func1);

						// set new text, verify and destroy
						BASESOFT.env.EditorManager._editing.setEditorHTML(lorum_1);
						BASESOFT.env.EditorManager._editing.destroy();
						YAHOO.util.Assert.areEqual(lorum_1, div1.innerHTML);

						// click div2.. continue in func2
						emec2.subscribe('editorReady', func2, null, this);
						YAHOO.util.UserAction.click(div2);

						this.wait(15000);

					} catch (e) {
						e.delay || console.error(e);
						throw e;
					}
				});
			};

			var func2 = function(e){
				this.resume(function(){
					try {
						// unsubscribe so this wont be called again when we click again
						emec2.unsubscribe('editorReady', func2);

						// edit, but dont destroy.. then we click div1 again
						BASESOFT.env.EditorManager._editing.setEditorHTML(lorum_2);
						YAHOO.util.Assert.areEqual(lorum_2, emec2.getContent());

						// also a click some time later should work.. this gave problems in the past
						setTimeout(function() {
							YAHOO.util.UserAction.click(div1);
						}, 3000);

						// continue in func3
						emec1.subscribe('editorReady', func3, null, this);
						this.wait(15000);

					} catch (e) {
						e.delay || console.error(e);
						throw e;
					}
				});
			};

			var func3 = function(e) {
				this.resume(function() {
					try {
						emec1.unsubscribe('editorReady', func3);

						// expect that lorum_1 was in the editor from func1
						YAHOO.util.Assert.areEqual(lorum_1, emec1.getContent());

						// unsubscribe so this wont be called again when we click again
						emec1.unsubscribe('editorReady', func3);

						// set new text verify
						BASESOFT.env.EditorManager._editing.setEditorHTML(lorum_3);
						YAHOO.util.Assert.areEqual(lorum_3, emec1.getContent());

						// closing the editor should also work by clicking on the document.body
						YAHOO.util.UserAction.click(document.body);
						YAHOO.util.Assert.areEqual(lorum_3, div1.innerHTML);

						// and div2 should now have lorum2
						YAHOO.util.Assert.areEqual(lorum_2, div2.innerHTML);
					}
					catch (e) {
						e.delay || console.error(e);
						throw e;
					}
				});
			};

			// click on the first div.. this should open the RTE
			emec1.subscribe('editorReady', func1, null, this);
			YAHOO.util.UserAction.click(div1);

			// wait for emec1 editorReady
			this.wait(15000);

		} catch (e) {
			e.delay || console.error(e);
			throw e;
		}

	},

	testInline: function() {

		var div2 = document.createElement('div');
		div2.innerHTML = 'Original text for div2';
		this.sandbox.appendChild(div2);

		// enable inline editing of div2
		var emecDiv2 = BASESOFT.env.EditorManager.makeInlineEditable(div2);

		try {

			// click on the div which opens the editor
			YAHOO.util.UserAction.click(div2);

			// wait for the editor to be loaded
			BASESOFT.util.waitForCondition(function() {
				return BASESOFT.env.EditorManager._editing;
			}, function() {
				this.resume(function() {

					// set another text in the editor and press enter
					BASESOFT.env.EditorManager._editing.__input.value = 'Edited text for div2';

					// too bad this doesnt work:
					//YAHOO.util.UserAction.keypress(input, { charCode: 13 });

					// so instead do what the keylistener normally does:
					BASESOFT.env.EditorManager._editing.destroy();

					YAHOO.util.Assert.areEqual("Edited text for div2", div2.innerHTML);
				});

			}, null, this);

			this.wait();


		} catch (e) {
			console.error(e);
			throw e;
		}
	},

	testOpenEditor: function() {

		// create a div
		var div2 = document.createElement('div');
		div2.innerHTML = 'Original text for div2';
		this.sandbox.appendChild(div2);

		// enable inline editing of div2
		var emecDiv2 = BASESOFT.env.EditorManager.makeInlineEditable(div2);

		// trigger the event to show the editor
		emecDiv2.showEditor();

		// wait for the editor to be loaded
		BASESOFT.util.waitForCondition(function() {
			return BASESOFT.env.EditorManager._editing;
		}, function() {
			this.resume(function() {

				// then the editor should be a inlinerte editor
				YAHOO.util.Assert.areEqual(true, BASESOFT.env.EditorManager._editing instanceof BASESOFT.util.editor_manager.InlineEditor);

				BASESOFT.env.EditorManager._editing.destroy();

			});

		}, null, this);
		this.wait();



	},

	/**
	 * Creates a edtiable textarea with replacement fields enabled. Its kinda hard to simulate the menu clicking so
	 * the least we can do is verifying that inputs dont get filtered.
	 */
	testReplacementFields: function(){

		var str = 'Dear sir <img class="replacement-field" title="contact.last_name" src="/basesoft.web.core/rte/make_image/Contact%3ALast%20Name"/>, <br>Please dont reply!';

		try {

			var div = document.createElement('div');
			div.innerHTML = str;
			this.sandbox.appendChild(div);

			var emec = BASESOFT.env.EditorManager.makeRteEditable(div, {
				replacementFields: {
					'date.weekday': 'Today\'s weekday',
					'Contact': {
						'contact.first_name': 'First Name',
						'contact.last_name': 'Last Name',
						'contact.telephone': 'Telephone'
					},
					'Company': {
						'company.name': 'Name',
						'address': 'Address',
						'Country': {
							'contact.country.name': 'Name',
							'contact.country.currency': 'Currency'
						}
					}
				},
				simpleToolbar: null
			});

			emec.showEditor();

			emec.subscribe('editorReady', function(){
				this.resume(function() {

					BASESOFT.env.EditorManager._editing.destroy();

					var html = div.innerHTML;

					YAHOO.util.Assert.isTrue( (html.indexOf('Dear sir') === 0) );
					YAHOO.util.Assert.isTrue( (html.indexOf('contact.last_name') > 0) );
					YAHOO.util.Assert.isTrue( (html.indexOf('Please dont reply') > 0) );
				});
			}, null, this);

			this.wait(15000);
		} catch (e) {
			e.delay || console.error(e);
			throw e;
		}

	},

	testPanel: function() {
		try {
			var p1 = new TestPanel({modal: true, width: "500px"});
			var p2 = new TestPanel();
			var p3 = new TestPanel();

			this._garbage.push(p1);
			this._garbage.push(p2);
			this._garbage.push(p3);

		}catch (e) {
			console.error(e);
			throw e;
		}
	},

	testMultipleEditors: function() {
		var sandbox = YAHOO.util.Dom.get('sandbox');

		var emecs = [];

		for (var i=0; i<5; i++) {
			var div = document.createElement('div');
			div.id = 'div-editor-'+i;
			div.style.height = '100px';
			div.style.border = '1px solid black';
			div.style.margin = '10px';
			div.innerHTML = 'editor ' + i;
			sandbox.appendChild(div);

			var config = {};

			// first editor is special
			if (i === 0) {
				config = {
					replacementFields: {
						'date.weekday': 'Today\'s weekday',
						'Contact': {
							'contact.first_name': 'First Name',
							'contact.last_name': 'Last Name',
							'contact.telephone': 'Telephone'
						}
					},
					collapse: false
				};
			}

			// make the 3rd editor fixed in height
			if (i == 2) {
				config.autoHeight = false;
			}

			var emec = BASESOFT.env.EditorManager.makeRteEditable(div, config);
			emecs.push(emec);
			this._garbage.push(emec);
		}

		emecs[0].subscribe('editorReady', function() {
			this.resume(function() {
				console.info('test done');
			});
		}, null, this);

		emecs[0].showEditor();
		this.wait();


	},

	test_rte_strip_yui_id_tag: function() {

		var str = '<strong id="yui-gen86">asddasdsa<br><br><img id="yui-gen85" alt="" src="http://pvcycle-acceptance.basesoft.com/site_media/pvcycle.core/themes/pvcycle/media/PV-CYCLE_Logo_wo%20background_small.png"><br>sadsdasdasadasd<br><br><br><br></strong>';

		var div = document.createElement('div');
		div.innerHTML = str;
		this.sandbox.appendChild(div);

		var emec = BASESOFT.env.EditorManager.makeRteEditable(div, {
			simpleToolbar: null
		});

		emec.subscribe('editorReady', function(){
				console.warn('fired');
			this.resume(function() {

				emec.subscribe('editFinished', function(evArgs, localArgs){
					this.resume(function() {
						YAHOO.util.Assert.isTrue( (evArgs[0].indexOf('yui-gen') === -1) );
					});
				}, null, this);

				BASESOFT.env.EditorManager._editing.destroy();
				this.wait();

			});
		}, null, this);

		emec.showEditor();
		this.wait();

	},

	test_editable_wiki: function() {

		try {

			var wiki_value = "Click to '''edit'''";

			var div = document.createElement('div');
			div.innerHTML = "clicky!!!";
			div.setAttribute('style', 'margin-left: 200px; margin-top: 100px;');
			this.sandbox.appendChild(div);


			var emec = BASESOFT.env.EditorManager.makeWikiEditable(div, {
				simpleToolbar: null,
				getWikiValue: function(args) {
					console.warn('getWikiValue', arguments);
					return wiki_value;
				},
				getWikiValueArgs: ['hello'],
				getWikiValueScope: this
			});

			/*
			emec.subscribe('editFinished', function(evArgs, localArgs){
				console.warn('received editFinished', arguments);

				var new_value = evArgs[0];
				var old_value = evArgs[1];
				var html = evArgs[2];

				console.warn('html', html);

				wiki_value = new_value;

			});
			*/

			emec.subscribe('editorReady', function(){
				//console.warn('')
				this.resume(function() {
					emec.subscribe('editFinished', function(evArgs, localArgs){
						this.resume(function() {

							console.warn('received editFinished', arguments);

							var new_value = evArgs[0];
							var old_value = evArgs[1];
							var html = evArgs[2];


							YAHOO.util.Assert.areSame("Click to '''edit'''", new_value);
							YAHOO.util.Assert.areSame("Click to '''edit'''", old_value);
							YAHOO.util.Assert.areSame("<p>\nClick to <strong>edit</strong> <br />\n</p>\n", html);

							//YAHOO.util.Assert.isTrue( (evArgs[0].indexOf('yui-gen') === -1) );
						});
					}, null, this);

					// simulate close clicky
					BASESOFT.env.EditorManager._editing.destroy();

					this.wait();

				});
			}, null, this);

			emec.showEditor();
			this.wait();


		} catch (ex) {
			console.error(ex);
			throw ex;
		}

	}

}));
YAHOO.register("test_editor_manager", null, {version: "1.0.0", build: "1"});


})();
