/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 */

(function() {
	"use strict";

BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "Fancy Select Test",
	assertClean: true,

	setUp : function () {
		// create the container in which notifications are shown
		var form = document.createElement('form');
		this.div = document.createElement('div');
		this.div.id = 'id_myfancyselect';
		form.appendChild(this.div);
		YAHOO.util.Dom.get('sandbox').appendChild(form);

	},

	/**
	 * check if defaultoptions and selectedoptions attributes are set correctly when moving options
	 */
	testMoveOptions : function () {

		var fs = new BASESOFT.widget.FancySelect(this.div.id, {
				name: 'myfancyselect',
				defaultoptions: [
					{ label: 'One', value: '1' },
					{ label: 'Two', value: '2' },
					{ label: 'Three', value: '3' }
				],
				selectedoptions:[{label: 'Four', value:'4'}]
			});

		this._garbage.push(fs);

		YAHOO.util.Assert.isArray(fs.get('selectedoptions'));
		YAHOO.util.ObjectAssert.propertiesAreEqual(fs.get('selectedoptions'),[{label: 'Four', value:'4'}]);
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[0].label,'Four');
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[0].value,'4');

		YAHOO.util.Assert.isArray(fs.get('defaultoptions'));
		//YAHOO.util.ObjectAssert.propertiesAreEqual(fs.get('defaultoptions'),[{ label: 'One', value: '1' },
		//																	 { label: 'Two', value: '2' },
		//																	 { label: 'Three', value: '3' }]);
		YAHOO.util.Assert.areEqual(fs.get('defaultoptions')[0].label,'One');
		YAHOO.util.Assert.areEqual(fs.get('defaultoptions')[0].value,'1');
		YAHOO.util.Assert.areEqual(fs.get('defaultoptions')[1].label,'Two');
		YAHOO.util.Assert.areEqual(fs.get('defaultoptions')[1].value,'2');
		YAHOO.util.Assert.areEqual(fs.get('defaultoptions')[2].label,'Three');
		YAHOO.util.Assert.areEqual(fs.get('defaultoptions')[2].value,'3');

		fs.select(['1', '3']);
		YAHOO.util.Assert.isArray(fs.get('selectedoptions'));
		YAHOO.util.ObjectAssert.propertiesAreEqual(fs.get('selectedoptions'),[{label: 'Four', value:'4'},
																			  {label: 'One', value:'1'},
																			  {label: 'Three', value:'3'}]);
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[0].label,'Four');
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[0].value,'4');
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[1].label,'One');
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[1].value,'1');
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[2].label,'Three');
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[2].value,'3');

		YAHOO.util.Assert.isArray(fs.get('defaultoptions'));
		YAHOO.util.Assert.areEqual(fs.get('defaultoptions')[0].label,'Two');
		YAHOO.util.Assert.areEqual(fs.get('defaultoptions')[0].value,'2');

		fs.deselect();
		YAHOO.util.Assert.isArray(fs.get('selectedoptions'));
		YAHOO.util.ArrayAssert.isEmpty(fs.get('selectedoptions'));

	},

	/**
	 * check adding/removing options to the two selects and if the internal model is updated correctly
	 */
	testAddOption: function() {
		var fs = new BASESOFT.widget.FancySelect(this.div.id, {
				name: 'myfancyselect',
				defaultoptions: [
					{ label: 'One', value: '1' },
					{ label: 'Two', value: '2' },
					{ label: 'Three', value: '3' }
				],
				selectedoptions:[{label: 'Four', value:'4'}]
			});

		this._garbage.push(fs);

		fs.addOptions('selectedoptions',[{label:'Five', value:'5'}]);
		YAHOO.util.Assert.isArray(fs.get('selectedoptions'));
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[0].label,'Four');
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[0].value,'4');
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[1].label,'Five');
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[1].value,'5');

		fs.removeOptions('selectedoptions',['4']);
		YAHOO.util.Assert.isArray(fs.get('selectedoptions'));
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[0].label,'Five');
		YAHOO.util.Assert.areEqual(fs.get('selectedoptions')[0].value,'5');
	},

	testGroupedFS : function () {

		var fs = new BASESOFT.widget.GroupedFancySelect(this.div.id, {
				name: 'myfancyselect',
				defaultoptions: {
					'group 1': [{label: 'One', value: '1'},
								{label: 'Two', value: '2'},
								{label: 'Three', value: '3'}
								],
					'group 2': [{label: 'Four', value: '4'},
								{label: 'Five', value: '5'},
								{label: 'Six', value: '6'}
								]
				},
				selectedoptions:{'group 2': [{label: 'Seven', value:'7'}]}
			});

		this._garbage.push(fs);

		fs.select(['1', '4'], true);
		var val = fs.get('value');
		var so = fs.get('selectedoptions');
		YAHOO.util.Assert.areEqual(so['group 2'].length, 2);
		YAHOO.util.Assert.areEqual(so['group 1'].length, 1);
		YAHOO.util.Assert.areEqual(val.length, 3);
		YAHOO.util.Assert.areEqual(val[0], '7');
		YAHOO.util.Assert.areEqual(val[1], '4');
		YAHOO.util.Assert.areEqual(val[2], '1');

		YAHOO.util.Assert.areEqual(so['group 2'][0].value, '7');
		YAHOO.util.Assert.areEqual(so['group 2'][1].value, '4');
		YAHOO.util.Assert.areEqual(so['group 1'][0].value, '1');

		var defo = fs.get('defaultoptions');
		YAHOO.util.Assert.areEqual(defo['group 2'].length, 2);
		YAHOO.util.Assert.areEqual(defo['group 1'].length, 2);

		fs.deselect();
		defo = fs.get('defaultoptions');
		YAHOO.util.Assert.areEqual(defo['group 2'].length, 4);
		YAHOO.util.Assert.areEqual(defo['group 1'].length, 3);
		so = fs.get('selectedoptions');
		YAHOO.util.Assert.isTrue(BASESOFT.util.is_empty(so));

		fs.addOptions('selectedoptions', {'group 3': [{label: 'Eight', value: '8'}]});
		so = fs.get('selectedoptions');
		YAHOO.util.Assert.areEqual(so['group 3'].length, 1);

		fs.removeOptions('defaultoptions', ['1']);
		defo = fs.get('defaultoptions');
		YAHOO.util.Assert.areEqual(defo['group 1'].length, 2);

	},

	/**
	 * check order items in selected options in simple FS
	 */
	testOrderOptions: function() {
		var fs = new BASESOFT.widget.FancySelect(this.div.id, {
				name: 'myfancyselect',
				selectedoptions: [
					{ label: 'One', value: '1' },
					{ label: 'Two', value: '2' },
					{ label: 'Three', value: '3' },
					{ label: 'Four', value:'4' },
					{ label: 'Five', value:'5' },
					{ label: 'Six', value:'6' },
					{ label: 'Seven', value:'7' }
				],
				defaultoptions:[]
			});
		this._garbage.push(fs);

		fs.set("orderSelectedItems", true);

		var vals;

		fs.moveItemsByValue('up', ['2', '4']);
		vals = fs.get('value');
		YAHOO.util.Assert.areEqual(vals[0],'2');
		YAHOO.util.Assert.areEqual(vals[1],'1');
		YAHOO.util.Assert.areEqual(vals[2],'4');
		YAHOO.util.Assert.areEqual(vals[3],'3');

		fs.moveItemsByValue('top', ['6']);
		vals = fs.get('value');
		YAHOO.util.Assert.areEqual(vals[0],'6');
		YAHOO.util.Assert.areEqual(vals[1],'2');

		fs.moveItemsByValue('down', ['1', '3', '5']);
		vals = fs.get('value');
		YAHOO.util.Assert.areEqual(vals[3],'1');
		YAHOO.util.Assert.areEqual(vals[5],'3');
		YAHOO.util.Assert.areEqual(vals[6],'5');

		fs.moveItemsByValue('bottom', ['4', '1', '7']);
		vals = fs.get('value');
		YAHOO.util.Assert.areEqual(vals[4],'4');
		YAHOO.util.Assert.areEqual(vals[5],'1');
		YAHOO.util.Assert.areEqual(vals[6],'7');
	},

	/**
	 * check order items in selected options in a grouped fs
	 */
	testOrderOptionsGroupedFS: function() {
		var gfs = new BASESOFT.widget.GroupedFancySelect(this.div.id, {
			name: 'mygfancyselect',
			selectedoptions: {
				'group 1': [{label: 'One', value: '1'},
							{label: 'Two', value: '2'},
							{label: 'Three', value: '3'}
							],
				'group 2': [{label: 'Four', value: '4'},
							{label: 'Five', value: '5'},
							{label: 'Six', value: '6'}
							],
				'group 3': [{label: 'Seven', value: '7'}
				            ]
			},
			defaultoptions:{}
		});
		this._garbage.push(gfs);

		gfs.set("orderSelectedItems", true);

		var vals;

		// move a whole group a level up
		gfs.moveItemsByGroup('up', 'group 3');
		vals = gfs.get('value');
		YAHOO.util.Assert.areEqual(vals[3],'7');

		// move a whole group a level up
		gfs.moveItemsByGroup('top', 'group 3');
		vals = gfs.get('value');
		YAHOO.util.Assert.areEqual(vals[0],'7');

		// move items inside a group
		gfs.moveItemsByValue('down', ['1']);
		vals = gfs.get('value');
		YAHOO.util.Assert.areEqual(vals[0],'7');
		YAHOO.util.Assert.areEqual(vals[1],'2');
		YAHOO.util.Assert.areEqual(vals[2],'1');

	},


	testOrderSelectedOptionsGroupedFS: function() {

		// helper function to select items from left to right
		var _select = function(number) {

			// select the number in left select
			YAHOO.util.Selector.query("#id_myfancyselect option[value="+number+"]")[0].selected = true;

			// click select
			YAHOO.util.UserAction.click(YAHOO.util.Selector.query("#id_myfancyselect button")[0]);

		};

		var gfs = new BASESOFT.widget.GroupedFancySelect(this.div.id, {
			name: 'mygfancyselect',
			defaultoptions: {
				'group 1': [{label: 'One', value: '1'},
							{label: 'Two', value: '2'},
							{label: 'Three', value: '3'}
							],
				'group 2': [{label: 'Four', value: '4'},
							{label: 'Five', value: '5'},
							{label: 'Six', value: '6'}
							],
				'group 3': [{label: 'Seven', value: '7'}
				            ]
			},
			selectedoptions:{}
		});

		gfs.set("orderSelectedItems", true);

		this._garbage.push(gfs);

		var _this = this;
		setTimeout(function() {

			(function() {

				console.warn('this', this);

				this.resume(function() {

					// select from left to right..
					_select.call(this, 4);
					_select.call(this, 1);
					_select.call(this, 5);

					// at this point the fs right looks like:
					// group 2
					//   four
					//   five
					// group 1
					//   one

					// assert values
					var vals = gfs.get('value');
					console.warn('vals 1', vals);
					YAHOO.util.Assert.areEqual(vals[0],'4');
					YAHOO.util.Assert.areEqual(vals[1],'5');
					YAHOO.util.Assert.areEqual(vals[2],'1');


					// only select the 1 to order (shouldnt change the value) and click order
					YAHOO.util.Selector.query("#id_myfancyselect option[value=4]")[0].selected = false;
					YAHOO.util.Selector.query("#id_myfancyselect option[value=5]")[0].selected = false;
					YAHOO.util.Selector.query("#id_myfancyselect option[value=1]")[0].selected = true;
					YAHOO.util.UserAction.click(YAHOO.util.Selector.query("#id_myfancyselect_btn_moveUp")[0]);


					// at this point: ["4", "5", "1"]
					vals = gfs.get('value');
					console.warn('vals 2', vals);
					YAHOO.util.Assert.areEqual(vals[0],'4');
					YAHOO.util.Assert.areEqual(vals[1],'5');
					YAHOO.util.Assert.areEqual(vals[2],'1');

				});

			}).call(_this);

		}, 1000);

		this.wait();

	},

	testSingleValue : function () {
		// helper function to select items from left to right
		var _select = function(number) {

			// select the number in left select
			YAHOO.util.Selector.query("#id_myfancyselect option[value="+number+"]")[0].selected = true;
			// click select
			YAHOO.util.UserAction.click(YAHOO.util.Selector.query("#id_myfancyselect button")[0]);

		};

		// helper function to dblClick an item from left to right
		var _dblClick = function(number) {
			// select the number in left select
			YAHOO.util.Selector.query("#id_myfancyselect option[value="+number+"]")[0].selected = true;
			// click select
			YAHOO.util.UserAction.dblclick(YAHOO.util.Selector.query("#id_myfancyselect option[value="+number+"]")[0]);

		};

		var fancyConfig = {
			name: 'mygfancyselect',
			label_default: gettext("Available"),
			label_selected: gettext("Selected"),
			allowOnlyOneValue: true,
			defaultoptions: {
				'group 1': [{label: 'One', value: '1'},
							{label: 'Two', value: '2'},
							{label: 'Three', value: '3'}
							],
				'group 2': [{label: 'Four', value: '4'},
							{label: 'Five', value: '5'},
							{label: 'Six', value: '6'}
							]
			}
		};

		var gfs = new BASESOFT.widget.GroupedFancySelect(this.div.id, fancyConfig);
		this._form = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
				children: ['mygfancyselect']
			})});
		this._garbage.push(gfs);
		var _this = this;
		setTimeout(function() {

			(function() {

				console.warn('this', this);

				this.resume(function() {

					// select from left to right..
					_select.call(this, 4);
					_select.call(this, 5);

					// assert values
					var vals = gfs.get('value');
					console.warn('vals 1', vals);
					YAHOO.util.Assert.areEqual(vals[0],'5');
					YAHOO.util.Assert.areEqual(vals.length,1);

					_dblClick.call(this, 1);
					//test validator
					YAHOO.util.Assert.isFalse(gfs.set('value', [4, 2], true));

					gfs.set('value', ['3']);
					vals = gfs.get('value');
					console.warn('vals 2', vals);

					YAHOO.util.Assert.areEqual(vals[0],'3');
					YAHOO.util.Assert.areEqual(vals.length,1);


				});

			}).call(_this);

		}, 1000);

		this.wait();

	},


	testDragMode : function () {

		var fs = new BASESOFT.widget.GroupedFancySelect(this.div.id, {
				name: 'myfancyselect',
				defaultoptions: {
					'group 1': [{label: 'One', value: '1'},
								{label: 'Two', value: '2'},
								{label: 'Three', value: '3'}
								],
					'group 2': [{label: 'Four', value: '4'},
								{label: 'Five', value: '5'},
								{label: 'Six', value: '6'}
								]
				},
				selectedoptions:{'group 2': [{label: 'Seven', value:'7'}]},
				//dragMode: true,
				dragInitCallback: function() {
					console.warn('dragInitOption', arguments);
				},

				label_default: 'available',
				label_selected: 'selected'
			});

		fs.set('dragMode', true);

		this._garbage.push(fs);
	},

	testSearchFilter: function () {
		BASESOFT.env.account.id = 1;
		var dfs = new BASESOFT.widget.GroupedFancySelect(this.div.id, {
			name: 'mygfancyselect',
			filterMode: true,
			optionsRpcUrl: '/basesoft.web.test/selenium/yui/fancy_select_options/',
			optionsRpcMethod: 'get_grouped_fancy_options',
			label_default: 'available',
			label_selected: 'selected',
			optionsRpcMethodArgs: [1] // mock user_id
		});

		var fs = new BASESOFT.widget.FancySelect(this.div.id, {
			name: 'myfancyselect',
			filterMode: true,
			optionsRpcUrl: '/basesoft.web.test/selenium/yui/fancy_select_options/',
			optionsRpcMethod: 'get_fancy_select_options',
			label_default: 'available',
			label_selected: 'selected',
			optionsRpcMethodArgs: [1] // mock user_id
		});
		this._garbage.push(fs);
		this._garbage.push(dfs);
	},

	testDisableOnLoading: function() {
		BASESOFT.env.account.id = 1;
		var dfs = new BASESOFT.widget.FancySelect(this.div.id, {
			name: 'mygfancyselect',
			optionsRpcUrl: '/basesoft.web.test/selenium/yui/fancy_select_options/',
			optionsRpcMethod: 'get_fancy_select_options',
			label_default: 'available',
			label_selected: 'selected',
			optionsRpcMethodArgs: [1, [2], null]
		});
		this._garbage.push(dfs);

		YAHOO.util.Assert.areEqual(true, dfs.get('disabled'));
		console.info('FIRST ASSERT PASSED');

		// wait a bit for the initial loading to end
		setTimeout(function() {
			dfs.set('optionsRpcMethod', 'rpc_loading');
			dfs.reloadFromRpc(true);
			checkDisabled();
		}, 100);

		var that = this;
		var checkDisabled = function() {

			YAHOO.util.Assert.areEqual(true, dfs.get('disabled'));
			YAHOO.util.Assert.areEqual('loading', dfs.get('defaultoptions')[0].value);
			console.info('SECOND ASSERT PASSED');

			setTimeout(function() {
				that.resume(function() {
					YAHOO.util.Assert.areEqual(false, dfs.get('disabled'));
					console.info('LAST ASSERT PASSED');
				});
			}, 5000);
		};

		this.wait();
	}

}));

YAHOO.register("test_fancyselect", null, {version: "1.0.0", build: "1"});

})();
