/*
 * Copyright (c) 2008 BaseSoft B.V.
 * See LICENSE for details.
 */

(function() {
	"use strict";


BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({
	name: "Form Manager Test",

	setUp: function(){
		this.container = document.createElement('div');
		this.container.style.width = '400px';
		this.container.style.textAlign = 'left';

		var sandbox = YAHOO.util.Dom.get('sandbox');
		sandbox.innerHTML = '';
		sandbox.appendChild(this.container);
	},

	/**
	 * A very simple test to see if basic form works.. dont put any other inputs in here!
	 */
	test_simple_form: function() {

		try {

			var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
				children: ['my_first_input1','my_first_input2', 'my_first_input3', 'my_first_textonly']
			})});
			this._garbage.push(myform);

			myform.appendTo(this.container);
			myform.addInput(BASESOFT.util.form.TextInput, {label: 'my container'},{name: 'my_first_input1'});
			myform.addInput(BASESOFT.util.form.TextInput, {label: 'my container'},{name: 'my_first_input2'});
			myform.addInput(BASESOFT.util.form.TextInput, {label: 'my container'},{name: 'my_first_input3'});
			myform.addInput(BASESOFT.util.form.TextOnly, {label: 'my text only'},{name: 'my_first_textonly', value: 'yello!'});
			myform.updateLayout();

			// FIXME: test is not asserting!

			myform.updateLayout();

			//myform.destroy();
			//myform = null;


		} catch (e) {
			console.error(e);
			throw e;
		}
	},

	test_form_with_fancy_select: function() {
		try {
			var fs = new BASESOFT.widget.FancySelect(null, {name: 'my_fancy_select'});

			var myform = new BASESOFT.util.form.Form({
				layout: new BASESOFT.util.form.Layout({
					children: ['my_fancy_select']
				})
			});
			this._garbage.push(myform);

			myform.appendTo(this.container);

			var fscontainer = new BASESOFT.util.form.InputContainer({});
			fscontainer.add(fs);
			myform.add(fscontainer);

			myform.updateLayout();

			// FIXME: test is not asserting!

		} catch(e) {
			console.error(e);
			throw e;
		}
	},

	test_form_rte: function() {
		try {
			var myform = new BASESOFT.util.form.Form({
				layout: new BASESOFT.util.form.Layout({
					children: ['rte_field']
				})
			});
			this._garbage.push(myform);

			var rte = new BASESOFT.util.form.RTEInput({
				name: 'rte_field',
				replacementFields: {}
			});

			rte.set('value', 'some test text');

			var container = new BASESOFT.util.form.InputContainer({label: 'test label'});
			container.add(rte);
			myform.add(container);

			myform.appendTo(this.container);
			myform.updateLayout();

			// change the rte text
			rte.set('value', '&lt;strong&gt;bold text&lt;/strong&gt;');

		} catch(e) {
			console.error(e);
			throw e;
		}
	},

	test_multiple_emecs: function() {
		try {
			var myform = new BASESOFT.util.form.Form({
				layout: new BASESOFT.util.form.Layout({
					children: ['field1', 'field2', 'field3']
				})
			});
			this._garbage.push(myform);

			var f1 = myform.addInput(BASESOFT.util.form.InlineTextAreaInput, { label: 'Field 1' }, { name: 'field1' });
			var f2 = myform.addInput(BASESOFT.util.form.InlineTextAreaInput, { label: 'Field 2' }, { name: 'field2' });
			var f3 = myform.addInput(BASESOFT.util.form.InlineTextAreaInput, { label: 'Field 3' }, { name: 'field3' });

			myform.appendTo(this.container);
			myform.updateLayout();

			var v1 = 'Value 1';
			var v2 = 'Value 2';
			var v3 = 'Value 3';
			var v2u = 'Value 2 updated';

			var start = function(){

				f1.emec.subscribe('editorReady', function(){
					this.resume(function(){
						try {
							// set some text in the first emec and close it
							f1.emec.setContent(v1);

							// close the rte and save the content
							YAHOO.util.UserAction.click(BASESOFT.env.recycleEditor.toolbar._titlebar.lastChild);
							YAHOO.util.Assert.areEqual(v1, f1.get('value'));

							cbEditField2.call(this);
						} catch(e) {
							e.delay || console.error(e);
							throw e;
						}
					});
				}, null, this);

				// open the first editor so we can set some text
				YAHOO.util.UserAction.click(f1.__editable); // need to access the textarea so we can click on it...
				this.wait(this.timeout);
			};

			var cbEditField2 = function(){
				f2.emec.subscribe('editorReady', function(){
					this.resume(function(){
						try {
							// set some text in the second emec and close it
							f2.emec.setContent(v2);

							// close the rte and save the content
							YAHOO.util.UserAction.click(BASESOFT.env.recycleEditor.toolbar._titlebar.lastChild);
							YAHOO.util.Assert.areEqual(v2, f2.get('value'));

							// remove the listener because we don't want this callback to be called again
							// if we reopen the second editor
							f2.emec.unsubscribeAll('editorReady');

							cbEditField3.call(this);
						} catch(e) {
							e.delay || console.error(e);
							throw e;
						}
					});
				}, null, this);

				// open the first editor so we can set some text
				YAHOO.util.UserAction.click(f2.__editable); // need to access the textarea so we can click on it...
				this.wait(this.timeout);
			};

			var cbEditField3 = function(){
				f3.emec.subscribe('editorReady', function(){
					this.resume(function(){
						try {
							// set some text in the third emec and close it
							f3.emec.setContent(v3);

							// close the rte and save the content
							YAHOO.util.UserAction.click(BASESOFT.env.recycleEditor.toolbar._titlebar.lastChild);
							YAHOO.util.Assert.areEqual(v3, f3.get('value'));

							cbEditField2Again.call(this);
						} catch(e) {
							e.delay || console.error(e);
							throw e;
						}
					});
				}, null, this);

				// open the first editor so we can set some text
				YAHOO.util.UserAction.click(f3.__editable); // need to access the textarea so we can click on it...
				this.wait(this.timeout);
			};

			var cbEditField2Again = function(){
				f2.emec.subscribe('editorReady', function(){
					this.resume(function(){
						try {
							// let's check that the editor was opened with the original value
							YAHOO.util.Assert.areEqual(v2, f2.emec.getContent());
							f2.emec.setContent(v2u); // update the value

							// close the rte and save the content
							YAHOO.util.UserAction.click(BASESOFT.env.recycleEditor.toolbar._titlebar.lastChild);

							// we should have in the second field the updated value
							YAHOO.util.Assert.areEqual(v2u, f2.get('value'));
						} catch(e) {
							e.delay || console.error(e);
							throw e;
						}
					});
				}, null, this);

				// open the first editor so we can set some text
				YAHOO.util.UserAction.click(f2.__editable); // need to access the textarea so we can click on it...
				this.wait(this.timeout);
			};

			// call start to begin test
			console.warn('start!');
			start.call(this);

		} catch(e) {
			e.delay || console.error(e);
			throw e;
		}
	},

	/**
	 * This test is currently not asserting anything. It was build to test the IE bug with calling updateLayout
	 * more than one times.
	 */
	test_layout_error: function() {
		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['input_1']
			})
		});
		this._garbage.push(myform);

		myform.addInput(BASESOFT.util.form.TextInput, {label: 'input 1'}, {name: 'input_1'});
		myform.updateLayout();
		myform.appendTo(this.container);

		myform.updateLayout();

		// The bug used to equal this length to 0. By removing el.innerHTML = ''; from updateLayout on the
		// form manager, we tackled this issues on IE.
		YAHOO.util.Assert.areEqual(3, myform.get('layout').get('element').childNodes[0].childNodes.length);
	},

	test_complex_layout: function() {
		try {
			var myform = new BASESOFT.util.form.Form();
			this._garbage.push(myform);

			myform.appendTo(this.container);

			myform.addInput(BASESOFT.util.form.TextInput, {label: 'input 1'}, {name: 'input_1'});
			myform.addInput(BASESOFT.util.form.TextInput, {label: 'input 2', labelDirection: BASESOFT.util.form.LABEL_RIGHT}, {name: 'input_2'});
			myform.addInput(BASESOFT.util.form.TextInput, {label: 'input 3'}, {name: 'input_3'});
			myform.addInput(BASESOFT.util.form.TextInput, {label: 'input 4'}, {name: 'input_4'});
			myform.addInput(BASESOFT.util.form.TextInput, {label: 'input 5'}, {name: 'input_5'});
			var btnOk = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {name: 'btn_ok', label: 'OK'});
			myform.addInput(BASESOFT.util.form.ButtonInput, {}, {name: 'btn_cancel', label: 'Cancel'});

			myform.set('layout', new BASESOFT.util.form.Fieldset({
				legend: 'My Fieldset',
				children: [
					'input_1',
					'input_2',
					new BASESOFT.util.form.ColumnView({children: [

						new BASESOFT.util.form.Column({
							children: ['input_3', 'input_4']
						}),

						new BASESOFT.util.form.Column({
							children: ['input_5']
						})
					]}),

					new BASESOFT.util.form.ButtonBar({
						children: ['btn_ok', 'btn_cancel']
					})

				]
			}));

			myform.updateLayout();

			btnOk.on('click', function() {
				console.info('clicky', arguments);
			});

			// FIXME: test is not asserting!

		} catch (e) {
			console.error(e);
			throw e;
		}


	},

	test_row_control1: function() {

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['myrowcontrol', 'normal']
			})
		});
		this._garbage.push(myform);

		myform.appendTo(this.container);

		var inputs = [{
			label: 'Value A',
			input: new BASESOFT.util.form.TextInput({
				name: 'value_a'
			})
		},{
			label: 'Value B',
			input: new BASESOFT.util.form.TextInput({
				name: 'value_b'
			})
		}, {
			label: 'select-label',
			input: new BASESOFT.util.form.Select({
				name: 'select-name'
			})
		}];

		var rowcontrolcontainer = new BASESOFT.util.form.RowControlContainer({inputs: inputs},{name: 'myrowcontrol'});
		myform.add(rowcontrolcontainer);
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'Yo Yo'},{name: 'normal'});
		myform.updateLayout();

		// add row
		rowcontrolcontainer.addRow();
		rowcontrolcontainer.input.set('value', [{value_a: 'a1', value_b: 'b1'},
		                                        {value_a: 'a2', value_b: 'b2'}]);
		var values = rowcontrolcontainer.input.get('value');

		YAHOO.util.Assert.areEqual(values[0].value_a, "a1");
		YAHOO.util.Assert.areEqual(values[0].value_b, "b1");
		YAHOO.util.Assert.areEqual(values[1].value_a, "a2");
		YAHOO.util.Assert.areEqual(values[1].value_b, "b2");

		// move row
		rowcontrolcontainer.moveRow(1, 0);
		values = rowcontrolcontainer.input.get('value');

		YAHOO.util.Assert.areEqual(values[0].value_a, "a2");
		YAHOO.util.Assert.areEqual(values[0].value_b, "b2");
		YAHOO.util.Assert.areEqual(values[1].value_a, "a1");
		YAHOO.util.Assert.areEqual(values[1].value_b, "b1");

		// remove row
		rowcontrolcontainer.removeRow(0);
		values = rowcontrolcontainer.input.get('value');

		YAHOO.util.Assert.areEqual(values.length, 1);
		YAHOO.util.Assert.areEqual(values[0].value_a, "a1");
		YAHOO.util.Assert.areEqual(values[0].value_b, "b1");

		// remove another row
		rowcontrolcontainer.removeRow(0);
		values = rowcontrolcontainer.input.get('value');

		// as view
		myform.set('asView', false);
		myform.updateLayout();

	},

	test_row_control_custom_button: function() {

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['myrowcontrol']
			})
		});
		this._garbage.push(myform);

		myform.appendTo(this.container);

		var inputs = [{
			label: 'Value A',
			input: new BASESOFT.util.form.TextInput({
				name: 'value_a'
			})
		},{
			label: 'Value B',
			input: new BASESOFT.util.form.TextInput({
				name: 'value_b'
			})
		}];

		var rowcontrolcontainer = new BASESOFT.util.form.RowControlContainer({inputs: inputs,customButtonLabel: 'Banana'},{name: 'myrowcontrol'});
		myform.add(rowcontrolcontainer);
		myform.updateLayout();

		// remove row
		rowcontrolcontainer.removeRow(0);
		rowcontrolcontainer.input.get('value');



	},

	test_row_control_pre_filled: function() {

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['myrowcontrol']
			})
		});
		this._garbage.push(myform);
		myform.appendTo(this.container);

		var inputs = [{
			label: 'Value A',
			input: new BASESOFT.util.form.TextInput({
				name: 'value_a'
			})
		},{
			label: 'Value B',
			input: new BASESOFT.util.form.TextInput({
				name: 'value_b'
			})
		}];

		var values = [{
			'value_a': 'My Value Row 1 Field A',
			'value_b': 'My Value Row 1 Field B'
		},{
			'value_a': 'My Value Row 2 Field A',
			'value_b': 'My Value Row 2 Field B'
		},{
			'value_a': 'My Value Row 3 Field A',
			'value_b': 'My Value Row 3 Field B'
		}];

		try {
			var rowcontrolcontainer = new BASESOFT.util.form.RowControlContainer({inputs: inputs, values: values},{name: 'myrowcontrol'});
			myform.add(rowcontrolcontainer);
		} catch(e) {
			console.error(e);
			throw e;
		}
		myform.updateLayout();
		var form_values = myform.getFormValues()['myrowcontrol'];
		for(var i=0; i < 3; i++) {
			YAHOO.util.Assert.areEqual(values[i].value_a, form_values[i].value_a);
			YAHOO.util.Assert.areEqual(values[i].value_b, form_values[i].value_b);
		}
	},

	test_with_validation: function() {

		try {
			var error_div = document.createElement('div');
			this.container.appendChild(error_div);

			var myform = new BASESOFT.util.form.Form({
				layout: new BASESOFT.util.form.Layout({
					children: ['my_first_input_1','my_first_input_2','my_first_input_3','my_first_input_4']
				}),
				validatonRpcUrl: '/basesoft.web.test/selenium/validation/test_non_wtform_validation/rpc/',
				validationNotifyElement: error_div
			});

			this._garbage.push(myform);

			myform.appendTo(this.container);
			var input1 = myform.addInput(BASESOFT.util.form.TextInput, {label: 'my container 1'},{name: 'my_first_input_1'});
			var input2 = myform.addInput(BASESOFT.util.form.TextInput, {label: 'my container 2'},{name: 'my_first_input_2'});
			var input3 = myform.addInput(BASESOFT.util.form.TextInput, {label: 'my container 3'},{name: 'my_first_input_3'});
			var input4 = myform.addInput(BASESOFT.util.form.TextInput, {label: 'my container 4'},{name: 'my_first_input_4'});

			myform.updateLayout();

			input1.set('value', 'kwak');
			input3.set('value', 'xxxxx');

			// FIXME: test is not asserting!

		}
		catch (e) {
			console.error(e);
			throw e;
		}
	},

	test_form_select1: function() {

		try {

			var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
				children: ['select-name']
			})});
			myform.appendTo(this.container);
			this._garbage.push(myform);

			var input = myform.addInput(BASESOFT.util.form.Select, {
				label: 'select-label'
			}, {
				name: 'select-name',
				value: 'option-value-4'
			});

			for (var i = 0; i < 10; i++) {
				input.addOption('option-name-' + i, 'option-value-' + i);
			}

			myform.updateLayout();

			// test if the value is set in the select object
			YAHOO.util.Assert.areSame('option-value-4', input.get('value'));

			// test if the <select> has the same value as the object
			YAHOO.util.Assert.areSame('option-value-4', input.input.value, "select element should have same value passed into constructor");

			// change the value
			input.set('value', 'option-value-2');

			// assert if the <select> has changed its value too
			YAHOO.util.Assert.areSame('option-value-2', input.input.value, "select element should have same value passed into constructor");

		} catch (e) {
			console.error(e);
			throw e;
		}

	},

	/**
	 * tests form select without a value
	 */
	test_form_select2: function() {

		try {

			var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
				children: ['select-name']
			})});
			myform.appendTo(this.container);
			this._garbage.push(myform);

			var input = myform.addInput(BASESOFT.util.form.Select, {
				label: 'select-label'
			}, {
				name: 'select-name'
			});

			for (var i = 0; i < 1; i++) {
				input.addOption('option-name-' + i, 'option-value-' + i);
			}

			myform.updateLayout();

			YAHOO.util.Assert.areSame('', input.get('value'));
			YAHOO.util.Assert.areSame(input.input.selectedIndex, -1);

		} catch (e) {
			console.error(e);
			throw e;
		}

	},

	test_form_select_no_value_bug: function() {

		var form = new BASESOFT.util.form.Form({
			layout : new BASESOFT.util.form.Fieldset({
				legend : gettext('Service Level'),
				children : [
					'status'
				],
				expandable : false,
				is_expanded : true
			})

		});

		var statusSelect = form.addInput(BASESOFT.util.form.Select, {label: 'Status' }, {
			name: 'status',
			options: [{
				value: 1,
				label: gettext('Active')
			}, {
				value: 2,
				label: gettext('Disabled for new registrations')
			}]
		});

		form.updateLayout();

		form.appendTo(this.container);
		this._garbage.push(form);

		// assert
		YAHOO.util.Assert.areSame(statusSelect.input.selectedIndex, -1);

	},

	testFormSelectAsView: function() {

		var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
			children: ['select-name']
			}),
			asView: true
		});
		myform.appendTo(this.container);
		this._garbage.push(myform);

		var input = myform.addInput(BASESOFT.util.form.Select, {
			label: 'select-label'
		}, {
			name: 'select-name'
		});

		console.warn('calling updateLauyout');
		myform.updateLayout();
		myform.setFormValues({'select-name': 'option-value-2'});


		BASESOFT.util.setTimeout(function() {
			this.resume(function() {
				for (var i = 0; i < 10; i++) {
					input.addOption('option-name-' + i, 'option-value-' + i);
				}
				YAHOO.util.Assert.areEqual(input.get('value'), 'option-value-2');
			});

		}, 1000, null, this);

		this.wait();

	},

	test_form_select_disable_on_single_option : function() {

		try {

			var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
				children: ['selector']
			})});
			myform.appendTo(this.container);
			this._garbage.push(myform);

			var input = myform.addInput(BASESOFT.util.form.Select, {
				label: 'SELECTOR'
			}, {
				name: 'selector',
				disableWhenSingleOption: true
			});

			input.addOption('just-one', 'just-one-value');

			myform.updateLayout();
			YAHOO.util.Assert.areSame(true, input.input.disabled);

		} catch (e) {
			console.error(e);
			throw e;
		}
	},

	test_auto_complete: function() {

		var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({ children: ['autocomplete1'] })});
		this._garbage.push(myform);

		myform.appendTo(this.container);

		var ds = new YAHOO.util.LocalDataSource(['value1', 'value2', 'value3']);

		var input = myform.addInput(BASESOFT.util.form.AutocompleteInput, {'label': 'my autocomplete'}, {'name': 'autocomplete1', 'datasource': ds});
		myform.updateLayout();

	},

	test_combobox: function() {

		var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({ children: ['combobox'] })});
		myform.appendTo(this.container);

		var ds = new YAHOO.util.LocalDataSource(['value1', 'value2', 'value3']);
		var combobox = myform.addInput(BASESOFT.util.form.Combobox, {'label': 'my combobox'}, {'name': 'combobox', 'datasource': ds});

		myform.updateLayout();
	},

	test_spotlight: function(){

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['spotlight']
			})
		});

		this._garbage.push(myform);

		myform.appendTo(this.container);

		var tango = '/site_media/basesoft.web.contrib.tango/16x16/mimetypes/';

		var ds = new YAHOO.util.DataSource([{
			'category': 'Top Hit',
			'icon': tango + 'video-x-generic.png',
			'name': 'this is a really long text that doesnt fit in the spotlight so it will need to do some cutting'
		}, {
			'category': 'Documents',
			'icon': tango + 'x-office-document.png',
			'name': 'readme.doc'
		}, {
			'category': 'Documents',
			'icon': tango + 'x-office-document.png',
			'name': 'todo.doc'
		}, {
			'category': 'Documents',
			'icon': tango + 'x-office-document.png',
			'name': 'overview.doc'
		}, {
			'category': 'Music',
			'icon': tango + 'audio-x-generic.png',
			'name': 'woof.mp3'
		}, {
			'category': 'Music',
			'icon': tango + 'audio-x-generic.png',
			'name': 'meow.mp3'
		}]);


		var spotlight = myform.addInput(BASESOFT.util.form.SpotLight, {
			'label': 'my spotlight'
		}, {
			'name': 'spotlight',
			'datasource': ds
		});

		spotlight.on('onAutocompleteLoaded', function() {
			this.resume(function() {
				spotlight.autocomplete.sendQuery('');
			});
		}, null, this);

		myform.updateLayout();

		this.wait();
	},

	/**
	 * This test the asView option on the form.
	 */
	test_as_view: function() {

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['input_a', 'input_b']
			}),
			asView: true
		});

		this._garbage.push(myform);

		myform.appendTo(this.container);

		var input_a = myform.addInput(BASESOFT.util.form.TextInput,
			{label: 'Input A'},{name: 'input_a'});
		var input_b = myform.addInput(BASESOFT.util.form.TextInput,
			{label: 'Input B'},{name: 'input_b'});

		input_a.set('value', 'This is the Value for Input A');
		myform.updateLayout();

		for(var i in myform._childs) {
			var child = myform._childs[i];
			YAHOO.util.Assert.areEqual(child.get('asView'), true);
		}
	},

	test_uploader: function() {
		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['my_uploader']
			})
		});

		this._garbage.push(myform);
		myform.appendTo(this.container);

		var input1 = myform.addInput(BASESOFT.util.form.FileUpload,
			{label: ''},{name: 'my_uploader', maxFiles: 1,
					allowMultipleFiles: false});
		myform.updateLayout();
	},

	test_datetime_input: function() {

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['my_datetime']
			})
		});

		this._garbage.push(myform);
		myform.appendTo(this.container);

		var dt = myform.addInput(BASESOFT.util.form.DateTime,
			{label: ''},{ name: 'my_datetime'});
		myform.updateLayout();

		dt.on('dateTimeReady', function() {
			this.resume( function() {
				var day = 15;
				var month = 3;
				var year = 2009;
				var d = new Date();
				d.setFullYear(year, month-1, day);
				dt.calendar.select(d);
				dt.time_input.value = '01:15:02';
				dt._createValue();
				YAHOO.util.Assert.isTrue(dt.get("value") instanceof Date);
				YAHOO.util.Assert.areEqual(dt.get("value").getFullYear(), year);
				YAHOO.util.Assert.areEqual(dt.get("value").getMonth(), month - 1); // getMonth returns ints from 0 to 11
				YAHOO.util.Assert.areEqual(dt.get("value").getDate(), day);
				YAHOO.util.Assert.areEqual(dt.get("value").getHours(), 1);
				YAHOO.util.Assert.areEqual(dt.get("value").getMinutes(), 15);
				YAHOO.util.Assert.areEqual(dt.get("value").getSeconds(), 0);
			});
		}, null, this);
		this.wait();
	},

	test_datetimef_input_from_input: function() {
		try {
			BASESOFT.env.L10n.DATE_FIELD_DELIMITER = '/';
			BASESOFT.env.L10n.DATE_RANGE_DELIMITER = '-';

			BASESOFT.env.L10n.MDY_DAY_POSITION = 1;
			BASESOFT.env.L10n.MDY_MONTH_POSITION = 2;
			BASESOFT.env.L10n.MDY_YEAR_POSITION = 3;

			console.warn('BASESOFT.env.L10', BASESOFT.env.L10n);

			var myform = new BASESOFT.util.form.Form({
				layout: new BASESOFT.util.form.Layout({
					children: ['my_datetime']
				})
			});

			this._garbage.push(myform);
			myform.appendTo(this.container);

			var dt = myform.addInput(BASESOFT.util.form.DateTime,
				{label: ''},{ name: 'my_datetime'});
			myform.updateLayout();

			dt.input.value = '31/12/2012 12:11';
			dt._input_blur();

			var date = dt.get("value");
			console.warn('date', date);

			YAHOO.util.Assert.isTrue(dt.get("value") instanceof Date);
			YAHOO.util.Assert.areEqual(dt.get("value").getFullYear(), 2012);
			YAHOO.util.Assert.areEqual(dt.get("value").getMonth(), 12 - 1); // getMonth returns ints from 0 to 11
			YAHOO.util.Assert.areEqual(dt.get("value").getDate(), 31);
			YAHOO.util.Assert.areEqual(dt.get("value").getHours(), 12);
			YAHOO.util.Assert.areEqual(dt.get("value").getMinutes(), 11);
			YAHOO.util.Assert.areEqual(dt.get("value").getSeconds(), 0);
		} catch (e) {
			console.error(e);
			throw e;
		}

	},

	test_inline: function() {

		try {


			var mail_form = new BASESOFT.util.form.Form({
				layout: new BASESOFT.util.form.Layout({
					children: ['mail_subject', 'mail_header', 'mail_content', 'mail_footer']
				}),
				values: {'mail_subject': 'mail subject'}
			});
			this._garbage.push(mail_form);
			mail_form.appendTo(this.container);

			// subject
			mail_form.addInput(BASESOFT.util.form.InlineTextInput, {
				label: 'Subject'
			}, {
				name: 'mail_subject'
			});

			// header
			mail_form.addInput(BASESOFT.util.form.InlineTextAreaInput, {
				label: 'Header'
			}, {
				name: 'mail_header'
			});

			// body
			mail_form.addInput(BASESOFT.util.form.InlineTextAreaInput, {
				label: 'Body'
			}, {
				name: 'mail_content',
				replacementFields: {
					'date.weekday': 'Today\'s weekday',
					'Contact': {
						'contact.first_name': 'First Name',
						'contact.last_name': 'Last Name',
						'contact.telephone': 'Telephone'
					},
					'Company': {
						'company.name': 'Name',
						'address': 'Address',
						'Country': {
							'contact.country.name': 'Name',
							'contact.country.currency': 'Currency'
						}
					}
				}
			});

			// footer
			mail_form.addInput(BASESOFT.util.form.InlineTextAreaInput, {
				label: 'Footer'
			}, {
				name: 'mail_footer'
			});

			mail_form.updateLayout();

			// wait a bit before finishing the test so that the RTE's n stuff can initialised
			// that happens async
			var self = this;
			setTimeout(function() {
				self.resume(function() {
					console.info('test done');
				});
			}, 1000);
			this.wait();
		}
		catch (e) {
			e.delay || console.error(e);
			throw e;
		}

	},

	/*
	 * this tests setting the form values using the 'values' attribute;
	 */
	test_set_form_values: function() {

		var form_values = {my_select: 2, my_input: "test_value"};

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
						children: ['my_select', 'my_input']
						}),
			values: form_values
		});
		this._garbage.push(myform);

		var my_input = myform.addInput(BASESOFT.util.form.TextInput, {label: 'My input'}, {name: 'my_input'});

		var sel = new BASESOFT.util.form.Select({name: 'my_select'});
		sel.addOption('',null);
		sel.addOption('Value 1', 1);
		sel.addOption('Value 2', 2);
		var sel_container = new BASESOFT.util.form.InputContainer({label: 'my select'});
		sel_container.add(sel);
		myform.add(sel_container);

		myform.appendTo(this.container);
		myform.updateLayout();

		YAHOO.util.Assert.areEqual(my_input.get("value"), form_values.my_input);
		YAHOO.util.Assert.areEqual(sel.get("value"), form_values.my_select);

	},

	test_clickable_fancy_select: function() {

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['clickable_fancy_select']
			})
		});
		this._garbage.push(myform);

		var container = new BASESOFT.util.form.ClickableFancySelectContainer({
			}, {
				name: 'clickable_fancy_select',
				label_default: gettext("Available options"),
				label_selected: gettext("Selected options"),
				defaultoptions: [
					{ label: 'One', value: '1' },
					{ label: 'Two', value: '2' },
					{ label: 'Three', value: '3'},
					{ label: 'Four', value:'4'}
				]
			});
		myform.add(container);
		myform.appendTo(this.container);
		myform.updateLayout();

		container.panel.render();
		container.input.select(['1', '3']);

		YAHOO.util.Assert.areEqual(container.input.get("selectedoptions").length, 2);
	},

	test_checkbox_input: function() {

		try {

			var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
				children: [new BASESOFT.util.form.ColumnView({children: [
					new BASESOFT.util.form.Column({
						children: ['checkbox1', 'checkbox2', 'checkbox3', 'checkbox4', 'checkbox5', 'checkbox6']
					})
				]})]
			})});
			myform.appendTo(this.container);
			this._garbage.push(myform);

			var chk1 = myform.addInput(BASESOFT.util.form.CheckboxInput, {label: 'Checkbox 1'}, {
				name: 'checkbox1', value: true
			});

			var chk2 = myform.addInput(BASESOFT.util.form.CheckboxInput, {label: 'Checkbox 2'}, {
				name: 'checkbox2', checked: true
			});

			var chk3 = myform.addInput(BASESOFT.util.form.CheckboxInput, {label: 'Checkbox 3'}, {
				name: 'checkbox3', value: false, checked: true
			});

			var chk4 = myform.addInput(BASESOFT.util.form.CheckboxInput, {label: 'Checkbox 4'}, {
				name: 'checkbox4', value: false
			});

			var chk5 = myform.addInput(BASESOFT.util.form.CheckboxInput, {label: 'Checkbox 5'}, {
				name: 'checkbox5', checked: false
			});

			var chk6 = myform.addInput(BASESOFT.util.form.CheckboxInput, {label: 'Checkbox 5'}, {
				name: 'checkbox6'
			});

			myform.updateLayout();

			// test if checkboxes have the expected value
			YAHOO.util.Assert.areSame(true, chk1.get('value'), 'checkbox 1 check failed');
			YAHOO.util.Assert.areSame(true, chk2.get('value'), 'checkbox 2 check failed');
			YAHOO.util.Assert.areSame(false, chk3.get('value'), 'checkbox 3 check failed');
			YAHOO.util.Assert.areSame(false, chk4.get('value'), 'checkbox 4 check failed');
			YAHOO.util.Assert.areSame(false, chk5.get('value'), 'checkbox 5 check failed');
			YAHOO.util.Assert.areSame(false, chk6.get('value'), 'checkbox 6 check failed');

		} catch (e) {
			console.error(e);
			throw e;
		}

	},

	test_checkbox_2_input: function() {

			var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
				children: [new BASESOFT.util.form.ColumnView({children: [
					new BASESOFT.util.form.Column({
						children: ['checkbox1', 'checkbox2']
					})
				]})]
			})});
			myform.appendTo(this.container);
			this._garbage.push(myform);

			var chk1 = myform.addInput(BASESOFT.util.form.CheckboxInput2, {label: 'Checkbox 1'}, {
				name: 'checkbox1', value: true
			});

			var chk2 = myform.addInput(BASESOFT.util.form.CheckboxInput2, {label: 'Checkbox 2'}, {
				name: 'checkbox2', value: false
			});

			myform.updateLayout();

			// assert initial value
			YAHOO.util.Assert.areSame(true, chk1.get('value'));
			YAHOO.util.Assert.areSame(true, chk1.input.checked);

			YAHOO.util.Assert.areSame(false, chk2.get('value'));
			YAHOO.util.Assert.areSame(false, chk2.input.checked);

			// wait for first checkbox click
			var chk1_cb = function() {
				this.resume(function() {

					// prevents cb to be called twice
					chk1.unsubscribe('valueChange', chk1_cb);

					// assert its unchecked now
					YAHOO.util.Assert.areSame(false, chk1.get('value'));
					YAHOO.util.Assert.areSame(false, chk1.input.checked);

					// change it back by changing setting value
					chk1.set('value', true);
					YAHOO.util.Assert.areSame(true, chk1.get('value'));
					YAHOO.util.Assert.areSame(true, chk1.input.checked);
				});

			};

			chk1.subscribe('valueChange', chk1_cb, null, this);

			// click on the first checkbox (goto ^)
			YAHOO.util.UserAction.click(chk1.input);
			this.wait();
	},

	testBitwiseCheckbox: function() {

		try {

			var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
				children: [new BASESOFT.util.form.ColumnView({children: [
					new BASESOFT.util.form.Column({
						children: ['bitwise_group']
					})
				]})]
			})});
			myform.appendTo(this.container);
			this._garbage.push(myform);

			var weight = myform.addInput(BASESOFT.util.form.BitWiseCheckboxGroup, {
				label: gettext('Bitwise Checkbox Group'),
				labelDirection: BASESOFT.util.form.LABEL_TOP
			}, {
				name: 'bitwise_group',
				disabled: true,
				options: [{
					label: gettext('Checkbox 1')
				},{
					label: gettext('Checkbox 2')
				}, {
					label: gettext('Checkbox 3')
				}]
			});

			myform.updateLayout();

			var child, i;
			for (i in weight._checkbox_els) {
				child = weight._checkbox_els[i];
				YAHOO.util.Assert.areSame(true, child.disabled, 'checkbox '+i+' check failed');
			}

			weight.set('disabled', false);

			for (i in weight._checkbox_els) {
				child = weight._checkbox_els[i];
				YAHOO.util.Assert.areSame(false, child.disabled, 'checkbox '+i+' check failed');
			}

		} catch (e) {
			console.error(e);
			throw e;
		}

	},

	test_period_input: function() {
		// define the env months which are used by this input and which are not defined normally in tests
		BASESOFT.env = BASESOFT.env || {};
		BASESOFT.env.L10n = BASESOFT.env.L10n || {};
		BASESOFT.env.L10n.MONTHS_LONG = ["January", "February", "March", "April", "May", "June", "July", "August",
										 "September", "October", "November", "December"];

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['myperiod']
			})
		});

		var start = new Date(2009, 0, 1);
		var end = new Date(2009, 2, 32);

		var i = myform.addInput(BASESOFT.util.form.PeriodInput, {
			label: 'Period'
		}, {
			name: 'myperiod',
			value: [start, end]
		});
		myform.appendTo(this.container);
		myform.updateLayout();

		// assert.
		YAHOO.util.Assert.areEqual(i.get("period_name"), 'q1');
		YAHOO.util.Assert.areEqual(i.get("value")[0].getFullYear(), 2009);

		start = new Date(2008, 10, 1);
		end = new Date(2008, 10, 32);

		i.set('value', [start, end]);

		YAHOO.util.Assert.areEqual(i.get("period_name"), 'm10');
		YAHOO.util.Assert.areEqual(i.get("value")[0].getFullYear(), 2008);
	},

	test_color_picker: function() {

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['mycolor']
			})
		});

		this._garbage.push(myform);


		var i = myform.addInput(BASESOFT.util.form.ColorPicker, {
			label: 'Color'
		}, {
			name: 'mycolor'
		});
		myform.appendTo(this.container);
		myform.updateLayout();

		BASESOFT.util.waitForCondition(function() {
			return YAHOO.util.Selector.query('.color-box').length > 0;
		}, function() {

			var _this = this;
			setTimeout(function() {
				_this.resume(function() {
					console.warn('done!');
				});
			}, 2000);


		}, null, this);

		this.wait();
	},

	test_grouped_select : function ()  {

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['myselect']
			}),
			values: {'myselect' : {'group_a_id': 'A2'}}
		});

		this._garbage.push(myform);

		var select = myform.addInput(BASESOFT.util.form.GroupedSelect, {
			label: 'Grouped select'
		}, {
			name: 'myselect',
			emptyFirstOption: true,
			options: [{
				'label': 'Group A',
				'id':'group_a_id',
				'options': [{
					'label': 'A-1',
					'value': 'A1'
				}, {
					'label': 'A-2',
					'value': 'A2'
				}]
			}, {
				'label': 'Group B',
				'id':'group_b_id',
				'options': [{
					'label': 'B-1',
					'value': 'B1'
				}, {
					'label': 'B-2',
					'value': 'B2'
				}, {
					'label': 'B-3',
					'value': 'B3'
				}]
			}]
		});

		myform.appendTo(this.container);
		myform.updateLayout();

		select.input.selectedIndex = 5;
		var evt = document.createEvent("HTMLEvents");
	    evt.initEvent("change", false, true);
	    select.input.dispatchEvent(evt);

		YAHOO.util.Assert.areEqual(myform.getFormValues().myselect['group_b_id'], 'B3');

	},


	test_decimal: function() {

		try {

			BASESOFT.env.L10n.DECIMAL_SEPARATOR = ',';
			BASESOFT.env.L10n.THOUSAND_SEPARATOR = '.';

			var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
				children: ['digit_input', 'decimal_input']
			})});
			this._garbage.push(myform);

			myform.appendTo(this.container);
			var input = myform.addInput(BASESOFT.util.form.DigitInput, {label: 'digit_input'},{name: 'digit_input'});
			input = myform.addInput(BASESOFT.util.form.DecimalInput, {label: 'decimal input'},{name: 'decimal_input', value: ''});
			myform.updateLayout();

			// assert that decimal returns null when empty string supplied
			YAHOO.util.Assert.areEqual(input.get('value'), null);
			// set decimal
			input.set('value', new BASESOFT.util.Decimal('123123.1234'));

			// assert what users sees
			YAHOO.util.Assert.areEqual('123.123,1234', input.input.value);
			// assert internal value
			YAHOO.util.Assert.areEqual('123123.1234', input.get('value')._value);

		} catch (e) {
			console.error(e);
			throw e;
		}


	},

	test_wiki_simple: function() {

		var myform, input;

		this.container.style.width = '800px';

		myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['text1', 'wiki_input', 'text2', 'wiki_input2']
			}),
			values: {'wiki_input':
			"= heading1 =\n" +
			"== heading2 ==\n" +
			"some '''bold''' text\n" +
			"this is a newline\n" +
			"\n"+
			"\n"+
			"a paragraph\n" +
			"\n"+
			"a ruler: \n"+
			"----\n"+
			"\n"+
			"list:\n" +
			" * item1\n"+
			" * item2\n"+
			" * item3\n"+
			"and some more text\n" +
			"second line\n" +
			""}

			//asView: true
		});

		this._garbage.push(myform);

		myform.addInput(BASESOFT.util.form.TextInput, {label: 'text 1'},{name: 'text1'});

		input = myform.addInput(BASESOFT.util.form.WikiInput, {label: 'wiki_input'}, {name: 'wiki_input', width: '800px'});

		myform.addInput(BASESOFT.util.form.TextAreaInput, {label: 'text 2', labelDirection: BASESOFT.util.form.LABEL_TOP},{name: 'text2'});

		input = myform.addInput(BASESOFT.util.form.WikiInput, {
			label: 'wiki_input',
			labelDirection: BASESOFT.util.form.LABEL_TOP
		}, {
			name: 'wiki_input2',
			width: '100%',
			previewAlignRight: false
		});

		myform.appendTo(this.container);
		myform.updateLayout();

		//YAHOO.util.Assert.areEqual(myform.getFormValues().myselect['group_b_id'], 'B3');
	},


	test_wiki_replacement_fields : function () {

		var myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['wiki_input']
			}),
			values: {'wiki_input':
			"= heading1 =\n" +
			"== heading2 ==\n" +
			"some '''bold''' text\n" +
			"this is a newline\n" +
			"\n"+
			"\n"+
			"a paragraph\n" +
			"\n"+
			"a ruler: \n"+
			"----\n"+
			"\n"+
			"list:\n" +
			" * item1\n"+
			" * item2\n"+
			" * item3\n"+
			"and some more text\n" +
			"second line\n" +
			""}
		});

		this._garbage.push(myform);
		var input = myform.addInput(BASESOFT.util.form.WikiInput, {label: 'wiki_input'}, {
			name: 'wiki_input',
			width: '800px',
			replacementFieldRpcUrl: '/basesoft.web.test/selenium/yui/replacement_fields/',
			replacementFieldRpcMethod: 'get_replacement_fields'
		});

		myform.appendTo(this.container);
		myform.updateLayout();

	},


	test_number_input: function() {

		var myform, input;

		myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['numeric_input', 'btn_debug']
			}),
			values: {'numeric_input': '1234'} // notice input type is a string like the user would type.. assert int when getting values
		});

		this._garbage.push(myform);

		myform.addInput(BASESOFT.util.form.NumberInput, {label: 'numeric_input'}, {name: 'numeric_input'});

		var btnOk = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {name: 'btn_debug', label: 'OK'});

		btnOk.subscribe('click', function() {
			console.warn('value is', myform.getFormValues()['numeric_input']);
		});

		myform.appendTo(this.container);
		myform.updateLayout();

		YAHOO.util.Assert.areSame(1234, myform.getFormValues()['numeric_input']);
	},

	test_text_blur: function() {

		var myform, input;

		myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['input', 'btn_debug']
			}),
			values: {'input': '1234'} // notice input type is a string like the user would type.. assert int when getting values
		});

		this._garbage.push(myform);

		myform.addInput(BASESOFT.util.form.TextInput, {label: 'input'}, {name: 'input'});

		var btnOk = myform.addInput(BASESOFT.util.form.ButtonInput, {}, {name: 'btn_debug', label: 'OK'});

		btnOk.subscribe('click', function() {
			console.warn('value is', myform.getFormValues()['input']);
		});

		myform.appendTo(this.container);
		myform.updateLayout();

	},

	test_year_input: function() {

		var myform;

		myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['year_input']
			}),
			values: {'year_input': '2013'}
		});

		this._garbage.push(myform);

		myform.addInput(BASESOFT.util.form.YearInput, {label: 'year input'}, {name: 'year_input'});

		myform.appendTo(this.container);
		myform.updateLayout();

		YAHOO.util.Assert.areSame(2013, myform.getFormValues()['year_input']);
	},

	test_ie_selected_index_bug : function ( ){

		var myform, input;
		myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['myselect']
			})
		});
		this._garbage.push(myform);

		input = myform.addInput(BASESOFT.util.form.Select, {
			label: 'BOOM Selectah!'
		}, {
			name: 'myselect'
		});

		input.addOption('Option A', 1);
		input.addOption('Option B', 2);
		input.addOption('Option C', 3);
		input.addOption('Option D', 4);

		myform.appendTo(this.container);

		YAHOO.util.Assert.areSame(-1, input.input.selectedIndex, 'Pre Update Layout');

		myform.updateLayout();

		YAHOO.util.Assert.areSame(-1, input.input.selectedIndex, 'Post Update Layout');

	},

	test_last_column_width : function () {

		var myform;

		var ca = new BASESOFT.util.form.ColumnView({
			children: [new BASESOFT.util.form.Column({children: ['a']}), new BASESOFT.util.form.Column({children: ['b']}), new BASESOFT.util.form.Column({children: ['c']})]
		});

		var cb = new BASESOFT.util.form.ColumnView({
			children: [new BASESOFT.util.form.Column({children: ['d'], width: 50}), new BASESOFT.util.form.Column({children: ['e']}), new BASESOFT.util.form.Column({children: ['f']})]
		});

		var cc = new BASESOFT.util.form.ColumnView({
			children: [
				new BASESOFT.util.form.Column({children: ['g']}),
				new BASESOFT.util.form.Column({children: ['h']}),
				new BASESOFT.util.form.Column({children: ['i']}),
				new BASESOFT.util.form.Column({children: ['j']}),
				new BASESOFT.util.form.Column({children: ['k']}),
				new BASESOFT.util.form.Column({children: ['l']}),
				new BASESOFT.util.form.Column({children: ['m']})
			]
		});

		myform = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: [ca, cb, cc]
			})
		});

		this._garbage.push(myform);

		myform.addInput(BASESOFT.util.form.TextInput, {label: 'A'}, {name: 'a'});
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'B'}, {name: 'b'});
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'C'}, {name: 'c'});

		myform.addInput(BASESOFT.util.form.TextInput, {label: 'D'}, {name: 'd'});
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'E'}, {name: 'e'});
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'F'}, {name: 'f'});

		myform.addInput(BASESOFT.util.form.TextInput, {label: 'G'}, {name: 'g'});
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'H'}, {name: 'h'});
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'I'}, {name: 'i'});
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'J'}, {name: 'j'});
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'K'}, {name: 'k'});
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'L'}, {name: 'l'});
		myform.addInput(BASESOFT.util.form.TextInput, {label: 'M'}, {name: 'm'});

		myform.appendTo(this.container);
		myform.updateLayout();

		YAHOO.util.Assert.areSame(ca.get('children')[0].get('width'), 33);
		YAHOO.util.Assert.areSame(ca.get('children')[2].get('width'), 34);
		YAHOO.util.Assert.areSame(cb.get('children')[0].get('width'), 50);
		YAHOO.util.Assert.areSame(cb.get('children')[2].get('width'), 25);
		YAHOO.util.Assert.areSame(cc.get('children')[0].get('width'), 14);
		YAHOO.util.Assert.areSame(cc.get('children')[6].get('width'), 16);

	},

	testGetInputFieldsetLegend : function () {

		var simpleForm = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Fieldset({
				legend: 'My Fieldset',
				children: ['input1', 'input2', 'input3']
			})
		});

		YAHOO.util.Assert.areSame(simpleForm.getInputFieldsetLegend('input2'), 'My Fieldset');


		var nestedForm = new BASESOFT.util.form.Form({
			layout: new BASESOFT.util.form.Layout({
				children: ['input1', new BASESOFT.util.form.Fieldset({
					legend: 'My Second Fieldset',
					children: ['input4', 'input5']
				}), 'input2', 'input3']
			})
		});

		YAHOO.util.Assert.areSame(nestedForm.getInputFieldsetLegend('input1'), null);
		YAHOO.util.Assert.areSame(nestedForm.getInputFieldsetLegend('input4'), 'My Second Fieldset');


	},
	/**
	 * Test jQuery File uploader
	 */
	testJQueryUploader: function () {

		try {

			var myform = new BASESOFT.util.form.Form({
				layout: new BASESOFT.util.form.Layout({
					children: ['uploader']
				})
			});

			this._garbage.push(myform);
			myform.appendTo(this.container);

			var input1 = myform.addInput(BASESOFT.util.form.FileUpload, {
				label: 'Upload a file'
			},{
				name: 'uploader',
				maxFiles: 30,
				maxFileSize: 1024*1024*250,
				fileTypes: ['application/pdf', 'image/jpg'],
				allowMultipleFiles: true,
				addDropzone: true
			});

			myform.updateLayout();


		} catch (e) {
			console.error(e);
			throw e;
		}

	},

	testHelp : function(){

		try {

			var myform = new BASESOFT.util.form.Form({
				layout: new BASESOFT.util.form.Layout({
					children: [
						'normal',
						new BASESOFT.util.form.ColumnView({children: [

							new BASESOFT.util.form.Column({
								children: ['top', 'bottom']
							}),

							new BASESOFT.util.form.Column({
								children: ['right']
							})
						]})
					]
				})
			});

			this._garbage.push(myform);
			myform.appendTo(this.container);

			myform.addInput(BASESOFT.util.form.TextInput, {
				label: 'my label 1',
				help: 'normal help'
			}, {
				name: 'normal'
			});

			myform.addInput(BASESOFT.util.form.TextInput, {
				label: 'my label 2',
				labelDirection: BASESOFT.util.form.LABEL_TOP,
				help: 'top help'
			}, {
				name: 'top'
			});

			myform.addInput(BASESOFT.util.form.TextInput, {
				label: 'my label 3',
				labelDirection: BASESOFT.util.form.LABEL_BOTTOM,
				help: 'bottom help'
			}, {
				name: 'bottom'
			});

			myform.addInput(BASESOFT.util.form.TextInput, {
				label: 'my label 4',
				labelDirection: BASESOFT.util.form.LABEL_RIGHT,
				help: 'right help'
			}, {
				name: 'right'
			});

			myform.updateLayout();


		} catch (e) {
			console.error(e);
			throw e;
		}

	},

	testBitwiseCheckboxGroup : function () {
		try {

			var myform = new BASESOFT.util.form.Form({
				layout: new BASESOFT.util.form.Layout({
					children: ['bitwise']
				})
			});

			myform.addInput(BASESOFT.util.form.BitWiseCheckboxGroup, {

			}, {
				name: 'bitwise',
				options: [{
					label: 'First bit',
					value: 1
				},{
					label: 'Second bit',
					value: 2
				},{
					label: 'Third bit',
					value: 4
				},{
					label: 'Fourth bit',
					value: 8
				}],
				value: 5
			});

			this._garbage.push(myform);
			myform.appendTo(this.container);

			myform.updateLayout();

		} catch (e) {
			console.error(e);
			throw e;
		}
	},

	test_form_select_index_bug: function() {

		var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({
			children: ['select-name']
		})});
		myform.appendTo(this.container);
		this._garbage.push(myform);

		var input = myform.addInput(BASESOFT.util.form.Select, {
			label: 'select-label'
		}, {
			name: 'select-name'
		});

		myform.updateLayout();

		input.addOption('1', 1);
		input.addOption('0', 0);

		YAHOO.util.Assert.areSame('', input.get('value'));
		YAHOO.util.Assert.areSame(input.input.selectedIndex, -1);

	},

	test_autocomplete2_local: function() {

		// FIXME: need a test that creates a local datasource and pass that to the Autocomplete2

		/*
		var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({ children: ['autocomplete1'] })});
		this._garbage.push(myform);

		myform.appendTo(this.container);

		var p = [{
			value: 1,
			label: 'WEEEE'
		}, {
			value: 2,
			label: 'HOOOO'
		}];
		var input = myform.addInput(BASESOFT.util.form.Autocomplete2, {'label': 'my autocomplete'}, {'name': 'autocomplete1', options: p});
		myform.updateLayout();
		input.set('value', 2);
		YAHOO.util.Assert.areSame(2, input.get('value'));
		*/
	},

	test_autocomplete2_rpc: function() {

		BASESOFT.env.account.id = 1;

		var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({ children: ['autocomplete1'] })});
		this._garbage.push(myform);

		myform.appendTo(this.container);


		var datasource = new BASESOFT.util.JsonRpcDataSource({
			rpcUrl: '/basesoft.web.test/selenium/yui/json_datasource_country/',
			rpcMethod: 'json_datasource_country'
		});

		var input = myform.addInput(BASESOFT.util.form.Autocomplete2, {
			'label': 'my autocomplete'
		}, {
			'name': 'autocomplete1',
			datasource: datasource
		});
		myform.updateLayout();

		// set country-id = 2
		input.set('value', 155);
		input.on('labelLoaded', function() {
			this.resume(function() {

				YAHOO.util.Assert.areSame(155, input.get('value'));
				YAHOO.util.Assert.areSame('Netherlands', input.input.value);

				console.warn('test done');
			});
		}, null, this);

		this.wait();

	},

	test_autocomplete2_rpc_rowcontrol: function() {

		BASESOFT.env.account.id = 1;

		var myform = new BASESOFT.util.form.Form({layout: new BASESOFT.util.form.Layout({ children: ['links'] })});
		this._garbage.push(myform);

		var datasource = new BASESOFT.util.JsonRpcDataSource({
			rpcUrl: '/basesoft.web.test/selenium/yui/json_datasource_country/',
			rpcMethod: 'json_datasource_country'
		});

		var input = new BASESOFT.util.form.Combobox2({
			'name': 'autocomplete1',
			datasource: datasource
		});

		// Row control inputs.
		var inputs = [{
			label: gettext('Countries'),
			input: input
		}];

		var rowControl = new BASESOFT.util.form.RowControlContainer({
			inputs: inputs
		}, {
			name: 'links'
		});
		myform.add(rowControl);
		myform.appendTo(this.container);


		myform.updateLayout();

		// set country-id = 2
		var rowInputs = rowControl.getRowInputs();
		input = rowInputs[0][0];
		input.set('value', 155);
		input.on('labelLoaded', function() {
			this.resume(function() {

				YAHOO.util.Assert.areSame(155, input.get('value'));
				YAHOO.util.Assert.areSame('Netherlands', input.input.value);

				console.warn('test done');
			});
		}, null, this);

		this.wait();

	}



}));


YAHOO.register("test_form_manager", null, {version: "1.0.0", build: "1"});

})();
