/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 */

BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "Test Growl",
	assertClean: true,

	_should: {
		error: {
			testUnregisteredGrowl: new Error("Application 'My application' is not registered."),
			testNotAllowedGrowl: new Error("Application 'Messenger' doesn't accept type 'Buddy buzz'.")
		}
	},

	setUp: function() {
		this.div = document.createElement('div');
		this.div.id = 'bs-growl-container';
		YAHOO.util.Dom.get('sandbox').appendChild(this.div);

		// create the growler that takes care of all the growls we send
		// for this test, we'll show the growls in the bottom left corner
		this._growler = new BASESOFT.widget.Growler({
			position: BASESOFT.widget.Growler.prototype.POSITION_BOTTOM_LEFT
		});
		this._garbage.push(this._growler);
	},

	testShowGrowl: function () {
		try {
			this._growler.register({
				application: 'Messenger',
				icon: 'download', // not mandatory
				allowed: ['Friend logs in', 'Friend logs out', 'Friend buzz']
			});

			var growl1 = new BASESOFT.widget.Growl({
				name: 'Friend logs out',
				title: 'A friend has logged out',
				type: 'warning',
				icon: 'download', // not mandatory
				message: 'test message 1', // message body is created automatically from string
				priority: -2, // lowest priority
				lifetime: 2000, // should be shown for 2 seconds
				opacity: 0.2 // make it almost transparent
			});

			growl1.destroyEvent.subscribe(function() {
				this.resume(function() {
					try {
						// we should have only one growl displayed,
						// because growl1 closes in 2 seconds
						YAHOO.util.Assert.areEqual(this._growler.growls.length, 1);

						// click on the second growl and wait for it to be destroyed
						YAHOO.util.UserAction.click(growl2.element);
						this.wait();
					}
					catch (e) {
						console.error(e);
						throw e;
					}
				});
			}, null, this);

			var growl2 = new BASESOFT.widget.Growl({
				name: 'Friend buzz',
				title: 'A friend buzzed you',
				type: 'critical',
				priority: 2, // highest priority
				lifetime: 0, // should be shown until user closes it
				minLifetime: 2000, // the user can't close this one by clicking it in the first 2 seconds
				fadeOutDuration: 0 // should dissapear without an animation when clicked on it
			});

			growl2.destroyEvent.subscribe(function() {
				this.resume(function() {
					console.info('test finished');
				});
			}, null, this);

			var message = document.createElement("p");
			message.textContent = 'test message 2';

			// add our html-built message to the body of the growl
			growl2.body.appendChild(message);

			this._growler.growl('Messenger', growl1);
			this._growler.growl('Messenger', growl2);

			// we should have 2 growls displayed
			YAHOO.util.Assert.areEqual(this._growler.growls.length, 2);

			// check that the first growl displayed is a 'Friend buzz'
			YAHOO.util.Assert.areEqual(this._growler.growls[1].get('title'), 'A friend buzzed you');

			// growls are shown, and after growl1 dissapears, the test is resumed
			this.wait();

		} catch(e){
			e.delay || console.error(e);
			throw e;
		}
	},

	testDontShowDuplicates: function () {
		try {
			this._growler.register({
				application: 'Messenger',
				allowed: ['Buddy logged in', 'Buddy logged out']
			});

			var growl1 = new BASESOFT.widget.Growl({
				name: 'Buddy logged in',
				title: 'Bubu has logged in',
				message: 'Your friend Bubu has just logged.',
				lifetime: 2000 // should be shown for 2 seconds
			});
			this._growler.growl('Messenger', growl1);
			// Add hide listener to verify length of growls and call resume.
			growl1.destroyEvent.subscribe(function() {
				this.resume(function() {
					try {
						// we should have only one growl displayed,
						// because growl1 closes in 2 seconds
						YAHOO.util.Assert.areEqual(this._growler.growls.length, 0);
					}
					catch (e) {
						console.error(e);
						throw e;
					}
				});
			}, null, this);

			// Should be suppressed
			var growl2 = new BASESOFT.widget.Growl({
				name: 'Buddy logged in',
				title: 'Bubu has logged in',
				message: 'Your friend Bubu has just logged.',
				lifetime: 2000 // should be shown for 2 seconds
			});
			this._growler.growl('Messenger', growl2);

			YAHOO.util.Assert.areEqual(this._growler.growls.length, 1);

			this.wait();

		} catch(e){
			console.error(e);
			throw e;
		}


	},


	testMultipleRegister: function () {
		try {
			this._growler.register({
				application: 'Messenger',
				allowed: ['Buddy logged in', 'Buddy logged out']
			});

			var growl1 = new BASESOFT.widget.Growl({
				name: 'Buddy logged in',
				title: 'Bubu has logged in',
				message: 'Your friend Bubu has just logged.'
			});

			// this should be ok, because it's allowed
			this._growler.growl('Messenger', growl1);

			var growl2 = new BASESOFT.widget.Growl({
				name: 'Buddy buzz',
				title: 'Bubu has buzzed you',
				message: 'Your friend Bubu is annoying you with buzzes.'
			});

			// this will fail if we try to growl it now, because 'Buddy buzz' is
			// not allowed, but we can register again and add that too
			this._growler.register({
				application: 'Messenger',
				allowed: ['Buddy buzz']
			});

			// now it should be ok, because it's allowed
			this._growler.growl('Messenger', growl2);

			var growl3 = new BASESOFT.widget.Growl({
				name: 'Buddy logged out',
				title: 'Bubu has logged out',
				message: 'Your friend Bubu has left the building.'
			});
			// this should still be ok, because it was registered
			this._growler.growl('Messenger', growl3);

			// kill all growls
			this._growler.killAll();


		} catch(e){
			console.error(e);
			throw e;
		}
	}

}));
YAHOO.register("test_grawl", null, {version: "1.0.0", build: "1"});
