/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 */

BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name : "Test Mail Compose",

	setUp : function() {
		this.div = document.createElement('div');
		YAHOO.util.Dom.get('sandbox').appendChild(this.div);

		this.div_form = document.createElement('div');
		this.div.appendChild(this.div_form);

		// get a broker for calling the method for resetting the db
		var test_rpc = "/basesoft.web.test/selenium/mail_manager/test_data/rpc/";
		this._rpc = new BASESOFT.util.JsonBroker(test_rpc, 60000);

		this.state = {
			'mail' : {
				'name' : 'Some email',
				'objectName' : 'MockMail',
				'dbID' : null,

				'locales' : {
					'nl_NL' : {
						'subject' : 'Mijn test onderworpen',
						'content' : 'Hallo <b>wereld !</b>'
					},
					'en_GB' : {
						'subject' : 'My test subject',
						'content' : 'Hello <b>world !</b>'
					}
				},

				'currentLocale' : {
					'locale' : 'nl_NL',
					'name' : 'Dutch'
				},
				'fallbackLocale' : {
					'locale' : 'en_GB',
					'name' : 'English'
				}
			}
		};

		this.loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
	},

	testMailCompose : function() {

		// login first
		this.loginrpc.call('login', ['test_account', 'test_password'], function(result) {
			this.resume(function() {// continue the test after login

				BASESOFT.env.account.id = result.id;
				// load the initial content from an existing state, then save it
				var mail_details = new BASESOFT.mail.MailPanel({
					mailType : 'MockMail'
				});

				mail_details.subscribe('onAvailable', function(e, mail_details) {
					this.resume(function() {

						var mail_name = 'Mock Mail ' + YAHOO.util.Dom.generateId();
						mail_details._nameInput.set('value', mail_name);
						mail_details._form.set('values', {
							name: mail_name,
							subject: 'Yo Yo',
							content: 'Hello World',
							mock_mail_plugin: 'Plugin Field Test',
							mail_from_text: 'mail from text'
						});

						BASESOFT.util.setTimeout(function() {
							this.resume(function() {


								mail_details.saveMail();
								BASESOFT.util.setTimeout(function() {
									this.resume(function() {
										mail_details.subscribe('onMailSave', function(e, args) {
											this.resume(function() {
												var md = args[0];
												var mn = args[1];
												var state = md.state.mail.locales[mail_details.state.mail.currentLocale.locale];
												console.warn('state', state);
												console.warn('state', md.state);

												//YAHOO.util.Assert.areEqual(md.state.mail.name, mn);
												YAHOO.util.Assert.areEqual(state.subject_wiki, 'Yo Yo');
												YAHOO.util.Assert.areEqual(state.content_wiki, 'Hello World');

												// focus some other field so we can check later on if the rte still opened fine
												// after something else got the focus
												md._subjectInput.input.focus();

												md.destroy();

												// now lets open it again and check if the editor still opens fine
												var new_md1 = new BASESOFT.mail.MailPanel({
													mailType : 'MockMail'
												});

												new_md1.subscribe('onAvailable', function(e, new_md1) {
													this.resume(function() {
														// editor was opened fine a second time.
														new_md1.destroy();
													});
												}, new_md1, this);
												this.wait(35000);
												// wait until new_md1 is available
											});
										}, [mail_details, mail_name], this);
										this.wait();
									});
								}, 5000, null, this);
							});
						}, 5000, null, this);
						this.wait();

					});
				}, mail_details, this);
				this.wait(35000);
				// wait for onavailable;
			});
		}, null, this);
		this.wait();
		// wait for login
	},
	testMailManager : function() {
		// login first
		this.loginrpc.call('login', ['test_account', 'test_password'], function(result) {
			this.resume(function() {// continue the test after login
				BASESOFT.env.account.id = result.id;
				// create the manager specifying that we want only MockMail (the mailType is optional)
				try {
					var mail_manager = new BASESOFT.mail.MailManagerPanel({
						mailType : ['MockMail']
					});

					this._garbage.push(mail_manager);

					// wait a bit before finishing the test so the mail types are loaded n stuff to avoud errors
					var _this = this;
					setTimeout(function() {
						_this.resume(function() {
							console.info('test done');
						});
					}, 2000);
					this.wait();

				} catch (e) {
					e.delay || console.error(e);
					throw e;
				}
			});
		}, null, this);
		this.wait();
		// wait for login
	}
}));
YAHOO.register("test_mail_manager", null, {
	version : "1.0.0",
	build : "1"
});
