 /**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 *
 * tests the mdx statement builder class
 */

BASESOFT.test.TestCases.push(new YAHOO.tool.TestCase({

	name: "MDX Statements",

	setUp : function () {
	},

	/**
	 */
	testMdx : function () {
		// TODO: create a test that builds a mdx statment using the MdxBuilder class and verify mdx string
	},


	/**
	 * This test implements the example described at :
	 * http://trac.office.basesoft.com/iqube/wiki/IQubeMDXDataDefinitionStatements/CreateCubeStatement
	 */
	testCreateCube: function() {

		try {

			// some shortcuts
			var mdx = BASESOFT.mdx;
			var FactTable = mdx.create_cube.FactTable;
			var JoinStatement = mdx.create_cube.JoinStatement;
			var Join = mdx.create_cube.Join;
			var Dimension = mdx.create_cube.Dimension;
			var DimensionDef = mdx.create_cube.DimensionDef;
			var Hierarchy = mdx.create_cube.Hierarchy;
			var HierarchyList = mdx.create_cube.HierarchyList;
			var Measure = mdx.create_cube.Measure;
			var MeasureDef = mdx.create_cube.MeasureDef;

			// CreateCube instance
			var o = new mdx.create_cube.Statement('HierarchicalOrder');

			// From FactTables
			o.addFrom(new FactTable('Order', 'O') );
			o.addFrom(new FactTable('Shop', 'S') );
			o.addFrom(new FactTable('ProductShop', 'PS') );
			o.addFrom(new FactTable('Product', 'P') );

			// From Joins
			o.addFrom(new JoinStatement([new Join('O', 'ShopID', 'S', 'ShopID')], 'S'));
			o.addFrom(new JoinStatement([
				new Join('O', 'ProductID', 'PS', 'ProductID'),
				new Join('S', 'ShopID', 'PS', 'ShopID')
			], 'PS'));
			o.addFrom(new JoinStatement([new Join('O', 'ProductID', 'P', 'ProductID')], 'P'));

			// With Dimensions
			o.addWith(new Dimension( new DimensionDef('S', 'CountryID', 'Country', 'C2', 'CountryID', 'Name'), 'Country') );
			o.addWith(new Dimension( new DimensionDef('S', 'ShopID', 'Shop', 'S', 'ShopID', 'Name'), 'Shop') );

			o.addWith(new HierarchyList([
				new Hierarchy(new DimensionDef('P', 'CategoryID', 'Category', 'C1', 'CategoryID', 'Name'), 'Category'),
				new Hierarchy(new DimensionDef('P', 'ProductID', 'Product', 'P', 'ProductID', 'Name'), 'Product')
			], 'Category'));

			o.addWith(new HierarchyList([
				new Hierarchy(new DimensionDef('O', 'CustomerID', 'Customer', 'C3', 'CustomerID', 'Name'), 'Customer'),
				new Hierarchy(new DimensionDef('C3', 'CountryID', 'Country', 'C2', 'CountryID', 'Name'), 'Country')
			], 'Customer'));

			// With Measures
			o.addWith(new Measure(new MeasureDef('O', 'Quantity', 'SUM', 'DOT', 'COMMA'), 'TotalQuantity'));
			o.addWith(new Measure(new MeasureDef('O', 'InvoicedPrice', 'SUM', 'DOT', 'COMMA'), 'TotalQuantity'));
			o.addWith(new Measure(new MeasureDef('O', 'Quantity', 'AVG', 'DOT', 'COMMA'), 'TotalQuantity'));
			o.addWith(new Measure(new MeasureDef('O', 'Weight', 'SUM', 'DOT', 'COMMA'), 'TotalQuantity'));


			// assert etc
			var querystr = o.querystring();
			var expected = 'CREATE CUBE [HierarchicalOrder] FROM (FACT_TABLE Order AS O , FACT_TABLE Shop AS S , FACT_TABLE ProductShop AS PS , FACT_TABLE Product AS P , JOIN (O.ShopID=S.ShopID) AS S , JOIN (O.ProductID=PS.ProductID, S.ShopID=PS.ShopID) AS PS , JOIN (O.ProductID=P.ProductID) AS P ) WITH (DIMENSION (S, CountryID, Country, C2, CountryID, Name) AS Country , DIMENSION (S, ShopID, Shop, S, ShopID, Name) AS Shop , DIMENSION (HIERARCHY (P, CategoryID, Category, C1, CategoryID, Name) AS Category , HIERARCHY (P, ProductID, Product, P, ProductID, Name) AS Product ) AS Category , DIMENSION (HIERARCHY (O, CustomerID, Customer, C3, CustomerID, Name) AS Customer , HIERARCHY (C3, CountryID, Country, C2, CountryID, Name) AS Country ) AS Customer , MEASURE (O, Quantity, SUM, DOT, COMMA) AS TotalQuantity , MEASURE (O, InvoicedPrice, SUM, DOT, COMMA) AS TotalQuantity , MEASURE (O, Quantity, AVG, DOT, COMMA) AS TotalQuantity , MEASURE (O, Weight, SUM, DOT, COMMA) AS TotalQuantity ) ';

			// spit out
			console.debug('create cube querystring: ', querystr);
			//alert(o.toString());

			YAHOO.util.Assert.areEqual(expected, querystr);


		} catch (e) {
			console.error(e);
			throw e;
		}

	}

}));

YAHOO.register("test_sqlbuilder", null, {version: "1.0.0", build: "1"});
