/**
 * Copyright (c) 2011 BaseSoft B.V.
 * See LICENSE for details.
 *
 * this test is used for checking if the user switch works as expected
 */

BASESOFT.test.TestCases.push(new YAHOO.tool.TestCase({

	name: "Test User Switch",
	assertClean: true,

	setUp : function () {
		// get a broker for calling the login methods
		this.loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
	},

	/**
	 * Test if we can succeed logging in as sysadmin, starting from test_account
	 */
	testHackUsername: function() {
		// login first
		this.loginrpc.call('login', ['test_account', 'test_password'], function(result){
			this.resume(function(){ // continue the test after login
				try {
					BASESOFT.env.account.id = result.id;

					this.loginrpc.call('login_as', ['sysadmin'], function(result){
						this.resume(function(){
							BASESOFT.env.account.id = result.id;

							// fail if the call succeedes
							YAHOO.util.Assert.fail();
						});
					}, function (failure){
						this.resume(function(){ // test is ok if we're not allowed to log in as intended
							YAHOO.util.Assert.isInstanceOf(BASESOFT.exceptions.PermissionDeniedException, failure);
						});
					}, this);
					this.wait(); // wait for login_as

				} catch (e) {
					e.delay || console.error(e);
					throw e;
				}
			});
		}, null, this);
		this.wait(); // wait for login
	},

	/**
	 * Test if we can call protected methods for logging in without checks
	 */
	testHackMethodCall: function() {
		// login first
		this.loginrpc.call('login', ['test_account', 'test_password'], function(result){
			this.resume(function(){ // continue the test after login
				try {
					BASESOFT.env.account.id = result.id;

					this.loginrpc.call('_login_as', ['sysadmin', false, false, true], function(result){
						this.resume(function(){
							BASESOFT.env.account.id = result.id;
							// fail if the call succeedes
							YAHOO.util.Assert.fail();
						});
					}, function (failure){
						this.resume(function(){ // test is ok if we're not allowed to log in as intended
							YAHOO.util.Assert.isInstanceOf(Error, failure);
						});
					}, this);
					this.wait(); // wait for login_as

				} catch (e) {
					e.delay || console.error(e);
					throw e;
				}
			});
		}, null, this);
		this.wait(); // wait for login
	},

	/**
	 * Test if we can call the login_as method with params for skipping checking
	 */
	testHackMethodParams: function() {
		// login first
		this.loginrpc.call('login', ['test_account', 'test_password'], function(){
			this.resume(function(){ // continue the test after login
				try {

					this.loginrpc.call('login_as', ['sysadmin', false, false, true], function(result){
						this.resume(function(){
							// fail if the call succeedes
							YAHOO.util.Assert.fail();
						});
					}, function (failure){
						this.resume(function(){ // test is ok if we're not allowed to use more then one param (username)
							YAHOO.util.Assert.isInstanceOf(Error, failure);
						});
					}, this);
					this.wait(); // wait for login_as

				} catch (e) {
					e.delay || console.error(e);
					throw e;
				}
			});
		}, null, this);
		this.wait(); // wait for login
	}

}));
YAHOO.register("test_user_switch", null, {version: "1.0.0", build: "1"});
