/**
 * Copyright (c) 2009 BaseSoft B.V.
 * See LICENSE for details.
 *
 * this test is used for checking methods BASESOFT.util.* of the iq_util module
 */

BASESOFT.test.TestCases.push(new YAHOO.tool.TestCase({

	name: "Test Util",
	assertClean: true,

	test_implode: function() {
		try {
			// test implode
			YAHOO.util.Assert.areEqual(BASESOFT.util.implode(['I', 'want', 'a', 'pie'], ' '), 'I want a pie');
			YAHOO.util.Assert.areEqual(BASESOFT.util.implode([], 'macaroni'), '');
		} catch(e){
			console.error(e);
			throw e;
		}
	},

	test_setNodeText: function() {
		var sandbox = YAHOO.util.Dom.get('sandbox');
		var d = document.createElement('div');
		sandbox.appendChild(d);

		BASESOFT.util.setNodeText(d, 'text1');
		YAHOO.util.Assert.areEqual(d.innerHTML, 'text1');

		BASESOFT.util.setNodeText(d, 'text2');
		YAHOO.util.Assert.areEqual(d.innerHTML, 'text2');
	},

	test_isEqual: function() {
		var v1 = null;
		var v2 = 1;
		var v3 = 'abc';
		var v4 = 'abcd';
		var v5 = '1';
		var v6 = 'abc'; // same as v3
		var v7 = [1, 2, 3];
		var v8 = [3, 2, 1];
		var v9 = [1, 2, 3, 4];
		var v10 = [1, 2, 3]; // same as v7
		var v11 = {a: 1, b: 2};
		var v12 = {b: 2, c: 3, d: 4};
		var v13 = {a: 1, b: 2, c: 3};
		var v14 = {a: 1, b: 2, e: {a1: 10, b1: 20}};
		var v15 = {b: 2, e: {a1: 10, b1: 20}, f: 6};
		var v16 = {e: {a1: 10, b1: 20}, a: 1, b: 2}; // same as v14
		var v17 = {b: 2, a: 1}; // same as v11

		YAHOO.util.Assert.isTrue(BASESOFT.util.isEqual(v3, v6));
		YAHOO.util.Assert.isTrue(BASESOFT.util.isEqual(v7, v10));
		YAHOO.util.Assert.isTrue(BASESOFT.util.isEqual(v14, v16));
		YAHOO.util.Assert.isTrue(BASESOFT.util.isEqual(v11, v17));

		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v1, v2));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v1, v2));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v1, v4));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v1, v5));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v3, v4));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v2, v5));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v7, v9));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v7, v8));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v11, v12));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v11, v13));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v11, v14));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v14, v15));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isEqual(v11, v15));
	},

	test_isFunction: function() {
		var obj1 = {};
		var obj2 = 42;
		var obj3 = 'test if isFunction works';
		var obj4 = function() {};

		YAHOO.util.Assert.isFalse(BASESOFT.util.isFunction(obj1));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isFunction(obj2));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isFunction(obj3));
		YAHOO.util.Assert.isTrue(BASESOFT.util.isFunction(obj4));
		YAHOO.util.Assert.isFalse(BASESOFT.util.isFunction(BASESOFT.widget.IconBar));
	},

	testClone: function() {

		var now = new Date();
		var x = {a: 1, b: {a: 1, b: 2, c: null, d: [1, 2, null]}, c: null, d: 'bier', some_date: now};
		var x1 = BASESOFT.util.clone(x);

		YAHOO.util.Assert.areNotSame(x, x1);
		YAHOO.util.Assert.isTrue(x1 instanceof Object);
		YAHOO.util.Assert.areEqual(x1.a, 1);
		YAHOO.util.Assert.areNotSame(x.b, x1.b);
		YAHOO.util.Assert.areEqual(x1.b.a, 1);
		YAHOO.util.Assert.areEqual(x1.b.b, 2);
		YAHOO.util.Assert.areEqual(x1.b.c, null);
		YAHOO.util.Assert.areNotSame(x.b.d, x1.b.d);
		YAHOO.util.Assert.areEqual(x1.b.d[0], 1);
		YAHOO.util.Assert.areEqual(x1.b.d[1], 2);
		YAHOO.util.Assert.areEqual(x1.b.d[2], null);
		YAHOO.util.Assert.areEqual(x1.c, null);
		YAHOO.util.Assert.areEqual(x1.d, 'bier');
		YAHOO.util.Assert.areEqual(x1.some_date.valueOf(), now.valueOf());

		var z = [1, 2, 3, {a: 1, b: 2}];
		var z1 = BASESOFT.util.clone(z);
		YAHOO.util.Assert.areNotSame(z, z1);
		YAHOO.util.Assert.isTrue(z1 instanceof Array);

		YAHOO.util.Assert.areEqual(z1[0], 1);
		YAHOO.util.Assert.areEqual(z1[1], 2);
		YAHOO.util.Assert.areEqual(z1[2], 3);
		YAHOO.util.Assert.areEqual(z1[3].a, 1);

		YAHOO.util.Assert.areEqual(1, BASESOFT.util.clone(1));
		YAHOO.util.Assert.areEqual('aap', BASESOFT.util.clone('aap'));

		// this used to fail
		var obj1 = BASESOFT.util.clone({
			a: ['aap'],
			b: ['bier'],
			c: ['kwak']
		});
		YAHOO.util.Assert.areEqual(1, obj1.a.length);
		YAHOO.util.Assert.areEqual(1, obj1.b.length);
		YAHOO.util.Assert.areEqual(1, obj1.c.length);

	},

	testDomMethods: function() {
		var div = document.createElement('div');

		// test getText without any children
		YAHOO.util.Assert.isNull(BASESOFT.util.Dom.getText(div));

		// set text on a newly created element
		BASESOFT.util.Dom.setText(div, 'test');
		YAHOO.util.Assert.areEqual('test', BASESOFT.util.Dom.getText(div));

		// set text when we already have a text node
		BASESOFT.util.Dom.setText(div, 'updated');
		YAHOO.util.Assert.areEqual('updated', BASESOFT.util.Dom.getText(div));

		// add some other elements and set text. should work
		var span = document.createElement('span');
		BASESOFT.util.Dom.setText(span, 'some span content');
		YAHOO.util.Assert.areEqual('some span content', BASESOFT.util.Dom.getText(span));
		div.appendChild(span);
		// reading the text should return the content of the text node
		YAHOO.util.Assert.areEqual('updated', BASESOFT.util.Dom.getText(div));
		YAHOO.util.Assert.areEqual(2, div.childNodes.length);
		// setting text will remove all nodes
		BASESOFT.util.Dom.setText(div, 'cleared');
		YAHOO.util.Assert.areEqual('cleared', BASESOFT.util.Dom.getText(div));
		YAHOO.util.Assert.areEqual(1, div.childNodes.length);

		// clear all elements and add a span. gettext should return null
		while (div.firstChild!==null)
			div.removeChild(div.firstChild);
		var span2 = document.createElement('span');
		span2.appendChild(document.createTextNode('some text'));
		div.appendChild(span2);
		YAHOO.util.Assert.isNull(BASESOFT.util.Dom.getText(div));
	},

	test_isValidHexColorCode: function() {
		var i;
		var goodLongValues = ['ff3344', '451f5a', '000000', 'ffffff'];
		var goodShortValues = ['fff', '123', '0ab'];
		var badLongValues = ['ff3344f', '451f5G', '1234567', '@$ffff'];
		var badShortValues = ['3344f', '1h3', '@$f'];

		for (i in goodLongValues)
			YAHOO.util.Assert.isTrue(BASESOFT.util.isValidHexColorCode(goodLongValues[i]));

		for (i in goodShortValues)
			YAHOO.util.Assert.isTrue(BASESOFT.util.isValidHexColorCode(goodShortValues[i]));

		for (i in goodShortValues)
			YAHOO.util.Assert.isTrue(BASESOFT.util.isValidHexColorCode(goodShortValues[i], true));

		for (i in badLongValues)
			YAHOO.util.Assert.isFalse(BASESOFT.util.isValidHexColorCode(badLongValues[i]));

		for (i in badShortValues)
			YAHOO.util.Assert.isFalse(BASESOFT.util.isValidHexColorCode(badShortValues[i]));

		for (i in badShortValues)
			YAHOO.util.Assert.isFalse(BASESOFT.util.isValidHexColorCode(badShortValues[i], true));
	},

	test_number_format: function() {

		// english
		BASESOFT.env.L10n.THOUSAND_SEPARATOR = ',';
		BASESOFT.env.L10n.DECIMAL_SEPARATOR = '.';

		var rv = BASESOFT.util.format_number(123123123.123, 2);
		YAHOO.util.Assert.areEqual(rv, '123,123,123.12');

		rv = BASESOFT.util.unformat_number('123,123,123.123');
		YAHOO.util.Assert.areEqual(rv, 123123123.123);


		// dutch
		BASESOFT.env.L10n.THOUSAND_SEPARATOR = '.';
		BASESOFT.env.L10n.DECIMAL_SEPARATOR = ',';

		rv = BASESOFT.util.format_number(123123123.123, 2);
		YAHOO.util.Assert.areEqual(rv, '123.123.123,12');

		rv = BASESOFT.util.unformat_number('123.123.123,123');
		YAHOO.util.Assert.areEqual(rv, 123123123.123);

		// italian
		BASESOFT.env.L10n.THOUSAND_SEPARATOR = '';
		BASESOFT.env.L10n.DECIMAL_SEPARATOR = ',';

		rv = BASESOFT.util.format_number(123123123.123, 2);
		YAHOO.util.Assert.areEqual(rv, '123123123,12');

		rv = BASESOFT.util.unformat_number('123123123,123');
		YAHOO.util.Assert.areEqual(rv, 123123123.123);

	},

	test_decimal: function() {

		BASESOFT.env.L10n.THOUSAND_SEPARATOR = '.';
		BASESOFT.env.L10n.DECIMAL_SEPARATOR = ',';

		var decimal;
		decimal = new BASESOFT.util.Decimal('100');
		YAHOO.util.Assert.areEqual('100,00', decimal.format(2));
		YAHOO.util.Assert.areEqual('100', decimal.format(0));

		decimal = new BASESOFT.util.Decimal('123.456');
		YAHOO.util.Assert.areEqual('123,45', decimal.format(2));

		decimal = new BASESOFT.util.Decimal('-100.123');
		YAHOO.util.Assert.areEqual('-100,12', decimal.format(2));

	},

	test_assert: function() {

		var assertRaises = function(cb, e_class) {
			try {
				cb();
			} catch (e) {
				if (e instanceof e_class)
					return;
			}
			YAHOO.util.Assert.fail('not raised');
		};

		// int
		BASESOFT.assert(1, 'int');
		assertRaises(function() { BASESOFT.assert('1', 'int'); }, BASESOFT.assert.AssertionException);

		// int zero
		BASESOFT.assert(0, 'int', {zero: true});
		assertRaises(function() { BASESOFT.assert(0, 'int'); }, BASESOFT.assert.AssertionException);

		// int negative
		BASESOFT.assert(-1, 'int', {negative: true});
		assertRaises(function() { BASESOFT.assert(-1, 'int'); }, BASESOFT.assert.AssertionException);

		// int null
		assertRaises(function() { BASESOFT.assert(null, 'int'); }, BASESOFT.assert.AssertionException);
		BASESOFT.assert(null, 'int', {'null_': true});

	}


}));
YAHOO.register("test_util", null, {version: "1.0.0", build: "1"});
