/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 *
 * This TestCase tests the Wizard widget.
 */

BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "Wizard Test",
	assertClean: true,

	setUp : function () {
		this.div_wizard = document.createElement('div');
		YAHOO.util.Dom.get('sandbox').appendChild(this.div_wizard);
		YAHOO.util.Dom.get('sandbox').appendChild(document.createElement('br'));
	},

	/**
	 */
	testJsWizard : function () {
		/**
		 * build a dynamic wizard
		 */

		try {
			var myWizard = new BASESOFT.widget.Wizard(this.div_wizard, {'title': 'My Javascript Wizard'});

			var page1 = new BASESOFT.widget.WizardPage({
				content: '<p>Page One Content</p>',
				title: 'Title Page 1',
				id: 'page1'
			});
			myWizard.addPage(page1);

			var page2 = new BASESOFT.widget.WizardPage({
				content: '<p>Page Two Content</p>',
				title: 'Title Page 2',
				id: 'page2'
			});
			myWizard.addPage( page2 );

			var page3 = new BASESOFT.widget.WizardPage({
				content: '<p>Page Three Content</p>',
				title: 'Title Page 3',
				id: 'page3'
			});
			myWizard.addPage( page3 );

			this._garbage.push(myWizard);

		} catch (e) {
			console.debug('Exception', e);
			throw e;
		}

		var _resume_count = 0;

		myWizard.addListener('activePageChange', function(type, args, me){

			this.resume(function() {

				_resume_count++;
				console.debug('in resume', _resume_count);

				// first resume-count menas we are on page 2.
				if (_resume_count == 1) {

					console.debug('wizard page 2', myWizard.get('activePage'));

					YAHOO.util.Assert.areSame(myWizard.get('activePage'), page2);
					YAHOO.util.Assert.areEqual(myWizard._pageTitle.innerHTML, 'Title Page 2');

					// assert key and title
					YAHOO.util.Assert.areEqual('Title Page 2', myWizard.get('activePage').get('title'));
					YAHOO.util.Assert.areEqual('page2',	myWizard.get('activePage').get('id'));

					// clicky on next button.. for some reason this needs to be in a timeout
					setTimeout(function() { YAHOO.util.UserAction.click(myWizard.btnNext.get('element')); }, 10);

					// wait for the test to resumes
					this.wait();
				}

				// resume 2 is last page
				else if (_resume_count == 2) {
					YAHOO.util.Assert.areSame(myWizard.get('activePage'), page3);
					YAHOO.util.Assert.areEqual(myWizard._pageTitle.innerHTML, 'Title Page 3');

					// see if finished is shown and next is hidden
					YAHOO.util.Assert.areNotEqual('none', myWizard.btnFinish.getStyle('display'));
					YAHOO.util.Assert.areEqual('none', myWizard.btnNext.getStyle('display'));
				}

			});

		}, [], this);

		// verify the disabled back button
		YAHOO.util.Assert.areEqual(myWizard.btnBack.get('disabled'), true);

		// assert wizard titles
		YAHOO.util.Assert.areEqual(myWizard._wizTitle.innerHTML, 'My Javascript Wizard');
		YAHOO.util.Assert.areEqual(myWizard._pageTitle.innerHTML, 'Title Page 1');

		// clicky on next button.. for some reason this needs to be in a timeout
		setTimeout(function() { YAHOO.util.UserAction.click(myWizard.btnNext.get('element')); }, 10);

		// wait for the test to resumes
		this.wait();
	},

	testHtmlWizard: function() {

		this.div_wizard.innerHTML =
		'<div id="html_wizard" title="My HTML Wizard">' +
		    '<div class="bs-wizard-content">' +
			'<div id="page1" title="page 1"><p>Page One Content</p></div>' +
			'<div id="page2" title="page 2"><p>Page Two Content</p></div>' +
			'<div id="page3" title="page 3"><p>Page Three Content</p></div>' +
		    '</div>' +
		'</div>';

		var myWizard = new BASESOFT.widget.Wizard('html_wizard');
		myWizard.addListener('activePageChange', function(type, args, me){
			console.debug('page change');
		}, [], this);

		this._garbage.push(myWizard);
	}

}));


YAHOO.register("test_wizard", null, {version: "1.0.0", build: "1"});
