# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.mailing.models import Mail, Mailing
from basesoft.web.core.models import PrivilegeObject
from basesoft.web.contrib.order.models import Order, OrderLine

class MockMail(Mail):
    """
    Model for a mock mail, used for testing
    @DynamicAttrs
    """

    NAME_I18N = "Mock Mail"

    JS_MODULE = 'test_mock_mail_plugin'

    def __repr__(self):
        return '<MockMail id=%s>' % self.id

    @staticmethod
    def replacement_fields():
        x = Mail.replacement_fields()
        x.update({
            'test.field.mockmail': 'Test field only for MockMail',
            'test.field.link': 'Test field linky'
        })
        return x

    def _replace_field(self, person, field_name, context):

        if field_name == 'test.field.link':
            link = self._base_url(context) + '#m=test'
            return '<a href="%s">%s</a>' % (link, link)
        else:
            return Mail._replace_field(self, person, field_name, context)



class MockMailing(Mailing):
    """
    Mock mailing model
    @DynamicAttrs
    """

class MockPrivilegeObject(PrivilegeObject):
    """
    @DynamicAttrs
    """

    def get_group_object(self):
        return (MockPrivilegeObjectGroup, 'mock_object')

    def __repr__(self):
        return '<MockPrivilegeObject id="%s">' % self.id

class MockPrivilegeObjectGroup(object):
    """
    @DynamicAttrs
    """
    pass

class MockOrder(Order):
    """
    @DynamicAttrs
    """
    permissions = ('view_mock_orders', )
    js_details_module = 'iq_order_details'

    def get_order_controller(self):
        from basesoft.web.test.unit.test_order_controller import MockOrderController
        return MockOrderController

    @staticmethod
    def get_simulate_order_line_types():
        return [
            ('Mock line', 'MockOrderLine'),
        ]


class MockOrder2(Order):
    """
    @DynamicAttrs
    """
    permissions = ('view_mock_orders', )
    js_details_module = 'iq_order_details'

    def get_order_controller(self):
        from basesoft.web.test.unit.test_order_controller import MockOrder2Controller
        return MockOrder2Controller


class MockOrderLine(OrderLine):
    """
    @DynamicAttrs
    """
    tax_class_code = 'MOCK_ORDER_TAX_CLASS'
    gl_account_code = 'MOCK_ORDER_GL_ACCOUNT'

    MAX_LINE_QUANTITY = 1000000

    def __repr__(self):
        return '<MockOrderLine id=%s index=%s>' % (self.id, self.index)
