# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher

from basesoft.web.contrib.account.controllers import AccountController
from basesoft.web.contrib.account.models import Account, GroupAccount
from basesoft.web.contrib.contact.models import Contact, ContactList
from basesoft.web.contrib.license.models import Permission
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.contrib.contact.models import Person
from basesoft.web.contrib.i18n.models import Locale

from basesoft.web.contrib.file.controllers import UploadFileController
from django.core.files.uploadedfile import SimpleUploadedFile

def contact_manager_rpc_view(request):
    """
    rpc view for contact manager yui test
    """

    def reset_db():
            """
            This is just for calling in the yui unit test to reset the data in the db
            and create a test account with correspondent permissions
            """

            test_domain = "test_domain_cm"
            test_license = "test_license_cm"
            test_group = "test_group_cm"
            test_account = "test_account_cm"
            test_password = "test_password_cm"

            session = request.Session()
            ac = AccountController(session)
            lc = LicenseController(session)

            # remove any old contacts from the db
            contacts = session.query(Contact).all()
            cls = session.query(ContactList).all()
            for c in contacts:
                session.delete(c)
            for cl in cls:
                session.delete(cl)
            session.flush()

            # create/use a test account to which we can assign the correspondent
            # permissions for testing the contact manager
            account = session.query(Account).filter(Account.name==test_account)
            if account.count() > 0:
                account = account.one()
                request.session['user_id'] = account.id
                request.session['username'] = account.name
                return


            domain = lc.save_or_update_domain(None, test_domain)
            contact = Person()
            license = lc.save_or_update_license(None,
                                                test_license,
                                                domain,
                                                None,
                                                contact,
                                                None,
                                                None)

            group = lc.save_or_update_group(None, test_group, license)

            account = Account()
            account.name = test_account
            account.password = ac.hash_password(test_password)
            account.person = Person()
            account.locale = session.query(Locale).filter(Locale.id==1).one()
            session.add(account)

            # make the link between account and group.
            ga = GroupAccount()
            ga.is_default = 1
            ga.active = True
            ga.account = account
            ga.group = group
            session.add(ga)

            # add correspondent permissions to this group
            group.permissions = session.query(
                Permission).filter(
                Permission.name.in_(['view_contact',
                                    'create_contact',
                                    'update_contact',
                                    'delete_contact',
                                    'view_contact_list',
                                    'create_contact_list',
                                    'update_contact_list',
                                    'delete_contact_list',
                                    'link_to_contact_list',
                                    'unlink_from_contact_list',
                                    'import_contacts'])).all()
            session.add(group)
            session.flush()

            #set this user as logged in
            request.session['user_id'] = account.id
            request.session['username'] = account.name
            ac = AccountController(session, None, account)
            request.session['config'] = ac.config.get_configs()

    def create_file():
        """
        Called from yui contacts import test to create a csv test file to import
        File will be /tmp/test_import_contacts.csv.
        This method will also call reset_db.
        """
        #reset_db()

        filename = "test_import_contacts.csv";
        data = "%s\n%s\n%s\n%s" \
                % ("first name,middle name,last name,email address,phone number,country",
                   "John,,Smith,john@mail-test.basesoft.com,0040740123456,romaniaaaa", # wrong country
                   "jerry,,seinfeld,jerry@mail-test.basesoft.com,,United States of America",
                   "anne,marie,franklin,anne@email,0012465889,netherlands") # wrong email
        file = {'filedata': SimpleUploadedFile(filename, data)}
        uc = UploadFileController(request.Session(), request.session['user_id'])
        tmp_file = uc.save_temp_files(file)[0]
        return tmp_file.file_hash

    def create_contact():
        """
        Creates a contact (used for the privilege distribution test
        """
        session = request.Session()
        from basesoft.web.core.util.controllers import PermissionController
        pc = PermissionController(session, account_id=request.session['user_id'])
        contact = Contact()
        contact.group = pc.get_default_group()
        contact.group_privileges = Contact.all_privileges
        contact.license_privileges = 0
        contact.domain_privileges = 0

        # add the contact
        session.add(contact)
        session.flush()
        return contact.id

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(reset_db, 'reset_db')
    dispatcher.register_function(create_file, 'create_file')
    dispatcher.register_function(create_contact, 'create_contact')
    return dispatcher.handle_request(request)