# -*- coding: utf-8 -*-

"""
Defines the a set of JSON-RPC views to test against.

Only really simple methods should go here.. If you want to make a JSON view you probably
have to make one in /jsonrpc/myview
"""

import datetime
import time
import random

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher

from basesoft.web.core.exceptions import PublicUserException, PublicSystemException, LoginRequiredException,\
    SecurityException

from basesoft.web.contrib.account.decorators import login_required


def dispatch_request(request, dispatcher):
    """Wrapper for checking post and dispatches the request"""
    return dispatcher.handle_request(request)


def simple_view(request):
    """Makes a dispatcher and registers simple mdx functions"""

    def echo(str):
        """simple echo function.. returns what it gets"""
        return str;

    def echo_unicode(text):
        assert isinstance(text, unicode)
        return text

    def multiply(a, b):
        """multiply's a*x"""
        return a*b

    def raise_public_user_exception():
        """
        raises a public user exception
        """
        raise PublicUserException('public user exception raised with utf-8: È')

    def raise_public_system_exception():
        """
        raises a public user exception
        """
        raise PublicSystemException('public system exception raised')

    def raise_private_exception():
        """
        raises a public user exception
        """
        raise Exception('private exception')

    def date_test(x1, x2):

        assert isinstance(x1, datetime.datetime)
        assert isinstance(x2, datetime.datetime)

        assert x1 == datetime.datetime(2009, 11, 5, 0, 3, 18)
        assert x2 == datetime.datetime(2009, 4, 5, 0, 3, 18)
        x3 = datetime.date(2009, 5, 5)

        return [x1,x2,x3]

    def date_test_db(x1):

        from django.utils import simplejson
        from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder
        from basesoft.web.core.jsonrpc.dispatcher import json_object_hook

        serialised  = simplejson.dumps(x1, ensure_ascii=False, cls=JsonRpcJSONEncoder)
        return simplejson.loads(serialised, object_hook=json_object_hook)

    def get_decimal():
        """
        SQLAlchemy makes numeric fields (Decimals, Number) into python decimals. We need to be
        able to transport them to JavaScript via the dispatcher. This is a one way conversion.
        """
        import decimal
        return decimal.Decimal("2.54")

    def get_decimal_removed_exponent(zero=True):
        import decimal
        if zero:
            return decimal.Decimal('0E-10')
        else:
            return decimal.Decimal('5E+3')

    def multiple_test(i):
        """
        used by the rpc/multiple test.. this adds a bit of random sleeping so one calls returns before another
        """
        time.sleep(random.random() * 2)
        return i

    def login_required_exception():
        """
        simply raises a LoginRequiredException
        """
        raise LoginRequiredException('login required!')

    def security_exception():
        raise SecurityException('No permission')

    def sleep(secs):
        time.sleep(secs)

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(multiply, 'multiply')
    dispatcher.register_function(echo, 'echo')
    dispatcher.register_function(echo_unicode, 'echo_unicode')
    dispatcher.register_function(raise_public_user_exception, 'raise_public_user_exception')
    dispatcher.register_function(raise_public_system_exception, 'raise_public_system_exception')
    dispatcher.register_function(raise_private_exception, 'raise_private_exception')
    dispatcher.register_function(date_test, 'date_test')
    dispatcher.register_function(get_decimal, 'get_decimal')
    dispatcher.register_function(get_decimal_removed_exponent, 'get_decimal_removed_exponent')
    dispatcher.register_function(multiple_test, 'multiple_test')
    dispatcher.register_function(login_required_exception, 'login_required_exception')
    dispatcher.register_function(security_exception, 'security_exception')
    dispatcher.register_function(date_test_db, 'date_test_db')
    dispatcher.register_function(sleep, 'sleep')

    return dispatch_request(request, dispatcher)

@login_required
def login_required_view(request):

    def echo(str):
        return str;

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(echo, 'echo')

    return dispatch_request(request, dispatcher)
