# -*- coding: utf-8 -*-

'''
Created on Aug 22, 2014

@author: ian
'''

import os
from os.path import abspath, dirname
from basesoft.web.contrib.contact.controllers import ContactController, ImportContactController, SimpleContactController
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase, XPath
from twisted.trial.unittest import SkipTest

# Selenium test for importing of contacts from a file.
class TestCase(SeleniumTestCase):

    def setUp(self):

        super(TestCase, self).setUp()

        self.connect_db()

        #create an account so we can check permission with it
        self.account = self._create_active_account()

        # create an instance of the controller. It is used in all test methods.
        self.controller = ContactController(self.session, account_id = self.account.id)
        self.lc = LicenseController(self.session)
        self.group = self.account.groups[0].group


    def test_import_contacts(self):
        """
        Test import contacts from external file and choose their fields to process

        trial basesoft.web.test.selenium.test_contacts.TestCase.test_import_contacts

        """
        # create an account and set privileges to import contacts
        account = self._create_active_account()
        self._attach_permissions_to_group(self.group,
            'view_dashboard', 'save_dashboard',
            'view_contact', 'create_contact', 'update_contact', 'import_contacts', 'view_contact_list',
            'view_file', 'create_file', 'link_to_contact_list', 'create_contact_list'
            )

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading(stable_time=2)

        # navigate to import wizard
        self.click_iconbar('icon-import-contacts', hover='icon-contact-lists-manager')
        self.wait_for_panel('contacts-import-wizard')

        # upload file to wizard
        file_path = os.path.join(abspath(dirname(__file__)), 'related', 'contacts.csv')

        self.fill_form(panel_name='contacts-import-wizard', values={
            'file_input': ('upload', file_path)
        })
        self.wait_for_loading()

        # FIXME: when new uploader is implemented we don't need any use for the OverlayManager, see ticket #3817
        self._driver.execute_script('BASESOFT.env.OverlayManager.overlays[0].wizard.btnNext.set("disabled", false);')

        css = self.wait_for_panel_or_form(panel_name='contacts-import-wizard')
        input_css = "%s input[name='file_input']" % css

        input_el = self._driver.find_element_by_css_selector(input_css)

        _hash = input_el.get_attribute('value')

        self._driver.execute_script('BASESOFT.env.OverlayManager.overlays[0]._uploader.set("value","%s");' % _hash)
        self._driver.execute_script('BASESOFT.env.OverlayManager.overlays[0].file_hash = {file_hashes: ["%s"]};' % _hash)

        self.click_button(panel_name='contacts-import-wizard', text=u'Next »')
        self.wait_for_loading()

        # Field mapping the data from file
        self.fill_form(panel_name='contacts-import-wizard', values={
            'separator_select': ('select', 'Comma'),
            'import_row1': ('checkbox', True),
            'unique_fields': ('fancy-select', ('First name', 'Last name'))
        })
        self.wait_for_loading()

        # Should give growl notification error because there is no field mapping done
        # and Unique import fields are not being used
        self.click_button(panel_name='contacts-import-wizard', text=u'Next »')
        self.assert_growl_errors_present()

        # Select only one Unique field out of two which should raise an error with growl notification
        self.fill_form(panel_name='contacts-import-wizard', values={
            'select_row_option_0': ('select', ('First name'))
        })
        self.click_button(panel_name='contacts-import-wizard', text=u'Next »')
        self.assert_growl_errors_present()

        # Map all fields to go the next screen
        self.fill_form(panel_name='contacts-import-wizard', values={
            'select_row_option_0': ('select', ('First name')),
            'select_row_option_1': ('select', ('Last name')),
            'select_row_option_2': ('select', ('Title')),
            'select_row_option_3': ('select', ('Gender')),
            'select_row_option_4': ('select', ('Birth date')),
            'select_row_option_5': ('select', ('Email')),
            'select_row_option_6': ('select', ('Street')),
            'select_row_option_7': ('select', ('City')),
            'select_row_option_8': ('select', ('Country name')),
            'select_row_option_9': ('select', ('Country code (iso_3166_a2)')),
        })
        self.click_button(panel_name='contacts-import-wizard', text=u'Next »')
        #self.wait_for_loading()

        # Check if 2 contacts will be imported
        self.assertTrue("div//*[@class='overview-subtext']' and text()='2 contacts to be imported.'")

        # Import contacts
        self.click_button(panel_name='contacts-import-wizard', text=u'Import')

        # Save contacts in contact list wih name 'contactlist'
        self.fill_form(form_name='save-to-contact-list', values={
            'cb_input': 'contactlist'
        })
        self.click_button(text=u'Add contacts')

        # Test if data is written to the database
        contacts = self.controller.get_contacts()
        self.assertEqual(len(contacts), 2)

        fn = ['harry', 'terence trend']
        ln = ['van schaamaap','darby']

        # Test results
        i = 0
        for x in contacts:
            self.assertTrue(x.person.first_name in fn)
            fn.remove(x.person.first_name)
            self.assertTrue(x.person.last_name in ln)
            ln.remove(x.person.last_name)
            i += 1
