# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase
from basesoft.web.contrib.i18n.controllers import I18nController

class TestCase(SeleniumTestCase):

    def setUp(self):
        super(TestCase, self).setUp()
        self.connect_db()

    def test_i18n(self):
        """
        tests that messages are correctly compiled
        """
        account = self._create_active_account()
        group = account.groups[0].group
        self._attach_permissions_to_group(group, 'view_dashboard', 'save_dashboard')

        self.login(account.name, 'test_password')

        # set mode to compose
        self.expand_layout_pane(0)
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        # drag widgy
        self.drag_item_from_palette('Test Widget')
        self.wait_for_loading()

        # assert english texts

        el = self._driver.find_element_by_css_selector(".test-widget-js-gettext")
        self.assertEqual(el.text, 'test Hello World!')

        el = self._driver.find_element_by_css_selector(".test-widget-es6-gettext")
        self.assertEqual(el.text, 'test es6 Hello World!')

        # save the dashboard
        self.click_palette('Save the dashboard')
        self.click_button(panel_name='dashboard-settings-panel', text='OK')
        self.wait_for_loading()

        # open user preferences

        self.click_iconbar('icon-settings')
        self.wait_for_loading()

        self.click_fieldset(panel_name='account-preferences', title='Language & formatting settings')

        self.fill_form(panel_name='account-preferences', values={
            'basesoft.web.contrib.locale' : ('select', 'Dutch')
        })
        self.click_button(panel_name='account-preferences', text='Save and refresh')
        self.wait_for_loading(stable_time=5) # wait for refresh

        # assert dutch texts

        el = self._driver.find_element_by_css_selector(".test-widget-js-gettext")
        self.assertEqual(el.text, 'test Hallo Wereld!')

        el = self._driver.find_element_by_css_selector(".test-widget-es6-gettext")
        self.assertEqual(el.text, 'test es6 Hallo Wereld!')





    def test_domain_country_manager(self):
        # set up new user with access to the domain country manager
        account = self._create_active_account()
        group = account.groups[0].group
        self._attach_permissions_to_group(group, 'view_dashboard', 'manage_domain_country_mapping')

        # login
        self.login(account.name, 'test_password')

        # open the manager
        self.click_iconbar('icon-domain-country-manager')
        self.wait_for_loading()

        # create a new mapping -> country name
        self.click_cq(panel_name='domain-country-manager', row_index=0, col_name='country', extra_css='')

        target = self._driver.find_element_by_css_selector('div.local_name_editor > form > input')
        target.clear()
        target.send_keys(unicode('My local Afghanistan'))

        self.click_cq(panel_name='domain-country-manager', row_index=0, col_name='original', extra_css='')
        self.wait_for_loading()

        c = I18nController(self.sa_session, account_id=account.id)
        c, l = c.get_domain_countries(1).pop() # afghanistan is 1
        self.assertTrue(l is not None)
        self.assertEqual(l.name, 'My local Afghanistan')

    def test_domain_language_alias_manager(self):
        # set up new user with access to the domain country manager
        account = self._create_active_account()
        group = account.groups[0].group
        self._attach_permissions_to_group(group, 'view_dashboard', 'manage_domain_language_aliases')

        # login
        self.login(account.name, 'test_password')

        # open the manager
        self.click_iconbar('icon-domain-language-manager')
        self.wait_for_loading()

        self.click_cq(panel_name='domain-language-manager', row_index=0, col_name='original')
        self.wait_for_loading()

        self.click_button(panel_name='domain-language-alias-details', text='Add an additional alias')
        self.fill_form(panel_name='domain-language-alias-details', rowcontrol_row_index=0, values={'alias': 'My local Abkhazian'})

        self.click_button(panel_name='domain-language-alias-details', text='Save')
        self.wait_for_loading()

        c = I18nController(self.sa_session, account_id=account.id)
        l = c.get_language_by_alias('My local Abkhazian')
        self.assertTrue(l is not None)
        self.assertEqual(l.name, 'Abkhazian')

