# -*- coding: utf-8 -*-

'''
Created on Oct 8, 2010

@author: martijn
'''

import random

from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase, XPath
from twisted.trial.unittest import SkipTest

class TestCase(SeleniumTestCase):
    '''
    tests altering and creation of a license
    '''

    def setUp(self):

        super(TestCase, self).setUp()

        self.login('sysadmin', 'RaKa4Rucru8r')
        self.click_iconbar('icon-license-manager')
        self.wait_for_panel('license-manager-panel')


    def test_new_license(self):
        """
        create a new license tru the wizard
        """

        rand_id = str(random.random())[2:]

        # open wizard
        self.click_menu(panel_name='license-manager-panel', text='New license wizard')
        self.wait_for_loading()

        # first page
        self.fill_form(panel_name='license-wizard-panel', values={
            'domain_name':'My Domain ' + rand_id,
            'license_name': 'My License ' + rand_id,
            'expiration_date': '10/15/2010 00:00'
        })
        self.click_button(panel_name='license-wizard-panel', text=u'Next »')
        self.wait_for_loading()
        self.assert_no_growl_errors()

        # second page
        self.fill_form(panel_name='license-wizard-panel', values={
            'first_name': 'Johnny', 'last_name': 'Depp',
            'title': 'Pirate', 'gender': ('select', 'Male'),
            'email': 'john@depp.com',
            'phone': '001234567', 'fax': '0012345678', 'mobile_phone': '0013456',
            'department': 'deppartment ;)', 'position': 'actor',
        })
        self.fill_form(panel_name='license-wizard-panel', form_name='contact-address-form', values={
            'street': 'deppstreet 25', 'postcode': '1000AA', 'state': 'DeppState',
            'city': 'DeppCity', 'country': ('select', 'Netherlands')
        })
        self.wait_for_loading()
        self.click_button(panel_name='license-wizard-panel', text=u'Next »')
        self.wait_for_loading()
        self.assert_no_growl_errors()
        self.wait_for_loading()

        # third page
        self.fill_form(panel_name='license-wizard-panel', values={
            'name': 'Depp Company', 'vat_no': 'HR24595836665',
            'website': 'http://depp.com',
            'number': 'bank 1234' + rand_id, 'bic': 'bic1234' + rand_id, 'iban': 'iban1234' + rand_id
        })
        self.fill_form(panel_name='license-wizard-panel', form_name='company-address-form', values={
            'street': 'deppstreet 25', 'postcode': '1000AA', 'state': 'DeppState',
            'city': 'DeppCity', 'country': ('select', 'Netherlands')
        })
        self.wait_for_loading()
        self.click_button(panel_name='license-wizard-panel', text=u'Next »')

        self.wait_for_loading(stable_time=2)

        # fourth page
        self.fill_form(panel_name='license-wizard-panel', values={
            'permissions': ('fancy-select', ('Scheduler manager access', 'Create surveys'))
        })
        self.click_button(panel_name='license-wizard-panel', text=u'Next »')
        self.wait_for_loading()

        # fith page
        #self.click_button(panel_name='license-wizard-panel', text='Add an additional row')
        self.fill_form(panel_name='license-wizard-panel', rowcontrol_row_index=0, values={
            'usage_option_id': ('select', 'Maximum number of surveys'),
            'credits': 5, 'price_per_unit': 10
        })
        self.click_button(panel_name='license-wizard-panel', text=u'Next »')
        self.wait_for_loading()

        self.click_button(panel_name='license-wizard-panel', text='Finish')
        self.wait_for_panel_closed(panel_name='license-wizard-panel')

        # filter the cq to the newly created license
        self.filter_cq(panel_name='license-manager-panel', filters=[
            ('License', 'Equal', 'My License ' + rand_id),
            ('Type', 'Contains', ('select', 'License'))
        ])

        # verify collins
        self.assert_cq(panel_name='license-manager-panel', row_index=0, data={
            'domain_name': 'My Domain ' + rand_id,
            'license_name': 'My License ' + rand_id,
            'license_polymorphic_type': 'License',
            'contact': 'Johnny Depp',
            'company_name': 'Depp Company',
        });


    def test_change_license(self):

        # wait for CQ loaded with 3 rows
        self.wait_for_cq(panel_name='license-manager-panel')

        self.filter_cq(panel_name='license-manager-panel', filters=[
            ('License', 'Equal', 'BaseSoft Solutions B.V.'),
        ])
        self.wait_for_loading(stable_time=2)

        # assert first row in the CQ
        self.assert_cq(panel_name='license-manager-panel', row_index=0, data={
            'domain_name': 'Resellers',
            'license_name': 'BaseSoft Solutions B.V.',
            'license_polymorphic_type': 'License',
            'contact': 'Paul de Graaf',
            'company_name': 'BaseSoft Solutions B.V.',
            'license_id': "View details",
            'license_groups': "View groups",
        })

        # click on the 'view details' in the third row (column license_id has a link in it)
        self.click_cq(panel_name='license-manager-panel', row_index=0, col_name='license_id')
        self.wait_for_panel('license-details-panel')
        self.wait_for_loading(stable_time=2)

        # open fieldset and select all permissions for sysadmin
        self.click_fieldset(panel_name='license-details-panel', title='License permissions')
        self.click_button(panel_name='license-details-panel', text='Select all')
        self.fill_form(panel_name='license-details-panel', values={
            'append_to_group': ('select', 'System Administrators'),
        })

        # save and wait for close
        self.click_button(panel_name='license-details-panel', text='Save')
        self.wait_for_loading()
        self.wait_for_panel_closed(panel_name='license-details-panel')

        # wait for cq reloaded
        self.wait_for_cq(panel_name='license-manager-panel', rows=1)

    def test_master_data_change_approval(self):
        raise SkipTest('FIXME: fill_forms should be able to handle two forms with same name')
        self.connect_db()

        test_account = self._create_active_account()
        group = test_account.groups[0].group

        self._attach_permissions_to_group(group, 'view_dashboard', 'view_license', 'update_license_masterdata')

        self.logout()


        self.login(test_account.name, 'test_password')
        self.wait_for_loading(stable_time=2)


        self.click_iconbar('icon-master-data', hover='icon-license-manager')

        forms_data = {
            'company_form': {
                'name': 'Scheepsbouw B.V.'
            },
            'person_form': {
                'preferred_locale': ('select', 'English'),
            }
        }

        # raising skiptest for line that follows
        self.fill_forms(panel_name='master-data-panel', forms_data=forms_data)

        self.click_button(panel_name='master-data-panel', text='Save')
        self.assert_no_growl_errors()

        self.logout()

        # login as sysadmin and assert that the status for master data
        self.login('sysadmin', 'RaKa4Rucru8r')
        self.click_iconbar('icon-license-manager')
        self.wait_for_panel('license-manager-panel')
        self.wait_for_loading()

        self.filter_cq(panel_name='license-manager-panel', filters=[
            ('Domain', 'Equal', group.license.domain.name),
        ])
        self.assert_cq(panel_name='license-manager-panel', row_index=0, data={
            'master_data': XPath("//a[text()='Outstanding request']"),
        })

        self.click_cq(panel_name='license-manager-panel', row_index=0, col_name='master_data')
        self.wait_for_loading()

        self.assertTrue(self.selenium.is_element_present("//div[@class='bd master-data-comparison']/table//td[@class='value-diff' and text()='Scheepsbouw B.V.']"))

        self.click_button(panel_name='master-data-compare-panel', text='Approve changes')
        self.wait_for_loading()
        self.assertEqual(self.growl_count(), 1)

        # we verify that the master data change request is no longer listed
        self.assert_cq(panel_name='license-manager-panel', row_index=0, data={
            'master_data': XPath("//a[text()='']"),
        })

        # check existence of records in database by using the controller

    def test_license_config(self):
        self.connect_db()

        test_account = self._create_active_account()
        group = test_account.groups[0].group

        self._attach_permissions_to_group(group, 'view_dashboard', 'view_license', 'manage_mock_configs', 'manage_license_configs')

        self.logout()


        self.login(test_account.name, 'test_password')
        self.wait_for_loading(stable_time=2)

        self.click_iconbar('icon-settings')
        self.wait_for_loading()

        self.click_fieldset(panel_name='account-preferences', title='License configs for %s' % group.license.name)
        self.click_fieldset(panel_name='account-preferences', title='Mock configs')

        self.fill_form(panel_name='account-preferences', values={
            'basesoft.test.mock.config.2' : '99'
        })
        self.click_button(panel_name='account-preferences', text='Save')
        self.wait_for_loading()

        self.fill_form(panel_name='account-preferences', values={
            'basesoft.test.mock.config.2' : '200'
        })

        self.click_button(panel_name='account-preferences', text='Save')
        self.wait_for_loading(assert_no_errors=False)
        self.assert_growl_errors_present()