# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.account.mailing import MailController
from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase


class TestCase(SeleniumTestCase):
    def setUp(self):
        super(TestCase, self).setUp()
        self.connect_db()

    def test_create_mail(self):
        account = self._create_active_account()
        group = account.groups[0].group
        self._attach_permissions_to_group(group, 'view_dashboard', 'save_dashboard', 'view_mail', 'create_mail', 'view_license',
                                          'update_mail', 'create_account_mail', 'view_account_mail', 'update_account_mail', 'view_domain')

        self.login(account.name, 'test_password')

        self.expand_layout_pane(0)

        # set mode to compose
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        self.click_palette('Mailing', palette_index=2)

        self.drag_item_from_palette('Mail overview', block_index=0, column_index=0)
        self.wait_for_loading()

        self.click_palette('New', palette_index=1)
        self.click_palette('Account activation mail', palette_index=1)
        self.wait_for_loading()

        self.fill_form(panel_name='mail-panel', values={
             'content': ('textarea', "My ''Wiki'' content")
        })
        self.wait_for_loading()

        self.fill_form(panel_name='mail-panel', values={
            'recipients': ('fancy-select', (group.license.name, )),
            'name': 'This is a test mail',
            'mail_from_text': 'My VROM VROOOOOM!',
            'subject': 'Test mail subject',
        })

        self.click_button(panel_name='mail-panel', text='Save and close')
        self.wait_for_loading()

        mc = MailController(self.sa_session, account_id=account.id)
        mails = mc.get_mails()
        self.assertEqual(len(mails), 1)

        mail = mails[0]
        self.assertEqual(len(mail.locales), 1)

        locale = mail.locales[0]
        self.assertEqual(locale.subject, 'Test mail subject')
        self.assertEqual(locale.content, "My ''Wiki'' content")

        self.click_cq(cq_name='basesoft.web.contrib.mailing.MailCustomQuery', row_index=0, col_name='mail_name')
        self.wait_for_loading()

        self.assert_form(panel_name='mail-panel', values={
            'name': 'This is a test mail',
            'mail_from_text': 'My VROM VROOOOOM!',
            'subject': 'Test mail subject',
        })

        self.fill_form(panel_name='mail-panel', values={
            'mail_from_text': 'My updated VROM text!',
            'subject': 'Test mail subject (updated)',
        })

        self.click_button(panel_name='mail-panel', text='Save and close')
        self.wait_for_loading()

        self.sa_session.expire(mail)
        self.assertEqual(mail.mail_from_text, 'My updated VROM text!')
        self.sa_session.expire(locale)
        self.assertEqual(locale.subject, 'Test mail subject (updated)')

        self.click_cq(cq_name='basesoft.web.contrib.mailing.MailCustomQuery', row_index=0, col_name='translate')
        self.wait_for_loading()

        self.fill_form(form_name='mail-translate-form-0', values={
            'locale': ('select', 'value=1') # english
        })
        self.wait_for_loading()

        self.fill_form(form_name='mail-translate-form-1', values={
            'locale': ('select', 'value=2') # dutch
        })
        self.wait_for_loading()
        self.fill_form(form_name='mail-translate-form-1', values={
            'content': ('textarea', "my translation ''wiki''.")
        })
        self.wait_for_loading()
        self.fill_form(form_name='mail-translate-form-1', values={
            'subject': 'Neerlands onderwerp'
        })

        self.click_button(panel_name='mail-translate-panel', text='Save and close')
        self.wait_for_loading()

        self.sa_session.expire(mail)
        self.assertEqual(len(mail.locales), 2)

