# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.notification.controllers import NotificationController
from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase


class TestCase(SeleniumTestCase):
    def setUp(self):
        super(TestCase, self).setUp()
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group
        self.controller = NotificationController(self.session, account_id=self.account.id)
        self._attach_permissions_to_group(self.group, 'view_notification', 'view_dashboard', 'create_notification')

    def test_create_notification(self):
        self.login(self.account.name, 'test_password')

        # open notifications manager
        self.click_iconbar('icon-notifications')
        self.wait_for_loading()

        # open notification panel
        self.click_menu(panel_name='notifications_manager', text='Create new')
        self.wait_for_loading()

        # add subject and select group
        self.fill_form(panel_name='notification_panel', values={'group': ('select', self.group.name)})
        self.fill_form(panel_name='notification_panel', values={'subject': ('input_text', 'Notification subject')})

        # save notification
        self.click_button(panel_name='notification_panel', text='Send')
        self.wait_for_loading()

        # check that cq has only 1 row
        self.assert_cq(panel_name='notifications_manager', row_index=0, data={'status': ''})
        self.assertEqual(self.cq_rows_length('notifications_manager'), 1)

        # change the status
        self.click_cq(panel_name='notifications_manager', row_index=0, col_name='status', extra_css=' div', is_datatable=True)
        self.fill_form(form_name='combobox_ceditor', values={'statuses': ('input_text', 'test status')})
        self.click_cq(panel_name='notifications_manager', row_index=0, col_name='license', extra_css=' div', is_datatable=True)

        self.assert_cq(panel_name='notifications_manager', row_index=0, data={'status': 'test status'})

    def test_read_notification(self):
        self._create_notification()

        self.login(self.account.name, 'test_password')

        # open notifications manager
        self.click_iconbar('icon-notifications')
        self.wait_for_loading()

        # open notification action
        self.click_cq('notifications_manager', row_index=0, col_name='callback')

    def _create_notification(self):

        subject = 'NOTIFICATION TEST'
        text = 'This is a test for notifications'
        callback = {
            'js_module': 'iqreport_manager'
        }

        notification = self.controller.set_notification(subject, text, callback=callback)
        self.controller.create_notification(notification, [self.group])