# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.i18n.models import Country

from django.shortcuts import render_to_response
from django.template import RequestContext
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.contrib.branding.models import Theme
from sqlalchemy.orm.exc import NoResultFound


def testrunner_view(request, yui_test_data):

    theme_module_name = None

    if request.COOKIES.has_key("yui_testrunner"):

        # cookie looks like: theme_module=', 'body_classes=', 'runtests_onload=', 'cleanup=0', 'test_jsonrpc=1
        keyvalues = {}
        for piece in request.COOKIES['yui_testrunner'].split('&'):
            k, v = piece.split('=')
            keyvalues[k] = v

        if keyvalues.has_key('theme_module') and keyvalues['theme_module'] != '':
            theme_module_name = keyvalues['theme_module']

    if theme_module_name:
        request.session['theme_module_name'] = theme_module_name

    theme_css = ''
    current_theme_module = ''

    sa_session = request.Session()
    try:
        theme = sa_session.query(Theme).filter(Theme.module_name == theme_module_name).one()
        theme_css = theme.css_class
        current_theme_module = theme.module_name
    except NoResultFound:
        pass

    available_themes = [ (x.module_name, x.name) for x in sa_session.query(Theme).all()]

    return render_to_response('selenium/yui/testrunner.html', {
        'theme_css': theme_css,
        'current_theme_module': current_theme_module,
        'available_themes': available_themes,
        'jsi18n': ['basesoft.web.test'],
        'base_requires': ['iq_testrunner'],
        'yui_test_data': yui_test_data
    }, context_instance=RequestContext(request))

def testrunner(request):

    return testrunner_view(request, [
            "test_jsonrpc",
            "test_wizard",
            "test_notify",
            "test_fancyselect",
            "test_editor_manager",
            "test_file",
            "test_history",
            "test_dashboard",
            "test_customdatatable",
            "test_registration_dialog",
            "test_custom_query",
            "test_contact_manager",
            "test_contact_forms",
            "test_dialogs",
            "test_growl",
            "test_form_manager",
            "test_license",
            "test_mail_manager",
            "test_menu",
            "test_csv_parser",
            "test_iconbar",
            "test_dynamictable",
            "test_infobox",
            "test_container",
            "test_L10n_settings",
            "test_user_switch",
            "test_util",
            "test_chart_legend",
            "test_rte",
            "test_notification",
            "test_design",
            "test_es6",
        ])



def account(request):

    def create_account_and_login(*permissions):
        """
        """
        from basesoft.web.test.unit.iqportal_unittest import IQPortalTestCase
        c = IQPortalTestCase()
        c.session = request.Session()
        account = c._create_active_account()

        c._attach_permissions_to_group(account.default_group, *permissions)

        request.session['user_id'] = account.id
        request.session['username'] = account.name
        request.session['config'] = {'basesoft.web.contrib.locale': request.LANGUAGE_CODE}

        return {
            'id': account.id,
            'username': account.name
        }

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(create_account_and_login, 'create_account_and_login')
    return dispatcher.handle_request(request)

def replacement_fields(request):

    def get_replacement_fields():
        return [{
            'key' : 'producer',
            'label': 'Producer',
            'submenu': [{
                'key': 'name',
                'label': 'Name'
            }, {
                'key': 'vat_no',
                'label': 'VAT Number'
            }]
        }]

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(get_replacement_fields, 'get_replacement_fields')
    return dispatcher.handle_request(request)

def fancy_select_options(request):

    def get_fancy_select_options(user_id, selected, filter_text=None):

        assert user_id == 1

        options = [
            {'label': 'One', 'value': 1},
            {'label': 'Two', 'value': 2},
            {'label': 'Three', 'value': 3}
        ]
        rv = []
        if filter_text in ['', None]:
            return options

        for i in xrange(0, len(options)):
            if filter_text in options[i]['label']:
                rv.append(options[i])
            elif unicode(options[i]['value']) in selected:
                rv.append(options[i])

        return rv

    def get_grouped_fancy_options(user_id, selected, filter_text=None):

        assert user_id == 1

        grouped_options = {
            'group 1': [
                        {'label': 'One', 'value': 1}],
            'group 2': [
                {'label': 'Two', 'value': 2},
                {'label': 'Three', 'value': 3}
        ]}

        rv = {}
        if filter_text in ['', None]:
            return grouped_options

        for key in grouped_options:
            rv[key] = []
            for j in xrange(0, len(grouped_options[key])):
                if filter_text in grouped_options[key][j]['label']:
                    rv[key].append(grouped_options[key][j])
                elif unicode(grouped_options[key][j]['value']) in selected:
                    rv[key].append(grouped_options[key][j])
        return rv

    def rpc_loading(user_id, selected, filter_text):
        import time
        time.sleep(4)
        return get_fancy_select_options(user_id, selected, filter_text)


    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(get_fancy_select_options, 'get_fancy_select_options')
    dispatcher.register_function(get_grouped_fancy_options, 'get_grouped_fancy_options')
    dispatcher.register_function(rpc_loading, 'rpc_loading')
    return dispatcher.handle_request(request)

def json_datasource_country(request):

    def json_datasource_country(filter):
        assert isinstance(filter, dict)
        value = filter.get('value', None)
        filter = filter.get('filter', '')

        rv = []

        sa_session = request.Session()

        if value is None:
            countries = sa_session.query(Country).filter(
                Country.name.like('%'+filter+'%')
            ).limit(5)
        else:
            countries = sa_session.query(Country).filter(
                Country.id == value
            ).limit(1)

        return [ {'value': c.id, 'label': c.name} for c in countries ]

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(json_datasource_country, 'json_datasource_country')
    return dispatcher.handle_request(request)
