# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
import subprocess, os, signal, time, commands
from exceptions import OSError

import re
import sqlalchemy

import basesoft # for path

from django.utils import simplejson
from django.core.serializers.json import DjangoJSONEncoder

from twisted.trial import unittest
from django.conf import settings

from basesoft.web.core.management.commands.iqresetdb import Command as ResetDBCommand
from basesoft.web.core.util.db import Session

from basesoft.web.contrib.license.models import License, Role

from basesoft import logging
from basesoft.web.contrib.file.models import FileTmp
logger = logging.getLogger('basesoft.web.test.unit')

class IQPortalTestCase(unittest.TestCase):
    """
    add some new methods to the python's TestCase
    """
    COUNTER = 0
    iqube = None

    __django_pid = None
    __scheduler_pid = None

    def reset_db(self):
        """
        Resets db for all IQPortal installed apps
        This uses the iqresetdb command from basesoft.web.core.management.commands
        """
        ResetDBCommand().handle()

    def connect_db(self):
        """
        Creates an engine based on settings and connects to the database
        """
        self.session = self.sa_session = Session()

    def connect_iqube(self):
        """
        Creates a connection to the IQube server.
        """
        pass

    def reset_iqube_project(self, project, user=None, password=None):
        """
        Resets an IQube Project, if the project does not yet exist, create it.
        """
        pass

    def assertDeepEqual(self, o1, o2):
        """
        Asserts that two objects are equals.
        """

        try:
            self.assertEqual(simplejson.dumps(o1, cls=DjangoJSONEncoder), simplejson.dumps(o2, cls=DjangoJSONEncoder))
        except Exception, e:
            print
            print
            print 'assertDeepEqual: a: %s' % o1
            print
            print 'assertDeepEqual: b: %s' % o2
            print

            raise e

    def _uniquestring(self, tag):
        import time,random
        from hashlib import sha256 as sha
        t1 = time.time()
        time.sleep( random.random() )
        t2 = time.time()

        sid = tag + '_' + sha(str(tag + str(t1 +t2))).hexdigest()[:32]

        return unicode(sid)

    def _create_license(self, last_name='TestLastName', type=License, domain=None):
        """
        This is used from multiple tests and can be used to add a fake license
        to the database.
        @param domain: if specified, it will create the license for this domain, otherwise create a new one
        """
        assert self.session is not None
        from basesoft.web.contrib.license.controllers import LicenseController
        from basesoft.web.contrib.account.controllers import AccountController
        from basesoft.web.contrib.contact.models import Person, Company

        # get BaseSoft Solutions.
        IQPortalTestCase.COUNTER += 1

        # creating licenses requires some perms, so use sysadmin for that (?)
        ac = AccountController(self.session)
        account = ac.get_account_by('sysadmin')

        lc = LicenseController(self.session, None, account.id)
        is_default = False
        if domain is None:
            domain = lc.save_or_update_domain(None, '%s' % self._uniquestring('test_domain')[0:39])
            is_default = True
        self.domain = domain

        contact = Person()
        contact.last_name = unicode(last_name)
        contact.email = u'%s@mail-test.basesoft.com' % self._uniquestring('random')

        company = Company()
        company.name = self._uniquestring('company')[0:39]
        company.vat_no = 'MyVatNoDontMatter'
        company.contact_person = contact

        license = lc.save_or_update_license(None,
                                            'test_license_%s' % IQPortalTestCase.COUNTER,
                                            domain,
                                            type,
                                            contact,
                                            company,
                                            None, is_default)
        return license

    def _set_license_usage(self, license, usage_option_name, credits, price_per_unit=None):
        """
        Sets a usage limit for specified license and usage option.
        @param license: License
        @param usage_option_name: string - usage option; if it does not exist, it will be created
                                (inserted in tblUsageOption)
        @param credits: int
        @param price_per_unit: decimal
        """
        from basesoft.web.contrib.license.models import UsageOption, LicenseUsage

        q = self.session.query(UsageOption).filter(UsageOption.name==usage_option_name)
        if q.count() == 0:
            uo = UsageOption(usage_option_name, u"")
            self.session.add(uo)
            self.session.flush()
        else:
            uo = q.one()

        lu = LicenseUsage()
        lu.license = license
        lu.usage_option = uo
        lu.credits = credits
        lu.price_per_unit = price_per_unit
        self.session.add(lu)
        self.session.flush()

    def _create_active_account(self, license=None, name=None, group_name=None, person=None, group=None, domain=None):
        """
        This creates an account and automatically actives it without email
        links etc. It is attached to a group, license and domain.
        It can be called multiple times to create multiple accounts.

        The optional license can be supplied to attach the account to an
        already existing license. A new group will be created non-the-less.
        """
        assert self.session is not None
        from basesoft.web.contrib.license.controllers import LicenseController
        from basesoft.web.contrib.i18n.models import Locale
        from basesoft.web.contrib.account.controllers import AccountController
        from basesoft.web.contrib.contact.models import Person
        from basesoft.web.contrib.account.models import Account, GroupAccount

        if license is None:
            license = self._create_license(domain)
        self.license = license

        # creating licenses requires some perms, so use sysadmin for that (?)
        ac = AccountController(self.session)
        account = ac.get_account_by('sysadmin')
        lc = LicenseController(self.session, None, account.id)
        ac = AccountController(self.session)

        if group is None:
            if group_name:
                group = lc.save_or_update_group(None, group_name, license, is_default=False)
            else:
                group = lc.save_or_update_group(None, 'test_group', license, is_default=True)
        else:
            assert group.license == license

        account = Account()
        if name is not None:
            account.name = self._uniquestring(name)[0:39]
        else:
            account.name = self._uniquestring('test_account')[0:39]
        account.password = ac.hash_password('test_password')

        logger.info('Created test_account: %s' % account.name)

        # if we have a person provided add that to the contact also
        if person is not None:
            account.person = person
        else:
            account.person = Person()

        if account.person.email is None:
            account.person.email = 'testing@mail.clusternet'

        account.locale = self.session.query(Locale).filter(Locale.id==1).one()
        self.session.add(account)

        # make the link between account and group.
        ga = GroupAccount()
        ga.is_default = 1
        ga.active = True
        ga.account = account
        ga.group = group

        self.session.add(ga)
        self.session.flush()
        return account

    def _add_address_and_bank_account_to_company(self, company):
        """
        This can be used to add address and bank account information to a company
        Add two addresses and two bank accounts
        """
        from basesoft.web.contrib.contact.models import Address, BankAccount, \
                                                               CompanyAddress, CompanyBankAccount
        from basesoft.web.contrib.i18n import models as i18n_models

        netherlands = self.session.query(i18n_models.Country).filter(
                                    i18n_models.Country.iso_3166_a2=='NL').one()

        address1 = Address()
        address1.street = u'Some Street'
        address1.postcode = 'Postcode 1234'
        address1.city = u'Some City'
        address1.country = netherlands
        ca1 = CompanyAddress()
        ca1.address = address1
        ca1.company = company
        ca1.is_invoice = True
        self.session.add(ca1)

        address2 = Address()
        address2.street = u'Some Other Street'
        address2.postcode = 'Postcode 5678'
        address2.city = u'Some Other City'
        address2.country = netherlands
        ca2 = CompanyAddress()
        ca2.address = address2
        ca2.company = company
        ca2.is_delivery = True
        self.session.add(ca2)

        ba1 = BankAccount()
        ba1.number = self._uniquestring('1234567890')[0:39]
        ba1.iban = self._uniquestring('IBAN A')
        ba1.bic = 'BIC A'
        cba1 = CompanyBankAccount()
        cba1.is_default = True
        cba1.bank_account = ba1
        cba1.company = company
        self.session.add(cba1)

        ba2 = BankAccount()
        ba2.number = self._uniquestring('0987654321')[0:39]
        ba2.iban = self._uniquestring('IBAN B')
        ba2.bic = 'BIC B'
        cba2 = CompanyBankAccount()
        cba2.is_default = False
        cba2.bank_account = ba2
        cba2.company = company
        self.session.add(cba2)

        self.session.flush()

    def _attach_permissions_from_role(self, group, role_name):
        """
        This is a protected method that can be used to grant permissions to a group based on a supplied role.
        """
        role = self.session.query(Role).filter(Role.name==role_name).one()
        for perm in role.permissions:
            group.license.permissions.append(perm)
            group.permissions.append(perm)
        self.session.add(group)
        self.session.flush()

    def _attach_permissions_to_group(self, group, *permissions):
        """
        This is a protected method that can be used in tests to give a group
        certain permissions. Can be easily combined with _create_active_account
        and makes the testing of Controller heaps easier.
        """
        from basesoft.web.contrib.license.models import Permission
        for perm in self.session.query(Permission).filter(Permission.name.in_(permissions)).all():
            group.license.permissions.append(perm)
            group.permissions.append(perm)
        self.session.add(group)
        self.session.flush()

    def _revoke_permissions_from_group(self, group, *permissions):
        """
        This is a protected method that can be used in tests to revoke permissions
        from a certain group. Can be easily combined with _create_active_account
        and makes the testing of Controller heaps easier.
        """
        from basesoft.web.contrib.license.models import Permission
        new = []
        revoke = self.session.query(Permission).filter(Permission.name.in_(permissions)).all()
        for perm in group.license.permissions:
            if perm not in revoke:
                new.append(perm)
        group.license.permissions = new
        group.permissions = new
        self.session.add(group)
        self.session.flush()

    def tearDown(self):

        if hasattr(self, 'session'):
            #logger.debug('Closing SQLAlchemy scoped session')
            self.session.remove()
        if self.iqube is not None:
            #logger.debug('Closing IQube connection')
            self.iqube.logout()
            self.iqube = None

        self._cleanup()

        unittest.TestCase.tearDown(self)

    def assertMdx(self, mdx1, mdx2):
        """
        removes all newlines and whitespace after them
        """

        mdx1 = re.sub("\n\s*", " ", mdx1).strip()
        mdx2 = re.sub("\n\s*", " ", mdx2).strip()

        mdx1 = re.sub(" , ", ", ", mdx1)
        mdx2 = re.sub(" , ", ", ", mdx2)

        self.assertEquals(mdx1, mdx2)

    def _create_uploaded_file(self, file_name, file_contents):

        file_hash = self._uniquestring('f')[-8:]
        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, file_hash)

        if not os.path.exists(settings.TMP_FILE_REPOSITORY):
            os.mkdir(settings.TMP_FILE_REPOSITORY)

        file = open(file_path, 'w')
        file.write(file_contents)
        file.close()

        f = FileTmp()
        f.file_name = file_name
        f.file_hash = file_hash
        self.sa_session.add(f)
        self.sa_session.flush()

        return f

    def _start_django(self):

        if not settings.SELENIUM['start_django']:
            return

        # this command is used for checking if the server is up. It's empty if the server is not running.
        if os.uname()[0] == 'Darwin':
            cmd_is_up = '/usr/sbin/netstat -na | grep LISTEN | grep %s' % settings.SELENIUM_DJANGO_ADDR.split(':')[1]
        else:
            cmd_is_up = "netstat -ln | grep %s" % settings.SELENIUM_DJANGO_ADDR

        # before starting the server check if there is not already a server running
        if commands.getoutput(cmd_is_up) != '':

            # give it another try after 5 seconds
            time.sleep(5)
            if commands.getoutput(cmd_is_up) != '':
                raise Exception('There is already a django server running on %s' % settings.SELENIUM_DJANGO_ADDR)

        python_bin = 'python'
        if os.environ.has_key('PYTHON_BIN'):
            python_bin = os.environ['PYTHON_BIN']

        # prepare django server command
        django_args = [
            python_bin,
            settings.SELENIUM_DJANGO_ADMIN,
            'runserver', '--noreload',
            settings.SELENIUM_DJANGO_ADDR
        ]

        # selenium python path can be overriden
        pythonpath = os.environ['PYTHONPATH']
        if settings.SELENIUM_PYTHONPATH is not None:
            pythonpath = settings.SELENIUM_PYTHONPATH

        # django server environment
        django_env = {
            'PATH': os.environ['PATH'],
            'DJANGO_SETTINGS_MODULE': os.environ['DJANGO_SETTINGS_MODULE'],
            #'PYTHONPATH': basesoft.__path__[0] + '/../' + ':' + django.__path__[0] + '/../'
            'PYTHONPATH': pythonpath,
        }

        # print 'django_args: %s' % django_args
        # print 'django_env: %s' % django_env

        logfile = os.path.join(settings.WRITABLE_DIRECTORY, 'django.log')

        # spawn the process
        try:
            p = subprocess.Popen(args=django_args, bufsize=0, env=django_env, stdout=open(logfile, 'a'), stderr=subprocess.STDOUT)
        except Exception, e:
            raise Exception('Failed to start process: %s: %s' % (django_args, e))

        logger.debug('server running pid %s' % p.pid)
        self.__django_pid = p.pid

        #print 'cmd_is_up: %s' % cmd_is_up

        # wait and retry for the django server to be available
        django_up = False
        for i in range(5):
            if commands.getoutput(cmd_is_up) != '':
                logger.debug('%s: django server is up!' % self)
                django_up = True
                break
            logger.debug('%s: django server not up yet.. i: %s' % (self, i))
            time.sleep(1)

        if not django_up:
            raise Exception('Couldnt verify that django server is up.. check %s' % logfile)

        # enable this for heavy debugging
        if False:
            logger.debug("IQSelenium: Selenium django server:")
            logger.debug("IQSelenium:  - os environment: %s" % os.environ)
            logger.debug("IQSelenium:  - pwd: %s" % os.getcwd())
            logger.debug("IQSelenium:  - django_args: %s" % django_args)
            logger.debug("IQSelenium:  - django_env: %s" % django_env)
            logger.debug("IQSelenium:  - pid %s" % p.pid)

            logger.debug("IQSelenium: Selenium selenium connection:")
            logger.debug("IQSelenium:  - host: %s:%s" % (settings.SELENIUM['host'], settings.SELENIUM['port']) )
            logger.debug("IQSelenium:  - browserStartCommand: %s" % settings.SELENIUM['browserStartCommand'] )
            logger.debug("IQSelenium:  - url: %s" % settings.SELENIUM['browserURL'] )

    def _start_scheduler(self):

        if not settings.SELENIUM['start_scheduler']:
            return

        # this command is used for checking if the server is up. It's empty if the server is not running.
        if os.uname()[0] == 'Darwin':
            cmd_is_up = '/usr/sbin/netstat -na | grep LISTEN | grep %s' % settings.SCHEDULER_SERVER_PORT
        else:
            cmd_is_up = "netstat -ln | grep :%s" % settings.SCHEDULER_SERVER_PORT

        # before starting the server check if there is not already a server running
        if commands.getoutput(cmd_is_up) != '':

            # give it another try after 5 seconds
            time.sleep(5)
            if commands.getoutput(cmd_is_up) != '':
                raise Exception('There is already a scheduler server running on %s' % settings.SCHEDULER_SERVER_PORT)

        # program arguments
        args = [
            settings.SCHEDULER_TWISTD, '-noy',
            basesoft.__path__[0] + '/../related/bin/scheduler_server.tac'
        ]

        # selenium python path can be overriden
        pythonpath = os.environ['PYTHONPATH']
        if settings.SELENIUM_PYTHONPATH is not None:
            pythonpath = settings.SELENIUM_PYTHONPATH

        # server environment
        env = {
            'DJANGO_SETTINGS_MODULE': os.environ['DJANGO_SETTINGS_MODULE'],

            # set pythonpath to the parent of the path of the 'basesoft' module
            # this should also have iqportal and eventually maybe other packages
            #'PYTHONPATH': basesoft.__path__[0] + '/../' + ':' + basesoft.__path__[0] + '/../related/libs/'
            'PYTHONPATH': pythonpath,
        }

        # spawn the process
        logfile = '/tmp/scheduler_server.%s.out' % settings.SCHEDULER_SERVER_PORT
        try:
            p = subprocess.Popen(args=args, bufsize=0, env=env, stdout=open(logfile, 'a'), stderr=subprocess.STDOUT)
        except Exception, e:
            raise Exception('Failed to start process: %s: %s' % (args, e))

        logger.debug('scheduler server running pid %s' % p.pid)
        self.__scheduler_pid = p.pid

        # wait and retry for the django server to be available
        server_up = False
        for i in range(5):
            if commands.getoutput(cmd_is_up) != '':
                logger.debug('%s: scheduler server is up!' % self)
                server_up = True
                break
            logger.debug('%s: scheduler server not up yet.. i: %s' % (self, i))
            time.sleep(1)

        # the command seems to be a bit fecked in osx so ignore errors there for now
        if not server_up:
            raise Exception('Couldnt verify that scheduler server is up.. check %s' % logfile)

        # enable this for heavy debugging
        """
        logger.debug("IQSelenium: Selenium django server:")
        logger.debug("IQSelenium:  - os environment: %s" % os.environ)
        logger.debug("IQSelenium:  - pwd: %s" % os.getcwd())
        logger.debug("IQSelenium:  - django_args: %s" % django_args)
        logger.debug("IQSelenium:  - django_env: %s" % django_env)
        logger.debug("IQSelenium:  - pid %s" % p.pid)

        logger.debug("IQSelenium: Selenium selenium connection:")
        logger.debug("IQSelenium:  - host: %s:%s" % (settings.SELENIUM['host'], settings.SELENIUM['port']) )
        logger.debug("IQSelenium:  - browserStartCommand: %s" % settings.SELENIUM['browserStartCommand'] )
        logger.debug("IQSelenium:  - url: %s" % settings.SELENIUM['browserURL'] )
        """

    def _cleanup(self):
        try:
            self.__stop_django()
            self.__stop_scheduler()
            logger.debug('%s: __cleanup: django server killed' % self)
        except Exception, e:
            logger.error('%s: __cleanup: swallowed exception' % e.message)

    def __stop_django(self):

        if not settings.SELENIUM['start_django']:
            return

        if self.__django_pid:
            try:
                os.kill(self.__django_pid, signal.SIGTERM)
                self.__django_pid = None

            except OSError, e:
                logger.error('%s: __stop_django: Exception while killing pid %s: %s' % self, (self.__django_pid, e))
                pass


    def __stop_scheduler(self):

        if not settings.SELENIUM['start_scheduler']:
            return

        if self.__scheduler_pid:
            try:
                os.kill(self.__scheduler_pid, signal.SIGTERM)
                self.__scheduler_pid = None

            except OSError, e:
                logger.error('%s: __stop_scheduler: Exception while killing pid %s: %s' % self, (self.__scheduler_pid, e))
                pass

