# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.test.unit import iqportal_unittest
import basesoft.web.core.L10n_settings as ls

class LocalizationSettingsTestCase(iqportal_unittest.IQPortalTestCase):
    """Test the localization settings"""

    def test_locale_retrieval(self):
        """
        Test and see which locale will be used from the localization settings based on what locale is required and what
        we have defined
        """

        # a dictionary with locales that we want to test as keys, and the class that we should receive as values
        locales_to_test = {
            'en': ls.en,
            'EN': ls.en,
            'en_gb': ls.en_gb,
            'en-gb': ls.en_gb,
            'en-GB': ls.en_gb,
            'en_GB': ls.en_gb,
            'en-ww': ls.en, # we don't have this, so it should fallback to en
            'nl': ls.nl,
            'nl-ro': ls.nl,
            'inexistent': ls.default
        }

        for locale in locales_to_test.keys():
            # get the class for the language we're looking for
            cls = ls.get(locale)
            self.assertTrue(isinstance(cls, locales_to_test[locale]), "expected class %s for locale %s, but we got class %s" %
                              (locales_to_test[locale], locale, cls))
