'''
Created on May 27, 2009

@author: martijn
'''
from twisted.trial.unittest import SkipTest
from django.conf import settings

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.util.parse import CodeSquasher
import re
from basesoft.web.contrib.branding.models import Theme

class CodeSquashTestCase(iqportal_unittest.IQPortalTestCase):
    """
    Tests the CodeSquasher class that squashes JS/CSS code into one big filetria
    """

    def test_dependencies(self):

        base_modules = ['i18n_basesoft_web_contrib_yui', 'yahoo', 'dom', 'event']

        # simple, just require a few and make sure there are no duplicate dependencies
        c = CodeSquasher(['yahoo', 'dom', 'event'], [])
        self.assertEqual(c.module_list('js'), base_modules)

        # button requires a bit more
        c = CodeSquasher(['button'], [])
        self.assertEqual(c.module_list('js'), base_modules + ['element', 'button'])

        # button and dom, order and no duplicates is important
        c = CodeSquasher(['button', 'dom'], [])
        self.assertEqual(c.module_list('js'), base_modules + [ 'element', 'button'])

        # dialog has a css file, this needs to be retrieved separate
        c = CodeSquasher(['iq_dialog'], base_modules)
        self.assertEqual(c.module_list('js'), [
            'element',
            'container',
            'i18n_basesoft_web_core',
            'iq_util',
            'i18n_basesoft_web_contrib_jquery',
            'jquery_ui_core',
            'jquery_ui_widget',
            'jquery_ui_mouse',
            'jquery_ui_resizable',
            'iq_container',
            'iq_dialog'])
        self.assertEqual(c.module_list('css'), ['skin-container', 'jquery_resizable_css'])
        self.assertTrue('iq_dialog_scss' in c.module_list('scss'))

        # now lets introduce the 'exclude'..
        # 'dom' depends on 'yahoo' but we pretend its already laoded so dont want it back
        c = CodeSquasher(['dom'], ['yahoo', 'i18n_basesoft_web_contrib_yui'])
        self.assertEqual(c.module_list('js'), ['dom'])

        # and a more complex example with iq_dialog but we already have some files
        c = CodeSquasher(['iq_dialog'], base_modules + [ 'iq_dialog_scss', 'i18n_basesoft_web_core'])
        self.assertEqual(c.module_list('js'), ['element',
         'container',
         'iq_util',
         'i18n_basesoft_web_contrib_jquery',
         'jquery_ui_core',
         'jquery_ui_widget',
         'jquery_ui_mouse',
         'jquery_ui_resizable',
         'iq_container',
         'iq_dialog'])
        self.assertEqual(c.module_list('css'), ['skin-container', 'jquery_resizable_css'])

        # test wierdness.. ask for a module but its already loaded.
        c = CodeSquasher(['dom'], ['yahoo', 'dom', 'i18n_basesoft_web_contrib_yui'])
        self.assertEqual(c.module_list('js'), [])

        # dashboard should include compose_css
        c = CodeSquasher(['iq_dashboard'], [])
        self.assertTrue('iq_compose_scss' in c.module_list('scss'))


    def test_load_js(self):

        if settings.YUI_DEBUG_FILES:
            raise SkipTest('this test only works with yui debug off')

        c = CodeSquasher(['yahoo', 'dom', 'event'], [])
        code = c.js_code_as_string()

        # assert that the YAHOO
        self.assertTrue(code.find('YAHOO.namespace=function(){') > 0)
        self.assertTrue(code.find('YAHOO.util.Event=function(){') > 0)

    def test_skins(self):
        c = CodeSquasher(['layout'], [])
        self.assertEqual(c.module_list('css'), ['skin-layout'])

        c = CodeSquasher(['container'], [])
        self.assertEqual(c.module_list('css'), ['skin-container'])

    def test_fix_css_img_path(self):
        """
        test if the skin paths get replaced in the
        """
        c = CodeSquasher(['editor'], [])
        css = c.css_code_as_string()

        # all urls must be absolute
        found = re.findall('url\([^/)]', css)
        self.assertTrue(len(found) == 0, 'not absolute urls: %s' % found)

        # there shall not be any relative urls.. this needs better checking
        self.assertTrue(len(re.findall('\.\./', css)) == 0 )

        # there shall not  be duplicate closed url tags
        self.assertTrue( len(re.findall('url\([^\)]*\)\)', css)) == 0 )

    def test_i18n(self):
        """
        loads the compose js files for the dutch language.. expect that the catalog strings are in there
        """

        c = CodeSquasher(['i18n_basesoft_web_test'], locale='nl')
        js = c.js_code_as_string()
        self.assertTrue(js.find('test hallo js unit test') > 0)


    def test_es6(self):

        c = CodeSquasher(['iq_order_simulation_details'], locale='nl')
        js = c.js_code_as_string()





# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_js_lint.py', 'basesoft.web.test.unit.test_code_squash.CodeSquashTestCase']
    run()
    sys.exit()
