# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from twisted.trial import unittest
from django.conf import settings

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.dev.util.controllers import IQuestPHP4ConversionController

class ConversionControllerTestCase(iqportal_unittest.IQPortalTestCase):
    def setUp(self):
        if settings.DB_PHP4_IQUEST is None:
            raise unittest.SkipTest("This test will only run if the PHP4 IQuest DB settings are provided")

        self.connect_db()
        self.account = self._create_active_account()
        self.group = self.account.groups[0].group
        self._attach_permissions_to_group(self.group, 'develop_tools', 'create_survey', 'update_survey',
                                          'view_itemqa_filter', 'update_itemqa_filter', 'delete_itemqa_filter')

        self.controller = IQuestPHP4ConversionController(self.session, account_id=self.account.id)
        self.controller.set_connection(**settings.DB_PHP4_IQUEST)

    def test_conversion(self):
        """
        This converts the first publication from the DB_PHP4_IQUEST, kind of a HOCUS BOGUS test
        """
        self.controller.convert_survey_publications({
            'surveys': False,
            'contacts': True,
            'publications': [self.controller.get_publications()[0]['value']]
        }, {
            'unique_identifier': None,
            'license': self.group.license.id
        })

    def test_account_conversion(self):
        """
        This tests the conversion of accounts from an IQuest PHP4 db
        """
        self.controller.convert_accounts(self.group.license, 'User')