# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.
import os

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.dev.util.controllers import IQuestPHP4ImportResponseController
from iqportal.client.test.controllers import FmeTestController
from basesoft.web.contrib.account.models import Account
from iqportal.client.core.controllers import FMEImportContactController

from django.conf import settings

class PHP4ImportResponsesTestCase(iqportal_unittest.IQPortalTestCase):
    def setUp(self):
        self.connect_db()
        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary fme account data

        c = FmeTestController(self.session, account_id=account.id)
        c.populate_fme_domain()
        self.feda_license = c.feda_license
        self.fedet_license = c.fedet_license
        self.account = c.admin_account

        self.group = account.groups[0].group

        self._attach_permissions_to_group(self.group, 'develop_tools')

        self.controller = IQuestPHP4ImportResponseController(self.sa_session, account_id=self.account.id)

    def _get_import_file_lines(self):
        return [
            ['some', 'fields', 'we', 'dont', 'use', '10102', 'other field'],
            ['second', 'fields', 'we', 'dont', 'use', '10065', 'other field']
        ]

    def _add_contacts(self):
        # create the file
        filepointer = os.path.join(settings.TMP_FILE_REPOSITORY, 'test_import_fme_contacts.csv')
        f = open(filepointer, 'w')
        f.write("%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s" % (
                    "CN,COMPANY,NAME,CL,PN",
                    "10065,Yokogawa Europe B.V.,Van der Geer,FEDA: fmecl1,18116249",
                    "10102,Eaton Industries B.V.,Oomen,FEDET: fmecl2,1552329",
                    "10102,Eaton Industries B.V.,Meulenbroek,FEDET: fmecl3,18483154",
                    "10102,Eaton Industries B.V.,Van der Geer,FEDET: fmecl3,18116249",
                    "10173,Honeywell B.V.,Vega,FEDA: fmecl4,1670613",
                    "10173,Honeywell B.V.,Van de Zwaard,FEDA: fmecl5,1887140",
                    "10173,Honeywell B.V.,Van der Poel,FEDA: fmecl6,2488331",
                    "10173,Honeywell B.V.,De Breuk,FEDA: fmecl6,18464084",

                    "10102,Eaton Industries B.V.,Oomen,FEDA: fmecl7,1552329",
        ))
        f.close()

        controller = FMEImportContactController(filepointer, 'test_import_fme_contacts.csv', self.session, self.account.id, 'FMEContact')
        controller.set_file_lines()
        controller.field_mappings = [
            {"file_header": "CN", "column": "company_number", "target": controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COMPANY", "column": "name", "target": controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "NAME", "column": "last_name", "target": controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "CL", "column": "contact_list_name", "target": controller.COLUMN_IS_CONTACT_LIST},
            {"file_header": "PN", "column": "person_number", "target": controller.COLUMN_IS_PERSON_INFO}
        ]
        unique_fields = [{"column": "company_number", "target": controller.COLUMN_IS_COMPANY_INFO},
            {"column": "person_number", "target": controller.COLUMN_IS_PERSON_INFO}]

        return controller.import_contacts(unique_fields=unique_fields)

    def test_import_response_contact_match(self):
        """
        This tests the matching of contacts (FME)
        """
        contacts = self._add_contacts()
        # create a contact list.
        contact_list = contacts[0].contact_lists[0]

        # multi
        lines = self._get_import_file_lines()

        company_number_field = 5

        failures = self.controller.match_contacts(lines, contact_list, company_number_field)
        self.assertEqual(len(failures), 1)

    def test_qas_from_file(self):
        rows = [
            ["","","","","","","","","","","","","","","","1. Diensten","","","","","","2. Hardware","","","","","","","","","","","","3. Nazorg","","","","4. Afzetmarkt","","","","","","","","","","","","","","","","","","","","Heeft u nog een toelichting op de door u ingevulde gegevens? Bijvoorbeeld omzetstijging ivm een (eenmalig) project. Of daling wegens annulering order? Dan kunnen wij dit meenemen bij het interpreteren van de cijfers.","voor administratie"],
            ["","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""],
            ["","","","","","","","","","","","","","","","Engineering","","Montage & Inbedrijfname","","<I>(hier niets invullen svp)</I>","","Transmitters","","Analyse apparatuur","","Regelafsluiters en toebehoren","","Overige instrumentatie","","Systeem","","Overige hardware","","Service etc.","","Opleiding","","Bulkchemie","","Fijnchemie & Farma","","Aardolieraffinaderijen & Petrochemie (HPI)","","Olie- en Gaswinning","","Voedings- & Genotmiddelen","","Papier & Karton","","Waterbehandeling (drinkwater. zuivering. distributie)","","Electriciteitscentrales","","Afvalverwerking (ook alternatieve afvalverwerkingsmethoden)","","Grondstoffenindustrie & Overige (glas. metaal. kunststof etc.)","","",""],
            ["LOGIN_ID","","Bedrijfsnaam","e-Mail adres","Voornaam","Tussenvoegsel","Achternaam","Geslacht","Straatnaam","Straatnummer","Postcode","Plaats","Land","Gebruikersnaam","FME Relatienummer","GEEN RAPPORT","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","Totaal K Euro","Waarvan Nederland","",""]
        ]

        qas = self.controller.build_qas_from_file(rows)
        self.assertEqual(qas['index_37']['answers'], [37, 38])
        self.assertEqual(qas['index_37']['questions'],[37, 39, 41, 43, 45, 47, 49, 51, 53, 55])










