# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

import commands
import os

from twisted.python.reflect import namedAny

from django.conf import settings

import basesoft
from basesoft.web.test.unit import iqportal_unittest

from basesoft import logging
logger = logging.getLogger('basesoft.web.test.unit')

class JsLintTestCase(iqportal_unittest.IQPortalTestCase):
    """I can test files against jslint"""

    def _test(self, path):
        """Private method for executing jsl for the specified directory"""


        if not os.path.exists(settings.JS_LINT_BIN):
            self.fail('%s not found.. you probably need to emerge JavaScript Lint: dev-util/jsl' % settings.JS_LINT_BIN)

        program = basesoft.__path__[0] + '/../related/bin/jsl.sh'
        cmd = '%s %s %s' % (program, settings.JS_LINT_BIN, path)

        logger.debug('JsLintTestCase: executing: %s' % cmd)
        (status, output) = commands.getstatusoutput(cmd)

        self.assertEqual(status, 0, 'jsl returned with non-zero exit code. Output: %s' % output)

    def test_installed_apps(self):
        """
        runs js_lint in all installed_apps (except yui, wireit, etc.)
        """

        exclude = ['basesoft.web.contrib.yui', 'basesoft.web.contrib.wireit', 'basesoft.web.contrib.lib',
                   'basesoft.web.contrib.excanvas', 'basesoft.web.contrib.fusioncharts', 'basesoft.web.contrib.jquery',
                   'basesoft.web.contrib.es6']

        for app in settings.INSTALLED_APPS:

            # ignore django apps
            if app.startswith('django.') or app in exclude:
                continue;

            try:
                module = namedAny(app)
            except Exception, e:
                raise Exception('Cannot import app: %s: %s' % (app, e))
            path = module.__path__[0]

            if os.path.exists(path + '/media'):
                logger.debug('running js_lint test for app %s' % app)

                self._test(path)


# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_js_lint.py', 'basesoft.web.test.unit.test_js_lint']
    run()
    sys.exit()
