# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from django.utils import simplejson

from basesoft.web.dev.compose.models import Mockup
from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.dev.compose.controllers import MockupComposeController
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder
from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException

class MockupComposeControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This will test the Form Compose Controller.
    All individual *public* methods should be tested separately.
    """

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        #create an account so we can check permission with it
        account = self._create_active_account()

        # create an instance of the controller. It is used in all test methods.
        self.controller = MockupComposeController(self.session, account_id = account.id)
        self.group = account.groups[0].group

    def get_state(self):
        return {'mockup':
            {'compose': True,
             'availableLocales': [],
             'hideMenu': True,
             'objectName': u'Form',
             'dbID': None,
             'loaderName': None,
             'domID': u'yui-gen25',
             'name': u'My Form',
             'category': {'label': 'My category'},
             'widgets': [],
             'themeList': [],
             'showMenuButton': True,
             'localeList': [],
             'children': {'fieldsets': [{'compose': True,
                                         'hideMenu': True,
                                         'objectName': u'FormFieldset',
                                         'dbID': None,
                                         'loaderName': None,
                                         'domID': u'yui-gen26',
                                         'legend': u'legend',
                                         'is_expanded': True,
                                         'showMenuButton': True,
                                         'expandable': True,
                                         'children': {'columns': [{'compose': True,
                                                                    'hideMenu': True,
                                                                    'objectName': u'FormColumn',
                                                                    'dbID': None, u'loaderName': None,
                                                                    'domID': u'yui-gen27',
                                                                    'width': 100,
                                                                    'showMenuButton': True,
                                                                    'children': {'items': [{'compose': True,
                                                                                            'hideMenu': True,
                                                                                            'objectName': u'FormItemTextInput',
                                                                                            'dbID': None,
                                                                                            'loaderName': None,
                                                                                            'domID': u'yui-gen28',
                                                                                            'label': u'input label',
                                                                                            'showMenuButton': True,
                                                                                            'children': {},
                                                                                            'locales': {},
                                                                                            'showEmptyIndication': True
                                                                                           },
                                                                                           {'compose': True,
                                                                                            'hideMenu': True,
                                                                                            'objectName': u'FormItemCheckbox',
                                                                                            'dbID': None,
                                                                                            'loaderName': None,
                                                                                            'domID': u'yui-gen68',
                                                                                            'label': u'checkbox label',
                                                                                            'showMenuButton': True,
                                                                                            'children': {},
                                                                                            'locales': {},
                                                                                            'showEmptyIndication': True},
                                                                                           {'compose': True,
                                                                                            'hideMenu': True,
                                                                                            'objectName': u'FormItemButton',
                                                                                            'dbID': None,
                                                                                            'loaderName': None,
                                                                                            'domID': u'yui-gen77',
                                                                                            'label': u'button',
                                                                                            'showMenuButton': True,
                                                                                            'children': {},
                                                                                            'locales': {},
                                                                                            'showEmptyIndication': True
                                                                                            }],
                                                                    'fieldsets': []},
                                                                    'locales': {},
                                                                    'showEmptyIndication': True}]},
                                         'locales': {},
                                         'showEmptyIndication': True}]},
             'locales': {},
             'showEmptyIndication': True}
        }

    def test_save_mockup_state(self):
        """
        This tests saving and updating of a form state
        """
        state = self.get_state()
        # first save
        # user doesn't have permission
        self.assertRaises(PermissionDeniedException,
                          self.controller.save_state,
                          state)
        # now add the perm
        self._attach_permissions_to_group(self.group, 'develop_tools')
        f = self.controller.save_state(state)
        self.assertEqual(f.name, state['mockup']['name'])
        # check if category was saved correctly
        self.assertEqual(f.category.label, state['mockup']['category']['label'])
        self.assertEqual(f.state, simplejson.dumps(state['mockup'], ensure_ascii=False, cls=JsonRpcJSONEncoder))

        # now update
        state['mockup']['name'] = "updated form name"
        state['mockup']['dbID'] = f.id

        self.controller.save_state(state)

        self.assertEqual(f.name, state['mockup']['name'])
        self.assertEqual(f.state, simplejson.dumps(state['mockup'], ensure_ascii=False, cls=JsonRpcJSONEncoder))

    def test_load_mockup_state(self):
        """
        This tests loading a form state
        """
        self._attach_permissions_to_group(self.group, 'develop_tools')
        state = self.get_state()
        f = self.controller.save_state(state)
        load_state = self.controller.load_state(f.id)
        state['mockup']['dbID'] = f.id
        self.assertEqual(load_state, state)

    def test_delete_mockup(self):
        """
        This tests deleting a form
        """
        self._attach_permissions_to_group(self.group, 'develop_tools', 'mark_as_deleted')
        state = self.get_state()
        id = self.controller.save_state(state).id

        self.controller.delete_mockups(id)
        c = self.controller.get_mockups(id)
        self.assertEqual(len(c), 0)
