# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

import sass
import re
from django.conf import settings
from basesoft.web.test.unit.test_abstract_controller import MockRequest

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.util.parse import CodeSquasher
from basesoft.web.core.exceptions import SCSSCompileException
from basesoft.web.contrib.branding.models import Theme

class TestCase(iqportal_unittest.IQPortalTestCase):
    regex = re.compile(r'/\*\*?.+?\*/\s*', re.DOTALL)
    comments = re.compile(r'/\*([^*]|[\r\n]|(\*+([^*/]|[\r\n])))*\*+/', re.DOTALL)

    def _do_module_test(self, modules, output_style='compressed'):

        requires = ['default_variables_scss', 'iq_core_scss']
        if type(modules) != list:
            modules = [modules]

        requires += modules

        cs = CodeSquasher(requires, [])

        modules = cs.module_list('scss')

        code = ''.join([cs._get_code(module_name) for module_name in modules])

        try:
            sass.compile(string=code, output_style=output_style)
        except Exception, e:
            regex = r'^:([\d]+):'
            m = re.search(regex, str(e))
            if m is not None:
                raise SCSSCompileException('Failed to compile. Error: %s, code: %s' % (e, "\n".join(code.split("\n")[int(m.group(1))-15:int(m.group(1))+15])))
            raise

    def _do_test(self, mixin, modules, output_style='compressed', source_comments=False, newlines=False):

        requires = ['default_variables_scss', 'iq_core_scss']
        if type(modules) != list:
            modules = [modules]

        requires += modules

        cs = CodeSquasher(requires, [])

        modules = cs.module_list('scss')

        code = ''.join([cs._get_code(module_name) for module_name in modules])

        try:
            rv = sass.compile(string=code+"\n"+mixin, output_style=output_style, source_comments=source_comments)
            #print rv
        except Exception, e:
            regex = r'^:([\d]+):'
            m = re.search(regex, str(e))
            if m is not None:
                raise SCSSCompileException('Failed to compile. Error: %s, code: %s' % (e, "\n".join(code.split("\n")[int(m.group(1))-15:int(m.group(1))+15])))
            raise

        if not source_comments:
            rv = re.sub(self.comments, '', rv)

        if output_style=='nested':
            rv = re.sub(self.regex, '', rv)

        if not newlines:
            return rv
        elif newlines:
            return rv.replace(';', ";\n")

    def _test_mixin(self, modules, mixin, *args, **kwargs):

        rv = ""
        if kwargs.has_key('prefix'):
            rv += kwargs['prefix']+"\n"
            del kwargs['prefix']

        rv += "@include %s(%s);" % (mixin, ', '.join(args))
        #print rv
        return self._do_test(rv, modules, **kwargs)

    def _test_function(self, modules, function, *args, **kwargs):

        rv = ""
        if kwargs.has_key('prefix'):
            rv += kwargs['prefix']+"\n"
            del kwargs['prefix']

        rv += "@warn %s(%s);" % (function, ', '.join(args))
        return self._do_test(rv, modules, **kwargs)

    def test_scss(self):
        self.connect_db()

        scss_modules = []
        for app in settings.INSTALLED_APPS:
            try:
                js_module = __import__(app + '.js_modules', {}, {}, [''])
            except ImportError:
                continue

            if not hasattr(js_module, 'modules'):
                continue

            if type(js_module.modules) == dict:
                mds = js_module.modules
            else:
                mds = js_module.modules()

            for module, attrs in mds.iteritems():
                if attrs['type'] == 'scss':
                    scss_modules.append(module)

        for theme in self.sa_session.query(Theme).all():
            for m in scss_modules:
                c = CodeSquasher(['default_variables_scss', theme.module_name, 'iq_core',
                    'iq_loader',
                    'iqportal_init',
                    'iq_growl',
                    'iconbar_mixins_scss',
                    'palette_mixins_scss', m], [], request=MockRequest())
                c.css_code_as_string()

    def test_container_mixins(self):
        self._test_mixin(['container_mixins_scss'], 'setContainerFonts', 'hd', newlines=True)

    def test_bg_color(self):
        self._test_mixin('iq_core_scss', 'backgroundColor', '#000000')

    def test_set_yui_button(self):

        def get_vars(
                button_id="""
                (margin: 2px 3px,
                padding: 0 2px,
                font-family: Verdana,
                font-size: 1rem,
                font-weight: 400,
                text-shadow: false,
                cornerlocation: all,
                corner-radius: 3px,

                bidacolors: (
                    normal: #BABABA,
                    active: #FFFFFF,
                    focus: #EBF3FF,
                    hover: #B3D4FF,
                    disabled: #FFFFFF
                  )
                )
                """,
                var='"test_scss"',
                error_location='"test_scss"'
            ):

            return [
                button_id,
                var,
                error_location
            ]
        self._test_mixin(['iq_core_scss'], 'setYUIbutton', *get_vars(), output_style='nested')

    def test_scss_maps(self):
        self._do_module_test('iq_core')

    def test_growl(self):

        def get_vars(growl_version='"warning"'):
            return [growl_version]

        self._test_mixin(['iq_growl_mixins_scss'], 'growl', *get_vars(), output_style='nested')

    def test_datatable_icon_generator(self):

        def get_vars(
                icon_css_path='".yui-panel-container .yui-module .bd .bs-custom_query .bs-customdatatable .yui-dt-data .yui-dt-col-status .yui-dt-liner"',
                icon_content='857',
                cursor='default',
                icon_font_list='("fontello" 1.4 200 #000000 normal)',
                icon_hover_color='#FFFFFF',
                icon_shadow='yes',
                icon_shadow_hover='yes'
            ):

            return [
                icon_css_path,
                icon_content,
                cursor,
                icon_font_list,
                icon_hover_color,
                icon_shadow,
                icon_shadow_hover
            ]

        self._test_mixin(['iq_customdatatable_mixins_scss'], 'datatableIconGenerator', *get_vars(), output_style='nested', newlines=False)

    def test_fontello_datatable_icons(self):

        self._test_mixin('iq_login_manager_scss', 'circled_cross', '".div-class"', output_style='nested', newlines=False)
        self._test_mixin('iq_login_manager_scss', 'circled_checkLogin', '".div-class"', output_style='nested', newlines=False)

    def test_fontello_publication_manager_icons(self):

        self._test_mixin('iquest_publication_manager_scss', 'icon_excel', '".export-cell.xls"', output_style='nested', newlines=False)
