# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.test.unit import iqportal_unittest

from basesoft.util import strings

class StringTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This will test the methods from the string module.
    """

    def test_trim_multiline(self):
        """
        Check that the trim_multiline method works properly
        """

        # a tuple of multiline strings that we'll test
        actual = (
            """A single line string""",
            """   A single line  with spaces   """,
            """A string
               With more
               lines""",
            """
            A string with the first line empty,
               With more
               lines""",
            """
            Another multiple
                lines string
                with a tab in front of this line
                """
        )

        # the expected values for the strings listed above
        expected = (
            """A single line string""",
            """A single line  with spaces""",
            """A string
With more
lines""",
            """
A string with the first line empty,
With more
lines""",
            """
Another multiple
lines string
with a tab in front of this line
"""
        )

        # the expected values for the strings listed above if the trim_empty_lines parameter is True
        expected_no_empty_lines = (
            """A single line string""",
            """A single line  with spaces""",
            """A string
With more
lines""",
            """A string with the first line empty,
With more
lines""",
            """Another multiple
lines string
with a tab in front of this line"""
        )

        for i in range(len(actual)):
            self.assertEquals(strings.trim_multiline(actual[i]), expected[i], "trim_multiline failed at %s" % actual[i])
            self.assertEquals(strings.trim_multiline(actual[i], True), expected_no_empty_lines[i], "trim_multiline (trimmed empty lines) failed at %s" % actual[i])
