# -*- coding: utf-8 -*-

'''
Created on Dec 11, 2013

@author: marten
'''
from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.wiki.parser import Parser
from basesoft.web.contrib.wiki.rpc import PreviewContext

class MockObject:
    def __init__(self, prefix):
        self.prefix = prefix.capitalize()
    def __getattr__(self, name):
        return '%s %s' % (self.prefix, name.capitalize())

class MockContext(object):
    def __getattr__(self, name):
        return MockObject(name)

class TestContext(object):
    def __init__(self, producer):
        self.producer = producer

class ProducerTestContext(object):
    def __init__(self, name, vat_no=None):
        self.name = name
        if vat_no is not None:
            self.vat_no = vat_no

class TestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()

    def test_center(self):
        p = Parser()

        wiki_str = '::center me::'

        result = p.render_to_html(wiki_str)
        self.assertEqual(result, '<p>\n<center>center me</center> <br />\n</p>\n')

        wiki_str = """
::
== Header ==
::
        """
        result = p.render_to_html(wiki_str)
        self.assertEqual(result, '<center> <br />\n<h2 id="">Header</h2>\n</center> <br />\n<p>\n         <br />\n</p>\n')

    def test_image(self):
        p = Parser()

        wiki_str = '[[Image(48c556f9)]]'
        result = p.render_to_html(wiki_str)
        self.assertEqual(result, '<p>\n<img src="/basesoft.web.contrib.file/view/48c556f9" /> <br />\n</p>\n')

    def test_replacement_fields_preview(self):
        p = Parser()

        wiki_str = 'some text @producer:name some more text'

        p.set_context(MockContext())

        result = p.render_to_html(wiki_str)
        self.assertEqual(result, '<p>\nsome text Producer Name some more text <br />\n</p>\n')

        wiki_str = 'some text @{}@ some other text'
        result = p.render_to_html(wiki_str)
        self.assertEqual(result, '<p>\nsome text @{}@ some other text <br />\n</p>\n')

    def test_replacement_fields_preview_formatted(self):
        p = Parser()

        wiki_str = "some text '''@producer:name''' some more text"

        p.set_context(MockContext())

        result = p.render_to_html(wiki_str)
        self.assertEqual(result, '<p>\nsome text <strong>Producer Name</strong> some more text <br />\n</p>\n')

    def test_replacement_fields_in_header(self):
        p = Parser()

        wiki_str = "= @producer:name ="

        p.set_context(MockContext())

        result = p.render_to_html(wiki_str)
        self.assertEqual(result, '<p>\n<h1 id="">Producer Name</h1><br />\n</p>\n')

    def test_replacement_fields_other_context(self):
        p = Parser()

        wiki_str = 'some text @producer:name some more text and then @producer:vat_no this is more text'

        p.set_context(TestContext(ProducerTestContext('Borat Inc.')))

        result = p.render_to_html(wiki_str)

        self.assertEqual(result, '<p>\nsome text Borat Inc. some more text and then <span class="wiki-error">Unsupported replacement field used `vat_no`.</span> this is more text <br />\n</p>\n')

        p.set_context(TestContext(ProducerTestContext('Borat Inc.', 'DE12345678')))

        result = p.render_to_html(wiki_str)

        self.assertEqual(result, '<p>\nsome text Borat Inc. some more text and then DE12345678 this is more text <br />\n</p>\n')

    def test_pagebreak(self):
        p = Parser()

        wiki_str = "some text --{pagebreak}-- some more text"

        result = p.render_to_html(wiki_str)

        self.assertEqual('<p>\nsome text <div style="page-break-after:always;" class="print-page-break"></div><div class="page"></div> some more text <br />\n</p>\n', result)

    def test_table(self):
        p = Parser()

        wiki_str = "|| ''Table'' || Column ||\n|| Row 2 || Col 4 ||"

        result = p.render_to_html(wiki_str)
        self.assertEqual('<p>\n<table class="wiki">\n<tr><td> <em>Table</em> </td><td> Column </td></tr>\n<tr><td> Row 2 </td><td> Col 4 </td></tr>\n</table>\n</p>\n', result)


    def test_context_extend(self):
        """
        Tests bug pvcycle #426
        """

        txt = "aap @Nr. sfdfsf"
        p = Parser()
        p.set_context(PreviewContext())
        p.render_to_html(txt)


    def test_replacement_fields_with_unicode(self):
        p = Parser()

        wiki_str = "= @producer:name处理货 ="

        p.set_context(MockContext())

        result = p.render_to_html(wiki_str)
        self.assertEqual(result, '<p>\n<h1 id="">Producer Name处理货</h1><br />\n</p>\n')

    def test_version_code_block(self):
        p = Parser()

        txt = """
{{{
#!version lt="1.3"
This should only show for versions lower than 1.3
}}}

        """
        p.render_to_html(txt)

    def test_highlight_words(self):
        p = Parser()
        p.set_highlight_words(['Is', 'My', 'Correct'])
        p.set_context(MockContext())

        txt = """
        ''This is a random'' test for my highlighter, is it working Correctly?
        """

        p.render_to_html(txt)


# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_wiki.py', 'basesoft.web.test.unit.test_wiki.TestCase.test_bug']
    run()
    sys.exit()

