'''
Created on Aug 31, 2011

@author: martijn
'''

from datetime import datetime
import random
import os

from basesoft.web.core.util.controllers import AbstractController, StateManager
from sqlalchemy.orm.exc import NoResultFound


from basesoft.web.contrib.license.models import License, Permission, PermissionCategory
from basesoft.web.contrib.contact.models import Person, ContactList
from basesoft.web.contrib.account.models import Account, GroupAccount
from basesoft.web.contrib.license.models import Domain
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.contrib.account.controllers import AccountController
from iqportal.iquest.core.models import SurveyItemQA, SurveyPublication, SurveyItemQAWeightFactor
from basesoft.web.contrib.compose.models import ComposeInvitee
from iqportal.iquest.compose.controllers import ComposeController,\
    PublicationStateController
from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.core.models import PrivilegeObject
from iqportal.iquest.core.schedule import PublishSurveysTask,\
    CreatePublicationInviteesTask
from iqportal.iquest.response.controllers import ParticipantController
from iqportal.iqube.core.models import DataSourceWidget, MdxMethod
from iqportal.client.core.controllers import FMEImportContactController

from django.conf import settings
from sqlalchemy import create_engine
from iqportal.iqube.core.controllers import MdxMethodController

class FmeTestController(AbstractController):

    def grant_configurable_widget_privs(self, fme_group_feda):
        self._grant_configurable_widget_privs(fme_group_feda)

    def _grant_configurable_widget_privs(self, fme_group_feda):
        try:
            # give configurable widget privs here
            dw = self.sa_session.query(DataSourceWidget).filter(DataSourceWidget.label=='Default QA Item').one()
            license_c = LicenseController(self.sa_session, account_id=self.account_id)
            license_c.grant_privilege_to(dw, [PrivilegeObject.PRIV_VIEW], domain=True, target_group=fme_group_feda)

            dw = self.sa_session.query(DataSourceWidget).filter(DataSourceWidget.label=='Growth Item').one()
            license_c.grant_privilege_to(dw, [PrivilegeObject.PRIV_VIEW], domain=True, target_group=fme_group_feda)

            dw = self.sa_session.query(DataSourceWidget).filter(DataSourceWidget.label=='QA Totals Item').one()
            license_c.grant_privilege_to(dw, [PrivilegeObject.PRIV_VIEW], domain=True, target_group=fme_group_feda)

            dw = self.sa_session.query(DataSourceWidget).filter(DataSourceWidget.label=='NPS Item').one()
            license_c.grant_privilege_to(dw, [PrivilegeObject.PRIV_VIEW], domain=True, target_group=fme_group_feda)

            dw = self.sa_session.query(DataSourceWidget).filter(DataSourceWidget.label=='Publication statistics').one()
            license_c.grant_privilege_to(dw, [PrivilegeObject.PRIV_VIEW], domain=True, target_group=fme_group_feda)

            dw = self.sa_session.query(DataSourceWidget).filter(DataSourceWidget.label=='Multi QA Item').one()
            license_c.grant_privilege_to(dw, [PrivilegeObject.PRIV_VIEW], domain=True, target_group=fme_group_feda)
        except NoResultFound:
            pass

    def _grant_mdx_method_privs(self, fme_group_feda):

        mm = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'TopCount').first()
        mmc = MdxMethodController(self.sa_session, account_id=self.account_id)
        mmc.grant_privilege_to(mm, [MdxMethod.PRIV_VIEW], group=True, target_group=fme_group_feda, check_own_privs=False)

        mm = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'BottomCount').first()
        mmc.grant_privilege_to(mm, [MdxMethod.PRIV_VIEW], group=True, target_group=fme_group_feda, check_own_privs=False)

    def populate_fme_domain(self):


        try:
            self.fme_domain = self.sa_session.query(Domain).filter(Domain.name == 'FME').one()
            licenses = self.sa_session.query(License).filter(License.domain_id==self.fme_domain.id).all()
            for l in licenses:
                if l.name == 'FEDA':
                    self.feda_license = l
                if l.name == 'FEDET':
                    self.fedet_license = l
            self.admin_account = self.sa_session.query(Account).filter(Account.name=='fme_admin').first()
            self._grant_configurable_widget_privs(self.admin_account.default_group)
            self._grant_mdx_method_privs(self.admin_account.default_group)
            return

        except NoResultFound:
            pass

        #print 'FME domain not found.. creating new..'

        lc = LicenseController(self.sa_session, account_id=self.get_account().id)
        fme_domain = self.sa_session.query(Domain).filter(Domain.name == 'FME').all()
        if len(fme_domain) == 0:
            self.fme_domain = lc.save_or_update_domain(None, 'FME')
        else:
            self.fme_domain = fme_domain[0]
        contact = Person()
        contact.last_name = 'FME Admin'

        contact_permissions = self.sa_session.query(Permission).join(PermissionCategory).filter(
                    PermissionCategory.name.in_(['Contacts', 'File', 'Mailing'])).all()

        permissions = self.sa_session.query(Permission).filter(Permission.name.in_([

            'view_dashboard', 'save_dashboard',

            'fme_contact_access', 'iquest_nps_access',

            'create_survey', 'view_survey', 'save_survey', 'delete_survey', 'update_survey',
            'create_survey_publication', 'view_survey_publication', 'save_survey_publication', 'delete_survey_publication',
            'update_survey_publication', 'update_published_survey',
            'view_survey_publication_response', 'schedule_survey_publication',
            'view_iquest_pool_category', 'create_iquest_pool_category',

            'create_report', 'view_report', 'update_report',
            'view_report_publication', 'create_report_publication', 'update_report_publication',
            'schedule_report_publication', 'use_iquest_report_publication',
            'view_datasource', 'datasource_manager', 'view_datasource_widget',

            'use_iquest_time_dimension', 'use_iquest_person_dimensions', 'use_iquest_company_dimensions',
            'manage_mdx_methods', 'create_individual_reports',

            'manage_license_company_logo', 'manage_license_configs',

        ])).all() + contact_permissions

        self.feda_license = lc.save_or_update_license(None, 'FEDA', self.fme_domain, License, contact, None, None, True)
        self.feda_license.permissions = permissions

        self.fedet_license = lc.save_or_update_license(None, 'FEDET', self.fme_domain, License, contact, None, None, False)
        self.fedet_license.permissions = permissions

        fme_group_feda = lc.save_or_update_group(None, 'FME FEDA Group', self.feda_license, is_default=True)
        fme_group_feda.permissions = permissions

        fme_group_vla = lc.save_or_update_group(None, 'FME VLA Group', self.fedet_license, is_default=True)
        fme_group_vla.permissions = permissions

        self.admin_account = self._create_account('fme_admin', [fme_group_feda, fme_group_vla])
        self.a_feda = self._create_account('fme_feda', [fme_group_feda])
        self.a_vla = self._create_account('fme_vla', [fme_group_vla])

        self._grant_configurable_widget_privs(fme_group_feda)
        self._grant_mdx_method_privs(self.admin_account.default_group)
        self.sa_session.flush()

        # run the db_sync cuz they check for fme license
        from iqportal.client.core import db_sync as core_sync
        core_sync.sync(self.sa_session)

        return

    def _create_account(self, name, groups):
        a = Account()

        a.name = name
        a.password = AccountController(self.sa_session).hash_password('fme_password')
        a.person = Person()
        self.sa_session.add(a)

        # attach admin to both licenses
        is_default = 1
        for g in groups:
            ga = GroupAccount()
            ga.is_default = is_default
            ga.active = True
            ga.account = a
            ga.group = g
            self.sa_session.add(ga)

            is_default = 0

        return a

    def __update_module_ids(self, state, id_map, stripDomID=False):
        """
        recursive updates state's dbID's for the given id_map
        """

        dbID = id_map[ state['domID'] ]
        state['dbID'] = dbID

        if stripDomID:
            del state['domID']

        if (state.has_key('children')):
            for child_attr in ['blocks', 'columns', 'items', 'questions', 'answers']:
                if state['children'].has_key(child_attr):
                    for child in state['children'][child_attr]:
                        self.__update_module_ids(child, id_map, stripDomID)

    def get_contact_lists(self, account):
        # save and return a contact list for FME license.
        # create the file
        filepointer = os.path.join(settings.TMP_FILE_REPOSITORY, 'test_import_fme_contacts.csv')
        f = open(filepointer, 'w')
        f.write("%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s" % (
                    "CN,COMPANY,NAME,CL,PN,GENDER,PERSON_COUNTRY,COMPANY_COUNTRY",
                    "10065,Yokogawa Europe B.V.,Van der Geer,FEDA: fmecl1,18116249,m,nl,jp",
                    "10102,Eaton Industries B.V.,Oomen,FEDET: fmecl2,1552329,f,de,gb",
                    "10102,Eaton Industries B.V.,Meulenbroek,FEDET: fmecl3,18483154,m,gb,gb",
                    "10102,Eaton Industries B.V.,Van der Geer,FEDET: fmecl3,18116249,m,nl,gb",
                    "10173,Honeywell B.V.,Vega,FEDA: fmecl4,1670613,f,nl,nl",
                    "10173,Honeywell B.V.,Van de Zwaard,FEDA: fmecl5,1887140,m,nl,nl",
                    "10173,Honeywell B.V.,Van der Poel,FEDA: fmecl6,2488331,m,de,nl",
                    "10173,Honeywell B.V.,De Breuk,FEDA: fmecl6,18464084,f,de,nl",
                    "10102,Eaton Industries B.V.,Oomen,FEDA: fmecl7,1552329,m,de,gb",
        ))
        f.close()

        controller = FMEImportContactController(filepointer, 'test.csv', self.sa_session, account.id, 'FMEContact')
        controller.set_file_lines()
        controller.field_mappings = [
            {"file_header": "CN", "column": "company_number", "target": controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COMPANY", "column": "name", "target": controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "NAME", "column": "last_name", "target": controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "CL", "column": "contact_list_name", "target": controller.COLUMN_IS_CONTACT_LIST},
            {"file_header": "PN", "column": "person_number", "target": controller.COLUMN_IS_PERSON_INFO},

            {"file_header": "GENDER", "column": "gender", "target": controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "PERSON_COUNTRY", "column": "country_iso_3166_a2", "target": controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "COMPANY_COUNTRY", "column": "company_country_iso_3166_a2", "target": controller.COLUMN_IS_COMPANY_INFO},
        ]
        unique_fields = [{"column": "company_number", "target": controller.COLUMN_IS_COMPANY_INFO},
                              {"column": "person_number", "target": controller.COLUMN_IS_PERSON_INFO}]

        controller.import_contacts(unique_fields=unique_fields)
        return self.sa_session.query(ContactList).filter(ContactList.name.in_(['fmecl1', 'fmecl2' ,'fmecl3' ,'fmecl4',
                                                                           'fmecl5', 'fmecl6', 'fmecl7'])).all()

    def populate_survey(self, validate_market_share=False):


        state = {
            "survey": {
                "name": u"FME Populated Survey",
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"canvas",

                "availableLocales": ['en_GB'],
                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },

                "savePoolItems": True,

                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },

                "children": {
                    "blocks": [


                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "domID":u"b1",
                            "pageEnd": False,

                            "children": {
                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"b1c1",
                                    "width":50,

                                    "children": {
                                        "items":[{
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa1",
                                            "questionsAlign": SurveyItemQA.ALIGN_LEFT,
                                            "answersAlign": SurveyItemQA.ALIGN_TOP,
                                            "trailingQuestions": False,
                                            "type": SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "validateMarketShare": validate_market_share,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'1. Handel (in-/verkoop zonder toegevoegde waarde)'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"qa1q1",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'1. Pompen & Nogwat',
                                                        }
                                                    },
                                                },{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"qa1q2",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'1.2 Motoren',
                                                        }
                                                    },
                                                },{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"qa1q3",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'1.3 Cilinders (Catalogus)',
                                                        }
                                                    },
                                                },{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"qa1q4",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'1.4 Filtertechniek (ook by-pass filtraties)',
                                                        }
                                                    },
                                                },{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"qa1q5",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'1.5 Accumulatoren',
                                                        }
                                                    },
                                                },{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"qa1q6",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'1.6 Warmtewisselaars',
                                                        }
                                                    },
                                                },

                                                ],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"qa1a1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Order Intake kEuro with a very long title that normally doesnt fit in the legend'
                                                        }
                                                    },
                                                },{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"qa1a2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Waarvan Nederland'
                                                        }
                                                    },
                                                },{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"qa1a3",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Omzet kEuro'
                                                        }
                                                    },
                                                },{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"qa1a4",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Waarvan Nederland'
                                                        }
                                                    },
                                                },
                                                ]
                                            }
                                        },

                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa2",
                                            "questionsAlign": SurveyItemQA.ALIGN_LEFT,
                                            "answersAlign": SurveyItemQA.ALIGN_TOP,
                                            "trailingQuestions": False,
                                            "type": SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "validateMarketShare": validate_market_share,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'5. Schatting totaalmarkt Hydrauliek'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"qa2q1",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Uw schatting',
                                                        }
                                                    },
                                                },

                                                ],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"qa2a1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'kEuro'
                                                        }
                                                    },
                                                },
                                                ]
                                            }
                                        },
                                        {u'objectName': u'ItemQANPS',
                                         u'dbID': None,
                                         u'domID': u'yui-gen622',
                                         u'showTitle': True,
                                         u'locales': {u'en_GB': {u'label': u'Would you?'}},
                                         u'sum_answers': False,
                                         u'sum_questions': False,
                                         u'loaderName': None,
                                         u'validateMarketShare': False,
                                         u'questionsAlign': SurveyItemQA.ALIGN_LEFT,
                                         u'answersAlign': SurveyItemQA.ALIGN_BOTTOM,
                                         u'poolItemID': None,
                                         u'showMenuButton': True,
                                         u'filters': [],
                                         u'type': SurveyItemQA.TYPE_SINGLE_CHOICE,
                                         u'alwaysShowAnswerLabels': True,
                                         u'children': {
                                                u'answers': [{
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'NO'}},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen625'
                                                }, {
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'1'}},
                                                    u'dbID': None, u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen626'
                                                }, {
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'2'}},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen627'
                                                }, {
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'3'}},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen628'
                                                }, {
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'4'}},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen629'
                                                }, {
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'neutral'}},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen630'
                                                }, {
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'6'}},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen631'
                                                }, {
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'7'}},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen632'
                                                }, {
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'8'}},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen633'
                                                }, {
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'9'}},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen634'
                                                }, {
                                                    u'locales': {u'en_GB': {u'trailing_label': None, u'label': u'YES'}},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyAnswer',
                                                    u'domID': u'yui-gen635'}
                                                ],
                                                u'questions': [{
                                                    u'locales': {},
                                                    u'dbID': None,
                                                    u'poolElementID': None,
                                                    u'objectName': u'SurveyQuestion',
                                                    u'domID': u'yui-gen623'}
                                                ]},
                                            u'showEmptyIndication': True,
                                            u'layout': None,
                                            u'alwaysShowQuestionLabels': False,
                                            u'filters': [],
                                            u'required': False,
                                            u'layoutAttributes': {u'3': u'2'},
                                            u'trailingQuestions': False
                                        }
                                        ]
                                    }
                                },
                                ]
                            }
                        },
                    ]
                }
            }
        }

        locale_en = self.sa_session.query(Locale).filter_by(locale='en_GB').one()

        # init accounts
        a_admin = self.sa_session.query(Account).filter(Account.name=='fme_admin').one()
        a_feda = self.sa_session.query(Account).filter(Account.name=='fme_feda').one()
        a_vla = self.sa_session.query(Account).filter(Account.name=='fme_vla').one()

        # controllers
        compose_c_feda = ComposeController(self.sa_session, self.locale, account_id=a_feda.id)
        compose_c_vla = ComposeController(self.sa_session, self.locale, account_id=a_vla.id)

        license_c_feda = LicenseController(self.sa_session, account_id=a_feda.id)
        license_c_vla = LicenseController(self.sa_session, account_id=a_feda.id)

        # save the survey as feda user
        map = compose_c_feda.save_state(state)
        survey_id = map['canvas']

        """

        # code below can be used when we have to test for pool items

        # empty the state dbids (except the pool items ids)
        state = compose_c_feda.load_state(survey_id)
        self._remove_dbids(state['survey'])

        pool_item_cat = self.sa_session.query(IQuestItemPoolCategory) \
            .join(IQuestItemPoolItem) \
            .join(SurveyItem) \
            .filter(SurveyItem.id == map['yui-gen77']) \
            .one()

        # let feda user promote the pool-item to domain level so vla can save it
        license_c_feda.grant_privilege_to(pool_item_cat, [PrivilegeObject.PRIV_VIEW], domain=True)

        # save state as vla
        compose_c_vla.save_state(state)
        """


        return survey_id

    def add_weight_factors(self, survey):
        # add weight factors for some of the QA's
        # QA1 is Text, we'll add weight per Answer
        mc = survey.itemQAs[0]
        for answer in mc.answers:
            w = SurveyItemQAWeightFactor()
            w.item = mc
            w.answer = answer
            w.weight = answer.position+1
            self.sa_session.add(w)

        # QA2 is Single Choice (NPS), we'll add weight per Question-Answer
        sc = survey.itemQAs[2]
        for question in sc.questions:
            for answer in sc.answers:
                w = SurveyItemQAWeightFactor()
                w.item = sc
                w.question = question
                w.answer = answer
                w.weight = (answer.position+1)*(question.position+1)
                self.sa_session.add(w)

        self.sa_session.flush()

    cnt = 0
    def _remove_dbids(self, state):

        self.cnt = self.cnt + 1

        state['dbID'] = None
        state['domID'] = 'yui-' + unicode(self.cnt)

        if (state.has_key('children')):
            for child_attr in ['blocks', 'columns', 'items', 'questions', 'answers']:
                if state['children'].has_key(child_attr):
                    for child in state['children'][child_attr]:
                        self._remove_dbids(child)


    def publish_survey(self, survey_id, begin_date=None, end_date=None, label='1', additional_state=None, anonymous=True):

        a_feda = self.sa_session.query(Account).filter(Account.name=='fme_feda').one()

        if begin_date is None:
            begin_date = datetime.now()

        # create publication
        publish_state = {
            'survey_id': survey_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": end_date,
            "status": SurveyPublication.STATUS_SCHEDULED,
            "label": u"publication %s" % label,
            "show_page_nrs": True,
            "anonymous": anonymous,
            "contact_lists": [],
            "mailings": [], # no mailings necessary,
            'viewer_print': False,
            'exit_url': None
        }
        if additional_state is not None:
            for i, v in additional_state.iteritems():
                publish_state[i] = v

        # save publication
        publication_controller = PublicationStateController(self.sa_session, StateManager(self.locale, a_feda.id, None))
        publication = publication_controller.save(publish_state)

        # publish the surveys like the scheduler would do
        PublishSurveysTask().execute()
        CreatePublicationInviteesTask().execute()

        return publication.id


    def populate_responses(self, publication_id, seed=None, invitee_hash=None):

        random.seed(seed)

        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.id==publication_id).one()
        survey = publication.survey

        if invitee_hash is None:
            invitees = self.sa_session.query(ComposeInvitee).filter(ComposeInvitee.publication_id==publication_id).all()
            invitee_hash = unicode(invitees[0].invitee_hash)

        controller = ParticipantController(self.sa_session, self.locale, invitee_hash)

        qa1_responses = []

        itemQAs = survey.itemQAs

        # first qa
        for i in range(6):
            for j in range(4):
                qa1_responses.append({
                    'q': itemQAs[0].questions[i].id,
                    'a': itemQAs[0].answers[j].id,
                    'v': int(random.random() * 1000) })

        # second qa
        qa1_responses.append({
            'q': itemQAs[1].questions[0].id,
            'a': itemQAs[1].answers[0].id,
            'v': int(random.random() * 1000)
        })

        # last qa (nps)
        qa1_responses.append({
            'q': itemQAs[len(itemQAs) - 1].questions[0].id,
            'a': itemQAs[len(itemQAs) - 1].answers[random.choice(range(0, 11))].id,
            'v': 1
        })

        state = []
        state.append({ 'item_id': survey.itemQAs[0].id,
            'responses': qa1_responses
        })

        controller.update_responses(state)

        # update the just created responses from today to a certain day in january
        day = int(random.random() * 30)+1

        sql = """
            UPDATE tblSurveyParticipant SET created = '2013-01-%s', last_updated='2013-01-%s'
            WHERE invitee_id = (SELECT id FROM tblComposeInvitee WHERE invitee_hash = '%s')
        """ % (day, day, invitee_hash)
        db_engine = create_engine(settings.DB_CONNSTR, echo=False)
        db_engine.execute(sql)





