/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 *
 * Testcase for the IQReport Compose
 */


BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "IQReport Compose Test",
	assertClean: true,

	_should: {
		ignore: {
			testSaveLoadUpdate: true
		}
	},

	setUp : function () {

		// style the sandbox a bit
		var sandbox = YAHOO.util.Dom.get('sandbox');
		sandbox.innerHTML = '';

		var composition_el = document.createElement('div');
		sandbox.appendChild(composition_el);

		var localeList = [
			{name: 'Dutch', locale: 'nl_NL'},
			{name: 'English', locale: 'en_GB'},
			{name: 'Deutch', locale: 'de_DE'}
		];

		var themeList = [
			{name: 'Debug', cssClass: 'iqreport-debug', module: 'iqreport_canvas_theme_debug'},
			{name: 'BaseSoft', cssClass: 'iqreport-basesoft', module: "iqreport_canvas_theme_basesoft"}
		];


		this.manager = new BASESOFT.iqreport.compose.ComposeManager();
		this.manager.init({
			'composition_el': composition_el,
			'compose_vars': {
				'localeList': localeList,
				'themeList': themeList
			}
		});

		// add the manager to the testcase garbage so it gets destroyed at tearDown
		this._garbage.push(this.manager);

		// start new empty survey
		this.manager.openNew();

		// selected locales
		this.manager.canvas.set('availableLocales', ['en_GB', 'nl_NL']);

		// testcase broker
		this.testcase_broker = new BASESOFT.util.JsonBroker('/iqportal.iqreport.test/selenium/yui/rpc/');
		this.loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', 60000);
	},

	testSaveLoadUpdate: function() {

		try {

			// scoped function starts running the test
			var start = function(){

				this.loginrpc.call('login', ['user_compose', 'pass_compose'], function(){
					this.resume(function(){
						try {
							console.debug('logged in.. calling initDataSource..');
							initDataSource.call(this);
						}
						catch (e) {
							e.delay || console.error(e);
							throw e;
						}
					});

				}, null, this);

				this.wait(this.timeout);
			};

			var initDataSource = function() {

				this.testcase_broker.call('init_datasource', [], function(datasourceId){

					this.resume(function(){
						try {
							console.debug('datasource created', datasourceId);
							initCanvas.call(this, datasourceId);
						}
						catch (e) {
							e.delay || console.error(e);
							throw e;
						}
					});

				}, null, this, this.timeout);
				this.wait(this.timeout);

			};


			var initCanvas = function(datasourceId){

				var Assert = YAHOO.util.Assert;
				var canvas = this.manager.canvas;

				canvas.set('datasource_id', datasourceId);

				var b1 = new BASESOFT.iqreport.widget.Block(this.manager);
				canvas.add('blocks', b1);

				var b1c1 = b1.children['columns'][0];

				var result = new BASESOFT.iqreport.widget.ItemResult(this.manager, {
					'mdx_statement': {
						columns: ['[Product].Members'],
						rows: ['[Shop].Members'],
						cube: 'Order',
						slicers: ['[Measures].[TotalInvoiced]']
					}
				});
				b1c1.add('items', result);

				// open and close panel.. that should load and write the mdx statement
				var panel = result.openPivotTable();
				panel.destroy();

				// add a graph
				result.add('graphs', new BASESOFT.iqreport.widget.Graph(this.manager, {
					'graph_type': BASESOFT.iqreport.widget.Graph.prototype.GRAPH_TYPE_BAR_CHART
				}));


				// now save it
				var e = this.manager.save();
				e.subscribe(function() {

					this.resume(function() {


						// now load it again
						e = this.manager.load(this.manager.canvas.get('dbID'));
						e.subscribe(function() {
							this.resume(function() {

								// and update it
								e = this.manager.save();
								e.subscribe(function() {
									this.resume(function() {
										console.info('test finished');
									});
								}, null, this);

								this.wait();

							});
						}, null, this);

						this.wait();

					});

				}, null, this);
				this.wait();

			};

			// call start to begin test
			start.call(this);

		}
		catch (e) {
			e.delay || console.error(e);
			throw e;
		}
	}

}));

YAHOO.register("test_compose", null, {version: "1.0.0", build: "1"});
