
(function() {
	'use strict';

	/*
	TODO:
	 * These tests should actually be asserting. the item-result should
	 * throw an event when its done rendering and the tests should wait for that. Then
	 * we can also test for leaking events.
	 *
	 * Check for IE!
	 */

BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name : "test_iqreport_graphs",

	setUp : function() {
		this.sandbox = YAHOO.util.Dom.get('sandbox');
		this.sandbox.innerHTML = '';

		this._rpc = new BASESOFT.util.JsonBroker('iqportal.iqube.test.IQubeTestRPCHandler', {
			timeout : 120000,
			executeIfLoggedOut : true
		});
		this._garbage.push(this._rpc);

		var composition_el = document.createElement('div');
		this.sandbox.appendChild(composition_el);

		var localeList = [{
			name : 'Dutch',
			locale : 'nl_NL'
		}, {
			name : 'English',
			locale : 'en_GB'
		}, {
			name : 'Deutch',
			locale : 'de_DE'
		}];

		var themeList = [{
			name : 'BaseSoft',
			cssClass : 'survey-basesoft',
			module : 'iquest_survey_theme_basesoft'
		}];

		this.manager = new BASESOFT.iqreport.compose.ComposeManager();
		this._garbage.push(this.manager);

		this.manager.init({
			'composition_el' : composition_el,
			'compose_vars' : {
				'localeList' : localeList,
				'themeList' : themeList
			}
		});

		// start new empty survey
		this.manager.openNew();

	},

	_setupDataSource : function() {

		var ev = new YAHOO.util.CustomEvent();

		this._rpc.call("get_foodmart_datasource", [], function(result) {
			BASESOFT.env.account.id = result.account_id;
			this._datasource_id = result.datasource_id;
			ev.fire();
		}, null, this);

		return ev;

	},

	_setupDataSourceBeer : function() {

		var ev = new YAHOO.util.CustomEvent();

		this._rpc.call("create_beer_project", [], function(result) {
			BASESOFT.env.account.id = result.account_id;
			this._datasource_id = result.datasource_id;
			ev.fire();
		}, null, this);

		return ev;

	},

	testSimpleChart: function() {

		this._setupDataSource().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : false,
						"show_graph" : true,
						"hide_empty_fields": false,
						"slicers": [],
						"columns" : [

						/*
						{
							"member_unique_name" : "[Measures].[quantity]",
							"depth" : "0",
							"hierarchy_unique_name" : "[Measures]"
						}
						*/
						{
							"member_unique_name" : "[Store].[All Stores].Children",
							"depth" : "0",
							"hierarchy_unique_name" : "[Store]"
						}
						],
						"rows" : [],
						"datasource_id" : this._datasource_id,
						"cube" : "blackbox_test_cube",
						"show_slicers" : true
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'column3d'
					},
					'dbID': 1 // to prevent opening of properties
				};

				config['appliedConfig']['rows'].push({
					"member_unique_name" : "[Product].[All Products].Children",
					"hierarchy_unique_name" : "[Product]"
				});


				var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
				b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, BASESOFT.util.clone(config)));


				this._waitForGraph();

			});

		}, null, this);
		this.wait(120000);
	},

	_waitForGraph: function() {

		BASESOFT.util.waitForCondition(function() {
			return YAHOO.util.Selector.query('svg').length > 0;
		}, function() {
			this.resume(function() {
				console.warn('test done');
			});
		}, null, this);
		this.wait();

	},

	testMultipleDimensionsPie : function() {

		this._setupDataSource().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : false,
						"show_graph" : true,
						"columns" : [{
							"member_unique_name" : "[Measures].[Unit Sales]",
							"depth" : "0",
							"hierarchy_unique_name" : "[Measures]"
						}],
						"rows" : [{
							"member_unique_name" : "[Store].[All Stores].Children",
							"hierarchy_unique_name" : "[Store]"
						}, {
							"member_unique_name" : "[Product].[All Products].Children",
							"hierarchy_unique_name" : "[Product]"
						}],
						slicers: [],
						"hide_empty_fields": false,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales",
						"show_slicers" : true
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'pie3d'
					}
				};

				var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
				b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, BASESOFT.util.clone(config)));

				this._waitForGraph();


			});

		}, null, this);
		this.wait(120000);
	},


	testMultipleDimensionsBarChart : function() {

		this._setupDataSource().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : false,
						"show_graph" : true,
						"columns" : [{
							"member_unique_name" : "[Store].[All Stores].Children",
							"hierarchy_unique_name" : "[Store]"
						},{
							"member_unique_name" : "[Store Type].[All Store Types].Children",
							"hierarchy_unique_name" : "[Store Type]"
						}],
						"rows" : [{
							"member_unique_name" : "[Product].[All Products].Children",
							"hierarchy_unique_name" : "[Product]"
						},{
							"member_unique_name" : "[Gender].[All Gender].Children",
							"hierarchy_unique_name" : "[Gender]"
						}],
						slicers: [],
						"hide_empty_fields": false,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales",
						"show_slicers" : true
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'mscolumn3d',
						'graphs_per_line': 2,
						'aspect_ratio': 1.33
					},

					'dbID': 1 // to prevent opening of properties
				};

				var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
				b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, BASESOFT.util.clone(config)));

				this._waitForGraph();

			});

		}, null, this);
		this.wait(120000);
	},

	testFusionSingleGraph : function() {

		this._setupDataSource().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : true,
						"show_graph" : true,
						"columns" : [
						{
							"member_unique_name" : "[Measures].[Unit Sales]",
							"depth" : "0",
							"hierarchy_unique_name" : "[Measures]"
						}
						],
						"rows" : [

						{
							"member_unique_name" : "[Product].[All Products].Children",
							"hierarchy_unique_name" : "[Product]"
						}

						],
						slicers: [],
						"hide_empty_fields": false,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales",
						"show_slicers" : true
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'column3d',
						'hide_small_values': true,
						'fusion_chart.showLegend': true
					},

					'dbID': 1 // to prevent opening of properties
				};

				var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
				b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, BASESOFT.util.clone(config)));

				this._waitForGraph();

			});

		}, null, this);
		this.wait(120000);
	},

	testFusionMultiGraph : function() {

		this._setupDataSource().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : true,
						"show_graph" : true,
						"columns" : [{
							"member_unique_name" : "[Store].[All Stores].Children",
							"hierarchy_unique_name" : "[Store]"
						}],
						"rows" : [{
							"member_unique_name" : "[Product].[All Products].Children",
							"hierarchy_unique_name" : "[Product]"
						}],
						"datasource_id" : this._datasource_id,
						"cube" : "Sales",
						"show_slicers" : true,
						slicers: [],
						"hide_empty_fields": false
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'mscolumn2d',
						'hide_small_values': true
					},

					'dbID': 1 // to prevent opening of properties
				};

				var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
				b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, BASESOFT.util.clone(config)));

				this._waitForGraph();

			});

		}, null, this);
		this.wait(120000);
	},


	testFusionSingleSeries : function() {

		this._setupDataSource().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : false,
						"show_graph" : true,
						"columns" : [
						{
							"member_unique_name" : "[Measures].[Unit Sales]",
							"depth" : "0",
							"hierarchy_unique_name" : "[Measures]"
						}
						],
						"rows" : [

						{
							"member_unique_name" : "[Product].[All Products].Children",
							"hierarchy_unique_name" : "[Product]"
						}

						],
						"datasource_id" : this._datasource_id,
						"cube" : "Sales",
						"show_slicers" : false,
						slicers: [],
						"hide_empty_fields": false
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'pie'
					},

					'dbID': 1 // to prevent opening of properties
				};

				var chart_types = [
					'column2d',
					'column3d',
					'pie2d',
					'pie3d',
					'line2d',
					'bar2d',
					'area2d',
					'doughnut2d',
					'doughnut3d',
					'pareto2d',
					'pareto3d'
				];

				for (var i in chart_types) {



					var config_ = BASESOFT.util.clone(config);
					config_['graphConfig']['type'] = chart_types[i];

					var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
					b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, config_));

				}

				this._waitForGraph();


			});

		}, null, this);
		this.wait(120000);
	},


	testFusionMultiSeries : function() {

		this._setupDataSource().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : true,
						"show_graph" : true,
						"columns" : [{
							"member_unique_name" : "[Store].[All Stores].Children",
							"hierarchy_unique_name" : "[Store]"
						}],
						"rows" : [{
							"member_unique_name" : "[Product].[All Products].Children",
							"hierarchy_unique_name" : "[Product]"
						}],
						"datasource_id" : this._datasource_id,
						"cube" : "Sales",
						"show_slicers" : false,
						slicers: [],
						"hide_empty_fields": false
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'pie'
					},

					'dbID': 1 // to prevent opening of properties
				};

				var chart_types = [
					'mscolumn2d',
					'mscolumn3d'
				];

				for (var i in chart_types) {

					var config_ = BASESOFT.util.clone(config);
					config_['graphConfig']['type'] = chart_types[i];

					var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
					b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, config_));

				}

				this._waitForGraph();


			});

		}, null, this);
		this.wait(120000);
	},


	testSingleColsOnly : function() {

		this._setupDataSource().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : true,
						"show_graph" : true,
						"columns" : [{
							"member_unique_name" : "[Store].[All Stores].Children",
							"hierarchy_unique_name" : "[Store]"
						},{
							"member_unique_name" : "[Product].[All Products].Children",
							"hierarchy_unique_name" : "[Product]"
						}],
						"rows" : [],
						"datasource_id" : this._datasource_id,
						"cube" : "Sales",
						"show_slicers" : true,
						slicers: [],
						"hide_empty_fields": false
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'pie3d'
					},

					'dbID': 1 // to prevent opening of properties
				};

				var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
				b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, BASESOFT.util.clone(config)));

				this._waitForGraph();

			});

		}, null, this);
		this.wait(120000);
	},

	testMultiColsOnly : function() {

		this._setupDataSource().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : true,
						"show_graph" : true,
						"columns" : [{
							"member_unique_name" : "[Store].[All Stores].Children",
							"hierarchy_unique_name" : "[Store]"
						},{
							"member_unique_name" : "[Product].[All Products].Children",
							"hierarchy_unique_name" : "[Product]"
						}, {
							"member_unique_name" : "[Gender].[All Gender].Children",
							"hierarchy_unique_name" : "[Gender]"
						}
						/*
						{
							"member_unique_name" : "GENERATE([period].Children, TOPCOUNT([period].CurrentMember * [crso].Children, 3, [Measures].[quantity]))",
							"hierarchy_unique_name" : "[period]"

						}*/
						],
						"rows" : [],
						slicers: [],
						"hide_empty_fields": false,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales",
						"show_slicers" : true
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'mscolumn3d',
						'graphs_per_line': 1
					},

					'dbID': 1 // to prevent opening of properties
				};

				var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
				b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, BASESOFT.util.clone(config)));

				this._waitForGraph();

			});

		}, null, this);
		this.wait(120000);
	},


	testTimeFormatter : function() {

		/**
		 * FIXME: need a datasource with a proper time dimension
		 */

		/*
		this._setupDataSourceBeer().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : true,
						"show_graph" : true,
						"columns": [
							{"member_unique_name" : "[Measures].[TIME__TIME]", "hierarchy_unique_name" : "[Measures]"}
						],
						"rows" : [
							{"member_unique_name" : "[NAME].Children", "hierarchy_unique_name" : "[NAME]"}
						],
						"datasource_id" : this._datasource_id,
						"cube" : "blackbox_test_cube",
						"show_slicers" : true
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "beer_test_cube"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'column3d',
						'graphs_per_line': 1
					},

					'dbID': 1 // to prevent opening of properties
				};

				var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
				b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, BASESOFT.util.clone(config)));

			});

		}, null, this);
		this.wait(120000);
		*/
	},

	/**
	 * tests a set on columns with stacked-bar graph.. shouldnt javascript error
	 */
	testInvalidGraph1: function() {


		this._setupDataSource().subscribe(function() {
			this.resume(function() {

				// initial config
				var config = {
					'appliedConfig' : {
						"show_table" : true,
						"show_graph" : true,
						"columns" : [{
							"member_unique_name" : "[Store].[All Stores].Children",
							"hierarchy_unique_name" : "[Store]"
						}
						],
						"rows" : [],
						slicers: [],
						"hide_empty_fields": false,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales",
						"show_slicers" : true
					},
					'config' : {
						"reporter" : "iqportal.iqube.core.ReportResultIQubeReporter",
						"has_drill_through" : true,
						"has_drill_position" : true,
						"has_drill_replace" : true,
						"show_excel_export" : true,
						"fireDrillEvents" : true,
						"has_table" : true,
						"has_graph" : true,
						"has_hierarchy_names" : true,
						"datasource_id" : this._datasource_id,
						"cube" : "Sales"
					},
					'showTitle' : false,
					'graphConfig' : {
						'type' : 'stcolumn3d',
						'graphs_per_line': 1
					},

					'dbID': 1 // to prevent opening of properties
				};

				var b1c1 = this.manager.canvas.children['blocks'][0].children['columns'][0];
				b1c1.add('items', new BASESOFT.iqreport.widget.ItemResult(this.manager, BASESOFT.util.clone(config)));

				// just wait a bit, making sure no JS errors exist
				BASESOFT.util.setTimeout(function() {
					this.resume(function() {
						console.info('test done');
					});
				}, 2000, null, this);

				console.warn('watiin..');
				this.wait();

			});

		}, null, this);
		this.wait(120000);



	},


	testChartFormatters : function() {


		BASESOFT.load({require: ['iqube_formatters'], scope: this, onSuccess: function() {
			this.resume(function() {

				var cF = BASESOFT.iqube.CellFormatter.chartFormat;

				// defualt formatter
				var f = cF('');
				console.warn('f', f);
				YAHOO.util.Assert.areEqual('.', f.decimalSeparator);
				YAHOO.util.Assert.areEqual(',', f.thousandSeparator, ',');

				// 2 decimals
				f = cF('#,###.00');
				YAHOO.util.Assert.areEqual('.', f.decimalSeparator, '#,###.00');
				YAHOO.util.Assert.areEqual(',', f.thousandSeparator, ',', '#,###.00');
				YAHOO.util.Assert.areEqual(2, f.decimals, '#,###.00');

				// 0 decimals
				f = cF('#,###');
				YAHOO.util.Assert.areEqual('.', f.decimalSeparator, '#,###');
				YAHOO.util.Assert.areEqual(',', f.thousandSeparator, '#,###');
				YAHOO.util.Assert.areEqual(0, f.decimals, '#,###');

			});

		}});
		this.wait();



	}

}));

})();

