# -*- coding: utf-8 -*-
"""
Created on Nov 6, 2014

@author: rado
"""
from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase
from django.conf import settings
from selenium.common.exceptions import NoSuchElementException


class TestCase(SeleniumTestCase):
    def setUp(self):

        super(TestCase, self).setUp()
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.default_group
        self._attach_permissions_to_group(self.group, 'create_report', 'create_report_publication', 'view_report', 'view_dashboard',
                                          'item_result', 'view_datasource', 'view_report_publication')

    def test_report_type_selection(self):

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        #click IQ Report icon
        self.click_iconbar(title='IQReport')
        self.wait_for_loading()

        #open pallete for widgets
        self.expand_layout_pane()
        self.wait_for_loading()

        #drag widget to dashboard
        self.drag_item_from_palette('Results', block_index=0, column_index=0)
        self.wait_for_loading()

        #select datasource, and cube
        values = {'datasource_id': ('select', '%s.FoodMart' % settings.SETTINGS_MODULE)}
        self.fill_form(panel_name='iqube-datasource-cube-panel', values= values)
        self.wait_for_loading()

        self.click_button(panel_name='iqube-datasource-cube-panel', text='Continue')
        self.wait_for_loading()

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        self.click_palette('Save as..', palette_index=0)
        self.wait_for_loading()

        # save report with a name and click 'ok' button
        self.save_dialog_input('Test me')
        self.wait_for_loading()
        self.assert_no_growl_errors()

        #icon is not shown due to no permissions
        self.assertRaises(NoSuchElementException, self.click_iconbar, 'icon-report-publication-wizard', 'icon-iqreport')
        self.wait_for_loading()

        #attach use iquest report permission and assert that report type selection page is not visible
        self._attach_permissions_to_group(self.group, 'use_iquest_report_publication')

        self.refresh_page()
        self.wait_for_loading()

        self.click_iconbar('icon-report-publication-wizard', 'icon-iqreport')
        self.wait_for_loading()

        #if user has 1 polymorphic type usable page is skipped, next page is select module
        module = self._driver.find_element_by_name('module')
        self.assertTrue(module.is_displayed(), 'Item not visible')

        #attach use iqreport report permission and assert that report type selection page is visible
        self._attach_permissions_to_group(self.group, 'use_iqreport_report_publication')

        self.refresh_page()
        self.wait_for_loading()

        self.click_iconbar('icon-report-publication-wizard', 'icon-iqreport')

        #element is found no errors
        report_type = self._driver.find_element_by_name('polymorphic_type')
        self.assertTrue(report_type.is_displayed(), 'Item not visible')

    def test_report_publication(self):

        #attach use iqreport report permission
        self._attach_permissions_to_group(self.group, 'use_iqreport_report_publication')

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # click IQ Report icon
        self.click_iconbar(title='IQReport')
        self.wait_for_loading()

        # open pallete for widgets
        self.expand_layout_pane()
        self.wait_for_loading()

        # drag widget to dashboard
        self.drag_item_from_palette('Results', block_index=0, column_index=0)
        self.wait_for_loading()

        # select datasource, and cube
        values = {'datasource_id': ('select', '%s.FoodMart' % settings.SETTINGS_MODULE)}
        self.fill_form(panel_name='iqube-datasource-cube-panel', values=values)
        self.wait_for_loading()

        self.click_button(panel_name='iqube-datasource-cube-panel', text='Continue')
        self.wait_for_loading()

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        self.click_palette('Save as..', palette_index=0)
        self.wait_for_loading()

        # save report with a name and click 'ok' button
        self.save_dialog_input('Test me')
        self.assert_no_growl_errors()

        # open report publication wizard
        self.click_iconbar('icon-report-publication-wizard', 'icon-iqreport')

        # first page should be skipped because we have only IQReport permission
        # so select a report to be published
        self.fill_form(panel_name='report-publication-wizard', values={'module': ('select', 'Test me')})
        self.click_button(panel_name='report-publication-wizard', text=u'Next »')
        self.wait_for_loading()

        # fill iqreport schedule form
        self.fill_form(panel_name='report-publication-wizard', values={'begin_date': ('input_text', '02/03/2015 00:00'),
                                                                       'label': ('input_text', 'IQReport report')})
        self.click_button(panel_name='report-publication-wizard', text=u'Next »')
        self.wait_for_loading()

        # should be  the last page, click finish
        self.click_button(panel_name='report-publication-wizard', text='Finish')
        self.assert_no_growl_errors()
