'''
Created on Sep 9, 2014

@author: rado
'''
# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase

from iqportal.iqube.compose.controllers import MdxCalculatedMemberController

class TestCase(SeleniumTestCase):
    def setUp(self):

        super(TestCase, self).setUp()
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.default_group
        self._attach_permissions_to_group(self.group, 'use_raw_mdx', 'view_dashboard', 'save_dashboard', 'create_mdx_result', 'view_mdx_result', 'use_mdx_result_on_dashboard',
                                           'view_datasource', 'item_result', 'create_report', 'view_report')

        # we create a controller to grant privs
        self.controller = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)

    def test_result_headers(self):

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        #click IQ Report icon
        self.click_iconbar(title='Home')
        self.wait_for_loading()
        widget = 'IQReport'
        #open pallete for widgets
        self.expand_layout_pane()
        self.wait_for_loading()
        self.click_palette('Mode', palette_index=0)
        self.wait_for_loading()
        self.click_palette('Compose', palette_index=0)
        self.wait_for_loading()

        self.click_palette(widget, palette_index=2)
        #drag widget to dashboard
        self.drag_item_from_palette('Results', block_index=0, column_index=0)
        self.wait_for_loading()

        #select datasource, and cube
        values = {'datasource_id': ('select', '%s.FoodMart' % settings.SETTINGS_MODULE)}
        self.fill_form(panel_name='iqube-datasource-cube-panel', values= values)
        self.wait_for_loading()

        self.click_button(panel_name='iqube-datasource-cube-panel', text='Continue')
        self.wait_for_loading()
        #open fieldset
        self.click_fieldset(panel_name='config-widget-panel', title='Store')
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'dimension_0': ('fancy-select', ['USA'])})
        self.wait_for_loading()
        #click Filter selection
        self.click_button(panel_name='config-widget-panel', text='Filter selection')
        self.wait_for_loading()

        self.click_button(panel_name='config-widget-panel', text='Deselect all')
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'dimension_0': ('fancy-select', ['CA', 'OR'])})
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'axes_0' : ('select', 'Rows')})
        self.wait_for_loading()

        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading(stable_time=2)
        # assert no full header
        self.assert_cube_table(1, ['CA'], 'table.basesoft-cubetable', assert_header=True)
        self.click_cube_table_icon('Settings')
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'result_full_headers_0': ('checkbox', True)})
        self.wait_for_loading()

        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()
        #assert full header
        self.assert_cube_table(1, ['USA, CA'], 'table.basesoft-cubetable', assert_header=True)