# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException
from basesoft.web.contrib.i18n import models as i18n_models

from iqportal.iqreport.core import models as iqreport_models
from iqportal.iqreport.compose.controllers import ComposeController
from iqportal.iqube.core.controllers import ProjectController

from django.conf import settings
from twisted.trial.unittest import SkipTest

class ComposeControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    """
    def setUp(self):
        """
        """
        #self.reset_db()
        self.connect_db()

        # create an account so we can check permission with it
        self.account = self._create_active_account()

        # get the default group of the user
        self.group = self.account.groups[0].group

        # locale
        locale = self.session.query(i18n_models.Locale
                                    ).filter_by(locale='en_GB').one()

        self.controller = ComposeController(self.session, locale, account_id=self.account.id)

        self.project_controller = ProjectController(self.session, account_id=self.account.id)
        self._attach_permissions_to_group(self.group, 'create_report', 'create_datasource', 'view_datasource', 'datasource_create_cube')

    def get_state(self):
        """
        """

        if 'iqportal.iqube.luciddb' not in settings.INSTALLED_APPS:
            raise SkipTest('luciddb disabled')

        ds_name = self._uniquestring('test_iqreport_compose')
        descr = ds_name + " descr"
        db = {"protocol": "luciddb",
              "host": "localhost",
              "user": "sa",
              "password": "",
              "port": "8034",
              "db_name": "LOCALDB.test_csv"}

        datasource = self.project_controller.save_or_update_datasource(None, ds_name, descr, ds_name, \
                        db["protocol"], db["user"], db["password"], db["host"], \
                        db["port"], db["db_name"])


        cube_name = 'blackbox_test_cube'


        return {
            "report": {
                "name": u"My Pretty Report",
                "objectName": u"Report",
                "dbID": None,
                "domID": u"iqreport-canvas-yui-gen5",

                "locales": {
                    'nl_NL': {
                        'title': u'Report Title Dutch',
                    },
                    'en_GB': {
                        'title': u'Report Title English'
                    },
                    'de_DE': {
                        'title': u'Report Title German'
                    }
                },

                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },

                "theme": {
                    "cssClass": u"iqreport-basesoft",
                    "name": u"BaseSoft",
                    "module": u"iqreport_canvas_theme_basesoft"
                },

                "children": {
                    "blocks": [

                        # First block contains only one column
                        {
                            "objectName": u"ReportBlock",
                            "pageEnd": False,
                            "dbID": None,
                            "domID": u"yui-gen42",
                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': [{
                                            "objectName":u"ReportItemResult",
                                            "dbID":None,
                                            "domID":u"yui-gen77",
                                            "mdxId": None,
                                            'title': 'my item result',
                                            'showTitle': True,
                                            'config': {
                                                'cube': cube_name,
                                                'datasource_id': datasource.id,
                                            },
                                            'appliedConfig': {},
                                            'tableConfig': {},
                                            'graphConfig': {}

                                        }]
                                    },
                                    "objectName": u"ReportBlockColumn",
                                    "dbID": None,
                                    "domID": u"yui-gen44",
                                    "width": 100
                                }],
                            }
                        },
                    ]}

            }
        }

    def assertLocales(self, obj, state):
        """
        Helps with asserting locales in an obj
        """
        count = 0 # keeps track of the number of texts in the state

        # start asserting text values
        for locale_str in state:
            for field_name in state[locale_str]:
                for locale_inst in obj.locales:
                    if locale_inst.locale.locale == locale_str:
                        self.assertEquals( getattr(locale_inst, field_name) , state[locale_str][field_name] )
                count = count + 1

        # the length of the locales should be the same as the number of text items tested
        self.assertEquals(count, len(obj.locales) )

    def test_save_new_state(self):
        """
        Test the saving of the state
        """
        state = self.get_state()

        id_map = self.controller.save_state(state)

        report = self.session.query(iqreport_models.Report
                                    ).get(id_map['iqreport-canvas-yui-gen5'])
        self.assertEqual(report.name, 'My Pretty Report')
        self.assertEquals(report.fallback_locale.locale, 'en_GB')

        self.assertLocales(report, {
            'en_GB': {'title': 'Report Title English'},
            'nl_NL': {'title': 'Report Title Dutch'},
            'de_DE': {'title': 'Report Title German'},
        })

        # blocks
        self.assertEquals(len(report.blocks), 1)

        # first block contains one column
        self.assertEquals(len(report.blocks[0].columns), 1)

        # the column contains a single item
        self.assertEquals(len(report.blocks[0].columns[0].items), 1)

        item = report.blocks[0].columns[0].items[0]
        self.assertTrue(item.result.id > 0)
