# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime

from basesoft.web.contrib.contact.controllers import ContactController

from iqportal.iqreport.compose.controllers import ComposeController
from iqportal.iqreport.core.mailing import MailController
from basesoft.web.core.util.controllers import StateManager
from basesoft.web.contrib.mailing.models import Mailing, MailResult
from basesoft.web.contrib.compose.models import ComposePublication
from iqportal.iqreport.core import schedule
from iqportal.iqube.core.controllers import ItemResultController
from iqportal.iqube.core.models import DataSource
from django.conf import settings
from iqportal.iqube.test.unit.test_configurable_widget_reporters import AbstractConfigurableWidgetReporterTestCase
from iqportal.iqube.xmla.client import XMLAClient
from iqportal.iqreport.core.controllers import ReportPublicationStateController

class TestCase(AbstractConfigurableWidgetReporterTestCase):
    """
    """

    def setUp(self):
        """
        """
        #self.reset_db()
        self.connect_db()

        #create an account so we can check permission with it
        self.account = self._create_active_account()

        # create the necessary controllers
        self.group = self.account.groups[0].group

        # initialise compose controller
        from basesoft.web.contrib.i18n import models as i18n_models
        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()
        self.compose_controller = ComposeController(self.session, self.locale, account_id=self.account.id)

        self.rpsc = ReportPublicationStateController(self.sa_session, StateManager(self.locale, self.account.id, None))

        # give current group access to create surveys
        self._attach_permissions_to_group(self.group, 'create_report', 'create_report_publication',
                                          'schedule_report_publication', 'view_report',
                                          'schedule_iqreport_mail', 'view_datasource')

        host, port = settings.MONDRIAN_XMLA_SERVER
        self.client = XMLAClient(host, port, settings.MONDRIAN_XMLA_PATH, '%s.FoodMart' % settings.SETTINGS_MODULE)
        # create some contact lists and mails
        self.setUpContactLists()
        self.setUpMails()

    def setUpContactLists(self):
        """
        Creates 3 contacts and 2 contact lists
        """
        self._attach_permissions_to_group(self.group, 'link_to_contact_list', 'create_contact', 'create_contact_list',
                                          'view_contact_list')
        contact_controller = ContactController(self.session, account_id=self.account.id)

        c1 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c1', 'email': u'testing@mail.clusternet'}, None, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c2', 'email': u'testing@mail.clusternet'}, {'name': 'Contact Company c2'}, self.group.id)
        c3 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c3', 'email': u'testing@mail.clusternet'}, None, self.group.id)

        self.cl1 = contact_controller.save_or_update_contact_list(None, 'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], self.cl1)

        self.cl2 = contact_controller.save_or_update_contact_list(None, 'Contact List 2', self.group.id)
        contact_controller.link_contacts([c2, c3], self.cl2)

    def get_mail_state(self, name):
        return {
            "mail": {
                "mail_from_text": "TestMailer",
                "name": name,
                "objectName": u"IQReportMail",
                "dbID": None,
                "locales": {
                    'nl_NL': {
                        'subject': u"iqreport mail",
                        'content': u'Test',
                    },
                    'en_GB': {
                        'subject': u"iqreport mail",
                        'content': u'Test',
                    },
                },
                "fallbackLocale": {'value': u'en_GB' },
            }
        }

    def setUpMails(self):

        self._attach_permissions_to_group(self.group, 'create_mail', 'create_iqreport_mail')
        controller = MailController(self.session, self.locale, account_id=self.account.id)
        self.mail1 = controller.save_state(self.get_mail_state(u'IQReport Mail 1'), self.group)


    def test_mailing(self):
        cube_name = 'Sales'
        datasource = self.sa_session.query(DataSource).filter(DataSource.name == '%s.FoodMart' % settings.SETTINGS_MODULE).first()
        #create result item
        result_item_state = self._create_result_item(datasource.id, cube_name)
        result_item_config = result_item_state['config']
        #add the missing axes and dimensions keys to the applied_config
        result_item_applied_config = self.build_applied_config(result_item_state['applied_filters'] )

        report_state = self._get_report_state(result_item_config, result_item_applied_config)
        # create anonymous report and publish
        self._create_report_and_publish(report_state, ComposePublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id, self.cl2.id])

        rv = schedule.PublishReportsTask().execute()
        # 1 publication set online
        self.assertEqual(len(rv), 1)

        # check that mail is created
        mailing = self.sa_session.query(Mailing).filter(Mailing.mail_id == self.mail1.id).all()
        self.assertEqual(len(mailing), 1)

        #we need invitees before we can send the scheduled mail
        schedule.CreatePublicationInviteesTask().execute()

        #generate cache to set publication status to processed
        schedule.ReportPublicationCacheTask().execute()

        #run publishing task again to send the mail and check that no other publications are set to online
        rvv = schedule.PublishReportsTask().execute()
        self.assertEqual(len(rvv), 0)
        #check that mail is sent
        mailing_result = self.sa_session.query(MailResult).filter(MailResult.mailing_id == mailing[0].id).all()
        self.assertEqual(len(mailing_result), 3)


    def _create_result_item(self, datasource_id, cube_name):

        item_result_controller = ItemResultController(self.sa_session, account_id=self.account.id)
        return item_result_controller.make_configurable_widget_config(datasource_id, cube_name)

    def _get_report_state(self, result_item_config, result_item_applied_config):

        item = [{
            'objectName': u'ReportItemDataSourceWidget',
            'loaderName': 'configurable_report_item',
            'requires': [u'level_filter_fancy_select', u'iq_fancyselect'],
            'datasource_widget_id': None,
            'title': u'Test title',
            'dbID': None,
            'domID': 'i1',
            'tableConfig': {u'slicersExpanded': False, u'isTransposed': False},
            'appliedConfig': result_item_applied_config,
            'config': result_item_config,
            'graphConfig': {},
            'showTitle': 1
        }]

        state = {
            'report': {
                'objectName': u'Report',
                'name': u'testReport',
                'fallbackLocale': {'locale': 'en_GB', 'name': 'English'},
                'dbID': None,
                'domID': u'r1',
                'availableLocales': [],
                'theme': {'cssClass': u'iqreport-basesoft', 'name': u'BaseSoft', 'module': u'iqreport_canvas_theme_basesoft', 'graph_config': None},
                'has_active_publications': False,
                "locales": {
                    'en_GB': {
                        u'label': u'Test report'
                    }
                },
                'children': {
                    'blocks': [{
                        'dbID': None,
                        'domID': 'b1',
                        'children': {
                            'columns': [{
                                'dbID': None,
                                'domID': 'c1',
                                'children': {
                                    'items': item
                                },
                                'objectName': u'ReportBlockColumn',
                                'width': None
                            }]
                        },
                        'objectName': u'ReportBlock',
                        'pageEnd': False
                    }]
                }
            }
        }
        return state

    def _create_report_and_publish(self, report_state, status, begin_date, end_date=None, contact_lists=None, anonymous=False, mailing_contact_lists=None):

        if contact_lists is None:
            contact_lists = []
        if mailing_contact_lists is None:
            mailing_contact_lists = contact_lists

        map_id = self.compose_controller.save_state(report_state, self.group.id)
        report = self.compose_controller.get_reports(map_id['r1'])[0]
        report_id = report.id

        publish_state = {
            'report_id': report_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": end_date,
            "status": status,
            "label": u"iqreport test",
            "show_page_nrs": True,
            "anonymous": anonymous,
            "contact_lists": contact_lists,
            "mailings": [{
                "dbID": None,
                "domID": u'publish_1-mailing_1',
                "mail_id": self.mail1.id,
                "send_date": begin_date,
                "contact_lists": mailing_contact_lists,
                'mail_from_text': 'mail from test',
                'status': Mailing.STATUS_SCHEDULED,
            }],
            'viewer_print': False,
            'polymorphic_type': 'ReportPublication',
        }

        # save publication
        return self.rpsc.save(publish_state)
