'''
Created on Oct 30, 2014

@author: rado
'''

from iqportal.iqreport.compose.controllers import ComposeController
from basesoft.web.core.util.controllers import StateManager
from basesoft.web.contrib.i18n import models as i18n_models
from datetime import datetime
from basesoft.web.contrib.compose.models import ComposePublication
from iqportal.iqreport.core.schedule import PublishReportsTask
from iqportal.iqreport.display.controllers import ReportDisplayController
from iqportal.iqube.core.controllers import ItemResultController
from iqportal.iqube.core.models import DataSource
from django.conf import settings
from iqportal.iqube.test.unit.test_configurable_widget_reporters import AbstractConfigurableWidgetReporterTestCase
from iqportal.iqreport.core.controllers import ReportPublicationStateController

class TestCase(AbstractConfigurableWidgetReporterTestCase):
    """
    Test non IQuest report publication
    """
    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group
        self._attach_permissions_from_role(self.group, 'IQ Report file import (advanced)')

        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()
        manager = StateManager(self.locale, self.account.id, None)

        self.report_controller = ComposeController(self.sa_session, account_id=self.account.id)
        self.rpsc = ReportPublicationStateController(self.sa_session, manager)

    def test_non_iquest_report_publication(self):
        cube_name = 'Sales'
        datasource = self.sa_session.query(DataSource).filter(DataSource.name == '%s.FoodMart' % settings.SETTINGS_MODULE).first()
        #create result item
        result_item_state = self._create_result_item(datasource.id, cube_name)
        result_item_config = result_item_state['config']
        #add the missing axes and dimensions keys to the applied_config
        result_item_applied_config = self.build_applied_config(result_item_state['applied_filters'] )

        map_id = self._create_report(result_item_applied_config, result_item_config)
        report = self.report_controller.get_reports(map_id.values())[0]
        report_id = report.id
        publish_state = {
            'report_id': report_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": datetime.now(),
            "end_date": None,
            "status": ComposePublication.STATUS_SCHEDULED,
            "label": u"publication test",
            "show_page_nrs": True,
            "anonymous": True,
            "contact_lists": [],
            "mailings": [], # no mailings necessary,
            'viewer_print': False,
            'polymorphic_type': 'ReportPublication',
        }

        # save publication
        publication = self.rpsc.save(publish_state)

        # run scheduler
        PublishReportsTask().execute()

        load_publication = self.rpsc.load(publication.id)
        if load_publication['anonymous']:
            invitee_hash = load_publication['anonymous_hash']

        self.assertEqual(load_publication['polymorphic_type'], publish_state['polymorphic_type'], 'Report type mismatch!')
        #instantiate report display controller to check if publication is online and invitee_hash is correct
        ReportDisplayController(self.sa_session, self.locale, invitee_hash=invitee_hash)

    def _create_result_item(self, datasource_id, cube_name):

        item_result_controller = ItemResultController(self.sa_session, account_id=self.account.id)
        return item_result_controller.make_configurable_widget_config(datasource_id, cube_name)

    def _create_report(self, applied_config, config):

        item = [{
            'objectName': u'ReportItemDataSourceWidget',
            'loaderName': 'configurable_report_item',
            'requires': [u'level_filter_fancy_select', u'iq_fancyselect'],
            'datasource_widget_id': None,
            'title': u'Test title',
            'dbID': None,
            'domID': 'i1',
            'tableConfig': {u'slicersExpanded': False, u'isTransposed': False},
            'appliedConfig': applied_config,
            'config': config,
            'graphConfig': {},
            'showTitle': 1
        }]

        state = {
            'report': {
                'objectName': u'Report',
                'name': u'testReport',
                'fallbackLocale': {'locale': 'en_GB', 'name': 'English'},
                'dbID': None,
                'domID': u'r1',
                'availableLocales': [],
                'theme': {'cssClass': u'iqreport-basesoft', 'name': u'BaseSoft', 'module': u'iqreport_canvas_theme_basesoft', 'graph_config': None},
                'has_active_publications': False,
                "locales": {
                    'en_GB': {
                        u'label': u'Test report'
                    }
                },
                'children': {
                    'blocks': [{
                        'dbID': None,
                        'domID': 'b1',
                        'children': {
                            'columns': [{
                                'dbID': None,
                                'domID': 'c1',
                                'children': {
                                    'items': item
                                },
                                'objectName': u'ReportBlockColumn',
                                'width': None
                            }]
                        },
                        'objectName': u'ReportBlock',
                        'pageEnd': False
                    }]
                }
            }
        }

        map_id = self.report_controller.save_state(state, self.group.id)
        return map_id