/**
 * Copyright (c) 2011 BaseSoft Solutions B.V.
 * See LICENSE for details.
 */
BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "test_cube_designer",

	setUp: function(){

		this.sandbox = YAHOO.util.Dom.get('sandbox');
		this.sandbox.innerHTML = '';

		// table state as it would come from optiq
		this._tables_state = [{
			"name": "date_dimension",
			"columns": [
				{"COLUMN_SIZE": "-1", "NULLABLE": "1", "TABLE_NAME": "date_dimension", "DATA_TYPE": "INTEGER", "COLUMN_NAME": "ID"},
				{"COLUMN_SIZE": "-1", "NULLABLE": "1", "TABLE_NAME": "date_dimension", "DATA_TYPE": "DATE", "COLUMN_NAME": "CDATE"},
				{"COLUMN_SIZE": "-1", "NULLABLE": "1", "TABLE_NAME": "date_dimension", "DATA_TYPE": "INTEGER", "COLUMN_NAME": "CYEAR"},
				{"COLUMN_SIZE": "-1", "NULLABLE": "1", "TABLE_NAME": "date_dimension", "DATA_TYPE": "INTEGER", "COLUMN_NAME": "QUARTER"},
				{"COLUMN_SIZE": "-1", "NULLABLE": "1", "TABLE_NAME": "date_dimension", "DATA_TYPE": "INTEGER", "COLUMN_NAME": "CMONTH"},
				{"COLUMN_SIZE": "-1", "NULLABLE": "1", "TABLE_NAME": "date_dimension", "DATA_TYPE": "INTEGER", "COLUMN_NAME": "WEEK_OF_YEAR"},
				{"COLUMN_SIZE": "-1", "NULLABLE": "1", "TABLE_NAME": "date_dimension", "DATA_TYPE": "INTEGER", "COLUMN_NAME": "DAY_OF_MONTH"}]
			}, {
			'name': 'FO_BB_DECL_2012',
			'columns': [
				{'COLUMN_NAME': 'EXPORT_DATE', 'DATA_TYPE': 'DATE'},
				{'COLUMN_NAME': 'EXPORT_DATE__FK', 'DATA_TYPE': 'INTEGER'},
				{'COLUMN_NAME': 'INVOICE_NR', 'DATA_TYPE': 'VARCHAR'},
				{'COLUMN_NAME': 'CRSO_NAME', 'DATA_TYPE': 'VARCHAR'},
				{'COLUMN_NAME': 'COMPANY_NAME', 'DATA_TYPE': 'VARCHAR'},
				{'COLUMN_NAME': 'VAT_NO', 'DATA_TYPE': 'VARCHAR'},
				{'COLUMN_NAME': 'PERIOD', 'DATA_TYPE': 'VARCHAR'},
				{'COLUMN_NAME': 'QUANTITY', 'DATA_TYPE': 'INTEGER'},
				{'COLUMN_NAME': 'STATUS', 'DATA_TYPE': 'VARCHAR'},
				{'COLUMN_NAME': 'DOCUMENT', 'DATA_TYPE': 'BOOLEAN'},
				{'COLUMN_NAME': 'UNLINK_DOCUMENT', 'DATA_TYPE': 'VARCHAR'},
				{'COLUMN_NAME': 'TIME', 'DATA_TYPE': 'VARCHAR'},
				{'COLUMN_NAME': 'TIME__TIME', 'DATA_TYPE': 'INTEGER'}
			]
		}];

	},

	testNewCube: function() {

		try {

			// render the cube designer
			var cd = new BASESOFT.iqube.CubeDesigner(this._tables_state);
			this._garbage.push(cd);
			cd.render(this.sandbox);

			// like dragdrop would, add a dimension
			cd.addDimension('FO_BB_DECL_2012', 'CRSO_NAME');

			// like dragdrop would do, add a measure:
			cd.addMeasure('FO_BB_DECL_2012', 'QUANTITY');

			// TODO: test dragging dimension on top of existing dimension

			// verify cube xml
			var xml = cd.getCubeXml();
			console.warn(xml);

			YAHOO.util.Assert.areEqual('<Cube name="my_cube"><Table name="FO_BB_DECL_2012"/><Dimension name="CRSO_NAME"><Hierarchy hasAll="true"><Level name="CRSO_NAME" column="CRSO_NAME" type="String" captionColumn="CRSO_NAME"/></Hierarchy></Dimension><Measure name="QUANTITY" column="QUANTITY" aggregator="sum" formatString=""/></Cube>', xml);


		} catch (e) {
			console.error(e);
			throw e;
		}

	},

	testAutofill: function() {

		try {

			// render the cube designer
			var cd = new BASESOFT.iqube.CubeDesigner(this._tables_state);
			this._garbage.push(cd);
			cd.render(this.sandbox);

			cd.setCubeName('test_cube_autofill');

			cd.autoSelect();

			// verify cube xml
			var xml = cd.getCubeXml();
			console.warn('actual', xml);

			var expected = '<Cube name="test_cube_autofill"><Table name="FO_BB_DECL_2012"/><Dimension name="UNLINK_DOCUMENT"><Hierarchy hasAll="true"><Level name="UNLINK_DOCUMENT" column="UNLINK_DOCUMENT" type="String" captionColumn="UNLINK_DOCUMENT"/></Hierarchy></Dimension><Dimension name="DOCUMENT"><Hierarchy hasAll="true"><Level name="DOCUMENT" column="DOCUMENT" type="String" captionColumn="DOCUMENT"/></Hierarchy></Dimension><Dimension name="STATUS"><Hierarchy hasAll="true"><Level name="STATUS" column="STATUS" type="String" captionColumn="STATUS"/></Hierarchy></Dimension><Dimension name="PERIOD"><Hierarchy hasAll="true"><Level name="PERIOD" column="PERIOD" type="String" captionColumn="PERIOD"/></Hierarchy></Dimension><Dimension name="VAT_NO"><Hierarchy hasAll="true"><Level name="VAT_NO" column="VAT_NO" type="String" captionColumn="VAT_NO"/></Hierarchy></Dimension><Dimension name="COMPANY_NAME"><Hierarchy hasAll="true"><Level name="COMPANY_NAME" column="COMPANY_NAME" type="String" captionColumn="COMPANY_NAME"/></Hierarchy></Dimension><Dimension name="CRSO_NAME"><Hierarchy hasAll="true"><Level name="CRSO_NAME" column="CRSO_NAME" type="String" captionColumn="CRSO_NAME"/></Hierarchy></Dimension><Dimension name="INVOICE_NR"><Hierarchy hasAll="true"><Level name="INVOICE_NR" column="INVOICE_NR" type="String" captionColumn="INVOICE_NR"/></Hierarchy></Dimension><Dimension name="EXPORT_DATE" foreignKey="EXPORT_DATE__FK"><Hierarchy hasAll="true" primaryKey="ID"><Table name="date_dimension"/><Level name="Year" column="CYEAR" uniqueMembers="true" levelType="TimeYears" type="Numeric" captionColumn="Year"/><Level name="Quarter" column="QUARTER" uniqueMembers="true" levelType="TimeQuarters" type="Numeric" captionColumn="Quarter"/><Level name="Month" column="CMONTH" uniqueMembers="true" levelType="TimeMonths" type="Numeric" captionColumn="Month"/></Hierarchy><Hierarchy name="Time Weekly" hasAll="true" primaryKey="ID"><Table name="date_dimension"/><Level name="Year" column="CYEAR" uniqueMembers="true" levelType="TimeYears" type="Numeric" captionColumn="Year"/><Level name="Week" column="WEEK_OF_YEAR" uniqueMembers="true" levelType="TimeWeeks" type="Numeric" captionColumn="Week"/><Level name="Day" column="DAY_OF_MONTH" uniqueMembers="true" levelType="TimeDays" type="Numeric" captionColumn="Day"/></Hierarchy></Dimension><Measure name="TIME__TIME" column="TIME__TIME" aggregator="sum" formatString="intToTime"/><Measure name="QUANTITY" column="QUANTITY" aggregator="sum" formatString=""/></Cube>';
			console.warn('expected', expected);

			YAHOO.util.Assert.areEqual(expected, xml);


		} catch (e) {
			console.error(e);
			throw e;
		}

	},



	testLoadCube: function () {

		try {

			// schema xml as it would come from mondrian
			var schema_xml = '\
				<Cube name="blackbox_test_cube" defaultMeasure="quantity">\
					<Table name="FO_BB_DECL_2012" />\
					<Dimension name="crso">\
						<Hierarchy hasAll="true">\
							<Level column="CRSO_NAME" name="crso_name" />\
							<Level column="COMPANY_NAME" name="company_name" />\
						</Hierarchy>\
					</Dimension>\
					<Dimension name="status">\
						<Hierarchy  hasAll="true" >\
							<Level column="STATUS" name="status" />\
						</Hierarchy>\
					</Dimension>\
					<Measure name="quantity" column="QUANTITY" aggregator="sum" />\
				</Cube>\
			';


			// render the cube designer
			var cd = new BASESOFT.iqube.CubeDesigner(this._tables_state);
			this._garbage.push(cd);
			cd.render(this.sandbox);

			// load existing schema
			cd.loadCube(schema_xml);

			// click on the first measure (quantity)
			YAHOO.util.UserAction.click(YAHOO.util.Selector.query(".cube-designer div[class~='measure']")[0]);

			// assert that on the right properties panel the aggregator is sum
			var input = YAHOO.util.Selector.query(".cube-designer input[name~='aggregator']")[0];
			YAHOO.util.Assert.areEqual(input.value, 'sum');

			// change it to 'count' (focus/unfocus for form-mager to read its value)
			input.focus();
			input.value = 'count';
			input.blur();

			YAHOO.util.UserAction.click(YAHOO.util.Selector.query(".cube-designer input[value~='Save']")[0]);

			// assert new xml
			var xml = cd.getCubeXml();
			console.warn('xml', xml);
			YAHOO.util.Assert.isTrue( xml.indexOf('aggregator="count"') !== -1 );

		} catch (e) {
			console.error(e);
			throw e;
		}

	},

	testTimeDimension: function() {

		try {

			var tables_state = [{"name": "BLE2_2012", "columns": [{"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "INVOICE_NO_", "ORDINAL_POSITION": 1, "PRECISION": 22, "IS_NULLABLE": true, "DATA_TYPE": "VARCHAR"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "CRSO_NAME", "ORDINAL_POSITION": 2, "PRECISION": 51, "IS_NULLABLE": true, "DATA_TYPE": "VARCHAR"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "COMPANY_NAME", "ORDINAL_POSITION": 3, "PRECISION": 75, "IS_NULLABLE": true, "DATA_TYPE": "VARCHAR"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "VAT_NO", "ORDINAL_POSITION": 4, "PRECISION": 16, "IS_NULLABLE": true, "DATA_TYPE": "VARCHAR"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "PERIOD", "ORDINAL_POSITION": 5, "PRECISION": 16, "IS_NULLABLE": true, "DATA_TYPE": "VARCHAR"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "QUANTITY", "ORDINAL_POSITION": 6, "PRECISION": null, "IS_NULLABLE": true, "DATA_TYPE": "INTEGER"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "STATUS", "ORDINAL_POSITION": 7, "PRECISION": 11, "IS_NULLABLE": true, "DATA_TYPE": "VARCHAR"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "DOCUMENT", "ORDINAL_POSITION": 8, "PRECISION": 5, "IS_NULLABLE": true, "DATA_TYPE": "VARCHAR"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "UNLINK_DOCUMENT", "ORDINAL_POSITION": 9, "PRECISION": 185, "IS_NULLABLE": true, "DATA_TYPE": "VARCHAR"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "EXPORT_DATE", "ORDINAL_POSITION": 10, "PRECISION": null, "IS_NULLABLE": true, "DATA_TYPE": "DATE"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "BLE2_2012", "COLUMN_NAME": "EXPORT_DATE__FK", "ORDINAL_POSITION": 11, "PRECISION": null, "IS_NULLABLE": true, "DATA_TYPE": "INTEGER"}]}, {"name": "DATE_DIMENSION", "columns": [{"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "DATE_DIMENSION", "COLUMN_NAME": "ID", "ORDINAL_POSITION": 1, "PRECISION": null, "IS_NULLABLE": true, "DATA_TYPE": "INTEGER"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "DATE_DIMENSION", "COLUMN_NAME": "CDATE", "ORDINAL_POSITION": 2, "PRECISION": null, "IS_NULLABLE": true, "DATA_TYPE": "DATE"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "DATE_DIMENSION", "COLUMN_NAME": "CYEAR", "ORDINAL_POSITION": 3, "PRECISION": null, "IS_NULLABLE": true, "DATA_TYPE": "SMALLINT"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "DATE_DIMENSION", "COLUMN_NAME": "MONTH", "ORDINAL_POSITION": 4, "PRECISION": null, "IS_NULLABLE": true, "DATA_TYPE": "SMALLINT"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "DATE_DIMENSION", "COLUMN_NAME": "DAY_OF_THE_WEEK", "ORDINAL_POSITION": 5, "PRECISION": null, "IS_NULLABLE": true, "DATA_TYPE": "SMALLINT"}, {"TABLE_TYPE": "LOCAL TABLE", "SCHEMA_NAME": "DS_47", "TABLE_NAME": "DATE_DIMENSION", "COLUMN_NAME": "QUARTER", "ORDINAL_POSITION": 6, "PRECISION": null, "IS_NULLABLE": true, "DATA_TYPE": "SMALLINT"}]}];

			var cd = new BASESOFT.iqube.CubeDesigner(tables_state);
			this._garbage.push(cd);
			cd.render(this.sandbox);

			cd.setCubeName('test_cube_autofill');

			cd.autoSelect('BLE2_2012');

			var xml = cd.getCubeXml();
			console.warn(xml);

			YAHOO.util.Assert.areEqual('<Cube name="test_cube_autofill"><Table name="BLE2_2012"/><Dimension name="EXPORT_DATE" foreignKey="EXPORT_DATE__FK"><Hierarchy hasAll="true" primaryKey="ID"><Table name="date_dimension"/><Level name="Year" column="CYEAR" uniqueMembers="true" levelType="TimeYears" type="Numeric" captionColumn="Year"/><Level name="Quarter" column="QUARTER" uniqueMembers="true" levelType="TimeQuarters" type="Numeric" captionColumn="Quarter"/><Level name="Month" column="CMONTH" uniqueMembers="true" levelType="TimeMonths" type="Numeric" captionColumn="Month"/></Hierarchy><Hierarchy name="Time Weekly" hasAll="true" primaryKey="ID"><Table name="date_dimension"/><Level name="Year" column="CYEAR" uniqueMembers="true" levelType="TimeYears" type="Numeric" captionColumn="Year"/><Level name="Week" column="WEEK_OF_YEAR" uniqueMembers="true" levelType="TimeWeeks" type="Numeric" captionColumn="Week"/><Level name="Day" column="DAY_OF_MONTH" uniqueMembers="true" levelType="TimeDays" type="Numeric" captionColumn="Day"/></Hierarchy></Dimension><Dimension name="UNLINK_DOCUMENT"><Hierarchy hasAll="true"><Level name="UNLINK_DOCUMENT" column="UNLINK_DOCUMENT" type="String" captionColumn="UNLINK_DOCUMENT"/></Hierarchy></Dimension><Dimension name="DOCUMENT"><Hierarchy hasAll="true"><Level name="DOCUMENT" column="DOCUMENT" type="String" captionColumn="DOCUMENT"/></Hierarchy></Dimension><Dimension name="STATUS"><Hierarchy hasAll="true"><Level name="STATUS" column="STATUS" type="String" captionColumn="STATUS"/></Hierarchy></Dimension><Dimension name="PERIOD"><Hierarchy hasAll="true"><Level name="PERIOD" column="PERIOD" type="String" captionColumn="PERIOD"/></Hierarchy></Dimension><Dimension name="VAT_NO"><Hierarchy hasAll="true"><Level name="VAT_NO" column="VAT_NO" type="String" captionColumn="VAT_NO"/></Hierarchy></Dimension><Dimension name="COMPANY_NAME"><Hierarchy hasAll="true"><Level name="COMPANY_NAME" column="COMPANY_NAME" type="String" captionColumn="COMPANY_NAME"/></Hierarchy></Dimension><Dimension name="CRSO_NAME"><Hierarchy hasAll="true"><Level name="CRSO_NAME" column="CRSO_NAME" type="String" captionColumn="CRSO_NAME"/></Hierarchy></Dimension><Dimension name="INVOICE_NO_"><Hierarchy hasAll="true"><Level name="INVOICE_NO_" column="INVOICE_NO_" type="String" captionColumn="INVOICE_NO_"/></Hierarchy></Dimension><Measure name="QUANTITY" column="QUANTITY" aggregator="sum" formatString=""/></Cube>', xml);



		} catch (e) {
			console.error(e);
			throw e;
		}

	}

}));
