/**
 * Copyright (c) 2012 BaseSoft B.V.
 * See LICENSE for details.
 */
BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name : "LucidDB Test",

	setUp : function() {
		var sandbox = YAHOO.util.Dom.get('sandbox');
		this.container = document.createElement('div');
		sandbox.appendChild(this.container);

		this.loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.test/selenium/yui/account/', {
			executeIfLoggedOut : true
		});
		this._garbage.push(this.loginrpc);
		var test_rpc = "/iqportal.iqube.test/luciddb/rpc/";
		this._rpc = new BASESOFT.util.JsonBroker(test_rpc, 10000);
		this._garbage.push(this._rpc);
	},


	_should: {
		ignore: {
			test_import_csv: true,
			test_import_xlsx: true,
			test_import_xlsx_old: true
		}
	},


	test_import_csv : function() {

		var perms = ['create_datasource', 'view_datasource', 'file_based_datasource_access', 'view_file', 'create_file'];
		// login first
		this.loginrpc.call('create_account_and_login', perms, function(result) {
			this.resume(function() {// continue the test after login
				BASESOFT.env.account.id = result.id;

				this.swp = new BASESOFT.iqube.luciddb.SchemaWizardPanel();
				this._garbage.push(this.swp);

				this._rpc.call("create_csv_file", [], function(result) {
					this.resume(function() {

						this.swp.file_hash = {
							file_hashes : [result]
						};

						// fill in datasource name
						BASESOFT.util.UserAction.type(YAHOO.util.Selector.query('.luciddb-schema-wizard input[name=datasource_name]')[0], 'test_luciddb_datasource_' + new Date().getTime());

						// click next when page code is executed
						var _this = this;
						setTimeout(function() {
							YAHOO.util.UserAction.click(YAHOO.util.Selector.query(".luciddb-schema-wizard button:contains(Next »)")[0]);
							_this._schema_wizard_csv_properties();
						}, 1);

						this.wait(60 * 1000);

					});
				}, null, this);
				this.wait();
			});
		}, null, this);
		this.wait();
		// wait for login
	},

	_schema_wizard_csv_properties : function() {

		// wait for preview table to have 10 rows
		BASESOFT.util.waitForCondition(function() {
			return YAHOO.util.Selector.query(".luciddb-schema-wizard .bs-customdatatable.preview-table tbody.yui-dt-data tr").length === 11;
		}, function() {
			this.resume(function() {

				// now in the csv properties page.. but not changing anything
                // check column order is consistent
                var headers =
                        YAHOO.util.Selector.query(".luciddb-schema-wizard .bs-customdatatable.preview-table thead tr th div span");
                var expected = ["COMPANY", "PERSON", "TURNOVER", "QUANTITY"];
                YAHOO.util.Assert.areEqual(4, headers.length);
                for(var i = 0; i < headers.length; i++) {
                    YAHOO.util.Assert.areEqual(expected[i], headers[i].innerHTML);
                }

				// click next
				YAHOO.util.UserAction.click(YAHOO.util.Selector.query(".luciddb-schema-wizard button:contains(Next »)")[0]);

				// wait for next page
				this._schema_wizard_csv_column_properties();
				this.wait(60 * 1000);

			});
		}, null, this);

	},

	_schema_wizard_csv_column_properties : function() {

		// wait for columns table to have 4 rows
		BASESOFT.util.waitForCondition(function() {
			return YAHOO.util.Selector.query(".luciddb-schema-wizard .bs-customdatatable.columns-table tbody.yui-dt-data tr").length === 4;
		}, function() {
			this.resume(function() {

				// now in the column properties page..

				// change the quantity column to be integer instead of tinyint
				var el = YAHOO.util.Selector.query(".luciddb-schema-wizard select[class~=yui-dt-dropdown]")[3];
				BASESOFT.util.UserAction.select(el, 'INTEGER');

				// click next
				YAHOO.util.UserAction.click(YAHOO.util.Selector.query(".luciddb-schema-wizard button:contains(Next »)")[0]);

				this._schema_wizard_csv_cube_builder();
				this.wait(60 * 1000);

			});
		}, null, this);
	},

	_schema_wizard_csv_cube_builder : function() {

		BASESOFT.util.waitForCondition(function() {
			return YAHOO.util.Selector.query(".luciddb-schema-wizard .cube-designer").length === 1;
		}, function() {
			this.resume(function() {

				// click finish
				YAHOO.util.UserAction.click(YAHOO.util.Selector.query(".luciddb-schema-wizard button:contains(Finish)")[0]);

				// wait for panel to be gone
				BASESOFT.util.waitForCondition(function() {
					return YAHOO.util.Selector.query(".luciddb-schema-wizard").length === 0;
				}, function() {

					this.resume(function() {
						console.info('done!');
					});

				}, null, this);

				this.wait();

			});

		}, null, this);

	},

	/**
	 * test xlsx
	 */

	test_import_xlsx : function() {

		var perms = ['create_datasource', 'view_datasource', 'file_based_datasource_access', 'view_file', 'create_file'];
		// login first
		this.loginrpc.call('create_account_and_login', perms, function(result) {
			this.resume(function() {// continue the test after login
				BASESOFT.env.account.id = result.id;

				this.swp = new BASESOFT.iqube.luciddb.SchemaWizardPanel();
				this._garbage.push(this.swp);

				this._rpc.call("create_xlsx_file", [], function(result) {
					this.resume(function() {

						this.swp.file_hash = {
							file_hashes : [result]
						};

						// fill in datasource name
						BASESOFT.util.UserAction.type(YAHOO.util.Selector.query('.luciddb-schema-wizard input[name=datasource_name]')[0], 'test_luciddb_datasource_' + new Date().getTime());

						// click next when page code is executed
						var _this = this;
						setTimeout(function() {
							YAHOO.util.UserAction.click(YAHOO.util.Selector.query(".luciddb-schema-wizard button:contains(Next »)")[0]);
							_this._schema_wizard_xls_column_properties();
						}, 1);

						this.wait(60 * 1000);

					});
				}, null, this);
				this.wait();
			});
		}, null, this);
		this.wait();
		// wait for login
	},

	_schema_wizard_xls_column_properties : function() {

		// wait for columns table to have 4 rows
		BASESOFT.util.waitForCondition(function() {
			return YAHOO.util.Selector.query(".luciddb-schema-wizard .bs-customdatatable.columns-table tbody.yui-dt-data tr").length === 4;
		}, function() {
			this.resume(function() {

				// now in the column properties page..

				// change the quantity column to be integer
				var el = YAHOO.util.Selector.query(".luciddb-schema-wizard select[class~=yui-dt-dropdown]")[3];
				BASESOFT.util.UserAction.select(el, 'INTEGER');

				// click next
				YAHOO.util.UserAction.click(YAHOO.util.Selector.query(".luciddb-schema-wizard button:contains(Next »)")[0]);

				this._schema_wizard_xlsx_cube_builder();
				this.wait(60 * 1000);

			});
		}, null, this);
	},

	_schema_wizard_xlsx_cube_builder : function() {

		BASESOFT.util.waitForCondition(function() {
			return YAHOO.util.Selector.query(".luciddb-schema-wizard .cube-designer").length === 1;
		}, function() {
			this.resume(function() {

				// click finish
				YAHOO.util.UserAction.click(YAHOO.util.Selector.query(".luciddb-schema-wizard button:contains(Finish)")[0]);

				// wait for panel to be gone
				BASESOFT.util.waitForCondition(function() {
					return YAHOO.util.Selector.query(".luciddb-schema-wizard").length === 0;
				}, function() {

					this.resume(function() {
						console.info('done!');
					});

				}, null, this);

				this.wait();

			});

		}, null, this);

	},

	/**
	 * test xls (old excel format)
	 */

	test_import_xls_old : function() {

		var perms = ['create_datasource', 'view_datasource', 'file_based_datasource_access', 'view_file', 'create_file'];

		// login first
		this.loginrpc.call('create_account_and_login', perms, function(result) {
			this.resume(function() {// continue the test after login
				BASESOFT.env.account.id = result.id;

				this.swp = new BASESOFT.iqube.luciddb.SchemaWizardPanel();
				this._garbage.push(this.swp);

				this._rpc.call("create_xls_file", [], function(result) {
					this.resume(function() {

						this.swp.file_hash = {
							file_hashes : [result]
						};

						// fill in datasource name
						BASESOFT.util.UserAction.type(YAHOO.util.Selector.query('.luciddb-schema-wizard input[name=datasource_name]')[0], 'test_luciddb_datasource_' + new Date().getTime());

						// click next when page code is executed
						var _this = this;
						setTimeout(function() {

							_this.resume(function() {

								// click next
								YAHOO.util.UserAction.click(YAHOO.util.Selector.query(".luciddb-schema-wizard button:contains(Next »)")[0]);

								// wait for preview table to have 10 rows
								BASESOFT.util.waitForCondition(function() {
									return YAHOO.util.Selector.query(".luciddb-schema-wizard .bs-customdatatable.columns-table tbody.yui-dt-data tr").length === 4;
								}, function() {
									this.resume(function() {

										// from here it should work the same as the xlsx so stop here
										console.info('done');

									});
								}, null, this);

								this.wait(60 * 1000);
							});

						}, 1);

						this.wait(60 * 1000);

					});
				}, null, this);
				this.wait();
			});
		}, null, this);
		this.wait();
		// wait for login
	}
}));
