'''
Created on Oct 21, 2014

@author: rado
'''
from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.action_chains import ActionChains
from django.conf import settings
import time

class TestCase(SeleniumTestCase):

    def setUp(self):
        super(TestCase, self).setUp()
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.default_group
        self._attach_permissions_to_group(self.group, 'use_raw_mdx', 'view_dashboard', 'save_dashboard', 'view_datasource', 'item_result')

    def test_drill_options_only_view_mode(self):

        # we login
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        #open pallete for widgets
        self.expand_layout_pane()
        self.wait_for_loading()
        self.click_palette('Mode', 0)
        self.wait_for_loading()
        self.click_palette('Compose', 0)
        self.wait_for_loading()

        self.click_palette('IQReport', 2)
        #drag widget to dashboard
        self.drag_item_from_palette('Results', block_index=0, column_index=0)
        self.wait_for_loading()

        #select datasource, and cube
        values = {'datasource_id': ('select', '%s.FoodMart' % settings.SETTINGS_MODULE)}
        self.fill_form(panel_name='iqube-datasource-cube-panel', values= values)
        self.wait_for_loading()

        self.click_button(panel_name='iqube-datasource-cube-panel', text='Continue')
        self.wait_for_loading()

        #open fieldset
        self.click_fieldset(panel_name='config-widget-panel', title='Quick filter settings')
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'quick_filter_dimension': ('select', 'Customers')})
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'quick_filter_options': ('fancy-select', ['USA', 'Canada', 'Mexico'])})
        self.wait_for_loading()
        self.click_button(panel_name='config-widget-panel', text='Add quick filter')

        #open fieldset
        self.click_fieldset(panel_name='config-widget-panel', title='Product')
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'dimension_4': ('fancy-select', ['Drink', 'Food', 'Non-Consumable'])})
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'axes_4': ('select', 'Rows')})
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        #check that drill options and quick filters are disabled in compose mode
        self.assertRaises(NoSuchElementException, self._driver.find_element_by_css_selector, 'table.basesoft-cubetable tr:nth-child(2) th .drill.position-expand')

        button = self.find_element_by_css_contains('div.quick-filter button', 'USA')
        self.assertTrue(button.get_attribute('disabled'), 'Fail! Button is enabled!')

        #check that drill options and quick filters are enabled in view mode
        self.click_palette('View', 0)

        self.wait_for_selector('table.basesoft-cubetable tr:nth-child(2) th .drill.position-expand')
        drill_down = self._driver.find_element_by_css_selector('table.basesoft-cubetable tr:nth-child(2) th .drill.position-expand')
        self.assertEqual(drill_down.get_attribute('title'), 'Drill Down')

        button1 = self.find_element_by_css_contains('div.quick-filter button', 'USA')
        self.assertEqual(button1.get_attribute('disabled'), None)
