# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.
import os

from sqlalchemy import func
from sqlalchemy.sql.expression import and_

from django.utils import simplejson

from basesoft.web.test.unit import iqportal_unittest

from iqportal.iqube.core.models import DataSource
from iqportal.iqube.core.reporters import ReporterRpcHandler
from iqportal.iqube.core.controllers import ConfigurableWidgetController
from iqportal.iqube.core.db_sync import AbstractConfigurableWidget


class AbstractConfigurableWidgetReporterTestCase(iqportal_unittest.IQPortalTestCase):
    DATASOURCE = None

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        pass

    def set_datasource(self):

        ds = self.DATASOURCE

        datasource = self.sa_session.query(DataSource).filter(func.substr(DataSource.name, -(len(ds))) == ds).all()
        if len(datasource) == 0:
            raise Exception('No %s datasource' % ds)
        self.datasource = datasource[0]
        self.client = self.datasource.connect(account_id=self.account.id)

    def _get_configurable_widget_defaults(self, label, **kwargs):

        cwc = ConfigurableWidgetController(self.sa_session, account_id=self.account.id)
        widget = cwc.get_configurable_widget_by_label(label).pop()
        config, applied_config = cwc.get_state(widget)
        return (ReporterRpcHandler(self.sa_session, self.datasource, config, account_id=self.account.id, **kwargs), applied_config, )

    def _get_mdx(self, mdx_config):

        rv = []
        for i, conf in mdx_config.iteritems():
            mdx = conf['mdx_config']
            if mdx.has_key('context'):
                self.client.set_context(mdx['context'])
                del mdx['context']

            if mdx.has_key('run_mode'):
                del mdx['run_mode']
            if mdx.has_key('result_full_headers'):
                del mdx['result_full_headers']

            rv.append(self.client.build_select_statement(**mdx))

        return rv

    def _reset_config(self, applied_config):
        for bitbucket, index in applied_config['dimensions'].iteritems():
            applied_config['dimension_%s' % index] = [simplejson.loads(x) for x in applied_config['dimension_%s' % index]]
        return applied_config

    def _smorgle_applied_config(self, applied_config, dimensions, cube):
        dims = applied_config['dimensions']
        for d, config in dimensions.iteritems():
            applied_config['dimension_%s' % dims[d]] = config

        acw = AbstractConfigurableWidget(self.sa_session)
        acw.modify_filters(applied_config, self.client, cube)

        fields = ['basket_analysis_questions', 'basket_analysis_answers', 'basket_analysis_qa']
        for i in fields:
            if not applied_config.has_key(i) or applied_config[i] is None:
                continue
            applied_config[i] = [acw.get_modified_filter(x) for x in applied_config[i]]

        return applied_config

    def build_applied_config(self, applied_config):
        """
        Used to add the missing axes and dimensions in the basic applied_config of a Result item
        """
        dims = applied_config['dimensions']
        for d, number in dims.iteritems():
            if not applied_config.has_key('dimension_%s' % number):
                applied_config['dimension_%s' % number] = []
            if not applied_config.has_key('axes_%s' % number):
                applied_config['axes_%s' % number] = ''
        return applied_config

