# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from basesoft.web.test.unit import iqportal_unittest
from iqportal.iqube.xmla.mdx import MdxBuilder
from iqportal.iqube.xmla.client import XMLAClient
from iqportal.iqube.xmla.result import MdxResult

from twisted.trial.unittest import SkipTest

class MdxBuilderTestCase(iqportal_unittest.IQPortalTestCase):


    def setUp(self):
        host, port = settings.MONDRIAN_XMLA_SERVER
        self.client = XMLAClient(host, port, settings.MONDRIAN_XMLA_PATH, '%s.FoodMart' % settings.SETTINGS_MODULE)
        if not self.client.test_connection():
            raise SkipTest('No Mondrian connection available')

    def test_build_select_statement(self):
        cube = 'Sales'
        columns = [{
            "member_unique_name":"[Measures].[Unit Sales]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0"
        }, {
            "member_unique_name":"[Measures].[Store Cost]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0"
        }]
        rows = [{
            "member_unique_name":"[Product].[All Products]",
            "hierarchy_unique_name":"[Product]",
            "depth": "0"
        }, {
            "member_unique_name":"[Product].[Food]",
            "hierarchy_unique_name":"[Product]",
            "depth": "0"
        }, {
            "member_unique_name":"[Product].[Drink]",
            "hierarchy_unique_name":"[Product]",
            "depth": "0"
        }]
        slicers = [{
            "member_unique_name":"[Time].[1997]",
            "hierarchy_unique_name":"[Time]",
            "depth": "0"
        }]


        builder = MdxBuilder(cube, columns, rows, slicers)
        mdx = builder.get_result()
        result = self.client.execute_mdx(mdx)

        # fixme: result is now a xml tag it seems.. do we need to assert that
        #self.assertIsInstance(result, MdxResult)

    def test_build_crossjoin_select_statement(self):
        cube = 'Sales'
        columns = [{
            "member_unique_name":"[Measures].[Unit Sales]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0",
        }, {
            "member_unique_name":"[Measures].[Store Cost]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0",
        }]
        rows = [{
            "member_unique_name":"[Promotion Media].[All Media]",
            "hierarchy_unique_name":"[Promotion Media]",
            "depth": "0",
        }, {
            "member_unique_name":"[Product].[All Products]",
            "hierarchy_unique_name":"[Product]",
            "depth": "0",
        }, {
            "member_unique_name":"[Product].[Food]",
            "hierarchy_unique_name":"[Product]",
            "depth": "1",
        }, {
            "member_unique_name":"[Product].[Drink]",
            "hierarchy_unique_name":"[Product]",
            "depth": "1",
        }]
        slicers = [{
            "member_unique_name":"[Time].[1997]",
            "hierarchy_unique_name":"[Time]",
            "depth": "0",
        }]


        builder = MdxBuilder(cube, columns, rows, slicers)
        mdx = builder.get_result()
        result = self.client.execute_mdx(mdx)
        # FIXME: result is not what it seems to be.. fix needed?
        #self.assertIsInstance(result, MdxResult)

    def test_sorting(self):
        cube = 'Sales'
        columns = [{
            "member_unique_name":"[Measures].[Unit Sales]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0",
        }, {
            "member_unique_name":"[Measures].[Store Cost]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0",
        }]
        rows = [{
            "member_unique_name":"[Promotion Media].[All Media]",
            "hierarchy_unique_name":"[Promotion Media]",
            "depth": "0",
        }, {
            "member_unique_name":"[Product].[All Products]",
            "hierarchy_unique_name":"[Product]",
            "depth": "0",
        }, {
            "member_unique_name":"[Product].[Food]",
            "hierarchy_unique_name":"[Product]",
            "depth": "1",
        }, {
            "member_unique_name":"[Product].[Drink]",
            "hierarchy_unique_name":"[Product]",
            "depth": "1",
        }]
        slicers = [{
            "member_unique_name":"[Time].[1997]",
            "hierarchy_unique_name":"[Time]",
            "depth": "0",
        }]
        sort = ({
            "member_unique_name":"[Measures].[Store Cost]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0",
        }, 'ASC')
        builder = MdxBuilder(cube, columns, rows, slicers, sort=sort)
        mdx = builder.get_result()
        self.assertMdx(mdx, """
            SELECT {[Measures].[Unit Sales],[Measures].[Store Cost]} ON COLUMNS,
                ORDER(CROSSJOIN({[Promotion Media].[All Media]}, HIERARCHIZE({[Product].[Food],[Product].[Drink],[Product].[All Products]})), [Measures].[Store Cost], ASC) ON ROWS
            FROM [Sales]
            WHERE {[Time].[1997]}""")

    def test_two_equal_members_on_axis(self):
        cube = 'Sales'
        columns = [{
            "member_unique_name":"[Measures].[Store Cost]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0",
        }, {
            "member_unique_name":"[Measures].[Unit Sales]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0",
        }, {
            "member_unique_name":"[Measures].[Store Cost]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0",
        }]
        rows = [{
            "member_unique_name":"[Store].Children",
            "hierarchy_unique_name":"[Store]",
            "depth": "0",
        }, {
            "member_unique_name":"[Store].[USA].[WA]",
            "hierarchy_unique_name":"[Store]",
            "depth": "0",
        }]
        slicers = []

        builder = MdxBuilder(cube, columns, rows, slicers)
        builder.get_result()
        #result = self.client.execute_mdx(mdx)

        #self.assertEqual(len(result.fetch_headers()['columns']), 3)

        mdx = """
            SELECT CROSSJOIN({[Measures].[Store Cost],[Measures].[Unit Sales],[Measures].[Store Cost],[Measures].[Store Cost]}, {[Store].Children}) ON COLUMNS,
                    {[Promotion Media].[All Media]} ON ROWS
                FROM [Sales]
        """
        result = self.client.execute_mdx(mdx)
        #print result.fetch_headers()

    def test_leveled_statement(self):
        cube = 'Sales'
        columns = [{u'dimension_unique_name': u'[Customers]', u'member_ordinal': u'0', u'children_cardinality': u'3', u'cube_name': u'Sales', u'parent_count': u'0', u'parent_level': u'0', u'member_unique_name': u'[Customers].Children', u'catalog_name': u'FoodMart', u'member_type': u'2', u'level_number': u'0', u'schema_name': u'FoodMart', u'member_name': u'All Customers', u'hierarchy_unique_name': u'[Customers]', u'level_unique_name': u'[Customers].[(All)]', u'depth': u'0'}, {u'dimension_unique_name': u'[Gender]', u'member_ordinal': u'0', u'children_cardinality': u'2', u'cube_name': u'Sales', u'parent_count': u'0', u'parent_level': u'0', u'member_unique_name': u'[Gender].Children', u'catalog_name': u'FoodMart', u'member_type': u'2', u'level_number': u'0', u'schema_name': u'FoodMart', u'member_name': u'All Gender', u'hierarchy_unique_name': u'[Gender]', u'level_unique_name': u'[Gender].[(All)]', u'depth': u'0'}]
        rows = [{u'dimension_unique_name': u'[Product]', u'member_ordinal': u'0', u'children_cardinality': u'3', u'cube_name': u'Sales', u'parent_count': u'0', u'parent_level': u'0', u'member_unique_name': u'[Product].Children', u'catalog_name': u'FoodMart', u'member_type': u'2', u'level_number': u'0', u'schema_name': u'FoodMart', u'member_name': u'All Products', u'hierarchy_unique_name': u'[Product]', u'level_unique_name': u'[Product].[(All)]', u'depth': u'0'}]
        slicers = [{u'dimension_unique_name': u'[Measures]', u'member_ordinal': u'0', u'children_cardinality': u'0', u'cube_name': u'Sales', u'parent_count': u'0', u'parent_level': u'0', u'member_unique_name': u'[Measures].[Unit Sales]', u'catalog_name': u'FoodMart', u'member_type': u'3', u'level_number': u'0', u'schema_name': u'FoodMart', u'member_name': u'Unit Sales', u'hierarchy_unique_name': u'[Measures]', u'level_unique_name': u'[Measures].[MeasuresLevel]', u'depth': u'0'}]

        builder = MdxBuilder(cube, columns, rows, slicers)
        mdx = builder.get_result()
        result = self.client.execute_mdx(mdx)

        raise SkipTest('fixme: this works different now')

        expected_column_headers = [{
            'dimension': 'Customers',
            'indent': 0,
            'key': '[Customers].[Canada]',
            'key_list': ['[Customers].[Canada]'],
            'uname': '[Customers].[Canada]',
            'label': 'Canada',
            'level': '[Customers].[Country]',
            'children': [{
                'dimension': 'Gender',
                'uname': '[Gender].[F]',
                'indent': 0,
                'key': '[Customers].[Canada][Gender].[F]',
                'key_list': ['[Customers].[Canada]', '[Gender].[F]'],
                'label': 'F',
                'level': '[Gender].[Gender]'
            }, {
               'dimension': 'Gender',
               'indent': 0,
               'uname': '[Gender].[F]',
               'key': '[Customers].[Canada][Gender].[M]',
               'key_list': ['[Customers].[Canada]', '[Gender].[M]'],
               'uname': '[Gender].[M]',
               'label': 'M',
               'level': '[Gender].[Gender]'
            }],
        }, {
            'dimension': 'Customers',
            'indent': 0,
            'uname': '[Customers].[Mexico]',
            'key': '[Customers].[Mexico]',
            'key_list': ['[Customers].[Mexico]'],
            'label': 'Mexico',
            'level': '[Customers].[Country]',
            'children': [{
                'dimension': 'Gender',
                'uname': '[Gender].[F]',
                'indent': 0,
                'key': '[Customers].[Mexico][Gender].[F]',
                'key_list': ['[Customers].[Mexico]', '[Gender].[F]'],
                'label': 'F',
                'level': '[Gender].[Gender]'
            },
            {
                'dimension': 'Gender',
                'uname': '[Gender].[F]',
                'indent': 0,
                'uname': '[Gender].[M]',
                'key': '[Customers].[Mexico][Gender].[M]',
                'key_list': ['[Customers].[Mexico]', '[Gender].[M]'],
                'label': 'M',
                'level': '[Gender].[Gender]'
            }],
        }, {
            'dimension': 'Customers',
            'indent': 0,
            'uname': '[Customers].[USA]',
            'key': '[Customers].[USA]',
            'key_list': ['[Customers].[USA]'],
            'label': 'USA',
            'level': '[Customers].[Country]',
            'children': [{
                'dimension': 'Gender',
                'indent': 0,
                'uname': '[Gender].[F]',
                'key': '[Customers].[USA][Gender].[F]',
                'key_list': ['[Customers].[USA]', '[Gender].[F]'],
                'label': 'F',
                'level': '[Gender].[Gender]'
            }, {
                'dimension': 'Gender',
                'indent': 0,
                'uname': '[Gender].[M]',
                'key': '[Customers].[USA][Gender].[M]',
                'key_list': ['[Customers].[USA]', '[Gender].[M]'],
                'label': 'M',
                'level': '[Gender].[Gender]'
            }],
        }]

        self.assertEqual(expected_column_headers, result.fetch_headers()['columns'])

    def test_indent(self):

        # 2 dimensions: time.1997.children and store.children
        # and q2 and q3.mexico are drilled down
        mdx = """
            SELECT
                {[Measures].[Unit Sales]} ON COLUMNS,
                HIERARCHIZE(
                  UNION(
                    CROSSJOIN(HIERARCHIZE(
                      {[Time].[1997].[Q2].Children,[Time].[1997].Children}),
                      {[Store].[All Stores].Children}),
                    CROSSJOIN(
                      {[Time].[1997].[Q3]},
                      {[Store].[Mexico].Children}
                ))) ON ROWS
            FROM [Sales]
        """

        result = self.client.execute_mdx(mdx)
        rows = result.fetch_headers()['rows']

        raise SkipTest('fixme: this works different now')

        self.assertEqual(rows[0]['label'], 'Q1')
        self.assertEqual(rows[0]['indent'], 0)
        self.assertEqual(len(rows[0]['children']), 3)

        self.assertEqual(rows[1]['label'], 'Q2')
        self.assertEqual(rows[1]['indent'], 0)
        self.assertEqual(len(rows[1]['children']), 3)

        self.assertEqual(rows[2]['label'], '4')
        self.assertEqual(rows[2]['indent'], 1)
        self.assertEqual(len(rows[2]['children']), 3)

        self.assertEqual(rows[3]['label'], '5')
        self.assertEqual(rows[3]['indent'], 1)
        self.assertEqual(len(rows[3]['children']), 3)

        self.assertEqual(rows[4]['label'], '6')
        self.assertEqual(rows[4]['indent'], 1)
        self.assertEqual(len(rows[3]['children']), 3)

        self.assertEqual(rows[5]['label'], 'Q3')
        self.assertEqual(rows[5]['indent'], 0)
        self.assertEqual(len(rows[5]['children']), 9)

        self.assertEqual(rows[6]['label'], 'Q4')
        self.assertEqual(rows[6]['indent'], 0)
        self.assertEqual(len(rows[3]['children']), 3)

        # assert q3 children (len 9)
        q3 = rows[5]['children']

        self.assertEqual(q3[0]['label'], 'Canada')
        self.assertEqual(q3[0]['indent'], 0)

        self.assertEqual(q3[1]['label'], 'Mexico')
        self.assertEqual(q3[1]['indent'], 0)

        self.assertEqual(q3[2]['label'], 'DF')
        self.assertEqual(q3[2]['indent'], 1)

        self.assertEqual(q3[3]['label'], 'Guerrero')
        self.assertEqual(q3[3]['indent'], 1)

        self.assertEqual(q3[4]['label'], 'Jalisco')
        self.assertEqual(q3[4]['indent'], 1)

        self.assertEqual(q3[5]['label'], 'Veracruz')
        self.assertEqual(q3[5]['indent'], 1)

        self.assertEqual(q3[6]['label'], 'Yucatan')
        self.assertEqual(q3[6]['indent'], 1)

        self.assertEqual(q3[7]['label'], 'Zacatecas')
        self.assertEqual(q3[7]['indent'], 1)

        self.assertEqual(q3[8]['label'], 'USA')
        self.assertEqual(q3[8]['indent'], 0)


        # this tests indent for bigger member lnums
        mdx = """
            SELECT
                {[Measures].[Unit Sales]} ON COLUMNS,
                {[Store].[Mexico].Children} ON ROWS
             FROM [Sales]
            """

        result = self.client.execute_mdx(mdx)
        rows = result.fetch_headers()['rows']

        self.assertEqual(rows[0]['label'], 'DF')
        self.assertEqual(rows[0]['indent'], 0)

    def test_drilldown_usa(self):
        cube = 'Sales'

        rows = [[{
                "member_unique_name":"[Measures].[Unit Sales]",
                "hierarchy_unique_name":"[Measures]",
                "depth": "0",
            }]]

        columns = [{
            "member_unique_name":"[Store].Children",
            "hierarchy_unique_name":"[Store]",
            "depth": "0",
        }, {
            "member_unique_name":"[Store].[USA].Children",
            "hierarchy_unique_name":"[Store]",
            "depth": "0",
        }]

        builder = MdxBuilder(cube, columns, rows, slicers=[])
        mdx = builder.get_result()

        self.assertMdx(mdx, """
            SELECT {[Store].Children,[Store].[USA].Children} ON COLUMNS,
                HIERARCHIZE({[Measures].[Unit Sales]}) ON ROWS
            FROM [Sales]
            """)

    def test_drilldown_gender_usa(self):

        cube = 'Sales'

        rows = [[{
                "member_unique_name":"[Measures].[Unit Sales]",
                "hierarchy_unique_name":"[Measures]",
                "depth": "0",
            }]]

        columns = [{
            "member_unique_name":"[Gender].Children",
            "hierarchy_unique_name":"[Gender]",
            "depth": "0",
        }, {
            "member_unique_name":"[Store].Children",
            "hierarchy_unique_name":"[Store]",
            "depth": "0",
        }, [{
            "member_unique_name":"[Gender].[M]",
            "hierarchy_unique_name":"[Gender]",
            "depth": "0",
        }, {
            "member_unique_name":"[Store].[USA].Children",
                "hierarchy_unique_name":"[Store]",
                "depth": "0",
            }
        ]]

        builder = MdxBuilder(cube, columns, rows, slicers=[])
        mdx = builder.get_result()

        self.assertMdx(mdx, """
            SELECT HIERARCHIZE(UNION(CROSSJOIN({[Gender].Children}, {[Store].Children}), CROSSJOIN({[Gender].[M]}, {[Store].[USA].Children}))) ON COLUMNS,
                HIERARCHIZE({[Measures].[Unit Sales]}) ON ROWS
            FROM [Sales]
            """)

    def test_slicers_cm(self):
        cube = "TntCLMResponse"
        rows = [{"hierarchy_unique_name": "[Wave]", "depth": "0", "member_unique_name": "[Wave].[Main Waves].Children"}]
        slicers = [{"hierarchy_unique_name": "[Service]", "depth": "0", "member_unique_name": "[Service].[Express]"},
                   {"hierarchy_unique_name": "[Position]", "depth": "0", "member_unique_name": "[Position].[Overall Satisfaction]"},
                   {"hierarchy_unique_name": "[Country]", "depth": "0", "member_unique_name": "[Country].[AR]"},
                   {"hierarchy_unique_name": "[Country]", "depth": "0", "member_unique_name": "[Country].[BR]"},
                   {"hierarchy_unique_name": "[Country]", "depth": "0", "member_unique_name": "[Country].[CA]"},
                   {"hierarchy_unique_name": "[Country]", "depth": "0", "member_unique_name": "[Country].[CL]"},
                   {"hierarchy_unique_name": "[Country]", "depth": "0", "member_unique_name": "[Country].[US]"}]
        columns = [{"hierarchy_unique_name": "[KPI]", "depth": "0", "member_unique_name": "[KPI].[Exceeding Cust. Exp.]"},
                   {"hierarchy_unique_name": "[KPI]", "depth": "0", "member_unique_name": "[KPI].[NCS Score]"},
                   {"hierarchy_unique_name": "[KPI]", "depth": "0", "member_unique_name": "[KPI].[Dissatisfied]"}]

        builder = MdxBuilder(cube, columns, rows, slicers=slicers, nonempty=True)
        builder.get_result()

    def test_drilldown_complex(self):

        cube = 'Sales'

        columns = [[{
                "member_unique_name":"[Product].Children",
                "hierarchy_unique_name":"[Product]",
                "depth": "0",
            }, {
                "member_unique_name":"[Time].[1997].Children",
                "hierarchy_unique_name":"[Time]",
                "depth": "0",
            }, {
                "member_unique_name":"[Store].Children",
                "hierarchy_unique_name":"[Store]",
                "depth": "0",
            }],
            [{
                "member_unique_name":"[Product].[Food]",
                "hierarchy_unique_name":"[Product]",
                "depth": "1",
            }, {
                "member_unique_name":"[Time].[1997].[Q2].Children",
                "hierarchy_unique_name":"[Time]",
                "depth": "1",
            }, {
                "member_unique_name":"[Store].Children",
                "hierarchy_unique_name":"[Store]",
                "depth": "0",
            }],
            [{
                "member_unique_name":"[Product].[Non-Consumable]",
                "hierarchy_unique_name":"[Product]",
                "depth": "1",
            }, {
                "member_unique_name":"[Time].[1997].[Q3].Children",
                "hierarchy_unique_name":"[Time]",
                "depth": "1",
            }, {
                "member_unique_name":"[Store].Children",
                "hierarchy_unique_name":"[Store]",
                "depth": "0",
            }],
        ]

        rows = [{
            "member_unique_name":"[Measures].[Unit Sales]",
            "hierarchy_unique_name":"[Measures]",
            "depth": "0",
        }]

        builder = MdxBuilder(cube, columns, rows, slicers=[], nonempty=True)
        mdx = builder.get_result()

        # execute it (shouldn't error)
        self.client.execute_mdx(mdx)

        self.assertMdx(mdx, """
            SELECT NON EMPTY HIERARCHIZE(UNION(UNION(CROSSJOIN(CROSSJOIN({[Product].Children}, {[Time].[1997].Children}), {[Store].Children}), CROSSJOIN(CROSSJOIN({[Product].[Food]}, {[Time].[1997].[Q2].Children}), {[Store].Children})), CROSSJOIN(CROSSJOIN({[Product].[Non-Consumable]}, {[Time].[1997].[Q3].Children}), {[Store].Children}))) ON COLUMNS,
                NON EMPTY {[Measures].[Unit Sales]} ON ROWS
            FROM [Sales]
            """)

    def test_build_nested_mdx_function(self):
        cube = 'Sales'

        rows = [{
            "member_unique_name": "TOPCOUNT({[Products].[Software].Children, [Products].[Hardware].Children}, 5, TOPCOUNT({[Sales].[Europe].Children, [Sales].[America].Children}, 5, [Measures].[Response]))",
            "hierarchy_unique_name": "",
            "depth": "0"
        }]

        columns = [{
            "member_unique_name": "[Company].Children",
            "hierarchy_unique_name": "[Company]",
            "depth": "0"
        }]

        slicers = [{
            "member_unique_name":"[Time].[1997]",
            "hierarchy_unique_name":"[Time]",
            "depth": "0",
        }]

        builder = MdxBuilder(cube, columns, rows, slicers=slicers)
        mdx = builder.get_result()

        self.assertMdx(mdx, """
            SELECT {[Company].Children} ON COLUMNS,
                {TOPCOUNT({[Products].[Software].Children, [Products].[Hardware].Children}, 5, TOPCOUNT({[Sales].[Europe].Children, [Sales].[America].Children}, 5, [Measures].[Response]))} ON ROWS
            FROM [Sales]
            WHERE {[Time].[1997]}
            """)

    def test_build_nested_mdx_functions_same_axis(self):
        cube = 'Sales'

        rows = [{
            "member_unique_name": "TOPCOUNT({[Products].[Software].Children, [Products].[Hardware].Children}, 5, TOPCOUNT({[Sales].[Europe].Children, [Sales].[America].Children}, 5, [Measures].[Response]))",
            "hierarchy_unique_name": "",
            "depth": "0"
        },{
            "member_unique_name": "TOPCOUNT({[Products].[Software].Children, [Products].[Hardware].Children}, 5, [Measures].[Response])",
            "hierarchy_unique_name": "",
            "depth": "0"
        }]

        columns = [{
            "member_unique_name": "[Company].Children",
            "hierarchy_unique_name": "[Company]",
            "depth": "0"
        }]

        slicers = [{
            "member_unique_name":"[Time].[1997]",
            "hierarchy_unique_name":"[Time]",
            "depth": "0",
        }]

        builder = MdxBuilder(cube, columns, rows, slicers=slicers)
        mdx = builder.get_result()

        self.assertMdx(mdx, """
            SELECT {[Company].Children} ON COLUMNS,
                {TOPCOUNT({[Products].[Software].Children, [Products].[Hardware].Children}, 5, TOPCOUNT({[Sales].[Europe].Children, [Sales].[America].Children}, 5, [Measures].[Response])),TOPCOUNT({[Products].[Software].Children, [Products].[Hardware].Children}, 5, [Measures].[Response])} ON ROWS
            FROM [Sales]
            WHERE {[Time].[1997]}
            """)

    def test_calculated_member_named_set(self):
        pass

    def test_data_transpose(self):
        x = 5
        y = 3
        input = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14]

        def formula(x, y, input):
            rv = []
            for i in input:
                rv.append(i*x-((i/y)*len(input))+(i/y))

            return rv

        result = formula(x, y, input)
        self.assertEqual(result, [0,5,10,1,6,11,2,7,12,3,8,13,4,9,14])
        self.assertEqual(formula(3, 2, [0,1,2,3,4,5]), [0,3,1,4,2,5])
        self.assertEqual(formula(2, 3, [0,3,1,4,2,5]), [0,1,2,3,4,5])


# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_mdx.py', 'iqportal.iqube.test.unit.test_mdx.MdxBuilderTestCase.test_indent']
    run()
    sys.exit()
