from basesoft.web.test.unit import iqportal_unittest

from iqportal.iqube.core.controllers import MdxMethodController
from iqportal.iqube.core.models import MdxMethod

class TestCase(iqportal_unittest.IQPortalTestCase):
    """
        Test user permissions for different mdx methods
    """
    def setUp(self):
        """
        Connect to db, create active account and instantiate controller
        """

        self.connect_db()
        account = self._create_active_account()
        self.group = account.default_group
        self.controller = MdxMethodController(self.sa_session, account_id=account.id)

    def test_get_mdx_methods(self):
        # if user unknown raise Exception
        # give permission to the active group
        self._attach_permissions_to_group(self.group, 'manage_mdx_methods')
        # if there are no methods returned the group has no privilege
        self.assertTrue(len(self.controller.get_mdx_methods()) == 0)
        # grant privilege to mdx method
        top_count = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'TopCount').first()
        self.controller.grant_privilege_to(top_count, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        # if there is a mdx method check if reference is correct
        self.assertTrue(len(self.controller.get_mdx_methods()) == 1, "More or less than 1 methods exist")
        self.assertIsInstance(self.controller.get_mdx_methods().pop(), MdxMethod, "Object not the same!")

        bot_count = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'BottomCount').first()
        self.controller.grant_privilege_to(bot_count, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        methods = self.controller.get_mdx_methods()
        self.assertTrue(len(methods) == 2)

        self.assertTrue(top_count in methods)
        self.assertTrue(bot_count in methods)

