'''
Created on Jun 4, 2014

@author: marten
'''

import os

from django.core.files.uploadedfile import SimpleUploadedFile

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.file.controllers import UploadFileController
from iqportal.iqube.optiq.controllers import OptiqCSVController,\
    AbstractOptiqController, OPTIQ_WRITABLE
from iqportal.iqube.core.controllers import ProjectController, IQubeController
from iqportal.iqube.xmla.client import XMLAClient

from django.conf import settings

from datetime import date, time, datetime
from iqportal.iqube.optiq.exceptions import OptiqImportValueError,\
    OptiqInvalidFile
from iqportal.iqube.optiq.importers import AbstractImporter
from twisted.trial.unittest import SkipTest
from basesoft.web.contrib.file.models import FileTmp
import commands

class TestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()

        account = self._create_active_account()
        self.account = account
        self.group = account.groups[0].group

        self._attach_permissions_to_group(self.group, 'view_datasource', 'create_datasource', 'view_file', 'create_file')

    def _create_csv_datasource(self, ds_name=None):

        if ds_name is None:
            ds_name = self._uniquestring('ds')

        # make a datasource
        controller = OptiqCSVController(self.session, account_id=self.account.id)
        datasource = controller.create_datasource(ds_name)

        ds_path = os.path.join(OPTIQ_WRITABLE, str(datasource.id), 'csv')
        status, output = commands.getstatusoutput("/usr/bin/find %s  -type f -and ! -name 'date_dimension.csv' -exec rm {} \;" % ds_path)
        assert status == 0, output

        return controller, datasource

    def _quick_import(self, data):

        if isinstance(data, FileTmp):
            tmp_file = data
        else:
            tmp_file = UploadFileController(self.session, None).save_temp_files(self._make_xls(data))[0]

        # detect columns
        importer = AbstractImporter.get_importer(tmp_file)
        columns = importer.get_file_columns()

        controller, datasource = self._create_csv_datasource()

        # import as a csv file
        controller.do_import(datasource, u'table1', importer, columns)

    def get_csv_content(self, filename):
        return open(os.path.join(os.path.dirname(os.path.realpath(__file__)),'../csv/%s' % filename)).read()

    def _make_xls(self, data):

        # http://packages.python.org/openpyxl
        from openpyxl.workbook import Workbook
        wb = Workbook(encoding="UTF-8")
        ws = wb.get_active_sheet()
        ws.title = "New Title"

        for i in range(len(data)):
            for j in range(len(data[i])):
                cell = ws.cell(row = i, column = j)
                cell.value = data[i][j]

        file_path = os.path.join(settings.WRITABLE_DIRECTORY, 'temp.xlsx')
        wb.save( file_path)

        uploaded_file = {'filedata': SimpleUploadedFile('test_import.xlsx', open(file_path).read() )}

        return uploaded_file


    def test_column_types(self):
        """
        """

        if settings.MONDRIAN_XMLA_CACHE_ENABLED is False:
            raise Exception('MONDRIAN_XMLA_CACHE_ENABLED must be set True to run this test')


        # note on the TIME: column. The first two decimal values are excel time definitions.. yay
        data = [
            ['C', 'P',   'T_I',   'S_I',         'I',                  'B_I',  'DECIMAL',  'DATE',                      'TIME',        'EMPTY'],
            ["c1", "p1", -128,  -32768,  -2147483648,   -9223372036854775808,  1000.1234,   date(2012, 1, 1),           '11:11:13', ''],
            ["c2", "p2",  127,   32767,   2147483647,    9223372036854775807,  999.12345,   datetime(2013, 2, 1, 0, 0), '23:15:00', ''],
            ["c3", "p3",    1,       1,          1.0,                      1, -1000.1234,   '2014-04-01',               '1900-02-01 01:15:00', ''],
            ["c4", "p4",    1,       1,           '',                      1, -999.12345,   '2014-04-01 00:00:00',      '',  ''],
        ]

        # make c4 string length a bit bigger
        data[4][0] = 'c4 bier ' * 100

        # make file
        tmp_file = UploadFileController(self.session, None).save_temp_files(self._make_xls(data))[0]

        # detect columns
        #file_path = os.path.join(settings.TMP_FILE_REPOSITORY, tmp_file.file_hash)
        importer = AbstractImporter.get_importer(tmp_file)
        columns = importer.get_file_columns()

        self.assertEqual(columns[0]['COLUMN_NAME'], 'C')
        self.assertEqual(columns[0]['DATA_TYPE'], 'VARCHAR', 'column %s' % 'C')
        self.assertEqual(columns[0]['PRECISION'], 1000, 'column %s' % 'C') # longest is 800.. +25% = 1000

        self.assertEqual(columns[1]['COLUMN_NAME'], 'P')
        self.assertEqual(columns[1]['DATA_TYPE'], 'VARCHAR', 'column %s' % 'P')

        self.assertEqual(columns[2]['COLUMN_NAME'], 'T_I')
        self.assertEqual(columns[2]['DATA_TYPE'], 'INTEGER', 'column %s' % 'T_I')

        self.assertEqual(columns[3]['COLUMN_NAME'], 'S_I')
        self.assertEqual(columns[3]['DATA_TYPE'], 'INTEGER', 'column %s' % 'S_I')

        self.assertEqual(columns[4]['COLUMN_NAME'], 'I')
        self.assertEqual(columns[4]['DATA_TYPE'], 'INTEGER', 'column %s' % 'I')

        self.assertEqual(columns[5]['COLUMN_NAME'], 'B_I')
        self.assertEqual(columns[5]['DATA_TYPE'], 'BIGINT', 'column %s' % 'B_I')

        self.assertEqual(columns[6]['COLUMN_NAME'], 'DECIMAL', 'column %s' % 'DECIMAL')
        self.assertEqual(columns[6]['DATA_TYPE'], 'DECIMAL')
        self.assertEqual(columns[6]['PRECISION'], 9)
        self.assertEqual(columns[6]['DEC_DIGITS'], 5)

        # for date types, the column is convert to integer so we have a fast fk to the
        # time dimension date table. however in remarks we can still see this is a date
        self.assertEqual(columns[7]['COLUMN_NAME'], 'DATE')
        self.assertEqual(columns[7]['DATA_TYPE'], 'DATE', 'column %s' % 'DATE')

        # the date column created a second fk column of type int
        self.assertEqual(columns[8]['COLUMN_NAME'], 'DATE__FK')
        self.assertEqual(columns[8]['DATA_TYPE'], 'INTEGER', 'column %s' % 'DATE FK')

        # time type should be auto dected as TIME
        self.assertEqual(columns[9]['COLUMN_NAME'], 'TIME')
        self.assertEqual(columns[9]['DATA_TYPE'], 'TIME', 'column %s' % 'TIME')

        # time shadow column
        self.assertEqual(columns[10]['COLUMN_NAME'], 'TIME__TIME')
        self.assertEqual(columns[10]['DATA_TYPE'], 'INTEGER', 'column %s' % 'TIME__TIME')

        # empty column should be detected as varchar
        self.assertEqual(columns[11]['COLUMN_NAME'], 'EMPTY')
        self.assertEqual(columns[11]['DATA_TYPE'], 'VARCHAR', 'column %s' % 'EMPTY')


        # make a datasource
        controller, datasource = self._create_csv_datasource(self._uniquestring('ds with spaces'))

        # import as a csv file
        controller.do_import(datasource, u'table1 with spaces', importer, columns)


        expected = [{
            'name': 'date_dimension', 'columns': [
                ['ID', 'INTEGER'],
                ['CDATE', 'DATE'],
                ['CYEAR', 'INTEGER'],
                ['QUARTER', 'INTEGER'],
                ['CMONTH', 'INTEGER'],
                ['WEEK_OF_YEAR', 'INTEGER'],
                ['DAY_OF_MONTH', 'INTEGER'],
            ]}, {
            'name': 'table1 with spaces', 'columns': [
                ['C', 'VARCHAR'],
                ['P', 'VARCHAR'],
                ['T_I', 'INTEGER'],
                ['S_I', 'INTEGER'],
                ['I', 'INTEGER'],
                ['B_I', 'BIGINT'],
                ['DECIMAL', 'DOUBLE'],
                ['DATE', 'DATE'],
                ['DATE__FK', 'INTEGER'],
                ['TIME', 'TIMESTAMP'],
                ['TIME__TIME', 'INTEGER'],
                ['EMPTY', 'VARCHAR'],
            ]}
        ]

        metadata = controller.get_metadata(datasource.id)
        tables_state = controller.get_tables_state(datasource.id)

        i = -1
        t_i = -1
        c_i = -1
        for t in expected:
            t_i += 1
            c_i = -1

            name = t['name']
            for c in t['columns']:
                c_i += 1
                i += 1

                self.assertEqual(metadata[i]['TABLE_NAME'], name);
                self.assertEqual(metadata[i]['COLUMN_NAME'], c[0]);
                self.assertEqual(metadata[i]['DATA_TYPE'], c[1]);

                self.assertEqual(tables_state[t_i]['name'], name)

                self.assertEqual(tables_state[t_i]['columns'][c_i]['COLUMN_NAME'], c[0]);
                self.assertEqual(tables_state[t_i]['columns'][c_i]['DATA_TYPE'], c[1]);


        # make a cube
        cube_xml = """
        <Schema name="%(schema_name)s">
            <Cube name="%(cube_name)s">
                <Table name="%(table_name)s"/>

                <Dimension name="C"><Hierarchy hasAll="true"><Level column="C" name="C" /></Hierarchy></Dimension>

                <Measure aggregator="sum" column="T_I" name="T_I" />
                <Measure aggregator="sum" column="S_I" name="S_I" />
                <Measure aggregator="sum" column="I" name="I" />
                <Measure aggregator="sum" column="B_I" name="B_I" />

            </Cube>
        </Schema>
        """ % {
            'schema_name': ('schema_' + datasource.name).upper(),
            'cube_name': 'cube1',
            'table_name': 'table1 with spaces'
        }

        pc = ProjectController(self.session, account_id=self.account.id)
        pc.write_schema(datasource.mondrian_project, cube_xml)

        # connect to mondrian
        xmla_client = XMLAClient(settings.MONDRIAN_XMLA_SERVER[0], settings.MONDRIAN_XMLA_SERVER[1], settings.MONDRIAN_XMLA_PATH, datasource.name)

        # assert 1 cube exists
        self.assertEquals(len(xmla_client.get_cubes()), 1)

        # do a mdx query over this datasource to see if that goes ok
        xmla_client.execute_mdx("""SELECT {[Measures].T_I, [Measures].S_I, [Measures].I, [Measures].B_I} ON COLUMNS FROM cube1""")

        # test clearing of cache
        pc = IQubeController()
        pc.clear_xmla_cache(datasource.mondrian_project)


    def _make_upload(self, upload_type):

        import iqportal.iqube.test

        if upload_type == 'csv':
            csv_file = os.path.join(iqportal.iqube.test.__path__[0], 'csv', '2012_rand.csv')
            uploaded_file = {'filedata': SimpleUploadedFile('2012_rand.csv', open(csv_file).read())}
        elif upload_type == 'xlsx':
            xlsx_file = os.path.join(iqportal.iqube.test.__path__[0], 'xlsx', '2012_rand.xlsx')
            uploaded_file = {'filedata': SimpleUploadedFile('2012_rand.xlsx', open(xlsx_file).read())}
        elif upload_type == 'xlsx-small':
            xlsx_file = os.path.join(iqportal.iqube.test.__path__[0], 'xlsx', '2012_rand_small.xlsx')
            uploaded_file = {'filedata': SimpleUploadedFile('2012_rand_small.xlsx', open(xlsx_file).read())}
        elif upload_type == 'xlsx-incompatible':
            xlsx_file = os.path.join(iqportal.iqube.test.__path__[0], 'xlsx', '2012_rand_incomp.xlsx')
            uploaded_file = {'filedata': SimpleUploadedFile('2012_rand_incomp.xlsx', open(xlsx_file).read())}
        elif upload_type == 'txt':
            csv_file = os.path.join(iqportal.iqube.test.__path__[0], 'csv', '2012_rand.csv')
            uploaded_file = {'filedata': SimpleUploadedFile('2012_rand.txt', open(csv_file).read())}
        elif upload_type == 'xls-time':
            xlsx_file = os.path.join(iqportal.iqube.test.__path__[0], 'xlsx', 'beer_time.xls')
            uploaded_file = {'filedata': SimpleUploadedFile('beer_time.xls', open(xlsx_file).read())}
        elif upload_type == 'xlsx-time':
            xlsx_file = os.path.join(iqportal.iqube.test.__path__[0], 'xlsx', 'beer_time.xlsx')
            uploaded_file = {'filedata': SimpleUploadedFile('beer_time.xlsx', open(xlsx_file).read())}
        elif upload_type == 'beer-time-country':
            xlsx_file = os.path.join(iqportal.iqube.test.__path__[0], 'xlsx', 'beer_time_country.xls')
            uploaded_file = {'filedata': SimpleUploadedFile('beer_time_country.xls', open(xlsx_file).read())}
        else:
            raise Exception('unknown upload_type: %s' % upload_type)

        return UploadFileController(self.session, None).save_temp_files(uploaded_file)[0]

    def test_files(self):

        files = ['csv', 'xlsx', 'xlsx-small', 'xlsx-incompatible', 'txt', 'xls-time', 'xlsx-time', 'beer-time-country']
        for f in files:
            self._quick_import(self._make_upload(f))



    def test_bad_columns(self):

        # empty column should error
        data = [
            ['C', '',   'T', 'Q', 'DECIMAL'],
            ["c1", "", 12,  23,   5.3],
            ["c2", "", 34,  12,   6.3],
            ["c3", "", 123, 4,    100],
            ["c4", "", 67,  8,    99.3],
        ]
        self.assertRaises(OptiqInvalidFile, self._quick_import, data)

        # dataset with extra column
        data = [
            ['C', 'T', 'Q', 'DECIMAL'],
            ["c1", 12,  23,  5.3,        6],
            ["c2", 34,  12,  6.3,        7],
            ["c3", 123, 4,   100,        8],
            ["c4", 67,  8,   99.3,       9],
        ]
        self.assertRaises(OptiqInvalidFile, self._quick_import, data)

        # dataset with duplicate column
        data = [
            ['C', 'T', 'T', 'DECIMAL', 'A'],
            ["c1", 12,  23,  5.3,        6],
            ["c2", 34,  12,  6.3,        7],
            ["c3", 123, 4,   100,        8],
            ["c4", 67,  8,   99.3,       9],
        ]
        self.assertRaises(OptiqInvalidFile, self._quick_import, data)

    def test_xlsx_refresh(self):
        #TODO: fixme when implementing data updating
        pass

    def test_excel_date_time(self):
        """
        tests various date and time combinations in xls and xlsx
        """

        for xls_type in ('xls', 'xlsx', ):

            table_name = u'my_table_'+xls_type

            xls_file = self._make_upload('%s-time' % xls_type)

            # detect columns
            importer = AbstractImporter.get_importer(xls_file)
            columns = importer.get_file_columns()

            # expect auto detection of time and date
            self.assertEqual(columns[2]['COLUMN_NAME'], 'TIME')
            self.assertEqual(columns[2]['DATA_TYPE'], 'TIME')
            self.assertEqual(columns[4]['COLUMN_NAME'], 'DATE')
            self.assertEqual(columns[4]['DATA_TYPE'], 'DATE')

            # make a datasource
            controller, datasource = self._create_csv_datasource()

            # import as a csv file
            controller.do_import(datasource, table_name, importer, columns)

            # FIXME: code below can be tested once we can SELECT ourselfs from optiq.. tru iqube?
            """
            # assert table data
            data = self._table_data(table_name)

            # in excel: 12:10:15 -> 43815s
            self.assertEqual(data[0][2], '1900-01-01 12:10:15' if xls_type == 'xls' else '12:10:15', xls_type) # time varchar
            self.assertEqual(data[0][3], 43815) # time seconds

            # first row date
            self.assertEqual(data[0][4], datetime.date(2012, 1, 1)) # native date column
            self.assertEqual(data[0][5], 20120101) # date fk

            # in excel: 25:10:15 -> 90615s
            self.assertEqual(data[1][2], '1900-01-02 01:10:15', xls_type)
            self.assertEqual(data[1][3], 90615)
            """
