'''
Created on Aug 27, 2014

@author: rado
'''
from basesoft.web.test.unit import iqportal_unittest

from iqportal.iqube.compose.controllers import MdxCalculatedMemberController
from iqportal.iqube.core.models import MdxMethod

class TestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()

        account = self._create_active_account()
        self.group = account.default_group
        self.mcm = MdxCalculatedMemberController(self.sa_session, account_id=account.id)
        self._attach_permissions_to_group(self.group, 'view_calculated_member')

    def _get_method(self, name):
        return self.sa_session.query(MdxMethod).filter(MdxMethod.name==name).first()

    def _grant_method_privs(self):
        for m in self.sa_session.query(MdxMethod).all():
            self.mcm.grant_privilege_to(m, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

    def test_parse_calculated_member(self):

        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [None, None, None],
        }

        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        cm1 = self.mcm.save_or_update_calculated_member(None, 'My TopCount', state)

        inputs = ['{[Store].[Store Country]}', '5', '[Measures].[Unit Sales]']
        parsed_cm = self.mcm.get_parsed(cm1.id, inputs)

        self.assertEqual('TopCount({[Store].[Store Country]},5,[Measures].[Unit Sales])', parsed_cm)

        inputs2 = ['{[Store].[Store Country]}', '5', None]
        parsed_cm1 = self.mcm.get_parsed(cm1.id, inputs2)

        self.assertEqual('TopCount({[Store].[Store Country]},5)', parsed_cm1)
