# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

import os
import shutil

import iqportal.iqube.test

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException
from iqportal.iqube.core.controllers import ProjectController
from iqportal.iqube.core.models import DataSource
from sqlalchemy.orm.exc import NoResultFound
from twisted.trial.unittest import SkipTest
from iqportal.iqube.xmla.client import XMLAClient
from basesoft.web.contrib.account.models import Account

class ProjectTestCase(iqportal_unittest.IQPortalTestCase):
    """
    """
    def setUp(self, account_id=None):
        """
        """
        self.connect_db()

        if account_id is None:
            account_id = self._create_active_account().id

        self.controller = ProjectController(self.session, account_id=account_id)
        self.group = self.sa_session.query(Account).filter(Account.id==account_id)[0].groups[0].group
        self._attach_permissions_to_group(self.group, 'create_datasource', 'view_datasource', 'datasource_create_cube')


    def test_write_schema_mysql(self):
        """
        """

        try:
            ds = self.session.query(DataSource).filter(DataSource.name=='%s_mysql_test' % settings.SETTINGS_MODULE).one()
        except NoResultFound:
            pass
        else:
            self.session.delete(ds)
            self.session.flush()

        ds_name = '%s_mysql_test' % settings.SETTINGS_MODULE
        descr = ds_name + " descr"
        db = {"protocol": "mysql",
              "user": "buildbot",
              "password": "buildbot",
              "host": "testorix.dev.ams",
              "port": "3306",
              "db_name": "iqportal_buildbot_unit"}

        datasource = self.controller.save_or_update_datasource(None, ds_name[:40], descr, ds_name, \
                        db["protocol"], db["user"], db["password"], db["host"], \
                        db["port"], db["db_name"])

        cube_name = self._uniquestring(u'test_cube')[:40]

        # fixme: make a proper cube
        cube_xml = """
        <Schema name="test_write_schema_mysql">
        <Cube name="%s">
            <Table name="sales_fact_1997" />

            <Dimension foreignKey="promotion_id" name="Promotions">
                <Hierarchy allMemberName="All Promotions" defaultMember="[All Promotions]" hasAll="true" primaryKey="promotion_id">
                    <Table name="promotion" />
                    <Level column="promotion_name" name="Promotion Name" />
                </Hierarchy>
            </Dimension>

            <Measure aggregator="sum" column="unit_sales" formatString="##,####.##" name="Unit Sales" />

            <Measure aggregator="sum" column="store_cost" formatString="##,####.##" name="Store Cost" />

            </Cube></Schema>
        """ % cube_name

        self.controller.write_schema(datasource.mondrian_project, cube_xml)

        # delete the cube again
        self.controller.delete_mondrian_cube(datasource.id, datasource.mondrian_project, cube_name)


