/**
 * Copyright (c) 2010 BaseSoft B.V.
 * See LICENSE for details.
 */
(function() {

	var localeList = [
		{name: 'Dutch', locale: 'nl_NL'},
		{name: 'English', locale: 'en_GB'},
		{name: 'Deutch', locale: 'de_DE'}
	];

	var themeList = [
		//{name: 'Debug', cssClass: 'survey-debug', module: 'iquest_survey_theme_debug'},
		{name: 'BaseSoft', cssClass: 'survey-basesoft', module: 'iquest_survey_theme_basesoft'},
		{name: 'TNT', cssClass: 'survey-tnt', module: 'iquest_survey_theme_tnt'}
	];


	BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

		name: "Branching Test",
		assertClean: true,

		setUp: function() {

			var sandbox = YAHOO.util.Dom.get('sandbox');
			sandbox.innerHTML = '';

			this.composition_el = document.createElement('div');
			sandbox.appendChild(this.composition_el);
		},

		testResponse: function() {

			var runner = null;

			var validateCB = function() {
				this.resume(function(){

					var pageNrs = YAHOO.util.Dom.getElementsBy(function(el){
						return (el.className == 'pageNrs');
					}, 'div', runner.canvas.element);

					// Make sure we are on the third page now.
					YAHOO.util.Assert.areEqual(pageNrs[0].innerHTML, '3/4');

					// make sure we destroy cleanly
					runner.destroy();
				});
			};

			var responseCB  = function() {
				this.resume(function(){

					// radios get created in a timeout so use a safe setTimeout call
					var _this = this;
					setTimeout(function() {
						_this.resume(function() {

							runner.activateChildren();
							var radios = YAHOO.util.Dom.getElementsBy(function(el){
								return (el.type == 'radio');
							}, 'input', runner.canvas.element);
							radios[1].checked = true;
							runner.canvas.on('pageIndexChange', validateCB, null, this);
							YAHOO.util.UserAction.click(runner.canvas._button_next);
							this.wait();

						});
					}, 200);
					this.wait();

				});
			};

			var loadCB = function(anonymous_hash) {
				this.resume(function(){
					runner = new BASESOFT.iquest.response.ParticipantSurveyRunner();
					runner.run({runner_id: anonymous_hash});

					runner.onSurveyLoad.subscribe(responseCB, null, this);
					//this._garbage.push(runner);
					this.wait(this.timeout);
				});
			};


			var rpc = new BASESOFT.util.JsonBroker('/iqportal.iquest.test/selenium/yui/rpc', {executeIfLoggedOut: true});
			rpc.call('publish_branching_survey', [], loadCB, null, this);

			this.wait();
		},

		/**
		 *
		 */
		testLoadAndUpdate: function() {

			var updateCB = function() {


				this.resume(function(){

					console.warn('resuming');

					var survey = this.manager.canvas;
					var b2 = survey.children['blocks'][1];
					var b2c1items = b2.children['columns'][0].children['items'];

					YAHOO.util.Assert.areEqual(b2c1items.length, 1);

					// Both items of the first column in the second block have a wire.
					YAHOO.util.Assert.areEqual(b2c1items[0]._terminals.length, 3);

					// Now lets add a wire from the first item in the first column of the third block to the first block.
					var item = survey.children['blocks'][2].children['columns'][0].children['items'][0];
					var t = item._terminals[0].terminal;
					var wire = new WireIt.Wire(t, survey.children['blocks'][0].terminal, this.manager.getLayoutContainer());
					t.redrawAllWires();

					BASESOFT.util.setTimeout(function() {

						var state = item.getState();
						console.warn('item el', item.element);
						console.warn('state', state);
						console.warn('state.branching.length', state.branching.length);
						YAHOO.util.Assert.areEqual(state.branching.length, 1);

					}, 2000, null, this);

					// make sure we destroy cleanly
					this.manager.destroy();

				});
			};

			var tabChangeCB = function() {

				this.resume(function(){
					YAHOO.util.Assert.areEqual(this.manager.canvas.children['blocks'].length, 4);

                                        console.warn('switching tab');
					this.manager.tabView.selectTab(1);

                                        console.warn('subscribing');
					this.manager.canvas.children['blocks'][3].terminalAvailable.subscribe(updateCB, null, this);


					console.warn('waiting');
					this.wait(35000);
				});
			};

			var loadCB = function(result, survey_id) {
				BASESOFT.env.permissions.has = function() {
					return true;
				};
				this.resume(function(){
					BASESOFT.env.account.id = result.id;
					this.manager = new BASESOFT.iquest.compose.ComposeManager();
					this.manager.init({
						'composition_el': this.composition_el,
						'compose_vars': {
							'localeList': localeList,
							'themeList': themeList
						}
					});
					var event = this.manager.load(survey_id);
					event.subscribe(tabChangeCB, null, this);
					this._garbage.push(this.manager);

					console.info('waiting for tabChangeCB');
					this.wait(35000);
				});
			};

			var loginCB = function(args) {

				this.resume(function() {
					var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
					loginrpc.call('login', [args['username'], args['password']],
						loadCB, null, this, 60000, args['survey_id']);

					this.wait(35000);
				});

			};

			var rpc = new BASESOFT.util.JsonBroker('/iqportal.iquest.test/selenium/yui/rpc', {executeIfLoggedOut: true});
			rpc.call('create_branching_survey', [], loginCB, null, this);

			this.wait(35000);

		},

		testDropDownBranching: function() {

			var tabChangeCB = function() {

				this.resume(function(){
					this.manager.tabView.selectTab(1);
					var survey = this.manager.canvas;
					var b1 = survey.children['blocks'][0];
					var b1c1items = b1.children['columns'][0].children['items'];

					// click on the dropdown will open the options div
					var select = YAHOO.util.Dom.getElementBy(function(){
						return true;
					}, 'select', b1c1items[0].element);

					YAHOO.util.UserAction.mousedown(select);
					var opt_div = YAHOO.util.Dom.getElementsByClassName("dropdown-terminals");
					// check if the options div is visible
					YAHOO.util.Assert.areEqual(opt_div.length, 1);
					YAHOO.util.Assert.areEqual(opt_div[0].style.display, "");
				});
			};

			var loadCB = function(result, survey_id) {
				BASESOFT.env.permissions.has = function() {
					return true;
				};
				this.resume(function(){
					BASESOFT.env.account.id = result.id;
					this.manager = new BASESOFT.iquest.compose.ComposeManager();
					this.manager.init({
						'composition_el': this.composition_el,
						'compose_vars': {
							'localeList': localeList,
							'themeList': themeList
						}
					});
					var event = this.manager.load(survey_id);
					event.subscribe(tabChangeCB, null, this);
					this._garbage.push(this.manager);
					this.wait(35000);
				});
			};

			var loginCB = function(args) {

				this.resume(function() {
					var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
					loginrpc.call('login', [args['username'], args['password']],
						loadCB, null, this, 60000, args['survey_id']);

					this.wait(35000);
				});

			};

			var rpc = new BASESOFT.util.JsonBroker('/iqportal.iquest.test/selenium/yui/rpc', {executeIfLoggedOut: true});
			rpc.call('create_branching_survey_dropdown', [], loginCB, null, this);

			this.wait(35000);
		},

		testSliderBranching: function() {

			var tabChangeCB = function() {

				this.resume(function(){
					this.manager.tabView.selectTab(1);
					var survey = this.manager.canvas;
					var b1 = survey.children['blocks'][0];
					var item = b1.children['columns'][0].children['items'][0];

					var br = item.get("branching");
					// there are two wires from the slider itemQA
					YAHOO.util.Assert.areEqual(br.length, 2);
				});
			};

			var loadCB = function(result, survey_id) {
				BASESOFT.env.permissions.has = function() {
					return true;
				};
				this.resume(function(){
					BASESOFT.env.account.id = result.id;
					this.manager = new BASESOFT.iquest.compose.ComposeManager();
					this.manager.init({
						'composition_el': this.composition_el,
						'compose_vars': {
							'localeList': localeList,
							'themeList': themeList
						}
					});
					var event = this.manager.load(survey_id);
					event.subscribe(tabChangeCB, null, this);
					this._garbage.push(this.manager);
					this.wait(35000);
				});
			};

			var loginCB = function(args) {

				this.resume(function() {
					var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
					loginrpc.call('login', [args['username'], args['password']],
						loadCB, null, this, 60000, args['survey_id']);

					this.wait(35000);
				});

			};

			var rpc = new BASESOFT.util.JsonBroker('/iqportal.iquest.test/selenium/yui/rpc', {executeIfLoggedOut: true});
			rpc.call('create_branching_survey_slider', [], loginCB, null, this);

			this.wait(35000);
		},

		getSurveyState: function() {
			return {
					"survey": {
					"objectName": "Survey",
					"availableLocales": ['en_GB'],
					"fallbackLocale": {
						'locale': 'en_GB',
						'name': 'English'
					},
					"theme": {
						"module": "iquest_survey_theme_basesoft",
						"name": "BaseSoft",
						"cssClass": "survey-basesoft"
					},

					"children": {
						"blocks": [{
							'objectName': 'SurveyBlock',
							'children': {
								'columns': [{
									"objectName": "SurveyBlockColumn",
									"children": {
										"items": [{
											"objectName": "SurveyItemQA",
											"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,
											"locales": {
												'en_GB': {
													'label': 'Will the real royal highness please stand up?'
												}
											},
											'required': false,
											'children': {
												'questions': [{
													"objectName": "SurveyQuestion"
												}],
												'answers': [{
													"objectName": "SurveyAnswer",
													"locales": {
														'en_GB': {
															'label': 'Freddie Prinze Jr.'
														}
													}
												},{
													"objectName": "SurveyAnswer",
													"locales": {
														'en_GB': {
															'label': 'Fresh Prince of Bel-Air.'
														}
													}
												}]
											}
										}]
									}
								}]
							}
						},{
							"objectName": "SurveyBlock",
							"children": {
								"columns": [{
									"objectName": "SurveyBlockColumn",
									"children": {
										"items": [{
											"objectName": "SurveyItemQA",
											"dbID": 10,
											"poolItemID": null,
											"questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
											"answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
											"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_TEXT_INPUT,

											"locales": {
												'en_GB': {
													'label': 'What are you gonna eat?'
												}
											},
											'required': false,
											"children": {
												"questions": [{
													"objectName": "SurveyQuestion",
													"dbID": 11
												}],

												"answers": [{
													"objectName": "SurveyAnswer",
													"dbID": 12,
													"locales": {
														'en_GB': {
															'label': 'Today'
														}
													}
												}, {
													"objectName": "SurveyAnswer",
													"dbID": 13,
													"locales": {
														'en_GB': {
															'label': 'Tomorrow'
														}
													}
												}]
											}
										}, {
											"objectName": "SurveyItemQA",
											"dbID": 30,
											"poolItemID": null,
											"questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
											"answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
											"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,

											"locales": {
												'en_GB': {
													'label': 'Gender please!'
												}
											},
											'required': false,
											"children": {
												"questions": [{
													"objectName": "SurveyQuestion",
													"dbID": 31,
													"locales": {
														'en_GB': {
															'label': 'Today'
														}
													}
												}, {
													"objectName": "SurveyQuestion",
													"dbID": 32,
													"locales": {
														'en_GB': {
															'label': 'Tomorrow'
														}
													}
												}],

												"answers": [{
													"objectName": "SurveyAnswer",
													"dbID": 33,
													"locales": {
														'en_GB': {
															'label': 'Male'
														}
													}
												}, {
													"objectName": "SurveyAnswer",
													"dbID": 34,
													"locales": {
														'en_GB': {
															'label': 'Female'
														}
													}
												}, {
													"objectName": "SurveyAnswer",
													"dbID": 35,
													"locales": {
														'en_GB': {
															'label': 'Kelly'
														}
													}
												}]
											}
										}, {
											"objectName": "SurveyItemQA",
											"dbID": 20,
											"poolItemID": null,
											"questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
											"answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
											"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_MULTIPLE_CHOICE,

											"locales": {
												'en_GB': {
													'label': 'Clicky what you like?'
												}
											},
											'required': false,
											"children": {
												"questions": [{
													"objectName": "SurveyQuestion",
													"dbID": 21
												}],

												"answers": [{
													"objectName": "SurveyAnswer",
													"dbID": 22,
													"locales": {
														'en_GB': {
															'label': 'Apples'
														}
													}
												}, {
													"objectName": "SurveyAnswer",
													"dbID": 23,
													"locales": {
														'en_GB': {
															'label': 'Pears'
														}
													}
												}, {
													"objectName": "SurveyAnswer",
													"dbID": 24,
													"locales": {
														'en_GB': {
															'label': 'Dogs'
														}
													}
												}]
											}
										}, {
											"objectName": "SurveyItemQA",
											"dbID": 40,
											"poolItemID": null,
											"questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
											"answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
											"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,
											"layout": BASESOFT.iquest.widget.survey.ItemQA.prototype.LAYOUT_SLIDER,

											"locales": {
												'en_GB': {
													'label': 'How do you feel?'
												}
											},
											'required': false,
											"children": {
												"questions": [{
													"objectName": "SurveyQuestion",
													"dbID": 41
												}],

												"answers": [{
													"objectName": "SurveyAnswer",
													"dbID": 42,
													"locales": {
														'en_GB': {
															'label': 'Good'
														}
													}
												}, {
													"objectName": "SurveyAnswer",
													"dbID": 43,
													"locales": {
														'en_GB': {
															'label': 'Normal'
														}
													}
												}, {
													"objectName": "SurveyAnswer",
													"dbID": 44,
													"locales": {
														'en_GB': {
															'label': 'Bad'
														}
													}
												}]
											}
										},{
											"objectName": "SurveyItemQA",
											"dbID": 50,
											"poolItemID": null,
											"questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
											"answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
											"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,
											"layout": BASESOFT.iquest.widget.survey.ItemQA.prototype.LAYOUT_DROPDOWN,

											"locales": {
												'en_GB': {
													'label': 'How do you feel tomorrow?'
												}
											},
											'required': false,
											"children": {
												"questions": [{
													"objectName": "SurveyQuestion",
													"dbID": 51
												}],

												"answers": [{
													"objectName": "SurveyAnswer",
													"dbID": 52,
													"locales": {
														'en_GB': {
															'label': 'Good'
														}
													}
												}, {
													"objectName": "SurveyAnswer",
													"dbID": 53,
													"locales": {
														'en_GB': {
															'label': 'Normal'
														}
													}
												}, {
													"objectName": "SurveyAnswer",
													"dbID": 54,
													"locales": {
														'en_GB': {
															'label': 'Bad'
														}
													}
												}]
											}
										}]
									}
								}]
							}
						}]
					}
				}
			};
		}

	}));

})();
