/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 *
 * Testcase for the IQuest Compose
 */
(function(){

	"use strict";

	BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

		name: "Compose Test",
		assertClean: true,

		setUp: function(){

			// style the sandbox a bit
			var sandbox = YAHOO.util.Dom.get('sandbox');
			sandbox.innerHTML = '';

			var composition_el = document.createElement('div');
			sandbox.appendChild(composition_el);

			var localeList = [{
				name: 'Dutch',
				locale: 'nl_NL'
			}, {
				name: 'English',
				locale: 'en_GB'
			}, {
				name: 'Deutch',
				locale: 'de_DE'
			}];

			var themeList = [			//{name: 'Debug', cssClass: 'survey-debug', module: 'iquest_survey_theme_debug'},
			{
				name: 'BaseSoft',
				cssClass: 'survey-basesoft',
				module: 'iquest_survey_theme_basesoft'
			}, {
				name: 'TNT',
				cssClass: 'survey-tnt',
				module: 'iquest_survey_theme_tnt'
			}];


			this.manager = new BASESOFT.iquest.compose.ComposeManager();
			this.manager.init({
				'composition_el': composition_el,
				'compose_vars': {
					'localeList': localeList,
					'themeList': themeList
				}
			});

			// add the manager to the testcase garbage so it gets destroyed at tearDown
			this._garbage.push(this.manager);

			// start new empty survey
			this.manager.openNew();

			// selected locales
			this.manager.canvas.set('availableLocales', ['en_GB', 'nl_NL']);
		},

		/**
		 * Tests adding, moving of block/items by drag n drop
		 */
		testMoveAddDnd: function(){
			try {

				var Assert = YAHOO.util.Assert;
				var survey = this.manager.canvas;

				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);

				var b2 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b2);

				var b3 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b3);

				// empty survey already has a block so the new blocks have index 1 and 2 and total=3
				Assert.areSame(survey.children['blocks'][1], b1, 'b1 not equal');
				Assert.areEqual(survey.children['blocks'][2], b2, 'b2 not equal');
				Assert.areEqual(survey.children['blocks'].length, 4, 'length incorrect');

				var b1c1 = b1.children['columns'][0];
				var b2c1 = b2.children['columns'][0];

				var i1 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b1c1.add('items', i1);

				var i2 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b1c1.add('items', i2);

				Assert.areEqual(b1c1.children['items'][0], i1, 'testSurveyModuleAdd - child order is not correct');
				Assert.areEqual(b1c1.children['items'][1], i2, 'testSurveyModuleAdd - child order is not correct');
				Assert.areEqual(b1c1.children['items'].length, 2, 'testSurveyModuleAdd - child order is not correct');

				var i3 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b2c1.add('items', i3);

				Assert.areEqual(b1c1.children['items'][0], i1, 'testSurveyModuleAdd - child order is not correct');
				Assert.areEqual(b1c1.children['items'][1], i2, 'testSurveyModuleAdd - child order is not correct');
				Assert.areEqual(b1c1.children['items'].length, 2, 'testSurveyModuleAdd - child order is not correct');
				Assert.areEqual(b2c1.children['items'][0], i3, 'testSurveyModuleAdd - child order is not correct');
				Assert.areEqual(b2c1.children['items'].length, 1, 'testSurveyModuleAdd - child order is not correct');

				// removing the last column should remove the whole block
				b3.children['columns'][0].remove();
				console.warn(survey.children['blocks']);
				Assert.areEqual(3, survey.children['blocks'].length);

			}
			catch (e) {
				console.error(e);
				throw e;
			}

		},

		/*
		 * This one tests movement between parents that already have children
		 */
		testSurveyModuleMovementToChildren: function(){
			try {

				var Assert = YAHOO.util.Assert;
				var survey = this.manager.canvas;

				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);

				var b2 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b2);

				var b1c1 = b1.children['columns'][0];
				var b2c1 = b2.children['columns'][0];

				var i1 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b1c1.add('items', i1);

				var i2 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b1c1.add('items', i2);

				var i3 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b2c1.add('items', i3);

				// move i2 to b2c1
				b2c1.append(i2);

				Assert.areEqual(b2c1.children['items'][0], i3, 'testSurveyModuleMovement - block item order is incorrect after movement');
				Assert.areEqual(b2c1.children['items'][1], i2, 'testSurveyModuleMovement - block item order is incorrect after movement');
				Assert.areEqual(b2c1.children['items'].length, 2, 'testSurveyModuleMovement - block items are not removed after move');
				Assert.areEqual(b1c1.children['items'][0], i1, 'testSurveyModuleMovement - block item order is incorrect after movement');
				Assert.areEqual(b1c1.children['items'].length, 1, 'testSurveyModuleMovement - block items are not removed after move');

			}
			catch (e) {
				console.error(e);
				throw e;
			}
		},

		/*
		 * This one tests movement to an empty parent module
		 */
		testSurveyModuleMovementToEmpty: function(){

			try {

				var Assert = YAHOO.util.Assert;
				var survey = this.manager.canvas;

				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);

				var b2 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b2);

				var b1c1 = b1.children['columns'][0];
				var b2c1 = b2.children['columns'][0];

				var i1 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b1c1.add('items', i1);

				var i2 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b1c1.add('items', i2);

				b2c1.append(i2);

				Assert.areEqual(b1c1.children['items'][0], i1, 'testSurveyModuleMovementToEmpty - block item order is incorrect after movement (1)');
				Assert.areEqual(b1c1.children['items'].length, 1, 'testSurveyModuleMovementToEmpty - block items are not removed after move (1)');

				Assert.areEqual(b2c1.children['items'][0], i2, 'testSurveyModuleMovementToEmpty - block item order is incorrect after movement (2)');
				Assert.areEqual(b2c1.children['items'].length, 1, 'testSurveyModuleMovementToEmpty - block items are not removed after move (2)');

			}
			catch (e) {
				console.error(e);
				throw e;
			}
		},

		testSurveyModuleDuplicate: function(){
			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			// Add a block, item and change the item text
			var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
			survey.add('blocks', b1);

			var b1c1 = b1.children['columns'][0];

			var i1 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
			b1c1.add('items', i1);

			// duplicate the item and assert there are two clilds now
			i1.duplicate();
			Assert.areEqual(b1c1.children['items'].length, 2, 'testSurveyModuleDuplicate - duplicate failed');

			// call undo and assert there is only one
			//var h = this.manager.history;
			//h.undo();
			//Assert.areEqual(b1c1.children['items'].length, 1, 'testSurveyModuleDuplicate - undo failed');

			// redo and assert two childs
			//h.redo();
			Assert.areEqual(b1c1.children['items'].length, 2, 'testSurveyModuleDuplicate - undo failed');
		},

		testGetParent: function(){

			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {

				// Add some blocks
				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);
				var b2 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b2);
				var b3 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b3);

				// add a column and get references
				b1.add('columns', new BASESOFT.iquest.widget.survey.Column(this.manager));
				var b1c1 = b1.children['columns'][0];
				var b1c2 = b1.children['columns'][1];
				b2.add('columns', new BASESOFT.iquest.widget.survey.Column(this.manager));
				var b2c1 = b2.children['columns'][0];
				var b2c2 = b2.children['columns'][1];

				// add an item
				var i1 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b1c1.add('items', i1);

				// assert that the parent of this block is the canvas
				Assert.areSame(this.manager.getParent(b1), this.manager.canvas);

				// b2c2's parent is b2
				Assert.areSame(this.manager.getParent(b2c2), b2);

				// i1's parent is b1c1
				Assert.areSame(this.manager.getParent(i1), b1c1);

				// but i1's parent isnt the canvas
				Assert.areNotSame(this.manager.getParent(i1), this.manager.canvas);

			}
			catch (e) {
				console.error(e);
				throw e;
			}
		},

		testDropDownEdit: function(){
			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {

				// set compose to false so we cant edit but we can clicky on the answer for selecting
				//survey.set('compose', false);
				var QA = BASESOFT.iquest.widget.survey.ItemQADropDown;

				// Add some blocks
				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);

				var b1c1 = b1.children['columns'][0];

				// singleChoice, that should create two default answers
				var qa1 = new QA(this.manager, {
					'answersAlign': QA.prototype.ALIGN_TOP,
					'questionsAlign': QA.prototype.ALIGN_LEFT
				});
				b1c1.add('items', qa1);
			}
			catch (e) {
				console.error(e);
				throw e;
			}

			// before ending the test (before cleanup) wait a bit to let async stuff do its job
			BASESOFT.util.setTimeout(function() {
				this.resume(function() {});
			}, 1000, null, this);
			this.wait();

		},

		/**
		 * Creates different types of questions and verifies its behaviour
		 */
		testQuestions: function(){

			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {

				// set compose to false so we cant edit but we can clicky on the answer for selecting
				//survey.set('compose', false);

				// Add some blocks
				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);
				var b2 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b2);
				var b3 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b3);

				// add a column and get references
				b1.add('columns', new BASESOFT.iquest.widget.survey.Column(this.manager));
				b1.add('columns', new BASESOFT.iquest.widget.survey.Column(this.manager));
				var b1c1 = b1.children['columns'][0];
				var b1c2 = b1.children['columns'][1];
				var b1c3 = b1.children['columns'][2];

				b2.add('columns', new BASESOFT.iquest.widget.survey.Column(this.manager));
				var b2c1 = b2.children['columns'][0];
				var b2c2 = b2.children['columns'][1];

				// text item on top
				var i1 = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b1c1.add('items', i1);

				// singleChoice, that should create two default answers
				var qa1 = new BASESOFT.iquest.widget.survey.ItemQASingleChoice(this.manager);
				b1c1.add('items', qa1);
				Assert.areEqual(1, qa1.children['questions'].length, 'ItemQASingleChoice - default question failed');
				Assert.areEqual(2, qa1.children['answers'].length, 'ItemQASingleChoice - default answers failed');

				// multipleChoice in the second column
				var qa2 = new BASESOFT.iquest.widget.survey.ItemQAMultipleChoice(this.manager, {
					'showTitle': false
				});
				b1c2.add('items', qa2);
				//qa2.set('answersAlign', qa2.ALIGN_TOP)

				// text question in the second block
				var qa3 = new BASESOFT.iquest.widget.survey.ItemQATextInput(this.manager);
				b2c1.add('items', qa3);

				// text question with textarea layout in second block, second column
				var qa4 = new BASESOFT.iquest.widget.survey.ItemQATextInput(this.manager, {
					"layout": 4
				});
				b2c2.add('items', qa4);

				// matrix singleChoice in the third block
				var qa_matrix = new BASESOFT.iquest.widget.survey.ItemQASingleChoice(this.manager, {
					'children': {
						'questions': [{
							objectName: 'SurveyQuestion'
						}, {
							objectName: 'SurveyQuestion'
						}, {
							objectName: 'SurveyQuestion'
						}]
					}
				});
				b3.children['columns'][0].add('items', qa_matrix);

				// trailing
				var QA = BASESOFT.iquest.widget.survey.ItemQASingleChoice;
				var qa_trailing = new QA(this.manager, {
					'locales': {
						'nl_NL': {
							'label': 'How happy are you?'
						}
					},
					'answersAlign': QA.prototype.ALIGN_TOP,
					'questionsAlign': QA.prototype.ALIGN_LEFT,
					'trailingQuestions': true,

					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'Miserable',
									trailing_label: 'Euphoric'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: '1'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: '2'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: '3'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 14,
							locales: {
								nl_NL: {
									label: '4'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 15,
							locales: {
								nl_NL: {
									label: '5'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 16,
							locales: {
								nl_NL: {
									label: '6'
								}
							}
						}]
					}
				});
				b3.children['columns'][0].add('items', qa_trailing);



				var qa_trailing_vertical = new QA(this.manager, {
					'locales': {
						'nl_NL': {
							'label': 'How happy are you?'
						}
					},
					'answersAlign': QA.prototype.ALIGN_RIGHT,
					'questionsAlign': QA.prototype.ALIGN_TOP,
					'trailingQuestions': true,

					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'Miserable',
									trailing_label: 'Euphoric'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: '1'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: '2'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: '3'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 14,
							locales: {
								nl_NL: {
									label: '4'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 15,
							locales: {
								nl_NL: {
									label: '5'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 16,
							locales: {
								nl_NL: {
									label: '6'
								}
							}
						}]
					}
				});
				b3.children['columns'][0].add('items', qa_trailing_vertical);

				// trailing inline
				var qa_trailing_inline = new BASESOFT.iquest.widget.survey.ItemQATextInput(this.manager, {
					'locales': {
						'nl_NL': {
							'label': 'Fill in the blank'
						}
					},
					'answersAlign': QA.prototype.ALIGN_TOP,
					'questionsAlign': QA.prototype.ALIGN_LEFT,
					'trailingQuestions': true,

					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'I am',
									trailing_label: 'years old.'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {}
						}]
					}
				});
				b3.children['columns'][0].add('items', qa_trailing_inline);

				// slider horizontal
				QA = BASESOFT.iquest.widget.survey.ItemQASingleChoice;
				var qa_trailing_slider = new QA(this.manager, {
					'locales': {
						'nl_NL': {
							'label': 'How happy are you?'
						}
					},
					'answersAlign': QA.prototype.ALIGN_TOP,
					'questionsAlign': QA.prototype.ALIGN_LEFT,

					'layout': QA.prototype.LAYOUT_SLIDER,
					'trailingQuestions': true,

					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'I feel',
									trailing_label: '.. ok?'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: 'Miserable'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: 'Sad'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: 'Normal'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 14,
							locales: {
								nl_NL: {
									label: 'Happy'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 15,
							locales: {
								nl_NL: {
									label: 'Euphoric'
								}
							}
						}]
					}
				});
				b3.children['columns'][0].add('items', qa_trailing_slider);

				// slider vertical
				var qa_trailing_slider_vertical = new QA(this.manager, {
					'locales': {
						'nl_NL': {
							'label': 'How happy are you?'
						}
					},
					'answersAlign': QA.prototype.ALIGN_LEFT,
					'questionsAlign': QA.prototype.ALIGN_TOP,
					'trailingQuestions': true,

					'layout': QA.prototype.LAYOUT_SLIDER,

					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'Yesterday',
									trailing_label: '.. ok?'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 2,
							locales: {
								nl_NL: {
									label: 'Today',
									trailing_label: '.. ok?'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 2,
							locales: {
								nl_NL: {
									label: 'Tomorrow',
									trailing_label: '.. ok?'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: 'Miserable'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: 'Sad'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: 'Normal'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 14,
							locales: {
								nl_NL: {
									label: 'Happy'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 15,
							locales: {
								nl_NL: {
									label: 'Euphoric'
								}
							}
						}]
					}
				});
				b3.children['columns'][0].add('items', qa_trailing_slider_vertical);


				// single choice with select input
				var qa_dropdown_vertical = new BASESOFT.iquest.widget.survey.ItemQASingleChoice(this.manager, {
					'layout': QA.prototype.LAYOUT_DROPDOWN,
					'answersAlign': QA.prototype.ALIGN_TOP,
					'questionsAlign': QA.prototype.ALIGN_LEFT,
					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: 'Option A'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: 'Option B'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: 'Option C'
								}
							}
						}]
					}
				});
				b3.children['columns'][0].add('items', qa_dropdown_vertical);

				// single choice with select input
				var qa_dropdown_horizontal = new BASESOFT.iquest.widget.survey.ItemQASingleChoice(this.manager, {
					'layout': QA.prototype.LAYOUT_DROPDOWN,
					'answersAlign': QA.prototype.ALIGN_LEFT,
					'questionsAlign': QA.prototype.ALIGN_TOP,
					'trailingQuestions': true,

					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'Yesterday',
									trailing_label: '.. ok?'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 2,
							locales: {
								nl_NL: {
									label: 'Today',
									trailing_label: '.. ok?'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: 'Option A'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: 'Option B'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: 'Option C'
								}
							}
						}]
					}
				});
				b3.children['columns'][0].add('items', qa_dropdown_horizontal);
			}
			catch (e) {
				console.error(e);
				throw e;
			}

			// before ending the test (before cleanup) wait a bit to let async stuff do its job
			BASESOFT.util.setTimeout(function() {
				this.resume(function() {});
			}, 1000, null, this);
			this.wait();

		},

		testQuestionMatrix: function(){
			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {

				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);

				// add a basic matrix question
				// for testing purposes we start question id's at 1 and answer id's at 11..
				// in realitity it could well be that they overlapping (they are differnet thing anyway)
				var qa = new BASESOFT.iquest.widget.survey.ItemQASingleChoice(this.manager, {
					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'A'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 2,
							locales: {
								nl_NL: {
									label: 'B'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 3,
							locales: {
								nl_NL: {
									label: 'C'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 4,
							locales: {
								nl_NL: {
									label: 'D'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 5,
							locales: {
								nl_NL: {
									label: 'E'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 6,
							locales: {
								nl_NL: {
									label: 'F'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 7,
							locales: {
								nl_NL: {
									label: 'G'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: '1'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: '2'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: '3'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 14,
							locales: {
								nl_NL: {
									label: '4'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 15,
							locales: {
								nl_NL: {
									label: '5'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 16,
							locales: {
								nl_NL: {
									label: '6'
								}
							}
						}]
					}
				});

				b1.children['columns'][0].add('items', qa);
				qa.activate();

				// show properties panel for debugging
				qa.showPropertiesPanel();

				// wait for the first input to be available and continue test
				YAHOO.util.Event.onAvailable(qa.id + '-c0-r0', function() {
					this.resume(function() {
						try {



							// function select answers in the matrix taking row and column index
							var check_qa = function(rowIndex, colIndex){
								// yui-gen68-c0-r0
								//console.warn('check_qa', qa.id, colIndex, rowIndex)
								var domID = qa.id + '-c' + colIndex + '-r' + rowIndex;
								console.warn('domID', domID);
								YAHOO.util.Dom.get(domID).checked = true;
							};

							// select some radio's in an inclined way. oh and do the last question manual
							var i;
							for (i = 0; i < 6; i++) {
								check_qa(i, i);
							}
							check_qa(2, 6);

							// this is what we expect to come out
							var expected_values = [{
								q: 1,
								a: 11,
								v: true
							}, {
								q: 1,
								a: 12,
								v: null
							}, {
								q: 1,
								a: 13,
								v: null
							}, {
								q: 1,
								a: 14,
								v: null
							}, {
								q: 1,
								a: 15,
								v: null
							}, {
								q: 1,
								a: 16,
								v: null
							}, {
								q: 2,
								a: 11,
								v: null
							}, {
								q: 2,
								a: 12,
								v: true
							}, {
								q: 2,
								a: 13,
								v: null
							}, {
								q: 2,
								a: 14,
								v: null
							}, {
								q: 2,
								a: 15,
								v: null
							}, {
								q: 2,
								a: 16,
								v: null
							}, {
								q: 3,
								a: 11,
								v: null
							}, {
								q: 3,
								a: 12,
								v: null
							}, {
								q: 3,
								a: 13,
								v: true
							}, {
								q: 3,
								a: 14,
								v: null
							}, {
								q: 3,
								a: 15,
								v: null
							}, {
								q: 3,
								a: 16,
								v: null
							}, {
								q: 4,
								a: 11,
								v: null
							}, {
								q: 4,
								a: 12,
								v: null
							}, {
								q: 4,
								a: 13,
								v: null
							}, {
								q: 4,
								a: 14,
								v: true
							}, {
								q: 4,
								a: 15,
								v: null
							}, {
								q: 4,
								a: 16,
								v: null
							}, {
								q: 5,
								a: 11,
								v: null
							}, {
								q: 5,
								a: 12,
								v: null
							}, {
								q: 5,
								a: 13,
								v: null
							}, {
								q: 5,
								a: 14,
								v: null
							}, {
								q: 5,
								a: 15,
								v: true
							}, {
								q: 5,
								a: 16,
								v: null
							}, {
								q: 6,
								a: 11,
								v: null
							}, {
								q: 6,
								a: 12,
								v: null
							}, {
								q: 6,
								a: 13,
								v: null
							}, {
								q: 6,
								a: 14,
								v: null
							}, {
								q: 6,
								a: 15,
								v: null
							}, {
								q: 6,
								a: 16,
								v: true
							}, {
								q: 7,
								a: 11,
								v: null
							}, {
								q: 7,
								a: 12,
								v: null
							}, {
								q: 7,
								a: 13,
								v: true
							}, {
								q: 7,
								a: 14,
								v: null
							}, {
								q: 7,
								a: 15,
								v: null
							}, {
								q: 7,
								a: 16,
								v: null
							}];

							// now ask the qa and assert responses value
							var actual_values = qa.getResponsesValues();

							// assert if all questions are present then loop them
							YAHOO.util.Assert.areEqual(expected_values.length, actual_values.length);
							for (i in expected_values) {

								// this checks if the keys (q, a, v) match
								YAHOO.util.ObjectAssert.propertiesAreEqual(expected_values[i], actual_values[i]);

								// check if the key-values values match
								YAHOO.util.Assert.areEqual(expected_values[i].q, actual_values[i].q);
								YAHOO.util.Assert.areEqual(expected_values[i].a, actual_values[i].a);
								YAHOO.util.Assert.areEqual(expected_values[i].v, actual_values[i].v);
							}

						} catch (e) {
							console.error(e);
						}

					});
				}, null, this);
				this.wait();


			}
			catch (e) {
				console.error(e);
				throw e;
			}
		},

		testAlign: function(){
			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {

				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);

				// add a golom
				var c1 = new BASESOFT.iquest.widget.survey.Column(this.manager);
				b1.add('columns', c1);

				// 1q QA - small texts
				var qa1 = new BASESOFT.iquest.widget.survey.ItemQASingleChoice(this.manager, {
					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'a'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: '1'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: '2'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: '3'
								}
							}
						}]
					}
				});
				b1.children['columns'][0].add('items', qa1);

				// 1q QA - big texts
				var qa2 = new BASESOFT.iquest.widget.survey.ItemQATextInput(this.manager, {
					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'aaaaa'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: '11111'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: '22222'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: '33333'
								}
							}
						}]
					}
				});
				b1.children['columns'][0].add('items', qa2);

				// matrix small texts
				var m1 = new BASESOFT.iquest.widget.survey.ItemQASingleChoice(this.manager, {
					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'a'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 2,
							locales: {
								nl_NL: {
									label: 'b'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 3,
							locales: {
								nl_NL: {
									label: 'c'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: '1'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: '2'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: '3'
								}
							}
						}]
					}
				});
				b1.children['columns'][0].add('items', m1);

				// matrix big
				var m2 = new BASESOFT.iquest.widget.survey.ItemQASingleChoice(this.manager, {
					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'aaaaa grrrrr bleee wrap weee oiii'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 2,
							locales: {
								nl_NL: {
									label: 'bbbbb'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 3,
							locales: {
								nl_NL: {
									label: 'ccccc'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: '11111'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: '22222'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: '33333'
								}
							}
						}]
					}
				});

				b1.children['columns'][0].add('items', m2);

			}
			catch (e) {
				console.error(e);
				throw e;
			}

			// before ending the test (before cleanup) wait a bit to let async stuff do its job
			BASESOFT.util.setTimeout(function() {
				this.resume(function() {});
			}, 1000, null, this);
			this.wait();


		},

		testColumns: function(){

			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {
				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);
				Assert.areEqual(1, b1.children['columns'].length, 'block should have one default column');

				var b2 = new BASESOFT.iquest.widget.survey.Block(this.manager, {
					children: {
						'columns': []
					}
				});
				survey.add('blocks', b2);
				Assert.areEqual(0, b2.children['columns'].length, 'block shouldnt have columns');

				var c1 = new BASESOFT.iquest.widget.survey.Column(this.manager);
				b1.add('columns', c1);
				var c2 = new BASESOFT.iquest.widget.survey.Column(this.manager);
				b1.add('columns', c2);

				Assert.areEqual(3, b1.children['columns'].length, 'block should have 3 columns');

			}
			catch (e) {
				console.error(e);
				throw e;
			}
		},

		testColumnWidth: function(){

			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {

				var b1 = survey.children['blocks'][0];
				var c1 = b1.children['columns'][0];

				// add another block
				var b2 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b2);
				var c2 = b2.children['columns'][0];

				// add two extra columns
				var c3 = new BASESOFT.iquest.widget.survey.Column(this.manager);
				b2.add('columns', c3);
				var c4 = new BASESOFT.iquest.widget.survey.Column(this.manager);
				b2.add('columns', c4);

				// assert width of each column
				Assert.areEqual(100, c1.getColumnWidth());
				Assert.areEqual(33, c2.getColumnWidth());
				Assert.areEqual(33, c3.getColumnWidth());
				Assert.areEqual(33, c4.getColumnWidth());

				// move a column to b1
				b1.append(c2);
				Assert.areEqual(50, c1.getColumnWidth());
				Assert.areEqual(50, c2.getColumnWidth());
				Assert.areEqual(50, c3.getColumnWidth());
				Assert.areEqual(50, c4.getColumnWidth());

				// set width on a column
				c1.set('width', 40);
				Assert.areEqual(40, c1.getColumnWidth());
				Assert.areEqual(60, c2.getColumnWidth());
				Assert.areEqual(50, c3.getColumnWidth());
				Assert.areEqual(50, c4.getColumnWidth());

				// move another column to first block
				b1.append(c3);
				Assert.areEqual(40, c1.getColumnWidth());
				Assert.areEqual(30, c2.getColumnWidth());
				Assert.areEqual(30, c3.getColumnWidth());
				Assert.areEqual(100, c4.getColumnWidth());

				// user width passes 100
				c1.set('width', null);
				c2.set('width', null);
				c3.set('width', 80);
				c4.set('width', 40);
				b2.append(c3);
				Assert.areEqual(50, c1.getColumnWidth());
				Assert.areEqual(50, c2.getColumnWidth());
				Assert.areEqual(66, c3.getColumnWidth()); //  = 80 / 1.2
				Assert.areEqual(33, c4.getColumnWidth()); // = 40 / 1.2
				// same as above but append a column without width..
				// that one should takes 20 and the rest should resize accordingly
				b2.append(c2);
				Assert.areEqual(100, c1.getColumnWidth());
				Assert.areEqual(26, c4.getColumnWidth());
				Assert.areEqual(53, c3.getColumnWidth());
				Assert.areEqual(20, c2.getColumnWidth());


			}
			catch (e) {
				console.error(e);
				throw e;
			}

		},


		testPageEnd: function(){

			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {

				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager, {
					'pageEnd': true
				});
				survey.add('blocks', b1);

				var b2 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b2);

				var b3 = new BASESOFT.iquest.widget.survey.Block(this.manager, {
					'pageEnd': true
				});
				survey.add('blocks', b3);

				var b4 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b4);


			}
			catch (e) {
				console.error(e);
				throw e;
			}
		},

		testLocale: function(){

			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;
			var locale = this.manager.canvas.get('locale');

			try {

				// add some extra available locales
				survey.set('availableLocales', ['en_GB', 'nl_NL', 'de_DE']);

				// Add a block, item and change the item text
				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);
				var b1c1 = b1.children['columns'][0];

				// Add a simple item.. this should show the default text for the current locale
				var item = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b1c1.add('items', item);
				Assert.areEqual(item.texts['html'].initial, item.elText.innerHTML, 'initial text failed');

				// Add a text item
				item = new BASESOFT.iquest.widget.survey.ItemText(this.manager, {
					locales: {
						'nl_NL': {
							'html_html': 'text for nl_NL',
							'html_wiki': 'text for nl_NL'
						},
						'en_GB': {
							'html_html': 'text for en_GB',
							'html_wiki': 'text for nl_NL'
						}
					}
				});
				b1c1.add('items', item);

				// verify default dutch locale
				Assert.areEqual('text for nl_NL', item.elText.innerHTML, 'item should have dutch language');

				// change language and verify
				this.manager.canvas.changeLocale('en_GB');
				Assert.areEqual('text for en_GB', item.elText.innerHTML, 'item should have english language');

				// now switch to german and we should see the english fallback
				this.manager.canvas.changeLocale('de_DE');
				Assert.areEqual('text for nl_NL', item.elText.innerHTML, 'item should have dutch fallback language');

				// change the german text and update
				var locales = item.get('locales');
				locales['de_DE'] = {};
				locales['de_DE']['html_html'] = 'text for de_DE';
				locales['de_DE']['html_wiki'] = 'text for de_DE';
				item.set('locales', locales);
				Assert.areEqual('text for de_DE', item.elText.innerHTML, 'item should have german language');

				// change back to en_GB and add a QA
				this.manager.canvas.changeLocale('en_GB');
				var qa = new BASESOFT.iquest.widget.survey.ItemQASingleChoice(this.manager, {
					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								en_GB: {
									label: 'Whats up?'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								en_GB: {
									label: 'Alot'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								en_GB: {
									label: 'Not much'
								}
							}
						}]
					}
				});
				b1c1.add('items', qa);

			}
			catch (e) {
				console.error(e);
				throw e;
			}

			// before ending the test (before cleanup) wait a bit to let async stuff do its job
			BASESOFT.util.setTimeout(function() {
				this.resume(function() {});
			}, 1000, null, this);
			this.wait();

		},

		testTranslations: function(){
			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {

				// add some extra available locales
				survey.set('availableLocales', ['en_GB', 'nl_NL', 'de_DE']);
				this.manager.canvas.changeLocale('en_GB');

				// Add a block with one column
				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);
				var b1c1 = b1.children['columns'][0];

				// Just test with a simple item, this should be enough
				var item = new BASESOFT.iquest.widget.survey.ItemText(this.manager);
				b1c1.add('items', item);
				Assert.areEqual(item.texts['html'].initial, item.elText.innerHTML, 'initial text failed');

				var item_locales = {
					'nl_NL': {
						'html_html': 'text for nl_NL',
						'html_wiki': 'text for nl_NL'
					},
					'en_GB': {
						'html_html': 'text for en_GB',
						'html_wiki': 'text for en_GB'
					}
				};


				// Add a text item
				item = new BASESOFT.iquest.widget.survey.ItemText(this.manager, {
					locales: item_locales
				});
				b1c1.add('items', item);

				// open the translations panel for this item
				var tp = new BASESOFT.widget.compose.TranslationsPanel(item);
				this._garbage.push(tp);

				// verify form values
				var vals = tp.form.getFormValues();
				var name_l1 = tp._prefix_language1 + item.get('domID') + 'html';
				var name_l2 = tp._prefix_language2 + item.get('domID') + 'html';
				var locale1 = vals['language_1'];
				var locale2 = vals['language_2'];

				// first locale should be the current one (which was set above to 'en_GB')
				Assert.areEqual(locale1, 'en_GB');

				Assert.areEqual(vals[name_l1], item_locales[locale1].html_wiki);
				Assert.areEqual(vals[name_l2], item_locales[locale2].html_wiki);

				// now change form values
				var new_vals = {};
				new_vals[name_l1] = 'new text for language ' + locale1;
				new_vals[name_l2] = 'new text for language ' + locale2;

				tp.form.setFormValues(new_vals);

				tp._updateLocales(locale1, locale2);

				var locales = item.get('locales');

				Assert.areEqual(new_vals[name_l1], locales[locale1].html_wiki);
				Assert.areEqual(new_vals[name_l2], locales[locale2].html_wiki);

				// now check if texts in canvas have changed (current language is en)
				// FIXME: this currently is wiki text and should be HTML
				Assert.areEqual(item.elText.innerHTML, new_vals[name_l1]);

			}
			catch (e) {
				console.error(e);
				throw e;
			}

			// before ending the test (before cleanup) wait a bit to let async stuff do its job
			BASESOFT.util.setTimeout(function() {
				this.resume(function() {});
			}, 1000, null, this);
			this.wait();

		},

		testSaveUpdateLoad: function(){
			try {
				var Assert = YAHOO.util.Assert;
				var survey = this.manager.canvas;

				Assert.areEqual(null, survey.get('dbID'), 'survey shouldnt have an id yet');

				// create a simple survey
				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);
				var b1c1 = b1.children['columns'][0];
				var item = new BASESOFT.iquest.widget.survey.ItemText(this.manager, {
					locales: {
						'nl_NL': {
							'html': 'text for nl_NL'
						},
						'en_GB': {
							'html': 'text for en_GB'
						}
					}
				});
				b1c1.add('items', item);

				var qa = new BASESOFT.iquest.widget.survey.ItemQA(this.manager);
				b1c1.add('items', qa);

				// login so we are allowed to save
				var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/',
									{executeIfLoggedOut: true});
				loginrpc.call('login', ['user_compose', 'pass_compose'], function(res){
					this.resume(function(){
						console.debug('logged in.. resuming test..');
						BASESOFT.env.account.id = res.id;

						// now save the survey
						var event = this.manager.save();
						event.subscribe(function(){

							// resume after survey saved
							this.resume(function(){

								try {

									// we arrive here when the survey is saved.. lets verify if all modules have an id.. recursivly
									var survey_db_id = survey.get('dbID');
									var assert_module = function(module){
										console.debug('Assert.areNotEqual', null, module.get('dbID'));
										Assert.areNotEqual(null, module.get('dbID'), module + ' should have an id now');

										for (var j in module.CHILDREN_GROUP_NAMES) {
											var group_name = module.CHILDREN_GROUP_NAMES[j];
											for (var i = 0; i < module.children[group_name].length; i++) {
												assert_module.call(this, module.children[group_name][i]);
											}
										}
									};
									assert_module.call(this, survey);

									// change some text and save again
									var block_index = 1; // 1 cuz survey starts with 1 block
									var column_index = 0;
									var item_index = 0;
									var item = survey.children['blocks'][block_index].children['columns'][column_index].children['items'][item_index];
									var locales = item.get('locales');
									locales['nl_NL']['html'] = 'new updated text';
									item.set('locales', locales);

									// make the server update and wait for it to return
									var event = this.manager.save();
									event.subscribe(function(){

										// resume after survey saved
										this.resume(function(){
											console.warn('resume after update');
											try {

												// verify that the survey db id is the same as before the update
												Assert.areEqual(survey_db_id, survey.get('dbID'), ' survey should have dbID ' + survey_db_id);

												// wait for survey load
												var event = this.manager.load(survey.get('dbID'));
												event.subscribe(function(){

													// resume when survey is loaded
													this.resume(function(){
														console.debug('survey loaded... resuming test..');

														// TODO: assert here if survey has been successfully loaded

														// kill any growl leftovers
														BASESOFT.env.growler.killAll();

														// important when running this test it must finish with this message!
														console.info('!! testSaveUpdateLoad finished.');
													});

												}, null, this);

												// wait till survey is loaded
												console.warn('waiting for survey loaded');
												this.wait();

											}
											catch (e) {
												e.delay || console.error(e);
												throw e;
											}


										});

									}, null, this);

									// wait till survey is updated
									console.warn('waiting for update');
									this.wait();

								}
								catch (e) {
									e.delay || console.error(e);
									throw e;
								}

								console.warn('waiting for second save');

								this.wait();
							});

						}, null, this);

						console.warn('waiting for first survey saved..');
						this.wait();
					});

				}, null, this);
			}
			catch (e) {
				console.error(e);
				throw e;
			}

			// wait for login callback
			console.debug('waiting for login..');
			this.wait();
		},

		testSaveDuplicate: function() {

			try {

				var Assert = YAHOO.util.Assert;
				var survey = this.manager.canvas;

				// create a simple survey
				var b1c1 = survey.children['blocks'][0].children['columns'][0];

				var qa1 = new BASESOFT.iquest.widget.survey.ItemQA(this.manager);
				b1c1.add('items', qa1);

				// login so we are allowed to save
				var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
				loginrpc.call('login', ['user_compose', 'pass_compose'], function(res){
					this.resume(function(){
						console.debug('logged in.. resuming test..');
						BASESOFT.env.account.id = res.id;

						// now save the survey
						var event = this.manager.save();
						event.subscribe(function(){

							// resume after survey saved
							this.resume(function(){

								try {

									console.warn('! test: duplicate the first qa');
									var qa2 = qa1.duplicate();

									// now save the survey again
									console.warn('! test: saving');
									event = this.manager.save();
									event.subscribe(function(){
										this.resume(function() {

											// duplicate and save
											console.warn('! test: duplicate');
											var qa3 = qa2.duplicate();

											// now save the survey again
											console.warn('! test: save');
											event = this.manager.save();
											event.subscribe(function(){
												this.resume(function() {

													// reload survey
													console.warn('! test: reload survey');
													var event = this.manager.load(survey.get('dbID'));
													event.subscribe(function(){
														this.resume(function() {

															survey = this.manager.canvas;

															var col = survey.children['blocks'][0].children['columns'][0];

															// assert 3 questions
															Assert.areSame(3, col.children['items'].length);

															// each question has 2 answers
															for (var i=0; i<3; i++) {
																var item = col.children['items'][i];
																Assert.areSame(1, item.children['questions'].length, 'item q'+i);
																Assert.areSame(2, item.children['answers'].length, 'item a'+i);
															}


														});

													}, null, this);
													this.wait();

												});

											}, null, this);
											this.wait();
										});

									}, null, this);
									this.wait();

								}
								catch (e) {
									e.delay || console.error(e);
									throw e;
								}

								console.warn('waiting for second save');

								this.wait();
							});

						}, null, this);

						console.warn('waiting for first survey saved..');
						this.wait();
					});

				}, null, this);
			}
			catch (e) {
				console.error(e);
				throw e;
			}

			// wait for login callback
			console.debug('waiting for login..');
			this.wait();


		},

		testProperties: function(){
			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;
			var locale = this.manager.canvas.get('locale');

			try {

				// set selected locales
				survey.set('availableLocales', ['en_GB', 'nl_NL']);

				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);
				var b1c1 = b1.children['columns'][0];

				survey.showPropertiesPanel();

				// TODO: test name change, themes etc

			}
			catch (e) {
				e.delay || console.error(e);
				throw e;
			}

			// before ending the test (before cleanup) wait a bit to let async stuff do its job
			BASESOFT.util.setTimeout(function() {
				this.resume(function() {});
			}, 1000, null, this);
			this.wait();

		},

		testNewQAFilters: function(){
			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {

				var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
				loginrpc.call('login', ['user_compose', 'pass_compose'], function(res){

					// resume after login
					this.resume(function(){
						BASESOFT.env.account.id = res.id;


						var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
						survey.add('blocks', b1);

						// Use a default text input.
						var qa = new BASESOFT.iquest.widget.survey.ItemQATextInput(this.manager);
						b1.children['columns'][0].add('items', qa);

						qa.subscribe('filtersPanelShown', function(){
							console.warn('received filtersPanelShown');

							this.resume(function(){
								// Not sure what to assert here, this was used for testing ticket #953
							});

						}, null, this);

						qa.showFiltersPanel();

						this.wait();
					});
				}, null, this);

			}
			catch (e) {
				console.error(e);
				throw e;
			}

			this.wait();


		},

		testFilterResponse: function(){
			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			try {

				var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
				survey.add('blocks', b1);

				// Use a default text input.
				var qa = new BASESOFT.iquest.widget.survey.ItemQATextInput(this.manager, {
					'children': {
						'questions': [{
							objectName: 'SurveyQuestion',
							dbID: 1,
							locales: {
								nl_NL: {
									label: 'A'
								}
							}
						}, {
							objectName: 'SurveyQuestion',
							dbID: 2,
							locales: {
								nl_NL: {
									label: 'B'
								}
							}
						}],
						'answers': [{
							objectName: 'SurveyAnswer',
							dbID: 11,
							locales: {
								nl_NL: {
									label: '1'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 12,
							locales: {
								nl_NL: {
									label: '2'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 13,
							locales: {
								nl_NL: {
									label: '3'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 14,
							locales: {
								nl_NL: {
									label: '4'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 15,
							locales: {
								nl_NL: {
									label: '5'
								}
							}
						}, {
							objectName: 'SurveyAnswer',
							dbID: 16,
							locales: {
								nl_NL: {
									label: '6'
								}
							}
						}]
					}
				});
				b1.children['columns'][0].add('items', qa);
				qa.activate();

				// wait a bit so the qa table is rendered
				var _this = this;
				setTimeout(function() {

					_this.resume(function() {

						var failures = {
							failures: ['Global failure'],
							q: [{
								id: 1,
								failures: ['Question level failure']
							}],
							a: [{
								id: 13,
								failures: ['Answer level failure A', 'Answer level failure B']
							}],
							qa: [{
								qid: 2,
								aid: 15,
								failures: ['Cell level failure']
							}]
						};

						qa.addFilterFailures(failures);
					});

				}, 1000);


			}
			catch (e) {
				console.error(e);
				throw e;
			}

			this.wait();

		},

		testFilters: function(){

			var Assert = YAHOO.util.Assert;
			var survey = this.manager.canvas;

			var composevars_rpc = new BASESOFT.util.JsonBroker('/iqportal.iquest.compose/rpc/', {executeIfLoggedOut: true});
			composevars_rpc.call('get_compose_vars', [], function(result){

				console.warn('result.existingFilters', result.existingFilters);

				BASESOFT.iquest.env.FILTERS = result.existingFilters;

				this.resume(function(){

					try {

						// first login
						var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
						loginrpc.call('login', ['user_compose', 'pass_compose'], function(res){

							// resume after login
							this.resume(function(){
								BASESOFT.env.account.id = res.id;

								var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
								survey.add('blocks', b1);

								// add a basic matrix question
								var qa = new BASESOFT.iquest.widget.survey.ItemQASingleChoice(this.manager, {
									'questionsAlign': 4,
									'answersAlign': 1,
									'children': {
										'questions': [{
											objectName: 'SurveyQuestion',
											dbID: 1,
											locales: {
												nl_NL: {
													label: 'Q1'
												}
											}
										}, {
											objectName: 'SurveyQuestion',
											dbID: 2,
											locales: {
												nl_NL: {
													label: 'Q2'
												}
											}
										}, {
											objectName: 'SurveyQuestion',
											dbID: 3,
											locales: {
												nl_NL: {
													label: 'Q3'
												}
											}
										}],
										'answers': [{
											objectName: 'SurveyAnswer',
											dbID: 4,
											locales: {
												nl_NL: {
													label: 'A1'
												}
											}
										}, {
											objectName: 'SurveyAnswer',
											dbID: 5,
											locales: {
												nl_NL: {
													label: 'A2'
												}
											}
										}, {
											objectName: 'SurveyAnswer',
											dbID: 6,
											locales: {
												nl_NL: {
													label: 'A3'
												}
											}
										}, {
											objectName: 'SurveyAnswer',
											dbID: 7,
											locales: {
												nl_NL: {
													label: 'A4'
												}
											}
										}]
									},
									// add an initial global numeric filter
									'filters': [{
										'dbID': BASESOFT.iquest.env.FILTERS.Numeric.id,
										'questionDbID': null,
										'answerDbID': null,
										'questionDomID': null,
										'answerDomID': null,
                                                                                'attributes': {
											'domain': 'int',
											'minValue': 1,
											'maxValue': 10,
											'sum': ''
										}
									}]
								});

								b1.children['columns'][0].add('items', qa);

								// now let's open the filter panel and check out if it has the proper filter information
								qa.filtersPanelShownEvent.subscribe(function(){

									this.resume(function(){

										// check that the dialog contains the global filter properly
										var itemFilter = qa.filtersDialog.getItemFilter(null, null); // params are the column and row index, both null for the global filter
                                                                                console.warn(itemFilter);
                                                                                console.warn("Got: " + itemFilter.dbID + " expected: " + BASESOFT.iquest.env.FILTERS.Numeric.id);
                                                                                YAHOO.util.Assert.areEqual(itemFilter.dbID, BASESOFT.iquest.env.FILTERS.Numeric.id);
										YAHOO.util.Assert.areEqual(itemFilter.attributes.domain, 'int');
										YAHOO.util.Assert.areEqual(itemFilter.attributes.minValue, 1);
										YAHOO.util.Assert.areEqual(itemFilter.attributes.maxValue, 10);

										// set a new filter for the whole first column (in this case we have answers on top)
										qa.filtersDialog.setFilterData(0, null, {
											dbID: BASESOFT.iquest.env.FILTERS.Email.id,
											attributes: {
												minAnswersRequired: 1,
												maxAnswersSupplied: 2
											}
										});

										// set a new filter for cell at second column, third row (A2, Q3)
										qa.filtersDialog.setFilterData(1, 2, {
											dbID: BASESOFT.iquest.env.FILTERS.Text.id,
											attributes: {
												minLength: 2,
												maxLength: 9,
												regexp: ''
											}
										});

										// now lets open the configuration panel for the global filter, to see if it shows the attributes
										var fcd = new BASESOFT.iquest.widget.survey.ItemQAFilterConfiguration(qa.filtersDialog, null, null);
										fcd.render();

										// checking the form to see that it rendered properly
										var fv = fcd.form.getFormValues();
										YAHOO.util.Assert.areEqual(fv.domain, 'int');
										YAHOO.util.Assert.areEqual(fv.minValue, 1);
										YAHOO.util.Assert.areEqual(fv.maxValue, 10);
										fcd.destroy();

										// save and close the filter window
										qa.filtersDialog.save();
										qa.filtersDialog.destroy();

										// get the filters part of the state
										var sf = qa.getState().filters;
										YAHOO.util.Assert.areEqual(sf.length, 3); // we have 3 filters
									        YAHOO.util.Assert.areEqual(sf[0].dbID, BASESOFT.iquest.env.FILTERS.Numeric.id); // id of the global filter
										YAHOO.util.Assert.areEqual(sf[0].answerDbID, null); // global filter has null as the answer id
										YAHOO.util.Assert.areEqual(sf[0].questionDbID, null); // global filter has null as the question id
										YAHOO.util.Assert.areEqual(sf[0].attributes.domain, 'int');
										YAHOO.util.Assert.areEqual(sf[0].attributes.maxValue, 10);
										YAHOO.util.Assert.areEqual(sf[0].attributes.minValue, 1);

										YAHOO.util.Assert.areEqual(sf[1].dbID, BASESOFT.iquest.env.FILTERS.Email.id); // id of the column filter
										YAHOO.util.Assert.areEqual(sf[1].answerDbID, 4); // column filter has the id of the answer
										YAHOO.util.Assert.areEqual(sf[1].questionDbID, null); // column filter has null as the question id
										YAHOO.util.Assert.areEqual(sf[1].attributes.minAnswersRequired, 1);
										YAHOO.util.Assert.areEqual(sf[1].attributes.maxAnswersSupplied, 2);

										YAHOO.util.Assert.areEqual(sf[2].dbID, BASESOFT.iquest.env.FILTERS.Text.id); // id of the cell filter
										YAHOO.util.Assert.areEqual(sf[2].answerDbID, 5); // cell filter has the id of the answer
										YAHOO.util.Assert.areEqual(sf[2].questionDbID, 3); // cell filter has the id of the question
										YAHOO.util.Assert.areEqual(sf[2].attributes.maxLength, 9);
										YAHOO.util.Assert.areEqual(sf[2].attributes.minLength, 2);
									});

								}, null, this);
                                                                qa.showFiltersPanel();
								this.wait(60000);

							});
						}, null, this);
					}
					catch (e) {
						console.error(e);
						throw e;
					}

					// wait for login callback
					this.wait(60000);
				});


			}, null, this);
			this.wait();


		}


	}));

})();


YAHOO.register("test_compose", null, {
	version: "1.0.0",
	build: "1"
});
