/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 *
 * Testcase for the IQuest Compose
 */
(function() { "use strict";

BASESOFT.env.iquest = {};

var localeList = [
	{name: 'Dutch', locale: 'nl_NL'},
	{name: 'English', locale: 'en_GB'},
	{name: 'Deutch', locale: 'de_DE'}
];

var themeList = [
	//{name: 'Debug', cssClass: 'survey-debug', module: 'iquest_survey_theme_debug'},
	{name: 'BaseSoft', cssClass: 'survey-basesoft', module: 'iquest_survey_theme_basesoft'},
	{name: 'TNT', cssClass: 'survey-tnt', module: 'iquest_survey_theme_tnt'}
];


BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "Publication Test",
	// assertClean: true, FIXME

	setUp : function () {

		// set some permissions
		BASESOFT.env.permissions._permissions['view_mail'] = true;
		BASESOFT.env.permissions._permissions['view_contact_list'] = true;

		// style the sandbox a bit
		var sandbox = YAHOO.util.Dom.get('sandbox');
		sandbox.innerHTML = '';

		var composition_el = document.createElement('div');
		sandbox.appendChild(composition_el);

		this.manager = new BASESOFT.iquest.compose.ComposeManager();
		this.manager.init({
			'composition_el': composition_el,
			'compose_vars': {
				'localeList': localeList,
				'themeList': themeList
			}
		});

		// add the manager to the testcase garbage so it gets destroyed at tearDown
		this._garbage.push(this.manager);

		// start new empty survey
		this.manager.openNew();

	},


	/**
	 * this gets called after we have a publication saved and gets the its state
	 * it will test setting a publication to scheduled status
	 */
	_testSurveyPublications: function(publication_state) {

		var openPublicationDetails = function(state) {

			var configs = {publication_state:  state};
			var panel = {
				xy: [50,50],
				visible: false,
				draggable: true,
				close: true,
				constraintoviewport: true,
				modal: true,
				width: "1100px"
			};

			var pd = new BASESOFT.compose.ComposePublicationDetails(panel, state.dbID, null);
			this._garbage.push(pd);

			// since the publication state was a basic one with no contact lists and mailings set,
			// the invitees and mailings should be empty
			pd.subscribe("onDetailsLoaded", function() {
				this.resume(function() {

					console.info('test: onDetailsLoaded corben');

					YAHOO.util.Assert.areEqual(pd.publicationState.mailings.length, 0);
					YAHOO.util.Assert.areEqual(pd.publicationState.contact_lists.length, 0);
				});
			}, null, this);

			console.info('test: waiting for onDetailsLoaded');

			this.wait(35000);
		};

		// first login
		var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
		loginrpc.call('login', ['user_compose', 'pass_compose'], function(result){

			// resume after login
			this.resume(function(){
				BASESOFT.env.account.id = result.id;

				console.info('test: logged in again');

				// save a survey
				var event = this.manager.save();
				event.subscribe(function(){
					this.resume(function(){

						console.info('test: saved survey again');

						var survey_state = this.manager.getState().survey;
						var configs = {survey_state: survey_state};
						var panel = {
							xy: [20,50],
							visible: false,
							draggable: true,
							close: true,
							constraintoviewport: true,
							modal: true,
							width: "1200px"
						};

						// open publication overview
						var spub = new BASESOFT.iquest.widget.survey.SurveyPublicationsManager(panel, configs);
						this._garbage.push(spub);

						// when publication status changed
						spub.subscribe("onPublicationStatusChanged", function(status) {
							this.resume(function() {

								console.info('test: onPublicationStatusChanged corben');

								YAHOO.util.Assert.areEqual(status, 1); // scheduled
								publication_state.status = status;

								// open the publication details
								openPublicationDetails.call(this, publication_state);
							});
						}, null, this);

						// when cq has loaded..
						spub.cq.subscribe('dataLoaded', function() {

							this.resume(function() {

								console.warn('received cq dataLoaded');

								// click on latest publication status
								var divs = YAHOO.util.Selector.query('div[class~=bs-publication-new]');
								YAHOO.util.UserAction.click(divs[divs.length-1]);

								// select second radio (scheduled)
								var radios = YAHOO.util.Selector.query('input[type=radio]');
								YAHOO.util.UserAction.click(radios[1]);
								YAHOO.util.UserAction.click(document.body);

								// wait for 'onPublicationStatusChanged'
								this.wait();

							});

						}, null, this);

						// wait for 'dataLoaded'
						this.wait();

					});
				},null, this);
				this.wait();
			});
		}, null, this);
		this._garbage.push(loginrpc);
		// wait for login callback
		this.wait();
	},

	/*
	 * This basicly tests the publication wizard, the publication details panel and scheduling a publication.
	 */
	testPublicationSaveAndLoad: function() {
		var publication_state = {
			dbID: null,
			domID: YAHOO.util.Dom.generateId(),
			begin_date: new Date(2009, 5, 5, 0, 3, 18),
			end_date: new Date(2009, 5, 15, 0, 3, 18),
			label: 'test publication 1',
			show_page_nrs: true,
			tags: '', // not implemented yet
			anonymous: true,
			status: 0, //new
			contact_lists: [],
			mailings: []
		};

		// first login
		var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
		loginrpc.call('login', ['user_compose', 'pass_compose'], function(result){

			// resume after login
			this.resume(function(){
				BASESOFT.env.account.id = result.id;

				console.info('test: logged in');

				// save a survey
				var event = this.manager.save();
				event.subscribe(function(){
					this.resume(function(){

						try {
							console.info('test: survey saved');

							var survey_state = this.manager.getState().survey;
							var pw = new BASESOFT.iquest.core.SurveyPublicationWizard();
							publication_state.survey_id = survey_state.dbID;
							pw.subscribe('onPagesCreated', function() {
								this.resume(function() {

									// we have a publication state saved in db
									// lets use it to open the publications details panel
									pw.subscribe('onPublicationCreated', function(state){
										this.resume(function(){

											console.info('test: publication created');

											// lets assert somethig from the initial and returned state
											YAHOO.util.Assert.areEqual(state.survey_id, publication_state.survey_id);
											YAHOO.util.Assert.areEqual(state.label, publication_state.label);
											YAHOO.util.DateAssert.datesAreEqual(state.begin_date, publication_state.begin_date);
											YAHOO.util.DateAssert.datesAreEqual(state.end_date, publication_state.end_date);
											YAHOO.util.Assert.areEqual(state.anonymous, publication_state.anonymous);
											YAHOO.util.Assert.areEqual(state.status, publication_state.status);

											// set this publication to scheduled
											// open the publication manager and pass it the new publication state
											// (which currently has status new)
											this._testSurveyPublications(state);

										});
									}, null, this);
									pw.save(publication_state);

									this.wait(35000);
								});
							}, null, this);
							this._garbage.push(pw);


						} catch (e) {
							console.error(e);
						}
						this.wait(35000);
					});
				}, null, this);
				this.wait(35000);
			});
		}, null, this);
		this._garbage.push(loginrpc);
		// wait for login callback
		this.wait(35000);
	},

	testVariables: function()  {

		var survey = this.manager.canvas;
		survey.set('availableLocales', ['en_GB', 'nl_NL']);

		var b1 = new BASESOFT.iquest.widget.survey.Block(this.manager);
		survey.add('blocks', b1);
		var b1c1 = b1.children['columns'][0];

		var item = new BASESOFT.iquest.widget.survey.ItemText(this.manager, {
			locales: {
				'nl_NL': {
					'html_wiki': 'survey titel {{product_name}} and {{other_product_name}}',
					'html_html': 'survey titel {{product_name}} and {{other_product_name}}'
				},
				'en_GB': {
					'html_wiki': 'survey title {{product_name}}',
					'html_html': 'survey title {{product_name}}'
				}
			}
		});
		b1c1.add('items', item);

		// first login
		var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
		loginrpc.call('login', ['user_compose', 'pass_compose'], function(result){
			// resume after login
			this.resume(function(){
				BASESOFT.env.account.id = result.id;

				// save a survey
				var event = this.manager.save();
				event.subscribe(function(){

					// resume after survey saved
					this.resume(function(){
						var survey_state = this.manager.getState().survey;

						var configs = {survey_state: survey_state, variable_state: {}};
						try {
							var panel = new BASESOFT.iquest.widget.survey.SurveyVariableDetails({}, configs);
							this._garbage.push(panel);

							this._testVariablePanel(panel);
						}
						catch (e) {
							e.delay || console.error(e);
							throw e;
						}

					});
				}, null, this);
				console.debug('waiting for survey save');
				this.wait(35000);
			});
		}, null, this);
		this._garbage.push(loginrpc);
		// wait for login callback
		console.debug('waiting for login');
		this.wait(35000);

	},

	_testVariablePanel: function(panel){

		console.info('_testVariablePanel()');

		var cbVarLoaded_NL = function(){
			console.warn('cbVarLoaded_NL');
			this.resume(function(){

				panel.unsubscribe('variablesLoaded', cbVarLoaded_NL);

				try {


					// fill basic form
					panel.form_basic.getChildByName('name').input.set('value', 'Yello');
					panel.form_locale.getChildByName('product_name').input.set('value', 'Heineken');
					panel.form_locale.getChildByName('other_product_name').input.set('value', 'San Miguel');

					var ev = panel.save();
					ev.subscribe(function(){
						this.resume(function(){

							// switch to en_EN
							panel.subscribe('variablesLoaded', cbVarLoaded_EN, null, this);
							panel.form_basic.getChildByName('locale_str').input.set('value', 'en_GB');

							this.wait();

						});
					}, null, this);

					this.wait();




				}
				catch (e) {
					e.delay || console.error(e);
					throw e;
				}

			});

		};

		var cbVarLoaded_EN = function(){
			console.warn('cbVarLoaded_EN');
			this.resume(function(){

				panel.unsubscribe('variablesLoaded', cbVarLoaded_EN);

				try {

					// fill basic form
					panel.form_locale.getChildByName('product_name').input.set('value', 'Heineken EN');

					// save english
					var ev = panel.save();
					ev.subscribe(function(){
						this.resume(function(){

							// switch back to Nl and verify values
							panel.subscribe('variablesLoaded', cbVarLoaded_NL2, null, this);
							panel.form_basic.getChildByName('locale_str').input.set('value', 'nl_NL');

							this.wait();

						});
					}, null, this);

					this.wait();


				}
				catch (e) {
					e.delay || console.error(e);
					throw e;
				}

			});

		};


		var cbVarLoaded_NL2 = function(){
			console.warn('cbVarLoaded_NL2');
			this.resume(function(){

				panel.unsubscribe('variablesLoaded', cbVarLoaded_NL2);

				try {

					YAHOO.util.Assert.areEqual(panel.form_basic.getChildByName('name').input.get('value'), 'Yello');
					YAHOO.util.Assert.areEqual(panel.form_locale.getChildByName('product_name').input.get('value'), 'Heineken');
					YAHOO.util.Assert.areEqual(panel.form_locale.getChildByName('other_product_name').input.get('value'), 'San Miguel');

					console.warn('done');


				}
				catch (e) {
					e.delay || console.error(e);
					throw e;
				}

			});

		};






		// wait till the panel has loaded (locale list n stuff)
		panel.subscribe('variablesLoaded', cbVarLoaded_NL, null, this);

		this.wait(35000);
	}


}));

})();


YAHOO.register("test_compose", null, {version: "1.0.0", build: "1"});
