/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 *
 * Testcase for the IQuest Response
 */

(function(){


	var localeList = [
		{name: 'Dutch', locale: 'nl_NL'},
		{name: 'English', locale: 'en_GB'},
		{name: 'Deutch', locale: 'de_DE'}
	];

	var themeList = [
		//{name: 'Debug', cssClass: 'survey-debug', module: 'iquest_survey_theme_debug'},
		{name: 'BaseSoft', cssClass: 'survey-basesoft', module: 'iquest_survey_theme_basesoft'},
		{name: 'TNT', cssClass: 'survey-tnt', module: 'iquest_survey_theme_tnt'}
	];


	BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

		name: "Response Test",
		assertClean: true,

		timeout: 120 * 1000,

		setUp: function(){
			// this broker does test commands like resetdb
			this.testcase_broker = new BASESOFT.util.JsonBroker('/iqportal.iquest.test/selenium/yui/rpc', {executeIfLoggedOut: true});
		},

		_should: {
			ignore: {
				test_branching: true
			}
		},


		/**
		 * test the survey in preview mode.. this works with the current logged in user
		 */
		testPreview: function(){

			// To begin testing we require a survey to be saved.. then continue to the callback
			this.testcase_broker.call('create_survey', [], function(result){

				this.resume(function(){
					try {
						this._testPreview_cbCreateSurvey(result);
					}
					catch (e) {
						e.delay || console.error(e); // ignore yui test wait exceptions
						throw e;
					}
				});

			}, null, this, this.timeout);

			this.wait(this.timeout);
		},

		// Called when the survey is created.
		// Then login as the same user that created the survey.. this information is in the result object
		_testPreview_cbCreateSurvey: function(result){

			var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/',
					{executeIfLoggedOut: true, timeout: this.timeout});
			loginrpc.call('login', [result.username, result.password], function(res){
				this.resume(function(){
					BASESOFT.env.account.id = res.id;
					try {
						this._testPreview_cbLogin(result.survey_id);
					}
					catch (e) {
						e.delay || console.error(e);
						throw e;
					}
				});
			}, null, this);

			this.wait(this.timeout);
		},

		// called when logged in with the survey_id as argument
		_testPreview_cbLogin: function(survey_id){
			console.debug('logged in, survey_id: ', survey_id);

			var p = new BASESOFT.iquest.widget.survey.SurveyPreviewPanel({
				'survey_id': survey_id
			});
			this._garbage.push(p);

			// assert that the
			var iframe = document.getElementsByClassName('iframe-survey')[0];
			var iframe_doc = iframe.contentDocument;

			YAHOO.util.Event.addListener(iframe, 'load', function() {
				this.resume(function() {

					var els;
					BASESOFT.util.waitForCondition(function() {
						iframe_doc = document.getElementsByClassName('iframe-survey')[0].contentDocument;
						els = iframe_doc.getElementsByClassName('bs-iquest-survey-editable');
						return els && els.length > 0;
					}, function() {

						this.resume(function() {
							YAHOO.util.Assert.isTrue(els[0].innerHTML.match(/Welcome to the test_compose survey/) !== null);
						}, null, this);

					}, null, this);

					this.wait();

				});
			}, null, this);
			this.wait();

		},

		/**
		 * test the survey runner with a hash like a real participant would do
		 */
		testSurveyHashRun: function(){

			// will get a refernce to the runner in the first callback
			var runner = null;

			// scoped function starts running the test
			var start = function(){

				// to begin testing we require a survey to be saved
				// so call a rpc method to do so and we get the hash back
				this.testcase_broker.call('publish_survey', [], function(invitee_hash){

					this.resume(function(){
						try {
							cbPublishSurvey.call(this, invitee_hash);
						}
						catch (e) {
							e.delay || console.error(e);
							throw e;
						}
					});

				}, null, this, this.timeout);
				this.wait(this.timeout);
			};

			// callback called when the survey has been created
			var cbPublishSurvey = function(invitee_hash){
				console.debug('ResponseTest:: _cbCreateSimpleSurvey: invitee_hash:', invitee_hash);

				runner = new BASESOFT.iquest.response.ParticipantSurveyRunner();
				this._garbage.push(runner);

				runner.run({runner_id: invitee_hash});

				runner.onSurveyLoad.subscribe(function(){
					this.resume(function(){
						try {

							// give some time to activate()
							var _this = this;
							BASESOFT.util.setTimeout(function() {
								this.resume(function() {
									cbSurveyLoad.call(_this);
								});
							}, 2000, null, this);
							this.wait();
						}
						catch (e) {
							e.delay || console.error(e);
							throw e;
						}
					});
				}, null, this);

				this.wait(this.timeout);
			};

			// called when the survey has loaded and we can start filling in
			var cbSurveyLoad = function(){

				var survey_container = YAHOO.util.Selector.query('.survey-runner', null, true);


				// find the first checkbox and select it
				var checks = YAHOO.util.Dom.getElementsBy(function(el){
					return (el.type == 'checkbox');
				}, 'input', survey_container);

				for (var i in checks)
					checks[i].checked = false;

				// and the first radio
				YAHOO.util.Dom.getElementBy(function(el){
					return (el.type == 'radio');
				}, 'input', survey_container).checked = false;

				// clicky submit in the runner
				runner.submitResponses();

				// wait till responses are submitted
				runner.subscribe('onResponsesUpdated', function() {
					this.resume(function() {
						// noop
					});
				}, null, this);
				this.wait();
			};

			// call start to begin test
			start.call(this);
		},

		testResponseQuestions: function(){
			try {

				var state = {
					"survey": {
						"objectName": "Survey",
						"availableLocales": ['en_GB'],
						"fallbackLocale": {
							'locale': 'en_GB',
							'name': 'English'
						},
						"theme": {
							"module": "iquest_survey_theme_basesoft",
							"name": "BaseSoft",
							"cssClass": "survey-basesoft"
						},

						"children": {
							"blocks": [{
								"objectName": "SurveyBlock",
								"children": {
									"columns": [{
										"objectName": "SurveyBlockColumn",
										"children": {
											"items": [{
												"objectName": "SurveyItemQA",
												"dbID": 10,
												"poolItemID": null,
												"questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
												"answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
												"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_TEXT_INPUT,

												"locales": {
													'en_GB': {
														'label': 'What are you gonna eat?'
													}
												},

												"children": {
													"questions": [{
														"objectName": "SurveyQuestion",
														"dbID": 11
													}],

													"answers": [{
														"objectName": "SurveyAnswer",
														"dbID": 12,
														"locales": {
															'en_GB': {
																'label': 'Today'
															}
														}
													}, {
														"objectName": "SurveyAnswer",
														"dbID": 13,
														"locales": {
															'en_GB': {
																'label': 'Tomorrow'
															}
														}
													}]
												}
											}, {
												"objectName": "SurveyItemQA",
												"dbID": 20,
												"poolItemID": null,
												"questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
												"answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
												"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_MULTIPLE_CHOICE,

												"locales": {
													'en_GB': {
														'label': 'Clicky what you like?'
													}
												},

												"children": {
													"questions": [{
														"objectName": "SurveyQuestion",
														"dbID": 21
													}],

													"answers": [{
														"objectName": "SurveyAnswer",
														"dbID": 22,
														"locales": {
															'en_GB': {
																'label': 'Apples'
															}
														}
													}, {
														"objectName": "SurveyAnswer",
														"dbID": 23,
														"locales": {
															'en_GB': {
																'label': 'Pears'
															}
														}
													}, {
														"objectName": "SurveyAnswer",
														"dbID": 24,
														"locales": {
															'en_GB': {
																'label': 'Dogs'
															}
														}
													}]
												}
											}, {
												"objectName": "SurveyItemQA",
												"dbID": 30,
												"poolItemID": null,
												"questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
												"answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
												"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,

												"locales": {
													'en_GB': {
														'label': 'Gender please!'
													}
												},

												"children": {
													"questions": [{
														"objectName": "SurveyQuestion",
														"dbID": 31,
														"locales": {
															'en_GB': {
																'label': 'Today'
															}
														}
													}, {
														"objectName": "SurveyQuestion",
														"dbID": 32,
														"locales": {
															'en_GB': {
																'label': 'Tomorrow'
															}
														}
													}],

													"answers": [{
														"objectName": "SurveyAnswer",
														"dbID": 33,
														"locales": {
															'en_GB': {
																'label': 'Male'
															}
														}
													}, {
														"objectName": "SurveyAnswer",
														"dbID": 34,
														"locales": {
															'en_GB': {
																'label': 'Female'
															}
														}
													}, {
														"objectName": "SurveyAnswer",
														"dbID": 35,
														"locales": {
															'en_GB': {
																'label': 'Kelly'
															}
														}
													}]
												}
											}, {
												"objectName": "SurveyItemQA",
												"dbID": 40,
												"poolItemID": null,
												"questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
												"answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
												"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,
												"layout": BASESOFT.iquest.widget.survey.ItemQA.prototype.LAYOUT_SLIDER,

												"locales": {
													'en_GB': {
														'label': 'How do you feel?'
													}
												},

												"children": {
													"questions": [{
														"objectName": "SurveyQuestion",
														"dbID": 41
													}],

													"answers": [{
														"objectName": "SurveyAnswer",
														"dbID": 42,
														"locales": {
															'en_GB': {
																'label': 'Good'
															}
														}
													}, {
														"objectName": "SurveyAnswer",
														"dbID": 43,
														"locales": {
															'en_GB': {
																'label': 'Normal'
															}
														}
													}, {
														"objectName": "SurveyAnswer",
														"dbID": 44,
														"locales": {
															'en_GB': {
																'label': 'Bad'
															}
														}
													}]
												}
											},{
												"objectName": "SurveyItemQA",
												"dbID": 50,
												"poolItemID": null,
												"questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
												"answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
												"type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,
												"layout": BASESOFT.iquest.widget.survey.ItemQA.prototype.LAYOUT_DROPDOWN,

												"locales": {
													'en_GB': {
														'label': 'How do you feel tomorrow?'
													}
												},

												"children": {
													"questions": [{
														"objectName": "SurveyQuestion",
														"dbID": 51
													}],

													"answers": [{
														"objectName": "SurveyAnswer",
														"dbID": 52,
														"locales": {
															'en_GB': {
																'label': 'Good'
															}
														}
													}, {
														"objectName": "SurveyAnswer",
														"dbID": 53,
														"locales": {
															'en_GB': {
																'label': 'Normal'
															}
														}
													}, {
														"objectName": "SurveyAnswer",
														"dbID": 54,
														"locales": {
															'en_GB': {
																'label': 'Bad'
															}
														}
													}]
												}
											}]
										}
									}]
								}
							}]
						}
					}
				};

				var responses = {
					11: {
						12: 'Meat',
						13: 'Fish'
					},
					21: {
						22: false,
						23: true,
						24: true
					},
					31: {
						33: true
					},
					32: {
						35: true
					},
					41: {
						44: true
					},
					51: {
						53: true
					}
				};

				var runner = new BASESOFT.iquest.response.SurveyRunner();
				runner.loadState(state, responses);
				this._garbage.push(runner);

				var _this = this;
				setTimeout(function() {
					_this.resume(function() {


					// retrive responses to verify if loaded responses get saved back
					responses = runner.getResponses()[0];

					YAHOO.util.Assert.areEqual(10, responses[0].item_id);
					YAHOO.util.Assert.areEqual(20, responses[1].item_id);
					YAHOO.util.Assert.areEqual(30, responses[2].item_id);

					YAHOO.util.Assert.areEqual('Meat', responses[0].responses[0].v);
					YAHOO.util.Assert.areEqual('Fish', responses[0].responses[1].v);

					YAHOO.util.Assert.areEqual(null, responses[1].responses[0].v);
					YAHOO.util.Assert.areEqual(true, responses[1].responses[1].v);
					YAHOO.util.Assert.areEqual(true, responses[1].responses[2].v);

					YAHOO.util.Assert.areEqual(true, responses[2].responses[0].v); // male
					YAHOO.util.Assert.areEqual(true, responses[2].responses[5].v); // kelly

					});

				}, 1000);


			}
			catch (e) {
				console.error(e);
				throw e;
			}

			this.wait();

		},

		/**
		 * We test branching response. This includes navigating back and forth (assert correct current page).
		 */
		test_branching: function() {
			try {
				// Mock broker, we already have server side tests for branch responses.
				// This is just to test current page, vs next page etc on client level.
				var broker = function(){};
				broker.prototype = {
					call: function() {
						var responses = arguments[1][1];
						var blocks = arguments[1][2];
						var m = arguments[2];
						var scope = arguments[4];
						var corben = function(responses) {
							try {

								// Loop responses:
								var val, resp, i, j, v;
								for(i in responses) {
									resp = responses[i];
									for(j in resp.responses) {
										v = resp.responses[j];
										val = this.nextBlock(blocks[0], resp.item_id, v.q, v.a, v.v);
										if(val!==false) {
											return {next_blocks: [val]};
										}

									}
								}
							} catch (e) {
								console.error(e);
								throw e;
							}
							return {};
						};
						var rv = corben.call(this, responses);
						return m.call(scope, rv);
					},

					nextBlock: function(b, i, q, a, v) {
						switch(i) {
							// First QA item
							case 3:
								if (b != 2)
									return false;
								// First Answer to second page
								if(a===5 && v===true) {
									return 8;
								// Second Answer to third page
								} else if (a===6 && v===true) {
									return 14;
								}
								return false;
								break;
							case 9:
								if (b != 8)
									return false;
								// If any answer was given, we go to the last page.
								if(v===true) {
									return 20;
								}
								return false;
							break;
							default:
								return false;
						}
						return false;
					}
				};

				var state = this.getSurveyState();
				console.warn('state', state);

				var runner = new BASESOFT.iquest.response.SurveyRunner();
				this._runner = runner;
				runner.broker = new broker();

				runner.loadState(state, {});
				this._garbage.push(runner);


				var el;
				var c = this._runner.canvas;
				var cState = c.getState();
				var item = this._getItemByDbID(cState, 3);
				console.warn('item', item);
				BASESOFT.util.waitForCondition(function() {
					el = YAHOO.util.Dom.get(item.domID+'-c0-r0');
					return el;
				}, function() {

					this.resume(function() {

						YAHOO.util.Assert.areEqual(0, c.get('pageIndex'));

						// We select the first answer on the first page.
						el.checked = true;

						this._runner.processResponses();
						YAHOO.util.Assert.areEqual(1, c.get('pageIndex'));

						// Now we go back
						c.set('pageIndex', c.previousPageMap['page_' + c.get('pageIndex')]);
						YAHOO.util.Assert.areEqual(0, c.get('pageIndex'));

						// We select the second answer this time around
						el = YAHOO.util.Dom.get(item.domID+'-c0-r1');
						el.checked = true;

						// This means we are on the third page.
						this._runner.processResponses();
						YAHOO.util.Assert.areEqual(2, c.get('pageIndex'));

						// Now we go to the last page;
						this._runner.processResponses();
						YAHOO.util.Assert.areEqual(3, c.get('pageIndex'));

						// And then all the way back to the beginning.
						c.set('pageIndex', c.previousPageMap['page_' + c.get('pageIndex')]);
						YAHOO.util.Assert.areEqual(2, c.get('pageIndex'));

						// We should skip page 1.
						c.set('pageIndex', c.previousPageMap['page_' + c.get('pageIndex')]);
						YAHOO.util.Assert.areEqual(0, c.get('pageIndex'));

						// Now that we are back at the beginning, we reselect the first answer.
						el = YAHOO.util.Dom.get(item.domID+'-c0-r0');
						console.warn('el', el);
						el.checked = true;

						// We move on to the second page
						this._runner.processResponses();
						YAHOO.util.Assert.areEqual(1, c.get('pageIndex'));

						BASESOFT.util.setTimeout(function() {
							this.resume(function() {

								// Either answer should get us to the last page. We pick the first.
								item = this._getItemByDbID(cState, 9);
								el = YAHOO.util.Dom.get(item.domID+'-c0-r0');
								el.checked = true;

								this._runner.processResponses();
								YAHOO.util.Assert.areSame(3, c.get('pageIndex'));

								// Now lets press Finish, and then navigate all the way back.
								this._runner.processResponses();
								YAHOO.util.Assert.areSame(3, c.get('pageIndex'));

								c.set('pageIndex', c.previousPageMap['page_' + c.get('pageIndex')]);
								YAHOO.util.Assert.areEqual(1, c.get('pageIndex'));
								c.set('pageIndex', c.previousPageMap['page_' + c.get('pageIndex')]);
								YAHOO.util.Assert.areEqual(0, c.get('pageIndex'));

								console.warn('test done');
							});
						}, 1000, null, this);

						this.wait();
					});

				}, null, this);
				this.wait();


			} catch (e) {
				console.error(e);
				throw e;
			}

		},

		_getItemByDbID: function(state, id) {
			var b, block, c, column, i, item;
			for(b in state.children['blocks']) {
				block = state.children['blocks'][b];
				for(c in block.children['columns']) {
					column = block.children['columns'][c];
					for(i in column.children['items']) {
						item = column.children['items'][i];
						if (item.dbID===id){
							return item;
						}
					}
				}
			}
			return false;
		},

		getSurveyState: function() {
			return {

	            "survey": {
	                "name": "IQPortalTestCase Branching Survey",
	                "objectName": "Survey",
	                "dbID": 1,
	                "domID": "iquest-canvas-dom-id",
	                "has_active_publications": false,
	                "availableLocales": ['nl_NL'],

	                "theme": {
	                    "module": "iquest_survey_theme_basesoft",
	                    "name": "BaseSoft",
	                    "cssClass": "survey-basesoft"
	                },

	                "fallbackLocale": {'locale': 'nl_NL', 'name': 'Dutch' },

	                "children": {
	                    "blocks": [{
	                        "objectName": "SurveyBlock",
	                        "dbID":2,
	                        "domID": "yui-gen001",
	                        "pageEnd": true,
	                        "children": {
	                            "columns": [{
	                                "children": {
	                                    'items': [{
	                                        "objectName":"SurveyItemQA",
	                                        "dbID":3,
	                                        "poolItemID": null,
	                                        "domID":"yui-gen002",
	                                        "questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
	                                        "answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
	                                        "trailingQuestions": false,
	                                        "type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,
	                                        "layout": null,
	                                        "showTitle": true,
	                                        "sum_questions": false,
	                                        "sum_answers": false,
	                                        "layoutAttributes": {},
	                                        'required': false,
	                                        "filters": [],
	                                        "alwaysShowAnswerLabels": false,
	                                        'alwaysShowQuestionLabels': false,
	                                        "branching": [{
	                                            'question': {'domID': "yui-gen003"},
	                                            'answer': {'domID': "yui-gen004"},
	                                            'block': {'domID': "yui-gen007"}
	                                        },{
	                                            'question': {'domID': "yui-gen003"},
	                                            'answer': {'domID': "yui-gen005"},
	                                            'block': {'domID': "yui-gen013"}
	                                        }],

	                                        "locales": {
	                                            'nl_NL': {
	                                                'label': 'Wat denkt u van Fritesaus?'
	                                            }
	                                        },

	                                        "children": {
	                                            "questions":[{
	                                                "objectName":"SurveyQuestion",
	                                                "dbID":4,
	                                                "domID":"yui-gen003",
	                                                "poolElementID":null,
	                                                "locales": {}
	                                            }],

	                                            "answers":[{
	                                                "objectName":"SurveyAnswer",
	                                                "dbID":5,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen004",

	                                                "locales": {
	                                                    'nl_NL': {
	                                                        'label': 'Goed',
	                                                        'trailing_label': null
	                                                    }
	                                                }
	                                            },
	                                            {
	                                                "objectName":"SurveyAnswer",
	                                                "dbID":6,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen005",

	                                                "locales": {
	                                                    'nl_NL': {
	                                                        'label': 'Ik vind Freddie beter.',
	                                                        'trailing_label': null
	                                                    }
	                                                }
	                                            }]
	                                        }
	                                    }]
	                                },
	                                "objectName": "SurveyBlockColumn",
	                                "dbID": 7,
	                                "domID": "yui-gen006",
	                                "width":100
	                            }]
	                        }
	                    },{
	                        "objectName": "SurveyBlock",
	                        "dbID":8,
	                        "domID": "yui-gen007",
	                        "pageEnd": true,
	                        "children": {
	                            "columns": [{
	                                "children": {
	                                    'items': [{
	                                        "objectName":"SurveyItemQA",
	                                        "dbID":9,
	                                        "poolItemID": null,
	                                        "domID":"yui-gen008",
	                                        "questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
	                                        "answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
	                                        "trailingQuestions": false,
	                                        "type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,
	                                        "layout": null,
	                                        "showTitle": true,
	                                        "sum_questions": false,
	                                        "sum_answers": false,
	                                        "layoutAttributes": {},
	                                        'required': false,
	                                        "filters": [],
	                                        "alwaysShowAnswerLabels": false,
	                                        'alwaysShowQuestionLabels': false,
	                                        "branching": [{
	                                                'question': {'domID': null},
	                                                'answer': {'domID': null},
	                                                'block': {'domID': "yui-gen019"}
	                                        }],

	                                        "locales": {
	                                            'nl_NL': {
	                                                'label': 'QA level branch?'
	                                            }
	                                        },

	                                        "children": {
	                                            "questions":[{
	                                                "objectName":"SurveyQuestion",
	                                                "dbID":10,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen009",
	                                                "locales": {}
	                                            }],

	                                            "answers":[{
	                                                "objectName":"SurveyAnswer",
	                                                "dbID":11,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen010",

	                                                "locales": {
	                                                    'nl_NL': {
	                                                        'label': 'Oke dan',
	                                                        'trailing_label': null
	                                                    }
	                                                }
	                                            },
	                                            {
	                                                "objectName":"SurveyAnswer",
	                                                "dbID":12,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen011",

	                                                "locales": {
	                                                    'nl_NL': {
	                                                        'label': 'Bah',
	                                                        'trailing_label': null
	                                                    }
	                                                }
	                                            }]
	                                        }
	                                    }]
	                                },
	                                "objectName": "SurveyBlockColumn",
	                                "dbID": 13,
	                                "domID": "yui-gen012",
	                                "width":100
	                            }]
	                        }
	                    },{
	                        "objectName": "SurveyBlock",
	                        "dbID":14,
	                        "domID": "yui-gen013",
	                        "pageEnd": true,
	                        "children": {
	                            "columns": [{
	                                "children": {
	                                    'items': [{
	                                        "objectName":"SurveyItemQA",
	                                        "dbID":15,
	                                        "poolItemID": null,
	                                        "domID":"yui-gen014",
	                                        "questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
	                                        "answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
	                                        "trailingQuestions": false,
	                                        "type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,
	                                        "layout": null,
	                                        "showTitle": true,
	                                        "sum_questions": false,
	                                        "sum_answers": false,
	                                        "layoutAttributes": {},
	                                        'required': false,
	                                        "filters": [],
	                                        "alwaysShowAnswerLabels": false,
	                                        'alwaysShowQuestionLabels': false,
	                                        "branching": [],

	                                        "locales": {
	                                            'nl_NL': {
	                                                'label': 'Geen enkele branch?'
	                                            }
	                                        },

	                                        "children": {
	                                            "questions":[{
	                                                "objectName":"SurveyQuestion",
	                                                "dbID":16,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen015",
	                                                "locales": {}
	                                            }],

	                                            "answers":[{
	                                                "objectName":"SurveyAnswer",
	                                                "dbID":17,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen016",

	                                                "locales": {
	                                                    'nl_NL': {
	                                                        'label': 'Hehe, eindelijk',
	                                                        'trailing_label': null
	                                                    }
	                                                }
	                                            },
	                                            {
	                                                "objectName":"SurveyAnswer",
	                                                "dbID":18,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen017",

	                                                "locales": {
	                                                    'nl_NL': {
	                                                        'label': 'Ik vind dat branchen wel wat hebben',
	                                                        'trailing_label': null
	                                                    }
	                                                }
	                                            }]
	                                        }
	                                    }]
	                                },
	                                "objectName": "SurveyBlockColumn",
	                                "dbID": 19,
	                                "domID": "yui-gen018",
	                                "width":100
	                            }]
	                        }
	                    },{
	                        "objectName": "SurveyBlock",
	                        "dbID":20,
	                        "domID": "yui-gen019",
	                        "pageEnd": true,
	                        "children": {
	                            "columns": [{
	                                "children": {
	                                    'items': [{
	                                        "objectName":"SurveyItemQA",
	                                        "dbID":21,
	                                        "poolItemID": null,
	                                        "domID":"yui-gen020",
	                                        "questionsAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_TOP,
	                                        "answersAlign": BASESOFT.iquest.widget.survey.ItemQA.prototype.ALIGN_LEFT,
	                                        "trailingQuestions": false,
	                                        "type": BASESOFT.iquest.widget.survey.ItemQA.prototype.TYPE_SINGLE_CHOICE,
	                                        "layout": null,
	                                        "showTitle": true,
	                                        "sum_questions": false,
	                                        "sum_answers": false,
	                                        "layoutAttributes": {},
	                                        'required': false,
	                                        "filters": [],
	                                        "alwaysShowAnswerLabels": false,
	                                        'alwaysShowQuestionLabels': false,
	                                        "branching": [],

	                                        "locales": {
	                                            'nl_NL': {
	                                                'label': 'En de laatste vraag?'
	                                            }
	                                        },

	                                        "children": {
	                                            "questions":[{
	                                                "objectName":"SurveyQuestion",
	                                                "dbID":22,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen021",
	                                                "locales": {}
	                                            }],

	                                            "answers":[{
	                                                "objectName":"SurveyAnswer",
	                                                "dbID":23,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen022",

	                                                "locales": {
	                                                    'nl_NL': {
	                                                        'label': 'Uiteraard',
	                                                        'trailing_label': null
	                                                    }
	                                                }
	                                            },
	                                            {
	                                                "objectName":"SurveyAnswer",
	                                                "dbID":24,
	                                                "poolElementID":null,
	                                                "domID":"yui-gen023",

	                                                "locales": {
	                                                    'nl_NL': {
	                                                        'label': 'Zal wel moeten',
	                                                        'trailing_label': null
	                                                    }
	                                                }
	                                            }]
	                                        }
	                                    }]
	                                },
	                                "objectName": "SurveyBlockColumn",
	                                "dbID": 25,
	                                "domID": "yui-gen024",
	                                "width":100
	                            }]
	                        }
	                    }]
	                }
	            }
	        };
		}
	}));

})();

YAHOO.register("test_response", function() { }, {version: "1.0.0", build: "1"});
