# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This is used for creating test users necessary for the YUI tests, etc.
"""
from basesoft.web.core.util.controllers import PermissionController
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.contrib.account.controllers import AccountController
from basesoft.web.contrib.contact.models import Person
from basesoft.web.contrib.account.models import GroupAccount

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:
        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        __mapping_defined = True

COUNTER = 0

def create_license(sa_session):
    global COUNTER
    COUNTER += 1

    domain = license_models.Domain()
    domain.name = 'user_compose_domain_%s' % COUNTER
    contact = Person()
    license = license_models.License()
    license.is_default = True
    license.name = 'user_compose_license_%s' % COUNTER
    license.domain = domain
    license.contact_person = contact
    sa_session.add(license)
    sa_session.flush()
    return license

def create_active_account(username, password, license=None):
    """
    This creates an account and automatically actives it without email
    links etc. It is attached to a group, license and domain.
    It can be called multiple times to create multiple accounts.

    The optional license can be supplied to attach the account to an
    already existing license. A new group will be created non-the-less.
    return the account and the sa_session
    """
    pc = PermissionController()
    sa_session = pc.sa_session

    if license is None: license = create_license(sa_session)

    ac = AccountController(sa_session)
    group = license_models.Group()
    group.name = 'user_compose_group'
    group.license = license
    group.is_default = True

    locale = sa_session.query(Locale).filter(Locale.id==1).one()
    account = ac.create_account(username, password, Person(), locale)

    # make the link between account and group.
    ga = GroupAccount()
    ga.is_default = 1
    ga.active = True
    ga.account = account
    ga.group = group
    sa_session.add(ga)
    sa_session.flush()
    return account, sa_session

def attach_permissions_to_group(sa_session, group, *permissions):
    """
    This is a protected method that can be used in tests to give a group
    certain permissions. Can be easily combined with create_active_account
    and makes the testing of Controller heaps easier.
    """
    from basesoft.web.contrib.license.models import Permission
    for perm in sa_session.query(Permission).filter(Permission.name.in_(permissions)).all():
        group.license.permissions.append(perm)
        group.permissions.append(perm)
    sa_session.add(group)
    sa_session.flush()

def fill_data(conn):
    """
    Create data needed for tests
    """
    # create user 'user_compose' with pass 'pass_compose', needed for the compose tests
    account, sa_session = create_active_account('user_compose', 'pass_compose')
    group = account.groups[0].group

    attach_permissions_to_group(sa_session, group,
        'create_survey', 'update_survey', 'view_survey', 'view_file', 'create_file','file_manager',
        'create_survey_publication', 'update_survey_publication', 'view_survey_publication',
        'schedule_survey_publication', 'delete_survey_publication',
        'view_contact', 'view_contact_list', 'update_contact_list',
        'view_mail', 'view_iquest_mail', 'schedule_mail', 'schedule_iquest_mail',
        'create_survey_variable_set', 'update_survey_variable_set', 'view_survey_variable_set',
        'delete_survey_variable_set', 'view_itemqa_filter', 'view_conditional_branch',
        'view_iquest_pool_category', 'create_iquest_pool_category',

        # iqreport permissions
        'create_report', 'view_datasource', 'create_datasource', 'view_report', 'update_report', 'mark_report_deleted'
    )

