"""
IQuest specific Selenium helper methods
"""

from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase


class IQuestSeleniumTestCase(SeleniumTestCase):
    """
    SeleniumTest class for testing IQuest specific components
    """

    def setUp(self):
        super(IQuestSeleniumTestCase, self).setUp()

    def fill_response(self, block_index=0, column_index=None, item_index=None, input_type=None, input_number=None, input_value=None):
        """
        Respond to a published survey
        Use block_index, column_index and item_index to find the element.
        Supply input_type such as checkbox, radio, input_text.
        If input_type = text then input_value should be used to supply the text for the input.
        If input_type = checkbox, radio etc. input_number should be used in order to click on the nth input el.
        Input_number should be string.
        """

        item = self._compose_target(block_index=block_index, column_index=column_index, item_index=item_index)
        input_el = None

        if input_type == 'text':
            xpath = "(%s//input[@type='text'])[%s]" % (item, input_number)
            input_el = self._driver.find_element_by_xpath(xpath)
            input_el.clear()
            input_el.send_keys(unicode(input_value))

        elif input_type == 'checkbox' or input_type == 'radio':
            xpath = "(%s//input[@type='%s'])[%s]" % (item, input_type, input_number)
            self._driver.find_element_by_xpath(xpath).click()

        else:
            raise Exception('IQuestSeleniumTestCase: invalid input type: %s' % input_type)

    def click_survey_response_button(self, button_name):
        self._driver.find_element_by_xpath("//button[text() = '%s']" % button_name).click()

