# -*- coding: utf-8 -*-

"""
Created on 5 Jun 2012

@author: martijn
"""
from twisted.trial.unittest import FailTest

from basesoft.web.contrib.contact.controllers import ContactController
from basesoft.web.contrib.mailing.models import Mailing
from basesoft.web.core.util.controllers import StateManager

from django.conf import settings

import datetime
from basesoft.web.contrib.i18n import models as i18n_models
from iqportal.iquest.compose.controllers import ComposeController, PublicationStateController
from iqportal.iquest.core import models as iquest_models, schedule
from iqportal.iquest.core.mailing import MailController
from iqportal.iquest.core.models import SurveyPublication
from iqportal.iquest.test.selenium.iquest_selenium_testcase import IQuestSeleniumTestCase
from selenium.common.exceptions import NoSuchElementException


class TestCase(IQuestSeleniumTestCase):

    def setUp(self):

        super(TestCase, self).setUp()

        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        # locale
        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()

        # controller
        self.controller = ComposeController(self.session, self.locale, account_id=self.account.id)

    def wait_for_response_loading(self):
        """
        helper method for iquest response testing that cant use the iqportal wait_for_loading()
        """
        self.wait_for_condition("!YAHOO.util.Dom.hasClass('body', 'busy')")

    def test_create_simple_survey(self):

        # set permissions
        self._set_permissions()

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the iquest app
        self.click_iconbar('icon-iquest')
        self.wait_for_loading()

        # collapse actions
        self.click_palette_title('Actions')

        # set the first block as a page end...
        # do this now for all blocks because later this action is prone to errors
        self.select_compose_module(block_index=0, column_index=0)
        self.click_palette("Block", palette_index=1)
        self.wait_for_loading()
        self.click_palette("Is a page end", palette_index=1)

        # open menu to add compose items
        self.click_palette("Default items", palette_index=2)

        # add 2 more blocks and set them as page end
        self.drag_item_from_palette('Block')
        self.wait_for_loading()
        self.click_palette("Is a page end", palette_index=1)

        self.drag_item_from_palette('Block')
        self.wait_for_loading()
        self.click_palette("Is a page end", palette_index=1)

        # add second column to third block
        self.drag_item_from_palette('Column', block_index=2)

        # add item html to first block
        self.drag_item_from_palette('Text', block_index=0, column_index=0)
        self.wait_for_loading()
        self.compose_edit_wiki('welcome to the selenium test survey', block_index=0, column_index=0, item_index=0)
        self.wait_for_loading()

        # add questions to second block
        self.drag_item_from_palette('Single choice', block_index=1, column_index=0)
        self.wait_for_loading()
        self.compose_edit_inline(('q1 title', 'a1 a1 title', 'q1 a2 title'), block_index=1, column_index=0, item_index=0)

        self.drag_item_from_palette('Numeric input', block_index=2, column_index=0)
        self.wait_for_loading()
        self.compose_edit_inline(('q2 title', 'q2 a1 title', 'q2 a2 title'), block_index=2, column_index=0, item_index=0)

        # add matrix to last block
        self.drag_item_from_palette('Matrix single choice', block_index=2, column_index=1)
        self.wait_for_loading()
        self.compose_edit_inline((
            'q3 title',
            'q3 a1 title', 'q3 a2 title', "q3 a3 title\n", # extra enter here to 'stop' adding questions
            'q3 q1 title', 'q3 q2 title', 'q3 q3 title'
        ), block_index=2, column_index=1, item_index=0)

        # clicky save
        self.click_palette("Save", palette_index=0)

        # confirm survey name
        self.click_button(panel_name='dialog-feedback', text='OK')
        self.assert_no_growl_errors()

    def test_preview(self):
        """
        creates a survey and run the preview runner
        """
        # set permissions
        self._set_permissions()

        self._create_simple_survey()

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the survey from the manager
        self.click_iconbar('icon-iquest-survey-overview', 'icon-iquest')
        self.wait_for_loading()

        self.click_cq(panel_name='survey-manager', row_index=0, col_name='survey_name')
        self.wait_for_loading()

        # clicky preview run
        self.click_palette("Preview run", palette_index=0)
        self.wait_for_loading()

        self.switch_to_frame('iframe-survey')

        # first page
        self.assert_compose_textitem("Welcome to the simple survey test", block_index=0, column_index=0, item_index=0)
        self.click_survey_response_button('Next')
        self.wait_for_response_loading()

        # select answer for the first question
        self.fill_response(block_index=2, column_index=0, item_index=0, input_type='checkbox', input_number=1)

        # select answers for the second question
        self.fill_response(block_index=2, column_index=1, item_index=0, input_type='radio', input_number=1)
        self.fill_response(block_index=2, column_index=1, item_index=0, input_type='radio', input_number=2)
        self.click_survey_response_button('Next')
        self.wait_for_response_loading()

        # go previous
        self.click_survey_response_button('Previous')
        self.wait_for_response_loading()

        # next and next to go to third page
        self.click_survey_response_button('Next')
        self.wait_for_response_loading()

        # type in a wrong numerical value for q3
        self.fill_response(block_index=3, column_index=0, item_index=0, input_type='text', input_number=1, input_value='wrong')
        self.click_survey_response_button('Finish')
        self.wait_for_response_loading()
        self.assert_compose_textitem("All values need to be floating point numbers",
                                     block_index=3, column_index=0, item_index=0, assert_failure=True)

        # now do it good
        self.fill_response(block_index=3, column_index=0, item_index=0, input_type='text', input_number=1, input_value='100')
        self.fill_response(block_index=3, column_index=0, item_index=0, input_type='text', input_number=2, input_value='200')

        self.assert_compose_textitem("Textitem html english", block_index=3, column_index=0, item_index=1)
        self.assert_compose_textitem("Textitem 2 html english", block_index=3, column_index=0, item_index=2)
        self.assert_compose_textitem("Textitem 3 html english", block_index=3, column_index=0, item_index=3)
        self.assert_compose_textitem("Textitem 4 html english", block_index=3, column_index=0, item_index=4)

        # finish
        self.click_survey_response_button('Finish')

    def test_survey_publication(self):
        """
        Create a simple survey and publish it
        """

        # set permissions
        self._set_permissions()

        # create a survey
        survey_name, survey_id = self._create_simple_survey()

        # create mail
        mail = self._create_mail()

        # create contacts for the mail
        contact_controller = ContactController(self.session, account_id=self.account.id)

        c1 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c1', 'email': u'testing@mail.clusternet'}, None, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c2', 'email': u'testing@mail.clusternet'}, {'name': 'Contact Company c2'}, self.group.id)
        c3 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c3', 'email': u'testing@mail.clusternet'}, None, self.group.id)

        cl1 = contact_controller.save_or_update_contact_list(None, 'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], cl1)

        cl2 = contact_controller.save_or_update_contact_list(None, 'Contact List 2', self.group.id)
        contact_controller.link_contacts([c2, c3], cl2)

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        self.click_iconbar('icon-iquest-publication-wizard', hover='icon-iquest')
        self.wait_for_loading()

        # first page
        self.fill_form(panel_name="survey-publication-wizard", values={'module': ('select', survey_name)})
        self.click_button(panel_name='survey-publication-wizard', text=u'Next »')
        self.wait_for_loading()

        d = datetime.datetime(2015, 3, 20, 00)
        # second page
        self.fill_form(panel_name="survey-publication-wizard", values={
            'begin_date': ('datetime', d),
            'show_page_nrs': ('select', 'Show progress bar'),
            'label': 'My Publication',
        })

        self.click_button(panel_name='survey-publication-wizard', text=u'Next »')
        self.wait_for_loading()

        # Invitees page
        self.fill_form(panel_name="survey-publication-wizard", values={
            'anonymous': ('checkbox', True),
            'contact_lists': ('fancy-select', ['Contact List 1', 'Contact List 2'])
        })
        self.click_button(panel_name='survey-publication-wizard', text=u'Next »')
        self.wait_for_loading()

        d = datetime.datetime(2015, 3, 21, 00)
        # Invitation mailing page
        self.assert_form(panel_name="survey-publication-wizard", form_name='invitation_form',
                         values={'reminder_setup_check': ('checkbox', True)})
        self.fill_form(panel_name="survey-publication-wizard", form_name='invitation_form',
                       values={'mail_id': ('select', mail.name),
                               'send_date': ('datetime', d),
                               'mail_from_text': ('input_text', 'Sent from test'),
                               'contact_lists': ('fancy-select', ['Contact List 1', 'Contact List 2'])})

        self.click_button(panel_name='survey-publication-wizard', text=u'Next »')
        self.wait_for_loading()

        d = datetime.datetime(2015, 3, 22, 00)
        # Reminder page
        self.fill_form(panel_name="survey-publication-wizard", form_name='reminder_form',
                       values={'mail_id': ('select', mail.name),
                               'contact_lists': ('fancy-select', ['Contact List 1', 'Contact List 2']),
                               'send_date': ('datetime', d),
                               'mail_from_text': ('input_text', 'Sent from test')})
        self.click_button(panel_name='survey-publication-wizard', text=u'Next »')
        self.wait_for_loading()

        # finish page
        self.click_button(panel_name='survey-publication-wizard', text='Finish')
        self.assert_no_growl_errors()

        import time
        time.sleep(1)
        # open publication manager and check that the survey is there
        self.click_iconbar('icon-iquest-publication-overview', hover='icon-iquest')
        self.wait_for_loading()

        self.assert_cq(panel_name='survey-publications', row_index=0, data={
            'survey': survey_name,
        })

        # there should be two mails saved 1 initial mail and 1 reminder
        get_mail = self.sa_session.query(Mailing).filter(Mailing.mail_id == mail.id).all()
        self.assertEqual(len(get_mail), 2)

    def test_survey_response(self):
        """
        creates a survey, publication, publishes using python and respond using selenium
        """

        # set permissions
        self._set_permissions()

        survey_name, survey_id = self._create_simple_survey()

        self._publish_survey(survey_id)

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        self.click_iconbar('icon-iquest-publication-overview', hover='icon-iquest')
        self.wait_for_loading()

        self.assert_cq(panel_name='survey-publications', row_index=0, data={
            'status': 'Online',
        })

        self.close_panel('survey-publications')

        # get the last created publication open the response page
        pub = self.sa_session.query(SurveyPublication).order_by(SurveyPublication.created.desc())[0]
        self._driver.get(settings.SELENIUM['browserURL'] + "iqportal.iquest.response/display/" + pub.anonymous_hash)
        self.wait_for_response_loading()

        # first page
        self.wait_for_selector("#bs-loading-dialog", count=0)
        self.assert_compose_textitem('Welcome to the simple survey test', block_index=0, column_index=0, item_index=0)
        self.click_survey_response_button('Next')
        self.wait_for_response_loading()

        # select answer for the first question
        self.fill_response(block_index=2, column_index=0, item_index=0, input_type='checkbox', input_number=1)

        # select answers for the second question
        self.fill_response(block_index=2, column_index=1, item_index=0, input_type='radio', input_number=1)
        self.fill_response(block_index=2, column_index=1, item_index=0, input_type='radio', input_number=2)
        self.click_survey_response_button('Next')
        self.wait_for_response_loading()

        # type in a wrong numerical value for q3
        self.fill_response(block_index=3, column_index=0, item_index=0, input_type='text', input_number=1, input_value='wrong')
        self.click_survey_response_button('Finish')
        self.wait_for_response_loading()
        self.assert_compose_textitem("All values need to be floating point numbers",
                                     block_index=3, column_index=0, item_index=0, assert_failure=True)

        # now do it good
        self.fill_response(block_index=3, column_index=0, item_index=0, input_type='text', input_number=1, input_value='100')
        self.fill_response(block_index=3, column_index=0, item_index=0, input_type='text', input_number=2, input_value='200')

        self.assert_compose_textitem("Textitem html english", block_index=3, column_index=0, item_index=1)
        self.assert_compose_textitem("Textitem 2 html english", block_index=3, column_index=0, item_index=2)
        self.assert_compose_textitem("Textitem 3 html english", block_index=3, column_index=0, item_index=3)
        self.assert_compose_textitem("Textitem 4 html english", block_index=3, column_index=0, item_index=4)

        # finish
        self.click_survey_response_button('Finish')

    def test_publication_details(self):
        """
        test publication details
        """

        # attach basic permissions
        self._attach_permissions_from_role(self.account.groups[0].group, 'IQuest (basic)')

        # create survey
        survey_name, survey_id = self._create_simple_survey()

        # publish survey
        publication_id, label = self._publish_survey(survey_id)

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open publication overview
        self.click_iconbar('icon-iquest-publication-overview', hover='icon-iquest')

        # open publication details
        self.click_cq(panel_name='survey-publications', row_index=0, col_name='label')
        self.wait_for_loading()

        # assert invitees and mailing fieldset not visible
        self.assertRaises(NoSuchElementException, self.get_form_element,
                          panel_name='publication-details', values={'contact_lists': ('fancy-select', [])})
        self.assertRaises(FailTest, self.assert_cq_empty, panel_name='publication-details', is_datatable=True)

        # add more permissions
        self._attach_permissions_from_role(self.account.groups[0].group, 'IQuest (default)')

        # refresh page
        self.refresh_page()

        # open publication overview
        self.click_iconbar('icon-iquest-publication-overview', hover='icon-iquest')
        # open publication details
        self.click_cq(panel_name='survey-publications', row_index=0, col_name='label')
        self.wait_for_loading()

        # assert invitees and mailing fieldset visible
        self.get_form_element(panel_name='publication-details', form_name='invitees_details_form', values={'contact_lists': ('fancy-select', [])})
        self.assert_cq_empty(panel_name='publication-details', is_datatable=True)

        self.click_menu(panel_name='publication-details', menu_index=2, text='Schedule new mailing')

    def _set_permissions(self):
        self._attach_permissions_to_group(self.group,
                'view_dashboard', 'save_dashboard',
                'create_survey', 'view_survey', 'update_survey',
                'create_survey_publication', 'view_survey_publication', 'update_survey_publication', 'schedule_survey_publication',
                'create_itemqa_filter', 'view_itemqa_filter', 'update_itemqa_filter', 'delete_itemqa_filter',
                'view_contact_list', 'view_mail', 'create_mail',
                'scheduler_manager',
                'view_iquest_mail', 'schedule_iquest_mail', 'update_iquest_mail', 'create_iquest_mail',
                'create_contact', 'view_contact', 'create_contact_list', 'view_contact_list', 'link_to_contact_list'
        )

    def _create_mail(self):
        state = {
                "mail": {
                    "mail_from_text": "TestMailer",
                    "name": 'IQuest test mail',
                    "objectName": u"IQuestMail",
                    "dbID": None,
                    "locales": {
                        'nl_NL': {
                            'subject': u"iquest mail",
                            'content': u'Test',
                        },
                        'en_GB': {
                            'subject': u"iquest mail",
                            'content': u'Test',
                        },
                    },
                    "fallbackLocale": {'value': u'en_GB'},
                }
        }

        controller = MailController(self.session, self.locale, account_id=self.account.id)
        return controller.save_state(state, self.group)

    def _create_simple_survey(self):
        """
        Saves a state for a simple survey
        """
        name = self._uniquestring('Survey_')[0:15]
        state = {
            "survey": {
                'alter_published_survey': False,
                "name": name,
                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-yui-gen5",
                "has_active_publications": False,

                "availableLocales": ['en_GB', 'nl_NL', 'de_DE'],

                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },

                "children": {
                    "blocks": [

                        # First block contains a column with welcome text
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID": u"yui-gen42",
                            "pageEnd": False,
                            "children": {

                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen-3001",
                                    "width":100,

                                    "children": {

                                        "items":[{
                                            "children": {},
                                            "objectName":u"SurveyItemHtml",
                                            "isWiki": True,
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen3002",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html_html': u'<p>\nWelkom bij de test survey test<br />\n</p>\n',
                                                    u'html_wiki': u'Welkom bij de survey test',
                                                },
                                                'en_GB': {
                                                    u'html_html': u'<p>\nWelcome to the simple survey test<br />\n</p>\n',
                                                    u'html_wiki': u'Welcome to the simple survey test'
                                                }
                                            },
                                        }]
                                    },
                                }],
                            }
                        },

                        # Second block (with pageEnd) contains two empty columns
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen31",
                            "pageEnd": True,

                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': []
                                    },
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen33",
                                    "width":50
                                    },
                                    {
                                    "children":{
                                        'items': []
                                    },
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen53",
                                    "width":50
                                    }
                                ],
                            }
                        },

                        # Third block contains two columns with each a question
                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "pageEnd": True,
                            "domID":u"yui-gen59",

                            "children": {
                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen61",
                                    "width":50,

                                    "children": {
                                        "items":[{
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen77",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,

                                            "type": iquest_models.SurveyItemQA.TYPE_MULTIPLE_CHOICE,
                                            "showTitle": True,
                                            "sum_questions": False,
                                            "sum_answers": False,
                                            "alwaysShowQuestionLabels": False,
                                            "alwaysShowAnswerLabels": True,

                                            "layout": None,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],
                                            "weight_factors": [],
                                            "branching": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'QA label Dutch'
                                                },
                                                'en_GB': {
                                                    u'label': u'QA label English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen205",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen79",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'First question, first answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'First question, first answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen84",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'First question, second answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'First question, second answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                }]
                                            }
                                        }]
                                    }
                                },
                                {
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen70",
                                    "width":50,

                                    "children": {
                                        "items":[{
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID": u"yui-gen92",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_BOTTOM,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_RIGHT,
                                            "trailingQuestions": True,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "sum_questions": False,
                                            "sum_answers": False,
                                            "alwaysShowQuestionLabels": False,
                                            "alwaysShowAnswerLabels": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],
                                            "weight_factors": [],
                                            "branching": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'Second QA label Dutch'
                                                },
                                                'en_GB': {
                                                    u'label': u'Second QA label English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen206",
                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Dutch question one label',
                                                            u'trailing_label': u'Dutch question one trailing'
                                                        },
                                                        'en_GB': {
                                                            u'label': u'English question one label',
                                                            u'trailing_label': u'English question one trailing'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen606",
                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Dutch question two label',
                                                            u'trailing_label': u'Dutch question two trailing'
                                                        },
                                                        'en_GB': {
                                                            u'label': u'English question two label',
                                                            u'trailing_label': u'English question two trailing'
                                                        }
                                                    },
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen94",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Second question, first answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Second question, first answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen99",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Second question, second answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Second question, second answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                }],
                                            }
                                        }],
                                    }
                                }]
                            }
                        },

                        # The last block contains a column with a numeric input and few textfields
                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen109",
                            "pageEnd": True,

                            "children": {

                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen111",
                                    "width":100,

                                    "children": {

                                        "items":[
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID": u"yui-gen120",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_BOTTOM,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_RIGHT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "layout": None,
                                            "showTitle": True,
                                            "sum_questions": False,
                                            "sum_answers": False,
                                            "alwaysShowQuestionLabels": True,
                                            "alwaysShowAnswerLabels": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [{
                                                        "dbID": 3,
                                                        "questionDbID": None,
                                                        "answerDbID": None,
                                                        "attributes": {
                                                                "domain": "float"
                                                        }
                                            }],
                                            "weight_factors": [],
                                            "branching": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'Numeric QA label Dutch'
                                                },
                                                'en_GB': {
                                                    u'label': u'Numeric QA label English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName": u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID": None,
                                                    "domID": u"yui-gen207",
                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Dutch Numeric Q label',
                                                        },
                                                        'en_GB': {
                                                            u'label': u'English Numeric Q label',
                                                        }
                                                    },
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen95",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Numeric question, first answer Dutch',
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Numeric question, first answer English',
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen97",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Numeric question, second answer Dutch',
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Numeric question, second answer English',
                                                        }
                                                    },
                                                }],
                                            }
                                        }, {
                                            "objectName":u"SurveyItemHtml",
                                            "isWiki": True,
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen200",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html_html': u'<p>\nTextitem html dutch <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem html dutch',
                                                },
                                                'en_GB': {
                                                    u'html_html': u'<p>\nTextitem html english <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem html english'
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "isWiki": True,
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen202",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html_html': u'<p>\nTextitem 2 html dutch <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 2 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html_html': u'<p>\nTextitem 2 html english <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 2 html english'
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "isWiki": True,
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen203",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html_html': u'<p>\nTextitem 3 html dutch <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 3 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html_html': u'<p>\nTextitem 3 html english <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 3 html english',
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "isWiki": True,
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen204",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html_html': u'<p>\nTextitem 4 html dutch <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 4 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html_html': u'<p>\nTextitem 4 html english <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 4 html english',
                                                }
                                            },
                                        },


                                        ],
                                    }
                                }],
                            }
                        }
                    ]
                }
            }
        }

        id_map = self.controller.save_state(state)
        key = [key for key in id_map if 'iquest-canvas' in key]
        survey_id = id_map[key[0]]

        return name, survey_id

    def _publish_survey(self, survey_id):

        contact_lists = []
        status = iquest_models.SurveyPublication.STATUS_SCHEDULED
        begin_date = datetime.datetime.now()
        label = self._uniquestring('Publication_')[0:20]

        publish_state = {
            'survey_id': survey_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": None,
            "status": status,
            "label": label,
            "show_page_nrs": True,
            "anonymous": True,
            "contact_lists": contact_lists,
            "viewer_print": False,
            "minimum_respondents": None,
            "maximum_market_share": None,
            "show_individual_measure": None,
            "show_compose_label": True,
            "show_publication_label": True,
            "mailings": []  # no mailings necessary
        }

        publication_controller = PublicationStateController(self.session, StateManager(self.locale, self.account.id, None))

        # save publication
        publication = publication_controller.save(publish_state)

        # publish the surveys like the scheduler would do
        schedule.PublishSurveysTask().execute()
        schedule.CreatePublicationInviteesTask().execute()

        self.session.expire(publication)
        return publication.id, label