# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from iqportal.iquest.core import models as iquest_models

from basesoft import logging
from basesoft.web.test.selenium.yui.views import testrunner_view
logger = logging.getLogger("iquest.test.yui")

def testrunner(request):

    return testrunner_view(request, [
            "test_iquest_compose",
            "test_iquest_publication",
            "test_iquest_response",
            #"test_iquest_branching",
            #"test_iquest_report",
    ])

def rpc(request):

    class handler:
        """
        Class that (ab)uses unit tests for creating and publishing surveys
        """

        def create_survey(self):
            """
            This creates a non published survey and can be used to test the preview run of survey
            """
            logger.debug('rpc::handler::create_survey()')

            from iqportal.iquest.test.unit.test_compose import ComposeControllerTestCase
            c = ComposeControllerTestCase()
            c.setUp()

            survey_id = c.yui_test_compose_survey()

            return {
                    'survey_id': survey_id,
                    'username': c.account.name,
                    'password': 'test_password'
            }

        def create_branching_survey(self):
            """
            This creates a non published survey with branching.
            """
            logger.debug('rpc::handler::create_branching_survey()')

            from iqportal.iquest.test.unit.test_branching import BranchingTestCase
            c = BranchingTestCase()
            c.setUp()

            survey_id = c.yui_test_branching_survey()

            return {
                    'survey_id': survey_id,
                    'username': c.account.name,
                    'password': 'test_password'
            }

        def create_branching_survey_dropdown(self):
            """
            """
            from iqportal.iquest.test.unit.test_branching import BranchingTestCase
            c = BranchingTestCase()
            c.setUp()

            survey_id = c.yui_test_branching_survey_dropdown()

            return {
                    'survey_id': survey_id,
                    'username': c.account.name,
                    'password': 'test_password'
            }

        def create_branching_survey_slider(self):
            """
            """
            from iqportal.iquest.test.unit.test_branching import BranchingTestCase
            c = BranchingTestCase()
            c.setUp()

            survey_id = c.yui_test_branching_survey_slider()

            return {
                    'survey_id': survey_id,
                    'username': c.account.name,
                    'password': 'test_password'
            }

        def publish_branching_survey(self):
            """
            This publishes a branching survey.
            """
            logger.debug('rpc::handler::publish_branching_survey()')

            from iqportal.iquest.test.unit.test_branching import BranchingTestCase
            c = BranchingTestCase()
            c.setUp()

            publication = c.yui_test_publish_branching_survey()
            return publication.anonymous_hash


        def publish_survey(self):
            """
            Use the response test for creating a survey, publishing it and pretend the user got an email and return
            the hash so he can fill in the survey with it
            """
            logger.debug('rpc::handler::publish_survey()')

            from iqportal.iquest.test.unit.test_response import ResponseTest
            t = ResponseTest()
            t.setUp()

            survey_id = t.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True)
            survey = t.session.query(iquest_models.Survey).get(survey_id)

            invitee_hash = survey.publications[0].invitees[0].invitee_hash

            return invitee_hash



    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(handler())
    return dispatcher.handle_request(request)


