# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

import datetime
from basesoft.web.test.unit import iqportal_unittest

class IQuestTestCase(iqportal_unittest.IQPortalTestCase):
    """
    Helper methods for IQuest test cases.
    """

    def setUp(self, permissions):
        """
        Connect, create account, permissions, etc the usual.
        """
        self.connect_db()
        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        # give current group access to create, schedule surveys and view datasources
        self._attach_permissions_to_group(self.group, *permissions)

    def _update_module_ids(self, state, id_map, stripDomID=False):
        """
        recursive updates state's dbID's for the given id_map
        """

        dbID = id_map[ state['domID'] ]
        state['dbID'] = dbID

        if stripDomID:
            del state['domID']

        if (state.has_key('children')):
            for child_attr in ['blocks', 'columns', 'items', 'questions', 'answers']:
                if state['children'].has_key(child_attr):
                    for child in state['children'][child_attr]:
                        self._update_module_ids(child, id_map, stripDomID)

    def publish_survey(self, status, begin_date=None, end_date=None, contact_lists=None, anonymous=False, state=None, mailings=None, label=None):
        """
        Helper method for publishing a survey
        """
        from iqportal.iquest.core import schedule
        from iqportal.iquest.core import models as iquest_models
        from iqportal.iquest.compose.controllers import ComposeController, PublicationStateController
        from basesoft.web.core.util.controllers import StateManager

        from basesoft.web.contrib.i18n import models as i18n_models
        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()

        if contact_lists is None:
            contact_lists = []

        if mailings is None:
            mailings = []

        if begin_date is None:
            begin_date = datetime.datetime.now()

        if state is None:
            state = {
            "survey": {
                'alter_published_survey': False,
                "name": u"IQPortalTestCase Survey",
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-dom-id",
                "has_active_publications": False,

                "availableLocales": ['nl_NL', 'en_GB'],

                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },

                "fallbackLocale": {'locale': u'nl_NL', 'name': u'Dutch' },

                "children": {
                    "blocks": [

                        # First block contains only one column
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID": u"yui-gen42",
                            "pageEnd": False,
                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': []
                                    },
                                    "objectName": u"SurveyBlockColumn",
                                    "dbID": None,
                                    "domID": u"yui-gen44",
                                    "width":100
                                }],
                            }
                        },

                        # Second block contains two empty columns
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen31",
                            "pageEnd": False,

                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': []
                                    },
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen33",
                                    "width":50
                                    },
                                    {
                                    "children":{
                                        'items': []
                                    },
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen53",
                                    "width":50
                                    }
                                ],
                            }
                        },

                        # Third block contains two columns with each a question
                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen59",
                            "pageEnd": False,

                            "children": {
                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen61",
                                    "width":50,

                                    "children": {
                                        "items":[{
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen77",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_MULTIPLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],
                                            "branching": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'Wat denkt u van {{product_name}}?',
                                                },
                                                'en_GB': {
                                                    u'label': u'What do you think of {{product_name}}?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID": None,
                                                    "domID":u"yui-gen205",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID": None,
                                                    "domID":u"yui-gen79",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Goed',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Good',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"yui-gen84",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Ik vind {{other_product_name}} better.',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            # notice: no 'other_product_name' var in EN
                                                            u'label': u'I think another product is better.',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                }]
                                            }
                                        }]
                                    }
                                },
                                {
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen70",
                                    "width":50,

                                    "children": {
                                        "items":[{
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID": u"yui-gen92",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_BOTTOM,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_RIGHT,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "trailingQuestions": False,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],
                                            "branching": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'Second question Dutch',
                                                },
                                                'en_GB': {
                                                    u'label': u'Second question English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID": None,
                                                    "domID":u"yui-gen206",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID": None,
                                                    "domID":u"yui-gen95",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Second question, first answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Second question, first answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID": None,
                                                    "domID":u"yui-gen100",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Second question, second answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Second question, second answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                }],
                                            }
                                        }, {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID": u"yui-gen93",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_BOTTOM,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_RIGHT,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "trailingQuestions": False,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],
                                            "branching": [],
                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'Third question Dutch',
                                                },
                                                'en_GB': {
                                                    u'label': u'Third question English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID": None,
                                                    "domID":u"yui-gen207",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID": None,
                                                    "domID":u"yui-gen94",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Third question, first answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Third question, first answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID": None,
                                                    "domID":u"yui-gen99",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Third question, second answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Third question, second answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                }],
                                            }
                                        }],
                                    }
                                }]
                            }
                        },

                        # The last block contains a column with a few textfields
                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen109",
                            "pageEnd": False,

                            "children": {

                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen111",
                                    "width":100,

                                    "children": {

                                        "items":[
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen201",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html': u'Textitem html dutch',
                                                },
                                                'en_GB': {
                                                    u'html': u'Textitem html english'
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen202",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html': u'Textitem 2 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html': u'Textitem 2 html english'
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen203",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html': u'Textitem 3 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html': u'Textitem 3 html english'
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen204",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html': u'Textitem 4 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html': u'Textitem 4 html english'
                                                }
                                            },
                                        },


                                        ],
                                    }
                                }],
                            }
                        }
                    ]
                }
            }
        }

        # save the survey
        compose_controller = ComposeController(self.session, self.locale, account_id=self.account.id)
        map = compose_controller.save_state(state)
        survey_id = map['iquest-canvas-dom-id']
        self._update_module_ids(state['survey'], map)
        self._survey_state = state

        # create publication
        publish_state = {
            'survey_id': survey_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": end_date,
            "status": status,
            "label": label if label is not None else u"publication 1",
            "show_page_nrs": True,
            "anonymous": anonymous,
            "contact_lists": contact_lists,
            "viewer_print": False,
            'maximum_market_share': None,
            'minimum_respondents': None,
            "mailings": mailings # no mailings necessary
        }

        # save publication
        publication_controller = PublicationStateController(self.session, StateManager(self.locale, self.account.id, None))
        self._publication = publication_controller.save(publish_state)
        publication_controller.save_branch_cache(self._publication)

        # publish the surveys like the scheduler would do
        schedule.PublishSurveysTask().execute()
        schedule.CreatePublicationInviteesTask().execute()

        return survey_id
