# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This is a unit test for testing the compose controller
"""
import exceptions
import copy

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.i18n import models as i18n_models
from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException
from basesoft.web.contrib.contact.controllers import ContactController

from iqportal.iquest.core import models as iquest_models
from iqportal.iquest.compose.controllers import ComposeController

from basesoft import logging
from twisted.trial.unittest import SkipTest
from basesoft.web.core.util.controllers import AbstractStateController
logger = logging.getLogger('basesoft.web.test.unit')

class ComposeControllerTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):

        logger.debug('')
        logger.debug('now in setUp()')

        #self.reset_db()
        self.connect_db()

        #create an account so we can check permission with it
        self.account = self._create_active_account()

        # create the necessary controllers
        self.group = self.account.groups[0].group

        # locale
        locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()

        self.setUpContactLists()

        # controller
        self.controller = ComposeController(self.session, locale, account_id=self.account.id)

    def setUpContactLists(self):
        self._attach_permissions_to_group(self.group, 'link_to_contact_list', 'create_contact', 'create_contact_list', 'view_survey_publication')

        # get all available contact lists
        contact_controller = ContactController(self.session, account_id=self.account.id)

        c1 = contact_controller.save_or_update_contact(None, None, None, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, None, None, self.group.id)
        c3 = contact_controller.save_or_update_contact(None, None, None, self.group.id)

        self.cl1 = contact_controller.save_or_update_contact_list(None, u'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], self.cl1)

        self.cl2 = contact_controller.save_or_update_contact_list(None, u'Contact List 2', self.group.id)
        contact_controller.link_contacts([c2, c3], self.cl2)

    def get_state(self):
        """
        Returns a simple state like the ComposeManager would send using JSONRPC which can be used for unit testing
        """

        return {
            "survey": {
                'alter_published_survey': False,
                "name": u"My Pretty Survey",
                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-yui-gen5",
                "has_active_publications": False,

                "availableLocales": ['en_GB', 'nl_NL', 'de_DE'],

                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },

                "children": {
                    "blocks": [

                        # First block contains a column with welcome text
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID": u"yui-gen42",
                            "pageEnd": False,
                            "children": {

                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen-3001",
                                    "width":100,

                                    "children": {

                                        "items":[
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "isWiki": True,
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen3002",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html_html': u'<p>\nWelkom bij de test_compose survey <br />\n</p>\n',
                                                    u'html_wiki': u'Welkom bij de test_compose survey',
                                                },
                                                'en_GB': {
                                                    u'html_html': u'<p>\nWelcome to the test_compose survey <br />\n</p>\n',
                                                    u'html_wiki': u'Welcome to the test_compose survey'
                                                }
                                            },
                                        },]
                                    },
                                }],
                            }
                        },

                        # Second block (with pageEnd) contains two empty columns
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen31",
                            "pageEnd": True,

                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': []
                                    },
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen33",
                                    "width":50
                                    },
                                    {
                                    "children":{
                                        'items': []
                                    },
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen53",
                                    "width":50
                                    }
                                ],
                            }
                        },

                        # Third block contains two columns with each a question
                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "pageEnd": True,
                            "domID":u"yui-gen59",

                            "children": {
                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen61",
                                    "width":50,

                                    "children": {
                                        "items":[{
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen77",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,

                                            "type": iquest_models.SurveyItemQA.TYPE_MULTIPLE_CHOICE,
                                            "showTitle": True,
                                            "sum_questions": False,
                                            "sum_answers": False,
                                            "alwaysShowQuestionLabels": False,
                                            "alwaysShowAnswerLabels": True,

                                            "layout": None,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],
                                            "weight_factors": [],
                                            "branching": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'QA label Dutch'
                                                },
                                                'en_GB': {
                                                    u'label': u'QA label English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen205",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen79",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'First question, first answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'First question, first answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen84",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'First question, second answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'First question, second answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                }]
                                            }
                                        }]
                                    }
                                },
                                {
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen70",
                                    "width":50,

                                    "children": {
                                        "items":[{
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID": u"yui-gen92",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_BOTTOM,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_RIGHT,
                                            "trailingQuestions": True,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "sum_questions": False,
                                            "sum_answers": False,
                                            "alwaysShowQuestionLabels": False,
                                            "alwaysShowAnswerLabels": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],
                                            "weight_factors": [],
                                            "branching": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'Second QA label Dutch'
                                                },
                                                'en_GB': {
                                                    u'label': u'Second QA label English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen206",
                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Dutch question one label',
                                                            u'trailing_label': u'Dutch question one trailing'
                                                        },
                                                        'en_GB': {
                                                            u'label': u'English question one label',
                                                            u'trailing_label': u'English question one trailing'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen606",
                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Dutch question two label',
                                                            u'trailing_label': u'Dutch question two trailing'
                                                        },
                                                        'en_GB': {
                                                            u'label': u'English question two label',
                                                            u'trailing_label': u'English question two trailing'
                                                        }
                                                    },
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen94",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Second question, first answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Second question, first answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "poolElementID":None,
                                                    "domID":u"yui-gen99",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Second question, second answer Dutch',
                                                            u'trailing_label': None
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Second question, second answer English',
                                                            u'trailing_label': None
                                                        }
                                                    },
                                                }],
                                            }
                                        }],
                                    }
                                }]
                            }
                        },

                        # The last block contains a column with a few textfields
                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen109",
                            "pageEnd": True,

                            "children": {

                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen111",
                                    "width":100,

                                    "children": {

                                        "items":[
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "isWiki": True,
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen201",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html_html': u'<p>\nTextitem html dutch <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem html dutch',
                                                },
                                                'en_GB': {
                                                    u'html_html': u'<p>\nTextitem html english <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem html english'
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "isWiki": True,
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen202",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html_html': u'<p>\nTextitem 2 html dutch <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 2 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html_html': u'<p>\nTextitem 2 html english <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 2 html english'
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "isWiki": True,
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen203",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html_html': u'<p>\nTextitem 3 html dutch <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 3 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html_html': u'<p>\nTextitem 3 html english <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 3 html english',
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "isWiki": True,
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen204",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html_html': u'<p>\nTextitem 4 html dutch <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 4 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html_html': u'<p>\nTextitem 4 html english <br />\n</p>\n',
                                                    u'html_wiki': u'Textitem 4 html english',
                                                }
                                            },
                                        },


                                        ],
                                    }
                                }],
                            }
                        }
                    ]
                }
            }
        }

    def assertLocales(self, obj, state):
        """
        Helps with asserting locales in an obj
        """
        if not getattr(obj, 'locales'):
            self.fail('Asserted object "%s" should have a locales attribute' % obj)

        # start asserting text values
        for locale_str in state:
            for field_name in state[locale_str]:
                passed = False
                for locale_inst in obj.locales:
                    if locale_inst.locale.locale == locale_str:
                        self.assertEquals( getattr(locale_inst, field_name) , state[locale_str][field_name] )
                        passed = True
                        break
                if not passed:
                    self.fail('Testing field_name %s for locate_str %s not found' % (field_name, locale_str))

    def test_save_new_state(self):
        """
        Tests saving of a new survey
        """

        state = self.get_state()
        # also save the items to the iquest pool
        state['survey']['savePoolItems'] = True

        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_iquest_pool_category',
                                          'create_iquest_pool_category')

        # set surveys number limit to 1 for this license so we can test the usage options check
        self._set_license_usage(self.group.license, 'max_number_of_surveys', 1)

        # first save is ok
        state_response = self.controller.save_state(state)

        # second one should raise an exception cuz the maximum limit of surveys was already reached
        self.assertRaises(PermissionDeniedException, self.controller.save_state, state)

        # now update the state with db id's like the JS compose_manager also does so it can be updated
        self.__update_module_ids(state['survey'], state_response)

        # assert the first survey
        survey = self.session.query(iquest_models.Survey).get(state['survey']['dbID'])

        self.assertTrue(len(survey.available_locales)==3)
        for l in survey.available_locales:
            self.assertTrue(l.locale in ['nl_NL', 'en_GB', 'de_DE'])

        self.assertEquals(survey.name, 'My Pretty Survey')
        self.assertEquals(survey.fallback_locale.locale, 'en_GB')

        # blocks and positions
        self.assertEquals(len(survey.blocks), 4)
        for i in range(3): self.assertEquals(survey.blocks[i].position, i)

        # first block contains one column
        self.assertEquals(len(survey.blocks[0].columns), 1)

        # second block contains two columns
        self.assertEquals(len(survey.blocks[1].columns), 2)

        # third block contains two columns with each a question
        self.assertEquals(len(survey.blocks[2].columns), 2)

        # column positions
        for i in range(1): self.assertEquals(survey.blocks[2].columns[i].position, i)

        # The last block contains a column 4 textfields
        self.assertEquals(len(survey.blocks[3].columns), 1)
        self.assertEquals(len(survey.blocks[3].columns[0].items), 4)

        # verify items position
        for i in range(1): self.assertEquals(survey.blocks[3].columns[0].items[i].position, i)

        # first verify text field (is a bit simpler)
        textfield = survey.blocks[3].columns[0].items[0]
        self.assertTrue(isinstance(textfield, iquest_models.SurveyItemHtml))
        self.assertLocales(textfield, {
            'en_GB': {'html': 'Textitem html english'},
            'nl_NL': {'html': 'Textitem html dutch'}
        })

        # back to block three.. verify QA and Question
        item_qa = survey.blocks[2].columns[0].items[0]
        self.assertNotEquals(item_qa.pool_item, None)
        self.assertTrue(isinstance(item_qa, iquest_models.SurveyItemQA))

        self.assertLocales(item_qa, {
            'en_GB': {'label': 'QA label English'},
            'nl_NL': {'label': 'QA label Dutch'}
        })

        # question
        q1 = item_qa.questions[0]
        self.assertTrue(isinstance(q1, iquest_models.SurveyItemQAElement))
        self.assertEquals(q1.position, 0)
        self.assertEquals(q1.is_question, 1)

        # answer 1
        q1a1 = item_qa.answers[0]
        self.assertTrue(isinstance(q1a1, iquest_models.SurveyItemQAElement))
        self.assertEquals(q1a1.position, 0)
        self.assertEquals(q1a1.is_question, 0)
        self.assertLocales(q1a1, {
            'en_GB': {'label': 'First question, first answer English'},
            'nl_NL': {'label': 'First question, first answer Dutch'}}
        )

        # answer 2
        q1a2 = item_qa.answers[1]
        self.assertTrue(isinstance(q1a2, iquest_models.SurveyItemQAElement))
        self.assertEquals(q1a2.position, 1)
        self.assertEquals(q1a2.is_question, 0)
        self.assertLocales(q1a2, {
            'en_GB': {'label': 'First question, second answer English'},
            'nl_NL': {'label': 'First question, second answer Dutch'}}
        )

        item_qa = survey.blocks[2].columns[1].items[0]
        self.assertNotEquals(item_qa.pool_item, None)
        self.assertTrue(isinstance(item_qa, iquest_models.SurveyItemQA))

        self.assertLocales(item_qa, {
            'en_GB': {'label': 'Second QA label English'},
            'nl_NL': {'label': 'Second QA label Dutch'}
        })

        q1 = item_qa.questions[0]
        self.assertLocales(q1, {
            'nl_NL': {
                u'label': u'Dutch question one label',
                u'trailing_label': u'Dutch question one trailing'
            },
            'en_GB': {
                u'label': u'English question one label',
                u'trailing_label': u'English question one trailing'
            }
        })

        q2 = item_qa.questions[1]
        self.assertLocales(q2,{
            'nl_NL': {
                u'label': u'Dutch question two label',
                u'trailing_label': u'Dutch question two trailing'
            },
            'en_GB': {
                u'label': u'English question two label',
                u'trailing_label': u'English question two trailing'
            }
        })

    def test_save_state_response(self):
        """
        Tests that after saving a state the state_response object is correct
        """

        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey')
        state_response = self.controller.save_state(self.get_state())

        # assert the state response.. we just expect that there are some items
        # in it and that the values are numeric
        self.assertTrue(len(state_response) > 0)
        for key in state_response:
            self.assertEquals(type(state_response[key]), long)

    def __update_module_ids(self, state, id_map, stripDomID=False):
        """
        recursive updates state's dbID's for the given id_map
        """

        dbID = id_map[ state['domID'] ]
        state['dbID'] = dbID

        if stripDomID:
            del state['domID']

        if (state.has_key('children')):
            for child_attr in ['blocks', 'columns', 'items', 'questions', 'answers']:
                if state['children'].has_key(child_attr):
                    for child in state['children'][child_attr]:
                        self.__update_module_ids(child, id_map, stripDomID)


    def test_save_update_state(self):
        """
        Tests the updating of a survey using a state filled with dbID's
        """

        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey')

        # first save a new survey and get the response
        state = self.get_state()
        state_response = self.controller.save_state(state)

        # now update the state with db id's like the JS compose_manager also does so it can be updated
        self.__update_module_ids(state['survey'], state_response)

        # verify that atleast the canvas id is updated.. the rest should just follow
        self.assertNotEquals(state['survey']['dbID'], None)

        # so now we can be sure that we have a state that has db id's in it.. lets modify it a bit

        state['survey']['name'] = u'My Pretty UPDATED survey'

        item_text = state['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]
        item_text['locales']['nl_NL']['html_wiki'] = u'Item text NL UPDATED'

        # add a new locale
        item_text['locales']['fr_BE'] = {}
        item_text['locales']['fr_BE']['html_wiki'] = u'Item text FR'

        # add an item to the first block/column
        block_index = 0; column_index=0;
        state['survey']['children']['blocks'][block_index]['children']['columns'][column_index]['children']['items'].append({
            "objectName":u"SurveyItemHtml",
            "dbID":None,
            "poolItemID": None,
            "domID":u"yui-gen121",
            "locales": {
                'nl_NL': {
                    u'html_wiki': u'New Textitem html dutch',
                },
                'en_GB': {
                    u'html_wiki': u'New Textitem html english'
                }
            }
        })

        # delete the second block
        block_index = 1;
        del state['survey']['children']['blocks'][block_index]

        # delete qa from third block (which index now is 2-1 cuz we deleted before), first column
        block_index = 2-1; column_index = 0; item_index = 0;
        del state['survey']['children']['blocks'][block_index]['children']['columns'][column_index]['children']['items'][item_index]

        # and in the same level add a new QA
        # with the option to save to pool
        state['survey']['children']['blocks'][block_index]['children']['columns'][column_index]['children']['items'].append({
            "objectName":u"SurveyItemQA",
            "dbID":None,
            "poolItemID": None,
            "savePoolItem": True,
            "poolItemCategory": u"test category",
            "poolItemName": u"QA name",
            "domID":u"yui-gen101",
            "type":iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
            "trailingQuestions": False,
            "layout": None,
            "showTitle": True,
            "layoutAttributes": {},
            "required": False,
            "filters": [],
            "weight_factors": [],

            "locales": {
                'nl_NL': {
                    u'label': u'new QA label from update',
                },
            },

            'children': {
                "questions":[{
                    "objectName":u"SurveyQuestion",
                    "dbID":None,
                    "domID":u"yui-gen107",
                    "locales": {},
                }],

                "answers":[{
                    "objectName":u"SurveyAnswer",
                    "dbID":None,
                    "domID":u"yui-gen102",
                    "locales": {
                        'nl_NL': {
                            u'label': 'New question from update answer dutch',
                        }
                    },
                }]
            }
        })

        # delete first question from a qa that has two questions
        item = state['survey']['children']['blocks'][block_index]['children']['columns'][1]['children']['items'][0]
        del item['children']['questions'][0]

        # don't have pool category permissions
        copy_state = copy.deepcopy(state) # else state gets changed while saving (see _save_locale())
        self.assertRaises(PermissionDeniedException, self.controller.save_state, copy_state)
        self._attach_permissions_to_group(self.group, 'view_iquest_pool_category', 'create_iquest_pool_category')

        # call save again
        state_response = self.controller.save_state(state)

        # update state with new id's
        self.__update_module_ids(state['survey'], state_response)

        # now query it and verify
        survey = self.session.query(iquest_models.Survey).get(state['survey']['dbID'])
        self.assertEquals(survey.name, u'My Pretty UPDATED survey')

        self.assertLocales(survey.blocks[0].columns[0].items[0], {
            'en_GB': {'html': u'Welcome to the test_compose survey'},
            'nl_NL': {'html': u'Item text NL UPDATED'},
            'fr_BE': {'html': u'Item text FR'},
        })

        self.assertEquals(len(survey.blocks[0].columns[0].items), 2)
        self.assertLocales(survey.blocks[0].columns[0].items[1], {
            'en_GB': {'html': u'New Textitem html english'},
            'nl_NL': {'html': u'New Textitem html dutch'},
        })

        # assert removed block and the new positions
        self.assertEquals(len(survey.blocks), 3)
        for i in range(3): self.assertEquals(survey.blocks[i].position, i)

        # assert removed QA and added question so length is 1
        # (btw original block index 2 but we removed the second block)
        self.assertEquals(len(survey.blocks[2-1].columns[0].items), 1)

        # and assert the new (and only) QA
        item_qa = survey.blocks[2-1].columns[0].items[0]
        self.assertLocales(item_qa, {
            'nl_NL': {'label': u'new QA label from update'},
        })
        # it has a pool item attached
        pool_item_qa = item_qa.pool_item
        self.assertNotEquals(pool_item_qa, None)
        self.assertEquals(pool_item_qa.name, u"QA name")
        self.assertEquals(pool_item_qa.pool_category.name, u"test category")
        self.assertLocales(pool_item_qa, {
            'nl_NL': {'label': u'new QA label from update'},
        })

        # assert deleted q in qa
        self.assertEquals(1, len(survey.blocks[1].columns[1].items[0].questions))

    def test_survey_qa_with_weight_factors(self):
        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey', 'manage_itemqa_weight_factor')

        # first save a new survey and get the response
        state = self.get_state()

        qa1 = state['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]
        q1 = qa1['children']['questions'][0]

        qa1['weight_factors'] = [{'questionDbID': None, 'questionDomID': q1['domID'], 'answerDbID': None, 'answerDomID': None, 'weight': 2}]

        state_response = self.controller.save_state(state)

        # now update the state with db id's like the JS compose_manager also does so it can be updated
        self.__update_module_ids(state['survey'], state_response)
        survey = self.session.query(iquest_models.Survey).get(state['survey']['dbID'])
        item_qa = survey.itemQAs[0]
        state = self.controller.load_state(survey.id)

        qa1 = state['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]
        self.assertEqual(qa1['weight_factors'], [{'questionDbID': item_qa.questions[0].id, 'answerDbID': None, 'weight': 2}])

    def test_update_active_survey(self):
        """
        Tests the updating of an active survey.
        """

        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey')

        # first save a new survey and get the response
        state = self.get_state()
        state_response = self.controller.save_state(state)

        # now update the state with db id's like the JS compose_manager also does so it can be updated
        self.__update_module_ids(state['survey'], state_response)

        # force an online publication
        p = iquest_models.SurveyPublication()
        p.survey_id = state['survey']['dbID']
        p.status = 2
        from datetime import datetime
        p.begin_date = datetime(2011, 4, 16, 14, 3)
        self.session.add(p)
        self.session.flush()

        # removing elements should raise exception
        del state['survey']['children']['blocks'][3]['children']['columns'][0]['children']['items'][0]
        del state['survey']['children']['blocks'][0]
        self.assertRaises(SecurityException, self.controller.save_state, state)

        state1 = self.get_state()
        self.__update_module_ids(state1['survey'], state_response)

        # adding elements should raise exception
        state1['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'].append({
            "objectName":u"SurveyItemHtml",
            "dbID":None,
            "poolItemID": None,
            "domID":u"yui-gen201",
            "locales": {
                'nl_NL': {
                    u'html_wiki': u'Textitem html dutch',
                },
                'en_GB': {
                    u'html_wiki': u'Textitem html english'
                }
            }
        })
        self.assertRaises(SecurityException, self.controller.save_state, state1)

        #cannot change type, required or filters of an item

        #type
        state2 = self.get_state()
        self.__update_module_ids(state2['survey'], state_response)
        item = state2['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]
        item['type'] = iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE
        self.assertRaises(SecurityException, self.controller.save_state, state2)

        #required
        state3 = self.get_state()
        self.__update_module_ids(state3['survey'], state_response)
        item = state3['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]
        item['required'] = True
        self.assertRaises(SecurityException, self.controller.save_state, state3)

        #filters
        state4 = self.get_state()
        self.__update_module_ids(state4['survey'], state_response)
        item = state4['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]
        item['filters'] = [{'dbID': 1, 'questionDbID': None, 'answerDbID': None, 'attributes': {}}]
        self.assertRaises(SecurityException, self.controller.save_state, state4)

        state5 = self.get_state()
        self.__update_module_ids(state5['survey'], state_response)
        # update translations and some layout
        item = state5['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]
        item['sum_questions'] = True
        item['locales']['nl_NL']['label'] = 'updated dutch text'
        # update positions
        state5['survey']['children']['blocks'][3]['children']['columns'][0]['children']['items'].reverse()
        self.controller.save_state(state5)

    def test_load_survey(self):

        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey')

        state = self.get_state()

        # first save a new state
        state_response = self.controller.save_state(state)

        # update state with new id's
        self.__update_module_ids(state['survey'], state_response)

        # update it like the client would do
        # strip yui dom id's from the state cuz that load doesnt provide these
        stripDomID = True
        self.__update_module_ids(state['survey'], state_response, stripDomID)

        # then load it again
        loaded_state = self.controller.load_state(state['survey']['dbID'])

        # now state should be the same as the loaded_state
        try:
            self.assertEquals(state, loaded_state)
        except exceptions.AssertionError, e:
            from django.utils import simplejson
            logger.info('diff failed! check the difference at http://tlrobinson.net/projects/js/jsondiff/')

            from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder

            logger.debug('state: %s' % simplejson.dumps(state, cls=JsonRpcJSONEncoder))
            logger.debug('loaded_state: %s' % simplejson.dumps(loaded_state, cls=JsonRpcJSONEncoder))
            raise e

    def test_move_module(self):
        """
        Tests the moving of a already saved module from one column to column that is in another lower block
        """

        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey')

        # first save a new survey and get the response
        state = self.get_state()
        state_response = self.controller.save_state(state)
        self.__update_module_ids(state['survey'], state_response)

        # do the moving. first reference the qa
        item_qa = state['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]

        # delete it from the original position
        del state['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]

        # append it to the last column
        state['survey']['children']['blocks'][3]['children']['columns'][0]['children']['items'].append(item_qa)

        # count the items in the src and destination block. this should be the same when loading
        src_col_len = len(state['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'])
        dest_col_len = len(state['survey']['children']['blocks'][3]['children']['columns'][0]['children']['items'])

        # save again
        state_response = self.controller.save_state(state)

        # load fresh state
        state = self.controller.load_state(state['survey']['dbID'])

        # assert the length of the items as we did before.. this should be the same in the loaded state so we know moving was successful
        self.assertEquals(len(state['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items']), src_col_len)
        self.assertEquals(len(state['survey']['children']['blocks'][3]['children']['columns'][0]['children']['items']), dest_col_len)


    def test_permissions_create_survey(self):
        """
        This tests creating of a survey
        """

        state = self.get_state()

        # can't save
        self.assertRaises(PermissionDeniedException, self.controller.save_state, state)

        self._attach_permissions_to_group(self.group, 'create_survey', 'view_survey')

        # try to save again
        state_response = self.controller.save_state(state)

        # update state with new id's
        self.__update_module_ids(state['survey'], state_response)

        survey = self.session.query(iquest_models.Survey).get(state['survey']['dbID'])
        self.assertEquals(survey.name, 'My Pretty Survey')
        self.assertEquals(survey.fallback_locale.locale, 'en_GB')

    def test_permissions_update_survey(self):
        """
        Tests the updating of a survey using a state filled with dbID's
        """

        # first save a new survey and get the response
        state = self.get_state()
        self._attach_permissions_to_group(self.group, 'create_survey', 'view_survey')

        state_response = self.controller.save_state(state)

        # now update the state with db id's like the JS compose_manager also does so it can be updated
        self.__update_module_ids(state['survey'], state_response)

        # verify that atleast the canvas id is updated.. the rest should just follow
        self.assertNotEquals(state['survey']['dbID'], None)

        # so now we can be sure that we have a state that has db id's in it.. lets modify it a bit
        state['survey']['name'] = u'My Pretty UPDATED survey'

        # can't update because we don't have update_survey
        self.assertRaises(PermissionDeniedException, self.controller.save_state, state)

        self._attach_permissions_to_group(self.group, 'update_survey')

        state_response = self.controller.save_state(state)

        # update state with new id's
        self.__update_module_ids(state['survey'], state_response)

        # now query it and verify
        survey = self.session.query(iquest_models.Survey).get(state['survey']['dbID'])
        self.assertEquals(survey.name, u'My Pretty UPDATED survey')

    def test_permissions_load_survey(self):
        """
        Test the loading of a survey
        """

        state = self.get_state()

        # first save a new state
        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey')
        state_response = self.controller.save_state(state)

        # update it like the client would do
        # strip yui dom id's from the state cuz that load doesnt provide these
        self.__update_module_ids(state['survey'], state_response, stripDomID=True)

        # then load it again, but it should fail now, because we don't have the
        # view_survey_permission
        self.assertRaises(PermissionDeniedException, self.controller.load_state, 1)

        # now it should work after adding the proper permission
        self._attach_permissions_to_group(self.group, 'view_survey')
        loaded_state = self.controller.load_state(state['survey']['dbID'])

        # now state should be the same as the loaded_state
        try:
            self.assertEquals(state, loaded_state)
        except exceptions.AssertionError, e:
            from django.utils import simplejson
            logger.info('diff failed! check the difference at http://tlrobinson.net/projects/js/jsondiff/')

            logger.debug('state: %s' % simplejson.dumps(state))
            logger.debug('loaded_state: %s' % simplejson.dumps(loaded_state))
            raise e

    def test_mark_as_archived(self):
        """
        This tests the marking of a survey as deleted.
        """
        state = self.get_state()

        # first set up a user that will create and save a survey
        self._attach_permissions_to_group(self.group, 'create_survey', 'view_survey')
        state_response = self.controller.save_state(state)

        self.__update_module_ids(state['survey'], state_response, True)

        id = state['survey']['dbID']

        self.assertRaises(PermissionDeniedException, self.controller.mark_as_archived, id)
        self._attach_permissions_to_group(self.group, 'update_survey')

        self.controller.mark_as_archived(id)
        self.assertTrue(len(self.controller.get_surveys(id))==0)

    def test_ticket_1002(self):
        """
        The questions are supplied without trialing_label set.
        This problem happens when the user saves a new survey, continues modifying it, and saving
        it again a second time around without having loaded it in between. The trailing_label locale
        is not set but it does exists on the object that gets loaded from the database. This required
        a change in the way locales are saved in the AbstractStateController.
        For more info see http://trac.office.basesoft.com/iqportal/ticket/1002
        """
        state = self.get_state()

        qa = {
            "objectName":u"SurveyItemQA",
            "dbID":None,
            "poolItemID": None,
            "domID": u"yui-gen666",
            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_BOTTOM,
            "answersAlign": iquest_models.SurveyItemQA.ALIGN_RIGHT,
            "trailingQuestions": False,
            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
            "layout": None,
            "showTitle": True,
            "layoutAttributes": {},
            "required": False,
            "filters": [],

            "locales": {
                'nl_NL': {
                    u'label': u'Second QA label Dutch'
                },
                'en_GB': {
                    u'label': u'Second QA label English'
                }
            },

            "children": {
                "questions":[{
                    "objectName":u"SurveyQuestion",
                    "dbID":None,
                    "domID":u"yui-gen2061",
                    "locales": {
                        'nl_NL': {
                            u'label': u'Dutch question one label'
                        },
                        'en_GB': {
                            u'label': u'English question one label'
                        }
                    },
                }],

                "answers":[{
                    "objectName":u"SurveyAnswer",
                    "dbID":None,
                    "domID":u"yui-gen941",

                    "locales": {
                        'nl_NL': {
                            u'label': u'Second question, first answer Dutch',
                        },
                        'en_GB': {
                            u'label': u'Second question, first answer English'
                        }
                    },
                },
                {
                    "objectName":u"SurveyAnswer",
                    "dbID":None,
                    "domID":u"yui-gen991",

                    "locales": {
                        'nl_NL': {
                            u'label': u'Second question, second answer Dutch',
                        },
                        'en_GB': {
                            u'label': u'Second question, second answer English'
                        }
                    },
                }],
            }
        }

        state['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0] = qa
        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey')

        # save it.
        state_response = self.controller.save_state(state)

        self.__update_module_ids(state['survey'], state_response)

        # try to save again, this should no longer raise a KeyError!
        self.controller.save_state(state)

    def test_delete_locale(self):
        """
        ticket http://trac.office.basesoft.com/iqportal/ticket/834
        """

        # save default state
        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey')
        state = self.get_state()
        state_response = self.controller.save_state(state)
        self.__update_module_ids(state['survey'], state_response)

        # delete a locale and save new state
        item = state['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]
        del item['locales']['nl_NL']
        state_response = self.controller.save_state(state)

        # load fresh state and assert that the locale is gone
        state = self.controller.load_state(state['survey']['dbID'])
        item = state['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]
        self.assertFalse(item['locales'].has_key('nl_NL'), 'nl_NL shouldnt be in state anymore')

    def test_clone_survey(self):

        # save survey
        state = self.get_state()

        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey', 'view_iquest_pool_category', 'create_iquest_pool_category')
        state_response = self.controller.save_state(state)
        self.__update_module_ids(state['survey'], state_response)

        survey_id = state['survey']['dbID']

        # clone survey
        survey = self.controller.clone(survey_id, 'cloned survey')

        # compare
        self.assertEqual(survey.name, 'cloned survey')
        self.assertFalse(self.controller.has_all_qas_as_pool_items(survey_id))

        # clone survey, save pool items
        second = self.controller.clone(survey.id, 'secondary clone', True)

        # check if pool items equal
        cloned_state = self.controller.load_state(survey.id)
        secondary_clone_state = self.controller.load_state(second.id)

        def compare_qas(a, b):
            if a['objectName'] == 'SurveyItemQA':
                self.assertEqual(a['poolItemID'], b['poolItemID'])

            if a.has_key('children'):
                for child_attr in ['blocks', 'columns', 'items', 'questions', 'answers']:
                    if a['children'].has_key(child_attr):
                        for i in range(len(a['children'][child_attr])):
                            compare_qas(a['children'][child_attr][i], b['children'][child_attr][i])

        compare_qas(cloned_state['survey'], secondary_clone_state['survey'])

    def test_clone_survey_previous_pool_item(self):
        state = self.get_state()

        def set_first_qa_as_pool_item(state):

            if state['objectName'] == 'SurveyItemQA':
                state['savePoolItem'] = True
                state['poolItemCategory'] = None
                return True

            if (state.has_key('children')):
                for child_attr in ['blocks', 'columns', 'items', 'questions', 'answers']:
                    if state['children'].has_key(child_attr):
                        for child in state['children'][child_attr]:
                            if set_first_qa_as_pool_item(child):
                                return True
            return False

        set_first_qa_as_pool_item(state['survey'])

        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey', 'view_iquest_pool_category', 'create_iquest_pool_category')
        state_response = self.controller.save_state(state)
        self.__update_module_ids(state['survey'], state_response)

        original = self.controller.load_state(state['survey']['dbID'])

        def get_first_qa_pool_item(state):
            if state['objectName'] == 'SurveyItemQA' and state['poolItemID'] != None:
                return state['poolItemID']

            if (state.has_key('children')):
                for child_attr in ['blocks', 'columns', 'items', 'questions', 'answers']:
                    if state['children'].has_key(child_attr):
                        for child in state['children'][child_attr]:
                            result = get_first_qa_pool_item(child)
                            if result is not False:
                                return result
            return False

        pool_item_id = get_first_qa_pool_item(original['survey'])
        clone = self.controller.clone(state['survey']['dbID'], 'Cloned', True)

        original = self.controller.load_state(state['survey']['dbID'])
        cloned_state = self.controller.load_state(clone.id)
        self.assertEqual(pool_item_id, get_first_qa_pool_item(cloned_state['survey']))

        def compare_qas(a, b):
            if a['objectName'] == 'SurveyItemQA':
                self.assertEqual(a['poolItemID'], b['poolItemID'])

            if a.has_key('children'):
                for child_attr in ['blocks', 'columns', 'items', 'questions', 'answers']:
                    if a['children'].has_key(child_attr):
                        for i in range(len(a['children'][child_attr])):
                            compare_qas(a['children'][child_attr][i], b['children'][child_attr][i])

        compare_qas(original['survey'], cloned_state['survey'])

    def yui_test_compose_survey(self):
        """
        Creates a survey for the yui response test and returns information about it
        """

        state = self.get_state()

        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey')
        state_response = self.controller.save_state(state)

        self.__update_module_ids(state['survey'], state_response)

        return state['survey']['dbID']



    def test_wiki_editor(self):
        """
        """

        # save default state
        self._attach_permissions_to_group(self.group, 'create_survey', 'update_survey', 'view_survey')
        state = self.get_state()

        # convert a text item to be wiki
        #item = state['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]
        state_response = self.controller.save_state(state)
        self.__update_module_ids(state['survey'], state_response)

        # delete a locale and save new state
        state['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]['locales']['en_GB'] = {}

        state_response = self.controller.save_state(state)

        #self.__update_module_ids(state['survey'], state_response)
        #self.assertFalse(item['locales'].has_key('nl_NL'), 'nl_NL shouldnt be in state anymore')

        self.sa_session.flush()


# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_compose.py', 'iqportal.iquest.test.unit.test_compose.ComposeControllerTestCase.test_ticket_1002']
    run()
    sys.exit()
