# -*- coding: utf-8 -*-

# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.util.controllers import StateManager

from iqportal.iquest.core import schedule
from iqportal.iquest.core import models as iquest_models
from iqportal.iquest.compose.controllers import ComposeController, PublicationStateController
from iqportal.iquest.response.controllers import ParticipantController
from iqportal.iquest.report.schedule import AggregateResponsesTask

from django.conf import settings
from iqportal.iqube.xmla.client import XMLAClient
from twisted.trial.unittest import SkipTest
from iqportal.iqube.xmla.mdx import MdxBuilder
from iqportal.iquest.core.models import SurveyItem, SurveyBlockColumn,\
    SurveyBlock, Survey, IQuestQAFilter
from iqportal.iquest.core.qa_filters.controllers import SurveyItemQAFilterController
from basesoft.web.contrib.contact.controllers import ContactController
from iqportal.iquest.export.controllers import Exporter,\
    ArrayExporter, CsvExporter, ArrayTwoDimExporter, XlsExporter, SpssExporter
from basesoft.web.contrib.compose.models import ComposeInvitee

import csv

import os
from decimal import Decimal

class ExportTest(iqportal_unittest.IQPortalTestCase):
    """
    """

    def setUp(self):
        """
        """
        self.connect_db()


    def setUpContactLists(self):
        self._attach_permissions_to_group(self.group, 'link_to_contact_list', 'create_contact', 'create_contact_list')

        # get all available contact lists
        contact_controller = ContactController(self.session, account_id=self.account.id)

        person_data1 = {
            'first_name': 'Henk',
            'middle_name': '',
            'last_name': 'de Vries'
        }

        person_data2 = {
            'first_name': 'Piet',
            'middle_name': '',
            'last_name': 'de Boer'
        }

        person_data3 = {
            'first_name': 'Jan',
            'middle_name': '',
            'last_name': 'de Beer'
        }

        company_data1 = {
            'name': 'Philips',
            'website': 'http://philips.com'
        }


        c1 = contact_controller.save_or_update_contact(None, person_data1, company_data1, self.group.id, contact_data={'email': 'person1@mail-test.basesoft.com'})
        c2 = contact_controller.save_or_update_contact(None, person_data2, company_data1, self.group.id, contact_data={'email': 'person2@mail-test.basesoft.com'})
        c3 = contact_controller.save_or_update_contact(None, person_data3, company_data1, self.group.id, contact_data={'email': 'person3@mail-test.basesoft.com'})

        self.cl1 = contact_controller.save_or_update_contact_list(None, u'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], self.cl1)

        self.cl2 = contact_controller.save_or_update_contact_list(None, u'Contact List 2', self.group.id)
        contact_controller.link_contacts([c2, c3], self.cl2)

    def create_survey(self):

        state = {
            "survey": {
                "name": u"My Response Survey",
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-dom-id",

                "availableLocales": ['en_GB'],

                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },

                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },

                "children": {
                    "blocks": [

                        # First block contains only one column
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID": u"yui-gen42",
                            "pageEnd": False,
                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': []
                                    },
                                    "objectName": u"SurveyBlockColumn",
                                    "dbID": None,
                                    "domID": u"yui-gen44",
                                    "width":100
                                }],
                            }
                        },

                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen59",
                            "pageEnd": False,

                            "children": {
                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen61",
                                    "width":50,

                                    "children": {
                                        "items":[

                                        # first question is a simple single choice question
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-1",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'Do you like cows? 互联网服务'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-1-1",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-1-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Yes 互联网服务'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-1-2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'No.. bleh'
                                                        }
                                                    },
                                                }]
                                            }
                                        },

                                        # second question is a matrix question
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-2",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'Do you like ..?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-1",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Dog'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-2",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cat'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-3",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cow'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-4",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Chicken'
                                                        }
                                                    },
                                                },
                                                ],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-2-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Little'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-2-2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Medium'
                                                        }
                                                    },
                                                }, {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-2-3",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'A Lot'
                                                        }
                                                    },
                                                }]
                                            }
                                        },

                                        # third question is an open question
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-3",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-3-1",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-3-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'But how many cows do you have?'
                                                        }
                                                    },
                                                },
                                            ]}
                                        },

                                        # fourth is float matrix
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-4",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'How much?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-1",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Dog'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-2",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cat'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-3",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cow'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-4",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Chicken'
                                                        }
                                                    },
                                                },
                                                ],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-4-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Yesterday'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-4-2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Today'
                                                        }
                                                    },
                                                }, {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-4-3",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Tomorrow'
                                                        }
                                                    },
                                                }]
                                            }
                                        },

                                        # fifth question is a simple multiple choice question
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-5",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_MULTIPLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'What do you like?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-5-1",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-5-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cows'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-5-2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Sheep'
                                                        }
                                                    },
                                                },{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-5-3",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Dogs'
                                                        }
                                                    },
                                                }
                                                ]
                                            }
                                        },


                                        ]
                                    }
                                }

                                ]
                            }
                        },
                    ]
                }
            }
        }

        # save the survey
        map = self.compose_controller.save_state(state)
        survey_id = map['iquest-canvas-dom-id']

        survey = self.session.query(iquest_models.Survey).get(survey_id)
        numeric_itemqa = survey.itemQAs[2]
        float_itemqa = survey.itemQAs[3]

        filter_numeric = self.session.query(IQuestQAFilter).filter(IQuestQAFilter.name==u'Numeric').one()

        ifc = SurveyItemQAFilterController(self.session, None, self.account.id, StateManager(None, None, None))

        #ifc.save_or_update_item_filter(None, filter_numeric, numeric_itemqa, attributes={'domain': 'int'})
        ifc.save_or_update_item_filter(None, filter_numeric, float_itemqa, attributes={'domain': 'float'})

        return survey_id

    def publish_survey(self, survey_id, status=None, begin_date=None, end_date=None, contact_lists=None, anonymous=False):

        if contact_lists is None:
            contact_lists = []

        if status is None:
            status = iquest_models.SurveyPublication.STATUS_SCHEDULED

        if begin_date is None:
            begin_date = datetime.now()

        publish_state = {
            'survey_id': survey_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": end_date,
            "status": status,
            "label": u"publication 1",
            "show_page_nrs": True,
            "anonymous": anonymous,
            "contact_lists": contact_lists,
            "viewer_print": False,
            "mailings": [] # no mailings necessary
        }

        # save publication
        publication = self.publication_controller.save(publish_state)

        # publish the surveys like the scheduler would do
        schedule.PublishSurveysTask().execute()
        schedule.CreatePublicationInviteesTask().execute()

        self.session.expire(publication)
        return publication.id


    def save_response(self, survey_id, publication_id, answer_q1=None, answer_q2=None, answer_q3=None, answer_q4=None, answer_q5=None, invitee_hash=None):

        # get some vars
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        publication = self.session.query(iquest_models.SurveyPublication).get(publication_id)
        assert publication.survey.id == survey_id

        # for now assume anonymous survey so get that hash
        if invitee_hash is None:
            invitee_hash = publication.invitees[0].invitee_hash

        # init the controller
        controller = ParticipantController(self.session, self.locale, invitee_hash)

        # instanciate survey by invitee
        state = controller.load_survey_state()
        self.assertTrue(isinstance(state['survey']['dbID'], (long, int) ))

        # q1 (do you like cows? yes/no)
        if answer_q1 is not None:
            responses = []
            q1_a_id = {'yes': survey.itemQAs[0].answers[0].id, 'no': survey.itemQAs[0].answers[1].id}[answer_q1]
            responses.append({'q': survey.itemQAs[0].questions[0].id, 'a': q1_a_id, 'v': 1},)
            controller.update_responses([{'item_id': survey.itemQAs[0].id, 'responses': responses}])

        if answer_q2 is not None:

            responses = []
            for i in range(len(answer_q2)):
                for j in range(len(answer_q2[i])):

                    q_id = survey.itemQAs[1].questions[i].id
                    a_id = survey.itemQAs[1].answers[j].id

                    if answer_q2[i][j]:
                        responses.append({'q': q_id, 'a': a_id, 'v': 1})

            controller.update_responses([{'item_id': survey.itemQAs[1].id, 'responses': responses}])

        if answer_q3 is not None:
            responses = []
            q_id = survey.itemQAs[2].questions[0].id
            a_id = survey.itemQAs[2].answers[0].id
            responses.append({'q': q_id, 'a': a_id, 'v': answer_q3}, )
            controller.update_responses([{'item_id': survey.itemQAs[2].id, 'responses': responses}])

        if answer_q4 is not None:

            responses = []
            for i in range(len(answer_q4)):
                for j in range(len(answer_q4[i])):

                    q_id = survey.itemQAs[3].questions[i].id
                    a_id = survey.itemQAs[3].answers[j].id

                    responses.append({'q': q_id, 'a': a_id, 'v': answer_q4[i][j]})

            controller.update_responses([{'item_id': survey.itemQAs[3].id, 'responses': responses}])

        if answer_q5 is not None:

            responses = []
            for i in range(len(answer_q5)):
                q_id = survey.itemQAs[4].questions[0].id
                a_id = survey.itemQAs[4].answers[i].id

                responses.append({'q': q_id, 'a': a_id, 'v': answer_q5[i]})

            controller.update_responses([{'item_id': survey.itemQAs[4].id, 'responses': responses}])


    def test_exporters(self, assert_=True):

        #create an account so we can check permission with i
        self.account = self._create_active_account()
        account_id = self.account.id

        # create the necessary controllers
        self.group = self.account.groups[0].group

        from basesoft.web.contrib.i18n import models as i18n_models
        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()

        self.compose_controller = ComposeController(self.session, self.locale, account_id=self.account.id)
        self.publication_controller = PublicationStateController(self.session, StateManager(self.locale, self.account.id, None))

        # create some contact lists
        self.setUpContactLists()

        self._attach_permissions_to_group(self.group, 'create_survey', 'link_to_contact_list', 'create_contact', 'create_contact_list',
                                          'view_survey', 'create_survey_publication', 'view_contact_list',
                                          'schedule_survey_publication', 'view_dashboard', 'view_survey_publication', 'view_datasource', 'view_survey_publication_response',
                                          'iquest_export_excel', 'iquest_export_spss',
                                          )

        survey_id = self.create_survey()
        publication1_id = self.publish_survey(survey_id, anonymous=False, contact_lists=[self.cl1.id, self.cl2.id])
        publication2_id = self.publish_survey(survey_id, anonymous=False)

        invitees = self.session.query(ComposeInvitee) \
            .join(iquest_models.SurveyPublication) \
            .filter(iquest_models.SurveyPublication.id==publication1_id).all()

        i = -1
        for invitee in invitees:
            i = i + 1

            hash = invitee.invitee_hash

            # q1: 66% yes, 33 nope
            if i > 2:
                self.save_response(survey_id, publication1_id, answer_q1='yes', invitee_hash=hash)
            else:
                self.save_response(survey_id, publication1_id, answer_q1='no', invitee_hash=hash)

            # q2 (do you like, ..)
            self.save_response(survey_id, publication1_id, answer_q2 = [
                # little, medium, # a lot
                [i in (1,), 0, i  >= 2],     # dog
                [i < 5, 0, i > 5],           # cat
                [i in (1,2,3,4), 0, i  > 4], # cow
                [i < 3, 0, i >= 3],          # chicken
            ], invitee_hash=hash)

            # q3: number of cows
            q3_answ = i*3
            if i == 0:
                q3_answ = u'互联网服务'
            self.save_response(survey_id, publication1_id, answer_q3=q3_answ, invitee_hash=hash)

            # q4: how much
            self.save_response(survey_id, publication1_id, answer_q4 = [
                [i*10.5, 0, i*5.3],
                [i*2.6, 0, i*1.8],
                [i*78.2, 0, 0],
                [0, 0, 0],
            ], invitee_hash=hash)

            # multiple choice
            self.save_response(survey_id, publication1_id, answer_q5 = [
                # cow, sheep, #dog
                1, 0, i  >= 1
            ], invitee_hash=hash)


        #
        # ArrayExporter
        #

        e = ArrayExporter(sa_session=self.session, account_id=self.account.id, publication_id=publication1_id)
        result = e.export()
        self.assertEquals(len(result), 3) # assert 3 rows

        #
        # ArrayTwoDimExporter
        #

        e = ArrayTwoDimExporter(sa_session=self.session, account_id=self.account.id, publication_id=publication1_id)
        result = e.export()

        # rewrite the participant id's for easier asserting
        for i in range(3):
            result[i+1][0] = None

        expected_result = [
            [u'Participant ID', u'Publication Label', u'First Name', u'Middle Name', u'Last Name', u'Email', u'Position', u'Department', u'Phone', u'Mobile Phone', u'Fax', u'Birth Date', 'Contact Email', u'Company name', u'VAT No.', u'Website', u'Do you like cows? 互联网服务 / Yes 互联网服务', u'Do you like cows? 互联网服务 / No.. bleh', u'Do you like ..? / Dog / Little', u'Do you like ..? / Dog / Medium', u'Do you like ..? / Dog / A Lot', u'Do you like ..? / Cat / Little', u'Do you like ..? / Cat / Medium', u'Do you like ..? / Cat / A Lot', u'Do you like ..? / Cow / Little', u'Do you like ..? / Cow / Medium', u'Do you like ..? / Cow / A Lot', u'Do you like ..? / Chicken / Little', u'Do you like ..? / Chicken / Medium', u'Do you like ..? / Chicken / A Lot', u'None / But how many cows do you have?', u'How much? / Dog / Yesterday', u'How much? / Dog / Today', u'How much? / Dog / Tomorrow', u'How much? / Cat / Yesterday', u'How much? / Cat / Today', u'How much? / Cat / Tomorrow', u'How much? / Cow / Yesterday', u'How much? / Cow / Today', u'How much? / Cow / Tomorrow', u'How much? / Chicken / Yesterday', u'How much? / Chicken / Today', u'How much? / Chicken / Tomorrow', u'What do you like? / Cows', u'What do you like? / Sheep', u'What do you like? / Dogs'],
            [None, 'publication 1', u'Henk', u'', u'de Vries', None, None, None, None, None, None, None, 'person1@mail-test.basesoft.com', u'Philips', None, u'http://philips.com', '', 1L, '', '', '', 1L, '', '', '', '', '', 1L, '', '', u'互联网服务', Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), 1L, 0L, 0L],
            [None, 'publication 1', u'Piet', u'', u'de Boer', None, None, None, None, None, None, None,  'person2@mail-test.basesoft.com', u'Philips', None, u'http://philips.com', '', 1L, 1L, '', '', 1L, '', '', 1L, '', '', 1L, '', '', 3L, Decimal('10.50000'), Decimal('0.00000'), Decimal('5.30000'), Decimal('2.60000'), Decimal('0.00000'), Decimal('1.80000'), Decimal('78.20000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), 1L, 0L, 1L],
            [None, 'publication 1', u'Jan', u'', u'de Beer', None, None, None, None, None, None, None,  'person3@mail-test.basesoft.com', u'Philips', None, u'http://philips.com', '', 1L, '', '', 1L, 1L, '', '', 1L, '', '', 1L, '', '', 6L, Decimal('21.00000'), Decimal('0.00000'), Decimal('10.60000'), Decimal('5.20000'), Decimal('0.00000'), Decimal('3.60000'), Decimal('156.40000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), 1L, 0L, 1L]
        ]
        self.assertDeepEqual(result, expected_result)

        #
        # ArrayTwoDimExporter (HEADERS_MODE_3ROWS)
        #q3_answ

        e = ArrayTwoDimExporter(sa_session=self.session, account_id=self.account.id, publication_id=publication1_id)
        e.headers_mode = ArrayTwoDimExporter.HEADERS_MODE_3ROWS
        result = e.export()

        # rewrite the participant id's for easier asserting
        for i in range(3):
            result[i+3][0] = None

        expected_result = [
            [u'Participant ID', u'Publication Label', u'First Name', u'Middle Name', u'Last Name', u'Email', u'Position', u'Department', u'Phone', u'Mobile Phone', u'Fax', u'Birth Date', 'Contact Email',  u'Company name', u'VAT No.', u'Website', u'Do you like cows? 互联网服务', '', u'Do you like ..?', '', '', '', '', '', '', '', '', '', '', '', '-', u'How much?', '', '', '', '', '', '', '', '', '', '', '', u'What do you like?', '', ''],
            ['', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '-', '', u'Dog', '', '', u'Cat', '', '', u'Cow', '', '', u'Chicken', '', '', '-', u'Dog', '', '', u'Cat', '', '', u'Cow', '', '', u'Chicken', '', '', '-', '', ''],
            ['', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', u'Yes 互联网服务', u'No.. bleh', u'Little', u'Medium', u'A Lot', u'Little', u'Medium', u'A Lot', u'Little', u'Medium', u'A Lot', u'Little', u'Medium', u'A Lot', u'But how many cows do you have?', u'Yesterday', u'Today', u'Tomorrow', u'Yesterday', u'Today', u'Tomorrow', u'Yesterday', u'Today', u'Tomorrow', u'Yesterday', u'Today', u'Tomorrow', u'Cows', u'Sheep', u'Dogs'],
            [None, 'publication 1', u'Henk', u'', u'de Vries', None, None, None, None, None, None, None, 'person1@mail-test.basesoft.com', u'Philips', None, u'http://philips.com', '', 1L, '', '', '', 1L, '', '', '', '', '', 1L, '', '', u'互联网服务', Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), 1L, 0L, 0L],
            [None, 'publication 1', u'Piet', u'', u'de Boer', None, None, None, None, None, None, None, 'person2@mail-test.basesoft.com', u'Philips', None, u'http://philips.com', '', 1L, 1L, '', '', 1L, '', '', 1L, '', '', 1L, '', '', 3L, Decimal('10.50000'), Decimal('0.00000'), Decimal('5.30000'), Decimal('2.60000'), Decimal('0.00000'), Decimal('1.80000'), Decimal('78.20000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), 1L, 0L, 1L],
            [None, 'publication 1', u'Jan', u'', u'de Beer', None, None, None, None, None, None, None, 'person3@mail-test.basesoft.com', u'Philips', None, u'http://philips.com', '', 1L, '', '', 1L, 1L, '', '', 1L, '', '', 1L, '', '', 6L, Decimal('21.00000'), Decimal('0.00000'), Decimal('10.60000'), Decimal('5.20000'), Decimal('0.00000'), Decimal('3.60000'), Decimal('156.40000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), Decimal('0.00000'), 1L, 0L, 1L]
        ]
        self.assertDeepEqual(result, expected_result)

        # CsvExporter
        e = CsvExporter(sa_session=self.session, account_id=self.account.id, publication_id=publication1_id)
        file_name = e.export()
        #print 'CsvExporter: %s' % file_name
        self.assertTrue(os.path.exists(file_name))

        # CsvExporter (HEADERS_MODE_3ROWS)
        e = CsvExporter(sa_session=self.session, account_id=self.account.id, publication_id=publication1_id)
        e.headers_mode = ArrayTwoDimExporter.HEADERS_MODE_3ROWS
        file_name = e.export()
        #print 'CsvExporter (HEADERS_MODE_3ROWS): %s' % file_name
        self.assertTrue(os.path.exists(file_name))

        # XlsExporter
        e = XlsExporter(sa_session=self.session, account_id=account_id, publication_id=publication1_id)
        e.headers_mode = ArrayTwoDimExporter.HEADERS_MODE_3ROWS
        file_name = e.export()
        #print 'XlsExporter: %s' % file_name
        self.assertTrue(os.path.exists(file_name))

        #
        # SpssExporter
        #
        e = SpssExporter(sa_session=self.session, account_id=account_id, publication_id=publication1_id)
        file_name = e.export()
        #print 'SpssExporter: %s' % file_name
        self.assertTrue(os.path.exists(file_name))

        expected_data = [
            ['Participant ID', 'Publication Label', 'First Name', 'Middle Name', 'Last Name', 'Email', 'Position', 'Department', 'Phone', 'Mobile Phone', 'Fax', 'Birth Date', 'Contact Email', 'Company name', 'VAT No.', 'Website', 'Q1', 'Q2', 'Q3', 'Q4', 'Q5', 'Q6', 'Q7', 'Q8', 'Q9', 'Q10', 'Q11'],
            [None, 'publication 1', 'Henk', '', 'de Vries', '', '', '', '', '', '', '', 'person1@mail-test.basesoft.com', 'Philips', '', 'http://philips.com', '2', '', '1', '', '1', u'互联网服务', '0.00000,0.00000,0.00000', '0.00000,0.00000,0.00000', '0.00000,0.00000,0.00000', '0.00000,0.00000,0.00000', '1'],
            [None, 'publication 1', 'Piet', '', 'de Boer', '', '', '', '', '', '', '', 'person2@mail-test.basesoft.com', 'Philips', '', 'http://philips.com', '2', '1', '1', '1', '1', '3', '10.50000,0.00000,5.30000', '2.60000,0.00000,1.80000', '78.20000,0.00000,0.00000', '0.00000,0.00000,0.00000', '1,3'],
            [None, 'publication 1', 'Jan', '', 'de Beer', '', '', '', '', '', '', '', 'person3@mail-test.basesoft.com', 'Philips', '', 'http://philips.com', '2', '3', '1', '1', '1', '6', '21.00000,0.00000,10.60000', '5.20000,0.00000,3.60000', '156.40000,0.00000,0.00000', '0.00000,0.00000,0.00000', '1,3'],
        ]

        #print 'e._csv_file_name: %s' % e._csv_file_name

        with open(e._csv_file_name, 'rb') as csvfile:
            spamreader = UnicodeReader(csvfile, delimiter="\t", quotechar='"')
            row_i = -1
            for row in spamreader:
                row_i = row_i + 1
                if row_i > 0:
                    row[0] = None
                self.assertDeepEqual(row, expected_data[row_i])

        self.assertEqual(open(e._meta_file_name, 'r').read(),
            """VARIABLE LABELS
            Q1 'Do you like cows? 互联网服务'
            Q2 'Do you like ..? / Dog'
            Q3 'Do you like ..? / Cat'
            Q4 'Do you like ..? / Cow'
            Q5 'Do you like ..? / Chicken'
            Q6 ''
            Q7 'How much? / Dog'
            Q8 'How much? / Cat'
            Q9 'How much? / Cow'
            Q10 'How much? / Chicken'
            Q11 'What do you like?'
            .
            VALUE LABELS Q1 1 'Yes 互联网服务' 2 'No.. bleh'.
            VALUE LABELS Q2 1 'Little' 2 'Medium' 3 'A Lot'.
            VALUE LABELS Q3 1 'Little' 2 'Medium' 3 'A Lot'.
            VALUE LABELS Q4 1 'Little' 2 'Medium' 3 'A Lot'.
            VALUE LABELS Q5 1 'Little' 2 'Medium' 3 'A Lot'.
            VALUE LABELS Q6 1 'But how many cows do you have?'.
            VALUE LABELS Q7 1 'Yesterday' 2 'Today' 3 'Tomorrow'.
            VALUE LABELS Q8 1 'Yesterday' 2 'Today' 3 'Tomorrow'.
            VALUE LABELS Q9 1 'Yesterday' 2 'Today' 3 'Tomorrow'.
            VALUE LABELS Q10 1 'Yesterday' 2 'Today' 3 'Tomorrow'.
            VALUE LABELS Q11 1 'Cows' 2 'Sheep' 3 'Dogs'.
            """.replace('    ', ''))



# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['trial.py', 'iqportal.iquest.test.unit.test_export.ExportTest.test_export_csv']
    run()
    sys.exit()


import codecs

class UTF8Recoder:
    """
    Iterator that reads an encoded stream and reencodes the input to UTF-8
    """
    def __init__(self, f, encoding):
        self.reader = codecs.getreader(encoding)(f)

    def __iter__(self):
        return self

    def next(self):
        return self.reader.next().encode("utf-8")


class UnicodeReader:
    """
    A CSV reader which will iterate over lines in the CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, encoding="utf-8", **kwds):
        f = UTF8Recoder(f, encoding)
        self.reader = csv.reader(f, **kwds)

    def next(self):
        row = self.reader.next()
        return [unicode(s, "utf-8") for s in row]

    def __iter__(self):
        return self


