# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import func, and_, or_

from basesoft.web.core.util.controllers import PermissionController

from iqportal.iquest.test.unit.iquest_unittest import IQuestTestCase
from iqportal.iquest.core.models import Survey, SurveyGroup, SurveyPublication
from basesoft.web.contrib.compose.models import ComposePublicationGroup

class IQuestTestCase(IQuestTestCase):
    def setUp(self):
        self.connect_db()
        self.account = self._create_active_account()
        self.group = self.account.groups[0].group
        self.controller = PermissionController(self.session, account_id=self.account.id)

    def test_get_query_filter_on_privileges_outerjoin(self):
        """
        This test is in iquest, cuz there is no basesoft framework example for this.
        """
        s = Survey()
        s.fallback_locale_id = 1
        s.theme_id = 1
        s.name = 'Test survey for privs'
        s.group = self.group
        s.group_privileges = Survey.all_privileges
        s.license_privileges = 0
        s.domain_privileges = 0
        s.world_privileges = 0
        s.deleted = False

        self.session.add(s)
        self.session.flush()

        filter, fo = self.controller.get_query_filter_on_privileges(Survey, SurveyGroup, Survey.PRIV_VIEW)
        fo = fo.outerjoin(SurveyPublication, Survey.id==SurveyPublication.survey_id).outerjoin(ComposePublicationGroup, SurveyPublication.id==ComposePublicationGroup.compose_publication_id)
        f, fo = self.controller.get_query_filter_on_privileges(SurveyPublication, ComposePublicationGroup, SurveyPublication.PRIV_VIEW, fo=fo)
        filter = and_(filter, or_(f, (SurveyPublication.id==None)))

        q = self.session.query(Survey.name, Survey.id, func.count(SurveyPublication.id).label('publications'))
        q = q.select_from(fo).filter(filter).group_by(Survey.id)

        self.assertTrue(len(q.all())==1)

        sp = SurveyPublication()
        sp.survey = s
        sp.status = SurveyPublication.STATUS_NEW
        sp.label = 'Publication for priv'
        sp.anonymous = True
        sp.show_page_nrs = False
        sp.deleted = False
        sp.group = self.group
        sp.group_privileges = SurveyPublication.PRIV_UPDATE # no view privs.
        sp.license_privileges = 0
        sp.domain_privileges = 0
        sp.world_privileges = 0

        self.session.add(sp)
        self.session.flush()

        q = self.session.query(Survey.name, Survey.id, func.count(SurveyPublication.id).label('publications'))
        q = q.select_from(fo).filter(filter).group_by(Survey.id)

        self.assertTrue(len(q.all())==0)
