# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime
from sqlalchemy import func

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.util.controllers import StateManager

from iqportal.iquest.core import schedule
from iqportal.iquest.core import models as iquest_models
from iqportal.iquest.compose.controllers import ComposeController, PublicationStateController
from iqportal.iquest.response.controllers import ParticipantController
from iqportal.iquest.report.schedule import AggregateResponsesTask

from django.conf import settings
from iqportal.iqube.xmla.client import XMLAClient
from twisted.trial.unittest import SkipTest
from iqportal.iqube.xmla.mdx import MdxBuilder
from iqportal.iqube.core.controllers import ProjectController
from iqportal.iquest.core.models import SurveyItem, SurveyBlockColumn,\
    SurveyBlock, Survey, IQuestQAFilter
from iqportal.iquest.core.qa_filters.controllers import SurveyItemQAFilterController

class ReportTest(iqportal_unittest.IQPortalTestCase):
    """
    """

    def setUp(self):
        """
        """
        self.connect_db()

        #create an account so we can check permission with i
        self.account = self._create_active_account()

        # create the necessary controllers
        self.group = self.account.groups[0].group

        from basesoft.web.contrib.i18n import models as i18n_models
        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()

        self.compose_controller = ComposeController(self.session, self.locale, account_id=self.account.id)
        self.publication_controller = PublicationStateController(self.session, StateManager(self.locale, self.account.id, None))

        # create some contact lists
        #self.setUpContactLists()

        self._attach_permissions_to_group(self.group, 'create_survey', 'link_to_contact_list', 'create_contact', 'create_contact_list',
                                          'view_survey', 'create_survey_publication', 'view_contact_list',
                                          'schedule_survey_publication', 'view_dashboard', 'view_survey_publication', 'view_datasource', 'view_survey_publication_response', 'view_report')



        host, port = settings.MONDRIAN_XMLA_SERVER
        self.mdx_client = XMLAClient(host, port, settings.MONDRIAN_XMLA_PATH, '%s.IQuest' % settings.SETTINGS_MODULE)
        if not self.mdx_client.test_connection():
            raise SkipTest('No Mondrian connection available')


    def create_survey(self):
        """
        Helper method for creating a survey state


        TODO: create various questions (text, int, float), matrix etc

        """

        state = {
            "survey": {
                "name": u"My Response Survey",
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-dom-id",

                "availableLocales": ['en_GB'],

                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },

                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },

                "children": {
                    "blocks": [

                        # First block contains only one column
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID": u"yui-gen42",
                            "pageEnd": False,
                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': []
                                    },
                                    "objectName": u"SurveyBlockColumn",
                                    "dbID": None,
                                    "domID": u"yui-gen44",
                                    "width":100
                                }],
                            }
                        },

                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen59",
                            "pageEnd": False,

                            "children": {
                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen61",
                                    "width":50,

                                    "children": {
                                        "items":[

                                        # first question is a simple single choice question
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-1",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'Do you like cows?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-1-1",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-1-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Yes'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-1-2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'No.. bleh'
                                                        }
                                                    },
                                                }]
                                            }
                                        },

                                        # second question is a matrix question
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-2",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'Do you like ..?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-1",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Dog'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-2",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cat'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-3",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cow'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-4",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Chicken'
                                                        }
                                                    },
                                                },
                                                ],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-2-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Little'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-2-2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Medium'
                                                        }
                                                    },
                                                }, {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-2-3",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'A Lot'
                                                        }
                                                    },
                                                }]
                                            }
                                        },

                                        # third question is an numeric open question
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-3",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'But how many cows do you have?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-3-1",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-3-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Number of cows'
                                                        }
                                                    },
                                                },
                                            ]}
                                        },

                                        # fourth is float matrix
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-4",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'How much?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-1",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Dog'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-2",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cat'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-3",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cow'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-4",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Chicken'
                                                        }
                                                    },
                                                },
                                                ],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-4-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Yesterday'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-4-2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Today'
                                                        }
                                                    },
                                                }, {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-4-3",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Tomorrow'
                                                        }
                                                    },
                                                }]
                                            }
                                        },

                                        ]
                                    }
                                }

                                ]
                            }
                        },
                    ]
                }
            }
        }

        # save the survey
        map = self.compose_controller.save_state(state)
        survey_id = map['iquest-canvas-dom-id']

        survey = self.session.query(iquest_models.Survey).get(survey_id)
        numeric_itemqa = survey.itemQAs[2]
        float_itemqa = survey.itemQAs[3]

        filter_numeric = self.session.query(IQuestQAFilter).filter(IQuestQAFilter.name==u'Numeric').one()

        ifc = SurveyItemQAFilterController(self.session, None, self.account.id, StateManager(None, None, None))

        ifc.save_or_update_item_filter(None, filter_numeric, numeric_itemqa, attributes={'domain': 'int'})
        ifc.save_or_update_item_filter(None, filter_numeric, float_itemqa, attributes={'domain': 'float'})

        return survey_id

    def publish_survey(self, survey_id, status=None, begin_date=None, end_date=None, contact_lists=None, anonymous=False):

        if contact_lists is None:
            contact_lists = []

        if status is None:
            status = iquest_models.SurveyPublication.STATUS_SCHEDULED

        if begin_date is None:
            begin_date = datetime.now()

        publish_state = {
            'survey_id': survey_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": end_date,
            "status": status,
            "label": u"publication 1",
            "show_page_nrs": True,
            "anonymous": anonymous,
            "contact_lists": contact_lists,
            "viewer_print": False,
            "mailings": [] # no mailings necessary
        }

        # save publication
        publication = self.publication_controller.save(publish_state)

        # publish the surveys like the scheduler would do
        schedule.PublishSurveysTask().execute()
        schedule.CreatePublicationInviteesTask().execute()

        self.session.expire(publication)
        return publication.id


    def save_response(self, survey_id, publication_id, answer_q1=None, answer_q2=None, answer_q3=None, answer_q4=None):

        # get some vars
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        publication = self.session.query(iquest_models.SurveyPublication).get(publication_id)
        assert publication.survey.id == survey_id

        # for now assume anonymous survey so get that hash
        invitee_hash = publication.invitees[0].invitee_hash

        # init the controller
        controller = ParticipantController(self.session, self.locale, invitee_hash)

        # instanciate survey by invitee
        state = controller.load_survey_state()
        self.assertTrue(isinstance(state['survey']['dbID'], (long, int)))


        # q1 (do you like cows? yes/no)
        if answer_q1 is not None:
            responses = []
            q1_a_id = {'yes': survey.itemQAs[0].answers[0].id, 'no': survey.itemQAs[0].answers[1].id}[answer_q1]
            responses.append({'q': survey.itemQAs[0].questions[0].id, 'a': q1_a_id, 'v': 1},)
            controller.update_responses([{'item_id': survey.itemQAs[0].id, 'responses': responses}])

        if answer_q2 is not None:

            responses = []
            for i in range(len(answer_q2)):
                for j in range(len(answer_q2[i])):

                    q_id = survey.itemQAs[1].questions[i].id
                    a_id = survey.itemQAs[1].answers[j].id

                    if answer_q2[i][j]:
                        responses.append({'q': q_id, 'a': a_id, 'v': 1})

            controller.update_responses([{'item_id': survey.itemQAs[1].id, 'responses': responses}])

        if answer_q3 is not None:
            responses = []
            q_id = survey.itemQAs[2].questions[0].id
            a_id = survey.itemQAs[2].answers[0].id
            responses.append({'q': q_id, 'a': a_id, 'v': answer_q3}, )
            controller.update_responses([{'item_id': survey.itemQAs[2].id, 'responses': responses}])

        if answer_q4 is not None:

            responses = []
            for i in range(len(answer_q4)):
                for j in range(len(answer_q4[i])):

                    q_id = survey.itemQAs[3].questions[i].id
                    a_id = survey.itemQAs[3].answers[j].id

                    responses.append({'q': q_id, 'a': a_id, 'v': answer_q4[i][j]})

            controller.update_responses([{'item_id': survey.itemQAs[3].id, 'responses': responses}])



    def test_report(self, assert_=True):

        from iqportal.iqube.core.models import DataSource
        datasource = self.session.query(DataSource).filter(func.substr(DataSource.name, -6) == 'IQuest').all()
        if len(datasource) == 0:
            raise Exception('No IQuest datasource')
        datasource = datasource[0]

        survey_id = self.create_survey()
        publication1_id = self.publish_survey(survey_id, anonymous=True)
        publication2_id = self.publish_survey(survey_id, anonymous=True)


        # q1: 3 yes, 2 nope
        for i in range(3):
            self.save_response(survey_id, publication1_id, answer_q1='yes')
        for i in range(2):
            self.save_response(survey_id, publication1_id, answer_q1='no')

        # q2
        for i in range(8):
            self.save_response(survey_id, publication1_id, answer_q2 = [
                [i in (1,), 0, i  > 2],
                [i < 5, 0, i > 5],
                [i in (1,2,3,4), 0, i  > 4],
                [i < 3, 0, i >= 3],
            ])

        # q3: number of cows
        for i in range(10):
            self.save_response(survey_id, publication1_id, answer_q3=i*3)

        # q4: how much
        for i in range(10):
            self.save_response(survey_id, publication1_id, answer_q4 = [
                [i*10.5, 0, i*5.3],
                [i*2.6, 0, i*1.8],
                [i*78.2, 0, 0],
                [0, 0, 0],
            ])

        # responses publication 2
        self.save_response(survey_id, publication2_id, answer_q1='yes')

        if assert_:

            items = self.session.query(SurveyItem).join(SurveyBlockColumn).join(SurveyBlock).join(Survey).filter(Survey.id==survey_id).all()
            item_1 = items[0].id
            item_2 = items[1].id

            response_data = {
                publication1_id: {
                    item_1: {
                        'Yes': 3,
                        'No.. bleh': 2
                    }
                },
                publication2_id: {
                    item_1: {
                        'Yes': 1
                    }
                }
            }

            pc = ProjectController(self.session)

            # rewrite the cube so it creates the correspondent role
            pc.flush_schema(datasource.id)
            agg = AggregateResponsesTask()

            # run the aggregators for the new responses
            agg.populate()

            locale_id = 1

            for publication_id in (publication1_id, publication2_id):

                mdx = """
                    SELECT
                            {[Question].[%(item_id)s].[%(locale_id)s].Children} ON COLUMNS,
                            {[Answer].[%(item_id)s].[%(locale_id)s].Children} ON ROWS
                    FROM [Response]
                    WHERE [Publication].[%(publication_id)s]
                """ % {
                    'item_id': item_1,
                    'locale_id': locale_id,
                    'survey_id': survey_id,
                    'publication_id': publication_id,
                }

                result = self.mdx_client.execute_mdx(mdx)

                data = result.get_data()
                #print 'data: %s' % data

                self.assertEquals(len(data), len(response_data[publication_id][item_1]))
                i = 0
                for key, v in response_data[publication_id][item_1].items():
                    self.assertEquals(data[i]['value'], v)
                    i = i+1

                headers = result.get_headers()
                #print headers['rows']

        return [publication1_id, publication2_id]

# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['trial.py', 'iqportal.iquest.test.unit.test_report.ReportTest.test_report']
    run()
    sys.exit()




