# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import func
from datetime import datetime

from iqportal.iquest.test.unit import iquest_unittest
from basesoft.web.contrib.contact.controllers import ContactController

from iqportal.iquest.core.models import Survey, SurveyPublication
from iqportal.iqreport.core.mailing import MailController
from basesoft.web.core.util.controllers import StateManager

from iqportal.iquest.report.controllers import IQuestReportPublicationStateController
from iqportal.iquest.report.models import ReportIQuestPublication
from iqportal.iqreport.compose.controllers import ComposeController as reportComposeController
from basesoft.web.contrib.compose.models import ComposeInvitee, ComposePublicationMailing
from iqportal.iquest.response.controllers import ParticipantController
from basesoft.web.contrib.mailing.models import MailResult
from iqportal.iqube.core.controllers import ConfigurableWidgetController, ProjectController
from iqportal.iqube.core.reporters import ReporterRpcHandler
from iqportal.iqube.core.db_sync import AbstractConfigurableWidget
from iqportal.iqube.core.models import DataSource, DataSourceWidget
from iqportal.iquest.report.schedule import AggregateResponsesTask
from iqportal.iqreport.core.models import ReportPublication
from iqportal.iqreport.core.schedule import ReportPublicationCacheTask,\
    PublishReportsTask, CreatePublicationInviteesTask

class IQuestReportPublicationTest(iquest_unittest.IQuestTestCase):
    """
    """

    def setUp(self):
        """
        """
        #self.reset_db()
        self.connect_db()

        #create an account so we can check permission with it
        self.account = self._create_active_account()

        # create the necessary controllers
        self.group = self.account.groups[0].group

        # initialise compose controller
        from basesoft.web.contrib.i18n import models as i18n_models
        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()

        self.manager = StateManager(self.locale, self.account.id, None)

        # give current group access to create surveys
        self._attach_permissions_to_group(self.group, 'create_survey', 'create_survey_publication',
                                          'schedule_survey_publication', 'view_survey',
                                          'schedule_iqreport_mail', 'view_datasource_widget', 'view_datasource')

        # create some contact lists and mails
        self.setUpContactLists()
        self.setUpMails()

    def set_datasource(self):

        ds = 'IQuest'

        datasource = self.sa_session.query(DataSource).filter(func.substr(DataSource.name, -(len(ds))) == ds).all()
        if len(datasource) == 0:
            raise Exception('No IQuest datasource')
        self.datasource = datasource[0]

        pc = ProjectController(self.sa_session, account_id=self.account.id)
        pc.flush_schema(self.datasource.id)

        self.client = self.datasource.connect(account_id=self.account.id)

    def _get_configurable_widget_defaults(self, label, **kwargs):

        cwc = ConfigurableWidgetController(self.sa_session, account_id=self.account.id)
        widget = self.sa_session.query(DataSourceWidget).filter(DataSourceWidget.label==label).first()
        # so dbsync did not run. Run the part that creates the confiurable widgets.
        if widget is None:
            from iqportal.iqube.core.db_sync import post_sync
            post_sync(self.sa_session, None)
            widget = self.sa_session.query(DataSourceWidget).filter(DataSourceWidget.label==label).first()
        cwc.grant_privilege_to(widget, widget.PRIV_VIEW, self.group, check_own_privs=False)
        config, applied_config = cwc.get_state(widget)
        return (ReporterRpcHandler(self.sa_session, self.datasource, config, account_id=self.account.id, **kwargs), applied_config, )

    def _smorgle_applied_config(self, applied_config, dimensions, cube):
        dims = applied_config['dimensions']
        for d, config in dimensions.iteritems():
            applied_config['dimension_%s' % dims[d]] = config

        acw = AbstractConfigurableWidget(self.sa_session)
        acw.modify_filters(applied_config, self.client, cube)

        return applied_config

    def setUpContactLists(self):
        """
        Creates 3 contacts and 2 contact lists
        """
        self._attach_permissions_to_group(self.group, 'link_to_contact_list', 'create_contact', 'create_contact_list',
                                          'view_contact_list')
        contact_controller = ContactController(self.session, account_id=self.account.id)

        c1 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c1', 'email': u'testing@mail.clusternet'}, None, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c2', 'email': u'testing@mail.clusternet'}, None, self.group.id)
        c3 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c3', 'email': u'testing@mail.clusternet'}, None, self.group.id)

        self.cl1 = contact_controller.save_or_update_contact_list(None, 'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], self.cl1)

        self.cl2 = contact_controller.save_or_update_contact_list(None, 'Contact List 2', self.group.id)
        contact_controller.link_contacts([c3], self.cl2)

    def get_mail_state(self, name):
        return {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": name,
                "objectName": u"IQReportMail",
                "dbID": None,
                "locales": {
                    'nl_NL': {
                        'subject': u"iqreport uitnodiging",
                        'content': u'Geachte <img class="replacement-field" src="Contact:Name" title="contact.person.last_name"/> <br /><br /> \
                                  Klikie op de ze link:.<br/>\
                                  <img class="replacement-field" src="Report:Link" title="iqreport.report.link"/>',
                    },
                    'en_GB': {
                        'subject': u"iqreport invite",
                        'content': u'Dear <img class="replacement-field" src="Contact:Name" title="contact.person.last_name"/> <br /><br /> \
                                  Clicky on ze link:.<br/>\
                                  <img class="replacement-field" src="Report:Link" title="iqreport.report.link"/>',
                    },
                },
                "fallbackLocale": {'value': u'en_GB' },
            }
        }

    def setUpMails(self):

        self._attach_permissions_to_group(self.group, 'create_mail', 'create_iqreport_mail')
        controller = MailController(self.session, self.locale, account_id=self.account.id)
        self.mail1 = controller.save_state(self.get_mail_state(u'IQReport Mail 1'), self.group)

    def get_report_state(self, survey):
        """
        """
        return {
            "report": {
                "name": u"My Pretty Report",
                "objectName": u"Report",
                "dbID": None,
                "domID": u"iqreport-canvas-yui-gen5",

                "locales": {
                    'nl_NL': {
                        'title': u'Report Title Dutch',
                    },
                    'en_GB': {
                        'title': u'Report Title English'
                    },
                    'de_DE': {
                        'title': u'Report Title German'
                    }
                },

                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },
                "availableLocales": ['nl_NL', 'en_GB'],

                "theme": {
                    "cssClass": u"iqreport-basesoft",
                    "name": u"BaseSoft",
                    "module": u"iqreport_canvas_theme_basesoft"
                },

                "children": {
                    "blocks": [

                        # First block contains only one column
                        {
                            "objectName": u"ReportBlock",
                            "pageEnd": False,
                            "dbID": None,
                            "domID": u"yui-gen42",
                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': self._get_widgets(survey)
                                    },
                                    "objectName": u"ReportBlockColumn",
                                    "dbID": None,
                                    "domID": u"yui-gen44",
                                    "width": 100
                                }],
                            }
                        },
                    ]}

            }
        }

    def _get_widgets(self, survey):
        rv = []

        self.set_datasource()
        reporter, applied_config, = self._get_configurable_widget_defaults('Default QA Item')

        qas = survey.itemQAs
        locale_id = survey.fallback_locale.id

        first = qas[0]

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (survey.id, survey.publications[0].id)],
            'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, first.answers[0].id)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualFilteredResponse')

        widget = {
            "objectName": "ReportItemDataSourceWidget",
            "loaderName": "configurable_report_item",
            "requires": ["level_filter_fancy_select", "iq_fancyselect"],
            "datasource_widget_id": reporter.config['id'],
            "title": "Default QA Item",
            "dbID": None,
            "domID": u"yui-gen666",
            "tableConfig": {
                "slicersExpanded": False,
                "isTransposed": False
            },
            "appliedConfig": applied_config,
            "config": reporter.config,
            "reporter": "iqportal.iquest.report.DefaultQAItemReporter",
            "graphConfig": {},
            "showTitle": True
        }
        rv.append(widget)

        return rv

    def test_publish_report(self):

        # start django for iqube to connect for roles info
        self._start_django()

        # first we need a survey publication
        survey_id = self.publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id])
        survey = self.session.query(Survey).get(survey_id)

        # we have 2 invites for this survey publication, add responses for one
        invitees = self.session.query(
            ComposeInvitee).join(SurveyPublication).join(Survey).filter(Survey.id==survey_id).all()

        invitee_hash = unicode(invitees[0].invitee_hash)
        controller = ParticipantController(self.session, self.locale, invitee_hash)

        state = []
        # multiple choice
        mqid = survey.itemQAs[0].questions[0].id
        maid1 = survey.itemQAs[0].answers[0].id
        maid2 = survey.itemQAs[0].answers[1].id
        state.append({ 'item_id': survey.itemQAs[1].id,
            'responses': [
                {'q': mqid, 'a': maid1, 'v': True}
            ]
        })
        controller.update_responses(state)

        AggregateResponsesTask().execute()

        self._attach_permissions_to_group(self.group, 'view_report', 'create_report', 'update_report', 'view_datasource', 'view_survey_publication_response')

        # now make a report
        report_state = self.get_report_state(survey)
        controller = reportComposeController(self.session, self.locale, account_id=self.account.id)

        # save twice to test cache clearing
        id_map = controller.save_state(report_state)
        report_id = id_map['iqreport-canvas-yui-gen5']
        report_state['report']['dbID'] = report_id
        id_map = controller.save_state(report_state)

        report_id = id_map['iqreport-canvas-yui-gen5']

        report_publish_state = {
            "report_id": report_id,
            "survey_publication_id": self._publication.id,
            "dbID": None,
            "domID": u'report_publish_1',
            "begin_date": datetime.now(),
            "end_date": None,
            "status": ReportIQuestPublication.STATUS_SCHEDULED,
            "label": u"publication 1",
            "show_page_nrs": True,
            "anonymous": True,
            "contact_lists": [self.cl1.id, self.cl2.id],
            "viewer_print": False,
            "mailings": [{
                "dbID": None,
                "domID": u'publish_1-mailing_1',
                "mail_id": self.mail1.id,
                "send_date": datetime.now(),
                "contact_lists": [self.cl1.id, self.cl2.id],
                'mail_from_text': 'mail from text',
                "filter": ComposePublicationMailing.FILTER_NON_RESPONDENTS
            }]
        }
        self._attach_permissions_to_group(self.group, 'create_report_publication', 'schedule_report_publication', 'view_survey_publication_response')
        controller = IQuestReportPublicationStateController(self.session, self.manager)
        report_publication = controller.save(report_publish_state)

        # publish the report like the scheduler would do
        rv = PublishReportsTask().execute()
        self.assertEqual(len(rv), 1)
        CreatePublicationInviteesTask().execute()
        # run the cache so that report will be approved for sending
        ReportPublicationCacheTask().execute()
        # and now mailings should be sent
        rv = PublishReportsTask().execute()
        self.assertEqual(len(rv), 0)

        # now check mail result. We should have 2 emails sent: one to the one contact in the cl2 (whcih was not linked to
        # the survey publication) and the other one is the survey respondent of cl1 (cuz we have a mailing filter
        # for non respondents on the report publication)
        self.assertEqual(len(report_publication.mailings), 1)
        mailing = report_publication.mailings[0]

        mail_results = self.session.query(MailResult).filter(MailResult.mailing_id==mailing.id).all()
        self.assertEqual(len(mail_results), 2)
        # first contact of the cl1 is respondent, so report email should be sent to him
        self.assertEqual(mail_results[0].contact.person_id, self.cl1.contacts[0].person_id)
        self.assertEqual(mail_results[1].contact.person_id, self.cl2.contacts[0].person_id)

        self.sa_session.expire(report_publication)
        # check if we have cached XMLA results for this publication.
        self.assertEqual(report_publication.cache_status, ReportPublication.STATUS_CACHE_PROCESSED)
