# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from basesoft.web.contrib.account.models import Account

from iqportal.iqube.test.unit.test_configurable_widget_reporters import AbstractConfigurableWidgetReporterTestCase
from iqportal.client.test.controllers import FmeTestController
from iqportal.iqube.core.controllers import ProjectController, MdxMethodController
from iqportal.iquest.core.models import Survey
from iqportal.iquest.report.schedule import AggregateResponsesTask
from iqportal.iqube.xmla.client import XMLAClient
from twisted.trial.unittest import SkipTest
from iqportal.iqube.core.models import MdxMethod
from iqportal.iqube.compose.controllers import MdxCalculatedMemberController

class TestCase(AbstractConfigurableWidgetReporterTestCase):
    DATASOURCE = 'IQuest'

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary fme account data
        c = FmeTestController(self.session, account_id=account.id)
        c.populate_fme_domain()
        self.feda_license = c.feda_license
        self.fedet_license = c.fedet_license
        account = c.admin_account

        self.group = account.groups[0].group
        self.account = account
        self._attach_permissions_to_group(self.group, 'create_individual_reports')

        self.set_datasource()

        survey_id = c.populate_survey()
        survey = self.sa_session.query(Survey).filter(Survey.id==survey_id).one()
        c.add_weight_factors(survey)

        self.publication_ids = []

        pc = ProjectController(self.sa_session, account_id=self.account.id)
        for x in range(2):
            publication_id = c.publish_survey(survey_id, label=x)
            self.publication_ids.append(publication_id)

            for i in range(3):  # 3 respondents
                c.populate_responses(publication_id, seed='bier'*i*x)

        # aggregator
        pc.flush_schema(self.datasource.id)
        AggregateResponsesTask().execute()
        self.survey = self.sa_session.query(Survey).filter(Survey.id==survey_id).one()

        # start django for iqube to connect for roles info
        self._start_django()

    def test_weight_factor_aggregator(self):
        # add weight factors for some of the QA's
        # QA1 is Text, we'll add weight per Answer
        mc = self.survey.itemQAs[0]
        # QA2 is Single Choice (NPS), we'll add weight per Question-Answer
        sc = self.survey.itemQAs[2]

        # validate Numerical input QA1
        validate_answer = mc.answers[len(mc.answers)-1]
        weight = validate_answer.position+1

        rows = self.sa_session.execute('SELECT response_int, weighted_int FROM aggFullIQuestResponse WHERE answer_id = %s' % validate_answer.id).fetchall()
        for response, weighted in rows:
            self.assertEqual(weighted, response*weight)

        # validate NPS input QA3
        validate_question = sc.questions[len(sc.questions)-1]
        validate_answer = sc.answers[len(sc.answers)-1]
        weight = (validate_answer.position+1)*(validate_question.position+1)

        rows = self.sa_session.execute('SELECT response_int, weighted_int FROM aggFullIQuestResponse WHERE answer_id = %s AND question_id = %s' % (validate_answer.id, validate_question.id)).fetchall()
        for response, weighted in rows:
            self.assertEqual(weighted, response*weight)

    def test_weight_factor_in_default_qa(self):
        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        last = qas[2] # this is the NPS

        sid = self.survey.id

        dimensions = {
            'Survey': ['[Survey].[%s]' % (sid)],
            'Question': ['[Question].[%s].[%s].[%s]' % (last.id, locale_id, q.id) for q in last.questions],
            'Answer': ['[Answer].[%s].[%s].Children' % (last.id, locale_id)],
            'Measures': ["[Measures].[Response Int Sum]", "[Measures].[Weighted Int Sum]", "[Measures].[Weighted Int Avg]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')

        # enable 3 measures (we use the response measure to assert the weighted result)
        applied_config['show_response_measure'] = True
        applied_config['show_weighted_measure'] = True
        applied_config['show_weighted_average_measure'] = True
        applied_config['hide_empty_fields'] = False

        answer_index = applied_config['dimensions']['Answer']
        applied_config['axes_%s' % answer_index] = 'rows'
        applied_config['column_order'] = []
        applied_config['row_order'] = ['Question', 'Answer']

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        host, port = settings.MONDRIAN_XMLA_SERVER
        mdx_client = XMLAClient(host, port, settings.MONDRIAN_XMLA_PATH, '%s.IQuest' % settings.SETTINGS_MODULE)
        result = mdx_client.execute_mdx(mdx[0])

        data = result.get_data()

        responses = {}
        # respones are filled randomly, but we should have a minimum of 1 answer filled.
        # (3 cells, one for response, one for weighted avg, one for weighted)
        self.assertTrue(len(data) > 2)
        for cell in data:
            ordinal = int(cell['cellOrdinal'])
            answer_i = int(ordinal/3)
            x = ordinal % 3
            if x == 0: # the responses
                responses[answer_i] = cell['value']
                continue

            multiplier = last.answers[answer_i].position+1

            if x == 2: # the weighted value:
                self.assertEqual((responses[answer_i]*multiplier), cell['value'])

    def test_default_qa_reporter_with_mdx_function(self):
        raise SkipTest('Make this work with the IQube compose instead')
        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id
        first = qas[0]

        sid = self.survey.id
        pid = self.survey.publications[0].id
        aid = first.answers[0].id

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            #'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')
        applied_config['show_response_measure'] = True
        applied_config['show_respondents_measure'] = False

        mmc = MdxMethodController(self.sa_session, account_id=self.account.id)
        m = mmc.save_or_update_mdx_method(None, 'Top 6', {"_slots":[{"_slots":[None, None, None],"_formula":{"name":"TopCount","fields":[{"field_type":"set","required_field":True,"method_id":1},{"field_type":"count","required_field":True,"method_id":1},{"field_type":"numeric","required_field":False,"method_id":1}],"example":None,"return_type":"set_expression","label":"Top count","id":1}}],"_formula":None}, 3)

        # add the mdx function we want to use
        applied_config['mdx_functions'] = [{
            'formula_id': m.id,
            'axis': 'rows',
            'replacements': ['{[Question].[%s].[%s].Children}' % (first.id, locale_id), 6, '[Measures].[Response]']
        }]

        # move the dimensions to the correct axis
        measures_index = applied_config['dimensions']['Measures']
        applied_config['axes_%s' % measures_index] = 'columns'
        applied_config['column_order'] = ['Measures']
        answer_index = applied_config['dimensions']['Answer']
        applied_config['axes_%s' % answer_index] = 'slicers'
        applied_config['row_order'] = ['formula_0']


        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH MEMBER [Measures].[Response] as '[Measures].[Responses Int Sum]+[Measures].[Responses Float Sum]'
                MEMBER [Measures].[Average] as '[Measures].[Responses Int Avg]+[Measures].[Responses Float Avg]'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Responses Int Avg per Respondent]+[Measures].[Responses Float Avg per Respondent]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as '[Measures].[Respondents]'
                MEMBER [Measures].[# of Companies] as '[Measures].[Companies]'
                MEMBER [Measures].[# of Responses] as '[Measures].[Response Count]'
                MEMBER [Measures].[Individual] as '0'
            SELECT NON EMPTY {[Measures].[Response]} ON COLUMNS,
                NON EMPTY {TOPCOUNT({%(questions)s}, 6, [Measures].[Response])} ON ROWS
            FROM [VirtualResponse]
            WHERE CROSSJOIN({[Answer].[%(iid)s].[%(lid)s].[%(aid)s]}, {[Survey].[%(sid)s].[%(pid)s]})
        """ % {
            'iid': first.id,
            'lid': locale_id,
            'aid': aid,
            'sid': sid,
            'pid': pid,
            'questions': '[Question].[%s].[%s].Children' % (first.id, locale_id)
        }, mdx[0])

    def test_default_qa_reporter_with_nested_mdx_function(self):
        raise SkipTest('Make this work with the IQube compose instead')
        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id
        first = qas[0]

        sid = self.survey.id
        pid = self.survey.publications[0].id
        aid = first.answers[0].id

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            #'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')
        applied_config['show_response_measure'] = True
        applied_config['show_respondents_measure'] = False

        mmc = MdxMethodController(self.sa_session, account_id=self.account.id)
        m = mmc.save_or_update_mdx_method(None, 'Top 6 of Top 6', {"_slots":[{"_slots":[{"_slots":[None,None,None],"_formula":{"name":"TopCount","fields":[{"field_type":"set","required_field":True,"method_id":1},{"field_type":"count","required_field":True,"method_id":1},{"field_type":"numeric","required_field":False,"method_id":1}],"example":None,"return_type":"set_expression","label":"Top count","id":1}},None,None],"_formula":{"name":"TopCount","fields":[{"field_type":"set","required_field":True,"method_id":1},{"field_type":"count","required_field":True,"method_id":1},{"field_type":"numeric","required_field":False,"method_id":1}],"example":None,"return_type":"set_expression","label":"Top count","id":1}}],"_formula":None}, 5)

        # add the mdx function we want to use
        applied_config['mdx_functions'] = [{
            'formula_id': m.id,
            'axis': 'rows',
            'replacements': ['{[Question].[%s].[%s].Children}' % (first.id, locale_id), 6, '[Measures].[Response]', 6, '[Measures].[Response]']
        }]

        # move the dimensions to the correct axis
        measures_index = applied_config['dimensions']['Measures']
        applied_config['axes_%s' % measures_index] = 'columns'
        applied_config['column_order'] = ['Measures']
        answer_index = applied_config['dimensions']['Answer']
        applied_config['axes_%s' % answer_index] = 'slicers'
        applied_config['row_order'] = ['formula_0']

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH MEMBER [Measures].[Response] as '[Measures].[Responses Int Sum]+[Measures].[Responses Float Sum]'
                MEMBER [Measures].[Average] as '[Measures].[Responses Int Avg]+[Measures].[Responses Float Avg]'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Responses Int Avg per Respondent]+[Measures].[Responses Float Avg per Respondent]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as '[Measures].[Respondents]'
                MEMBER [Measures].[# of Companies] as '[Measures].[Companies]'
                MEMBER [Measures].[# of Responses] as '[Measures].[Response Count]'
                MEMBER [Measures].[Individual] as '0'
            SELECT NON EMPTY {[Measures].[Response]} ON COLUMNS,
                NON EMPTY {TOPCOUNT(TOPCOUNT({%(questions)s}, 6, [Measures].[Response]), 6, [Measures].[Response])} ON ROWS
            FROM [VirtualResponse]
            WHERE CROSSJOIN({[Answer].[%(iid)s].[%(lid)s].[%(aid)s]}, {[Survey].[%(sid)s].[%(pid)s]})
        """ % {
            'iid': first.id,
            'lid': locale_id,
            'aid': aid,
            'sid': sid,
            'pid': pid,
            'questions': '[Question].[%s].[%s].Children' % (first.id, locale_id)
        }, mdx[0])

    def test_default_qa_reporter(self):
        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        first = qas[0]

        sid = self.survey.id
        pid = self.survey.publications[0].id
        aid = first.answers[0].id

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH MEMBER [Measures].[Response] as '[Measures].[Responses Int Sum]+[Measures].[Responses Float Sum]'
                MEMBER [Measures].[Average] as '[Measures].[Responses Int Avg]+[Measures].[Responses Float Avg]'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Responses Int Avg per Respondent]+[Measures].[Responses Float Avg per Respondent]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as '[Measures].[Respondents]'
                MEMBER [Measures].[# of Companies] as '[Measures].[Companies]'
                MEMBER [Measures].[# of Responses] as '[Measures].[Response Count]'
                MEMBER [Measures].[Individual] as '0'
            SELECT NON EMPTY {[Answer].[%(iid)s].[%(lid)s].[%(aid)s]} ON COLUMNS,
                NON EMPTY {%(questions)s} ON ROWS
            FROM [VirtualResponse]
            WHERE {[Survey].[%(sid)s].[%(pid)s]}
        """ % {
            'iid': first.id,
            'lid': locale_id,
            'aid': aid,
            'sid': sid,
            'pid': pid,
            'questions': ','.join(dimensions['Question'])
        }, mdx[0])

        # now we enable some other measures and put them on columns
        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')
        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')

        # we show some additional measures and we change the handle and decimals of the "average" measure
        applied_config['show_respondents_measure'] = True
        applied_config['show_response_measure'] = True
        applied_config['show_average_measure'] = True
        applied_config['average_label'] = 'AVG.'
        applied_config['average_decimals'] = 1
        applied_config['show_companies_measure'] = True

        measures_index = applied_config['dimensions']['Measures']
        applied_config['axes_%s' % measures_index] = 'columns'
        applied_config['column_order'] = ['Measures']
        answer_index = applied_config['dimensions']['Answer']
        applied_config['axes_%s' % answer_index] = 'slicers'

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH MEMBER [Measures].[Response] as '[Measures].[Responses Int Sum]+[Measures].[Responses Float Sum]'
                MEMBER [Measures].[AVG.] as '[Measures].[Responses Int Avg]+[Measures].[Responses Float Avg]', FORMAT_STRING='#,##0.0'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Responses Int Avg per Respondent]+[Measures].[Responses Float Avg per Respondent]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as '[Measures].[Respondents]'
                MEMBER [Measures].[# of Companies] as '[Measures].[Companies]'
                MEMBER [Measures].[# of Responses] as '[Measures].[Response Count]'
                MEMBER [Measures].[Individual] as '0'
            SELECT NON EMPTY {[Measures].[Response],[Measures].[AVG.],[Measures].[# of Respondents],[Measures].[# of Companies]} ON COLUMNS,
                NON EMPTY {%(questions)s} ON ROWS
            FROM [VirtualResponse]
            WHERE CROSSJOIN({[Answer].[%(iid)s].[%(lid)s].[%(aid)s]}, {[Survey].[%(sid)s].[%(pid)s]})
        """ % {
            'iid': first.id,
            'lid': locale_id,
            'aid': aid,
            'sid': sid,
            'pid': pid,
            'questions': ','.join(dimensions['Question'])
        }, mdx[0])

    def test_default_qa_reporter_basket_analysis(self):
        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        first = qas[0]

        sid = self.survey.id
        pid = self.survey.publications[0].id
        aid = first.answers[0].id

        second = qas[1]

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }

        ba_question = '[Question].[%s].[%s].[%s]' % (second.id,locale_id,second.questions[0].id)
        ba_answer = '[Answer].[%s].[%s].[%s]' % (second.id,locale_id,second.answers[0].id)

        applied_config['basket_analysis_questions'] = [ba_question]
        applied_config['basket_analysis_answers'] = [ba_answer]
        applied_config['basket_analysis_qa'] = ['[Item].[%s].[%s]' % (locale_id,second.id)]

        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH SET [BasketAnalysis] AS '[Item].[%(locale_id)s].[%(ba_item)s] * %(ba_question)s * %(ba_answer)s'
                MEMBER [Measures].[Response] as 'SUM([Participant].Children, IIF(SUM([BasketAnalysis]) > 0, [Measures].[Responses Int Sum]+[Measures].[Responses Float Sum], 0))'
                MEMBER [Measures].[Average] as '[Measures].[Response] / [Measures].[# of Responses]'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Response] / [Measures].[# of Respondents]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as 'SUM([Participant].Children, IIF(SUM([BasketAnalysis]) > 0, [Measures].[Respondents], 0))'
                MEMBER [Measures].[# of Companies] as 'SUM([Participant].Children, IIF(SUM([BasketAnalysis]) > 0, [Measures].[Companies], 0))'
                MEMBER [Measures].[# of Responses] as 'SUM([Participant].Children, IIF(SUM([BasketAnalysis]) > 0, 1, 0))'
                MEMBER [Measures].[Individual] as '0'
            SELECT NON EMPTY {%(answers)s} ON COLUMNS,
                NON EMPTY {%(questions)s} ON ROWS
            FROM [VirtualResponse]
            WHERE {%(survey)s}
        """ % {
            'locale_id': locale_id,
            'ba_item': second.id,
            'ba_question': ba_question,
            'ba_answer': ba_answer,
            'answers': dimensions['Answer'][0],
            'questions': ','.join(dimensions['Question']),
            'survey': dimensions['Survey'][0]
        }, mdx[0])

    def test_default_qa_reporter_indiv(self):
        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        first = qas[0]

        sid = self.survey.id
        pid = self.survey.publications[0].id
        aid = first.answers[0].id

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')
        applied_config['show_individual_measure'] = True

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH MEMBER [Measures].[Response] as '[Measures].[Responses Int Sum]+[Measures].[Responses Float Sum]'
                MEMBER [Measures].[Average] as '[Measures].[Responses Int Avg]+[Measures].[Responses Float Avg]'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Responses Int Avg per Respondent]+[Measures].[Responses Float Avg per Respondent]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as '[Measures].[Respondents]'
                MEMBER [Measures].[# of Companies] as '[Measures].[Companies]'
                MEMBER [Measures].[# of Responses] as '[Measures].[Response Count]'
                MEMBER [Measures].[Individual] as '0'
            SELECT NON EMPTY {[Answer].[%(iid)s].[%(lid)s].[%(aid)s]} ON COLUMNS,
                NON EMPTY {%(questions)s} ON ROWS
            FROM [VirtualResponse]
            WHERE CROSSJOIN({[Measures].[Individual]}, {[Survey].[%(sid)s].[%(pid)s]})
        """ % {
            'iid': first.id,
            'lid': locale_id,
            'aid': aid,
            'sid': sid,
            'pid': pid,
            'questions': ','.join(dimensions['Question'])
        }, mdx[0])

        # applied_config['show_market_share_measure'] = True

    def test_growth_reporter(self):
        reporter, applied_config = self._get_configurable_widget_defaults('Growth Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        first = qas[0]

        sid = self.survey.id
        aid = first.answers[0].id

        dimensions = {
            'Survey': ['[Survey].[%s].Children' % sid],
            'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH MEMBER [Measures].[Response] as '[Measures].[Responses Int Sum]+[Measures].[Responses Float Sum]'
                MEMBER [Measures].[Average] as '[Measures].[Responses Int Avg]+[Measures].[Responses Float Avg]'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Responses Int Avg per Respondent]+[Measures].[Responses Float Avg per Respondent]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as '[Measures].[Respondents]'
                MEMBER [Measures].[# of Companies] as '[Measures].[Companies]'
                MEMBER [Measures].[# of Responses] as '[Measures].[Response Count]'
                MEMBER [Measures].[Individual] as '0'
            SELECT NON EMPTY CROSSJOIN({%(questions)s}, {[Answer].[%(iid)s].[%(lid)s].[%(aid)s]}) ON COLUMNS,
                NON EMPTY {[Survey].[%(sid)s].Children} ON ROWS
            FROM [VirtualResponse]
        """ % {
            'iid': first.id,
            'lid': locale_id,
            'aid': aid,
            'sid': sid,
            'questions': ','.join(dimensions['Question'])
        }, mdx[0])

    def test_growth_reporter_with_aggregate_publications(self):
        """
        Test case to validate that MEMBER AGGREGATE() works correctly within the growth item (#4596)
        """
        reporter, applied_config = self._get_configurable_widget_defaults('Growth Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        first = qas[0]

        sid = self.survey.id
        pid1 = self.survey.publications[0].id
        pid2 = self.survey.publications[1].id
        aid = first.answers[0].id

        dimensions = {
            'Survey': ["MEMBER [Survey].[Grouped1] AS 'AGGREGATE([Survey].[%s].[%s])'" % (sid, pid1), "MEMBER [Survey].[Grouped2] AS 'AGGREGATE([Survey].[%s].[%s])'" % (sid, pid2)],
            'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')
        applied_config['show_growth_measure'] = True

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH MEMBER [Measures].[Response] as '[Measures].[Responses Int Sum]+[Measures].[Responses Float Sum]'
                MEMBER [Measures].[Average] as '[Measures].[Responses Int Avg]+[Measures].[Responses Float Avg]'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Responses Int Avg per Respondent]+[Measures].[Responses Float Avg per Respondent]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as '[Measures].[Respondents]'
                MEMBER [Measures].[# of Companies] as '[Measures].[Companies]'
                MEMBER [Measures].[# of Responses] as '[Measures].[Response Count]'
                MEMBER [Measures].[Individual] as '0'
                MEMBER [Survey].[Grouped1] AS 'AGGREGATE([Survey].[%(sid)s].[%(pid1)s])'
                MEMBER [Survey].[Grouped2] AS 'AGGREGATE([Survey].[%(sid)s].[%(pid2)s])'
                SET [SurveySet] as '{[Survey].[Grouped1],[Survey].[Grouped2]}'
                SET [SurveySet2] as '{[Survey].[Grouped1],[Survey].[Grouped2]}'
                MEMBER [Measures].[Growth] AS '(([Measures].[Response], [SurveySet2].Item([SurveySet].CurrentOrdinal))-([Measures].[Response], [SurveySet2].Item([SurveySet].CurrentOrdinal-1)))/([Measures].[Response], [SurveySet2].Item([SurveySet].CurrentOrdinal-1))', FORMAT_STRING='0.0%%'
            SELECT NON EMPTY CROSSJOIN(CROSSJOIN({%(questions)s}, {[Answer].[%(iid)s].[%(lid)s].[%(aid)s]}), {[Measures].[Growth]}) ON COLUMNS,
                NON EMPTY {[SurveySet]} ON ROWS
            FROM [VirtualResponse]
        """ % {
            'iid': first.id,
            'lid': locale_id,
            'aid': aid,
            'sid': sid,
            'pid1': pid1,
            'pid2': pid2,
            'questions': ','.join(dimensions['Question'])
        }, mdx[0])

    def test_qa_totals_reporter(self):
        reporter, applied_config = self._get_configurable_widget_defaults('QA Totals Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        first = qas[0]
        second = qas[1]

        sid = self.survey.id
        pid = self.survey.publications[0].id
        aid1 = first.answers[0].id
        aid2 = second.answers[0].id

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            'Answer': ["MEMBER [Answer].[Turnover] AS '[Answer].[%s].[%s].[%s]+[Answer].[%s].[%s].[%s]'" % (first.id, locale_id, aid1, second.id, locale_id, aid2)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"],
            'Item': ['[Item].[%s].[%s]' % (locale_id, first.id), '[Item].[%s].[%s]' % (locale_id, second.id)]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')

        applied_config['row_total'] = True
        applied_config['total_label'] = 'Totally' # I know, right?

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH MEMBER [Answer].[Turnover] AS '[Answer].[%(iid1)s].[%(lid)s].[%(aid1)s]+[Answer].[%(iid2)s].[%(lid)s].[%(aid2)s]'
                MEMBER [Measures].[Response] as '[Measures].[Responses Int Sum]+[Measures].[Responses Float Sum]'
                MEMBER [Measures].[Average] as '[Measures].[Responses Int Avg]+[Measures].[Responses Float Avg]'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Responses Int Avg per Respondent]+[Measures].[Responses Float Avg per Respondent]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as '[Measures].[Respondents]'
                MEMBER [Measures].[# of Companies] as '[Measures].[Companies]'
                MEMBER [Measures].[# of Responses] as '[Measures].[Response Count]'
                MEMBER [Measures].[Individual] as '0'
                MEMBER [Item].[Totally] as '[Item].[%(lid)s].[%(iid1)s]+[Item].[%(lid)s].[%(iid2)s]'
            SELECT NON EMPTY {[Answer].[Turnover]} ON COLUMNS,
                NON EMPTY {[Item].[%(lid)s].[%(iid1)s],[Item].[%(lid)s].[%(iid2)s],[Item].[Totally]} ON ROWS
            FROM [VirtualResponse]
            WHERE {[Survey].[%(sid)s].[%(pid)s]}
        """ % {
            'iid1': first.id,
            'iid2': second.id,
            'lid': locale_id,
            'aid1': aid1,
            'aid2': aid2,
            'sid': sid,
            'pid': pid,
        }, mdx[0])

    def test_nps_reporter(self):
        reporter, applied_config = self._get_configurable_widget_defaults('NPS Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        nps = None
        for qa in qas:
            if qa.object_name == 'ItemQANPS':
                nps = qa
        assert nps is not None

        sid = self.survey.id
        a_ids = [a.id for a in nps.answers]

        dimensions = {
            'Survey': ['[Survey].[%s].Children' % sid],
            'Item': ['[Item].[%s].[%s]' % (locale_id, nps.id)],
            'Measures': ["[Measures].[Respondents]"]
        }
        applied_config['show_nps_detractors_measure'] = True
        applied_config['show_nps_passives_measure'] = True
        applied_config['show_nps_promotors_measure'] = True
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx(u"""
             WITH MEMBER [Measures].[Response] as '[Measures].[Responses Int Sum]+[Measures].[Responses Float Sum]'
                MEMBER [Measures].[Average] as '[Measures].[Responses Int Avg]+[Measures].[Responses Float Avg]'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Responses Int Avg per Respondent]+[Measures].[Responses Float Avg per Respondent]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as '[Measures].[Respondents]'
                MEMBER [Measures].[# of Companies] as '[Measures].[Companies]'
                MEMBER [Measures].[# of Responses] as '[Measures].[Response Count]'
                MEMBER [Measures].[Individual] as '0'
                MEMBER [Measures].[NPS Passives] AS '[Answer].[%(iid)s].[%(lid)s].[%(aid7)s]+[Answer].[%(iid)s].[%(lid)s].[%(aid8)s]'
                MEMBER [Measures].[NPS Promotors] AS '[Answer].[%(iid)s].[%(lid)s].[%(aid9)s]+[Answer].[%(iid)s].[%(lid)s].[%(aid10)s]'
                MEMBER [Measures].[NPS Detractors] AS '[Answer].[%(iid)s].[%(lid)s].[%(aid0)s]+[Answer].[%(iid)s].[%(lid)s].[%(aid1)s]+[Answer].[%(iid)s].[%(lid)s].[%(aid2)s]+[Answer].[%(iid)s].[%(lid)s].[%(aid3)s]+[Answer].[%(iid)s].[%(lid)s].[%(aid4)s]+[Answer].[%(iid)s].[%(lid)s].[%(aid5)s]+[Answer].[%(iid)s].[%(lid)s].[%(aid6)s]'
            SELECT NON EMPTY CROSSJOIN({[Item].[%(lid)s].[%(iid)s]}, {[Measures].[NPS Passives],[Measures].[NPS Promotors],[Measures].[NPS Detractors]}) ON COLUMNS,
                NON EMPTY {[Survey].[%(sid)s].Children} ON ROWS
            FROM [VirtualResponse]
        """ % {
            'iid': nps.id,
            'lid': locale_id,
            'sid': sid,
            'aid0': a_ids[0],
            'aid1': a_ids[1],
            'aid2': a_ids[2],
            'aid3': a_ids[3],
            'aid4': a_ids[4],
            'aid5': a_ids[5],
            'aid6': a_ids[6],
            'aid7': a_ids[7],
            'aid8': a_ids[8],
            'aid9': a_ids[9],
            'aid10': a_ids[10],
        }, mdx[0])

    def test_qa_totals_reporter_last_child(self):
        """
        This tests verifies that last child does not use unpublished surveys
        """
        import datetime
        begin_date = datetime.datetime.now()+datetime.timedelta(days=1)

        reporter, applied_config = self._get_configurable_widget_defaults('Growth Item')

        my_label = 'Unpublished'

        c = FmeTestController(self.session, account_id=self.account.id)
        c.publish_survey(self.survey.id, label=my_label, begin_date=begin_date)

        AggregateResponsesTask().execute()
        pc = ProjectController(self.sa_session, account_id=self.account.id)
        pc.flush_schema(self.datasource.id)

        sid = self.survey.id
        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        first = qas[0]

        dimensions = {
            'Survey': ['[Survey].[%s].Children' % (sid)],
            'Answer': ['[Answer].[%s].[%s].Children' % (first.id, locale_id)],
            'Question': ['[Question].[%s].[%s].Children' % (first.id, locale_id)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"],
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')
        applied_config['axis_0'] = 'rows'
        applied_config['hide_empty_fields'] = False # the publication of the future does not have data yet...

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))

        result = self.client.execute_mdx(mdx[0])
        publications = [x[0]['caption'] for x in result.get_headers()['rows']]
        self.assertFalse('publication %s' % my_label in publications)

    def test_calculated_member_in_measures(self):
        from django.utils import simplejson

        self._attach_permissions_to_group(self.group, 'view_calculated_member', 'create_calculated_member')

        cmc = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)

        sid = self.survey.id
        pid = self.survey.publications[0].id

        # create a calculation which uses the expression objects and datasource/cube.
        m = self.sa_session.query(MdxMethod).filter(MdxMethod.name=='Aggregate').first()
        cmc.grant_privilege_to(m, privileges=MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'numeric',
            'method_id': m.id,
            'inputs': [{
                'objectName': 'MdxDataSourceExpressionItem',
                'loaderName': 'iqube_datasource_expression_item',
                'return_type': 'set',
                'selection': [simplejson.dumps(x) for x in self.client.get_member_tree_op('VirtualResponse', '[Survey].[%s].[%s]' % (sid, pid), self.client.MDTREEOP_SELF)]
            }]
        }
        # create new CM (aggregate)
        cm = cmc.save_or_update_calculated_member(None, 'My Named Set', state, datasource_id=self.datasource.id, cube='VirtualResponse')

        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        last = qas[2]

        dimensions = {
            'Survey': ['[Survey].[%s]' % (sid)],
            'Question': ['[Question].[%s].[%s].[%s]' % (last.id, locale_id, q.id) for q in last.questions],
            'Answer': ['[Answer].[%s].[%s].Children' % (last.id, locale_id)],
            'Measures': ["[Measures].[Response Int Sum]", "[Measures].[Weighted Int Sum]", "[Measures].[Weighted Int Avg]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')

        # enable 3 measures (we use the response measure to assert the weighted result)
        applied_config['show_response_measure'] = True
        applied_config['calculated_measures'] = [{
            'calculated_member_id': cm.id,
            'replacements': [],
            'settings': {'label': 'My Test'}
        }]
        applied_config['hide_empty_fields'] = False

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH MEMBER [Measures].[Response] as '[Measures].[Responses Int Sum]+[Measures].[Responses Float Sum]'
                MEMBER [Measures].[Average] as '[Measures].[Responses Int Avg]+[Measures].[Responses Float Avg]'
                MEMBER [Measures].[Average per Respondent] as '[Measures].[Responses Int Avg per Respondent]+[Measures].[Responses Float Avg per Respondent]'
                MEMBER [Measures].[Weighted Average] as '[Measures].[Weighted Int Avg]+[Measures].[Weighted Float Avg]'
                MEMBER [Measures].[Weighted] as '[Measures].[Weighted Int Sum]+[Measures].[Weighted Float Sum]'
                MEMBER [Measures].[# of Respondents] as '[Measures].[Respondents]'
                MEMBER [Measures].[# of Companies] as '[Measures].[Companies]'
                MEMBER [Measures].[# of Responses] as '[Measures].[Response Count]'
                MEMBER [Measures].[Individual] as '0'
                MEMBER [Measures].[My Test] AS Aggregate({[Survey].[%(sid)s].[%(pid)s]})
            SELECT CROSSJOIN({[Answer].[%(iid)s].[%(lid)s].Children}, {[Measures].[Response],[Measures].[My Test]}) ON COLUMNS,
                {[Question].[%(iid)s].[%(lid)s].[%(qid)s]} ON ROWS
            FROM [VirtualResponse]
            WHERE {[Survey].[%(sid)s]}
        """ % {
            'iid': last.id,
            'lid': locale_id,
            'qid': q.id,
            'sid': sid,
            'pid': pid,
        }, mdx[0])

class IQuestConfigurableWidgetFilteredCubeTestCase(AbstractConfigurableWidgetReporterTestCase):
    DATASOURCE = 'IQuest'

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary fme account data
        c = FmeTestController(self.session, account_id=account.id)
        c.populate_fme_domain()
        self.feda_license = c.feda_license
        self.fedet_license = c.fedet_license
        account = c.admin_account

        self.group = account.groups[0].group
        self.account = account
        self._attach_permissions_to_group(self.group, 'create_individual_reports')

        self.set_datasource()

        survey_id = c.populate_survey(validate_market_share=True)

        pc = ProjectController(self.sa_session, account_id=self.account.id)
        r = [2, 10]
        for x in r:
            publication_id = c.publish_survey(survey_id, label='test_%s' % x, additional_state={'maximum_market_share': 65, 'minimum_respondents': x})

            for i in range(3):  # 3 respondents
                c.populate_responses(publication_id, seed=1*i*x)

        # aggregator
        AggregateResponsesTask().execute()
        pc.flush_schema(self.datasource.id)
        self.survey = self.sa_session.query(Survey).filter(Survey.id==survey_id).one()

        # start django for iqube to connect for roles info
        self._start_django()

    def test_filtered_cube(self):
        """
        Test retrieval of data for VirtualResponse vs. VirtualFilteredResponse
        """
        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id

        first = qas[0]

        sid = self.survey.id
        # the first publication only has the market share check working.
        pid = self.survey.publications[0].id

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            'Question': ['[Question].[%s].[%s].Children' % (first.id, locale_id)],
            'Answer': ['[Answer].[%s].[%s].Children' % (first.id, locale_id)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')

        applied_config['show_response_measure'] = True
        applied_config['hide_empty_fields'] = False

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        result = self.client.execute_mdx(mdx[0])
        unfiltered_data = result.get_data()

        filtered = mdx[0].replace('VirtualResponse', 'VirtualFilteredResponse')
        result = self.client.execute_mdx(filtered)
        filtered_data = result.get_data()

        self.assertNotEqual(unfiltered_data, filtered_data)
        self.assertEqual(len(unfiltered_data), 24)
        # question position 5 with answer position 4 is not in the filtered cube.
        self.assertEqual(len(filtered_data), 23)

        # ==== Validate the above statement ====

        # now we query only that question and answer
        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        qid = first.questions[4].id
        aid = first.answers[3].id

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, qid)],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')
        applied_config['show_response_measure'] = True
        applied_config['hide_empty_fields'] = False

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        result = self.client.execute_mdx(mdx[0])
        unfiltered_data = result.get_data()

        filtered = mdx[0].replace('VirtualResponse', 'VirtualFilteredResponse')
        result = self.client.execute_mdx(filtered)
        filtered_data = result.get_data()

        self.assertNotEqual(unfiltered_data, filtered_data)
        self.assertEqual(len(unfiltered_data), 1)
        # question position 2 with answer position 4 is not in the filtered cube.
        self.assertEqual(len(filtered_data), 1)
        # make sure the value is empty
        self.assertEqual(filtered_data[0]['value'],'')

        # Now we validate the second publication
        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        pid = self.survey.publications[1].id

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            'Question': ['[Question].[%s].[%s].Children' % (first.id, locale_id)],
            'Answer': ['[Answer].[%s].[%s].Children' % (first.id, locale_id)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse')
        applied_config['show_response_measure'] = True
        applied_config['hide_empty_fields'] = False

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        result = self.client.execute_mdx(mdx[0])
        unfiltered_data = result.get_data()

        filtered = mdx[0].replace('VirtualResponse', 'VirtualFilteredResponse')
        result = self.client.execute_mdx(filtered)
        filtered_data = result.get_data()

        self.assertNotEqual(unfiltered_data, filtered_data)
        self.assertEqual(len(unfiltered_data), 24)
        # question position 2 with answer position 4 is not in the filtered cube.
        self.assertEqual(len(filtered_data), 1)
        # make sure the value is empty
        self.assertEqual(filtered_data[0]['value'],'')

