# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime, timedelta

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.contact.controllers import ContactController
from basesoft.web.core import exceptions as basesoft_exceptions

from iqportal.iquest.core import schedule
from iqportal.iquest.core import models as iquest_models
from iqportal.iquest.response import models as response_models
from basesoft.web.contrib.compose.models import ComposeInvitee,\
    ComposePublicationLocale, ComposePublication

from iqportal.iquest.compose.controllers import ComposeController, PublicationStateController
from iqportal.iquest.response.controllers import ParticipantController, PublicationResponseController
from basesoft.web.core.exceptions import PermissionDeniedException

from basesoft.web.core.util.controllers import StateManager
from iqportal.iquest.core.models import SurveyPublication, Survey
from basesoft.web.contrib.license.models import LicenseConfig
from basesoft.web.contrib.account.models import Config
from basesoft.web.contrib.i18n.models import Locale

class ResponseTest(iqportal_unittest.IQPortalTestCase):
    """
    """

    def setUp(self):
        """
        """
        self.connect_db()

        #create an account so we can check permission with i
        self.account = self._create_active_account()

        # create the necessary controllers
        self.group = self.account.groups[0].group

        from basesoft.web.contrib.i18n import models as i18n_models
        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()

        self.compose_controller = ComposeController(self.session, self.locale, account_id=self.account.id)
        self.publication_controller = PublicationStateController(self.session, StateManager(self.locale, self.account.id, None))

        # create some contact lists
        self.setUpContactLists()

        # give current group access to create surveys
        self._attach_permissions_to_group(self.group, 'create_survey')


    def setUpContactLists(self):
        """
        Creates 3 contacts and 2 contact lists
        """
        self._attach_permissions_to_group(self.group, 'link_to_contact_list', 'create_contact', 'create_contact_list',
                                          'view_survey', 'create_survey_publication', 'view_contact_list',
                                          'schedule_survey_publication')
        contact_controller = ContactController(self.session, account_id=self.account.id)


        # 3 contacts.. first two have a person with locale set.. last is just a contact

        c1 = contact_controller.save_or_update_contact(None, {'last_name': 'de Vries'}, None, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, {'last_name': 'the Freeze'}, None, self.group.id)
        c3 = contact_controller.save_or_update_contact(None, None, None, self.group.id)
        c4 = contact_controller.save_or_update_contact(None, {'last_name': 'sie Freusch'}, None, self.group.id)
        c5 = contact_controller.save_or_update_contact(None, {'last_name': 'le Vurie'}, None, self.group.id)

        locale_nl = self.sa_session.query(Locale).filter(Locale.locale=='nl_NL').one()
        locale_en = self.sa_session.query(Locale).filter(Locale.locale=='en_GB').one()
        locale_de = self.sa_session.query(Locale).filter(Locale.locale=='de_DE').one()
        locale_fr = self.sa_session.query(Locale).filter(Locale.locale=='fr_FR').one()

        c1.person.preferred_locale = locale_nl
        c2.person.preferred_locale = locale_en
        c4.person.preferred_locale = locale_de
        c5.person.preferred_locale = locale_fr

        self.cl1 = contact_controller.save_or_update_contact_list(None, u'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], self.cl1)

        self.cl2 = contact_controller.save_or_update_contact_list(None, u'Contact List 2', self.group.id)
        contact_controller.link_contacts([c2, c3], self.cl2)

        self.cl3 = contact_controller.save_or_update_contact_list(None, u'Contact List 3', self.group.id)
        contact_controller.link_contacts([c1, c2, c3, c4, c5], self.cl3)


    def publish_survey(self, status, begin_date, end_date=None, contact_lists=None, anonymous=False):
        """
        Helper method for creating a survey state
        """


        if contact_lists is None:
            contact_lists = []

        state = {
            "survey": {
                "name": u"My Response Survey",
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-dom-id",
                "availableLocales": ['en_GB', 'nl_NL', 'fr_FR'],

                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },

                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },

                "children": {
                    "blocks": [

                        # First block contains only one column
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID": u"yui-gen42",
                            "pageEnd": False,
                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': [{
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen7756",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'Text question Dutch',
                                                },
                                                'en_GB': {
                                                    u'label': u'Text question English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"yui-gen20544",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"yui-gen7933",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'', # untranslated, should use fallback
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Text question, first answer English'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"yui-gen8433",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'', # empty label and fallback is not set
                                                        },
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"yui-gen8434",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'', # only fallback locale is set (but empty)
                                                        },
                                                    },
                                                }]
                                            }
                                        }]
                                    },
                                    "objectName": u"SurveyBlockColumn",
                                    "dbID": None,
                                    "domID": u"yui-gen44",
                                    "width":100
                                }],
                            }
                        },

                        # Second block contains two empty columns
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen31",
                            "pageEnd": False,

                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': []
                                    },
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen33",
                                    "width":50
                                    },
                                    {
                                    "children":{
                                        'items': []
                                    },
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen53",
                                    "width":50
                                    }
                                ],
                            }
                        },

                        # Third block contains two columns with each a question
                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen59",
                            "pageEnd": False,

                            "children": {
                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen61",
                                    "width":50,

                                    "children": {
                                        "items":[{
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen77",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_MULTIPLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'First question Dutch',
                                                },
                                                'en_GB': {
                                                    u'label': u'First question English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"yui-gen205",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"yui-gen79",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'First question, first answer Dutch',
                                                        },
                                                        'en_GB': {
                                                            u'label': u'First question, first answer English'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"yui-gen84",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'First question, second answer Dutch',
                                                        },
                                                        'en_GB': {
                                                            u'label': u'First question, second answer English'
                                                        }
                                                    },
                                                }]
                                            }
                                        }]
                                    }
                                },
                                {
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen70",
                                    "width":50,

                                    "children": {
                                        "items":[{
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID": u"yui-gen92",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_BOTTOM,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_RIGHT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'Second question Dutch',
                                                },
                                                'en_GB': {
                                                    u'label': u'Second question English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"yui-gen206",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"yui-gen94",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Second question, first answer Dutch',
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Second question, first answer English'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"yui-gen99",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Second question, second answer Dutch',
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Second question, second answer English'
                                                        }
                                                    },
                                                }],
                                            }
                                        }, {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID": u"yui-gen9452",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_BOTTOM,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_RIGHT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'Second question Dutch',
                                                },
                                                'en_GB': {
                                                    u'label': u'Second question English'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"yui-gen20636",
                                                    "locales": {},
                                                },{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"yui-gen2063436",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"yui-gen92354",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Second question, first answer Dutch',
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Second question, first answer English'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"yui-gen6299",

                                                    "locales": {
                                                        'nl_NL': {
                                                            u'label': u'Second question, second answer Dutch',
                                                        },
                                                        'en_GB': {
                                                            u'label': u'Second question, second answer English'
                                                        }
                                                    },
                                                }],
                                            }
                                        }],
                                    }
                                }]
                            }
                        },

                        # The last block contains a column with a few textfields
                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen109",
                            "pageEnd": False,

                            "children": {

                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen111",
                                    "width":100,

                                    "children": {

                                        "items":[
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen201",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html': u'Textitem html dutch',
                                                },
                                                'en_GB': {
                                                    u'html': u'Textitem html english'
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen202",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html': u'Textitem 2 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html': u'Textitem 2 html english'
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen203",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html': u'Textitem 3 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html': u'Textitem 3 html english'
                                                }
                                            },
                                        },
                                        {
                                            "objectName":u"SurveyItemHtml",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen204",
                                            "locales": {
                                                'nl_NL': {
                                                    u'html': u'Textitem 4 html dutch',
                                                },
                                                'en_GB': {
                                                    u'html': u'Textitem 4 html english'
                                                }
                                            },
                                        },


                                        ],
                                    }
                                }],
                            }
                        }
                    ]
                }
            }
        }

        # save the survey
        map = self.compose_controller.save_state(state)
        survey_id = map['iquest-canvas-dom-id']

        # create publication
        publish_state = {
            'survey_id': survey_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": end_date,
            "status": status,
            "label": u"publication 1",
            "show_page_nrs": True,
            "anonymous": anonymous,
            "contact_lists": contact_lists,
            "mailings": [] # no mailings necessary
        }

        # save publication
        publication = self.publication_controller.save(publish_state)
        self.publication_controller.save_branch_cache(publication)

        # publish the surveys like the scheduler would do
        schedule.PublishSurveysTask().execute()
        schedule.CreatePublicationInviteesTask().execute()

        return survey_id

    def test_response(self):
        """
        Creates a non-anonymous survey and tries to init the controller for one of the contacts
        """

        # send out a survey that goes online now for a few contact lists
        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id, self.cl2.id])
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        # now lets pretend the contacts got an email with his hash which they needs to access the survey
        invitees = self.session.query(
            ComposeInvitee).join(
            iquest_models.SurveyPublication).join(
            iquest_models.Survey).filter(
                iquest_models.Survey.id==survey_id).all()

        invitee_hash = unicode(invitees[0].invitee_hash)
        invitee_hash2 = unicode(invitees[1].invitee_hash)
        p = invitees[0].publication
        self.assertEqual(p.number_of_invitees, 5)

        # instanciate a response controller for the first participan
        # and we expect there is a session created
        controller = ParticipantController(self.session, self.locale, invitee_hash)
        participant_session_id = controller.get_participant_session_id()

        # we are checking that we have one participant now.
        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==p.id).one()
        self.assertEqual(publication.number_of_participants, 1)
        self.assertEqual(publication.number_of_respondents, 0)

        # now imagine we are a new http request, we need to initialise the controller again but by passing in the
        # session_id so we continue the session from before. Expected is that no new session is created and that the
        # session_id stays the same.
        controller = ParticipantController(self.session, self.locale, participant_session_id=participant_session_id)
        self.assertEquals(participant_session_id, controller.get_participant_session_id())

        # we are checking that we still have one participant now.
        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==p.id).one()
        self.assertEqual(publication.number_of_participants, 1)

        # now lets say the user closed his browser and clicks again on the email link.. this should create a new session
        controller = ParticipantController(self.session, self.locale, invitee_hash)
        self.assertNotEquals(participant_session_id, controller.get_participant_session_id())

        # we are checking that we still have one participant now.
        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==p.id).one()
        self.assertEqual(publication.number_of_participants, 1)

        #
        # Fill responses
        # the user entered some responses which he likes to submi
        #

        # first a quick survey check.. there should be two QA's, and for the first QA we have 1 qt and 2 answers in the survey.. then get its id's
        self.assertEqual(len(survey.itemQAs), 4)
        self.assertEqual(len(survey.itemQAs[0].questions), 1)
        self.assertEqual(len(survey.itemQAs[0].answers), 3)

        question_id = survey.itemQAs[0].questions[0].id
        answer1_id = survey.itemQAs[0].answers[0].id
        answer2_id = survey.itemQAs[0].answers[1].id

        # text
        state = [{ 'item_id': survey.itemQAs[0].id,
          'responses': [
             {'q': question_id, 'a': answer1_id, 'v': u'question 1, answer 1'},
             {'q': question_id, 'a': answer2_id, 'v': u'question 1, answer 2'}
             ]
        }]

        # multiple choice
        mqid = survey.itemQAs[1].questions[0].id
        maid1 = survey.itemQAs[1].answers[0].id
        maid2 = survey.itemQAs[1].answers[1].id
        state.append({ 'item_id': survey.itemQAs[1].id,
            'responses': [
                {'q': mqid, 'a': maid1, 'v': True}
            ]
        })
        # single choice
        sqid = survey.itemQAs[2].questions[0].id
        said1 = survey.itemQAs[2].answers[0].id
        said2 = survey.itemQAs[2].answers[1].id
        state.append({ 'item_id': survey.itemQAs[2].id,
            'responses': [
                {'q': sqid, 'a': said2, 'v': True}
            ]
        })

        # single matrix
        msqid1 = survey.itemQAs[3].questions[0].id
        msqid2 = survey.itemQAs[3].questions[1].id
        msaid1 = survey.itemQAs[3].answers[0].id
        msaid2 = survey.itemQAs[3].answers[1].id
        state.append({ 'item_id': survey.itemQAs[3].id,
            'responses': [
                {'q': msqid1, 'a': msaid1, 'v': True},
                {'q': msqid2, 'a': msaid2, 'v': True}
            ]
        })

        controller.update_responses(state)

        # we are checking that we still have one participant now and one respondent
        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==p.id).one()
        self.assertEqual(publication.number_of_participants, 1)
        self.assertEqual(publication.number_of_respondents, 1)

        responses = controller.get_responses()
        self.assertEqual({
            question_id: {answer1_id: u'question 1, answer 1', answer2_id: u'question 1, answer 2'},
            mqid: {maid1: 1L},
            sqid: {said2: 1L},
            msqid1: {msaid1: 1L},
            msqid2: {msaid2: 1L},
        }, responses)

        # now we change the responses and save them again.
        # set different texts for the text item
        state[0]['responses'][0]['v'] = u'question 1, answer 1, update'
        state[0]['responses'][1]['v'] = u'question 1, answer 2, update'

        # set the other checkbox for the multiple choice also.
        state[1]['responses'].append({'q': mqid, 'a': maid2, 'v': True})

        # set the other answer for the single selec
        state[2]['responses'][0]['v'] = None
        state[2]['responses'].append({'q': sqid, 'a': said1, 'v': True})

        # switch the answers for the matrix single
        state[3]['responses'][0]['v'] = None
        state[3]['responses'][1]['v'] = None
        state[3]['responses'].append({'q': msqid1, 'a': msaid2, 'v': True})
        state[3]['responses'].append({'q': msqid2, 'a': msaid1, 'v': True})

        controller.update_responses(state)

        # we are checking that we still have one participant now and still one respondent
        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==p.id).one()
        self.assertEqual(publication.number_of_participants, 1)
        self.assertEqual(publication.number_of_respondents, 1)

        responses = controller.get_responses()
        self.assertEqual({
            question_id: {answer1_id: u'question 1, answer 1, update', answer2_id: u'question 1, answer 2, update'},
            mqid: {maid1: 1L, maid2: 1L},
            sqid: {said1: 1L},
            msqid1: {msaid2: 1L},
            msqid2: {msaid1: 1L},
        }, responses)

        # now we update again and we remove one of the checks from the multiple choice
        state[1]['responses'][1]['v'] = None
        controller.update_responses(state)
        responses = controller.get_responses()
        self.assertEqual({
            question_id: {answer1_id: u'question 1, answer 1, update', answer2_id: u'question 1, answer 2, update'},
            mqid: {maid1: 1L},
            sqid: {said1: 1L},
            msqid1: {msaid2: 1L},
            msqid2: {msaid1: 1L},
        }, responses)


        # now try to instanciate the controller again with a non matching invitee_hash with participant_session_id
        # we expect that the participant_session_id is regenerated because the user cant use the same session for a different invitee
        controller = ParticipantController(self.session, self.locale, invitee_hash2, participant_session_id)
        self.assertNotEquals(participant_session_id, controller.get_participant_session_id())

    def test_save_filter_responses(self):
        # send out a survey that goes online now for a few contact lists
        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id, self.cl2.id])
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        #now add a numeric filter on the text input
        from iqportal.iquest.core.qa_filters.controllers import SurveyItemQAFilterController
        from iqportal.iquest.core.models import IQuestQAFilter
        from iqportal.iquest.response.models import SurveyQAResponse

        ifc = SurveyItemQAFilterController(self.session, None, self.account.id, StateManager(None, None, None))
        filter = self.session.query(IQuestQAFilter).filter(IQuestQAFilter.name==u'Numeric').one()
        ifc.save_or_update_item_filter(None, filter, survey.itemQAs[0], attributes={})

        # now lets pretend the contacts got an email with his hash which they needs to access the survey
        invitees = self.session.query(
            ComposeInvitee).join(
            iquest_models.SurveyPublication).join(
            iquest_models.Survey).filter(
                iquest_models.Survey.id==survey_id).all()

        invitee_hash = unicode(invitees[0].invitee_hash)
        invitee_hash2 = unicode(invitees[1].invitee_hash)

        # instanciate a response controller for the first participan
        # and we expect there is a session created
        controller = ParticipantController(self.session, self.locale, invitee_hash)

        question_id = survey.itemQAs[0].questions[0].id
        answer1_id = survey.itemQAs[0].answers[0].id
        answer2_id = survey.itemQAs[0].answers[1].id

        # tex
        state = [{ 'item_id': survey.itemQAs[0].id,
          'responses': [
             {'q': question_id, 'a': answer1_id, 'v': u'1'},
             {'q': question_id, 'a': answer2_id, 'v': u'1.2'}
             ]
        }]

        # multiple choice
        mqid = survey.itemQAs[1].questions[0].id
        maid1 = survey.itemQAs[1].answers[0].id
        maid2 = survey.itemQAs[1].answers[1].id
        state.append({ 'item_id': survey.itemQAs[1].id,
            'responses': [
                {'q': mqid, 'a': maid1, 'v': True}
            ]
        })
        # single choice
        sqid = survey.itemQAs[2].questions[0].id
        said1 = survey.itemQAs[2].answers[0].id
        said2 = survey.itemQAs[2].answers[1].id
        state.append({ 'item_id': survey.itemQAs[2].id,
            'responses': [
                {'q': sqid, 'a': said2, 'v': True}
            ]
        })

        # single matrix
        msqid1 = survey.itemQAs[3].questions[0].id
        msqid2 = survey.itemQAs[3].questions[1].id
        msaid1 = survey.itemQAs[3].answers[0].id
        msaid2 = survey.itemQAs[3].answers[1].id
        state.append({ 'item_id': survey.itemQAs[3].id,
            'responses': [
                {'q': msqid1, 'a': msaid1, 'v': True},
                {'q': msqid2, 'a': msaid2, 'v': True}
            ]
        })
        controller.update_responses(state)

        from decimal import Decimal
        responses = controller.get_responses()
        self.assertEqual({
            question_id: {answer1_id: 1L, answer2_id: '1.2'}, # one response_int and one response_float, FIXME decimal
            mqid: {maid1: 1L},
            sqid: {said2: 1L},
            msqid1: {msaid1: 1L},
            msqid2: {msaid2: 1L},
        }, responses)

        # make sure nothing is saved in the text column
        resp = self.session.query(SurveyQAResponse).filter(SurveyQAResponse.participant==controller._participant).all()
        self.assertEqual(len(resp), 6)
        for r in resp:
            self.assertEqual(r.response_text, None)

        # and now update
        state[0]['responses'][0]['v'] = u'5'
        state[0]['responses'][1]['v'] = u'5.5'
        controller.update_responses(state)

        from decimal import Decimal
        responses = controller.get_responses()
        self.assertEqual({
            question_id: {answer1_id: 5L, answer2_id: '5.5'}, # one response_int and one response_float, FIXME decimal
            mqid: {maid1: 1L},
            sqid: {said2: 1L},
            msqid1: {msaid1: 1L},
            msqid2: {msaid2: 1L},
        }, responses)

        # make sure nothing is saved in the text column
        resp = self.session.query(SurveyQAResponse).filter(SurveyQAResponse.participant==controller._participant).all()
        self.assertEqual(len(resp), 6)
        for r in resp:
            self.assertEqual(r.response_text, None)

    def test_simple_anonymous(self):
        """
        Publish a anonymous survey and login with the publication_id
        """
        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True)

        # get some vars
        survey = self.session.query(iquest_models.Survey).get(survey_id)
        question_id = survey.itemQAs[0].questions[0].id
        answer1_id = survey.itemQAs[0].answers[0].id
        answer2_id = survey.itemQAs[0].answers[1].id

        p = survey.publications[0]

        # now pretend that the someone received a invitee_hash which is needed to access an anonymous survey
        invitee_hash = unicode(self.session.query(iquest_models.Survey).get(survey_id).publications[0].invitees[0].invitee_hash)

        # init the controller
        controller = ParticipantController(self.session, self.locale, invitee_hash)

        # we are checking that we have one participant now
        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==p.id).one()
        self.assertEqual(publication.number_of_participants, 1)
        self.assertEqual(publication.number_of_respondents, 0)


        # instanciate survey by invitee
        state = controller.load_survey_state()
        self.assertTrue(isinstance(state['survey']['dbID'], long))

        # fill in responses
        controller.update_responses([{'item_id': survey.itemQAs[0].id, 'responses': [
            {'q': question_id, 'a': answer1_id, 'v': u'question 1, answer 1'},
            {'q': question_id, 'a': answer2_id, 'v': u'question 1, answer 2'}
        ]}])

        # now lets say the ANOTHER USER clicks on the same survey link.. we expect NOT to have the old responses loaded
        controller = ParticipantController(self.session, self.locale, invitee_hash)

        # we now should have 2 participants and 1 respondent
        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==p.id).one()
        self.assertEqual(publication.number_of_participants, 2)
        self.assertEqual(publication.number_of_respondents, 1)

        # assert that responses are empty
        responses = controller.get_responses()
        self.assertEqual({}, responses)


    def test_exception_invalid_invitee(self):
        """
        Tests that an exception is raised when trying to access a survey with a non-existing participant id
        """
        # he shouldnt be able to init the controller.. assert with the exception trown
        self.assertRaises(basesoft_exceptions.PublicUserException, ParticipantController, self.session, self.locale, invitee_hash=u'bad hash')

    def test_exception_expired_invitee(self):
        now = datetime.now()
        last_week = now+timedelta(days=-7)

        lc = LicenseConfig()
        lc.config = self.sa_session.query(Config).filter(Config.name=='basesoft.web.response.invitee_expiry_days').one()
        lc.license = self.group.license
        lc.value = 4
        self.sa_session.add(lc)

        # schedule a survey that is online from yesterday.. this will also create the invitees
        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, last_week, anonymous=True)
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        # now set the invitee_hash back in time
        invitee = survey.publications[0].invitees[0]
        invitee.created = last_week
        self.sa_session.add(invitee)
        self.sa_session.flush()

        self.assertRaises(basesoft_exceptions.PublicSystemException, ParticipantController, self.session, self.locale, invitee_hash=invitee.invitee_hash)

    def test_exception_survey_not_online(self):

        now = datetime.now()
        yesterday = now+timedelta(days=-1)

        # schedule a survey that is online from yesterday.. this will also create the invitees
        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, yesterday, anonymous=True)
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        # now pretend that the someone found a invitee_hash which is needed to fill in a anonymous survey
        invitee_hash = unicode(survey.publications[0].invitees[0].invitee_hash)

        # now end the survey by setting the end date to now and calling the scheduler
        survey.publications[0].end_date = now
        self.session.add(survey)
        self.session.flush()

        # publish the surveys like the scheduler would do
        schedule.FinishSurveysTask().execute()
        self.session.expire(survey.publications[0])

        # he shouldnt be able to init the controller.. assert with the exception trown
        self.assertRaises(basesoft_exceptions.PublicUserException, ParticipantController, self.session, self.locale, invitee_hash=invitee_hash)

    def _set_publication_responses(self, all_types=False):
        """
        Protected method to be used in the publication responses manager tests.
        Creates a publication, invitees, participants and adds responses.
        """
        # send out a survey that goes online now for a few contact lists
        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id, self.cl2.id])
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        # now lets pretend the contacts got an email with his hash which they needs to access the survey
        invitees = self.session.query(
            ComposeInvitee).join(
            iquest_models.SurveyPublication).join(
            iquest_models.Survey).filter(
                iquest_models.Survey.id==survey_id).all()
        invitee_hash = unicode(invitees[0].invitee_hash)

        controller = ParticipantController(self.session, self.locale, invitee_hash)
        # Fill responses
        # the user entered some responses which he likes to submit
        question_id = survey.itemQAs[0].questions[0].id
        answer1_id = survey.itemQAs[0].answers[0].id
        answer2_id = survey.itemQAs[0].answers[1].id

        state = [{ 'item_id': survey.itemQAs[0].id,
          'responses': [
             {'q': question_id, 'a': answer1_id, 'v': u'question 1, answer 1'},
             {'q': question_id, 'a': answer2_id, 'v': u'question 1, answer 2'}
             ]
        }]

        if all_types:
            # multiple choice
            qid = survey.itemQAs[1].questions[0].id
            aid1 = survey.itemQAs[1].answers[0].id
            aid2 = survey.itemQAs[1].answers[1].id
            state.append({ 'item_id': survey.itemQAs[1].id,
                'responses': [
                    {'q': qid, 'a': aid1, 'v': True},
                    {'q': qid, 'a': aid2, 'v': False}
                ]
            })
            # single choice
            qid = survey.itemQAs[2].questions[0].id
            aid1 = survey.itemQAs[2].answers[0].id
            aid2 = survey.itemQAs[2].answers[1].id
            state.append({ 'item_id': survey.itemQAs[2].id,
                'responses': [
                    {'q': qid, 'a': aid1, 'v': False},
                    {'q': qid, 'a': aid2, 'v': True}
                ]
            })

        # use the already initialised controller to submit the responses
        controller.update_responses(state)

        participant = self.session.query(response_models.SurveyParticipant).filter(
                              response_models.SurveyParticipant.invitee_id==invitees[0].id).one()
        return {'survey': survey,
                'respondent': participant
                }

    def test_get_publication_respondents(self):
        """
        Tests getting a publication's respondents.
        """
        pr = self._set_publication_responses()

        controller = PublicationResponseController(self.session, self.account.id)

        # we should get an exception cuz the user doesn't have the correct perms
        self.assertRaises(PermissionDeniedException,
                          controller.get_publication_invitees_and_participants,
                          pr["survey"].publications[0].id)
        # now with permissions
        self._attach_permissions_to_group(self.group, 'view_survey_publication_response')
        part, invitees = controller.get_publication_invitees_and_participants(pr["survey"].publications[0].id)
        self.assertEquals(part[0].invitee.invitee_hash, pr["respondent"].invitee.invitee_hash)

    def test_update_respondent_responses(self):
        """
        Tests updating a respondent's responses.
        """
        pr = self._set_publication_responses()
        survey = pr["survey"]
        self.assertEquals(pr["respondent"].is_respondent, response_models.SurveyParticipant.RESPONSES_INTERNET)

        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==survey.publications[0].id).one()
        self.assertEqual(publication.number_of_participants, 1)
        self.assertEqual(publication.number_of_respondents, 1)

        controller = PublicationResponseController(self.session, self.account.id)

        # Fill responses
        # the user entered some responses which he likes to submit
        question_id = survey.itemQAs[0].questions[0].id
        answer1_id = survey.itemQAs[0].answers[0].id
        answer2_id = survey.itemQAs[0].answers[1].id

        new_responses = [
                { 'item_id': survey.itemQAs[0].id,
                  'responses': [
                     {'q': question_id, 'a': answer1_id, 'v': u'question 1, answer 1 update'},
                     {'q': question_id, 'a': answer2_id, 'v': u'question 1, answer 2 update'}
                     ]
                }
        ]
        # we should get an exception cuz the user doesn't have the correct perms
        self.assertRaises(PermissionDeniedException,
                          controller.update_responses,
                          pr["respondent"].invitee.id, new_responses, pr["respondent"].id)
        # now with permissions
        self._attach_permissions_to_group(self.group, 'update_survey_publication_response')
        controller.update_responses(pr["respondent"].invitee.id, new_responses, participant_id=pr["respondent"].id)

        responses = self.session.query(response_models.SurveyQAResponse).filter(
                        response_models.SurveyQAResponse.participant_id==pr["respondent"].id).all()
        self.assertEquals(len(responses), 2)
        self.assertEquals(responses[0].participant.is_respondent, response_models.SurveyParticipant.RESPONSES_BACK_OFFICE)
        self.assertEquals(responses[0].response_text, u'question 1, answer 1 update')
        self.assertEquals(responses[1].response_text, u'question 1, answer 2 update')

        # verify we still have the same number of participants and respondents
        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==survey.publications[0].id).one()
        self.assertEqual(publication.number_of_participants, 1)
        self.assertEqual(publication.number_of_respondents, 1)

        # now lets create responses
        invitees = self.session.query(
            ComposeInvitee).join(
            iquest_models.SurveyPublication).filter(
                iquest_models.SurveyPublication.survey_id==pr["respondent"].invitee.publication.survey.id).all()

        invitee = None
        for i in invitees:
            if i.id != pr["respondent"].invitee.id:
                invitee = i
                break
        # there shouldn't be any participant for this invitee
        p = self.session.query(response_models.SurveyParticipant).filter(
                response_models.SurveyParticipant.invitee_id==invitee.id).all()
        self.assertEquals(len(p), 0)

        # don't have correct permission for create responses yet
        self.assertRaises(PermissionDeniedException,
                          controller.update_responses,
                          invitee.id, new_responses)
        self._attach_permissions_to_group(self.group, 'create_survey_publication_response')
        controller.update_responses(invitee.id, new_responses)

        responses = self.session.query(response_models.SurveyQAResponse).filter(
                        response_models.SurveyQAResponse.participant_id==controller.get_participant().id).all()
        self.assertEquals(len(responses), 2)
        self.assertEquals(responses[0].participant.is_respondent, response_models.SurveyParticipant.RESPONSES_BACK_OFFICE)
        self.assertEquals(responses[0].response_text, u'question 1, answer 1 update')
        self.assertEquals(responses[1].response_text, u'question 1, answer 2 update')


    def test_delete_respondent_responses(self):
        """
        Tests deleting a respondent's responses and saving them in the responses history table.
        """
        pr = self._set_publication_responses()
        survey = pr['survey']

        controller = PublicationResponseController(self.session, self.account.id)

        # verify we have 1 participant and 1 respondent
        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==survey.publications[0].id).one()
        self.assertEqual(publication.number_of_participants, 1)
        self.assertEqual(publication.number_of_respondents, 1)

        # we should get an exception cuz the user doesn't have the correct perms
        self.assertRaises(PermissionDeniedException,
                          controller.delete_responses,
                          pr["respondent"].id)
        # now with permissions
        self._attach_permissions_to_group(self.group, 'delete_survey_publication_response')
        controller.delete_responses(pr["respondent"].id)

        # verify we have 1 participant and 0 respondents
        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.publication_id==survey.publications[0].id).one()
        self.assertEqual(publication.number_of_participants, 1)
        self.assertEqual(publication.number_of_respondents, 0)

        # the respondent should still be in the surveyParticipant table, except he has 0 responses
        p = self.session.query(response_models.SurveyParticipant).filter(
                        response_models.SurveyParticipant.id==pr["respondent"].id).all()
        self.assertEquals(len(p), 1)
        self.assertEquals(p[0].is_respondent, response_models.SurveyParticipant.NO_RESPONSES)

        # no more responses
        responses = self.session.query(response_models.SurveyQAResponse).filter(
                        response_models.SurveyQAResponse.participant_id==pr["respondent"].id).all()
        self.assertEquals(len(responses), 0)

        # the deleted answers should be in the response history table
        responses = self.session.query(response_models.SurveyQAResponseHistory).filter(
                        response_models.SurveyQAResponseHistory.participant_id==pr["respondent"].id).all()
        self.assertEquals(len(responses), 2)

    def test_locales(self):
        """
        tests loading of the state for different locales
        """

        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl3.id])
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        # now lets pretend the contacts got an email with his hash which they needs to access the survey
        invitees = self.session.query(
            ComposeInvitee).join(
            iquest_models.SurveyPublication).join(
            iquest_models.Survey).filter(
                iquest_models.Survey.id==survey_id).all()

        # assert the invitees have the expected locales set
        self.assertEquals(invitees[0].contact.person.preferred_locale.locale, 'nl_NL')
        self.assertEquals(invitees[1].contact.person.preferred_locale.locale, 'en_GB')
        self.assertEquals(invitees[2].contact.person, None)
        self.assertEquals(invitees[3].contact.person.preferred_locale.locale, 'de_DE')

        # assert invitee 0 (dutch)
        pc_i0 = ParticipantController(self.session, None, invitees[0].invitee_hash)
        state_i0 = pc_i0.load_survey_state()
        locales_i0_qa0 = state_i0['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]['locales']
        self.assertEquals(len(locales_i0_qa0), 1)
        self.assertEquals(locales_i0_qa0['nl_NL']['label'], 'Text question Dutch')
        self.assertEqual(pc_i0.get_participant().locale.locale, 'nl_NL')

        # test fallback locale. in this case nL_NL exists but is empty
        locales_i0_qa0_a0 = state_i0['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]['children']['answers'][0]['locales']
        self.assertEquals(locales_i0_qa0_a0['nl_NL']['label'], 'Text question, first answer English')

        # assert invitee 1 (english)
        pc_i1 = ParticipantController(self.session, None, invitees[1].invitee_hash)
        state_i1 = pc_i1.load_survey_state()
        locales_i1_qa0 = state_i1['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]['locales']
        self.assertEquals(len(locales_i1_qa0), 1)
        self.assertEquals(locales_i1_qa0['en_GB']['label'], 'Text question English')
        self.assertEqual(pc_i1.get_participant().locale.locale, 'en_GB')

        # assert invitee 2 (no language.. should fallback to english)
        # this also tests locale caching (2nd time english is loaded)
        pc_i2 = ParticipantController(self.session, None, invitees[2].invitee_hash)
        state_i2 = pc_i2.load_survey_state()
        locales_i2_qa0 = state_i2['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]['locales']
        self.assertEquals(len(locales_i2_qa0), 1)
        self.assertEquals(locales_i2_qa0['en_GB']['label'], 'Text question English')
        self.assertEqual(pc_i2.get_participant().locale.locale, 'en_GB')

        # assert invitee 3 (german language not in available locales.. should fallback to english)
        pc_i3 = ParticipantController(self.session, None, invitees[3].invitee_hash)
        state_i3 = pc_i3.load_survey_state()
        locales_i3_qa0 = state_i3['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]['locales']
        self.assertEquals(len(locales_i3_qa0), 1)
        self.assertEquals(locales_i3_qa0['en_GB']['label'], 'Text question English')
        self.assertEqual(pc_i3.get_participant().locale.locale, 'en_GB')

        # assert invitee 4 (french in available locales but no texts defined)
        pc_i4 = ParticipantController(self.session, None, invitees[4].invitee_hash)
        state_i4 = pc_i4.load_survey_state()
        locales_i4_qa0 = state_i4['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]['locales']
        self.assertEquals(len(locales_i4_qa0), 1)
        self.assertEquals(locales_i4_qa0['fr_FR']['label'], 'Text question English')
        self.assertEqual(pc_i4.get_participant().locale.locale, 'fr_FR')

        # assert that 3 locale states have been cached
        compose_locales = self.sa_session.query(ComposePublicationLocale) \
            .join(SurveyPublication) \
            .join(Survey) \
            .filter(Survey.id == survey_id).order_by(ComposePublicationLocale.locale_id).all()
        self.assertEqual(len(compose_locales), 3)
        self.assertEqual(compose_locales[0].locale.locale, 'en_GB')
        self.assertEqual(compose_locales[1].locale.locale, 'nl_NL')
        self.assertEqual(compose_locales[2].locale.locale, 'fr_FR')



# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_compose.py', 'iqportal.iquest.test.unit.test_response.ResponseTest.test_simple_response']
    run()
    sys.exit()

