# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime

from iqportal.iquest.test.unit import iquest_unittest

from iqportal.iquest.core.models import SurveyPublication, SurveyItemQA, IQuestQAFilter
from iqportal.iquest.core.qa_filters.controllers import SurveyItemQAFilterController
from iqportal.iquest.response.controllers import ResponseFilterController
from decimal import Decimal
from basesoft.web.core.util.controllers import StateManager

class TestCase(iquest_unittest.IQuestTestCase):

    def setUp(self):
        """
        Publish a survey
        """
        iquest_unittest.IQuestTestCase.setUp(self, ['create_survey', 'view_survey', 'create_survey_publication',
                                                   'schedule_survey_publication'])
        self.publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True)
        self.ifc = SurveyItemQAFilterController(self.session, None, self.account.id, StateManager(None, None, None))

    def test_numerical_response_filter(self):
        """
        Test simple numerical filters with responses.
        """
        item_qa = None
        for qa in self._publication.survey.itemQAs:
            if qa.type == SurveyItemQA.TYPE_SINGLE_TEXT_INPUT:
                item_qa = qa
                break

        if item_qa is None:
            self.fail('No Text input question in publication')

        # attach a simple filter.
        filter = self.session.query(IQuestQAFilter).filter(IQuestQAFilter.name==u'Numeric').one()

        self.ifc.save_or_update_item_filter(None, filter, item_qa, attributes={'minValue': 0, 'maxValue': 100, 'domain': 'int'})

        responses = [{
            'q': item_qa.questions[0].id,
            'a': item_qa.answers[0].id,
            'v': '20'
        },{
            'q': item_qa.questions[0].id,
            'a': item_qa.answers[1].id,
            'v': '140'
        }]

        rfc = ResponseFilterController(self.session, [{'item_id': item_qa.id, 'responses': responses}])
        self.assertFalse(rfc.is_valid())

        responses[0]['v'] = '-10'
        rfc = ResponseFilterController(self.session, [{'item_id': item_qa.id, 'responses': responses}])
        self.assertFalse(rfc.is_valid())

        responses[0]['v'] = '20'
        responses[1]['v'] = '40'
        rfc = ResponseFilterController(self.session, [{'item_id': item_qa.id, 'responses': responses}])
        self.assertTrue(rfc.is_valid())

        responses[0]['v'] = '2.4'
        rfc = ResponseFilterController(self.session, [{'item_id': item_qa.id, 'responses': responses}])
        rfc.is_valid()
        self.assertEqual(rfc._result[0]['messages']['failures'], [u'All values need to be whole numbers'])

    def test_numerical_float_filter(self):
        """
        Make sure that the Responses are cast to floats even if the input is an int.
        Related to ticket #1012
        """
        item_qa = None
        for qa in self._publication.survey.itemQAs:
            if qa.type == SurveyItemQA.TYPE_SINGLE_TEXT_INPUT:
                item_qa = qa
                break

        if item_qa is None:
            self.fail('No Text input question in publication')

        # attach a simple filter.
        filter = self.session.query(IQuestQAFilter).filter(IQuestQAFilter.name==u'Numeric').one()

        self.ifc.save_or_update_item_filter(None, filter, item_qa, attributes={'domain': 'float'})

        responses = [{
            'q': item_qa.questions[0].id,
            'a': item_qa.answers[0].id,
            'v': 20
        },{
            'q': item_qa.questions[0].id,
            'a': item_qa.answers[1].id,
            'v': 1.40
        }]

        rfc = ResponseFilterController(self.session, [{'item_id': item_qa.id, 'responses': responses}])
        self.assertTrue(rfc.is_valid())
        self.assertTrue(type(responses[0]['v']) == Decimal)
        self.assertTrue(type(responses[1]['v']) == Decimal)

        responses[0]['v'] = 'Blah blah blah'
        self.assertFalse(rfc.is_valid())

