# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime

from iqportal.iquest.test.unit import iquest_unittest
from iqportal.iquest.core.models import SurveyPublication, SurveyItemQA
from iqportal.iquest.response.controllers import ResponseRequiredController

class ResponseFilterTestCase(iquest_unittest.IQuestTestCase):

    def setUp(self):
        """
        Publish a survey
        """
        iquest_unittest.IQuestTestCase.setUp(self, ['create_survey', 'view_survey', 'create_survey_publication',
                                                   'schedule_survey_publication'])
        self.publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True)

    def test_response_required(self):
        """
        Test if the required items are checked properly
        """
        for qa in self._publication.survey.itemQAs:
            if qa.type == SurveyItemQA.TYPE_SINGLE_TEXT_INPUT:
                item_qa_sti = qa
                break

        for qa in self._publication.survey.itemQAs:
            if qa.type == SurveyItemQA.TYPE_MULTIPLE_CHOICE:
                item_qa_mc = qa
                break

        if item_qa_sti is None:
            self.fail('No Text input question in publication')

        if item_qa_mc is None:
            self.fail('No Multiple Choice input question in publication')

        # let's make the items required
        item_qa_sti.required = True
        item_qa_mc.required = True
        self.session.add(item_qa_sti)
        self.session.add(item_qa_mc)
        self.session.flush()

        # first lets test the single text input
        responses_sti = [{
            'q': item_qa_sti.questions[0].id,
            'a': item_qa_sti.answers[0].id,
            'v': None
        },{
            'q': item_qa_sti.questions[0].id,
            'a': item_qa_sti.answers[1].id,
            'v': None
        }]

        rrc = ResponseRequiredController(self.session, [{'item_id': item_qa_sti.id, 'responses': responses_sti}])
        self.assertFalse(rrc.is_valid()) # nothing filled in

        responses_sti[0]['v'] = 'some text'
        rrc = ResponseRequiredController(self.session, [{'item_id': item_qa_sti.id, 'responses': responses_sti}])
        self.assertFalse(rrc.is_valid()) # still not valid until we fill all answers

        responses_sti[1]['v'] = 'another text'
        rrc = ResponseRequiredController(self.session, [{'item_id': item_qa_sti.id, 'responses': responses_sti}])
        self.assertTrue(rrc.is_valid()) # now it should be valid

        # now we'll test the multiple choice item
        response_mc = [{
            'q': item_qa_mc.questions[0].id,
            'a': item_qa_mc.answers[0].id,
            'v': None
        },{
            'q': item_qa_mc.questions[0].id,
            'a': item_qa_mc.answers[1].id,
            'v': None
        }]

        rrc = ResponseRequiredController(self.session, [{'item_id': item_qa_mc.id, 'responses': []}])
        self.assertFalse(rrc.is_valid()) # no input checked

        response_mc[0]['v'] = True
        rrc = ResponseRequiredController(self.session, [{'item_id': item_qa_mc.id, 'responses': response_mc}])
        self.assertTrue(rrc.is_valid()) # valid, because we have at least one answer

        response_mc[1]['v'] = True
        rrc = ResponseRequiredController(self.session, [{'item_id': item_qa_mc.id, 'responses': response_mc}])
        self.assertTrue(rrc.is_valid()) # valid with all answers too, of course
