# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from iqportal.iquest.test.unit.iquest_unittest import IQuestTestCase
from iqportal.iquest.core.models import SurveyPublication

from iqportal.iquest.compose.controllers import SurveyVariableStateController
from basesoft.web.core.util.controllers import StateManager

class SurveyVariablesTestCase(IQuestTestCase):

    def setUp(self):
        IQuestTestCase.setUp(self, ['create_survey', 'view_survey', 'create_survey_publication', 'create_survey_variable_set',
                                    'update_survey_variable_set', 'view_survey_variable_set'])

        self.variable_controller = SurveyVariableStateController(self.session, StateManager(None, self.account.id, None))

    def test_save_and_load(self):

        # create a survey.. this contains some survey-variables already
        survey_id = self.publish_survey(SurveyPublication.STATUS_NEW)

        # create a variable-set for the product 'apples' and another one for 'pears'
        set_apples_state = {
            'dbID': None,
            'domID': None,
            'name': u'Apples',
            'survey_id': survey_id,
        }
        set_apples_id = self.variable_controller.save(set_apples_state)

        # create another set for english
        self.variable_controller.save({
            'dbID': None,
            'domID': None,
            'name': u'Pears',
            'survey_id': survey_id,
        })

        # ask the set for NL and EN (EN is missing one var)
        vars_nl =  self.variable_controller.list_variables(survey_id, 'nl_NL')
        self.assertEquals(vars_nl, ['product_name', 'other_product_name'])

        vars_en =  self.variable_controller.list_variables(survey_id, 'en_GB')
        self.assertEquals(vars_en, ['product_name'])

        # fill the set with variables and update set_apples
        self.variable_controller.update_variables(set_apples_id, u'en_GB', {
            'product_name': u'Apples'
        })

        self.variable_controller.update_variables(set_apples_id, u'nl_NL', {
            'product_name': u'Appels',
            'other_product_name': u'Aardbeien',
        })

        # load the available lists for the survey..
        sets = self.variable_controller.get_variable_sets(survey_id=survey_id)
        self.assertEquals(sets[0].name, u'Apples')
        self.assertEquals(sets[1].name, u'Pears')

        # load the set_apples list variables
        vars = self.variable_controller.load_variables(set_apples_id, u'nl_NL')
        self.assertEquals(vars, {
            'other_product_name':u'Aardbeien',
            'product_name':u'Appels'
        })


# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_compose.py', 'iqportal.iquest.test.unit.test_variables']
    run()
    sys.exit()
