# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.license.models import Group, License, Domain, Permission
from basesoft.web.core.util.controllers import AbstractController
from basesoft.web.contrib.contact.models import Person
from basesoft.web.contrib.account.models import Account, GroupAccount

class SAPTestController(AbstractController):

    def get_sap_account(self):
        # SAP Management license
        sap_license = self.sa_session.query(License).filter(License.name == 'SAP Management').all()
        if len(sap_license) == 0:
            from basesoft.web.contrib.license.controllers import LicenseController

            permissions = self.sa_session.query(Permission).filter(Permission.name.in_([
               "view_log", "view_login_overview", "kill_session", "file_manager", "view_file",
               "create_file", "update_file", "delete_file", "view_contact", "create_contact", "update_contact", "delete_contact",
               "import_contacts", "view_contact_list", "create_contact_list", "update_contact_list", "delete_contact_list",
               "link_to_contact_list", "unlink_from_contact_list", "distribute_group_permissions", "distribute_license_permissions",
               "view_domain", "view_license", "create_license", "update_license", "create_mail", "view_mail", "update_mail",
               "delete_mail", "schedule_mail", "view_mailing_results", "view_dashboard", "save_dashboard",
               "create_account", "update_account", "delete_account", "account_config_manager", "create_account_mail",
               "view_account_mail", "update_account_mail", "delete_account_mail", "create_notification", "view_notification",
               "update_notification", "delete_notification", "send_to_license_default_group", "send_to_domain_default_group",
               "branding_view_header_footer", "branding_create_header_footer", "branding_update_header_footer", "branding_delete_header_footer",
               "create_calculated_member", "view_calculated_member"
               "create_report", "view_report", "update_datasource", "view_datasource", "datasource_manager",

               "create_sap_project", "view_sap_project", "update_sap_project", "delete_sap_project", "sap_project_manager",
               "create_sap_component_element", "view_sap_component_element", "update_sap_component_element", "delete_sap_component_element"
               ])).all()

            lc = LicenseController(self.sa_session, account_id=self.get_account().id)
            domain = self.sa_session.query(Domain).filter(Domain.name == 'SAP').all()
            if len(domain) == 0:
                domain = lc.save_or_update_domain(None, 'SAP')
            else:
                domain = domain[0]
            contact = Person()
            contact.first_name = 'Cees'
            sap_license = lc.save_or_update_license(None, 'SAP Management', domain, License, contact, None, None, True)
            sap_license.permissions = permissions
            sap_group = lc.save_or_update_group(None, 'SAP Admins', sap_license, is_default=True)
            sap_group.permissions = permissions

            # run all the db_sync cuz they check for sap domain
            from iqportal.sap.core import db_sync as core_sync
            core_sync.sync(self.sa_session)
        else:
            sap_group = sap_license[0].default_group
            sap_license = sap_license[0]

        # make a sap account
        sap_account = self.sa_session.query(Account).filter(Account.name == 'sap_account_test').all()
        if len(sap_account) == 0:
            from basesoft.web.contrib.account.controllers import AccountController
            sap_account = Account()
            sap_account.name = 'sap_account_test'
            sap_account.password = AccountController(self.sa_session).hash_password('sap_password')
            sap_account.person = Person()
            self.sa_session.add(sap_account)

            # make the link between account and group.
            ga = GroupAccount()
            ga.is_default = 1
            ga.active = True
            ga.account = sap_account
            ga.group = sap_group

            self.sa_session.add(ga)
            self.sa_session.flush()
        else:
            sap_account = sap_account[0]

        return sap_license, sap_account

    def populate(self):
        """
        Adds data for sap reports.
        """
        sap_license, sap_account = self.get_sap_account()
