# -*- coding: utf-8 -*-
# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.license.models import Group
from basesoft.web.core.util.controllers import AbstractController, transaction_decorator
from basesoft.web.contrib.contact.controllers import ContactController

from iqportal.tnt.clm.controllers import WaveController
from iqportal.tnt.response.controllers import ParticipantController, ResponseController
from iqportal.tnt.clm.models import TntCLMWaveType, TntCLMAttribute, TntCLMScaleAnswer, TntCLMWave, \
    TntCLMWaveAttribute, TntCLMAttributeAnswer, TntCLMWaveAttributeCategory, \
    TntCLMLocale, TntCLMWaveAttributeLocale, TntCLMWaveScaleAnswerLocale, TntCLMWaveScaleAnswer, \
    TntCLMWaveAnswer, TntCLMWaveLocale, TntCLMWaveLocaleRevision, TntCLMWaveAttributeCategoryLocale, \
    TntCLMWaveAnswerLocale, TntCLMContact, TntCLMService
from iqportal.tnt.core.models import TntDepot, TntGeolevel, TntGeolevelType, TntDepotAlias
from basesoft.web.contrib.i18n.models import Country, DomainCountry, Locale

from iqportal.tnt.report.controllers import TntCLMTntCLMWaveAggregator

from basesoft.web.contrib.license.models import License, Permission
from basesoft.web.contrib.contact.models import Person, Contact
from basesoft.web.contrib.account.models import Account, GroupAccount
from iqportal.tnt.response.models import TntCLMWaveInvitee,\
    TntCLMWaveParticipant, TntCLMWaveResponse, TntCLMWaveParticipantSession
from sqlalchemy.engine import create_engine
from sqlalchemy import and_
from sqlalchemy.orm.exc import NoResultFound

from basesoft.logging import logger

class TntWaveController(AbstractController):
    def __init__(self, session, locale=None, account_id=None):
        self.service_map = {}
        AbstractController.__init__(self, session, locale, account_id)

    def _create_wave(self, type, scale_answers_def, attributes_def, opt_out_text='', priorities=None, wave_name='test wave'):
        """
        """
        locale = self.sa_session.query(TntCLMLocale).filter(TntCLMLocale.label == 'XXENI').one()

        if priorities is None:
            priorities = {}

        wt = TntCLMWaveType()
        wt.module = type
        wt.label = type

        self.save_default_privilege_object_group(wt)
        self.sa_session.add(wt)

        wave = TntCLMWave()
        wave.type = wt
        wave.name = wave_name
        wave.code = 'W.%s' % wt.id

        lr = TntCLMWaveLocaleRevision()
        lr.wave = wave
        lr.account = self.get_account()
        lr.locale = locale

        wl = TntCLMWaveLocale()
        wl.label = '%s - wave label' % (locale.label)
        wl.locale_revision = lr
        wl.opt_out_text = opt_out_text
        wl.is_default = True
        wl.header_text = '%s - wave header' % locale.label
        wl.sub_header_text = '%s - wave sub header' % locale.label
        self.sa_session.add(wl)

        self.save_default_privilege_object_group(wave)
        self.sa_session.add(wave)

        position = 0
        for i in scale_answers_def:
            position = position + 1
            if isinstance(i, TntCLMScaleAnswer):
                sa = i
            else:
                sa = TntCLMScaleAnswer()
                sa.type = wt
                sa.default_label = i
                sa.position = position
                self.sa_session.add(sa)
            wsa = TntCLMWaveScaleAnswer()
            wsa.scale_answer = sa
            wsa.position = position
            wsa.wave = wave
            self.sa_session.add(wsa)
            wsal = TntCLMWaveScaleAnswerLocale()
            wsal.locale_revision = lr
            wsal.label = sa.default_label
            wsal.wave_scale_answer = wsa
            self.sa_session.add(wsal)

        position = 1
        categories = {}
        for at in attributes_def:
            category, i, answers, t = at
            priority = None

            if category is not None:
                if not categories.has_key(category):
                    cat = TntCLMWaveAttributeCategory()
                    cat.wave = wave
                    self.sa_session.add(cat)
                    catl = TntCLMWaveAttributeCategoryLocale()
                    catl.locale_revision = lr
                    catl.label = category
                    catl.category = cat
                    self.sa_session.add(catl)
                    categories[category] = cat
                else:
                    cat = categories[category]
            else:
                cat = None

            if isinstance(i, TntCLMAttribute):
                a = i
            else:
                a = TntCLMAttribute()
                a.type = t
                a.default_label = i
                self.save_default_privilege_object_group(a)

            wa = TntCLMWaveAttribute()
            wa.wave = wave
            wa.attribute = a
            wa.position = position
            wa.category = cat
            if priorities.has_key('attribute_%s' % position):
                priority, is_global = priorities['attribute_%s' % position]
                if is_global:
                    wa.priority_trigger_at = priority
                    priority = None

            wal = TntCLMWaveAttributeLocale()
            wal.locale_revision = lr
            wal.label = a.default_label
            wal.wave_attribute = wa
            if priority is not None:
                wal.priority_trigger_at = priority

            position = position + 1
            self.sa_session.add(a)
            self.sa_session.add(wa)
            self.sa_session.add(wal)

            pos = 0
            for j in answers:
                pos = pos + 1
                if isinstance(j, TntCLMAttributeAnswer):
                    answer = j
                else:
                    answer = TntCLMAttributeAnswer()
                    answer.attribute = a
                    answer.default_label = j
                    answer.position = pos
                    self.sa_session.add(answer)
                wanswer = TntCLMWaveAnswer()
                wanswer.wave_attribute = wa
                wanswer.position = pos
                wanswer.answer = answer
                self.sa_session.add(wanswer)
                wanswerl = TntCLMWaveAnswerLocale()
                wanswerl.locale_revision = lr
                wanswerl.label = answer.default_label
                wanswerl.wave_answer = wanswer
                self.sa_session.add(wanswerl)

        self.sa_session.flush()
        return wave.id

    def create_wave_33(self):
        scale_answers = [
            'Much lower than expected',
            'Lower than expected',
            'As expected',
            'Higher than expected',
            'Much higher than expected',
            'Not applicable',
        ]

        attributes = [
            (None, 'Which of the attributes most influence your choice of an express transportation company (Select up to 5 from page 1 and 2)', [], TntCLMAttribute.TYPE_MULTIPLE_CHOICE,),

            ('Overall service', 'Overall level of performance of TNT Express transportation services', [], TntCLMAttribute.TYPE_SCALE,),
            ('Contacting TNT Express', 'Ease of getting in contact with the required person within TNT Express', [], TntCLMAttribute.TYPE_SCALE,),
            ('Contacting TNT Express', 'Speed of Customer Services answering your telephone calls', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'Suitability of Same Day express services', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'Suitability of Next Day / Time Critical express services', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'Suitability of Day Definite services (longer than 2 days)', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'TNT Express\' capability to handle all your shipments (weights/sizes)', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'TNT Express\' capability of handling your national deliveries', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'TNT Express\' capability of handling your international deliveries', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'Suitability of shipment collection times', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'Suitability of TNT Express\' range of internet services', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'Ease of use of TNT Express\' internet services', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'Availability of customised PC solutions', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'Suitability of TNT Express\' packaging solutions', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' range of services', 'Clarity of TNT Express\' shipping documents', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' pick up and delivery', 'On time delivery of shipments you sent', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' pick up and delivery', 'On time delivery of shipments you receive', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' pick up and delivery', 'Timely notification of delivery problems', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' pick up and delivery', 'Effectiveness in delivering shipments undamaged', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' pick up and delivery', 'Effectiveness of handling multi-piece shipments', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' pick up and delivery', 'Availability of suitable solutions to problems when necessary', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' pick up and delivery', 'Efficiency of the shipment order process through Customer Services', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' pick up and delivery', 'On time pick up of your shipments', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' pick up and delivery', 'Ease of tracking shipments with TNT Express when necessary', [], TntCLMAttribute.TYPE_SCALE,),

            (None, 'Suggestions for service improvement to exceed your expectations', [], TntCLMAttribute.TYPE_TEXT,),
            (None, 'Please validate contact information and amend as appropriate', ['Last name', 'First name', 'Company', 'Industry', 'Street', 'Postal code', 'E-mail', 'Mr/Mrs/Miss', 'Title', 'Department', 'Phone number', 'Building', 'City'], TntCLMAttribute.TYPE_TEXT,),

            ('TNT Express invoicing', 'Accuracy of TNT Express\' invoices', [], TntCLMAttribute.TYPE_SCALE,),

            ('TNT Express\' people and customer relationship', 'Level of personalized service offered by TNT Express', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Ability of TNT Express Customer Services representatives to answer your questions', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Friendliness and attitude of our Customer Services representatives', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Ability of TNT Express Credit Control to resolve any billing related issues', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Friendliness and attitude of our credit control representatives', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Ability of TNT Express sales representatives to understand your business', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Friendliness and attitude of our sales representatives', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Ability of our drivers to offer assistance', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Friendliness and attitude of our drivers', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'TNT Express\' ability to assist with customs documentation', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Adequacy of information provided to you (phone , e-mail etc .)', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'TNT Express\' ability to fulfill commitments', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Speed of complaint resolution', [], TntCLMAttribute.TYPE_SCALE,),
            ('TNT Express\' people and customer relationship', 'Overall image of TNT Express', [], TntCLMAttribute.TYPE_SCALE,),

            ('General', 'Would you recommend TNT Express to others ?', ['not at all', 'maybe', 'yes', 'strongly'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Compared to other providers how would you rate the overall level of service provided by TNT Express', ['much lower', 'lower', 'equal', 'higher', 'much higher'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Compared to other providers how would you rate the TNT Express internet services', ['much lower', 'lower', 'equal', 'higher', 'much higher'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Compared to other providers how would you rate TNT Express\' ability to fulfill commitments', ['much lower', 'lower', 'equal', 'higher', 'much higher'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Compared to other providers how would you rate TNT Express\' effectiveness in complaint resolution', ['much lower', 'lower', 'equal', 'higher', 'much higher'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Compared to other providers how would you rate TNT Express\' ability to provide timely notification of delivery problems', ['much lower', 'lower', 'equal', 'higher', 'much higher'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'What will be your level of purchase with TNT Express over the next year ?', ['none', 'less', 'same', '1-10% more', '> 10% more'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Do you use express distribution companies other than TNT Express ?', ['no', 'yes, less frequently', 'yes, as frequently', 'yes, more frequently'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'If yes, which other company do you use most often ?', ['FedEx', 'UPS', 'DHL', 'other'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'How much of your total spend on Express transportation (all next/fastest day deliveries) is given to TNT Express?', ['< 10%', '10-30%', '30-50%', '50-70%', '> 70%'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'When you use a provider other than TNT Express what is the most important reason for this:', ['lower price(s) but less service', 'lower price(s) and similar service', 'lower price(s) and better service', 'comparable prices but better service', 'none of the above reasons'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'What is your responsibility concerning express shipments in your company?', ['decision maker', 'infleuncer', 'user', 'other'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
        ]

        return self._create_wave('clm_wave_33', scale_answers, attributes)


    def create_wave_36(self, wave_name='test wave'):
        scale_answers = [
            'Much lower than expected',
            'Lower than expected',
            'As expected',
            'Higher than expected',
            'Much higher than expected',
            'Not experienced service',
        ]

        attributes = [
            (None, 'Ease of getting in contact with the required person within TNT Express’ test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test ', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Speed of Customer Service answering your telephone calls', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Suitability of Next Day / Time Critical express services', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Suitability of Day Definite services (longer than 2 days)', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Capability of TNT Express to handle all your shipments, weights & sizes', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Capability of TNT Express handling your national deliveries', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Capability of TNT Express handling your international deliveries', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Suitability of TNT Express range of internet services', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Ease of use of TNT Express internet services', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'On time delivery of shipments you sent', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'On time delivery of shipments you receive', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Timely notification of delivery problems', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Effectiveness in delivering shipments undamaged', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Effectiveness of handling multi-piece shipments', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Availability of suitable solutions to problems when necessary', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Efficiency of the shipment order process through Customer Service', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Level of personalised service offered by TNT Express', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Ability of TNT Express Customer Service representatives to answer your questions', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Friendliness and attitude of our Customer Service representatives', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Friendliness and attitude of our Credit Control representatives', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Ability of TNT Express sales representatives to understand your business', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Friendliness and attitude of our sales representatives', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Ability of our drivers to offer assistance', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Friendliness and attitude of our drivers', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Overall image of TNT Express', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Adequacy of information provided to you (phone , e-mail etc .)', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'TNT Express’ ability to fulfill commitments', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Speed of complaint resolution', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Overall level of performance of TNT Express transportation services', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Country question', [], TntCLMAttribute.TYPE_SCALE, ),
            ('General', 'Would you recommend TNT Express to others ?', ['not at all', 'maybe', 'yes', 'strongly'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Compared to other providers how would you rate TNT Express’ ability to provide timely notification of delivery problems', ['much lower', 'lower', 'equal', 'higher', 'much higher'], TntCLMAttribute.TYPE_SINGLE_SCALE,),
            ('General', 'What will be your level of purchase with TNT Express over the next year ?', ['none', 'less', 'same', '1-10% more', '> 10% more'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Do you use express distribution companies other than TNT Express ?', ['no', 'yes, less frequently', 'yes, as frequently', 'yes, more frequently'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'If yes, which other company do you use most often ? ', ['FedEx', 'UPS', 'DHL', 'other', '', '', '', ''], TntCLMAttribute.TYPE_MULTIPLE_CHOICE,),
            ('General', 'How much of your total spend on Express transportation (all next/ fastest day deliveries) is given to TNT Express ?', ['< 10%', '10-30%', '30-50%', '50-70%', '> 70%'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'What is your responsibility concerning express shipments in your company ?', ['decision maker', 'influencer', 'user', 'other'], TntCLMAttribute.TYPE_MULTIPLE_CHOICE,),
            (None, 'Suggestions for service improvement to exceed your expectations', [], TntCLMAttribute.TYPE_TEXT,),
            (None, 'If the address information was incorrect please provide us with the correct information', ['Last name', 'First name', 'Company', 'Industry', 'Street', 'Postal code', 'E-mail', 'Mr/Mrs/Miss', 'Title', 'Departments', 'Phone number', 'Building', 'City'], TntCLMAttribute.TYPE_TEXT,),
        ]

        priorities = {
            'attribute_1': (2, False,),
            'attribute_2': (1, True,),
            'attribute_15': (2, False,),
            'attribute_16': (2, False,),
            'attribute_33': (2, False,),
            'attribute_34': (2, False,),
        }

        return self._create_wave('clm_wave_36', scale_answers, attributes, opt_out_text='Please tick here if you no longer wish to participate in this survey', priorities=priorities, wave_name=wave_name)

    def create_wave_40a(self, wave_name='test wave', create_in_pool=True):
        if create_in_pool:
            scale_answers = [
                'Much lower than expected',
                'Lower than expected',
                'As expected',
                'Higher than expected',
                'Much higher than expected',
                'Not experienced service',
            ]

            attributes = [
                (None, 'Ease of getting in contact with the required person within TNT Express’', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Speed of Customer Service answering your telephone calls', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Suitability of Next Day / Time Critical express services', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Suitability of Day Definite services (longer than 2 days)', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Capability of TNT Express to handle all your shipments, weights & sizes', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Capability of TNT Express handling your national deliveries', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Capability of TNT Express handling your international deliveries', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Suitability of TNT Express range of internet services', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Ease of use of TNT Express internet services', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'On time delivery of shipments you sent', [], TntCLMAttribute.TYPE_SCALE,),

                ('General', 'How likely are you to recommend to a friend or colleague?',
                 ['Not At All Likely', '', '', '', '', 'Neutral', '', '', '', '', 'Extremely Likely'],
                 TntCLMAttribute.TYPE_NPS,),

                (None, 'Suggestions for service improvement to exceed your expectations', [], TntCLMAttribute.TYPE_TEXT,),
                (None, 'If the address information was incorrect please provide us with the correct information',
                 ['Last name', 'First name', 'Company', 'Industry', 'Street', 'Postal code', 'E-mail', 'Mr/Mrs/Miss', 'Title',
                  'Departments', 'Phone number', 'Building', 'City'], TntCLMAttribute.TYPE_TEXT,),
            ]
        else:
            # we have to make it match stable
            scale_answers = self.sa_session.query(TntCLMScaleAnswer).all()

            attributes = attributes = [
                (None, 'Ease of getting in contact with the required person within TNT Express’', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Speed of Customer Service answering your telephone calls', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Suitability of Next Day / Time Critical express services', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Suitability of Day Definite services (longer than 2 days)', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Capability of TNT Express to handle all your shipments, weights & sizes', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Capability of TNT Express handling your national deliveries', [], TntCLMAttribute.TYPE_SCALE,),
                (None, 'Capability of TNT Express handling your international deliveries', [], TntCLMAttribute.TYPE_SCALE,)]

            attrs = self.sa_session.query(TntCLMAttribute).filter(TntCLMAttribute.type==TntCLMAttribute.TYPE_SCALE).limit(3).all()
            attributes = attributes + [(None, a, [], a.type) for a in attrs]

            attributes.append(('General', 'How likely are you to recommend to a friend or colleague?',
                 ['Not At All Likely', '', '', '', '', 'Neutral', '', '', '', '', 'Extremely Likely'],
                 TntCLMAttribute.TYPE_NPS,))
            texts = self.sa_session.query(TntCLMAttribute).filter(TntCLMAttribute.type==TntCLMAttribute.TYPE_TEXT).limit(2).all()
            for a in texts:
                attributes.append((None, a, a.answers, a.type))

            # need to create attributes so that ids will match stable
            # (on stable, one attr was created before we had NPS type)
            attrs = [('improve..', TntCLMAttribute.TYPE_TEXT, ['']),
                     ('nothing..', TntCLMAttribute.TYPE_SINGLE_CHOICE, ['1', '', '', '', '5'])]
            for x in attrs:
                l, t, answers = x
                a = TntCLMAttribute()
                a.type = t
                a.default_label = l
                self.save_default_privilege_object_group(a)
                pos = 0
                for ans in answers:
                    pos = pos + 1
                    answer = TntCLMAttributeAnswer()
                    answer.attribute = a
                    answer.default_label = ans
                    answer.position = pos
                    self.sa_session.add(answer)

        priorities = {
            'attribute_1': (2, False,),
            'attribute_2': (1, True,),
        }

        return self._create_wave('clm_wave_36', scale_answers, attributes, opt_out_text='Please tick here if you no longer wish to participate in this survey', priorities=priorities, wave_name=wave_name)

    def create_wave_39c(self):

        scale_answers = [
            'Much lower than expected',
            'Lower than expected',
            'As expected',
            'Higher than expected',
            'Much higher than expected',
            'Not experienced service',
        ]

        attributes = [
            (None, 'Ease of getting in contact with the required person within TNT Express test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test test ', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Overall image of TNT Express', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Adequacy of information provided to you (phone , e-mail etc .)', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'TNT Express’ ability to fulfill commitments', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Speed of complaint resolution', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Overall level of performance of TNT Express transportation services', [], TntCLMAttribute.TYPE_SCALE,),
            ('General', 'Would you recommend TNT Express to others ?', ['not at all', 'maybe', 'yes', 'strongly'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Compared to other providers how would you rate TNT Express’ ability to provide timely notification of delivery problems', ['much lower', 'lower', 'equal', 'higher', 'much higher'], TntCLMAttribute.TYPE_SINGLE_SCALE,),
            ('General', 'What will be your level of purchase with TNT Express over the next year ?', ['none', 'less', 'same', '1-10% more', '> 10% more'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Do you use express distribution companies other than TNT Express ?', ['no', 'yes, less frequently', 'yes, as frequently', 'yes, more frequently'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'If yes, which other company do you use most often ? ', ['FedEx', 'UPS', 'DHL', 'other'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'How much of your total spend on Express transportation (all next/ fastest day deliveries) is given to TNT Express ?', ['< 10%', '10-30%', '30-50%', '50-70%', '> 70%'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'What is your responsibility concerning express shipments in your company ?', ['decision maker', 'influencer', 'user', 'other'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'If the address information was incorrect please provide us with the correct information', ['Last name', 'First name', 'Company', 'Industry', 'Street', 'Postal code', 'E-mail', 'Mr/Mrs/Miss', 'Title', 'Departments', 'Phone number', 'Building', 'City'], TntCLMAttribute.TYPE_TEXT,),
            ('General', 'Another text attribute in the General o7 section', ['answer 1', 'answer 2', 'answer 3'], TntCLMAttribute.TYPE_TEXT,),
            (None, 'Suggestions for service improvement to exceed your expectations', [], TntCLMAttribute.TYPE_TEXT,),
        ]

        priorities = {
            'attribute_1': (2, False,),
            'attribute_2': (1, True,),
            'attribute_15': (2, False,),
            'attribute_16': (2, False,),
            'attribute_33': (2, False,),
            'attribute_34': (2, False,),
        }

        return self._create_wave('clm_wave_36', scale_answers, attributes, opt_out_text='Please tick here if you no longer wish to participate in this survey', priorities=priorities)


    def create_wave_01(self):
        scale_answers = [
            'Much lower than expected',
            'Lower than expected',
            'As expected',
            'Higher than expected',
            'Much higher than expected'
        ]

        attributes = [
            ('About our overall service', 'Overall performance of TNT express distribution services', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our services', 'Ease of getting in contact with the required person within TNT', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our services', 'Speed of answering your telephone calls to Customer Service', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our services', 'Efficiency of the shipment order process', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our services', 'Ability of TNT Customer Service staff to answer all questions', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our services', 'Clarity of TNT\'s consignment note', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our services', 'Ease of finding required information in the Solutions Guide', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our services', 'Clarity of the product range offered', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our services', 'Suitability of TNT’s product range to your business needs', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our services', 'On time pick-up of your consignments', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our services', 'On time delivery of your consignments', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Which of the above attributes are the most critical to you ? (Select up to 3)', [], TntCLMAttribute.TYPE_RANGE,),
            (None, 'Which of the above attributes show major variations in service level from time to time ? (Select up to 3)', [], TntCLMAttribute.TYPE_RANGE,),
            (None, 'Which of the above attributes most influence your choice of an express distribution company ? (Select up to 3)', [], TntCLMAttribute.TYPE_RANGE,),
            (None, 'Comments', [], TntCLMAttribute.TYPE_TEXT,),
            (None, 'In case of name or address change please indicate the correct information', ['Mr/Mrs/Miss', 'Last name', 'Company', 'Department', 'Street', 'City', 'E-mail', 'First name', 'Building', 'Postal code', 'Country'], TntCLMAttribute.TYPE_TEXT,),
            ('About our people', 'Friendliness and attitude of our Customer Service representatives', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our people', 'Professionalism of our Customer Service representatives', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our people', 'Friendliness and attitude of our sales people', [], TntCLMAttribute.TYPE_SCALE,),
            ('About our people', 'Professionalism of our sales people', [], TntCLMAttribute.TYPE_SCALE,),
            (None, 'Which, if any, of the above attributes require priority improvement ? (Select up to 3)', [], TntCLMAttribute.TYPE_RANGE,),
            (None, 'Which of the above attributes most influence your choice of an express distribution company ? (Select up to 3)', [], TntCLMAttribute.TYPE_RANGE,),
            ('General', 'What is your level of expectation for an express distribution service ?', ['very low', 'low', 'medium', 'high', 'very high'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'What are your expectations primarily based on ?', ['experience', 'competition', 'your requirements', 'price'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            ('General', 'Would you recommend TNT to others', ['not at all', 'maybe', 'yes', 'strongly'], TntCLMAttribute.TYPE_SINGLE_CHOICE,),
            (None, 'Comments', [], TntCLMAttribute.TYPE_TEXT,),
        ]

        return self._create_wave('clm_wave_01', scale_answers, attributes, opt_out_text='Tick here if you wish to remain anonymous')

    def get_chinese_translations(self):
        scale_answers = ['比我要求的差很多', '比我要求的差', '与我要求的一致', '比我要求的好', '比我要求的好很多', '未曾使用此服务']

        attributes = [
            '是否容易联系上您要找的TNT工作人员',
            '客服中心接听您电话的速度',
            '次日/限时快递服务的适用性',
            '定日（两天以上）服务的适用性',
            'TNT快递处理您各类不同重量和大小物品的能力',
            'TNT快递处理您国内递送业务的能力(如適用)',
            'TNT快递处理您国际递送业务的能力',
            'TNT快递提供的各类互联网服务的适用性',
            '使用TNT各类互联网服务的简易性',
            '您所寄出的物品的准时送达率',
            '您所收到的物品准时送达率',
            '递送出现问题后及时通知',
            '有效完成物品的完好运送',
            '有效完成多件货的运送',
            '必要时针对问题提供适当解决方案的能力',
            '客服中心处理货运订单的效率',
            'TNT快递提供个性化服务的水平',
            'TNT客服代表解答您问题的能力',
            'TNT客服代表的态度及友好程度',
            'TNT收账员的态度和友好程度',
            'TNT快递销售代表对您业务的理解能力',
            'TNT快递销售代表的态度及友好程度',
            'TNT司机提供帮助的能力',
            'TNT司机的态度和友好程度.',
            '对TNT快递的整体印象',
            '给您提供的信息（电话号码、电子信箱等）的充足性',
            'TNT快递兑现其承诺的能力',
            '解决投诉的速度',
            'TNT快递服务的总体表现',
        ]

        return {
            'scale_answers': scale_answers,
            'attributes': attributes
        }

    def get_arabic_translations(self):

        scale_answers = [
            'أقل بكثير من المتوقع',
            'أقل من المتوقع',
            'كما هو متوقع',
            'أعلى مما كان متوقعا',
            'أعلى بكثير مما كان متوقعا',
            'ليس من ذوي الخبرة خدمة',
        ]

        # feel free to finish this table
        attributes = [
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس ',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي',
            'سرعة الرد على المكالمات الهاتفية لخدمة العملاء',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
            'سهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريس',
        ]

        return {
            'scale_answers': scale_answers,
            'attributes': attributes
        }

    def translate(self, wave_id, locale, skip_attribute=None, set_approved=False):
        assert isinstance(locale, TntCLMLocale)

        if not isinstance(wave_id, TntCLMWave):
            wave = self.sa_session.query(TntCLMWave).filter(TntCLMWave.id == wave_id).one()
        else:
            wave = wave_id

        lr = TntCLMWaveLocaleRevision()
        lr.wave = wave
        lr.account = self.get_account()
        lr.locale = locale
        if set_approved:
            lr.status = 4 # active approved

        wl = TntCLMWaveLocale()
        wl.label = '%s - wave label' % locale.label
        wl.locale_revision = lr
        wl.header_text = '%s - wave header' % locale.label
        wl.sub_header_text = '%s - wave sub header' % locale.label

        self.sa_session.add(wl)

        for attr in wave.attributes:
            if skip_attribute is not None and attr.position in skip_attribute:
                continue
            translation = TntCLMWaveAttributeLocale()
            translation.wave_attribute = attr
            translation.locale_revision = lr
            translation.label = '[%s] - %s' % (locale.label, attr.id)
            self.sa_session.add(translation)

        for sa in wave.scale_answers:
            translation = TntCLMWaveScaleAnswerLocale()
            translation.wave_scale_answer = sa
            translation.locale_revision = lr
            translation.label = '[%s] - %s' % (locale.label, sa.id)
            self.sa_session.add(translation)

        self.sa_session.flush()

    def generateContactList(self, wave_id, number):

        contact_controller = ContactController(self.sa_session, account_id=self.get_account().id, contact_type='TntCLMContact')
        wc = WaveController(self.sa_session, account_id=self.get_account().id)

        locales = wc.get_available_wave_locales(wave_id)
        cl = []
        from random import choice

        revenue_bands = ['A', 'B', 'C', 'D', 'E', 'F', 'G']

        for i in range(0, number):
            person = {
                'first_name': u'fn_%s' % i,
                'middle_name': u'',
                'last_name': u'ln_%s' % i,
                'title': u'mr',
                'gender': None,
                'email': u'clm_contact_%s@mail-test.basesoft.com' % i,
                'position': u'assistent',
                'department': u'marketing',
                'phone': u'001010010011',
                'mobile_phone': '',
                'fax': '',
                'birth_date': '1978-10-10',
                'address_data': {
                    'country': '181',
                    'postcode': u"400295",
                    'street': u" Bihorului 10",
                    'city': u"Cluj Napoca",
                    'state': u"Cluj"
                }
            }

            locale = choice(locales)
            depot = self.sa_session.query(TntDepot).filter(TntDepot.country==locale.country).first()

            company = {
                'name': 'tnt company %s' % i,
                'company_type': 'TntCLMCompany',
                'account_number': '%s' % i,
                'locale_id': locale.id,
                'revenue_band': revenue_bands[i % len(revenue_bands)],
                'depot_id': depot.id
            }
            contact = contact_controller.save_or_update_contact(None, person, company, self.get_default_group().id)
            cl.append(contact)

        contact_list = contact_controller.save_or_update_contact_list(None, u'CLM list', self.get_default_group().id)
        contact_controller.link_contacts(cl, contact_list)
        return contact_list

    def generateResponses(self, wave_id, static_response=None, random=True, factor=0):
        """
        responses for wave type 36
        """
        if static_response is None:
            static_response = {}

        from random import choice

        wc = WaveController(self.sa_session, account_id=self.get_account().id)
        wave = wc.get_waves(wave_id).pop()

        wave_state = wc.get_wave_state(wave_id)['wave']


        def get_responses(wave, start=None, dissatisfied=False, extreme=False, boring=False):
            state = {}
            position = 1

            scale_i = start
            single_i = start - 1
            nps_i = start + 4 # starts is between 1 and 6 but nps goes till 11

            for a in wave['attributes']:
                state[a['id']] = {'type': a['type']}

                if a['type'] == 1: #scale

                    if static_response.has_key('attribute_%s' % position):
                        state[a['id']]['value'] = static_response['attribute_%s' % position]
                    else:

                        # when responses are set to be boring they are all the same
                        if boring:
                            state[a['id']]['value'] = 4 # 4, so we get a ncs score

                        # random
                        elif random:
                            state[a['id']]['value'] = choice(range(1, 7)) #select random radio

                        # else make it more dynamic
                        else:

                            # generate representative data

                            # till i=6, set value = scale_i (linear)
                            value = scale_i

                            # 6 is positive or negative to get a ncs score
                            if value == 7 and not dissatisfied:
                                value = 4
                            if value == 7 and dissatisfied:
                                value = 2

                            # when 'extreme' answer nr 8 gets set to extra positive or negative
                            if value == 8:
                                if extreme:
                                    if dissatisfied:
                                        value = 1
                                    else:
                                        value = 6
                                else:
                                    value = 4 # default to positive

                            # everything > 8 is set to '='
                            if value > 8:
                                value = 3

                            state[a['id']]['value'] = value

                            # use the factor (default 0, incremented +1 per wave in tnt_populate_responses) to give some difference between waves
                            if scale_i == 12 + factor:
                                scale_i = 1
                            else:
                                scale_i = scale_i + 1

                if a['type'] == 2 or a['type'] == 6: #single choice
                    if random:
                        state[a['id']]['value'] = a['answers'][choice(range(0, 3))]['id'] # select random answer radio
                    else:
                        try:
                            state[a['id']]['value'] = a['answers'][single_i]['id']
                        # some questions have only 4 answers
                        except IndexError:
                            state[a['id']]['value'] = a['answers'][choice(range(0, 4))]['id']
                        if single_i == 4:
                            single_i = 0
                        else:
                            single_i = single_i + 1

                if a['type'] == 3: #text
                    if not a.has_key('answers'):
                        state[a['id']]['value'] = u' 客服中心接听您电话的速度'
                    else:
                        state[a['id']]['value'] = {}
                        for answ in a['answers']:
                            state[a['id']]['value'][answ['id']] = 'answer %s' % answ['id']

                if a['type'] == 4: #multiple choice
                    state[a['id']]['value'] = [a['answers'][choice(range(0, 3))]['id'],
                                               a['answers'][choice(range(0, 3))]['id']] # select 2 random answers

                if a['type'] == 7: #nps
                    if random:
                        state[a['id']]['value'] = a['answers'][choice(range(0, 11))]['id'] # select random answer radio
                    else:
                        state[a['id']]['value'] = a['answers'][nps_i]['id']
                        if nps_i == 10:
                            nps_i = 0
                        else:
                            nps_i = nps_i + 1

                position = position + 1

                # type 5 (range) not used for w36+

            return state

        start = 1
        for invitee in wave.invitees:
            pc = ParticipantController(self.sa_session, invitee.invitee_hash)

            dissatisfied = False
            extreme = False
            boring = False

            country = invitee.contact.company.locale.country.iso_3166_a2

            # limit to 10 countries
            if country not in ['NL', 'GB', 'DE', 'AU', 'FR', 'PL', 'TW', 'BE', 'CN', 'IT']:
                continue

            if country == 'DE':
                dissatisfied = True

            if country == 'NL':
                extreme = True

            if country == 'GB':
                dissatisfied = True
                extreme = True

            if country == 'FR':
                boring = True

            logger.info('inserting responses for %s: dissatisfied: %s, extreme: %s, boring: %s' % (country, dissatisfied, extreme, boring))

            state = {'responses': get_responses(wave_state, start, dissatisfied, extreme, boring), 'opt_out': False}
            if start == 6:
                start = 1
            else:
                start = start + 1
            pc.save_responses(state)

    def import_users(self, tnt_domain):
        users = [
            {'person': 'Anuja Waghre', 'username': 'Z436TSO', 'geolevel': '1 - 2011 - ASPAC'},
            {'person': 'Belinda Giovenco', 'username': 'S936MPT', 'country': 'Australia'},
            {'person': 'Bob Yu', 'username': 'U623ULE', 'country': 'China'},
            {'person': 'Carla Creighton', 'username': 'N695FVP', 'country': 'United Kingdom'},
            {'person': 'FM Mok', 'username': 'L915CZQ', 'geolevel': 'AS - S-E Asia - IN 2009'},
            {'person': 'Harri Janakka', 'username': 'R486AJQ', 'geolevel': 'CE - Nordic region 09'},
            {'person': 'HJ Boudewijn', 'username': 'U125PLE', 'geolevel':  'CE - Benelux'},
            {'person': 'Ilario DeFilippo', 'username': 'L815CVC', 'country': 'Italy'},
            {'person': 'Isabelle Truchet', 'username': 'K450HME', 'country': 'France'},
            {'person': 'Magda Stafa', 'username': 'Z150BTE', 'country': 'Poland'},
            {'person': 'Mandy Hsu', 'username': 'A889SRZ', 'country': 'Taiwan'},
            {'person': 'Meiqing Ma', 'username': 'Y855AMR', 'geolevel': 'AS - North Asia 2009'},
            {'person': 'OlivierGrootjans', 'username': 'K414UWE', 'geolevel': '3. CMI2009 - C-E EUROPE'},
            {'person': 'Owen Maybank', 'username': 'Z052QTJ', 'geolevel': '5. UK/IE BU'},
            {'person': 'Roger Prine', 'username': 'A984SJT', 'geolevel': 'AS - AU-NZ - Pacifics 09'},
            {'person': 'Thomas Kraus', 'username': 'T999LTR', 'country': 'Germany'},
        ]

        session = self.sa_session
        geolevel_objs = {}
        locales = {} # locales list for countries

        from iqportal.tnt.clm.models import TntCLMAccount
        from hashlib import sha256 as sha
        from sqlalchemy import and_

        # create a new group for this new users
        g = session.query(Group).filter(Group.name=='Report managers').all()
        if len(g) > 0:
            group = g[0]
        else:
            group = Group()
            group.name = 'Report managers'
            group.license = tnt_domain.default_license
            group.permissions = session.query(Permission).filter(Permission.name.in_([
                        'view_dashboard', 'create_notification', 'view_notification', 'update_notification',
                        'delete_notification', 'send_to_license_default_group', 'send_to_domain_default_group',
                        'view_geolevel', 'view_wave_responses', 'view_datasource', 'view_tnt_clm_wave',
                        'view_datasource_widget'])).all()
            session.add(group)
            session.flush()

        # create accounts (the password will be same as username)
        for account in users:

            a = session.query(Account).filter(Account.name==account['username']).all()
            if len(a) > 0:
                continue

            a = TntCLMAccount()
            a.name = account['username']
            a.password = sha(account['username']).hexdigest()

            ga = GroupAccount()
            ga.account = a
            ga.group = group
            ga.active = True
            ga.is_default = True
            a.groups = [ga]

            # person
            p = Person()
            sp = account['person'].rfind(' ')
            if sp == -1:
                p.first_name = account['person']
            else:
                p.first_name = account['person'][:sp]
                p.last_name = account['person'][sp+1:]
            a.person = p

            # locales
            if account.has_key('country'):
                c = account['country']
                if not locales.has_key(c):
                    co = session.query(Country).filter(Country.name==c).all()
                    if len(co) == 0:
                        co = session.query(DomainCountry).filter(and_(
                            DomainCountry.domain==tnt_domain, DomainCountry.name==c)).all()
                        if len(co) == 0:
                            raise Exception('Invalid country %s' % c)
                        co = co[0].country
                    else:
                        co = co[0]
                    l = session.query(TntCLMLocale).filter(TntCLMLocale.country==co).all()
                    if len(l) == 0:
                        print 'Cannot find tnt locale for country %s. Skipping...' % c
                        continue
                    locales[c] = l
                a.locales = locales[c]

            # geolevel
            if account.has_key('geolevel'):
                g = account['geolevel']
                if not geolevel_objs.has_key(g):
                    geol = session.query(TntGeolevel).filter(TntGeolevel.name==g).all()
                    if len(geol) == 0:
                        print 'Invalid geolevel %s. Skipping...' % g
                        continue
                    geolevel_objs[g] = geol
                a.geolevels = geolevel_objs[g]

            #locale
            if account.has_key('locale'):
                l = session.query(TntCLMLocale).filter(TntCLMLocale.label==account['locale']).all()
                if len(l) == 0:
                    print 'Invalid locale %s. Skipping...' % account['locale']
                    continue
                a.locales = l

            session.add(a)

    @transaction_decorator
    def create_services(self):
        from iqportal.tnt.clm.models import TntCLMService
        tnt_services = ['Special Services', 'Business Solutions', 'Riteway', 'Fashion', 'Associates', 'Express']

        self.service_map = {}
        for service in self.sa_session.query(TntCLMService).all():
            self.service_map[service.label] = service
        for l in tnt_services:
            if self.service_map.has_key(l):
                continue
            s = TntCLMService()
            s.label = l
            self.sa_session.add(s)
            self.service_map[l] = s

    @transaction_decorator
    def create_locales(self):
        tnt_codes = {
            'AEENI': (None,'AE', 'English (United Arab Emirates)', 'ltr', None, None, ),
            'ARESI': ('es_AR','AR' ,'Spanish (Argentina)', 'ltr', None, None, ),
            'ATATI': (None, 'AT', 'Austrian', 'ltr', None, None, ),
            'ATDEI': ('de_AT', 'AT', 'German (Austria)', 'ltr', None, None, ),
            'AUAPD': (None, 'AU', 'Australian', 'ltr', None, None, ),
            'AUEND': ('en_AU', 'AU', 'English (Austrialia)', 'ltr', None, None, ),
            'AUENI': ('en_AU', 'AU', 'English (Austrialia)', 'ltr', None, None, ),
            #'AURTW': ('en_AU', 'AU', 'AU Riteway', 'ltr', 'Riteway', None, ),
            'BDENI': (None, 'BD', 'English (Bangladesh)', 'ltr', 'Associates', None, ),
            'BEFRD': ('fr_BE', 'BE', 'French (Belgium)', 'ltr', None, None, ),
            'BEFRI': ('fr_BE', 'BE', 'French (Belgium)', 'ltr', None, None, ),
            'BENLD': ('nl_BE', 'BE', 'Dutch (Belgium)', 'ltr', None, None, ),
            'BENLI': ('nl_BE', 'BE', 'Dutch (Belgium)', 'ltr', None, None, ),
            'BGBGI': ('bg_BG', 'BG', 'Bulgarian', 'ltr', None, None, ),
            'BHEND': (None, 'BH', 'English (Bahrain)', 'ltr', None, None, ),
            'BHENI': (None, 'BH', 'English (Bahrain)', 'ltr', None, None, ),
            'BRPTI': ('pt_BR', 'BR', 'Portuguese (Brazil)', 'ltr', None, None, ),
            'CAENI': ('en_CA', 'CA', 'English (Canada)', 'ltr', None, None, ),
            'CAFRI': ('fr_CA', 'CA', 'French (Canada)', 'ltr', None, None, ),
            'CHDEI': ('de_CH', 'CH', 'German (Switzerland)', 'ltr', None, None, ),
            'CHFRI': ('fr_CH', 'CH', 'French (Switzerland)', 'ltr', None, None, ),
            'CHITI': ('it_CH', 'CH', 'Italian (Switzerland)', 'ltr', None, None, ),
            'CLESI': (None, 'CL', 'Spanish (Chile)', 'ltr', None, None, ),
            'CNCNI': (None, 'CN', 'Chinese', 'ltr', None, None, ),
            'CYENI': (None, 'CY', 'English (Cyprus)', 'ltr', None, None, ),
            'CZCZI': ('cz_CZ', 'CZ', 'Czech', 'ltr', None, None, ),
            'DEDED': ('de_DE', 'DE', 'German', 'ltr', None, None, ),
            'DEDEI': ('de_DE', 'DE', 'German', 'ltr', None, None, ),
            'DKDKI': ('dk_DK', 'DK', 'Danish', 'ltr', None, None, ),
            'EEEEI': (None, 'EE', 'Estonian', 'ltr', None, None, ),
            'EGARI': ('ar_EG', 'EG', 'Arabic (Egypt)', 'rtl', None, None, ),
            'EGENI': (None, 'EG', 'English (Egypt)', 'ltr', None, None, ),
            'ESESD': ('es_ES', 'ES', 'Spanish', 'ltr', None, None, ),
            'ESESI': ('es_ES', 'ES', 'Spanish', 'ltr', None, None, ),
            'FIFII': ('fi_FI', 'FI', 'Finnish', 'ltr', None, None, ),
            'FJENI': (None, 'FJ', 'English (Fiji)', 'ltr', None, None, ),
            'FRFRD': ('fr_FR', 'FR', 'French', 'ltr', None, None, ),
            'FRFRI': ('fr_FR', 'FR', 'French', 'ltr', None, None, ),
            #'GBBS':  ('en_GB', 'GB', 'UK Business Solutions', 'ltr', 'Business Solutions', None, ),
            'GBEND': ('en_GB', 'GB', 'English', 'ltr', None, None, ),
            'GBENI': ('en_GB', 'GB', 'English', 'ltr', None, None, ),
            #'GBSpS': ('en_GB', 'GB', 'UK Special Services', 'ltr', 'Special Services', None, ),
            'GRGRI': ('gr_GR', 'GR', 'Greek', 'ltr', None, None, ),
            'HKCNI': (None, 'HK', 'Chinese (Hong Kong)', 'ltr', None, None, ),
            'HUHUI': ('hu_HU', 'HU', 'Hungarian', 'ltr', None, None, ),
            'IDIDI': (None, 'ID', 'Indonesian', 'ltr', None, None, ),
            'IEENI': (None, 'IE', 'Irish', 'ltr', None, None, ),
            'ILHEI': (None, 'IL', 'Hebrew', 'rtl', None, None, ),
            'INEND': ('en_IN', 'IN', 'English (India)', 'ltr', None, None, ),
            'INENI': ('en_IN', 'IN', 'English (India)', 'ltr', None, None, ),
            'ITITD': ('it_IT', 'IT', 'Italian', 'ltr', None, None, ),
            'ITITI': ('it_IT', 'IT', 'Italian', 'ltr', None, None, ),
            'JOENI': (None, 'JO', 'English (Jordan)', 'ltr', None, None, ),
            'JPJPI': ('jp_JP', 'JP', 'Japanese', 'ltr', None, None, ),
            'KEENI': (None, 'KE', 'English (Kenya)', 'ltr', None, None, ),
            'KRKRI': (None, 'KR', 'Korean', 'ltr', None, None, ),
            'KWARI': ('ar_KW', 'KW', 'Arabic (Kuwait)', 'rtl', None, None, ),
            'KWENI': (None, 'KW', 'English (Kuwait)', 'ltr', None, None, ),
            'LKENI': (None, 'LK', 'English (Sri Lanka', 'ltr', 'Associates', None, ),
            'LTLTI': (None, 'LT', 'Lithuanian', 'ltr', None, None, ),
            'LUFRI': ('fr_LU', 'LU', 'French (Luxembourg)', 'ltr', None, None, ),
            'LVLVI': (None, 'LV', 'Latvian', 'ltr', None, None, ),
            'MYENI': (None, 'MY', 'English (Malaysia)', 'ltr', None, None, ),
            'NLENI': (None, 'NL', 'English (Netherlands)', 'ltr', None, None, ),
            'NLEND': (None, 'NL', 'English (Netherlands)', 'ltr', None, None, ),
            'NLNLD': ('nl_NL', 'NL', 'Dutch', 'ltr', None, None, ),
            'NLNLI': ('nl_NL', 'NL', 'Dutch', 'ltr', None, None, ),
            #'NLFAS': ('nl_NL', 'NL', 'TNT Fashion', 'ltr', 'Fashion', None, ),
            'NONOI': ('no_NO', 'NO', 'Norwegian', 'ltr', None, None, ),
            'NZENI': ('en_NZ', 'NZ', 'English (New Zealand)', 'ltr', None, None, ),
            'OMENI': (None, 'OM', 'English (Oman)', 'ltr', 'Associates', None, ),
            'PHENI': (None, 'PH', 'English (Philippines)', 'ltr', None, None, ),
            'PLPLI': ('pl_PL', 'PL', 'Polish', 'ltr', None, None, ),
            'PTPTI': ('pt_PT', 'PT', 'Portuguese', 'ltr', None, None, ),
            'ROROI': ('ro_RO', 'RO', 'Romanian', 'ltr', None, None, ),
            'RURUI': ('ru_RU', 'RU', 'Russian', 'ltr', None, None, ),
            'SAARI': ('ar_SA', 'SA', 'Arabic (Saudi Arabia)', 'rtl', None, None, ),
            'SAENI': (None, 'SA', 'English (Saudi Arabia)', 'ltr', None, None, ),
            'SESEI': ('se_SE', 'SE', 'Swedish', 'ltr', None, None, ),
            'SGENI': (None, 'SG', 'English (Singapore)', 'ltr', None, None, ),
            'SISII': ('si_SI', 'SI', 'Slovanian', 'ltr', None, None, ),
            'SKSKI': ('sk_SK', 'SK', 'Slovakian', 'ltr', None, None, ),
            'THTHI': ('th_TH', 'TH', 'Thai', 'ltr', None, None, ),
            'THTHD': ('th_TH', 'TH', 'Thai', 'ltr', None, None, ),
            'TRTRI': ('tr_TR', 'TR', 'Turkish', 'ltr', None, None, ),
            'TWTWI': (None, 'TW', 'Taiwanese', 'ltr', None, None, ),
            'UAUAI': (None, 'UA', 'Ukrainian', 'ltr', 'Associates', None, ),
            'USENI': ('en_US', 'US', 'English (USA)', 'ltr', None, None, ),
            'VNVNI': ('vn_VN', 'VN', 'Vietnamese', 'ltr', None, None, ),
            'ZAENI': (None, 'ZA', 'English (South Africa)', 'ltr', None, None, ),
            'XXENI': (None, 'GB', 'English (Default)', 'ltr', None, None,  ),
        }
        if len(self.service_map) == 0:
            for s in self.sa_session.query(TntCLMService).all():
                self.service_map[s.label] = s

        current_codes = [c.label for c in self.sa_session.query(TntCLMLocale).all()]
        for label, config in tnt_codes.iteritems():
            locale, country, description, direction, service, location = config
            # if we already have a type for the same module, skip it for now.
            if label in current_codes:
                continue
            if location is None:
                if label[-1] in ['D', 'I']:
                    location = label[-1]
                else:
                    location = 'D'
            location_2015 = location
            language_code = label[2:4]
            if label == 'XXENI':
                language_code = 'XX'

            l = TntCLMLocale()
            l.label = label
            l.description = description
            l.language_code = language_code
            l.direction = direction
            l.location = location
            l.location_2015 = location_2015
            l.country = self.sa_session.query(Country).filter(Country.iso_3166_a2==country).one()
            try:
                l.locale = self.sa_session.query(Locale).filter(Locale.locale==locale).one()
            except:
                pass
            if service is None:
                service = 'Express'
            if not self.service_map.has_key(service):
                print 'Unable to find service (%s) for CLM locale (%s)' % (service, label)
            else:
                l.service = self.service_map[service]

            self.sa_session.add(l)

    @transaction_decorator
    def create_depots(self, tnt_domain):
        depots = [('AR', 'Buenos Aires', 'BUE', None, None, ),
            ('AR', 'Cordoba', 'COR', None, None, ),
            ('AR', 'Mendoza', 'MDZ', None, None, ),
            ('AR', 'Argentina Others', 'ARO', None, None, ),
            ('AU', 'Adelaide', 'ADL', None, None, ),
            ('AU', 'Brisbane', 'BNE', None, None, ),
            ('AU', 'Bankstown', 'BWU', None, None, ),
            ('AU', 'Canberra', 'CBR', None, None, ),
            ('AU', 'Cairns', 'CNS', None, None, ),
            ('AU', 'Darwin', 'DRW', None, None, ),
            ('AU', 'Melbourne', 'MEL', None, None, ),
            ('AU', 'Perth', 'PER', None, None, ),
            ('AU', 'Sydney', 'SYD', None, None, ),
            ('AU', 'NSW International', 'NSI', None, None, ),
            ('AU', 'NT International', 'NTI', None, None, ),
            ('AU', 'QLD International', 'QLI', None, None, ),
            ('AU', 'SA International', 'SA', None, None, ),
            ('AU', 'TAS International', 'TAI', None, None, ),
            ('AU', 'VIC International', 'VII', None, None, ),
            ('AU', 'WA International', 'WA', None, None, ),
            ('AU', 'AU Capital Territory', 'ACT', None, None, ),
            ('AU', 'Newcastle AU', 'NTL', None, None, ),
            ('AU', 'Townsville', 'TSV', None, None, ),
            ('AU', 'Gold Coast', 'OOL', None, None, ),
            ('AU', 'Brisbane North - AU Dom', 'BN1', None, ('BZJ', ), ),
            ('AU', 'Brisbane South - AU Dom', 'BN2', None, None, ),
            ('AU', 'Enfield - AU Dom', 'EN1', None, ('ENR', ), ),
            ('AU', 'Chullora - AU Dom', 'WS1', None, None, ),
            ('AU', 'Hallam - AU Dom', 'HL1', None, ('HLQ', ), ),
            ('AU', 'Campbellfield', 'CB3', None, None, ),
            ('AT', 'Graz', 'GRZ', None, None, ),
            ('AT', 'Innsbruck', 'INN', None, None, ),
            ('AT', 'Klagenfurt', 'KLU', None, None, ),
            ('AT', 'Linz', 'LNZ', None, None, ),
            ('AT', 'Lauterach', 'LTA', None, None, ),
            ('AT', 'Salzburg', 'SZG', None, None, ),
            ('AT', 'Vienna', 'VIE', None, None, ),
            ('BE', 'Antwerpen', 'ANR', None, None, ),
            ('BE', 'Brussel', 'BRU', None, None, ),
            ('BE', 'Kortrijk', 'KOR', None, None, ),
            ('BE', 'Liege', 'LGE', None, None, ),
            ('BE', 'Belgium MAC', 'MAC', None, None, ),
            ('BR', 'Brasilia', 'BSB', None, None, ),
            ('BR', 'Blumenau', 'BNU', None, None, ),
            ('BR', 'Porto Alegro', 'POA', None, None, ),
            ('BR', 'Rio de Janeiro', 'RIO', None, None, ),
            ('BR', 'Sao Paolo', 'SAO', None, None, ),
            ('BR', 'Santos', 'SSZ', None, None, ),
            ('BR', 'Barueri', 'EBI', None, None, ),
            ('BR', 'Novo Hamburgo', 'QHV', None, None, ),
            ('BR', 'San Bernardo do Campo', 'QSB', None, None, ),
            ('BR', 'Itajai', 'ITJ', None, None, ),
            ('BR', 'Salvador', 'SSA', None, None, ),
            ('BR', 'Bela Horizonte', 'BHZ', None, None, ),
            ('BR', 'Campinas', 'CPQ', None, None, ),
            ('BR', 'Curitiba', 'CWB', None, None, ),
            ('BR', 'Vitoria', 'VIX', None, None, ),
            ('CA', 'Edmonton', 'YEG', None, None, ),
            ('CA', 'Halifax', 'YHZ', None, None, ),
            ('CA', 'Ottawa', 'YOW', None, None, ),
            ('CA', 'Montreal', 'YUL', None, None, ),
            ('CA', 'Vancouver', 'YVR', None, None, ),
            ('CA', 'Winnipeg', 'YWG', None, None, ),
            ('CA', 'London', 'YXU', None, None, ),
            ('CA', 'Calgary', 'YYC', None, None, ),
            ('CA', 'Toronto', 'YYZ', None, None, ),
            ('CA', 'Melville', 'YYX', None, None, ),
            ('CL', 'Concepcion', 'CCP', None, None, ),
            ('CL', 'Santiago', 'SCL', None, None, ),
            ('CL', 'Valparaiso', 'VAP', None, None, ),
            ('CN', 'Guangzhou', 'CAN', None, None, ),
            ('CN', 'Changzhou', 'CZX', None, None, ),
            ('CN', 'Hangzhou', 'HGH', None, None, ),
            ('CN', 'Beijing', 'PEK', None, ('BE3', ), ),
            ('CN', 'Shanghai', 'SHA', None, None, ),
            ('CN', 'Shenzen', 'SZX', None, None, ),
            ('CN', 'Dalian', 'DLC', None, None, ),
            ('CN', 'Tianjin', 'TSN', None, None, ),
            ('CN', 'Nanjing', 'NKG', None, None, ),
            ('CN', 'Ningbo', 'NGB', None, None, ),
            ('CN', 'Wuhan', 'WUH', None, None, ),
            ('CN', 'Zhengzhou', 'CGO', None, None, ),
            ('CN', 'Chongqing', 'CKG', None, None, ),
            ('CN', 'Chengdu', 'CT3', None, None, ),
            ('CN', 'Kunming', 'KMG', None, None, ),
            ('CN', 'Shengyang', 'SH3', None, None, ),
            ('CN', 'Shijiazhuang', 'SJ1', None, None, ),
            ('CN', 'Suzhou', 'SZV', None, None, ),
            ('CN', 'Qingdao', 'TAO', None, None, ),
            ('CN', 'Jinan', 'TNA', None, None, ),
            ('CN', 'Wuxi', 'WUX', None, None, ),
            ('CZ', 'Prague', 'PRG', None, None, ),
            ('CZ', 'Brno', 'BRQ', None, None, ),
            ('CZ', 'Ostrava', 'OSR', None, None, ),
            ('CZ', 'Czech Others', 'CZO', None, None, ),
            ('DK', 'Aalborg', 'AAL', None, None, ),
            ('DK', 'Kolding', 'BLL', None, None, ),
            ('DK', 'Copenhagen', 'CPH', None, None, ),
            ('EE', 'Estonia others', 'EEO', None, None, ),
            ('EE', 'Tallinn', 'TLL', None, None, ),
            ('FI', 'Helsinki', 'HEL', None, None, ),
            ('FI', 'Jyvaskyla', 'JYV', None, None, ),
            ('FI', 'Lahti', 'LTI', None, None, ),
            ('FI', 'Oulu', 'OUL', None, None, ),
            ('FI', 'Turku', 'TKU', None, None, ),
            ('FI', 'Tampere', 'TRE', None, None, ),
            ('FI', 'Vaasa', 'VAA', None, None, ),
            ('FR', 'Bordeaux', 'BOD', None, None, ),
            ('FR', 'Besancon', 'BSA', None, None, ),
            ('FR', 'Bercy', 'BYE', None, None, ),
            ('FR', 'Lille', 'LIL', None, None, ),
            ('FR', 'Lyon', 'LYS', None, None, ),
            ('FR', 'Marseille', 'MRS', None, None, ),
            ('FR', 'Nice', 'NCE', None, None, ),
            ('FR', 'Nantes', 'NTE', None, None, ),
            ('FR', 'Strasbourg', 'SXB', None, None, ),
            ('FR', 'Toulouse', 'TLS', None, None, ),
            ('FR', 'Tours', 'TUF', None, None, ),
            ('FR', 'Rouen', 'URO', None, None, ),
            ('FR', 'Est', 'EST', None, None, ),
            ('FR', 'Sud Est', 'SES', None, None, ),
            ('FR', 'Ouest', 'OUE', None, None, ),
            ('FR', 'Paris Nord JS', 'PNO', None, None, ),
            ('FR', 'Paris Sud JS', 'PSU', None, None, ),
            ('FR', 'Sud Ouest', 'SUO', None, None, ),
            ('FR', 'Sud', 'SUD', None, None, ),
            ('FR', 'France Nord', 'NOR', None, None, ),
            ('FR', 'France Ouest', 'FOU', None, None, ),
            ('FR', 'France Sud', 'FSU', None, None, ),
            ('FR', 'Paris -  Nord', 'PN', None, None, ),
            ('FR', 'Paris - Sud', 'PS', None, None, ),
            ('FR', 'France Sud Ouest', 'FSO', None, None, ),
            ('FR', 'France Sud Est', 'FSE', None, None, ),
            ('FR', 'France Est', 'FES', None, None, ),
            ('FR', 'Avignon', '121', None, None, ),
            ('FR', 'Bordeaux - JS', '901', None, None, ),
            ('FR', 'Alfortville', '401', None, None, ),
            ('FR', 'Chalon sur Saone', '514', None, None, ),
            ('FR', 'Clamart', '411', None, None, ),
            ('FR', 'Clermont Ferrand', '116', None, None, ),
            ('FR', 'Compiegne - JS', '512', None, None, ),
            ('FR', 'Eragny', '413', None, None, ),
            ('FR', 'Evry - JS', '412', None, None, ),
            ('FR', 'Ferrieres en Brie', '407', None, None, ),
            ('FR', 'Gonesse', '402', None, None, ),
            ('FR', 'Le Mans - JS', '308', None, None, ),
            ('FR', 'Lille - JS', '511', None, None, ),
            ('FR', 'Limoges', '914', None, None, ),
            ('FR', 'Lyon - JS', '101', None, None, ),
            ('FR', 'Marseille - JS', '120', None, None, ),
            ('FR', 'Montpellier', '118', None, None, ),
            ('FR', 'Mulhouse', '505', None, None, ),
            ('FR', 'Nancy', '501', None, None, ),
            ('FR', 'Nantes - JS', '301', None, None, ),
            ('FR', 'Nice - JS', '124', None, None, ),
            ('FR', 'Niort - JS', '913', None, None, ),
            ('FR', 'Niort', 'NIT', None, None, ),
            ('FR', 'Orleans - JS', '321', None, None, ),
            ('FR', 'Pau', '905', None, None, ),
            ('FR', 'Plaisir', '409', None, None, ),
            ('FR', 'Reims', '503', None, None, ),
            ('FR', 'Rennes', '302', None, None, ),
            ('FR', 'Rouen - JS', '318', None, None, ),
            ('FR', 'Saint Etienne', '104', None, None, ),
            ('FR', 'St Quentin Fallavier', '102', None, None, ),
            ('FR', 'Strasbourg - JS', '502', None, None, ),
            ('FR', 'Toulouse - JS', '902', None, None, ),
            ('FR', 'Tours - JS', '305', None, None, ),
            ('FR', 'Valence', '105', None, None, ),
            ('FR', 'Angers', 'ANE', None, None, ),
            ('FR', 'Compiegne', 'XCP', None, None, ),
            ('FR', 'Evry', 'JEV', None, None, ),
            ('FR', 'France Others', 'FXO', None, None, ),
            ('FR', 'Garonor', 'GNO', None, None, ),
            ('FR', 'Laval', 'LVA', None, None, ),
            ('FR', 'Le Mans', 'LME', None, None, ),
            ('FR', 'Metz', 'ETZ', None, None, ),
            ('FR', 'Orleans', 'ORE', None, None, ),
            ('FR', 'Charles de Gaulle', 'CDG', None, None, ),
            ('FR', 'Creteil', 'QFC', None, None, ),
            ('FR', 'Annecy Int', 'AN2', None, None, ),
            ('FR', 'Chartres I', 'CH4', None, None, ),
            ('FR', 'Montpellier I', 'MPL', None, None, ),
            ('FR', 'Rennes I', 'RN7', None, None, ),
            ('FR', 'Chalon sur Saone I', 'XCD', None, None, ),
            ('DE', 'Augsburg', 'AGB', None, None, ),
            ('DE', 'Berlin', 'BER', None, None, ),
            ('DE', 'Bielefeld', 'BIE', None, None, ),
            ('DE', 'Bremen', 'BRE', None, None, ),
            ('DE', 'Cologne', 'COL', None, None, ),
            ('DE', 'Dresden', 'DRS', None, None, ),
            ('DE', 'Dortmund', 'DTM', None, None, ),
            ('DE', 'Hilden', 'DUS', None, None, ),
            ('DE', 'Erfurt', 'ERF', None, None, ),
            ('DE', 'Freiburg', 'FBG', None, None, ),
            ('DE', 'Frankfurt', 'FRT', None, None, ),
            ('DE', 'Guestrow', 'GSW', None, None, ),
            ('DE', 'Hannover', 'HAJ', None, None, ),
            ('DE', 'Hamburg', 'HAM', None, None, ),
            ('DE', 'Hamminkeln', 'HMK', None, None, ),
            ('DE', 'Koblenz', 'KBZ', None, None, ),
            ('DE', 'Kassel', 'KLE', None, None, ),
            ('DE', 'Leipzig', 'LPZ', None, None, ),
            ('DE', 'Magdeburg', 'MDG', None, None, ),
            ('DE', 'Mannheim', 'MHN', None, None, ),
            ('DE', 'Munich', 'ZMU', None, None, ),
            ('DE', 'Nuernberg', 'NBE', None, None, ),
            ('DE', 'Dusseldorf', 'QDU', None, None, ),
            ('DE', 'Straubing', 'SBG', None, None, ),
            ('DE', 'Saarbruecken', 'SCN', None, None, ),
            ('DE', 'Siegen', 'SGE', None, None, ),
            ('DE', 'Ulm', 'ULM', None, None, ),
            ('DE', 'Villingen', 'VGN', None, None, ),
            ('DE', 'Wuerzburg', 'WZB', None, None, ),
            ('DE', 'Stuttgart', 'ZWS', None, None, ),
            ('DE', 'Wiesbaden', 'ZRB', None, None, ),
            ('DE', 'Troisdorf', 'TRS', None, None, ),
            ('GB', 'Barking', 'BK4', None, None, ),
            ('GB', 'Byfleet Domestic', 'BY4', None, None, ),
            ('GB', 'Crawley', 'CW4', None, None, ),
            ('GB', 'Croydon Domestic', 'CY4', None, None, ),
            ('GB', 'Deeside', 'DS4', None, None, ),
            ('GB', 'Hornsey', 'HS4', None, None, ),
            ('GB', 'Leicester Domestic', 'LC4', None, None, ),
            ('GB', 'Maidstone Domestic', 'MS4', None, None, ),
            ('GB', 'Milton', 'MT4', None, None, ),
            ('GB', 'Nottingham', 'NT4', None, None, ),
            ('GB', 'Offshore Guernsey', 'OFG', None, None, ),
            ('GB', 'Park Royal', 'PK4', None, None, ),
            ('GB', 'Ramsbottom', 'RM4', None, ('RB4', ), ),
            ('GB', 'Slough Domestic', 'SL4', None, None, ),
            ('GB', 'Stirling Domestic', 'ED4', None, None, ),
            ('GB', 'Thetford Domestic', 'TT4', None, None, ),
            ('GB', 'Wellingborough Domestic', 'WB4', None, None, ),
            ('GB', 'Bristol', 'BRS', None, None, ),
            ('GB', 'Aberdeen', 'ABZ', None, None, ),
            ('GB', 'Belfast', 'BFS', None, None, ),
            ('GB', 'Birmingham', 'XHB', None, None, ),
            ('GB', 'Carlisle', 'CAR', None, None, ),
            ('GB', 'Southampton', 'SOU', None, None, ),
            ('GB', 'Stansted', 'NTS', None, None, ),
            ('GB', 'Durham (NCL)', 'NCL', None, None, ),
            ('GB', 'Teesside', 'TEJ', None, None, ),
            ('GB', 'Northampton', 'NTH', None, None, ),
            ('GB', 'Reading', 'RED', None, None, ),
            ('GB', 'Leeds', 'LBA', None, None, ),
            ('GB', 'Llantrisant', 'LLT', None, None, ),
            ('GB', 'London City', 'LCY', None, None, ),
            ('GB', 'London Heathrow Depot', 'LON', None, None, ),
            ('GB', 'Luton', 'LTN', None, None, ),
            ('GB', 'Trafford Park (renamed)', 'MAN', None, None, ),
            ('GB', 'Exeter', 'EXT', None, None, ),
            ('GB', 'Paisley (renamed)', 'GLA', None, None, ),
            ('GB', 'Nuneaton', 'CVT', None, None, ),
            ('GB', 'Crawley (LGW)', 'LGW', None, None, ),
            ('GB', 'East Midlands', 'EMC', None, None, ),
            ('GB', 'Edinburgh', 'EDH', None, None, ),
            ('GB', 'Cannock', 'CN4', None, None, ),
            ('GB', 'Worcester', 'WOF', None, None, ),
            ('GB', 'Basildon', 'BZ4', None, None, ),
            ('GB', 'Bradford', 'BA4', None, None, ),
            ('GB', 'Liverpool', 'LXL', None, None, ),
            ('GB', 'Alton', 'AT4', None, None, ),
            ('GB', 'Vauxhall', 'VA4', None, None, ),
            ('GB', 'Rotherham', 'RH4', None, None, ),
            ('GB', 'Jersey - Dom', 'OFJ', None, None, ),
            ('GB', 'Milton Keynes', 'MK4', None, None, ),
            ('GB', 'Enfield UK', 'EN4', None, None, ),
            ('GB', 'Preston', 'PN4', None, None, ),
            ('GB', 'Manchester (new)', 'MC4', None, None, ),
            ('GB', 'Glasgow (new)', 'GW4', None, None, ),
            ('GB', 'UK BS SV1', '1SV', 'Business Solutions', None, ),
            ('GB', 'UK BS SV2', '2SV', 'Business Solutions', None, ),
            ('GB', 'UK BS SV3', '3SV', 'Business Solutions', None, ),
            ('GB', 'UK BS SV4', '4SV', 'Business Solutions', None, ),
            ('GB', 'UK SPS R01', 'R01', 'Special Services', None, ),
            ('GB', 'UK SPS R02', 'R02', 'Special Services', None, ),
            ('GB', 'UK SPS R03', 'R03', 'Special Services', None, ),
            ('GB', 'UK SPS R04', 'R04', 'Special Services', None, ),
            ('GB', 'UK SPS R05', 'R05', 'Special Services', None, ),
            ('GB', 'UK SPS R06', 'R06', 'Special Services', None, ),
            ('GR', 'Athens', 'ATH', None, None, ),
            ('GR', 'Thessaloniki', 'SKG', None, None, ),
            ('GR', 'Maroussi', 'GMZ', None, None, ),
            ('GR', 'Piraeus', 'PRE', None, None, ),
            ('GR', 'Greece Others', 'GR3', None, None, ),
            ('GR', 'Heraklion', 'HK1', None, None, ),
            ('HK', 'Hong kong', 'HKG', None, None, ),
            ('HU', 'Budapest', 'BUD', None, None, ),
            ('HU', 'Hungary Others', 'HUO', None, None, ),
            ('IN', 'Bangalore', 'BLR', None, None, ),
            ('IN', 'Bombay', 'BOM', None, None, ),
            ('IN', 'Calcutta', 'CCU', None, None, ),
            ('IN', 'Delhi', 'DEL', None, None, ),
            ('IN', 'Hyderabad', 'HYD', None, None, ),
            ('IN', 'Ludhiana', 'LUH', None, None, ),
            ('IN', 'Madras', 'MAA', None, None, ),
            ('IN', 'Tirupur', 'TPB', None, None, ),
            ('IN', 'Ahmedabad', 'AMD', None, None, ),
            ('IN', 'Bhubaneswar', 'BBI', None, None, ),
            ('IN', 'Guwahati', 'GAU', None, None, ),
            ('IN', 'Chandigarh', 'IXC', None, None, ),
            ('IN', 'JAMSHEDPUR', 'IXW', None, None, ),
            ('IN', 'Jaipur', 'JAI', None, None, ),
            ('IN', 'Kanpur', 'KNU', None, None, ),
            ('IN', 'Poona', 'PNQ', None, None, ),
            ('ID', 'Jakarta', 'JKT', None, None, ),
            ('ID', 'Medan', 'MES', None, None, ),
            ('ID', 'Bandung', 'BDO', None, None, ),
            ('ID', 'Surabaya', 'SUB', None, None, ),
            ('ID', 'Denpasar', 'DPS', None, None, ),
            ('ID', 'Bandar Ipoh Baru', 'IPH', None, None, ),
            ('ID', 'Semarang', 'SRG', None, None, ),
            ('IE', 'Dublin - 001', 'DUB', None, None, ),
            ('IE', 'Cork', 'ORK', None, None, ),
            ('IE', 'Shannon', 'SNN', None, None, ),
            ('IE', 'Sligo', 'SXL', None, None, ),
            ('IE', 'Waterford', 'WAT', None, None, ),
            ('IE', 'Dublin - 003', 'DU3', None, None, ),
            ('IE', 'Dublin - 002', 'DU2', None, None, ),
            ('IT', 'Milano', 'MI', None, None, ),
            ('IT', 'Padova', 'PD', None, None, ),
            ('IT', 'Bologna', 'BO', None, None, ),
            ('IT', 'Genoa', 'GE', None, None, ),
            ('IT', 'Verona', 'VR', None, None, ),
            ('IT', 'Bergamo', 'BG', None, None, ),
            ('IT', 'Napoli', 'NA', None, None, ),
            ('IT', 'Lainate', 'BUS', None, None, ),
            ('IT', 'Bari', 'BA', None, None, ),
            ('IT', 'Belluno', 'BL', None, None, ),
            ('IT', 'Biella', 'BI', None, None, ),
            ('IT', 'l\'Aquila', 'AQ', None, None, ),
            ('IT', 'Livorno', 'LI', None, None, ),
            ('IT', 'Pavia', 'PV', None, None, ),
            ('IT', 'Palermo', 'PA', None, None, ),
            ('IT', 'Frosinone', 'FR', None, None, ),
            ('IT', 'Reggio Emilia', 'RE', None, None, ),
            ('IT', 'Roma', 'RM', None, None, ),
            ('IT', 'Civitanova', 'CIV', None, None, ),
            ('IT', 'Sassari', 'SS', None, None, ),
            ('IT', 'Torino', 'TO', None, None, ),
            ('IT', 'Firenze', 'FI', None, None, ),
            ('IT', 'Arezzo', 'AR', None, None, ),
            ('IT', 'Ancona', 'AN', None, None, ),
            ('IT', 'Cagliari', 'CA', None, None, ),
            ('IT', 'Perugia', 'PG', None, None, ),
            ('IT', 'Rho (MI)', 'IPO', None, None, ),
            ('IT', 'PIACENZA', 'PC', None, None, ),
            ('IT', 'IT Region 1', 'R1', None, None, ),
            ('IT', 'IT Region 2', 'R2', None, None, ),
            ('IT', 'IT Region 3', 'R3', None, None, ),
            ('IT', 'IT Region 4', 'R4', None, None, ),
            ('IT', 'IT Region 5', 'R5', None, None, ),
            ('IT', 'IT Region 6', 'R6', None, None, ),
            ('IT', 'IT Region 7', 'R7', None, None, ),
            ('JP', 'Hiroshima', 'HIJ', None, None, ),
            ('JP', 'Osaka', 'OSA', None, None, ),
            ('JP', 'Tokyo', 'TYO', None, None, ),
            ('JP', 'Nagoya', 'NGO', None, None, ),
            ('JP', 'Yokohama', 'YOK', None, None, ),
            ('JP', 'Urawa', 'URW', None, None, ),
            ('JP', 'Kobe', 'UKB', None, None, ),
            ('JP', 'Fukuoka', 'FUK', None, None, ),
            ('LU', 'Luxembourg', 'LUX', None, None, ),
            ('MY', 'Johorebaru', 'JHB', None, None, ),
            ('MY', 'Penang', 'PEN', None, None, ),
            ('MY', 'Kuala Lumpur', 'KUL', None, None, ),
            ('MY', 'Pangkor', 'PKG', None, None, ),
            ('MY', 'Batu Pahat', 'BPT', None, None, ),
            ('NL', 'Territory 85', 'T85', None, None, ),
            ('NL', 'Territory 10', 'T10', None, None, ),
            ('NL', 'Territory 11', 'T11', None, None, ),
            ('NL', 'Territory 31', 'T31', None, None, ),
            ('NL', 'Territory 50', 'T50', None, None, ),
            ('NL', 'Territory 51', 'T51', None, None, ),
            ('NL', 'Territory 71', 'T71', None, None, ),
            ('NL', 'Territory 52', 'T52', None, None, ),
            ('NL', 'C10', 'C10', None, None, ),
            ('NL', 'C30', 'C30', None, None, ),
            ('NL', 'C50', 'C50', None, None, ),
            ('NL', 'C70', 'C70', None, None, ),
            ('NL', 'Territory 30', 'T30', None, None, ),
            ('NL', 'Territory 40', 'T40', None, None, ),
            ('NL', 'Territory 41', 'T41', None, None, ),
            ('NL', 'Territory 80', 'T80', None, None, ),
            ('NL', 'Territory 15', 'T15', None, None, ),
            ('NL', 'Territory 55', 'T55', None, None, ),
            ('NL', 'Territory 56', 'T56', None, None, ),
            ('NL', 'Territory 70', 'T70', None, None, ),
            ('NL', 'Territory 35', 'T35', None, None, ),
            ('NL', 'Territory 57', 'T57', None, None, ),
            ('NL', 'Territory 75', 'T75', None, None, ),
            ('NL', 'Territory 58', 'T58', None, None, ),
            ('NL', 'Overig', 'OVE', None, None, ),
            ('NL', 'Eindhoven', 'EIN', None, None, ),
            ('NL', 'Schiphol', 'SP8', None, None, ),
            ('NL', 'Amsterdam', 'HE2', None, None, ),
            ('NZ', 'Auckland', 'AKL', None, None, ),
            ('NZ', 'Christchurch', 'CHC', None, None, ),
            ('NZ', 'Dunedin', 'DUD', None, None, ),
            ('NZ', 'Hamilton', 'HLZ', None, None, ),
            ('NZ', 'Napier', 'NPE', None, None, ),
            ('NZ', 'Wellington', 'WLG', None, None, ),
            ('NZ', 'New Plymouth', 'NPL', None, None, ),
            ('NZ', 'Palmerston', 'PMR', None, None, ),
            ('NZ', 'Tauranga', 'TRG', None, None, ),
            ('NO', 'Bergen', 'BGO', None, None, ),
            ('NO', 'Oslo', 'OSL', None, None, ),
            ('NO', 'Stavanger', 'SVG', None, None, ),
            ('PH', 'Manila', 'MNL', None, None, ),
            ('PH', 'Cebu', 'CEB', None, None, ),
            ('PH', 'San Fernando', 'SF6', None, None, ),
            ('PL', 'Szczecin', 'SZZ', None, None, ),
            ('PL', 'Gdansk', 'GDN', None, None, ),
            ('PL', 'Krakow', 'KRK', None, None, ),
            ('PL', 'Katowice', 'KTW', None, None, ),
            ('PL', 'Poznan', 'POZ', None, None, ),
            ('PL', 'Warsaw', 'WAW', None, None, ),
            ('PL', 'Bydgoszcz', 'BZG', None, None, ),
            ('PL', 'Lublin', 'QLU', None, None, ),
            ('PL', 'Lodz', 'QLZ', None, None, ),
            ('PL', 'Rzeszow', 'RZE', None, None, ),
            ('PL', 'Wroclaw', 'WRO', None, None, ),
            ('PL', 'Bialystok', 'QYY', None, None, ),
            ('PL', 'Olsztyn', 'QYO', None, None, ),
            ('PL', 'Radom', 'RA2', None, None, ),
            ('PL', 'Pila', 'PI5', None, None, ),
            ('PL', 'Czestochowa', 'CZW', None, None, ),
            ('PT', 'Lisbon', 'LIS', None, None, ),
            ('PT', 'Porto', 'OPO', None, None, ),
            ('RU', 'St. Petersburg', 'LED', None, None, ),
            ('RU', 'Moscow', 'MOW', None, None, ),
            ('RU', 'Ekaterinburg', 'SVX', None, None, ),
            ('RU', 'Samara', 'KUF', None, None, ),
            ('RU', 'Nizhnij Novgorod', 'GOJ', None, None, ),
            ('RU', 'Novosibirks', 'OVB', None, None, ),
            ('RU', 'Irkutsk', 'IK3', None, None, ),
            ('RU', 'Kaliningrad', 'KG7', None, None, ),
            ('RU', 'Khaborovsk', 'KH6', None, None, ),
            ('RU', 'Krasnodar', 'KR4', None, None, ),
            ('RU', 'Kazan', 'KZ7', None, None, ),
            ('RU', 'Rostov-On-Don', 'RO8', None, None, ),
            ('RU', 'Vladivostok', 'VK3', None, None, ),
            ('SA', 'Dhahran', 'DHA', None, ('DH7', ), ),
            ('SA', 'Jeddah', 'JED', None, None, ),
            ('SA', 'Riyadh', 'RUH', None, None, ),
            ('SG', 'Singapore', 'SIN', None, None, ),
            ('ZA', 'Bloemfontein', 'BFN', None, None, ),
            ('ZA', 'Capetown', 'CPT', None, ('ML8', ), ),
            ('ZA', 'Durban', 'DUR', None, None, ),
            ('ZA', 'East London', 'ELS', None, None, ),
            ('ZA', 'Hulls', 'JHO', None, None, ),
            ('ZA', 'Johannesburg', 'JNB', None, ('EF2', ), ),
            ('ZA', 'Port Elizabeth', 'PLZ', None, None, ),
            ('ZA', 'Pretoria', 'PRY', None, None, ),
            ('ZA', 'Pietermarisburg', 'PZB', None, None, ),
            ('ZA', 'Richards b', 'RCB', None, None, ),
            ('ZA', 'S Africa others', 'SXO', None, None, ),
            ('KR', 'Incheon', 'JCN', None, None, ),
            ('KR', 'Seoul', 'SEL', None, None, ),
            ('KR', 'Pusan', 'PUS', None, None, ),
            ('ES', 'Malaga', 'AGP', None, None, ),
            ('ES', 'Barcelona', 'BCN', None, None, ),
            ('ES', 'Alicante', 'ALC', None, None, ),
            ('ES', 'Madrid', 'MAD', None, None, ),
            ('ES', 'Palma de Mallorca', 'PMI', None, None, ),
            ('ES', 'Sevilla', 'SVQ', None, None, ),
            ('ES', 'Vigo', 'VGO', None, None, ),
            ('ES', 'Vittoria', 'VIT', None, None, ),
            ('ES', 'Valencia', 'VLC', None, None, ),
            ('ES', 'Zaragoza', 'ZAZ', None, None, ),
            ('ES', 'Bilbao', 'BIO', None, ('BI6', ), ),
            ('ES', 'Madrid HO', 'EHO', None, None, ),
            ('ES', 'Almeria', 'AL1', None, None, ),
            ('ES', 'Barcelona CIM', 'CIM', None, None, ),
            ('ES', 'Barcelona Hospitalet', 'HOS', None, None, ),
            ('ES', 'Cadiz', 'XR7', None, None, ),
            ('ES', 'Castellon', 'CL3', None, None, ),
            ('ES', 'Gerona', 'GRO', None, None, ),
            ('ES', 'Huelva', 'HL7', None, None, ),
            ('ES', 'Madrid Getafe', 'TF3', None, None, ),
            ('ES', 'Murcia', 'MJV', None, None, ),
            ('ES', 'Pamplona', 'PNA', None, None, ),
            ('ES', 'San Sebastian', 'EAS', None, None, ),
            ('ES', 'Santander', 'SD6', None, None, ),
            ('ES', 'Tarragona', 'QGN', None, None, ),
            ('ES', 'Valladolid', 'EVL', None, None, ),
            ('ES', 'Cordoba Spain', 'ODB', None, None, ),
            ('ES', 'Vilafranca', 'VL4', None, None, ),
            ('ES', 'Lerida', 'QLQ', None, None, ),
            ('SE', 'Goeteborg', 'GOT', None, None, ),
            ('SE', 'Helsingborg', 'HLB', None, None, ),
            ('SE', 'Joenkoeping', 'JKG', None, None, ),
            ('SE', 'Orebro', 'ORB', None, None, ),
            ('SE', 'Stockholm', 'STO', None, None, ),
            ('SE', 'Malmo', 'MMA', None, None, ),
            ('SE', 'Stockholm Others', 'SWO', None, None, ),
            ('CH', 'Basel', 'BSL', None, None, ),
            ('CH', 'Geneva', 'GVA', None, None, ),
            ('CH', 'Mendrisio', 'LUG', None, None, ),
            ('CH', 'Zurich', 'ZRH', None, None, ),
            ('TW', 'Taipei', 'TPY', None, None, ),
            ('TW', 'Taichung', 'TXG', None, None, ),
            ('TH', 'Bangkok', 'BKK', None, None, ),
            ('TR', 'Adana', 'ADA', None, None, ),
            ('TR', 'Ankara', 'ANK', None, None, ),
            ('TR', 'Bursa', 'BTZ', None, None, ),
            ('TR', 'Denizili', 'DNZ', None, None, ),
            ('TR', 'Istanbul', 'IST', None, None, ),
            ('TR', 'Izmir', 'IZM', None, None, ),
            ('TR', 'Antalya', 'AYT', None, None, ),
            ('TR', 'Eskisehir', 'AOE', None, None, ),
            ('TR', 'Kayseri', 'ASR', None, None, ),
            ('TR', 'Giziantep', 'GZT', None, None, ),
            ('TR', 'Iskenderun', 'IS3', None, None, ),
            ('TR', 'Kahramanmaras', 'KH3', None, None, ),
            ('TR', 'Konya', 'KYA', None, None, ),
            ('TR', 'Malatya', 'ML3', None, None, ),
            ('TR', 'Mersin', 'QIN', None, None, ),
            ('AE', 'Abu Dhabi', 'AUH', None, ('AU7', ), ),
            ('AE', 'Dubai', 'DXB', None, ('DX7',), ),
            ('AE', 'Sharjah', 'SHJ', None, None, ),
            ('AE', 'Jebel Ali', 'JBA', None, None, ),
            ('US', 'Boston', 'BOS', None, None, ),
            ('US', 'Baltimore', 'BWI', None, None, ),
            ('US', 'Dallas', 'DFW', None, None, ),
            ('US', 'Detroit', 'DTT', None, None, ),
            ('US', 'Newark', 'EWR', None, None, ),
            ('US', 'Houston', 'IAH', None, None, ),
            ('US', 'Los Angeles', 'LAX', None, None, ),
            ('US', 'Miami', 'MIA', None, None, ),
            ('US', 'Milwaukee', 'MKE', None, None, ),
            ('US', 'New York', 'NYC', None, None, ),
            ('US', 'Chicago', 'ORD', None, None, ),
            ('US', 'Philadelphia', 'PHL', None, None, ),
            ('US', 'Pittsburgh', 'PIT', None, None, ),
            ('US', 'San Francisco', 'SFO', None, None, ),
            ('US', 'Garden City', 'GCY', None, None, ),
            ('US', 'Atlanta', 'AT9', None, None, ),
            ('AU', 'Sydney - McPhee', 'SYM', 'Riteway', None, ),
            ('AU', 'Melbourne - McPhee', 'MEM', 'Riteway', None, ),
            ('AU', 'Brisbane - McPhee', 'BNM', 'Riteway', None, ),
            ('AU', 'Adelaide - McPhee', 'ADM', 'Riteway', None, ),
            ('AU', 'Perth - McPhee', 'PEM', 'Riteway', None, ),
            ('AU', 'Queensland', 'QLD', 'Riteway', None, ),
            ('AU', 'Victoria - McPhee', 'VIP', 'Riteway', None, ),
            ('AU', 'South Australia', 'SAU', 'Riteway', None, ),
            ('AU', 'West Australia', 'WAU', 'Riteway', None, ),
            ('AU', 'New South Wales', 'NSW', 'Riteway', None, ),
            ('AU', 'Riteway Adelaide', 'AN5', 'Riteway', None, ),
            ('AU', 'Riteway Brisbane', 'BR4', 'Riteway', None, ),
            ('AU', 'Riteway Melbourne', 'MR3', 'Riteway', None, ),
            ('AU', 'Riteway Perth', 'PR6', 'Riteway', None, ),
            ('AU', 'Riteway Sydney', 'SR2', 'Riteway', None, ),
            ('VN', 'Ho Chi Minh City', 'SGN', None, None, ),
            ('VN', 'Hanoi', 'HAN', None, None, ),
            ('IL', 'Tel Aviv', 'TLV', None, None, ),
            ('IL', 'Be\'er Sheva', 'EHS', None, None, ),
            ('IL', 'Haifa', 'HFA', None, None, ),
            ('IL', 'Jerusalem', 'JRS', None, None, ),
            ('IL', 'Israel Others', 'TVO', None, None, ),
            ('BG', 'Sofia', 'SOF', None, None, ),
            ('BG', 'Bulgaria Others', 'BUO', None, None, ),
            ('BG', 'Burgas', 'BG3', None, None, ),
            ('BG', 'Plovdiv', 'PLO', None, None, ),
            ('BG', 'Sevlievo', 'SV3', None, None, ),
            ('BG', 'Varna', 'VAR', None, None, ),
            ('LT', 'Vilnius', 'VNO', None, None, ),
            ('LT', 'Lithuania Others', 'LTO', None, None, ),
            ('LT', 'Kaunas', 'MA', None, None, ),
            ('LT', 'Klaipeda', 'KLA', None, None, ),
            ('LT', 'Kaunas 09', 'KUN', None, None, ),
            ('LV', 'Riga', 'RIX', None, None, ),
            ('RO', 'Bucharest', 'BUH', None, None, ),
            ('RO', 'Romania Others', 'RXO', None, None, ),
            ('RO', 'Timisoara', 'TSR', None, None, ),
            ('RO', 'Cluj', 'CLJ', None, None, ),
            ('RO', 'Constanta', 'CND', None, None, ),
            ('RO', 'Brasov', 'BS4', None, None, ),
            ('RO', 'Craiova', 'CRA', None, None, ),
            ('RO', 'Iasi', 'IAS', None, None, ),
            ('RO', 'Ploiesti', 'QPL', None, None, ),
            ('SK', 'Bratislava', 'BTS', None, None, ),
            ('SK', 'Slovakia Others', 'BTO', None, None, ),
            ('EG', 'Alexandria', 'ALY', None, None, ),
            ('EG', 'Cairo', 'CAI', None, None, ),
            ('KW', 'Kuwait', 'KWI', None, None, ),
            ('BH', 'Bahrain', 'BAH', None, ('BH5', ), ),
            ('JO', 'Amman', 'AMM', None, None, ),
            ('JO', 'Aquaba', 'AQ7', None, None, ),
            ('FJ', 'Nadi', 'NAN', None, None, ),
            ('FJ', 'Suva', 'SUV', None, None, ),
            ('KE', 'Mombasa', 'MBA', None, None, ),
            ('KE', 'Nairobi', 'NBO', None, None, ),
            ('CY', 'Nicosia', 'NIC', None, None, ),
            ('CY', 'Limassol', 'LMS', None, None, ),
            ('SI', 'Ljubljana', 'LJU', None, None, ),
            ('DE', 'XX - DEdepot', 'F03', 'Fashion', None, ),
            #('FOT', 'XX - Othdepot', 'F04', None, None, ),
            ('NL', 'XX - Oldenzaal', 'F01', 'Fashion', None, ),
            ('UK', 'XX - Leicester', 'F02', 'Fashion', None, ),
            ('OM', 'Muscat', 'MCT', None, None, ),
            ('LK', 'Colombo', 'CMB', None, None, ),
            ('BD', 'Dhaka', 'DAC', None, None, ),
            ('BD', 'Chittagong', 'CGP', None, None, ),
            ('UA', 'Kiev', 'IEV', None, None, ),
        ]

        group = tnt_domain.default_group
        current_depots = {}
        for d in self.sa_session.query(TntDepot).all():
            current_depots[d.default_code] = d
        for da in self.sa_session.query(TntDepotAlias).all():
            current_depots[da.code] = da.depot

        countries = {}
        for c in self.sa_session.query(Country).all():
            countries[c.iso_3166_a2] = c

        def add_aliases(current_depots, aliases, depot):
            if aliases is None:
                return
            for alias in aliases:
                if current_depots.has_key(alias):
                    continue
                da = TntDepotAlias()
                da.code = alias
                da.depot = depot
                self.sa_session.add(da)
                current_depots[alias] = depot

        for d in depots:
            cc, label, code, service, aliases = d
            if current_depots.has_key(code):
                add_aliases(current_depots, aliases, current_depots[code])
                continue
            if not countries.has_key(cc):
                continue

            depot = TntDepot()
            depot.default_code = code
            depot.label = label
            depot.country = countries[cc]
            if service is not None:
                depot.service = self.service_map[service]

            current_depots[code] = depot
            add_aliases(current_depots, aliases, depot)

            object = depot.get_group_object()[0]
            for privilege in depot.all_privileges:
                go = object()
                go.group = group
                go.privilege = privilege
                go.group_has_privilege = True
                go.license_has_privilege = True
                self.sa_session.add(go)
                depot.groups.append(go)

            self.sa_session.add(depot)

    @transaction_decorator
    def create_geolevels(self, tnt_domain):
        geolevels = [
        # ------ BUSINESS UNITS
            {'name': '1 - 2011 - Americas', 'type': 'BusinessUnit',
             'countries': []},

            {'name': '1 - 2011 - ASPAC', 'type': 'BusinessUnit',
             'countries': []},

            {'name': '1 - 2011 - Northern Europe', 'type': 'BusinessUnit',
             'countries': ['Germany', 'Switzerland', 'Austria']},

            {'name': '1 - 2011 - Southern Europe', 'type': 'BusinessUnit',
             'countries': ['Italy', 'Turkey', 'Oman', 'France']},

            {'name': '1. AMEA BU', 'type': 'BusinessUnit',
             'countries': []},

            {'name': '2. CMI2009 - ASIA PACIFIC', 'type': 'BusinessUnit',
             'countries': []},

            {'name': '3. CMI2009 - C-E EUROPE', 'type': 'BusinessUnit',
             'countries': []},

            {'name': '4. CMI2009 - SOUTHERN EUROPE', 'type': 'BusinessUnit',
             'countries': ['Italy', 'France', 'Turkey']},

            {'name': 'CMI - Asia', 'type': 'BusinessUnit',
             'countries': []},

            {'name': 'CMI - APAC', 'type': 'BusinessUnit',
             'countries': []},

            {'name': 'CMI - BRIC', 'type': 'BusinessUnit',
             'countries': ['Russia', 'India', 'China', 'Brazil']},

            {'name': 'CMI - CPB', 'type': 'BusinessUnit',
             'countries': ['Switzerland', 'Netherlands', 'Italy', 'Germany', 'France', 'Belgium', 'United Kingdom']},

            {'name': 'XX - C-E Europe wo Austria', 'type': 'BusinessUnit',
             'countries': ['Estonia', 'Switzerland', 'Latvia', 'Denmark', 'Czech Republic', 'Belgium', 'Hungary', 'Germany',
                           'Finland', 'Sweden', 'Slovakia', 'Russia', 'Romania', 'Poland', 'Norway', 'Netherlands',
                           'Luxembourg', 'Lithuania']},

            {'name': '5. UK/IE BU', 'type': 'BusinessUnit',
             'countries': ['United Kingdom', 'Ireland']},

            {'name': '99. TNT Associates', 'type': 'BusinessUnit',
             'countries': ['Ukraine', 'Sri Lanka', 'Oman', 'Bangladesh']},

            # ------ REGIONS
            {'name': 'AM - Americas 09', 'type': 'Region',
             'countries': [],
             'parents': ['1. AMEA BU']},

            {'name': 'AS - AU-NZ - Pacifics 09', 'type': 'Region',
             'countries': ['New Zealand', 'Fiji', 'Australia'],
             'parents': ['1 - 2011 - ASPAC', '2. CMI2009 - ASIA PACIFIC']},

            {'name': 'AS - North Asia 2009', 'type': 'Region',
             'countries': [],
             'parents': ['1 - 2011 - ASPAC', '2. CMI2009 - ASIA PACIFIC', 'CMI - Asia']},

            {'name': 'AS - S-E Asia - IN 2009', 'type': 'Region',
             'countries': ['India'],
             'parents': ['1 - 2011 - ASPAC', '2. CMI2009 - ASIA PACIFIC', 'CMI - Asia']},

            {'name': 'CE - Benelux', 'type': 'Region',
             'countries': ['Netherlands'],
             'parents': ['1 - 2011 - Northern Europe', '3. CMI2009 - C-E EUROPE']},

            {'name': 'CE - E Europe Region 09', 'type': 'Region',
             'countries': ['Poland', 'Lithuania', 'Latvia', 'Hungary', 'Slovakia', 'Russia', 'Romania', 'Estonia', 'Czech Republic'],
             'parents': ['1 - 2011 - Northern Europe', '3. CMI2009 - C-E EUROPE']},

            {'name': 'CE - Nordic region 09', 'type': 'Region',
             'countries': ['Sweden', 'Finland', 'Norway', 'Denmark'],
             'parents': ['1 - 2011 - Northern Europe', '3. CMI2009 - C-E EUROPE']},

            {'name': 'CE - UK-IE', 'type': 'Region',
             'countries': ['Ireland', 'United Kingdom'],
             'parents': ['1 - 2011 - Northern Europe']},

            {'name': 'SE - AM - MEA 09', 'type': 'Region',
             'countries': [],
             'parents': ['1 - 2011 - Southern Europe', '1. AMEA BU']},

            # ------ SUBREGIONS
            {'name': 'AM - Latin America CL-AR', 'type': 'Subregion',
             'countries': ['Argentina', 'Chile'],
             'parents': ['1 - 2011 - Americas', 'AM - Americas 09']},

            {'name': 'AM - North America 09', 'type': 'Subregion',
             'countries': ['USA', 'Canada'],
             'parents': ['1 - 2011 - Americas', 'AM - Americas 09']},

            {'name': 'AM - South America 09', 'type': 'Subregion',
             'countries': ['Chile', 'Brazil', 'Argentina'],
             'parents': ['1 - 2011 - Americas', 'AM - Americas 09']},

            {'name': 'AS - FJ and NZ combined', 'type': 'Subregion',
             'countries': ['New Zealand', 'Fiji'],
             'parents': ['1 - 2011 - ASPAC', '2. CMI2009 - ASIA PACIFIC']},

            {'name': 'AS - NA - 09 JP-KR combined', 'type': 'Subregion',
             'countries': ['Republic of Korea', 'Japan'],
             'parents': ['CMI - APAC', 'AS - North Asia 2009']},

            {'name': 'AS - NA - Gr China 2009', 'type': 'Subregion',
             'countries': ['Taiwan', 'Hong Kong', 'China'],
             'parents': ['AS - North Asia 2009']},

            {'name': "AS - South-East Asia ('07) wo India", 'type': 'Subregion',
             'countries': ['Vietnam', 'Thailand', 'Singapore', 'Philippines', 'Malaysia', 'Indonesia'],
             'parents': ['CMI - APAC', 'AS - S-E Asia - IN 2009']},

            {'name': 'CE - Baltics 09', 'type': 'Subregion',
             'countries': ['Lithuania', 'Latvia', 'Estonia'],
             'parents': ['1 - 2011 - Northern Europe', '3. CMI2009 - C-E EUROPE']},

            {'name': 'CE - BeLux 09', 'type': 'Subregion',
             'countries': ['Luxembourg', 'Belgium'],
             'parents': ['CE - Benelux']},

            {'name': 'SE - AM - Africa2011', 'type': 'Subregion',
             'countries': ['South Africa', 'Kenya', 'Egypt'],
             'parents': ['SE - AM - MEA 09']},

            {'name': 'SE - AM - ME2011', 'type': 'Subregion',
             'countries': ['United Arab Emirates', 'Saudi Arabia', 'Kuwait', 'Jordan', 'Bahrain'],
             'parents': ['SE - AM - MEA 09']},

            {'name': 'SE - AM - Saudi - BH', 'type': 'Subregion',
             'countries': ['Saudi Arabia', 'Bahrain'],
             'parents': ['SE - AM - MEA 09']},

            {'name': 'SE - Greece - Balkins 09', 'type': 'Subregion',
             'countries': ['Israel', 'Greece', 'Bulgaria'],
             'parents': ['4. CMI2009 - SOUTHERN EUROPE', '1 - 2011 - Southern Europe']},

            {'name': 'SE - Iberia 09', 'type': 'Subregion',
             'countries': ['Spain', 'Portugal'],
             'parents': ['4. CMI2009 - SOUTHERN EUROPE', '1 - 2011 - Southern Europe']},

            {'name': 'SE - S-E Europe 09 - new', 'type': 'Subregion',
             'countries': ['Slovenia', 'Israel', 'Greece', 'Cyprus', 'Bulgaria'],
             'parents': ['4. CMI2009 - SOUTHERN EUROPE', '1 - 2011 - Southern Europe']},
        ]

        country_objs = {}
        type_objs = {}
        geolevel_objs = {}

        types = ['BusinessUnit', 'Region', 'Subregion']
        for t in types:
            tt = self.sa_session.query(TntGeolevelType).filter(TntGeolevelType.name==t).all()
            if len(tt) > 0:
                type_objs[t] = tt[0]
            else:
                tt = TntGeolevelType()
                tt.name = t
                self.sa_session.add(tt)
                type_objs[t] = tt

        # add privileges for tnt
        group = tnt_domain.default_group

        for geolevel in geolevels:
            g = self.sa_session.query(TntGeolevel).filter(TntGeolevel.name==geolevel['name']).all()
            if len(g) > 0:
                continue
            if not type_objs.has_key(geolevel['type']):
                raise Exception('Invalid type %s' % geolevel['type'])

            g = TntGeolevel()
            g.name = geolevel['name']
            g.type = type_objs[geolevel['type']]

            if geolevel.has_key('parents') and len(geolevel['parents']) > 0:
                parents = []
                for p in geolevel['parents']:
                    if not geolevel_objs.has_key(p):
                        raise Exception('Invalid parent %s' % p)
                    parents.append(geolevel_objs[p])
                g.parent_geolevels = parents

            if geolevel.has_key('countries') and len(geolevel['countries']) > 0:
                countries = []
                for c in geolevel['countries']:
                    if not country_objs.has_key(c):
                        co = self.sa_session.query(Country).filter(Country.name==c).all()
                        if len(co) == 0:
                            co = self.sa_session.query(DomainCountry).filter(and_(
                                DomainCountry.domain==tnt_domain, DomainCountry.name==c)).all()
                            if len(co) == 0:
                                raise Exception('Invalid country %s' % c)
                            co = co[0].country
                        else:
                            co = co[0]
                        country_objs[c] = co
                    countries.append(country_objs[c])
                g.countries = countries

            object = g.get_group_object()[0]
            for privilege in g.all_privileges:
                go = object()
                go.group = group
                go.privilege = privilege
                go.group_has_privilege = True
                go.license_has_privilege = True
                self.sa_session.add(go)
                g.groups.append(go)

            self.sa_session.add(g)
            geolevel_objs[geolevel['name']] = g

    @transaction_decorator
    def sync_tnt_domain_countries(self, tnt_domain):
        countries = {
            'AR': 'Argentina',
            'AU': 'Australia',
            'AT': 'Austria',
            'BH': 'Bahrain',
            'BD': 'Bangladesh',
            'BE': 'Belgium',
            'BR': 'Brazil',
            'BG': 'Bulgaria',
            'CA': 'Canada',
            'CL': 'Chile',
            'CN': 'China',
            'CY': 'Cyprus',
            'CZ': 'Czech Republic',
            'DK': 'Denmark',
            'EG': 'Egypt',
            'EE': 'Estonia',
            'FJ': 'Fiji',
            'FI': 'Finland',
            'FR': 'France',
            'DE': 'Germany',
            'GR': 'Greece',
            'HK': 'Hong Kong',
            'HU': 'Hungary',
            'IN': 'India',
            'ID': 'Indonesia',
            'IE': 'Ireland',
            'IL': 'Israel',
            'IT': 'Italy',
            'JP': 'Japan',
            'JO': 'Jordan',
            'KE': 'Kenya',
            'KR': 'Korea, Republic of ',
            'KW': 'Kuwait',
            'LV': 'Latvia',
            'LT': 'Lithuania',
            'LU': 'Luxembourg',
            'MY': 'Malaysia',
            'NL': 'Netherlands',
            'NZ': 'New Zealand',
            'NO': 'Norway',
            'OM': 'Oman',
            'PH': 'Philippines',
            'PL': 'Poland',
            'PT': 'Portugal',
            'RO': 'Romania',
            'RU': 'Russia',
            'SA': 'Saudi Arabia',
            'SG': 'Singapore',
            'SK': 'Slovakia',
            'SI': 'Slovenia',
            'ZA': 'South Africa',
            'ES': 'Spain',
            'LK': 'Sri Lanka',
            'SE': 'Sweden',
            'CH': 'Switzerland',
            'TW': 'Taiwan',
            'TH': 'Thailand',
            'TR': 'Turkey',
            'UA': 'Ukraine',
            'AE': 'United Arab Emirates',
            'GB': 'United Kingdom',
            'US': 'USA',
            'VN': 'Vietnam',
        }

        active_countries = [country_code for country_code, domain_country_name in countries.iteritems()]

        # loop all countries and disable the ones that arent in active_countries (and not already de-activated)
        disc = 0
        for country in self.sa_session.query(Country).all():

            if not country.iso_3166_a2 in active_countries:

                try:
                    dc = self.sa_session.query(DomainCountry) \
                        .join(Country) \
                        .filter(DomainCountry.domain == tnt_domain) \
                        .filter(Country.iso_3166_a2 == country.iso_3166_a2).one()
                except NoResultFound:
                    disc = disc + 1
                    dc = DomainCountry()
                    dc.domain = tnt_domain
                    dc.country = country
                    dc.active = False
                    self.sa_session.add(dc)
        #print 'TNT::db_sync: disabling %s tnt countries' % disc

        # override tnt countries that have different naming
        for country_code, domain_country_name in countries.iteritems():

            dc = self.sa_session.query(DomainCountry) \
                .join(Country) \
                .filter(DomainCountry.domain == tnt_domain) \
                .filter(Country.iso_3166_a2 == country_code).all()
            if len(dc) > 0: # already in
                continue

            # check if related country exists
            try:
                country = self.sa_session.query(Country).filter(Country.iso_3166_a2==country_code).one()
            except NoResultFound:
                raise Exception('Could not find country for code %s' % country_code)

            # only override name if it differes from the iso country name
            if country.name == domain_country_name:
                continue

            #print 'TNT::db_sync: inserting tnt country for %s' % country_code
            dc = DomainCountry()
            dc.domain = tnt_domain
            dc.name = domain_country_name

            # update country
            dc.country = country
            dc.active = True

            # save or update
            self.sa_session.add(dc)

    def populate_tnt_account(self):
        # TNT CLM license
        tnt_license = self.sa_session.query(License).filter(License.name == 'TNT CLM').all()
        if len(tnt_license) == 0:
            from basesoft.web.contrib.license.controllers import LicenseController
            from basesoft.web.contrib.license.models import Domain

            permissions = self.sa_session.query(Permission).filter(Permission.name.in_([
               "view_log", "view_login_overview", "kill_session", "file_manager", "view_file",
               "create_file", "update_file", "delete_file", "view_contact", "create_contact", "update_contact", "delete_contact",
               "import_contacts", "view_contact_list", "create_contact_list", "update_contact_list", "delete_contact_list",
               "link_to_contact_list", "unlink_from_contact_list", "distribute_group_permissions", "view_license", "create_mail",
               "view_mail", "update_mail", "delete_mail", "schedule_mail", "view_mailing_results", "view_dashboard", "save_dashboard",
               "create_account", "update_account", "delete_account", "account_config_manager", "create_account_mail",
               "view_account_mail", "update_account_mail", "delete_account_mail", "create_notification", "view_notification",
               "update_notification", "delete_notification", "send_to_license_default_group", "send_to_domain_default_group",
               "branding_view_header_footer", "branding_create_header_footer", "branding_update_header_footer", "branding_delete_header_footer",
               "view_datasource", "create_geolevel", "view_geolevel", "update_geolevel", "delete_geolevel", "geolevel_manager", "view_depot",
               "create_depot", "update_depot", "delete_depot", "depot_manager", "create_depot_alias ", "tnt_clm_wave_manager",
               "tnt_clm_attribute_manager", "create_tnt_clm_attribute", "view_tnt_clm_attribute", "update_tnt_clm_attribute", "delete_tnt_clm_attribute",
               "merge_tnt_clm_attribute", "translate_tnt_clm_attribute", "create_tnt_clm_scale_answer", "update_tnt_clm_scale_answer",
               "delete_tnt_clm_scale_answer", "view_tnt_clm_scale_answer", "create_tnt_clm_wave", "update_tnt_clm_wave",
               "delete_tnt_clm_wave", "view_tnt_clm_wave", "translate_tnt_clm_wave", "view_tnt_clm_wave_translation_history",
               "approve_tnt_clm_translations", "set_wave_global_attribute_priority", "create_tnt_clm_wave_type", "update_tnt_clm_wave_type",
               "delete_tnt_clm_wave_type", "view_tnt_clm_wave_type", "tnt_clm_locale_manager", "create_tnt_clm_locale", "tnt_clm_contact_access",
               "create_tnt_wave_mail", "view_tnt_wave_mail", "update_tnt_wave_mail", "delete_tnt_wave_mail", "approve_tnt_clm_wave_mail_translations",
               "set_custom_query_preset", "schedule_tnt_clm_wave", "view_wave_responses", "update_tnt_clm_locale",
               "tnt_view_business_function", "tnt_create_business_function", "tnt_manage_global_business_function",
               "create_calculated_member", "view_calculated_member", 'view_datasource_widget',

               # fixme: we shouldnt need to give report permissions to be able to flush a datasource
               "create_report", "view_report", "update_datasource", "view_datasource", "datasource_manager", "manage_mdx_methods"

               ])).all()

            lc = LicenseController(self.sa_session, account_id=self.get_account().id)
            domain = self.sa_session.query(Domain).filter(Domain.name == 'TNT').all()
            if len(domain) == 0:
                domain = lc.save_or_update_domain(None, 'TNT')
            else:
                domain = domain[0]
            contact = Person()
            contact.last_name = 'DDK'
            tnt_license = lc.save_or_update_license(None, 'TNT CLM', domain, License, contact, None, None, True)
            tnt_license.permissions = permissions
            tnt_group = lc.save_or_update_group(None, 'Tnt Admins', tnt_license, is_default=True)
            tnt_group.permissions = permissions

            # run all the db_sync cuz they check for tnt license
            from iqportal.tnt.core import db_sync as core_sync
            from iqportal.tnt.clm import db_sync as clm_sync

            core_sync.sync(self.sa_session)
            clm_sync.sync(self.sa_session)

            self.create_services()
            self.create_locales()
            self.create_depots(domain)
            self.sync_tnt_domain_countries(domain)
            self.create_geolevels(domain)

            self.import_users(domain)
            self.sa_session.flush()
        else:
            tnt_group = tnt_license[0].default_group
            tnt_license = tnt_license[0]

        # make a tnt account which has DDK perms
        tnt_account = self.sa_session.query(Account).filter(Account.name == 'tnt_account_test').all()
        if len(tnt_account) == 0:
            from basesoft.web.contrib.account.controllers import AccountController
            tnt_account = Account()
            tnt_account.name = 'tnt_account_test'
            tnt_account.password = AccountController(self.sa_session).hash_password('tnt_password')
            tnt_account.person = Person()
            self.sa_session.add(tnt_account)

            # make the link between account and group.
            ga = GroupAccount()
            ga.is_default = 1
            ga.active = True
            ga.account = tnt_account
            ga.group = tnt_group

            self.sa_session.add(ga)
            self.sa_session.flush()
        else:
            tnt_account = tnt_account[0]

        return tnt_license, tnt_account

    def populate_for_reports(self):
        """
        Adds data for tnt reports: contacts for different locales, TNT CLM license, tnt accounts, wave responses
        """

        # start filling
        tnt_license, tnt_account = self.populate_tnt_account()
        tnt_group = tnt_license.default_group

        # make the TNT-LM datasource available for all tnt license (add view privileges)
        from iqportal.iqube.core.models import DataSource
        from sqlalchemy import func
        datasource = self.sa_session.query(DataSource).filter(func.substr(DataSource.name, -7) == 'TNT-CLM').all()
        if len(datasource) > 0:
            datasource = datasource[0]
            self.grant_privilege_to(datasource, datasource.PRIV_VIEW, group=False, license=True, domain=False, target_group=tnt_group)

        # all next tnt related thing will be saved using this new account
        self._account = tnt_account
        self._account_default_group = tnt_group
        self._active_account_groups = [tnt_group]

        agg = TntCLMTntCLMWaveAggregator()
        # now start filling
        wave = None
        for i in range(5):

            logger.info('creating wave_%s' % i)

            if wave is None:

                # make a wave
                wave_id = self.create_wave_36(wave_name='wave_%s' % i)
                wave = self.sa_session.query(TntCLMWave).filter(TntCLMWave.id == wave_id).one()

                # add translations
                locales = self.sa_session.query(TntCLMLocale).all()
                for locale in locales:
                    if locale.label != 'XXENI':
                        self.translate(wave, locale, set_approved=True)
            elif i < 4:
                self.sa_session.expire(wave)

                wc = WaveController(self.sa_session, account_id=self.get_account().id)
                wave = wc.clone_wave(wave.id)

                wave.name = 'wave_%s' % i
                wave.code = 'W.%s' % i
                wave.is_main = ((i+1) % 2)
                self.sa_session.add(wave)
                self.sa_session.flush()

                wave_id = wave.id
            else:
                # make a 40a wave
                wave_id = self.create_wave_40a(wave_name='wave_%s' % i, create_in_pool=False)
                wave = self.sa_session.query(TntCLMWave).filter(TntCLMWave.id == wave_id).one()

                # add translations
                locales = self.sa_session.query(TntCLMLocale).all()
                for locale in locales:
                    if locale.label != 'XXENI':
                        self.translate(wave, locale, set_approved=True)

            # add priv to view wave responses for the tnt accounts group
            g = self.sa_session.query(Group).filter(Group.name == 'Report managers').all()
            if len(g) > 0:
                self.grant_privilege_to(wave, [wave.PRIV_VIEW, wave.PRIV_VIEW_RESPONSE],
                                        group=True, license=False, domain=False, target_group=g[0])

            # make contact list
            contact_list = self.generateContactList(wave_id, 75)

            #set the wave online
            wave.status = TntCLMWave.STATUS_ONLINE
            wave.contact_lists.append(contact_list)
            self.sa_session.add(wave)
            self.sa_session.flush()
            agg.run(wave.id)

            # set wave invitees
            response_controller = ResponseController(self.sa_session)
            response_controller.save_or_update_invitees(wave)

            # generate some responses
            self.generateResponses(wave_id, None, False, factor=i)

            # workaround for #2568.. re-run db_sync for tnt report so that aggTntCLMTntCLMWave gets filled for the aggregator
            from iqportal.tnt.report import db_sync
            db_sync.sync(self.sa_session)

            # run aggregator
            # dont refresh because the datasource doesnt exist yet.. iqubesync will run after this
            from iqportal.tnt.report.schedule import AggregateResponsesTask
            t = AggregateResponsesTask()
            t.execute(refresh=False)


