'''
Created on Feb 2, 2015

@author: rado
'''
from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase
from iqportal.tnt.clm.models import TntCLMWave
from iqportal.tnt.report.controllers import TntCLMTntCLMWaveAggregator
from iqportal.tnt.test.controllers import TntWaveController
from iqportal.tnt.clm.controllers import TntCLMLocaleController
from basesoft.web.contrib.i18n.models import Country
from iqportal.tnt.report.schedule import AggregateResponsesTask
from iqportal.tnt.response.controllers import ResponseController
from iqportal.iqube.core.models import DataSource, DataSourceWidget,\
    DataSourceCube
from django.conf import settings
from iqportal.iqube.core.controllers import ProjectController
from iqportal.tnt.core.models import TntDepot
from basesoft.web.contrib.license.models import License
from basesoft.web.contrib.contact.controllers import ContactController

class TestCase(SeleniumTestCase):

    def setUp(self):
        super(TestCase, self).setUp()

        self.connect_db()
        self.account = self._create_active_account(license=self.sa_session.query(License).filter(License.name=='TNT CLM').first())
        self.group = self.account.groups[0].group
        self._attach_permissions_to_group(self.group, 'view_dashboard', 'save_dashboard', 'view_datasource', 'create_datasource',
                                          'create_tnt_clm_wave', 'view_tnt_clm_wave', 'view_geolevel',
                                           'create_tnt_clm_locale', 'view_wave_responses', 'create_contact', 'view_depot',
                                           'create_contact_list', 'update_contact_list', 'link_to_contact_list', 'tnt_view_business_function',
                                           'tnt_manage_global_business_function', 'view_datasource_widget', 'manage_mdx_methods')

    def test_location_2015_dimension(self):
        wave_id = self._create_complete_wave_36()
        # login with account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # setup compose
        self.expand_layout_pane(0)
        self.wait_for_loading()

        self.click_palette('Mode', palette_index=0)
        self.click_palette('Compose', palette_index=0)
        self.wait_for_loading()

        # drag widget
        self.click_palette('CLM Report', palette_index=2)
        self.drag_item_from_palette('Satisfaction Table (new)', block_index=0, column_index=0)

        # wait 2 seconds for drag and drop to finish
        import time
        time.sleep(2)
        self.assert_growl_errors_present()

        # xpath to find Settings button on the selected action palette... use empty text because of unknown reason
        palette = "(//div[contains(@class, 'compose-right')]/div[contains(@class, 'bs-collapse-module')])[2]//span[text()]"
        settings = self._driver.find_element_by_xpath(palette)
        settings.click()

        # use sleep because growl is present
        time.sleep(2)

        # select option to hide empty fields
        self.fill_form(panel_name='config-widget-panel', values={'hide_empty_fields': ('checkbox', True)})
        # open location_2015 dimension
        self.click_fieldset(panel_name='config-widget-panel', title='Location 2015')
        self.fill_form(panel_name='config-widget-panel', values={'dimension_10': ('fancy-select', ('D', 'I')), 'axes_10': ('select', 'Columns')})
        self.click_fieldset(panel_name='config-widget-panel', title='Location 2015')

        # select the wave
        self.click_fieldset(panel_name='config-widget-panel', title='Wave')
        self.click_button(panel_name='config-widget-panel', text='Deselect all')
        self.fill_form(panel_name='config-widget-panel', values={'dimension_7': ('fancy-select', ['W.%s' % wave_id])})
        self.click_fieldset(panel_name='config-widget-panel', title='Wave')

        # deselect Attribute
        self.click_fieldset(panel_name='config-widget-panel', title='Attribute')
        self.click_button(panel_name='config-widget-panel', text='Deselect all')
        self.fill_form(panel_name='config-widget-panel', values={'axes_0': ('select', '-')})
        self.click_fieldset(panel_name='config-widget-panel', title='Attribute')

        # select all KPI
        self.click_fieldset(panel_name='config-widget-panel', title='KPI')
        self.click_button(panel_name='config-widget-panel', text='Select all')
        self.click_fieldset(panel_name='config-widget-panel', title='KPI')

        # deselect Service
        self.click_fieldset(panel_name='config-widget-panel', title='Service')
        self.click_button(panel_name='config-widget-panel', text='Deselect all')
        self.fill_form(panel_name='config-widget-panel', values={'axes_6': ('select', '-')})
        # apply
        self.click_button(panel_name='config-widget-panel', text='Apply and close')

        time.sleep(2)

        # assert KPI value for location 2015
        self.assert_cube_table(row_index=1, value=[1, 30], css_selector='table.basesoft-cubetable')


    def _create_complete_wave_36(self):
        c = TntWaveController(self.session, account_id=self.account.id)

        # make a wave
        wave_id = c.create_wave_36(wave_name='wave_random')
        wave = self.sa_session.query(TntCLMWave).filter(TntCLMWave.id == wave_id).one()

        agg = TntCLMTntCLMWaveAggregator()

        # add priv to view wave responses for the tnt accounts group
        c.grant_privilege_to(wave, [wave.PRIV_VIEW, wave.PRIV_VIEW_RESPONSE],
                                group=True, target_group=self.group)
        # grant Depot privileges
        [c.grant_privilege_to(depot, TntDepot.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False) \
            for depot in self.sa_session.query(TntDepot).all()]

        # make contact list
        contact_list = c.generateContactList(wave_id, 1)

        # change country to FR to have same number of responses
        country = self.sa_session.query(Country).filter(Country.name == 'France').one()

        contact_list.contacts[0].company.locale.country = country
        contact_controller = ContactController(self.sa_session, account_id=self.account.id, contact_type='TntCLMContact')
        contact_controller.save_or_update_contact_list(contact_list.id, contact_list.name)

        #set the wave online
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(contact_list)
        self.sa_session.add(wave)
        self.sa_session.flush()
        agg.run(wave.id)

        # set wave invitees
        response_controller = ResponseController(self.sa_session, self.account.id)
        response_controller.save_or_update_invitees(wave)

        # generate some responses
        c.generateResponses(wave_id, None, False)

        # get TNT datasource and flush it
        ds = self.session.query(DataSource).filter(DataSource.name=='%s.TNT-CLM' % settings.SETTINGS_MODULE).first()
        pc = ProjectController(self.sa_session, self.account.id)
        pc.flush_schema(ds.id)

        # run db_sync so that aggTntCLMTntCLMWave gets filled for the aggregator
        from iqportal.tnt.report import db_sync
        db_sync.sync(self.sa_session)

        # grant widget privileges
        [c.grant_privilege_to(widget, DataSourceWidget.all_privileges, group=True, target_group=self.group, check_own_privs=False) \
            for widget in self.sa_session.query(DataSourceWidget).filter(DataSourceWidget.category == 'CLM Report').all()]

        # grant cube privileges
        [c.grant_privilege_to(cube, DataSourceCube.all_privileges, group=True, target_group=self.group, check_own_privs=False) \
            for cube in self.sa_session.query(DataSourceCube).filter(DataSourceCube.name == 'TntCLMVirtualCube').all()]

        # grant ds privileges
        c.grant_privilege_to(ds, DataSource.all_privileges, group=True, target_group=self.group, check_own_privs=False)

        # run aggregator
        t = AggregateResponsesTask()
        t.execute()

        return wave_id