# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
from iqportal.tnt.sr.models import TntSRQualityAuditResponse
import datetime

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException

from iqportal.tnt.sr.controllers import ServiceRecoveryController

class TntSrControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    Test scenarios related to TNT's Service Recovery
    """
    def setUp(self):
        # connect to the db
        self.connect_db()

        # create an account so we can check permission with it
        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.controller = ServiceRecoveryController(self.session, account_id=self.account.id)

    def _get_state(self):
        return {
            'supervisor_name': u'Theodore Roosevelt',
            'auditee': u'Elton John',
            'date_of_audit': datetime.datetime(2010,2,2),
            'country_id': 1,
            'depot': u'DHO',
            'date_case_opened': datetime.datetime(2010,1,1),
            'contact_number': 12312345,
            'contact_name': u'Brigitte Kaandorp',
            'telephone_number': u'0202345435',
            'customer_perspective': {
                'customer_perspective_question1': 5,
                'customer_perspective_question2': 4,
                'customer_perspective_question3': 5,
                'customer_perspective_question4': 2,
                'customer_perspective_question5': 5,
                'customer_perspective_question6': 1,
            },
            'customer_perspective_remark': u'Some remark',
            'data_capture_question1': u'No other feedback',
            'data_capture_question2': {
                'data_capture_question2_a': 2,
                'data_capture_question2_b': 1
            },
            'data_capture_question3': {
                'data_capture_question3_a': 2,
                'data_capture_question3_b': 2,
                'data_capture_question3_c': 0,
                'data_capture_question3_d': 2
            },
            'supervisor_perspective_question1': u'Nothing',
            'supervisor_perspective_question2': 'no',
            'supervisor_perspective_question3': u'No other comments'
        }

    def test_audit_responses(self):

        state = self._get_state()
        # we do not have the correct permissions to save an audit.
        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_audit_response_from_state, None, state)
        self._attach_permissions_to_group(self.group, 'create_tnt_sr_audit_form', 'view_depot', 'create_depot')

        audit = self.controller.save_or_update_audit_response_from_state(None, state)
        self.assertEqual(audit.customer_perspective_score, 22)
        self.assertEqual(audit.data_capture_score, 9)
        self.assertEqual(audit.total_score, 31)
        # assert values

        state['customer_perspective']['customer_perspective_question6'] = 4
        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_audit_response_from_state, audit.id, state)
        self._attach_permissions_to_group(self.group, 'view_tnt_sr_audit_form', 'update_tnt_sr_audit_form')

        audit = self.controller.save_or_update_audit_response_from_state(audit.id, state)
        self.assertEqual(audit.customer_perspective_score, 25)
        self.assertEqual(audit.data_capture_score, 9)
        self.assertEqual(audit.total_score, 34)

        self.controller.mark_as_archived(TntSRQualityAuditResponse, audit.id)
        results = self.controller.get_audit_responses(audit.id)
        self.assertEqual(len(results), 0)
