'''
Created on Sep 2, 2011

@author: martijn
'''

import os

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.account.models import Account
from iqportal.xbrl.core.controllers import XBRLController
from django.core.files.uploadedfile import SimpleUploadedFile
from basesoft.web.contrib.file.controllers import UploadFileController,\
    ProcessFileController
from iqportal.xbrl.core.models import XBRLContextEntityIdentifier,\
    XBRLContextEntity, XBRLPeriod, XBRLContext, XBRLElement, XBRLUnit, XBRLFact



class XbrlTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()
        self.account = self._create_active_account()
        self.group = self.account.groups[0].group
        self._attach_permissions_to_group(self.group, 'create_xbrl_project', 'view_xbrl_project', 'update_xbrl_project')


    def test_hello_world(self):


        sysadmin = self.account

        xsd_content = open(os.path.dirname(__file__) + '/import/hello_world/HelloWorld.xsd').read()

        c = XBRLController(self.session, account_id=sysadmin.id)
        project = c.create_project('hello_world', xsd_content)
        self.assertEquals(self.session.query(XBRLElement) \
            .filter(XBRLElement.project==project).count(), 6)

        c.import_xbrl(project, [ os.path.dirname(__file__) + '/import/hello_world/HelloWorld.xml' ])

        identifiers = self.session.query(XBRLContextEntityIdentifier) \
            .filter(XBRLContextEntityIdentifier.project==project).all()

        self.assertEqual(len(identifiers), 1)
        self.assertEqual(identifiers[0].identifier, 'Example Company')

        self.assertEquals(self.session.query(XBRLContextEntity).filter(XBRLContextEntity.project==project).count(), 1)
        self.assertEquals(self.session.query(XBRLPeriod).filter(XBRLPeriod.project==project).count(), 2)
        self.assertEquals(self.session.query(XBRLContext).filter(XBRLContext.project==project).count(), 2)
        self.assertEquals(self.session.query(XBRLUnit).filter(XBRLUnit.project==project).count(), 1)

        self.assertEquals(self.session.query(XBRLFact) \
            .join(XBRLElement) \
            .filter(XBRLElement.project==project).count(), 12)


    def test_gaap(self):
        """
        comprehensive example from
        http://xbrl.squarespace.com/journal/2008/4/18/some-great-examples-of-xbrl.html
        """

        c = XBRLController(self.session, account_id=self.account.id)

        xsd_content = open(os.path.dirname(__file__) + '/import/gaap/gaap.xsd').read()
        project = c.create_project('gaap', xsd_content)

        c.import_xbrl(project, [ os.path.dirname(__file__) + '/import/gaap/Sample-Instance-Proof.xml' ])

    def test_annual_report(self):
        """
        tests multiple xml files from
        http://jaarverslag.info/xbrl
        """

        c = XBRLController(self.session, account_id=self.account.id)

        xsd_content = open(os.path.dirname(__file__) + '/import/annual_report/ifrs-gp-2005-05-15.xsd').read()
        project = c.create_project('DutchAnnualReports', xsd_content)

        xml_dir = os.path.dirname(__file__) + '/import/annual_report/xml/'
        files = [xml_dir + f for f in os.listdir(xml_dir)]

        c.import_xbrl(project, files)



