# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

import exceptions
from datetime import datetime

from basesoft.web.contrib.i18n import models as i18n_models

from iqportal.iquest.test.unit import iquest_unittest
from iqportal.iquest.compose.controllers import ComposeController, BranchController
from iqportal.iquest.response.controllers import BranchController as ResponseBranchController
from iqportal.iquest.core import models as iquest_models
from iqportal.iquest.response import models as response_models
from iqportal.iquest.response.controllers import ParticipantController

from basesoft import logging
logger = logging.getLogger('basesoft.web.test.unit')

import time

class BranchingSpeedTestCase(iquest_unittest.IQuestTestCase):
    """
    """
    def setUp(self):
        iquest_unittest.IQuestTestCase.setUp(self, [
            'create_survey', 'view_survey', 'update_survey',
            'create_survey_publication', 'schedule_survey_publication',
            'view_conditional_branch', 'create_conditional_branch',
            'view_iquest_pool_category', 'create_iquest_pool_category'])

        locale = self.session.query(i18n_models.Locale).filter_by(locale='nl_NL').one()

        # controller
        self.controller = ComposeController(self.session, locale, account_id=self.account.id)

    def _update_module_ids(self, state, id_map, stripDomID=False):

        if state['objectName'] == 'SurveyItemQA':
            for b in state['branching']:
                for i in ['question', 'answer', 'block']:
                    b[i]['dbID'] = None
                    if b[i]['domID'] is not None:
                        b[i]['dbID'] = id_map[b[i]['domID']]
                    if stripDomID:
                        del b[i]['domID']

        iquest_unittest.IQuestTestCase._update_module_ids(self, state, id_map, stripDomID)


    def test_branching_speed(self):
        """
        """

        state = self._get_state()

        self._dom_id_counter = 0

        def _add_dom_ids(_state, prefix=''):

            #print '_add_dom_ids?'

            if _state.has_key('dbID') and not _state.has_key('domID'):
                #print 'setting dom-id %s' % self._dom_id_counter
                if _state['dbID'] is not None:
                    _state['domID'] = 'dom-%s-%s' % (prefix, _state['dbID'])
                    _state['dbID'] = None
                else:
                    _state['domID'] = None

            if _state.has_key('children'):
                for y in _state['children'].keys():
                    for i in range(len(_state['children'][y])):
                        #print 'processing children %s / %s' % (y, i)
                        _state['children'][y][i] = _add_dom_ids(_state['children'][y][i], y)

            if _state.has_key('branching'):
                for y in _state['branching']:
                    for i in ['question', 'answer', 'block']:
                        #print 'processing children %s / %s' % (y, i)
                        y[i] = _add_dom_ids(y[i], '%ss' % i)

            return _state

        state['survey'] = _add_dom_ids(state['survey'])
        state['survey']['domID'] = 'iquest-canvas-dom-id'

        # publish a survey
        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True, state=state)

        survey = self.session.query(iquest_models.Survey).get(survey_id)

        bc = BranchController(self.sa_session, account_id=self.account.id)
        items = bc.get_branch_cache(survey)

        should_never_be_removed = survey.blocks[1].columns[0].items[0].id

        for bitbucket, unreachable_items in items.iteritems():
            self.assertFalse(should_never_be_removed in unreachable_items)

        branch = survey.blocks[20].branches[0]

        unreachable_items = []

        for i in range(2,20):
            for c in survey.blocks[i].columns:
                [unreachable_items.append(i.id) for i in c.items]

        # validate the unreachable items for the first branch (it skips to page 21)
        self.assertEqual(items['branch_%s' % branch.id], unreachable_items)

        responses = []

        items = survey.itemQAs

        qa = items[0]
        responses.append({
            'item_id': qa.id,
            'responses': [{
                'q': qa.questions[0].id,
                'a': qa.answers[0].id,
                'v': True
            }]
        })

        pc = ParticipantController(self.session, None, invitee_hash=self._publication.anonymous_hash)
        pc.load_survey_state() # call this to store the caches.

        time_a = time.time()
        rv = pc.update_responses(responses, blocks=[survey.blocks[1].id])
        logger.info('update_responses for survey took %s seconds' % (time.time() - time_a) )


        time_a = time.time()
        bc = ResponseBranchController(self.sa_session, responses)
        x =  {'next_blocks': bc.get_next_blocks([survey.blocks[1]])}
        logger.info('get_next_blocks for block %s took %s seconds' % (survey.blocks[1], time.time() - time_a) )

    def _get_state(self):
        """
        """
        return {"survey": {"fallbackLocale": {"locale": "nl_NL", "name": "Dutch"}, "has_active_publications": True, "theme": {"cssClass": "survey-basesoft", "name": "BaseSoft", "module": "iquest_survey_theme_basesoft"}, "name": "Politie", "objectName": "Survey", "dbID": 63, "alter_published_survey": True, "children": {"blocks": [{"dbID": 422, "children": {"columns": [{"dbID": 446, "children": {"items": [{"dbID": 883, "poolItemID": None, "locales": {"nl_NL": {"html": u''}}, "objectName": "SurveyItemHtml"}, {"dbID": 884, "poolItemID": None, "locales": {"nl_NL": {"html": u''}}, "objectName": "SurveyItemHtml"}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 423, "children": {"columns": [{"dbID": 447, "children": {"items": [{"branching": [{"answer": {"dbID": 3525}, "question": {"dbID": 3524}, "block": {"dbID": 442}}], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 885, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3525, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3526, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3527, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3528, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3524, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}, {"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 886, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": False, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3530, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3529, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 424, "children": {"columns": [{"dbID": 448, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 887, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3532, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3533, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3531, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 425, "children": {"columns": [{"dbID": 449, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 888, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3535, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3536, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3537, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3538, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3539, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3540, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3534, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 426, "children": {"columns": [{"dbID": 450, "children": {"items": [{"branching": [{"answer": {"dbID": 3543}, "question": {"dbID": 3541}, "block": {"dbID": 428}}, {"answer": {"dbID": 3544}, "question": {"dbID": 3541}, "block": {"dbID": 428}}], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 889, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3542, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3543, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3544, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3541, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 427, "children": {"columns": [{"dbID": 451, "children": {"items": [{"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 890, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3546, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3545, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 428, "children": {"columns": [{"dbID": 452, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 891, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3550, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3551, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3552, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3555, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3556, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3557, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3547, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3548, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3549, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3553, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3554, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 429, "children": {"columns": [{"dbID": 453, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 892, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3561, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3562, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3563, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3564, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3565, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3566, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3567, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3568, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3569, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3570, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3571, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3558, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3559, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3560, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 430, "children": {"columns": [{"dbID": 454, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 893, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3577, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3578, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3579, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3572, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3573, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3574, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3575, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3576, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 431, "children": {"columns": [{"dbID": 455, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 894, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3585, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3586, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3587, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3588, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3589, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3590, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3580, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3581, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3582, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3583, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3584, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 432, "children": {"columns": [{"dbID": 456, "children": {"items": [{"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 895, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3592, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3591, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 433, "children": {"columns": [{"dbID": 457, "children": {"items": [{"branching": [{"answer": {"dbID": 3598}, "question": {"dbID": 3593}, "block": {"dbID": 434}}, {"answer": {"dbID": 3599}, "question": {"dbID": 3593}, "block": {"dbID": 434}}, {"answer": {"dbID": 3600}, "question": {"dbID": 3593}, "block": {"dbID": 435}}, {"answer": {"dbID": 3601}, "question": {"dbID": 3593}, "block": {"dbID": 435}}, {"answer": {"dbID": 3602}, "question": {"dbID": 3593}, "block": {"dbID": 435}}, {"answer": {"dbID": 3603}, "question": {"dbID": 3593}, "block": {"dbID": 435}}], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 896, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3598, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3599, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3600, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3601, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3602, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3603, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3593, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3594, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3595, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3596, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3597, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 434, "children": {"columns": [{"dbID": 458, "children": {"items": [{"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 897, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3605, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3604, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 435, "children": {"columns": [{"dbID": 459, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 898, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3611, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3612, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3613, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3614, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3615, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3616, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3606, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3607, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3608, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3609, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3610, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 436, "children": {"columns": [{"dbID": 460, "children": {"items": [{"branching": [{"answer": {"dbID": 3622}, "question": {"dbID": 3618}, "block": {"dbID": 438}}, {"answer": {"dbID": 3623}, "question": {"dbID": 3618}, "block": {"dbID": 438}}, {"answer": {"dbID": 3624}, "question": {"dbID": 3618}, "block": {"dbID": 438}}, {"answer": {"dbID": 3625}, "question": {"dbID": 3618}, "block": {"dbID": 437}}, {"answer": {"dbID": 3626}, "question": {"dbID": 3618}, "block": {"dbID": 437}}, {"answer": {"dbID": 3627}, "question": {"dbID": 3618}, "block": {"dbID": 438}}], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 899, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3622, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3623, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3624, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3625, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3626, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3627, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3617, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3618, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3619, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3620, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3621, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 437, "children": {"columns": [{"dbID": 461, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 900, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 1, "children": {"answers": [{"dbID": 3629, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3630, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3631, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3632, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3633, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3628, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}, {"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 940, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": False, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3882, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3881, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 438, "children": {"columns": [{"dbID": 462, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 901, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3636, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3637, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3638, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3639, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3640, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3641, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3634, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3635, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 439, "children": {"columns": [{"dbID": 463, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 902, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3645, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3646, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3647, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3648, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3649, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3650, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3642, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3643, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3644, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 440, "children": {"columns": [{"dbID": 464, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 903, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3654, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3655, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3656, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3657, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3658, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3659, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3651, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3652, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 441, "children": {"columns": [{"dbID": 465, "children": {"items": [{"branching": [{"answer": {"dbID": None}, "question": {"dbID": None}, "block": {"dbID": 459}}], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 904, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 1, "children": {"answers": [{"dbID": 3661, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3662, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3663, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3664, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3665, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3660, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}, {"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 955, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": False, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3925, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3924, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 442, "children": {"columns": [{"dbID": 466, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 905, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3667, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3668, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3666, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 443, "children": {"columns": [{"dbID": 467, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 906, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3670, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3671, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3672, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3673, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3674, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3675, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3669, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 444, "children": {"columns": [{"dbID": 468, "children": {"items": [{"branching": [{"answer": {"dbID": 3677}, "question": {"dbID": 3676}, "block": {"dbID": 445}}, {"answer": {"dbID": 3678}, "question": {"dbID": 3676}, "block": {"dbID": 446}}, {"answer": {"dbID": 3679}, "question": {"dbID": 3676}, "block": {"dbID": 446}}], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 907, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3677, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3678, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3679, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3676, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 445, "children": {"columns": [{"dbID": 469, "children": {"items": [{"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 908, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3681, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3680, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 446, "children": {"columns": [{"dbID": 470, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 909, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3686, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3687, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3688, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3689, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3690, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3691, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3682, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3683, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3684, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3685, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 447, "children": {"columns": [{"dbID": 471, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 910, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3696, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3697, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3698, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3699, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3700, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3701, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3702, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3703, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3704, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3705, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3706, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3692, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3693, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3694, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3695, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 448, "children": {"columns": [{"dbID": 472, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 911, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3712, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3713, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3714, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3707, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3708, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3709, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3710, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3711, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 449, "children": {"columns": [{"dbID": 473, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 912, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3720, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3721, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3722, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3723, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3724, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3725, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3715, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3716, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3717, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3718, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3719, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 450, "children": {"columns": [{"dbID": 474, "children": {"items": [{"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 913, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3727, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3726, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 451, "children": {"columns": [{"dbID": 475, "children": {"items": [{"branching": [{"answer": {"dbID": 3732}, "question": {"dbID": 3728}, "block": {"dbID": 452}}, {"answer": {"dbID": 3733}, "question": {"dbID": 3728}, "block": {"dbID": 452}}, {"answer": {"dbID": 3734}, "question": {"dbID": 3728}, "block": {"dbID": 453}}, {"answer": {"dbID": 3735}, "question": {"dbID": 3728}, "block": {"dbID": 453}}, {"answer": {"dbID": 3736}, "question": {"dbID": 3728}, "block": {"dbID": 453}}, {"answer": {"dbID": 3737}, "question": {"dbID": 3728}, "block": {"dbID": 453}}], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 914, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3732, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3733, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3734, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3735, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3736, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3737, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3728, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3729, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3730, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3731, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 452, "children": {"columns": [{"dbID": 476, "children": {"items": [{"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 915, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3739, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3738, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 453, "children": {"columns": [{"dbID": 477, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 916, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3745, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3746, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3747, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3748, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3749, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3750, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3740, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3741, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3742, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3743, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3744, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 454, "children": {"columns": [{"dbID": 478, "children": {"items": [{"branching": [{"answer": {"dbID": 3756}, "question": {"dbID": 3752}, "block": {"dbID": 456}}, {"answer": {"dbID": 3757}, "question": {"dbID": 3752}, "block": {"dbID": 456}}, {"answer": {"dbID": 3758}, "question": {"dbID": 3752}, "block": {"dbID": 456}}, {"answer": {"dbID": 3759}, "question": {"dbID": 3752}, "block": {"dbID": 455}}, {"answer": {"dbID": 3760}, "question": {"dbID": 3752}, "block": {"dbID": 455}}, {"answer": {"dbID": 3761}, "question": {"dbID": 3752}, "block": {"dbID": 456}}], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 917, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3756, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3757, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3758, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3759, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3760, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3761, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3751, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3752, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3753, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3754, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3755, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 455, "children": {"columns": [{"dbID": 479, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 918, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 1, "children": {"answers": [{"dbID": 3763, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3764, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3765, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3766, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3767, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3762, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}, {"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 953, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": False, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3923, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3922, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 456, "children": {"columns": [{"dbID": 480, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 919, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3770, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3771, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3772, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3773, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3774, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3775, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3768, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3769, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 457, "children": {"columns": [{"dbID": 481, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 920, "validateMarketShare": False, "questionsAlign": 4, "trailingQuestions": False, "alwaysShowQuestionLabels": True, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 1, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3778, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3779, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3780, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3781, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3782, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3783, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3776, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}, {"dbID": 3777, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 458, "children": {"columns": [{"dbID": 482, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 921, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 1, "children": {"answers": [{"dbID": 3785, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3786, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3787, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3788, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3789, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3784, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}, {"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 922, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": False, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3791, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3790, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 459, "children": {"columns": [{"dbID": 483, "children": {"items": [{"branching": [{"answer": {"dbID": 3793}, "question": {"dbID": 3792}, "block": {"dbID": 460}}], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 923, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3793, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3794, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3795, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3792, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 460, "children": {"columns": [{"dbID": 484, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 924, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 1, "children": {"answers": [{"dbID": 3797, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3798, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3799, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3800, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3801, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3802, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3803, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3804, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3805, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3806, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3807, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3808, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3809, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3796, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}, {"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 956, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": False, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3927, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3926, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": False}, {"dbID": 461, "children": {"columns": [{"dbID": 485, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 925, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 1, "children": {"answers": [{"dbID": 3811, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3812, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3813, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3814, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3815, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3810, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}, {"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 934, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": False, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3869, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3868, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 462, "children": {"columns": [{"dbID": 486, "children": {"items": [{"dbID": 927, "poolItemID": None, "locales": {"nl_NL": {"html": u''}}, "objectName": "SurveyItemHtml"}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": False}, {"dbID": 463, "children": {"columns": [{"dbID": 487, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 929, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "50", "3": "1"}, "poolItemID": None, "showTitle": True, "filters": [{"dbID": 3, "attributes": {"domain": "float"}, "questionDbID": None, "answerDbID": None}, {"dbID": 3, "attributes": {"domain": "int", "maxValue": "1993", "minValue": "1930"}, "questionDbID": None, "answerDbID": 3822}], "answersAlign": 4, "alwaysShowAnswerLabels": True, "type": 3, "children": {"answers": [{"dbID": 3822, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3821, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}, {"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 930, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3824, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3825, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3826, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3827, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3828, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3829, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3830, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3823, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 464, "children": {"columns": [{"dbID": 488, "children": {"items": [{"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 931, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3832, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3833, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3834, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3835, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3836, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3837, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3838, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3839, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3840, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3831, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 465, "children": {"columns": [{"dbID": 489, "children": {"items": [{"dbID": 932, "poolItemID": None, "locales": {"nl_NL": {"html": u''}}, "objectName": "SurveyItemHtml"}, {"branching": [], "required": True, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 933, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 2, "alwaysShowAnswerLabels": True, "type": 2, "children": {"answers": [{"dbID": 3842, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3843, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3844, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3845, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3846, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3847, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3848, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3849, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3850, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3851, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3852, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3853, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3854, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3855, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3856, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3857, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3858, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3859, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3860, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3861, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3862, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3863, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3864, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3865, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3866, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3867, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3932, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}, {"dbID": 3933, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3841, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 466, "children": {"columns": [{"dbID": 490, "children": {"items": [{"branching": [], "required": False, "layout": None, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 935, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "50", "3": "1"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": True, "type": 3, "children": {"answers": [{"dbID": 3871, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3870, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 467, "children": {"columns": [{"dbID": 491, "children": {"items": [{"branching": [], "required": False, "layout": 4, "sum_answers": False, "objectName": "SurveyItemQA", "dbID": 926, "validateMarketShare": False, "questionsAlign": 1, "trailingQuestions": False, "alwaysShowQuestionLabels": False, "sum_questions": False, "layoutAttributes": {"1": "99", "2": "2", "3": "2"}, "poolItemID": None, "showTitle": True, "filters": [], "answersAlign": 4, "alwaysShowAnswerLabels": False, "type": 3, "children": {"answers": [{"dbID": 3817, "poolElementID": None, "locales": {"nl_NL": {"trailing_label": None, "label": u""}}, "objectName": "SurveyAnswer"}], "questions": [{"dbID": 3816, "poolElementID": None, "locales": {}, "objectName": "SurveyQuestion"}]}, "locales": {"nl_NL": {"label": u''}}}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": True}, {"dbID": 468, "children": {"columns": [{"dbID": 492, "children": {"items": [{"dbID": 936, "poolItemID": None, "locales": {"nl_NL": {"html": u''}}, "objectName": "SurveyItemHtml"}]}, "objectName": "SurveyBlockColumn", "width": None}]}, "objectName": "SurveyBlock", "pageEnd": False}]}, "availableLocales": ["nl_NL"]}}


